// PrecisionTimer.cpp : PreciseTimer implementation
//
// Note:
// - Under Windows, class PrecisionTimer requires multi-media timers.
//   With Microsoft Visual C++ this is achieved by adding "winmm.lib" to "Project Properties > Linker > Input > Additional Dependencies".
//   Alternatively, this may achieved using a suitable "pragma comment" (see below).
//   



// included header files.

#include "stdafx.h"
#include <stdio.h>
#define MULTI_MEDIA_SYSTEM_REQUIRED
#include "OperatingSystemDependencies.h"
#undef MULTI_MEDIA_SYSTEM_REQUIRED
#include "PrecisionTimer.h"



//#define REPORT_PROCEEDINGS



#if defined(WIN32)

// constructors & destructors.

PrecisionTimer::PrecisionTimer (void)
{
	this->liFrequencyPerSecond.QuadPart = 0;
	this->liBaseTime.QuadPart = 0;
	this->liEndTime.QuadPart = 0;
	this->uiTimeUnitsPerSecond = 1000; // milliseconds.
	QueryPerformanceFrequency (&this->liFrequencyPerSecond);
	QueryPerformanceCounter (&this->liBaseTime);

	// set the closest-to-the-requested timer resolution available.
	this->uiTimerResolution = 0;
	TIMECAPS tc;
	if (TIMERR_NOERROR == timeGetDevCaps (&tc, sizeof(TIMECAPS)))
	{
		this->uiTimerResolution = min(max(tc.wPeriodMin, 0), tc.wPeriodMax);
		if (TIMERR_NOERROR != timeBeginPeriod (this->uiTimerResolution))
		{
			printf ("PrecisionTimer.Constructor.Error: timeBeginPeriod (%i) failed!\n", this->uiTimerResolution);
		}
	}

	// create an event that will be used to signal the end of current single sleeping period.
	this->hSleepSyncObject = CreateEvent (((LPSECURITY_ATTRIBUTES)NULL), TRUE, FALSE, NULL);
	if (((void*)NULL) == this->hSleepSyncObject)
	{
		DWORD dwError = GetLastError ();
		printf ("PrecisionTimer.Constructor.Error: CreateEvent failed (%i)!\n", dwError);
	}
}


PrecisionTimer::~PrecisionTimer (void)
{
	// restore the original timer resolution.
	if (TIMERR_NOERROR != timeEndPeriod (this->uiTimerResolution))
	{
		//printf ("PrecisionTimer.Destructor.Error: timeEndPeriod (%i) failed!\n", this->uiTimerResolution);
	}
}



// methods.

int PrecisionTimer::SetTimeUnit (unsigned int uiTimeUnitsPerSecond)
// set the desired time units for the Get*() methods, which is pronounced by the desired unit's rate per second.
// for example: 1 = seconds, 1000 = milliseconds, 1000000 = microseconds.
// zero is ignored.
// return 0 if succeeded, -1 if failed.
{
	if (0U != uiTimeUnitsPerSecond)
	{
		this->uiTimeUnitsPerSecond = uiTimeUnitsPerSecond;
	}

	return 0;
}


void PrecisionTimer::SetBaseTime (void)
// set the base time to the current time.
{
	QueryPerformanceCounter (&this->liBaseTime);
}


double PrecisionTimer::GetBaseTime (void)
// get the recorded base time in desired time units.
{
	return ((double)this->liBaseTime.QuadPart * (double)this->uiTimeUnitsPerSecond / (double)this->liFrequencyPerSecond.QuadPart);
}


void PrecisionTimer::SetEndTime (void)
// set the end time to the current time.
{
	QueryPerformanceCounter (&this->liEndTime);
}


double PrecisionTimer::GetEndTime (void)
// get the recorded end time in desired time units.
{
	return ((double)this->liEndTime.QuadPart * (double)this->uiTimeUnitsPerSecond / (double)this->liFrequencyPerSecond.QuadPart);
}


double PrecisionTimer::GetDuration (void)
// calculate how much time has passed since the recorded base time in desired time units.
{
	return (((double)this->liEndTime.QuadPart - (double)this->liBaseTime.QuadPart) * (double)this->uiTimeUnitsPerSecond / (double)this->liFrequencyPerSecond.QuadPart);
}


double PrecisionTimer::GetTime (void)
// get the current time in desired time units.
// note that this method does NOT record the current time in any way, so calling this method does not influence future method calls.
{
	LARGE_INTEGER liTime;
	QueryPerformanceCounter (&liTime);
	return ((double)liTime.QuadPart * (double)this->uiTimeUnitsPerSecond / (double)this->liFrequencyPerSecond.QuadPart);
}


static VOID CALLBACK TimerRoutine (UINT uTimerID, UINT uMsg, DWORD_PTR dwUser, DWORD_PTR dw1, DWORD_PTR dw2)
{
#ifdef REPORT_PROCEEDINGS
	static PrecisionTimer pt;
	printf ("PrecisionTimer.TimerRoutine.Invoked: time - %.0f ms\n", pt.GetTime ());
#endif // REPORT_PROCEEDINGS
	SetEvent ((HANDLE)dwUser);
#ifdef REPORT_PROCEEDINGS
	printf ("PrecisionTimer.TimerRoutine.EventSet: time - %.0f ms\n", pt.GetTime ());
#endif // REPORT_PROCEEDINGS
}


void PrecisionTimer::AccuratelySleep (unsigned int uiMilliseconds)
// sleep for a period that's near as possible to given number of milliseconds.
// NOTE: though more exact than the Windows API Sleep, once for a while this method of wakeup reaches undesired peak wakeup periods due to its use of an event for signaling another thread, and a successive thread context switching.
{
	// try to set a time event, if possible.
	unsigned int uiTimerId;
	double dSleepStartTime;
	if (((void*)NULL) != this->hSleepSyncObject)
	{
		// verify not using the unsupported timeout of zero milliseconds.
		if (0U == uiMilliseconds)	uiMilliseconds = 1;

		uiTimerId = timeSetEvent (uiMilliseconds, this->uiTimerResolution, (LPTIMECALLBACK)::TimerRoutine, (DWORD_PTR)this->hSleepSyncObject, TIME_ONESHOT);
		 // IMPORTANT: above command requires that the project is linked with <Winmm.lib>!
		if (((unsigned int)NULL) == uiTimerId)
		{
#ifdef REPORT_PROCEEDINGS
			printf ("PrecisionTimer.Sleep.Accurate.Error: timeSetEvent (%i ms) failed!\n", uiMilliseconds);
#endif // REPORT_PROCEEDINGS
		}
		else
		{
			dSleepStartTime = this->GetTime ();
#ifdef REPORT_PROCEEDINGS
			printf ("PrecisionTimer.Sleep.Accurate.Started: time - %.0f ms\n", dSleepStartTime);
#endif // REPORT_PROCEEDINGS
		}
	}

	// if failed to set a time event, use the regular (and inaccurate) sleep function, otherwise wait for the time event to occur.
	if ((0U == this->hSleepSyncObject) || (0U == uiTimerId))
	{
#ifdef REPORT_PROCEEDINGS
		printf ("PrecisionTimer.Sleep.Regular.Started: time - %.0f ms\n", this->GetTime ());
#endif // REPORT_PROCEEDINGS
		Sleep (uiMilliseconds);
#ifdef REPORT_PROCEEDINGS
		printf ("PrecisionTimer.Sleep.Regular.Ended: time - %.0f ms\n", this->GetTime ());
#endif // REPORT_PROCEEDINGS
	}
	else
	{
		// wait till signaled for the end of desired sleeping period.
		DWORD dwWaitResult = WaitForSingleObject (this->hSleepSyncObject, (uiMilliseconds + 1000L));
		if (WAIT_OBJECT_0 != dwWaitResult) 
		{ 
			// synchronization object stays non-signaled, so a time-out occurred: this may cause longer sleep periods, but continue operation anyway.
#ifdef REPORT_PROCEEDINGS
			printf ("PrecisionTimer.Sleep.Accurate.Error: WaitForSingleObject timeout! (result - %i/%i, time - %.0f)\n", dwWaitResult, WAIT_OBJECT_0, this->GetTime ());
#endif // REPORT_PROCEEDINGS
		}
		else
		{
#ifdef REPORT_PROCEEDINGS
			printf ("PrecisionTimer.Sleep.Accurate.Ended: time - %.0f ms, slept - %.0f ms\n", this->GetTime (), (this->GetTime () - dSleepStartTime));
#endif // REPORT_PROCEEDINGS
		}
		ResetEvent (this->hSleepSyncObject);
	}
}

#elif ( defined(__unix__) || defined (VX_WORKS) )

#if defined (VX_WORKS)

//#define INCLUDE_getTimeZone
#ifdef INCLUDE_getTimeZone
void getTimeZone (time_t t, int* ipMin, int* ipDst)
{
	struct tm tmDst;
	char* cpTz = getenv (TZ_ENV);
	*ipDst = ( (OK == localtime_r (&t, &tmDst)) ? tmDst.tm_isdst : 0 );

	if (cpTz)
	{
		const char *p = cpTz;
		int i = 0;
		while ( (i < 2) && (p = strchr (p, ':')) )
		{
			++p;
			++i;
		}
		if (p)
		{
			if ( 1 != sscanf(p, "%d", ipMin) )
			{
				(*ipMin) = 0;
			}
		}
	}
}
#endif // INCLUDE_getTimeZone


int gettimeofday (struct timeval *tv, struct timezone *tz)
{
	int iReturnedValue;
	struct timespec ts;

	iReturnedValue = clock_gettime (CLOCK_REALTIME, &ts);
	if  (0U == iReturnedValue)
	{
		tv->tv_sec = ts.tv_sec;
		tv->tv_usec = (ts.tv_nsec + 500) / 1000;

#ifdef INCLUDE_getTimeZone
		if (((struct timezone *)NULL) != tz)
		{
			getTimeZone (ts.tv_sec, &tz->tz_minuteswest, &tz->tz_dsttime);
		}
#endif // INCLUDE_getTimeZone
	}

	return iReturnedValue;
}


#endif // VX_WORKS



// constructors & destructors.

PrecisionTimer::PrecisionTimer (void)
{
	this->tvBaseTime.tv_sec = 0;
	this->tvBaseTime.tv_usec = 0;
	this->tvEndTime.tv_sec = 0;
	this->tvEndTime.tv_usec = 0;
	this->uiTimeUnitsPerSecond = 1000; // milliseconds.
	gettimeofday (&(this->tvBaseTime), NULL);
}


PrecisionTimer::~PrecisionTimer (void)
{
}



// methods.

int PrecisionTimer::SetTimeUnit (unsigned int uiTimeUnitsPerSecond)
// set the desired time units for the Get*() methods, which is pronounced by the desired unit's rate per second.
// for example: 1 = seconds, 1000 = milliseconds, 1000000 = microseconds.
// zero is ignored.
// return 0 if succeeded, -1 if failed.
{
	if (0U != uiTimeUnitsPerSecond)
	{
		this->uiTimeUnitsPerSecond = uiTimeUnitsPerSecond;
	}

	return 0;
}


void PrecisionTimer::SetBaseTime (void)
// set the base time to the current time.
{
	gettimeofday (&(this->tvBaseTime), NULL);
}


double PrecisionTimer::GetBaseTime (void)
// get the recorded base time in desired time units.
{
	return ((double)((this->tvBaseTime.tv_sec * 1000000) + this->tvBaseTime.tv_usec) * ((double)this->uiTimeUnitsPerSecond / (double)1000000));
}


void PrecisionTimer::SetEndTime (void)
// set the end time to the current time.
{
	gettimeofday (&(this->tvEndTime), NULL);
}


double PrecisionTimer::GetEndTime (void)
// get the recorded end time in desired time units.
{
	return ((double)((this->tvEndTime.tv_sec * 1000000) + this->tvEndTime.tv_usec) * ((double)this->uiTimeUnitsPerSecond / (double)1000000));
}


double PrecisionTimer::GetDuration (void)
// calculate how much time has passed since the recorded base time in desired time units.
{
	return ((double)(((this->tvEndTime.tv_sec * 1000000) + this->tvEndTime.tv_usec) - ((this->tvBaseTime.tv_sec * 1000000) + this->tvBaseTime.tv_usec)) * ((double)this->uiTimeUnitsPerSecond / (double)1000000));
}


double PrecisionTimer::GetTime (void)
// get the current time in desired time units.
// note that this method does NOT record the current time in any way, so calling this method does not influence future method calls.
{
	struct timeval tvTime;
	gettimeofday (&tvTime, NULL);
	return ((double)((tvTime.tv_sec * 1000000) + tvTime.tv_usec) * ((double)this->uiTimeUnitsPerSecond / (double)1000000));
}


void PrecisionTimer::AccuratelySleep (unsigned int uiMilliseconds)
// sleep for a period that's near as possible to given number of milliseconds.
// NOTE: though more exact than the Windows API Sleep, once for a while this method of wakeup reaches undesired peak wakeup periods due to its use of an event for signaling another thread, and a successive thread context switching.
{
#if defined(__unix__)
	int iResult = 0;
	iResult = usleep (1000U * uiMilliseconds); // 1000 microseconds = 1 millisecond.
#elif defined (VX_WORKS)
	Sleep_NonOsDependent(uiMilliseconds);
#endif // __unix__
}

#endif // __unix__ || VX_WORKS
