// InterProcessSemaphore.cpp : InterProcessSemaphore implementation
//



// included header files.

#include "stdafx.h"
#include <stdio.h>
#include "OperatingSystemDependencies.h"
#include "InterProcessSemaphore.h"



/// <summary>
/// Constructor.
///
/// Note:
/// - A semaphore always gets started in an initial count, and from that point it may function in one of two ways:
///   - Countdown semaphore (maximum count is irrelevant): The semaphore's count is incremented when released, decremented when caught, and may not get lower than zero.
///   - Count-up semaphore (maximum count is relevant): The semaphore's count is decremented when released, incremented when caught, and may not get higher than its maximal count.
///   Each operating-system offers only semaphores of one of these two types.
///   This class offers a semaphore that behaves as follows when the operating-system offers semaphores of type:
///   - Countdown semaphore: The semaphore's initial count is always zero, and given maximum count is ignored.
///   - Count-up semaphore: The semaphore's initial count always zero, and given maximum count is ignored is set as the semaphore's maximum count.
///
/// Parameters:
/// szpName (in)	A pointer to a null-terminated string that contains the global name of the semaphore.
/// uiMaximumCount (in)	The maximum count possible for the semaphore.
/// </summary>
InterProcessSemaphore::InterProcessSemaphore (const char* szpName, unsigned int uiMaximumCount)
{
	this->szName[0] = '\0';
	if (MAX_SEMAPHORE_NAME >= strlen (szpName))
	{
		char* cpResult = ((char*)NULL);
		cpResult = strcpy (this->szName, szpName);
	}
	this->uiMaximumCount = uiMaximumCount;
	this->bIsActive = false;

#if defined(WIN32)

	this->hSemaphore = ((void*)NULL);

#endif // WIN32

}

/// <summary>
/// Destructor.
/// </summary>
InterProcessSemaphore::~InterProcessSemaphore (void)
{
	if (this->bIsActive)
	{
		int iResult = 0;
		iResult = this->Close ();
	}
}

bool InterProcessSemaphore::IsActive (void)
{
	return this->bIsActive;
}

/// <summary>
/// Create the semaphore.
///
/// Returned values:
/// 0 if function successfully completed.
/// Negative if an error condition encountered or function failed.
/// </summary>
int InterProcessSemaphore::Create (void)
{
	if (this->bIsActive)
	{
		return -1;
	}

#if defined(WIN32)

	this->hSemaphore = CreateSemaphore (((LPSECURITY_ATTRIBUTES)NULL) /*default security*/, ((LONG)0) /*initial count*/, ((LONG)(this->uiMaximumCount)) /*maximum count*/, ((LPCSTR)(this->szName)) /*name*/);
	if (((void*)NULL) == this->hSemaphore)
	{
		return -1;
	}

#elif ( defined(__unix__) || defined (VX_WORKS) )

	this->semtspSemaphore = sem_open (this->szName, O_CREAT, (S_IRWXU | S_IRWXG | S_IRWXO), 0);
#if defined (__unix__)
	if (SEM_FAILED == this->semtspSemaphore)
#elif defined (VX_WORKS)
	if (ERROR == ((long)(this->semtspSemaphore)))
#endif // __unix__
	{
		int iResult = 0;
		iResult = this->Close ();
		return -1;
	}

#endif // __unix__ || VX_WORKS

	this->bIsActive = true;
	return 0;
}

/// <summary>
/// Open the semaphore.
///
/// Returned values:
/// 0 if function successfully completed.
/// Negative if an error condition encountered or function failed.
/// </summary>
int InterProcessSemaphore::Open (void)
{
	if (this->bIsActive)
	{
		return -1;
	}

#if defined(WIN32)

	this->hSemaphore = OpenSemaphore (SEMAPHORE_MODIFY_STATE /*desired access*/, FALSE /*no handle inherit*/, this->szName /*name*/);
	if (((void*)NULL) == this->hSemaphore)
	{
		return -1;
	}

#elif ( defined(__unix__) || defined (VX_WORKS) )

	this->semtspSemaphore = sem_open (this->szName, 0, (S_IRWXU | S_IRWXG | S_IRWXO), 0);
#if defined (__unix__)
	if (SEM_FAILED == this->semtspSemaphore)
#elif defined (VX_WORKS)
	if (ERROR == ((long)(this->semtspSemaphore)))
#endif // __unix__
	{
		int iResult = 0;
		iResult = this->Close ();
		return -1;
	}

#endif // __unix__ || VX_WORKS

	this->bIsActive = true;
	return 0;
}

/// <summary>
/// Try to catch the semaphore.
/// If the semaphore cannot be caught on the spot, then wait for not more than the defined wait time.
/// If the semaphore still cannot be caught, then report failure.
///
/// Parameters:
/// uiWaitTime (in)	The maximum time (in units of milliseconds) to wait when trying to catch the semaphore.
///
/// Returned values:
/// 0 if function successfully completed.
/// Negative if an error condition encountered or function failed.
/// </summary>
int InterProcessSemaphore::Catch (unsigned int uiWaitTime)
{
	if ( ! this->bIsActive )
	{
		return -1;
	}

#if defined(WIN32)

	DWORD dwWaitResult;
	dwWaitResult = WaitForSingleObject (hSemaphore, uiWaitTime);
	if (WAIT_TIMEOUT == dwWaitResult)
	{
		return -1;
	}

	return 0;

#elif ( defined(__unix__) || defined (VX_WORKS) )

	while (0U < uiWaitTime)
	{
		if (((int)(0U)) == sem_trywait (this->semtspSemaphore))
		{
			return 0;
		}

#if defined(__unix__)
		int iResult = 0;
		iResult = usleep (1000U); // 1000 microseconds = 1 millisecond.
#elif defined (VX_WORKS)
		Sleep_NonOsDependent(1U);
#endif // __unix__

		uiWaitTime--;
	}
	return sem_trywait (this->semtspSemaphore);

#endif // __unix__ || VX_WORKS
}

/// <summary>
/// Release the semaphore.
///
/// Returned values:
/// 0 if function successfully completed.
/// Negative if an error condition encountered or function failed.
/// </summary>
int InterProcessSemaphore::Release (void)
{
	if ( ! this->bIsActive )
	{
		return -1;
	}

#if defined(WIN32)

	if ( ! ReleaseSemaphore (this->hSemaphore, 1, NULL) )
	{
		return -1;
	}

	return 0;

#elif ( defined(__unix__) || defined (VX_WORKS) )

	return sem_post (this->semtspSemaphore);

#endif // __unix__ || VX_WORKS
}

/// <summary>
/// Close the semaphore.
///
/// Returned values:
/// 0 if function successfully completed.
/// Negative if an error condition encountered or function failed.
/// </summary>
int InterProcessSemaphore::Close (void)
{

#if defined(WIN32)

	if ( ! this->bIsActive )
	{
		return 0;
	}

	if ( ! CloseHandle (this->hSemaphore) )
	{
		return -1;
	}

	this->hSemaphore = ((void*)NULL);

#elif ( defined(__unix__) || defined (VX_WORKS) )

	if (this->bIsActive)
	{
		if (-1 == sem_close (this->semtspSemaphore))
		{
			return -1;
		}
	}

	int iResult = 0;
	iResult = sem_unlink (this->szName);

#endif // __unix__ || VX_WORKS

	this->bIsActive = false;
	return 0;
}
