/// @file
/// Operating-system specific definitions & functions for a library

// Description  : A source file that contains the implementation of WIN32 operating-system specific definitions & functions for a library
// Version      : 1.0.0.0
// Author       : Sital Technology R&D
//
// Modification history:
//



#include "stdafx.h"
//#if defined(WIN32)
//#include <delayimp.h>
//#endif // WIN32
#include <stdio.h>
#include <stdlib.h>
#include "OperatingSystemDependencies.h"
#include "LibraryOperatingSystemDependencies.h"

// Constant definitions.

// Function return codes.
#define sitalReturnCode_SUCCESS												0		///< No error occurred.
#define sitalReturnCode_UNKNOWN_ERROR										-1		///< An unknown error occurred.
#define sitalReturnCode_VERSION_ERROR										-2		///< Failed to get version number.
#define sitalReturnCode_LIBRARY_LOAD_ERROR									-3		///< Failed to load a required function library.
#define sitalReturnCode_INVALID_PARAMETER									-65		///< Bad parameter.



#ifdef _MANAGED
#pragma managed(push, off)
#endif



// Static variables.
#if defined(WIN32)
static HMODULE s_hLibraryModuleHandle = (HMODULE)0;
#endif // WIN32



// Internal functions.


#if defined(WIN32)
/// <summary>
/// Load any library required by the stld1553 library.
///
/// @return sitalReturnCode_SUCCESS	Function successfully completed
/// @return Negative sitalReturnCode_*	Error condition or function failed
/// </summary>
S16BIT helperLibraries_Load (void)
{
	/// @pseudocode

	// Nothing to do, as under WIN32 the driver interface library is automatically loaded.

//	The following commented-in code was used in the past to delay-load the driver interface library.
//	Delayed-load had been originally preferred as this way this library can explicitly inform the caller/user in case the driver interface library could not be loaded.
//  But as loading libraries from within function DllMain, which indirectly calls this function, proved problematic, the delayed-load was abandoned in favor of auto-load.

//	/// If the stl1553-driver-API library may not be loaded upon process attach:
//	///		Return.
//	// Note: In order to accomplish delayed-load, set Project > Properties > Configuration > Linker > Input > Delay Loaded Dlls to be "\<config>\stl1553drvapi_win32.dll %(DelayLoadDLLs)", where <config> stands for either "Debug" or "Release".
//	if (FAILED(__HrLoadAllImportsForDll("stl1553drvapi_win32.dll")))
//	{
//#if _DEBUG
//		printf ("Error: Failed to load <stl1553drvapi_win32.dll>!\n" );
//#endif // _DEBUG
//		return sitalReturnCode_LIBRARY_LOAD_ERROR;
//	}

	/// Loaded required libraries: Return.
	return sitalReturnCode_SUCCESS;
}


/// <summary>
/// Get the library version numbers.
///
/// @param wpMajorVersion (out)	A pointer to the variable in which the first version number is returned
/// @param wpMinorVersion (out)	A pointer to the variable in which the second version number is returned
/// @param wpBuildNumber (out)	A pointer to the variable in which the third version number is returned
/// @param wpRevisionNumber (out)	A pointer to the variable in which the fourth version number is returned
/// @return sitalReturnCode_SUCCESS	Function successfully completed
/// @return Negative sitalReturnCode_*	Error condition or function failed
/// </summary>
S16BIT library_GetVersion (	U16BIT* wpMajorVersion,
							U16BIT* wpMinorVersion,
							U16BIT* wpBuildNumber,
							U16BIT* wpRevisionNumber)
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.
	if ((((U16BIT*)NULL) == wpMajorVersion) || (((U16BIT*)NULL) == wpMinorVersion) || (((U16BIT*)NULL) == wpBuildNumber) || (((U16BIT*)NULL) == wpRevisionNumber))
	{
		return sitalReturnCode_INVALID_PARAMETER;
	}

	/// Get the name of this library module.
	if ( 0 == s_hLibraryModuleHandle )
	{
		return sitalReturnCode_LIBRARY_LOAD_ERROR;
	}
	CHAR szLibraryModuleName[MAX_PATH]; // The name of this library module.
	DWORD dwLibraryNameLength; // The length of the name of this library module.
	dwLibraryNameLength = GetModuleFileName (s_hLibraryModuleHandle, szLibraryModuleName, ARRAYSIZE(szLibraryModuleName));
	if ( (0U == dwLibraryNameLength) || (ARRAYSIZE(szLibraryModuleName) <= dwLibraryNameLength) )
	{
		return sitalReturnCode_LIBRARY_LOAD_ERROR;
	}

	DWORD dwLibraryHandle;
	U32BIT dwFileVersionInformationSize;
	dwFileVersionInformationSize = GetFileVersionInfoSize (szLibraryModuleName, &dwLibraryHandle);
	if (0U == dwFileVersionInformationSize)
	{
		return sitalReturnCode_VERSION_ERROR;
	}

	LPTSTR szlpFileVersionInformationData;
	szlpFileVersionInformationData = (LPTSTR) malloc (dwFileVersionInformationSize);
	if (!szlpFileVersionInformationData)
	{
		return sitalReturnCode_VERSION_ERROR;
	}

	if ( 0 == GetFileVersionInfo (szLibraryModuleName, dwLibraryHandle, dwFileVersionInformationSize, szlpFileVersionInformationData) )
	{
		free (szlpFileVersionInformationData);
		return sitalReturnCode_VERSION_ERROR;
	}

	VS_FIXEDFILEINFO* ffipFileInformation;
	UINT uiBufLen;
	if ( VerQueryValue (szlpFileVersionInformationData, "\\", (LPVOID*)&ffipFileInformation, (PUINT)&uiBufLen) )
	{
		*wpMajorVersion = HIWORD(ffipFileInformation->dwFileVersionMS);
		*wpMinorVersion = LOWORD(ffipFileInformation->dwFileVersionMS);
		*wpBuildNumber = HIWORD(ffipFileInformation->dwFileVersionLS);
		*wpRevisionNumber = LOWORD(ffipFileInformation->dwFileVersionLS);

		free (szlpFileVersionInformationData);
		return sitalReturnCode_SUCCESS;
	}

	free (szlpFileVersionInformationData);

	return sitalReturnCode_SUCCESS;
}

#elif ( defined(__unix__) || defined (VX_WORKS) )

/// <summary>
/// Load any library required by the stld1553 library.
///
/// @return sitalReturnCode_SUCCESS	Function successfully completed
/// @return Negative sitalReturnCode_*	Error condition or function failed
/// </summary>
S16BIT helperLibraries_Load (void)
{
	/// @pseudocode

	// Nothing to do, as under Linux the driver interface library is automatically loaded.
	// In case this is changed, then do the following (with suitable adjustments):
	// - include <dlfcn.h>
	// - use code similar to the following commented-out code in order to load the driver interface library.
	//   void* handleDriverInterfaceSharedLibrary = dlopen ("libstl1553drvapi.so", RTLD_LAZY);
	// - use code similar to the following commented-out code in order to get required function pointers for the driver interface library.
	//   S16BIT (*sitalDriverInterfaceLibrary_GetVersion)(U16BIT*, U16BIT*, U16BIT*, U16BIT*) = dlsym (handleDriverInterfaceSharedLibrary, "sitalDriverInterfaceLibrary_GetVersion");
	// - use code similar to the following commented-out code in order to call the functions of the driver interface library.
	//   S16BIT swResult = (*sitalDriverInterfaceLibrary_GetVersion)(swp1, swp2, swp3, swp4);
	// - if required, use code similar to the following commented-out code in order to unload the driver interface library.
	//   dlclose (handleDriverInterfaceSharedLibrary);

	/// Loaded required libraries: Return.
	return sitalReturnCode_SUCCESS;
}


/// <summary>
/// Get the library version numbers.
///
/// @param wpMajorVersion (out)	A pointer to the variable in which the first version number is returned
/// @param wpMinorVersion (out)	A pointer to the variable in which the second version number is returned
/// @param wpBuildNumber (out)	A pointer to the variable in which the third version number is returned
/// @param wpRevisionNumber (out)	A pointer to the variable in which the fourth version number is returned
/// @return sitalReturnCode_SUCCESS	Function successfully completed
/// @return Negative sitalReturnCode_*	Error condition or function failed
/// </summary>
S16BIT library_GetVersion (	U16BIT* wpMajorVersion,
							U16BIT* wpMinorVersion,
							U16BIT* wpBuildNumber,
							U16BIT* wpRevisionNumber)
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.
	if ((((U16BIT*)NULL) == wpMajorVersion) || (((U16BIT*)NULL) == wpMinorVersion) || (((U16BIT*)NULL) == wpBuildNumber) || (((U16BIT*)NULL) == wpRevisionNumber))
	{
		return sitalReturnCode_INVALID_PARAMETER;
	}

	/// Return built-in numbers.
	*wpMajorVersion = versionNumber_Major;
	*wpMinorVersion = versionNumber_Minor;
	*wpBuildNumber = versionNumber_Build;
	*wpRevisionNumber = versionNumber_Revision;

	return sitalReturnCode_SUCCESS;
}

#endif // __unix__ || VX_WORKS



// Exported functions.


#if defined(WIN32)
/// <summary>
/// The main entry point of the stld1553 library.
///
/// Note:
/// - The stld1553 library requires that its initialization function will be automatically called under any operating system.
///
/// @param hModule				See documentation at MSDN
/// @param ul_reason_for_call	See documentation at MSDN
/// @param lpReserved			See documentation at MSDN
/// @return TRUE	on success
/// @return FALSE	on failure
/// </summary>
BOOL APIENTRY DllMain (	HMODULE hModule, DWORD  ul_reason_for_call, LPVOID lpReserved)
{
	/// @pseudocode

	switch (ul_reason_for_call)
	{
	case DLL_PROCESS_ATTACH:
		{
			// Record the handle of this library module.
			s_hLibraryModuleHandle = hModule;

			/// If could not perform required initialization:
			///		Return.
			S16BIT swResult; // Result of operation or function call.
			swResult = initializeLibrary();
			if (0 > swResult)
			{
#if _DEBUG
				FILE* fileOutput;
				fopen_s (&fileOutput, "Sital.Library.LoadError.txt", "wt");
				fprintf_s (fileOutput, "Sital.Library.Load.Error: error - %i\n", swResult);
				fclose (fileOutput);
#endif // _DEBUG

				return FALSE;
			}

			break;
		}

    case DLL_THREAD_ATTACH:
		{
			break;
		}

    case DLL_THREAD_DETACH:
		{
			break;
		}

    case DLL_PROCESS_DETACH:
		{
			uninitializeLibrary();
			break;
		}
    }

	return TRUE;
}

#elif ( defined(__unix__) || defined (VX_WORKS) )

/// <summary>
/// The main entry point of the stld1553 library.
///
/// Note:
/// - This function is automatically called when the library is loaded, already before the call to function dlopen returns.
/// - The stld1553 library requires that its initialization function will be automatically called under any operating system.
/// </summary>
void sitalLibrary_Load (void)
{
	/// If could not perform required initialization:
	///		Return.
	static BOOLEAN s_bLibraryAlreadyLoaded = FALSE;
	if (TRUE == s_bLibraryAlreadyLoaded)
	{
		return;
	}

	S16BIT swResult; // Result of operation or function call.
	swResult = initializeLibrary();
	if (0 > swResult)
	{
#if _DEBUG
		FILE* fileOutput;
		fopen_s (&fileOutput, "Sital.Library.LoadError.txt", "wt");
		fprintf_s (fileOutput, "Sital.Library.Load.Error: error - %i\n", swResult);
		fclose (fileOutput);
#endif // _DEBUG

		exit (-1);
	}

	REGISTER_EXIT_FUNCTION(sitalLibrary_Unload);

	s_bLibraryAlreadyLoaded = TRUE;
}

/// <summary>
/// The main exit point of the stld1553 library.
///
/// Note:
/// - This function is automatically called when the library is unloaded, already before the call to function dlclose returns.
/// </summary>
void sitalLibrary_Unload (void)
{
	/// If could not perform required deconstruction:
	///		Return.
	uninitializeLibrary();
}

#endif // __unix__ || VX_WORKS
