/// @file
/// stl1553-driver-API library's exported definitions and declarations.

// Description  : A header file that contains the BRM driver interface library's exported definitions and declarations
// Version      : 1.0.0.0
// Author       : Sital Technology R&D
//
// Modification history:
//



#ifndef stld1553DriverInterface_INCLUDED
#define stld1553DriverInterface_INCLUDED



#ifdef __cplusplus
// C++ compilers mangle function names, unless the function definitions are protected with extern "C"{}.
extern "C" {
#endif



// Enumeration definitions.

/// Types of device access operations.
typedef enum // sitalDeviceAccessOperationEnum
	{
	sitalDeviceAccessOperation_Read															= 0,	///< Read operation.
	sitalDeviceAccessOperation_Write,																///< Write operation.
	sitalDeviceAccessOperation_Zero,																///< Zero memory operation.
	sitalDeviceAccessOperation_WriteMasked															///< Masked write operation: Bits that are off in the mask are left unchanged, bits that are on in the mask are set to their counterparts in the desired value.
	} sitalDeviceAccessOperationEnum;

/// Sections of device memory.
typedef enum // sitalDeviceMemorySectionEnum
	{
	sitalDeviceMemorySection_Registers														= 0,	///< Registers memory section.
	sitalDeviceMemorySection_Ram																	///< RAM memory section.
	} sitalDeviceMemorySectionEnum;

#ifndef stld1553_PRINTINGS_LEVEL
#define stld1553_PRINTINGS_LEVEL
/// Levels of printings to console.
typedef enum // sitalPrintingsLevelEnum
	{
		sitalPrintingsLevel_Information														= 0,	///< Any required informative.
		sitalPrintingsLevel_Warning,																///< Warning message.
		sitalPrintingsLevel_Error,																	///< Error message.
		sitalPrintingsLevel_None																	///< No printing.
	} sitalPrintingsLevelEnum;
#endif // stld1553_PRINTINGS_LEVEL



// Constant definitions.
#define BRMC_INTERRUPT_FLAG_SEVERE_PROBLEM													0x8000U ///< A interrupt signal mode bit mask for field swFlags of structure sitalCardInformationStructure: on/off when pulse/level interrupts, respectively, suit the operating system and card type.

#ifndef BRMC_CAP_IRQ_PULSE
#define BRMC_CAP_IRQ_PULSE																	0x100U ///< A interrupt signal mode bit mask for field swFlags of structure sitalCardInformationStructure: on/off when pulse/level interrupts, respectively, suit the operating system and card type.
#endif //BRMC_CAP_IRQ_PULSE


// Macro definitions.



// Type/structure declarations.
struct sitalDeviceCapabilitiesStructure;
struct InterruptInformationStructure;



// Type/structure definitions.
// Note: To avoid compiler compatibility issues, the structures are defined so that each member is properly aligned on its size boundary, and the whole structure is padded to the size of the largest member.
// This is instead of using compiler specific alignment and packing directives or pragmas.

/// Hardware independent card information.
typedef struct sitalCardInformationStructure
{
	/// The size (in bytes) of the supplied structure.
	/// When passing this structure in function calls, the caller must set this field to the size (in bytes) of this structure.
	U32BIT  dwStructureSize;
	U16BIT  wFlags;
	U16BIT  wId;
	U16BIT  wUnits;
	U16BIT  wFirstUnitId;
	U32BIT  dwUserData;
	U16BIT  waDeviceCapabilities[16]; // 16 is the maximum possible units on a BRM card.
} sitalCardInformationStruct;

/// Device information and capabilities.
typedef struct sitalDeviceCapabilitiesStructure
{
	/// The size (in bytes) of the supplied structure.
	/// When passing this structure in function calls, the caller must set this field to the size (in bytes) of this structure.
	U32BIT dwStructureSize;
	/// The size (in words) of the RAM memory section of this unit.
	/// This field contains actual information only if field bExists contains TRUE.
	U32BIT dwRamSize;
	/// The logical number of the card to which this unit belongs.
	S16BIT swCard;
	/// Current operation mode (zero if this unit is currently uninitialized, or sitalMode_*).
	/// This field contains actual information only if field bExists contains TRUE.
	U16BIT wMode;
	/// A flag that says whether this unit currently exists at all in the system.
	BOOLEAN bExists;
	/// A flag that says whether this unit may be used by the caller, i.e., it is either closed at the moment or already opened by the caller's process.
	/// This field contains actual information only if field bExists contains TRUE.
	/// Note that the value of this field is known to be correct only at the moment it is filled.
	BOOLEAN bIsFreeForUse;
	/// A flag that says whether this unit may be used as BC.
	/// This field contains actual information only if field bExists contains TRUE.
	BOOLEAN bIsBcModeAllowed;
	/// A flag that says whether this unit may be used as RT.
	/// This field contains actual information only if field bExists contains TRUE.
	BOOLEAN bIsRtModeAllowed;
	/// A flag that says whether this unit may be used as MT.
	/// This field contains actual information only if field bExists contains TRUE.
	BOOLEAN bIsMtModeAllowed;
	/// A flag that says whether this unit is connected to 2 buses, A and B.
	BOOLEAN bIsDualBus;
	/// This field is composed as follows:
	/// - Bit 7: A flag that says whether this unit has a latched RT address.
	/// - Bits 0-5: The latched RT address of this unit. Contains actual information only if Bit 7 is on.
	/// This field contains actual information only if both fields bExists and bRtAddressLatched contain TRUE.
	U16BIT wLatchedRtAddress;
} sitalDeviceCapabilitiesStruct;

/// Information on the now-reported interrupt.
typedef struct InterruptInformationStructure
{
	/// The size (in bytes) of the supplied structure.
	/// When passing this structure in function calls, the caller must set this field to the size of InterruptInformationStructure.
	U32BIT dwStructureSize;
	/// Interrupt type.
	/// If the MSbit of the type is off, then it is a normal device interrupt.
	/// Otherwise it is a sever problem (e.g., card-level interrupt), and the software must be aborted.
	/// If bit 0x8000 of wFlags is 1, it indicates some error condition, such as hardware error.
	/// if bit 0x4000 of wFlags is 1, then it is interrupt of a card.
	U16BIT wFlags;
	/// Logical number of device (0-31).
	S16BIT swDevice;
	/// Logical number of card (>=0).
	S16BIT swCard;
	/// The value of the involved device's time tag register.
	U16BIT wDeviceTimer;
	union
	{
		/// The involved device's IRQ state.
		U32BIT dwIrqStatus;
		struct
		{
			/// The involved device's interrupt status register 1.
			U16BIT wInterruptStatusRegister1;
			/// The involved device's interrupt status register 2.
			U16BIT wInterruptStatusRegister2;
		} DeviceIrqStatusRegisters;
	};
	/// The involved device's time tag register.
	/// If wFlags bit 0x0004 is set, this value is valid.
	U32BIT dwCardTimer;
	U32BIT dwReserved2;	// for alignment
	/// The operating system time [optional; format is OS-specific].
	/// If wFlags bit 0x0008 is set, this value is valid.
	U64BIT qwSystemTime;
} InterruptInformationStruct;



// Type definitions.

///< A pointer to an interrupt service routine.
typedef S16BIT(_DECL *ISR_FUNCTION)(const struct InterruptInformationStructure* iisp);

// Internal-function declarations.

// Exported-function declarations.

// General driver-interface-library functions.
_EXTERN S16BIT _DECL sitalDriverInterfaceLibrary_GetVersion (	U16BIT* wpMajorVersion,
																U16BIT* wpMinorVersion,
																U16BIT* wpBuildNumber,
																U16BIT* wpRevisionNumber);
_EXTERN S16BIT _DECL sitalDriverInterfaceLibrary_ConfigurePrintings (BOOLEAN bIsConsoleScreen, sitalPrintingsLevelEnum pleBasePrintingsLevel);

// The following function is an internal backdoor, and as it is subject to future change or removal, it shall not be used by users!
_EXTERN S16BIT _DECL sitalDriverCall (	U32BIT dwFunction,
										void* vpInputData,
										U32BIT dwInputByteCount,
										void* vpOutputData,
										U32BIT dwOutputByteCount,
										U32BIT* dwpReturnedByteCount);

// Card configuration & manipulation functions.
_EXTERN S16BIT _DECL sitalCard_GetCount (U16BIT* wpCardCount);
_EXTERN S16BIT _DECL sitalCard_GetInformation (S16BIT swCard, struct sitalCardInformationStructure *cisp);
_EXTERN S16BIT _DECL sitalCard_GetType (S16BIT swCard, U8BIT* bpType, U32BIT* dwpSubType);
_EXTERN S16BIT _DECL sitalCard_Fpga_Program (S16BIT swCard, FILE* filepFpga);
_EXTERN S16BIT _DECL sitalCard_Fpga_Read (S16BIT CardNum, FILE* filepFpga);
_EXTERN S16BIT _DECL sitalCard_Fpga_GetVersion (S16BIT swCard, U16BIT* wpVersion);
_EXTERN S16BIT _DECL sitalCard_GetTime (S16BIT swCard, U32BIT* dwpTime);
_EXTERN S16BIT _DECL sitalCard_Irq_Setup (S16BIT swCard, U16BIT wInterruptLevel, U16BIT wInterruptLine);
_EXTERN S16BIT _DECL sitalCard_Irq_Enable (S16BIT swCard, U16BIT wInterruptEnable);
_EXTERN S16BIT _DECL sitalCard_Irq_Disable (S16BIT swCard, U16BIT wInterruptDisable);
_EXTERN S16BIT _DECL sitalCard_Irq_GetStatus (S16BIT swCard, U16BIT* wpInterruptState);
_EXTERN S16BIT _DECL sitalCard_Irq_Reset (S16BIT swCard, U16BIT wInterruptReset);
_EXTERN S16BIT _DECL sitalCard_RunTimer (S16BIT swCard, U32BIT dwLength, BOOLEAN bIsAutoRestart);
_EXTERN S16BIT _DECL sitalCard_Ip_ExternallyTriggerBc (S16BIT swCard, U8BIT bIps);
_EXTERN S16BIT _DECL sitalCard_Ip_SetRtSubSystemFlag (S16BIT swCard, U8BIT bIps);
_EXTERN S16BIT _DECL sitalCard_Ip_Reset (S16BIT swCard, U16BIT wIpResetLines);
_EXTERN S16BIT _DECL sitalCard_Ip_SetCommonRtAddress (S16BIT swCard, U8BIT bRtAddress, BOOLEAN bEnable, BOOLEAN bWired );
_EXTERN S16BIT _DECL sitalCard_Ip_GetCommonRtAddress (S16BIT swCard, U8BIT* bpRtAddress, BOOLEAN* bpEnable, BOOLEAN* bpWired);
_EXTERN S16BIT _DECL sitalCard_ConfigureSimultaneousAccess (S16BIT swCard, BOOLEAN bIsSimultaneousAccessEnabled);
_EXTERN S16BIT _DECL sitalCard_Open (S16BIT swCard);
_EXTERN S16BIT _DECL sitalCard_Close (S16BIT swCard);

_EXTERN S16BIT _DECL sitalCard_Read(S16BIT swCard, U32BIT address, U32BIT len, U16BIT* buffer);
_EXTERN S16BIT _DECL sitalCard_Write(S16BIT swCard, U32BIT address, U32BIT len, U16BIT* buffer);
_EXTERN S16BIT _DECL sitalCard_Read_Bar2(S16BIT swCard, U32BIT address, U32BIT len, U32BIT* buffer);
_EXTERN S16BIT _DECL sitalCard_Write_Bar2(S16BIT swCard, U32BIT address, U32BIT len, U32BIT* buffer);
_EXTERN S16BIT _DECL sitalCard_Read_DWORD(S16BIT swCard, U32BIT address, U32BIT len, U32BIT* buffer);
_EXTERN S16BIT _DECL sitalCard_Write_DWORD(S16BIT swCard, U32BIT address, U32BIT len, U32BIT* buffer);

// Device configuration & manipulation functions.
_EXTERN S16BIT _DECL sitalDevice_GetCount (U16BIT* wpDeviceCount);
_EXTERN S16BIT _DECL sitalDevice_GetInformation (S16BIT swDevice, struct sitalDeviceCapabilitiesStructure* dcspDeviceCapabilities);
_EXTERN S16BIT _DECL sitalDevice_SetMode (S16BIT swDevice, U16BIT wMode);
_EXTERN S16BIT _DECL sitalDevice_Open (S16BIT swDevice, struct sitalDeviceCapabilitiesStructure* dcspDeviceCapabilities);
_EXTERN S16BIT _DECL sitalDevice_Close (S16BIT swDevice);
_EXTERN S16BIT _DECL sitalDevice_AccessMemory	(	S16BIT swDevice,
													sitalDeviceAccessOperationEnum daoeAccessType,
													U32BIT dwWordOffset,
													sitalDeviceMemorySectionEnum dmseAccessedSection,
													U32BIT dwWordCountOrMask, // Interpreted as mask in masked operations, in which word count is always 1.
													U16BIT* wpIoContents);
_EXTERN S16BIT _DECL sitalDevice_ConfigureSimultaneousAccess (S16BIT swDevice, BOOLEAN bIsSimultaneousAccessEnabled);

// Process configuration & manipulation functions.
_EXTERN S16BIT _DECL sitalProcess_WaitForInterrupts (U32BIT dwMilliseconds, BOOLEAN bSingle);
_EXTERN S16BIT _DECL sitalProcess_SetInterruptServiceRoutine (ISR_FUNCTION funcpExternalIsr);
//_EXTERN S16BIT _DECL sitalProcess_SetInterruptServiceRoutine (S16BIT (_DECL *funcpExternalIsr) (const InterruptInformationStructure* iisp));
_EXTERN S16BIT _DECL sitalProcess_GetInterruptLatency (U32BIT* dwpMaximumInterruptLatency);
_EXTERN void _DECL sitalProcess_LibraryCleanup (void);

// Process oriented logging configuration & manipulation functions.
// When working with any Windows operating-system the path of the log is taken from registry value "file" under [HKEY_CURRENT_USER\Software\sital\trace-profiles\default].
// The name of this file may be set to either a specific path, "STDOUT", or "STDERR".
_EXTERN S16BIT _DECL sitalProcess_Log_Start (char const *szProfileName);
_EXTERN S16BIT _DECL sitalProcess_Log_PrintLine (char const *szFormat, ...);
_EXTERN S16BIT _DECL sitalProcess_Log_Stop (BOOLEAN bIsLogSaved);



#ifdef __cplusplus
}
#endif



#endif // stld1553DriverInterface_INCLUDED
