/// @file
/// stld1553 library's device memory management internal definitions and declarations.

// Description  : A header file that contains stld1553 library's device memory management internal definitions and declarations
// Version      : 1.0.0.0
// Author       : Sital Technology R&D
//
// Modification history:
//



#ifndef stld1553_memoryManager_internal_INCLUDED
#define stld1553_memoryManager_internal_INCLUDED



#ifdef __cplusplus
extern "C" {
#endif



// Compilation flag definitions.
//#define PRINT_MEMORY_MANAGEMENT_MESSAGES



// Enumeration definitions.



// Constant definitions.



// Macro definitions.

// Determine if given integer number is a multiplication of 32.
// Parameter[s]:
// - testedNumber (in):
//   - Description: A value to test.
//   - Type: Integer.
// Return value:
// - TRUE: If given integer number is a multiplication of 32.
// - FALSE: Otherwise.
#define IS_MULTIPLICATION_OF_32(testedNumber)	\
	( ((testedNumber) == (((testedNumber) >> 5) << 5)) ? TRUE : FALSE )

// Round up given integer number to the near multiplication of 32.
// Parameter[s]:
// - roundedNumber (in):
//   - Description: A value to round up to the near multiplication of 32, if not already a multiplication of 32.
//   - Type: Integer.
// Return value:
// - TRUE: If given integer number is a multiplication of 32.
// - FALSE: Otherwise.
#define ROUND_UP_TO_NEAR_MULTIPLICATION_OF_32(roundedNumber)	\
	{	\
		if (FALSE == (IS_MULTIPLICATION_OF_32(roundedNumber)))	\
		{	\
			(roundedNumber) = ((((roundedNumber) >> 5) << 5) + 32);	\
		}	\
	}

// Calculate the index of the bit that represents the 32-word segment to which given address belongs.
// Parameter[s]:
// - address (in):
//   - Description: An address in device memory.
//   - Type: Integer.
//   - Value: 0 to (sitalMaximum_SIZE_OF_DEVICE_MEMORY-1).
// Return value:
// - The index of the bit that represents the 32-word segment to which given address belongs.
#define ADDRESS_TO_INDEX_OF_RELATED_BIT(address)	\
	( (address) >> 5 )

// Turn device memory map entry index and entry bit index pair into the index of the bit that represents the same 32-word segment.
// Parameter[s]:
// - entry (in):
//   - Description: The index of a device memory map entry.
//   - Type: Integer.
//   - Value: 0 to ((sitalMaximum_SIZE_OF_DEVICE_MEMORY>>10) - 1).
// - entry bit (in):
//   - Description: The index of a bit within a device memory map entry.
//   - Type: Integer.
//   - Value: 0-31.
// Return value:
// - The index of the bit that represents given 32-word segment.
#define ENTRY_BIT_PAIR_TO_BIT_INDEX(entry,bit)	\
	( ((entry) << 5) | (bit) )

// Calculate the base address of the 32-word segment that's represented by the bit whose index is given.
// Parameter[s]:
// - bitIndex (in):
//   - Description: The index of a bit that represents a 32-word segment.
//   - Type: Integer.
//   - Value: 0 to ((sitalMaximum_SIZE_OF_DEVICE_MEMORY-1) >> 5).
// Return value:
// - The base address of the 32-word segment that's represented by the bit whose index is given.
#define SEGMENT_BASE_ADDRESS(bitIndex)	\
	( (bitIndex) << 5 )

// Calculate the bottom address of the 32-word segment that's represented by the bit whose index is given.
// Parameter[s]:
// - bitIndex (in):
//   - Description: The index of a bit that represents a 32-word segment.
//   - Type: Integer.
//   - Value: 0 to ((sitalMaximum_SIZE_OF_DEVICE_MEMORY-1) >> 5).
// Return value:
// - The bottom address of the 32-word segment that's represented by the bit whose index is given.
#define SEGMENT_BOTTOM_ADDRESS(bitIndex)	\
	( ((bitIndex) << 5) + 31 )

// Calculate the base address of the 32-word segment that follows the segment that's represented by the bit whose index is given.
// Parameter[s]:
// - bitIndex (in):
//   - Description: The index of a bit that represents a 32-word segment.
//   - Type: Integer.
//   - Value: 0 to ((sitalMaximum_SIZE_OF_DEVICE_MEMORY-1) >> 5).
// Return value:
// - The base address of the 32-word segment that follows the segment that's represented by the bit whose index is given.
// Warning[s]:
// - The user of this macro is responsible to verify that there is a following 32-word segment in device memory!
#define NEXT_SEGMENT_BASE_ADDRESS(bitIndex)	\
	( (SEGMENT_BOTTOM_ADDRESS(bitIndex)) + 1 )

// Calculate the bottom address of the 32-word segment that preceds the segment that's represented by the bit whose index is given.
// Parameter[s]:
// - bitIndex (in):
//   - Description: The index of a bit that represents a 32-word segment.
//   - Type: Integer.
//   - Value: 0 to ((sitalMaximum_SIZE_OF_DEVICE_MEMORY-1) >> 5).
// Return value:
// - The bottom address of the 32-word segment that precedes the segment that's represented by the bit whose index is given.
// Warning[s]:
// - The user of this macro is responsible to verify that there is a preceding 32-word segment in device memory!
#define PREVIOUS_SEGMENT_BOTTOM_ADDRESS(bitIndex)	\
	( (SEGMENT_BASE_ADDRESS(bitIndex)) - 1 )

// Find the indices of the device memory entry and bit that correspond the 32-word segment whose base address is given.
// Parameter[s]:
// - dwBaseAddress (in):
//   - Description: The base address of the a 32-word segment in device memory.
//   - Type: Integer.
//   - Value: 0 to (sitalMaximum_SIZE_OF_DEVICE_MEMORY-1), a multiplication of 32.
// - dwEntry (out):
//   - Description: The variable within which the index of the corresponding entry is returned.
//   - Type: Integer.
//   - Returned value: 0 to ((sitalMaximum_SIZE_OF_DEVICE_MEMORY-1) / 1024).
// - wBit (out):
//   - Description: The variable within which the index of the corresponding bit within the corresponding entry is returned (integer).
//   - Type: Integer.
//   - Returned value: 0-31.
#define FIND_CORRESPONDING_MEMORY_MAP_INDICES(dwBaseAddress,dwEntry,wBit) \
	{	\
		(dwEntry) = ((dwBaseAddress) >> 10);	\
		(wBit) = (U16BIT)((U32BIT)0x001F & ((dwBaseAddress) >> 5));	\
	}



// Type/structure definitions.



// Internal-function declarations.



// Exported-function declarations.



#ifdef __cplusplus
}
#endif



#endif // stld1553_memoryManager_internal_INCLUDED
