// Description  : A source file that contains the implementation of stld1553 library's device memory management functions
// Version      : 1.0.0.0
// Author       : Sital Technology R&D
//
// Modification history:
//



#include "stdafx.h"
#include <stdio.h>
#include "OperatingSystemDependencies.h"
#include "stld1553.h"
#include "stld1553_returnCodes.h"
#include "stld1553_driverInterface.h"
#include "stld1553_memoryManager.h"
#include "stld1553_memoryManager_internal.h"



#ifdef _MANAGED
#pragma managed(push, off)
#endif



// static variables.



// internal functions.

/// <summary>
/// Initialize device memory management for given device memory map.
///
/// @param hdmmspDeviceMemoryMap (in)	A pointer to a device memory map
/// @param dwRamSize (in)	The size (in words) of the RAM memory section of the device whose memory map is initialized.
/// @return sitalReturnCode_SUCCESS	Function successfully completed
/// @return Negative sitalReturnCode_*	Error condition or function failed
S16BIT Device_Memory_Initialize (HostDeviceMemoryMapStructure* const hdmmspDeviceMemoryMap, U32BIT dwRamSize)
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.
	if (sitalMaximum_SIZE_OF_DEVICE_MEMORY < dwRamSize)
	{
		return sitalReturnCode_INVALID_PARAMETER;
	}

	/// Reset top and bottom section borders.
	hdmmspDeviceMemoryMap->dwRamSize = dwRamSize;
	hdmmspDeviceMemoryMap->dwCurrentTopSectionBorderAddress = 0;
	hdmmspDeviceMemoryMap->dwCurrentBottomSectionBorderAddress = (hdmmspDeviceMemoryMap->dwRamSize - 1);

	/// Release any past allocations of device memory.
	void* vp = memset (hdmmspDeviceMemoryMap->dwaMemoryMap, 0, ((sitalMaximum_SIZE_OF_DEVICE_MEMORY >> 10) << 2) /* Number of BYTEs in a memory map */);

	return sitalReturnCode_SUCCESS;
}


/// <summary>
/// Free whole given region of word addresses (base to top, both included) of top section of given device memory map.
///
/// Note:
/// - The top (or regular) section of device memory is regularly used for allocations/deallocations.
///   That is, the user may directly/indirectly catch memory in this section, this way enlarging or shrinking this section,
///   and usually even turning it over time to intermittently contain strips of used and unused memory, what may conclude in non optimal usage of device memory.
///   Memory in this top section may be allocated/deallocated in parts, according to exact needs.
/// - The minimal unit of allocated/freed device memory is 32 words.
///   Yet, the caller is allowed to supply this function with base/bottom addresses which are not multiplications of 32.
///   In such cases this function will allocate/free the whole 32-word unit in which given base/bottom address falls.
///
/// @param hdmmspDeviceMemoryMap (in)	A pointer to a device memory map
/// @param dwBaseRegionAddress (in)	The starting address of a device memory region (>=0, <hdmmspDeviceMemoryMap->dwRamSize)
/// @param dwBottomRegionAddress (in)	The ending address of a device memory region (>=0, <hdmmspDeviceMemoryMap->dwRamSize)
/// @return sitalReturnCode_SUCCESS	Function successfully completed
/// @return Negative sitalReturnCode_*	Error condition or function failed
/// </summary>
S16BIT Device_Memory_TopSection_FreeRegion (HostDeviceMemoryMapStructure* const hdmmspDeviceMemoryMap, U32BIT dwBaseRegionAddress, U32BIT dwBottomRegionAddress)
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.
	if (((HostDeviceMemoryMapStructure*)NULL) == hdmmspDeviceMemoryMap)
	{
		return sitalReturnCode_INVALID_PARAMETER;
	}

	if	(
		(0 > dwBaseRegionAddress) ||
		(hdmmspDeviceMemoryMap->dwRamSize <= dwBaseRegionAddress)
		)
	{
		return sitalReturnCode_INVALID_PARAMETER;
	}

	if	(
		(0 > dwBottomRegionAddress) ||
		(hdmmspDeviceMemoryMap->dwCurrentTopSectionBorderAddress <= dwBottomRegionAddress)
		)
	{
		return sitalReturnCode_INVALID_PARAMETER;
	}

	if (dwBaseRegionAddress > dwBottomRegionAddress)
	{
		return sitalReturnCode_INVALID_PARAMETER;
	}

	/// With both given region specifiers, find the indices of the corresponding entry and bit in given device memory map.
	U32BIT dwBaseRegionEntry; // The index of the device memory map entry that contains the bit that corresponds with given region base address.
	U16BIT wBaseRegionBit; // The index of the bit that corresponds with given region base address (within the corresponding device memory map entry).
	FIND_CORRESPONDING_MEMORY_MAP_INDICES(dwBaseRegionAddress,dwBaseRegionEntry,wBaseRegionBit);

	U32BIT dwBottomRegionEntry; // The index of the device memory map entry that contains the bit that corresponds with given region bottom address.
	U16BIT wBottomRegionBit; // The index of the bit that corresponds with given region bottom address (within the corresponding device memory map entry).
	FIND_CORRESPONDING_MEMORY_MAP_INDICES(dwBottomRegionAddress,dwBottomRegionEntry,wBottomRegionBit);

	/// If both base and bottom segments are both mapped to [a] bit[s] of a common entry in their device map:
	///		Zero bits base-to-bottom of this common entry.
	/// Else:
	///		Zero bits base and above in the base entry.
	///		Zero any intermediate entries.
	///		Zero bits bottom and below in the bottom entry.
	U32BIT* dwpMemoryMapEntry; // A pointer to a memory map entry.
	if (dwBaseRegionEntry == dwBottomRegionEntry)
	{
		dwpMemoryMapEntry = &(hdmmspDeviceMemoryMap->dwaMemoryMap[dwBaseRegionEntry]);
		(*dwpMemoryMapEntry) &= ~((((0xFFFFFFFF >> wBaseRegionBit) << wBaseRegionBit) << (31 - wBottomRegionBit)) >> (31 - wBottomRegionBit));
	}
	else
	{
		dwpMemoryMapEntry = &(hdmmspDeviceMemoryMap->dwaMemoryMap[dwBaseRegionEntry]);
		(*dwpMemoryMapEntry) &= ~((0xFFFFFFFF >> wBaseRegionBit) << wBaseRegionBit);

		U32BIT dwEntryDifference; // The difference between the base and bottom entries (equals one in case of successive entries).
		dwEntryDifference = (dwBottomRegionEntry - dwBaseRegionEntry);
		if (1 < dwEntryDifference)
		{
			U32BIT dwIntermediateEntryCount; // The number of entries in-between the base and bottom ones.
			dwIntermediateEntryCount = (dwEntryDifference - 1);
			dwpMemoryMapEntry++;
			void* vp = memset (dwpMemoryMapEntry, 0, (dwIntermediateEntryCount << 2));
		}

		dwpMemoryMapEntry = &(hdmmspDeviceMemoryMap->dwaMemoryMap[dwBottomRegionEntry]);
		(*dwpMemoryMapEntry) &= ~((0xFFFFFFFF << (31 - wBottomRegionBit)) >> (31 - wBottomRegionBit));
	}

	/// If it is the bottom part of the top section of device memory that has been just freed:
	///		Update the record of the border address of the top section
	// Note that it isn't enough to move the border to the base address of the now-freed memory region, as the memory right before this address
	// may have been freed too already. The actual bottom of the top section of device memory must be therefore traced back.
	if (dwBottomRegionEntry == (hdmmspDeviceMemoryMap->dwCurrentTopSectionBorderAddress - 1))
	{
		// Loop backward from the bit that represents the top 32-word segment of the currently released region to the beginning of the device memory map.
		// Within this loop try to find the bottom allocated 32-word segment (if any 32-word segment is currently allocated at all).
		// Update the border of the top section of device memory to be the base address of the first-encountered free 32-word segment in this backward search.
		U32BIT dwBottomAddressOfCurrentSegment; // The bottom address that belongs to the 32-word segment that's represented by the currently scanned bit.
		dwBottomAddressOfCurrentSegment = (SEGMENT_BOTTOM_ADDRESS(ADDRESS_TO_INDEX_OF_RELATED_BIT(dwBaseRegionAddress)));
		const U32BIT* dwpCurrentEntry; // A pointer to the currently scanned device memory map entry.
		dwpCurrentEntry = &(hdmmspDeviceMemoryMap->dwaMemoryMap[dwBaseRegionEntry]);
		U16BIT wCurrentBit; // The index of the currently scanned bit of the currently scanned device memory map entry.
		wCurrentBit = wBaseRegionBit;
		const U32BIT* dwpFirstEntry; // A pointer to the first entry of given device memory map.
		dwpFirstEntry = hdmmspDeviceMemoryMap->dwaMemoryMap;
		BOOLEAN bIsSearchCompleted; // A flag that says whether the search after the current bottom of the top section of device memory has already completed.
		bIsSearchCompleted = FALSE;
		while (((dwpCurrentEntry > dwpFirstEntry) || (wCurrentBit > 0)) && (FALSE == bIsSearchCompleted))
		{
			if (((U32BIT)0U) == (((U32BIT)(*dwpCurrentEntry)) & (((U32BIT)0x0001) << wCurrentBit)))
			{
				// Current bit represents a currently free 32-word segment:
				// Move still backward in search for the actual bottom of top section of device memory.
				if (0U == wCurrentBit)
				{
					dwpCurrentEntry--;
					wCurrentBit = 31;
				}
				else
				{
					wCurrentBit--;
				}

				dwBottomAddressOfCurrentSegment -=32;
			}
			else
			{
				// Current bit represents a currently allocated 32-word segment:
				// Found the actual bottom of the top section of device memory.
				hdmmspDeviceMemoryMap->dwCurrentTopSectionBorderAddress = (dwBottomAddressOfCurrentSegment + 1);
				bIsSearchCompleted = TRUE;
			}
		}

		// If found no allocated memory:
		// - Reset the record of the bottom of the top section of device memory.
		if (FALSE == bIsSearchCompleted)
		{
			hdmmspDeviceMemoryMap->dwCurrentTopSectionBorderAddress = 0;
		}
	}

#if _DEBUG
#ifdef PRINT_MEMORY_MANAGEMENT_MESSAGES
	sitalProcess_Log_PrintLine ("#StldDll.MemMan.Top.Free: base_addr-0x%X, bottom_addr-0x%X, section_bottom-0x%X\n", dwBaseRegionAddress, dwBottomRegionAddress, hdmmspDeviceMemoryMap->dwCurrentTopSectionBorderAddress);
#endif // PRINT_MEMORY_MANAGEMENT_MESSAGES
#endif // _DEBUG

	return sitalReturnCode_SUCCESS;
}


/// <summary>
/// Sign whole given region of word addresses (base to top, both included) of top section of given device memory map as allocated.
/// Memory region that's signed this way may not be later allocated per request, until it is freed.
///
/// Note:
/// - The top (or regular) section of device memory is regularly used for allocations/deallocations.
///   That is, the user may directly/indirectly catch memory in this section, this way enlarging or shrinking this section,
///   and usually even turning it over time to intermittently contain strips of used and unused memory, what may conclude in non optimal usage of device memory.
///   Memory in this top section may be allocated/deallocated in parts, according to exact needs.
/// - The minimal unit of allocated/freed device memory is 32 words.
///   Yet, the caller is allowed to supply this function with base/bottom addresses which are not multiplications of 32.
///   In such cases this function will allocate/free the whole 32-word unit in which given base/bottom address falls.
///
/// @param hdmmspDeviceMemoryMap (in)	A pointer to a device memory map
/// @param dwBaseRegionAddress (in)	The starting address of a device memory region (>=0, <hdmmspDeviceMemoryMap->dwRamSize)
/// @param dwBottomRegionAddress (in)	The ending address of a device memory region (>=0, <hdmmspDeviceMemoryMap->dwRamSize)
/// @return sitalReturnCode_SUCCESS	Function successfully completed
/// @return Negative sitalReturnCode_*	Error condition or function failed
/// </summary>
S16BIT Device_Memory_TopSection_CatchRegion (HostDeviceMemoryMapStructure* const hdmmspDeviceMemoryMap, U32BIT dwBaseRegionAddress, U32BIT dwBottomRegionAddress)
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.
	if (((HostDeviceMemoryMapStructure*)NULL) == hdmmspDeviceMemoryMap)
	{
		return sitalReturnCode_INVALID_PARAMETER;
	}

	if	(
			(0 > dwBaseRegionAddress) ||
			(hdmmspDeviceMemoryMap->dwRamSize <= dwBaseRegionAddress)
		)
	{
		return sitalReturnCode_INVALID_PARAMETER;
	}

	if	(
			(0 > dwBottomRegionAddress) ||
			(hdmmspDeviceMemoryMap->dwCurrentBottomSectionBorderAddress < dwBottomRegionAddress)
		)
	{
		return sitalReturnCode_INVALID_PARAMETER;
	}

	if (dwBaseRegionAddress > dwBottomRegionAddress)
	{
		return sitalReturnCode_INVALID_PARAMETER;
	}

	/// With both given region specifiers, find the indices of the corresponding entry and bit in given device memory map.
	U32BIT dwBaseRegionEntry; // The index of the device memory map entry that contains the bit that corresponds with given region base address.
	U16BIT wBaseRegionBit; // The index of the bit that corresponds with given region base address (within the corresponding device memory map entry).
	FIND_CORRESPONDING_MEMORY_MAP_INDICES(dwBaseRegionAddress,dwBaseRegionEntry,wBaseRegionBit);

	U32BIT dwBottomRegionEntry; // The index of the device memory map entry that contains the bit that corresponds with given region bottom address.
	U16BIT wBottomRegionBit; // The index of the bit that corresponds with given region bottom address (within the corresponding device memory map entry).
	FIND_CORRESPONDING_MEMORY_MAP_INDICES(dwBottomRegionAddress,dwBottomRegionEntry,wBottomRegionBit);

	/// If both base and bottom segments are both mapped to [a] bit[s] of a common entry in their device map:
	///		Zero bits base-to-bottom of this common entry.
	/// Else:
	///		Zero bits base and above in the base entry.
	///		Zero any intermediate entries.
	///		Zero bits bottom and below in the bottom entry.
	U32BIT* dwpMemoryMapEntry; // A pointer to a memory map entry.
	if (dwBaseRegionEntry == dwBottomRegionEntry)
	{
		dwpMemoryMapEntry = &(hdmmspDeviceMemoryMap->dwaMemoryMap[dwBaseRegionEntry]);
		(*dwpMemoryMapEntry) |= ((((0xFFFFFFFF >> wBaseRegionBit) << wBaseRegionBit) << (31 - wBottomRegionBit)) >> (31 - wBottomRegionBit));
	}
	else
	{
		dwpMemoryMapEntry = &(hdmmspDeviceMemoryMap->dwaMemoryMap[dwBaseRegionEntry]);
		(*dwpMemoryMapEntry) |= ((0xFFFFFFFF >> wBaseRegionBit) << wBaseRegionBit);

		U32BIT dwEntryDifference; // The difference between the base and bottom entries (equals one in case of successive entries).
		dwEntryDifference = (dwBottomRegionEntry - dwBaseRegionEntry);
		if (1 < dwEntryDifference)
		{
			U32BIT dwIntermediateEntryCount; // The number of entries in-between the base and bottom ones.
			dwIntermediateEntryCount = (dwEntryDifference - 1);
			dwpMemoryMapEntry++;
			void* vp = memset (dwpMemoryMapEntry, 0xFF, (dwIntermediateEntryCount << 2));
		}

		dwpMemoryMapEntry = &(hdmmspDeviceMemoryMap->dwaMemoryMap[dwBottomRegionEntry]);
		(*dwpMemoryMapEntry) |= ((0xFFFFFFFF << (31 - wBottomRegionBit)) >> (31 - wBottomRegionBit));
	}

	/// If the top section of device memory has been just enlarged towards the bottom of device memory:
	///		Update the record of the border address of the top section
	if (dwBottomRegionAddress >= hdmmspDeviceMemoryMap->dwCurrentTopSectionBorderAddress)
	{
		hdmmspDeviceMemoryMap->dwCurrentTopSectionBorderAddress = (NEXT_SEGMENT_BASE_ADDRESS(ADDRESS_TO_INDEX_OF_RELATED_BIT(dwBottomRegionAddress)));
	}

#if _DEBUG
#ifdef PRINT_MEMORY_MANAGEMENT_MESSAGES
	sitalProcess_Log_PrintLine ("#StldDll.MemMan.Top.Catch: base_addr-0x%X, bottom_addr-0x%X, section_bottom-0x%X\n", dwBaseRegionAddress, dwBottomRegionAddress, hdmmspDeviceMemoryMap->dwCurrentTopSectionBorderAddress);
#endif // PRINT_MEMORY_MANAGEMENT_MESSAGES
#endif // _DEBUG

	return sitalReturnCode_SUCCESS;
}


/// <summary>
/// Allocate a block of given size (in words) of continuous memory in given region of word addresses (base to top, both included) of top section of given device memory map.
///
/// Note:
/// - The top (or regular) section of device memory is regularly used for allocations/deallocations.
///   That is, the user may directly/indirectly catch memory in this section, this way enlarging or shrinking this section,
///   and usually even turning it over time to intermittently contain strips of used and unused memory, what may conclude in non optimal usage of device memory.
///   Memory in this top section may be allocated/deallocated in parts, according to exact needs.
/// - The minimal unit of allocated/freed device memory is 32 words, and memory is allocated in units of 32 words.
///   In case the caller specifies an allocation size which isn't a multiplication of 32 words, this function round it up to the near multiplication of 32 words.
/// - The caller is allowed to supply this function with base/bottom addresses which are not multiplications of 32.
///   In such cases this function will avoid using for allocations the whole 32-word units in which given base/bottom addresses fall.
///
/// @param hdmmspDeviceMemoryMap (in)	A pointer to a device memory map
/// @param dwBaseRegionAddress (in)	The starting address of a device memory region (>=0, <hdmmspDeviceMemoryMap->dwRamSize)
/// @param dwBottomRegionAddress (in)	The ending address of a device memory region (>=0, <hdmmspDeviceMemoryMap->dwRamSize)
/// @param dwpAllocationSize (in)	A pointer to a variable in which the size (in words) of the allocation is given, and within which the size of the actual allocation is returned
/// @param wAlignment (in)	An exponent of 2 so that the power (2^wAlignment) will be a divider of the allocation base address (5-31)
/// @param bpIsAllocationSuccessfullyAccomplished (out)	A pointer to a flag variable within which the caller is reported whether the allocation succeeded/failed (TesterDll_FALSE or TesterDll_TRUE)
/// @param dwpAllocationBaseAddress (out)	A pointer to a variable within which the base address of the allocated area is returned in case the allocation succeeded
/// @return sitalReturnCode_SUCCESS	Function successfully completed
/// @return Negative sitalReturnCode_*	Error condition or function failed
/// </summary>
S16BIT Device_Memory_TopSection_AllocateContinuity (	HostDeviceMemoryMapStructure* hdmmspDeviceMemoryMap,
														U32BIT dwBaseRegionAddress,
														U32BIT dwBottomRegionAddress,
														U32BIT* dwpAllocationSize,
														U16BIT wAlignment,
														BOOLEAN* bpIsAllocationSuccessfullyAccomplished,
														U32BIT* dwpAllocationBaseAddress)
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.
	if (((HostDeviceMemoryMapStructure*)NULL) == hdmmspDeviceMemoryMap)
	{
		return sitalReturnCode_INVALID_PARAMETER;
	}

	if	(0 > dwBaseRegionAddress)
	{
		return sitalReturnCode_INVALID_PARAMETER;
	}

	/// In case of a base address which isn't a multiplication of 32 is given:
	///		Avoid using for allocations the whole 32-word unit in which given base address falls.
	// This is achieved by simply setting the relevant address to the nearest higher multiplication of 32.
	if (FALSE == (IS_MULTIPLICATION_OF_32(dwBaseRegionAddress)))
	{
		dwBaseRegionAddress = ( ((dwBaseRegionAddress >> 5) + 1) << 5 );
	}

	if	(hdmmspDeviceMemoryMap->dwRamSize <= dwBaseRegionAddress)
	{
		return sitalReturnCode_INVALID_PARAMETER;
	}

	if	(31 > dwBottomRegionAddress)
	{
		return sitalReturnCode_INVALID_PARAMETER;
	}

	/// In case that given bottom address does not give a reminder of 31 when divided by 32:
	///		Avoid using for allocations the whole 32-word unit in which given bottom address falls.
	// This is achieved by simply setting the relevant address to the nearest lower that gives a reminder of 31 when divided by 32.
	// Recall/note that:
	// - Th bottom address is greater/equal 31.
	// - A dwBottomRegionAddress of 31 stands above term, so we have to fix only a bottom address which is greater/equal 32;
	//   With such bottom addresses, there is a 32-word segment to move backward to.
	if (FALSE == (IS_MULTIPLICATION_OF_32(dwBottomRegionAddress - 31)))
	{
		dwBottomRegionAddress = ( ( ((dwBottomRegionAddress >> 5) - 1) << 5 ) + 31 );
	}

	if	(hdmmspDeviceMemoryMap->dwRamSize <= dwBottomRegionAddress)
	{
		return sitalReturnCode_INVALID_PARAMETER;
	}

	if	(dwBaseRegionAddress >= dwBottomRegionAddress)
	{
		return sitalReturnCode_INVALID_PARAMETER;
	}

	if (0 >= (*dwpAllocationSize))
	{
		return sitalReturnCode_INVALID_PARAMETER;
	}

	U32BIT dwRequestedAllocationSize; // The originally requested size (in words) of device memory allocation.
	dwRequestedAllocationSize = (*dwpAllocationSize);
	ROUND_UP_TO_NEAR_MULTIPLICATION_OF_32(*dwpAllocationSize);

	if ((1 + dwBottomRegionAddress - dwBaseRegionAddress) < (*dwpAllocationSize))
	{
		return sitalReturnCode_INVALID_PARAMETER;
	}

	if ((5 > wAlignment) || (31 < wAlignment))
	{
		return sitalReturnCode_INVALID_PARAMETER;
	}

	if ( (((BOOLEAN*)NULL) == bpIsAllocationSuccessfullyAccomplished) || (((U32BIT*)NULL) == dwpAllocationBaseAddress) )
	{
		return sitalReturnCode_INVALID_PARAMETER;
	}

	/// With both given region specifiers, find the indices of the corresponding entry and bit in given device memory map.
	U32BIT dwBaseRegionEntry; // The index of the device memory map entry that contains the bit that corresponds with given region base address.
	U16BIT wBaseRegionBit; // The index of the bit that corresponds with given region base address (within the corresponding device memory map entry).
	FIND_CORRESPONDING_MEMORY_MAP_INDICES(dwBaseRegionAddress,dwBaseRegionEntry,wBaseRegionBit);

	U32BIT dwBottomRegionEntry; // The index of the device memory map entry that contains the bit that corresponds with given region bottom address.
	U16BIT wBottomRegionBit; // The index of the bit that corresponds with given region bottom address (within the corresponding device memory map entry).
	FIND_CORRESPONDING_MEMORY_MAP_INDICES(dwBottomRegionAddress,dwBottomRegionEntry,wBottomRegionBit);

	if	(
			(dwBaseRegionEntry > dwBottomRegionEntry)
			||
			( (dwBaseRegionEntry == dwBottomRegionEntry) && (wBaseRegionBit > wBottomRegionBit) )
		)
	{
		return sitalReturnCode_INVALID_PARAMETER;
	}

	/// Calculate the number of continuous free bits that's required in given device memory region in order to support given allocation request.
	U32BIT dwAllocationBitSize; // The number of continuous free bits that's required in given device memory region in order to support given allocation request.
	dwAllocationBitSize = ((*dwpAllocationSize) >> 5);

	/// Loop over all the bits of given region in effort to find a piece of free space large enough for the requested allocation:
	///		If currently tested address stands in given alignment request:
	///			Loop starting from current region bit and on to check whether there's enough free space to support the requested allocation:
	///				If currently tested bit is signed as allocated:
	///					Flag that there are not enough free bits starting from the currently scanned bit.
	///				Else:
	///					Proceed to the next entry - bit pair to test in the device memory map.
	///					Increment the number of free bits that were already found.
	///		If found enough free space to support the requested allocation:
	///			If the 'free' space that was found does not invade the bottom section of device memory:
	///				Use this free space to perform the requested allocation.
	///			Properly respond the caller using supplied pointer parameters.
	///		Else:
	///			Proceed to the next entry - bit pair to scan in the device memory map.
	U32BIT* dwpCurrentRegionEntry; // A pointer to the currently scanned device memory map entry.
	dwpCurrentRegionEntry = &(hdmmspDeviceMemoryMap->dwaMemoryMap[dwBaseRegionEntry]);
	const U32BIT* dwpBottomRegionEntry; // A pointer to the last scanned device memory map entry.
	dwpBottomRegionEntry = &(hdmmspDeviceMemoryMap->dwaMemoryMap[dwBottomRegionEntry]);
	U16BIT wCurrentRegionBit; // The index of the currently scanned bit of the currently scanned device memory map entry.
	wCurrentRegionBit = wBaseRegionBit;
	BOOLEAN bIsRequestHandlingCompleted; // A flag that says whether allocation request has already been handled.
	bIsRequestHandlingCompleted = FALSE;
	(*bpIsAllocationSuccessfullyAccomplished) = FALSE;
	(*dwpAllocationBaseAddress) = dwBaseRegionAddress;
	while (((dwpCurrentRegionEntry < dwpBottomRegionEntry) || ((dwpCurrentRegionEntry == dwpBottomRegionEntry) && (wCurrentRegionBit <= wBottomRegionBit))) && (FALSE == bIsRequestHandlingCompleted))
	{
		// If currently tested address stands in given alignment request:
		// Loop starting from current region bit to check whether there's enough free space to support the requested allocation.
		U32BIT dwFreeBitCount; // The number of continuous free bits that where found already starting at the currently scanned bit of device memory map.
		dwFreeBitCount = 0;
		if ((*dwpAllocationBaseAddress) == (((*dwpAllocationBaseAddress) >> wAlignment) << wAlignment))
		{
			const U32BIT* dwpCurrentTestedEntry; // A pointer to the device memory map entry whose availability is currently tested.
			dwpCurrentTestedEntry = dwpCurrentRegionEntry;
			U16BIT wCurrentTestedBit;  // The index of the device memory map bit whose availability is currently tested.
			wCurrentTestedBit = wCurrentRegionBit;
			BOOLEAN bIsAllocationPossibility; // A flag that says whether the availability of enough free space starting from currently scanned bit of device memory map is still an option.
			bIsAllocationPossibility = TRUE;
			while ((dwFreeBitCount < dwAllocationBitSize) && ((dwpCurrentRegionEntry < dwpBottomRegionEntry) || ((dwpCurrentRegionEntry == dwpBottomRegionEntry) && (wCurrentRegionBit <= wBottomRegionBit))) && (TRUE == bIsAllocationPossibility))
			{
				// If currently tested bit is signed as allocated: Flag that there are not enough free bits starting from the currently scanned bit.
				// Else proceed to the next entry - bit pair to test in the device memory map, and increment the number of free bits that were already found.
				if (((U32BIT)0U) != (((U32BIT)(*dwpCurrentTestedEntry)) & (((U32BIT)0x0001) << wCurrentTestedBit)))
				{
					bIsAllocationPossibility = FALSE;
				}
				else
				{
					wCurrentTestedBit++;
					if (32 == wCurrentTestedBit)
					{
						dwpCurrentTestedEntry++;
						wCurrentTestedBit = 0;
					}

					dwFreeBitCount++;
				}
			}
		}

		// If found enough free space to support the requested allocation:
		// - If the 'free' space that was found does not invade the bottom section of device memory:
		//   - Use this free space to perform the requested allocation.
		// - Properly respond the caller using supplied pointer parameters.
		// Else:
		// - Proceed to the next entry - bit pair to scan in the device memory map.
		if (dwFreeBitCount == dwAllocationBitSize)
		{
			// In case the next function call succeeds, then the required allocation has succeeded.
			// The only reason for this function call to fail is that the suggested allocation invades the bottom section of device memory.
			// In that case there's no reason to continue searching for enough free space, as every free block will be beyond the possible limit as well.
			// So in either case the search for free memory has ended.
			if (sitalReturnCode_SUCCESS == Device_Memory_TopSection_CatchRegion (hdmmspDeviceMemoryMap, (*dwpAllocationBaseAddress), ((*dwpAllocationBaseAddress) + (*dwpAllocationSize) - 1)))
			{
				(*bpIsAllocationSuccessfullyAccomplished) = TRUE;
			}
			bIsRequestHandlingCompleted = TRUE;
		}
		else
		{
			wCurrentRegionBit++;
			if (32 == wCurrentRegionBit)
			{
				dwpCurrentRegionEntry++;
				wCurrentRegionBit = 0;
			}

			(*dwpAllocationBaseAddress) += 32;
		}
	}

#if _DEBUG
#ifdef PRINT_MEMORY_MANAGEMENT_MESSAGES
	sitalProcess_Log_PrintLine ("#StldDll.MemMan.Top.Alloc: base_addr-0x%X, bottom_addr-0x%X, req_size-0x%X, alloc_size-0x%X, align - %i, success - %i, alloc_base-0x%X, section_bottom-0x%X\n", dwBaseRegionAddress, dwBottomRegionAddress, dwRequestedAllocationSize, (*dwpAllocationSize), wAlignment, (*bpIsAllocationSuccessfullyAccomplished), (*dwpAllocationBaseAddress), hdmmspDeviceMemoryMap->dwCurrentTopSectionBorderAddress);
#endif // PRINT_MEMORY_MANAGEMENT_MESSAGES
#endif // _DEBUG

	return sitalReturnCode_SUCCESS;
}


/// <summary>
/// Free all existing memory allocations in the bottom section of given device memory map.
///
/// Note:
/// - The bottom (or special) section of device memory may be used for a series of allocations that will be all deallocated at once.
///   With each allocation the bottom section of device memory is enlarged, and upon deallocation it is thoroughly shrank.
///   In consequence of the requirement to release all the allocations within the bottom section at once, this section may not intermittently
///   contain strips of used and unused memory (if not taking in account alignment issues), what may conclude in optimal usage of device memory.
///   Besides, this scheme enables the user to minimize the appearance of 'holes' of unused memory within the top section of device memory.
///
/// @param hdmmspDeviceMemoryMap (in)	A pointer to a device memory map
/// @return sitalReturnCode_SUCCESS	Function successfully completed
/// @return Negative sitalReturnCode_*	Error condition or function failed
/// </summary>
S16BIT Device_Memory_BottomSection_FreeAll (HostDeviceMemoryMapStructure* const hdmmspDeviceMemoryMap)
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.
	if (((HostDeviceMemoryMapStructure*)NULL) == hdmmspDeviceMemoryMap)
	{
		return sitalReturnCode_INVALID_PARAMETER;
	}

	/// Reset bottom section border.
	hdmmspDeviceMemoryMap->dwCurrentBottomSectionBorderAddress = (hdmmspDeviceMemoryMap->dwRamSize - 1);

#if _DEBUG
#ifdef PRINT_MEMORY_MANAGEMENT_MESSAGES
	sitalProcess_Log_PrintLine ("#StldDll.MemMan.Bottom.FreeAll: section_top-0x%X\n", hdmmspDeviceMemoryMap->dwCurrentBottomSectionBorderAddress);
#endif // PRINT_MEMORY_MANAGEMENT_MESSAGES
#endif // _DEBUG

	return sitalReturnCode_SUCCESS;
}


/// <summary>
/// Allocate a block of given size (in words) of continuous memory in the bottom section of given device memory map.
///
/// Note:
/// - The bottom (or special) section of device memory may be used for a series of allocations that will be all deallocated at once.
///   With each allocation the bottom section of device memory is enlarged, and upon deallocation it is thoroughly shrank.
///   In consequence of the requirement to release all the allocations within the bottom section at once, this section may not intermittently
///   contain strips of used and unused memory (if not taking in account alignment issues), what may conclude in optimal usage of device memory.
///   Besides, this scheme enables the user to minimize the appearance of 'holes' of unused memory within the top section of device memory.
/// - The minimal unit of allocated/freed device memory is 32 words, and memory is allocated in units of 32 words.
///   In case the caller specifies an allocation size which isn't a multiplication of 32 words, this function round it up to the near multiplication of 32 words.
///
/// @param hdmmspDeviceMemoryMap (in)	A pointer to a device memory map
/// @param dwpAllocationSize (in)	A pointer to a variable in which the size (in words) of the allocation is given, and within which the size of the actual allocation is returned
/// @param wAlignment (in)	An exponent of 2 so that it is a request that (2^wAlignment) will be a divider of the allocation base address (5-31)
/// @param bpIsAllocationSuccessfullyAccomplished (out)	A pointer to a flag variable within which the caller is reported whether the allocation succeeded (TesterDll_FALSE or TesterDll_TRUE)
/// @param dwpAllocationBaseAddress (out)	A pointer to a variable within which the base address of the allocated area is returned in case the allocation succeeded
/// @return sitalReturnCode_SUCCESS	Function successfully completed
/// @return Negative sitalReturnCode_*	Error condition or function failed
/// </summary>
S16BIT Device_Memory_BottomSection_AllocateContinuity (	HostDeviceMemoryMapStructure* const hdmmspDeviceMemoryMap,
														U32BIT* dwpAllocationSize,
														U16BIT wAlignment,
														BOOLEAN* bpIsAllocationSuccessfullyAccomplished,
														U32BIT* dwpAllocationBaseAddress)
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.
	if (((HostDeviceMemoryMapStructure*)NULL) == hdmmspDeviceMemoryMap)
	{
		return sitalReturnCode_INVALID_PARAMETER;
	}

	if (0 >= (*dwpAllocationSize))
	{
		return sitalReturnCode_INVALID_PARAMETER;
	}

#if _DEBUG
#ifdef PRINT_MEMORY_MANAGEMENT_MESSAGES
	U32BIT dwRequestedAllocationSize; // The originally requested size (in words) of device memory allocation.
	dwRequestedAllocationSize = (*dwpAllocationSize);
#endif // PRINT_MEMORY_MANAGEMENT_MESSAGES
#endif // _DEBUG
	ROUND_UP_TO_NEAR_MULTIPLICATION_OF_32(*dwpAllocationSize);

	if ((5 > wAlignment) || (31 < wAlignment))
	{
		return sitalReturnCode_INVALID_PARAMETER;
	}

	if ( (((BOOLEAN*)NULL) == bpIsAllocationSuccessfullyAccomplished) || (((U32BIT*)NULL) == dwpAllocationBaseAddress) )
	{
		return sitalReturnCode_INVALID_PARAMETER;
	}

	(*bpIsAllocationSuccessfullyAccomplished) = FALSE;

	/// Use the allocation size and the current border of the bottom section of device memory to calculate a potential allocation base address.
	// Recall/note that:
	// - Allocation size is a multiplication of 32.
	// - The border of the bottom section of device memory gives a reminder of 31 when divided by 32.
	// - The following line of code therefore calculates a potential allocation base address that is a multiplication of 32.
	(*dwpAllocationBaseAddress) = (hdmmspDeviceMemoryMap->dwCurrentBottomSectionBorderAddress + 1 - (*dwpAllocationSize));

	/// While there's still free space between top and bottom sections of device memory AND current potential allocation base address isn't well aligned:
	///		Move a single 32-word segment backwards (i.e., towards the top section of device memory).
	// Recall/note that:
	// - The current potential allocation base address is a multiplication of 32, and must stay a multiplication of 32.
	// - The border of the top section of device memory is a multiplication of 32.
	while (((*dwpAllocationBaseAddress) >= hdmmspDeviceMemoryMap->dwCurrentTopSectionBorderAddress) && ((*dwpAllocationBaseAddress) != (((*dwpAllocationBaseAddress) >> wAlignment) << wAlignment)))
	{
		(*dwpAllocationBaseAddress) -= 32;
	}

	/// If found enough free device memory for the bottom section to spread and the current potential allocation base address is well aligned:
	///		Actually use the current potential allocation base address as allocation base address.
	///		Properly update the border of the bottom section of device memory.
	///	Properly respond the caller using supplied pointer parameters.
	// Recall/note that the border of the bottom section of device memory gives a reminder of 31 when divided by 32, and must stay so.
	if	(
			((*dwpAllocationBaseAddress) >= hdmmspDeviceMemoryMap->dwCurrentTopSectionBorderAddress)
			&&
			((*dwpAllocationBaseAddress) == (((*dwpAllocationBaseAddress) >> wAlignment) << wAlignment))
		)
	{
		(*bpIsAllocationSuccessfullyAccomplished) = TRUE;
		hdmmspDeviceMemoryMap->dwCurrentBottomSectionBorderAddress = ((*dwpAllocationBaseAddress) - 1);
	}

#if _DEBUG
#ifdef PRINT_MEMORY_MANAGEMENT_MESSAGES
	sitalProcess_Log_PrintLine ("#StldDll.MemMan.Bottom.Alloc: req_size-0x%X, alloc_size-0x%X, align - %i, success - %i, alloc_base-0x%X, section_top-0x%X\n", dwRequestedAllocationSize, (*dwpAllocationSize), wAlignment, (*bpIsAllocationSuccessfullyAccomplished), (*dwpAllocationBaseAddress), hdmmspDeviceMemoryMap->dwCurrentBottomSectionBorderAddress);
#endif // PRINT_MEMORY_MANAGEMENT_MESSAGES
#endif // _DEBUG

	return sitalReturnCode_SUCCESS;
}



// Exported functions.



#ifdef _MANAGED
#pragma managed(pop)
#endif

