// Description  : Sital's stld1553 library test functions that have been originally implemented by DDC and by its clients.
// Version      : 1.0.0.0
// Author       : Sital Technology R&D
//
// Note:
// - All configuration functions must adhere the following rules in order to be used with Sital's TestPlanRunner app:
//   - have one of the following signatures:
//     - S16BIT _DECL <name-of-configuration-function> ()
//     - S16BIT _DECL <name-of-configuration-function> (int iIntegerParameter)
//     - S16BIT _DECL <name-of-configuration-function> (char* szNullTerminatedStringParameter)
//   - Return the following values:
//     - sitalReturnCode_SUCCESS if function successfully completed.
//     - Negative sitalReturnCode_* if an error condition encountered or function failed.
// - All test functions must adhere the following rules in order to be used with Sital's TestPlanRunner app:
//   - have the following signature: S16BIT _DECL <name-of-test-function> ()
//   - Return the following values:
//     - sitalReturnCode_SUCCESS if function successfully completed.
//     - Negative sitalReturnCode_* if an error condition encountered or function failed.
// - Sital supports the following architectures:
//   - Using BOTH Sital and DDC function, structure, and constant names over Sital's stld1553 library.
//     Such users must BOTH define macro ENABLE_USAGE_OF_DDC_NAMING *AND* undefine macro USE_DDC_LIBRARY.
//   - Using BOTH Sital and DDC function, structure, and constant names over Sital's DDC-like library.
//     Such users must define macro USE_DDC_LIBRARY.
//   (See both macro definitions below.)
// - With all below defined Acceptance Test Plan functions stldTester_Atp_DoTest*:
//   - A list of participant devices in the following form is specified for each such test function:
//     Participant[s]: <list of devices>.
//     Within this list, each device is specified in the pattern <device-type>[<device-logical-number>] (e.g., RT, BC, MT, 1, 2, 3, 4, RT1, BC2, MT3, RT4), where:
//     - <device-type> is either TEST, RT, BC, or MT.
//       In case the device-type isn't specified, it means that given device is used, but as a general device on its card, and not as a device of a specific device-type.
//     - <device-logical-number> is the index of a below defined s_swDevice* ATP variable that is used by this test function for the related device.
//       In case the device-logical-index isn't specified, it means that such a device is used, but its device-logical-number may not be dynamically configured by the caller.
//   - The addresses that are set for participating RTs arte as follows: s_wRtAddress1 for the first listed RT, s_wRtAddress2 for the second one, &c.
//
// Modification history:
// Dec 2018, ECIL 5 MHz project modifications
// NOTE - same IP as original 1 MHz project runs with a clock frequency that is 5 times faster.
// As a result, all counters such as frame time counter now run 5 times faster.
// for example, 20 ms frame time will result in 5 ms frame time run. 
// A general parameter named IP_FREQUENCY_FACTOR, it is either 1 (1MHz) or 5 (5MHz). Will influence all time measurements and comparisons.
//

#include "stdafx.h"
#include <stdio.h>
#include <stdlib.h>
#include "OperatingSystemDependencies.h"
#include "PrecisionTimer.h"
#include "InterProcessSemaphore.h"
//#define USE_DDC_LIBRARY
//#define ENABLE_USAGE_OF_DDC_NAMING
// Note:
// - In case the library that's actually used while running is:
//   - Sital's own stld1553 library:
//     - Un-define USE_DDC_LIBRARY for both Debug and Release versions in Project Properties > C/C++ > Preprocessor Definitions (may also be done below, uniquely within each file where required).
//     - If in below defined function BOTH Sital and DDC function, structure, and constant names are used: define ENABLE_USAGE_OF_DDC_NAMING for both Debug and Release versions in Project Properties > C/C++ > Preprocessor Definitions (may also be done below, uniquely within each file where required).
//     - Add "Debug\Stld1553.lib" for the debug version and "Release\Stld1553.lib" for the release version in Project Properties > Linker > Input > Additional Dependencies.
//   - Either DDC's original library over DDC cards *OR* Sital's DDC-like library over Sital card:
//     - Add "..\DdcRemakeOverSitalCards\stlemace\Include" for both Debug and Release versions in Project Properties > C/C++ > General > Additional Include Directories.
//     - Add the path to the library to use in Project Properties > Linker > Input > Additional Dependencies.
//       E.g., for Sital's DDC-like library over Sital card add "..\DdcRemakeOverSitalCards\emace_32.lib" for both Debug and Release versions.
#ifndef USE_DDC_LIBRARY
#include "stld1553_returnCodes.h"
#include "stld1553.h"
#ifdef ENABLE_USAGE_OF_DDC_NAMING
#include "ddc2stld1553.h"
#endif // ENABLE_USAGE_OF_DDC_NAMING
#else // USE_DDC_LIBRARY
#include "stdemace.h" // File located at directory "..\DdcRemakeOverSitalCards\stlemace\Include".
#include "UniqueStldDefs.h"
#include "stld2ddc.h"
#endif // USE_DDC_LIBRARY
#include "stld1553_driverInterface.h"
#include "TestsBySital.h"



// Compilation flag definitions.



// Constant definitions.

// When using the DDC (or a DDC-like) library, add declarations and definitions that are unique to Sital and aren't exported by DDC.
#ifdef USE_DDC_LIBRARY

#define commandWord_OFFSET_OF_RT_ADDRESS															11		///< The index of the lowest of the bits of the RT address field.
#define commandWord_RT_ADDRESS																		0xF800	///< The RT address field mask (all these bit are on in case of broadcast).
#define commandWord_BROADCAST																		0xF800	///< The value of the RT address field mask in case of broadcast command.
#define commandWord_OFFSET_OF_RX_OR_TX																10		///< The index of the bit of the Rx/Tx field.
#define commandWord_RX_OR_TX																		0x0400	///< The Rx/Tx field mask (bit on in case of TX).
#define commandWord_TX																				0x0400	///< The value of the Rx/Tx field in case of TX command.
#define commandWord_OFFSET_OF_SUBADDRESS_OR_MODE													5		///< The index of the lowest of the bits of the subaddress/mode field.
#define commandWord_SUBADDRESS_OR_MODE																0x03E0	///< The subaddress/mode field mask (all bits are either on or off in case of a mode command).
#define commandWord_MODE																			0x0000	///< The value of the subaddress/mode field in case of mode command.
#define commandWord_OFFSET_OF_WORD_COUNT_OR_MODE_CODE												0		///< The index of the lowest of the bits of the word-count/mode-code field.
#define commandWord_WORD_COUNT_OR_MODE_CODE															0x001F	///< The word-count/mode-code field mask.
#define commandWord_MODE_WITH_OR_WITHOUT_DATA														0x0010	///< The mode with data field mask (bit on in case of a mode with data command).
#define commandWord_MODE_WITH_DATA																	0x0010	///< The value of the mode with data field mask in case of a mode with data command.

#endif // USE_DDC_LIBRARY

#define BcHost																1 ///< The host computer that controls target BC.
#define BcCore																2 ///< The core of target BC.
#define RtHost																3 ///< The host computer that controls target RT.
#define RtCore																4 ///< The core of target RT.

#define rtAddress_BORDER													32 ///< Border value for RT addresses.

#define MAXIMUM_SLEEP_INACCURACY											150 ///< The maximum possible wakeup latency (in units of ms) in sleeping periods in any expected operating-system.

#define bcMemoryObjectSize_GPQ_ENTRY										2 ///< The size (in words) of a single GPQ entry in device memory.



// Macro definitions.



// Enumeration definitions.



// Structure definitions.



// Static variables.
static U16BIT s_wBus1 = sitalBcControlWord_BUS_A; ///< Acceptance Test Plan (ATP) tests variable: The bus (sitalBcControlWord_BUS_*) to use for bus-1.
static S16BIT s_swDevice1 = 0; ///< Acceptance Test Plan (ATP) tests variable: The logical number of device (0-(sitalMaximum_DEVICES-1)) to use for device-1.
static S16BIT s_swDevice2 = 1; ///< Acceptance Test Plan (ATP) tests variable: The logical number of device (0-(sitalMaximum_DEVICES-1)) to use for device-2.
static S16BIT s_swDevice3 = 2; ///< Acceptance Test Plan (ATP) tests variable: The logical number of device (0-(sitalMaximum_DEVICES-1)) to use for device-3.
static S16BIT s_swDevice4 = 3; ///< Acceptance Test Plan (ATP) tests variable: The logical number of device (0-(sitalMaximum_DEVICES-1)) to use for device-4.
static S16BIT s_swDeviceCount = 4; ///< Acceptance Test Plan (ATP) tests variable: The total number of devices used in the tests.
static U16BIT s_wRtAddress1 = sitalRtAddress_0; ///< Acceptance Test Plan (ATP) tests variable: The RT address (sitalRtAddress_0-sitalRtAddress_31) to use for RT-device-1.
static U16BIT s_wRtAddress2 = sitalRtAddress_1; ///< Acceptance Test Plan (ATP) tests variable: The RT address (sitalRtAddress_0-sitalRtAddress_31) to use for RT-device-1.
static S16BIT s_swRtAddressCount = 2; ///< Acceptance Test Plan (ATP) tests variable: The total number of RT addresses used in the tests.
static U16BIT s_wFrameTime = 1000 /* = 100 ms */ ; ///< Acceptance Test Plan (ATP) tests variable: Frame time (in units of 100 us).
static U32BIT s_dwTestIterationsCount = 100; ///< Acceptance Test Plan (ATP) tests variable: Number of test iterations to perform (0 means infinite).
static U32BIT s_dwBcIrqStatus = 0; ///< The IRQ status reported by the driver when the BC ISR callback function was called in the last time.
static U32BIT s_dwBcIsrCallCounter = 0; ///< The overall number of times that the BC ISR has been called.
static U32BIT s_dwMinorFrameCounter = 0; ///< The overall number of transmitted minor frames.
static BOOLEAN s_bIsRtMtModeTest = FALSE; ///< A flag that says whether the RT and MT devices that participate in the executed tests should be initialized in mode RT&MT, or otherwise regularly initialized (i.e., RTs in mode RT, and MTs in mode MT).
static U16BIT s_wRtInitializationMode = (U16BIT)(s_bIsRtMtModeTest ? sitalMode_RT_AND_MT : sitalMode_RT); ///< The mode in which the RT devices that participate in the executed tests should be initialized in.
static U16BIT s_wMtInitializationMode = (U16BIT)(s_bIsRtMtModeTest ? sitalMode_RT_AND_MT : sitalMode_MT); ///< The mode in which the MT devices that participate in the executed tests should be initialized in.
static PrecisionTimer s_ptMinorFrameRunTimeMeasurer; ///< A precision timer that's used to measure the time (in units of ms) it took minor frames to complete (not including its passive time).
static PrecisionTimer s_ptFullMinorFrameRunTimeMeasurer; ///< A precision timer that's used to measure the time (in units of ms) it took minor frames to complete (including its passive time).
static BOOLEAN s_bIsPrecisionTimerRegressive = false; ///< A flag that says whether time-regression has been identified with the precision timer.
static BOOLEAN s_bIsPrecisionTimerHasty = false; ///< A flag that says whether time-leaps has been identified with the system's precision timer.
static BOOLEAN s_bFirstFrame = true; // A flag that says whether this is the first transmitted frame.
static U16BIT s_wMaxFrameTime = 0; ///< The maximum reasonable time for the currently transmitted minor frame to grow long.
static U32BIT s_dwRtIrqStatus = 0; ///< The IRQ status reported by the driver when the RT ISR callback function was called in the last time.
static U32BIT s_dwRtIsrCallCounter = 0; ///< The overall number of times that the RT ISR has been called.
static U32BIT s_dwMtIrqStatus = 0; ///< The IRQ status reported by the driver when the MT ISR callback function was called in the last time.
static U32BIT s_dwMtIsrCallCounter = 0; ///< The overall number of times that the MT ISR has been called.
static BOOLEAN s_bIsIsrPrintoutEnabled = false; ///< A flag that says whether debug printouts are enabled along ISR callbacks.
static const char s_szBcSemaphoreName[] = "Global\\StldTester.Bc.Semaphore.Isr"; ///< The name of the semaphore that's used to signal BC interrupts to waiting threads.
static InterProcessSemaphore* s_ipspBcSemaphore; ///< A pointer to the signaling object that is used by an assigned ISR function to signal interrupts to waiting threads. Note: It is here assumed that callback functions are called by the driver's interface process from a thread that belongs to the process of this tester-console-application.
static const char s_szRtSemaphoreName[] = "Global\\StldTester.Rt.Semaphore.Isr"; ///< The name of the semaphore that's used to signal RT interrupts to waiting threads.
static InterProcessSemaphore* s_ipspRtSemaphore; ///< A pointer to the signaling object that is used by an assigned ISR function to signal interrupts to waiting threads. Note: It is here assumed that callback functions are called by the driver's interface process from a thread that belongs to the process of this tester-console-application.
static const char s_szMtSemaphoreName[] = "Global\\StldTester.Mt.Semaphore.Isr"; ///< The name of the semaphore that's used to signal MT interrupts to waiting threads.
static InterProcessSemaphore* s_ipspMtSemaphore; ///< A pointer to the signaling object that is used by an assigned ISR function to signal interrupts to waiting threads. Note: It is here assumed that callback functions are called by the driver's interface process from a thread that belongs to the process of this tester-console-application.

// ECIL 5MHz project addition
static U16BIT IP_FREQUENCY_FACTOR = 5;

// Macro definitions.
#define INFINITE_ITERATIONS															(0U == s_dwTestIterationsCount)



// Internal function declarations.

// When using the DDC (or a DDC-like) library, add declarations and definitions that are unique to Sital and aren't exported by DDC.
#ifdef USE_DDC_LIBRARY

S16BIT _DECL sitalBc_GetActivationState (S16BIT swDevice, U16BIT *wpCurrentState);
S16BIT _DECL sitalBcStld1553_Command_Parse (U16BIT wCommandWord, U16BIT* wpRtAddress, U16BIT* wpMessageDirection, U16BIT* wpSubaddressOrMode, U16BIT* wpWordCountOrModeCode);

#endif // USE_DDC_LIBRARY

U32BIT _DECL stldTestDeviceNumber_Get (S16BIT swDevice);
U32BIT _DECL stldTestRtAddress_Get (S16BIT swRtAddress);
S16BIT _DECL stldSyncStart (const char* szpCallerFunctionName);
S16BIT _DECL stldSyncPoint (const char* szpCallerFunctionName, U16BIT wPoint);
S16BIT _DECL stldSyncEnd (const char* szpCallerFunctionName);
U32BIT _DECL stldInterruptLatency_Get (void);
S16BIT _DECL stldInterruptLatency_Wait (U32BIT* dwpIsrCallCounter, U32BIT dwWaitedIsrCallCount);
S16BIT _DECL stldTester_Bc_WaitForEndOfFrameRun (S16BIT swDevice, U32BIT dwRemainingFrameRunTime, U16BIT wMinorFrameCount, U32BIT* dwpIsrMinorFrameCounter);
void _DECL stldTester_Bc_UserIsr (S16BIT swDeviceId, U32BIT dwIrqStatus);
S16BIT _DECL stldTester_Bc_RunFrame (S16BIT swDeviceId, S16BIT swCommandIdCallMinor, S16BIT swFrameIdMinor, S16BIT* swapCommandIdsMinorFrame, U16BIT wCommandIdCountMinorFrame, S16BIT swFrameIdMajor, U16BIT wFrameTime);


// Internal functions.

// When using the DDC (or a DDC-like) library, add declarations and definitions that are unique to Sital and aren't exported by DDC.
#ifdef USE_DDC_LIBRARY

/// <summary>
/// Get the activation state, idle or busy, of given BC device.
///
/// Note:
/// - A BC device becomes busy when it starts running a frame, and automatically returns to be idle when all required frame iterations are finished (i.e., halt command reached).
/// - This is a rather basic implementation of the really intended function, i.e., Sital's stld1553 library function sitalBc_GetActivationState, which corresponds DDC's ACE library function aceBCGetStatus.
///   It is only used when below defined tests are run over a version of DDC (or DDC-like) library that does not yet export function aceBCGetStatus, which isn't part of the original API of the DDC ACE library.
///   For the sake of simplicity, this implementation assumes without checking that given device has been initialized as a BC device.
///
/// Equivalent DDC definition: aceBCGetStatus
/// @param swDevice (in)	Logical number of device (0-(sitalMaximum_DEVICES-1))
/// @param wpCurrentState (out)	A pointer to a variable within which current state, idle (sitalBcActivationState_IDLE) or busy (sitalBcActivationState_BUSY), is returned
/// @return sitalReturnCode_SUCCESS	Function successfully completed
/// @return Negative sitalReturnCode_*	Error condition or function failed
/// </summary>
S16BIT _DECL sitalBc_GetActivationState (S16BIT swDevice, U16BIT *wpCurrentState)
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.
	if ((0 > swDevice) || (sitalMaximum_DEVICES <= swDevice))
	{
		return sitalReturnCode_INVALID_DEVICE_NUMBER;
	}

	U16BIT wRegisterValue; // The current value of a target register, or the value to set for a target register.
	wRegisterValue = aceRegRead	(swDevice, sitalRegisterAddress_CONFIGURATION_1);
	
	(*wpCurrentState) = ( (sitalConfigurationRegister1_BC_BUSY == (sitalConfigurationRegister1_BC_BUSY & wRegisterValue)) ? sitalBcActivationState_BUSY : sitalBcActivationState_IDLE );

	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Parse given IEEE-1553 command word.
///
/// Equivalent DDC definition: aceCmdWordParse
/// @param wCommandWord (in)	Command word
/// @param wpRtAddress (out)	A pointer to a variable in which the parsed RT address is returned as sitalRtAddress_0-sitalRtAddress_31
/// @param wpMessageDirection (out)	A pointer to a variable in which the parsed RT-related message direction is returned as sitalMessageDirection_RX or sitalMessageDirection_TX
/// @param wpSubaddressOrMode (out)	A pointer to a variable in which the parsed subaddress/mode selector is returned as sitalRtSubaddress_0-sitalRtSubaddress_31
/// @param wpWordCountOrModeCode (out)	A pointer to a variable in which the parsed word-count/mode-word, depending whether regular/mode command, respectively, is returned as 0-31
/// @return sitalReturnCode_SUCCESS	Function successfully completed
/// @return Negative sitalReturnCode_*	Error condition or function failed
/// </summary>
S16BIT _DECL sitalBcStld1553_Command_Parse (	U16BIT wCommandWord,
												U16BIT* wpRtAddress,
												U16BIT* wpMessageDirection,
												U16BIT* wpSubaddressOrMode,
												U16BIT* wpWordCountOrModeCode)
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.
	if ( (((U16BIT*)NULL) == wpRtAddress) || (((U16BIT*)NULL) == wpMessageDirection) || (((U16BIT*)NULL) == wpSubaddressOrMode) || (((U16BIT*)NULL) == wpWordCountOrModeCode) )
	{
		return sitalReturnCode_INVALID_PARAMETER;
	}

	/// Parse given command word into required output message qualifiers.
	(*wpRtAddress) = ((commandWord_RT_ADDRESS & wCommandWord) >> commandWord_OFFSET_OF_RT_ADDRESS);
	(*wpMessageDirection) = ((commandWord_RX_OR_TX & wCommandWord) >> commandWord_OFFSET_OF_RX_OR_TX);
	(*wpSubaddressOrMode) = ((commandWord_SUBADDRESS_OR_MODE & wCommandWord) >> commandWord_OFFSET_OF_SUBADDRESS_OR_MODE);
	(*wpWordCountOrModeCode) = ((commandWord_WORD_COUNT_OR_MODE_CODE & wCommandWord) >> commandWord_OFFSET_OF_WORD_COUNT_OR_MODE_CODE);

	return sitalReturnCode_SUCCESS;
}

#endif // USE_DDC_LIBRARY



/// <summary>
/// Get the logical number of given test device.
///
/// @param swDevice (in)	Number of test device (1-s_swDeviceCount)
/// @return positive integer equal to the logical number of given device, if could determine it.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
U32BIT _DECL stldTestDeviceNumber_Get (S16BIT swDevice)
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.
	if ( (0 >= swDevice) || (s_swDeviceCount < swDevice) )
	{
		return sitalReturnCode_INVALID_DEVICE_NUMBER;
	}

	/// Get the logical number of given device.
	switch (swDevice)
	{
	case 1: return s_swDevice1; break;
	case 2: return s_swDevice2; break;
	case 3: return s_swDevice3; break;
	case 4: return s_swDevice4; break;
	default: return sitalReturnCode_INVALID_DEVICE_NUMBER;
	}

	return sitalReturnCode_UNKNOWN_ERROR;
}



/// <summary>
/// Get the RT address whose index is given.
///
/// @param swRtAddress (in)	Index of RT address (1-s_swRtAddressCount)
/// @return positive integer equal to the logical number of given device, if could determine it.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
U32BIT _DECL stldTestRtAddress_Get (S16BIT swRtAddress)
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.
	if ( (0 >= swRtAddress) || (s_swRtAddressCount < swRtAddress) )
	{
		return sitalReturnCode_INVALID_DEVICE_NUMBER;
	}

	/// Get the logical number of given device.
	switch (swRtAddress)
	{
	case 1: return s_wRtAddress1; break;
	case 2: return s_wRtAddress2; break;
	default: return sitalReturnCode_INVALID_DEVICE_NUMBER;
	}

	return sitalReturnCode_UNKNOWN_ERROR;
}



/// <summary>
/// Do the synchronization operations required upon the beginning of given function.
///
/// @param szpCallerFunctionName (in)	The name of the function for which synchronization services are required.
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldSyncStart (const char* szpCallerFunctionName)
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	szpCallerFunctionName++; // Just in order to prevent compilation warning.

	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Do the synchronization operations required given point in the life of given function.
///
/// @param szpCallerFunctionName (in)	The name of the function for which synchronization services are required.
/// @param wPoint (in)	The index of the requested specific synchronization point of given function.
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldSyncPoint (const char* szpCallerFunctionName, U16BIT wPoint)
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	szpCallerFunctionName++; // Just in order to prevent compilation warning.
	wPoint++; // Just in order to prevent compilation warning.

	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Do the synchronization operations required when given function ends.
///
/// @param szpCallerFunctionName (in)	The name of the function for which synchronization services are required.
/// @return sitalReturnCode_SUCCESS if function successfully completed.
///	@return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldSyncEnd (const char* szpCallerFunctionName)
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	szpCallerFunctionName++; // Just in order to prevent compilation warning.

	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Get the maximum possible interrupt latency for this process under the current operating-system.
///
/// @return unsigned integer equal to the maximum possible interrupt latency for this process under the current operating-system.
/// </summary>
U32BIT _DECL stldInterruptLatency_Get (void)
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	/// If this is the first entry to this function:
	///		Get the maximum possible interrupt latency (milliseconds) for this process under the current operating-system.
	static BOOLEAN s_bFirstEntry = TRUE; // A flag that says whether this is the first time that this function is called.
	S16BIT swResult; // Result of operation or function call.
	static U32BIT s_dwMaximumInterruptLatency; // The maximum possible interrupt latency for the current combination of process and operating-system.
	if (TRUE == s_bFirstEntry)
	{
		swResult = sitalProcess_GetInterruptLatency (&s_dwMaximumInterruptLatency);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Process.InterruptLatency.Get.Warning: error - %i\n", swResult);
			s_dwMaximumInterruptLatency = 1000; // 1000 milliseconds.
		}
		sitalProcess_Log_PrintLine ("#StldTester.Process.InterruptLatency.Get: latency - %ims\n", s_dwMaximumInterruptLatency);

		s_bFirstEntry = FALSE;
	}

	return s_dwMaximumInterruptLatency;
}



/// <summary>
/// Wait enough time for all the expected interrupts to be:
/// - Issued by the device.
/// - Detected by the driver and the driver interface DLL.
/// - Informed by the driver interface DLL to this user app.
///
/// @param dwpIsrCallCounter (in)	A pointer to the ISR call counter in which the interrupt service routine counts the detected interrupts.
/// @param dwWaitedIsrCallCount (in)	The expected interrupt count, i.e., the count of interrupts that the variable that's pointed by parameter dwpIsrCallCounter should eventually contain.
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldInterruptLatency_Wait (U32BIT* dwpIsrCallCounter, U32BIT dwWaitedIsrCallCount)
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	/// As long as the expected number of interrupts still didn't occur AND the maximum possible interrupt latency didn't pass:
	///		Wait another 1 millisecond.
	U32BIT dwMaximumInterruptLatency; // The maximum possible interrupt latency for the current combination of process and operating-system.
	dwMaximumInterruptLatency = stldInterruptLatency_Get ();
	U32BIT dwSleepTime; // Sleeping period (in ms).
	dwSleepTime = 1;
	PrecisionTimer pt; // A precision timer.
	pt.SetBaseTime ();
	pt.SetEndTime ();
	while ( (dwWaitedIsrCallCount != (*dwpIsrCallCounter)) && (dwMaximumInterruptLatency >= pt.GetDuration ()) )
	{
		///			Wait for a while.
		sitalProcess_Log_PrintLine ("#StldTester.Wait: time_passed - %ims, isr_calls - %i\n", dwSleepTime, (*dwpIsrCallCounter));
		sitalProcess_Log_PrintLine ("\tWAIT_FOR %i us\n", (dwSleepTime * 1000));
		pt.AccuratelySleep (dwSleepTime); // 1 millisecond.
		pt.SetEndTime ();
	}

	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Wait until frame run ends on given BC device.
///
/// @param swDevice (in)	Logical number of device (0-(sitalMaximum_DEVICES-1))
/// @param dwRemainingFrameRunTime (in)	The time (in units of milliseconds) left until the last frame should end
/// @param wMinorFrameCount (in)	The total number of minor frames that are about to be executed
/// @param dwpIsrMinorFrameCounter (in)	A pointer to the ISR minor frame counter in which the interrupt service routine counts the executed minor frames, or NULL if no ISR counts them
/// @return sitalReturnCode_SUCCESS	Function successfully completed
/// @return Negative sitalReturnCode_*	Error condition or function failed
/// </summary>
S16BIT _DECL stldTester_Bc_WaitForEndOfFrameRun (S16BIT swDevice, U32BIT dwRemainingFrameRunTime, U16BIT wMinorFrameCount, U32BIT* dwpIsrMinorFrameCounter)
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	swDevice++; // Just in order to prevent compilation warning.

	/// As long as given BC device is running and the expected number of minor frame wasn't still executed AND the maximum frame run time limit didn't pass:
	///		Wait another 1 millisecond.
	PrecisionTimer ptSleepMeasurer; // A precision timer.
	ptSleepMeasurer.SetBaseTime ();
	bool bTimeEnded; // A flag that says whether the maximum time alloted for a certain operation has already elapsed.
	bTimeEnded = false;
	bool bWaitMore; // A flag that says whether should wait more time.
	bWaitMore = true;
	U16BIT wCurrentState; // The current state of a BC device (sitalBcActivationState_*).
	wCurrentState = sitalBcActivationState_BUSY;
	unsigned int uiWaitMilliseconds; // The number of milliseconds to sleep in each iteration.
	uiWaitMilliseconds = ( (0U == dwRemainingFrameRunTime) ? 0 : 1 ); // The number of milliseconds to sleep in the first iteration.
	bool bFirstTurn = true;
	bool bSleepAtFirstTurn = false;
	while ((!bTimeEnded) && bWaitMore)
	{
		if (bFirstTurn)
		{
			if (bSleepAtFirstTurn)
			{
				ptSleepMeasurer.AccuratelySleep (uiWaitMilliseconds);
			}
			uiWaitMilliseconds = 1; // The number of milliseconds to sleep from the second iteration and on.
			bFirstTurn = false;
		}
		else
		{
			ptSleepMeasurer.AccuratelySleep (uiWaitMilliseconds);
			//sitalProcess_Log_PrintLine ("#StldTester.Wait.Error: time - %ims\n", uiWaitMilliseconds); // Only required in cases where this function is called after frame run ended, what the caller may know by verifying that an end-of-frame interrupts is received (see, for example, test stldTester_Atp_DoTest461_13e_ExternRt).
		}

		/// 	Get the activation state of device-2 (BC).
#ifndef CRITICAL_TIMING
		//sitalProcess_Log_PrintLine ("#StldTester.Bc.GetActivationState: unit - %i\n", s_swDevice2);
#endif // CRITICAL_TIMING
		S16BIT swResult; // Result of operation or function call.
		swResult = sitalBc_GetActivationState (s_swDevice2, &wCurrentState);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.GetActivationState.Error: error - %i\n", swResult);
			return swResult;
		}
#ifndef CRITICAL_TIMING
		//sitalProcess_Log_PrintLine ("#StldTester.Bc.GetActivationState: unit - %i, state - %i\n", s_swDevice2, wCurrentState);
#endif // CRITICAL_TIMING

		ptSleepMeasurer.SetEndTime ();
		bTimeEnded = ( (dwRemainingFrameRunTime + 10000) < ptSleepMeasurer.GetDuration () );
		bWaitMore = ( ((NULL != dwpIsrMinorFrameCounter) && (wMinorFrameCount > (*dwpIsrMinorFrameCounter))) || (sitalBcActivationState_BUSY == wCurrentState) );
	}

#ifndef CRITICAL_TIMING
	sitalProcess_Log_PrintLine ("#StldTester.Wait: time - %ims\n", U32BIT(ptSleepMeasurer.GetDuration ()));
	sitalProcess_Log_PrintLine ("\tWAIT_FOR %i us\n", U32BIT(ptSleepMeasurer.GetDuration () * 1000));
#endif // CRITICAL_TIMING

	if (sitalBcActivationState_BUSY == wCurrentState)
	{
		return sitalReturnCode_DEVICE_BUSY;
	}

	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// BC user's ISR.
/// Count the overall number of BC interrupts and the overall number of transmitted minor frames into the dedicated global variables.
/// Record frame end time with each notification on end of minor frame, so that if the currently executing test recorded the frame run start time, it will be able to calculate in the end the overall duration of the frame run.
///
/// Note:
/// - Debug printouts shall be better disabled, as printing along callbacks may cause the loss of interrupts.
///   The caller is therefore encouraged to disable this function's printout, unless along debug session, by setting the global flag s_bIsIsrPrintoutEnabled to false.
///
/// @param swDeviceId (in)	Logical number of device (0-1)
/// @param dwIrqStatus (in)	IRQ status
/// </summary>
void _DECL stldTester_Bc_UserIsr (S16BIT swDeviceId, U32BIT dwIrqStatus)
{
	/// @pseudocode

	/// If a signaling object has been created by a currently executed test function but was not opened yet by this callback function:
	///		Open a signaling object.
	if (((InterProcessSemaphore*)NULL) == s_ipspBcSemaphore)
	{
		s_ipspBcSemaphore = new InterProcessSemaphore (s_szBcSemaphoreName, 0);
		if (((InterProcessSemaphore*)NULL) == s_ipspBcSemaphore)
		{
			if (TRUE == s_bIsIsrPrintoutEnabled)
			{
				sitalProcess_Log_PrintLine ("#StldTester.UserIsr.Bc.Semaphore.Allocation.Error: error - %i\n", GetLastError_NonOsDependent());
			}
		}
		else
		{
			if (TRUE == s_bIsIsrPrintoutEnabled)
			{
				sitalProcess_Log_PrintLine ("#StldTester.UserIsr.Bc.Semaphore.Open: name - %s\n", s_szBcSemaphoreName);
			}
			if (0U != s_ipspBcSemaphore->Open ())
			{
				if (TRUE == s_bIsIsrPrintoutEnabled)
				{
					sitalProcess_Log_PrintLine ("#StldTester.UserIsr.Bc.Semaphore.Open.Error: error - %i\n", GetLastError_NonOsDependent());
				}

				// No signaling object has been created by a currently executed test function.
				// The just allocated semaphore object must be deleted.
				delete (s_ipspBcSemaphore);
				s_ipspBcSemaphore = NULL;
			}
		}
	}

	/// Increment the count of ISR calls.
	s_dwBcIsrCallCounter++;

	/// Record the informed IRQ status.
	s_dwBcIrqStatus = dwIrqStatus;

	/// If called for IRQ3 (end of minor frame):
	///		Increment the count of minor frames.
	///		Record the frame end time.
	BOOLEAN bSignaledInterruptOccurred; // A flag that says whether any interrupt about which waiting threads must be signaled occurred.
	bSignaledInterruptOccurred = FALSE;
	if (sitalInterruptRegister2_BC_IRQ3 == (sitalInterruptRegister2_BC_IRQ3 & dwIrqStatus))
	{
		s_dwMinorFrameCounter++;
		if (s_bFirstFrame)
		{
			s_bFirstFrame = false;
		}
		else
		{
			// Verify that the precision timer does not jump backward nor runs too fast since the previous end-of-frame.
			double dPrevEndTime; // The previously recorded minor frame end time.
			dPrevEndTime = s_ptMinorFrameRunTimeMeasurer.GetEndTime();
			double dCurTime; // The current time.
			dCurTime = s_ptMinorFrameRunTimeMeasurer.GetTime();
			if (dPrevEndTime > dCurTime)
			{
				if (TRUE == s_bIsIsrPrintoutEnabled)	sitalProcess_Log_PrintLine ("#StldTester.UserIsr.Bc.MinorFrameTimeMeasurer.Error: device - %i, prev_end_time - %fms, cur_end_time - %fms\n", swDeviceId, dPrevEndTime, dCurTime);
				s_bIsPrecisionTimerRegressive = true;
			}
			if ((dPrevEndTime + s_wMaxFrameTime) < dCurTime)
			{
				if (TRUE == s_bIsIsrPrintoutEnabled)	sitalProcess_Log_PrintLine ("#StldTester.UserIsr.Bc.MinorFrameTimeMeasurer.Error: device - %i, prev_end_time - %fms, max_frame_time - %ims, cur_end_time - %fms, leap - %fms\n", swDeviceId, dPrevEndTime, s_wMaxFrameTime, dCurTime, (dCurTime - dPrevEndTime));
				s_bIsPrecisionTimerHasty = true;
			}
		}
		s_ptMinorFrameRunTimeMeasurer.SetEndTime ();
		bSignaledInterruptOccurred = TRUE;
	}
	if (sitalInterruptRegister2_BC_IRQ2 == (sitalInterruptRegister2_BC_IRQ2 & dwIrqStatus))
	{
		s_ptFullMinorFrameRunTimeMeasurer.SetEndTime ();
		bSignaledInterruptOccurred = TRUE;
	}

	if (TRUE == s_bIsIsrPrintoutEnabled)
	{
		sitalProcess_Log_PrintLine ("#StldTester.UserIsr.Bc.Recorded: device - %i, irq_status - %08X, bc_isr_calls - %i, minor_frame_count - %i\n", swDeviceId, dwIrqStatus, s_dwBcIsrCallCounter, s_dwMinorFrameCounter);
	}

	/// If any signaled interrupt occurred and any thread[s] may be waiting now for interrupts signals:
	///		Signal the possibly waiting threads about the interrupt.
	if ((TRUE == bSignaledInterruptOccurred) && (NULL != s_ipspBcSemaphore) && s_ipspBcSemaphore->IsActive ())
	{
		if (TRUE == s_bIsIsrPrintoutEnabled)
		{
			sitalProcess_Log_PrintLine ("#StldTester.UserIsr.Bc.Semaphore.Release: name - %s\n", s_szBcSemaphoreName);
		}
		if ( ! s_ipspBcSemaphore->Release () )
		{
			if (TRUE == s_bIsIsrPrintoutEnabled)
			{
				sitalProcess_Log_PrintLine ("#StldTester.UserIsr.Bc.Semaphore.Release.Error: error - %i\n", GetLastError_NonOsDependent());
			}
		}
	}
}



/// <summary>
/// RT user's ISR.
/// Count the overall number of RT interrupts into the dedicated global variable.
///
/// Note:
/// - Debug printouts shall be better disabled, as printing along callbacks may cause the loss of interrupts.
///   The caller is therefore encouraged to disable this function's printout, unless along debug session, by setting the global flag s_bIsIsrPrintoutEnabled to false.
///
/// @param swDeviceId (in)	Logical number of device (0-1)
/// @param dwIrqStatus (in)	IRQ status
/// </summary>
void _DECL stldTester_Rt_UserIsr (S16BIT swDeviceId, U32BIT dwIrqStatus)
{
	/// @pseudocode

	/// If a signaling object has been created by a currently executed test function but was not opened yet by this callback function:
	///		Open a signaling object.
	if (((InterProcessSemaphore*)NULL) == s_ipspRtSemaphore)
	{
		s_ipspRtSemaphore = new InterProcessSemaphore (s_szRtSemaphoreName, 0);
		if (((InterProcessSemaphore*)NULL) == s_ipspRtSemaphore)
		{
			if (TRUE == s_bIsIsrPrintoutEnabled)
			{
				sitalProcess_Log_PrintLine ("#StldTester.UserIsr.Rt.Semaphore.Allocation.Error: error - %i\n", GetLastError_NonOsDependent());
			}
		}
		else
		{
			if (TRUE == s_bIsIsrPrintoutEnabled)
			{
				sitalProcess_Log_PrintLine ("#StldTester.UserIsr.Rt.Semaphore.Open: name - %s\n", s_szRtSemaphoreName);
			}
			if (0U != s_ipspRtSemaphore->Open ())
			{
				if (TRUE == s_bIsIsrPrintoutEnabled)
				{
					sitalProcess_Log_PrintLine ("#StldTester.UserIsr.Rt.Semaphore.Open.Error: error - %i\n", GetLastError_NonOsDependent());
				}

				// No signaling object has been created by a currently executed test function.
				// The just allocated semaphore object must be deleted.
				delete (s_ipspRtSemaphore);
				s_ipspRtSemaphore = NULL;
			}
		}
	}

	/// Increment the count of ISR calls.
	s_dwRtIsrCallCounter++;

	/// Record the informed IRQ status.
	s_dwRtIrqStatus = dwIrqStatus;

	if (TRUE == s_bIsIsrPrintoutEnabled)
	{
		sitalProcess_Log_PrintLine ("#StldTester.UserIsr.Rt.Recorded: device - %i, irq_status - %08X, rt_isr_calls - %i\n", swDeviceId, dwIrqStatus, s_dwRtIsrCallCounter);
	}

	/// If any thread[s] may be waiting now for interrupts signals:
	///		Signal the possibly waiting threads about the interrupt.
	if ((NULL != s_ipspRtSemaphore) && s_ipspRtSemaphore->IsActive ())
	{
		if (TRUE == s_bIsIsrPrintoutEnabled)
		{
			sitalProcess_Log_PrintLine ("#StldTester.UserIsr.Rt.Semaphore.Release: name - %s\n", s_szRtSemaphoreName);
		}
		if ( ! s_ipspRtSemaphore->Release () )
		{
			if (TRUE == s_bIsIsrPrintoutEnabled)
			{
				sitalProcess_Log_PrintLine ("#StldTester.UserIsr.Rt.Semaphore.Release.Error: error - %i\n", GetLastError_NonOsDependent());
			}
		}
	}
}



/// <summary>
/// MT user's ISR.
/// Count the overall number of MT interrupts into the dedicated global variable.
///
/// Note:
/// - Debug printouts shall be better disabled, as printing along callbacks may cause the loss of interrupts.
///   The caller is therefore encouraged to disable this function's printout, unless along debug session, by setting the global flag s_bIsIsrPrintoutEnabled to false.
///
/// @param swDeviceId (in)	Logical number of device (0-1)
/// @param dwIrqStatus (in)	IRQ status
/// </summary>
void _DECL stldTester_Mt_UserIsr (S16BIT swDeviceId, U32BIT dwIrqStatus)
{
	/// @pseudocode

	/// If a signaling object has been created by a currently executed test function but was not opened yet by this callback function:
	///		Open a signaling object.
	if (((InterProcessSemaphore*)NULL) == s_ipspMtSemaphore)
	{
		s_ipspMtSemaphore = new InterProcessSemaphore (s_szMtSemaphoreName, 0);
		if (((InterProcessSemaphore*)NULL) == s_ipspMtSemaphore)
		{
			if (TRUE == s_bIsIsrPrintoutEnabled)
			{
				sitalProcess_Log_PrintLine ("#StldTester.UserIsr.Mt.Semaphore.Allocation.Error: error - %i\n", GetLastError_NonOsDependent());
			}
		}
		else
		{
			if (TRUE == s_bIsIsrPrintoutEnabled)
			{
				sitalProcess_Log_PrintLine ("#StldTester.UserIsr.Mt.Semaphore.Open: name - %s\n", s_szMtSemaphoreName);
			}
			if (0U != s_ipspMtSemaphore->Open ())
			{
				if (TRUE == s_bIsIsrPrintoutEnabled)
				{
					sitalProcess_Log_PrintLine ("#StldTester.UserIsr.Mt.Semaphore.Open.Error: error - %i\n", GetLastError_NonOsDependent());
				}

				// No signaling object has been created by a currently executed test function.
				// The just allocated semaphore object must be deleted.
				delete (s_ipspMtSemaphore);
				s_ipspMtSemaphore = NULL;
			}
		}
	}

	/// Increment the count of ISR calls.
	s_dwMtIsrCallCounter++;

	/// Record the informed IRQ status.
	s_dwMtIrqStatus = dwIrqStatus;

	if (TRUE == s_bIsIsrPrintoutEnabled)
	{
		sitalProcess_Log_PrintLine ("#StldTester.UserIsr.Mt.Recorded: device - %i, irq_status - %08X, rt_isr_calls - %i\n", swDeviceId, dwIrqStatus, s_dwMtIsrCallCounter);
	}

	/// If any thread[s] may be waiting now for interrupts signals:
	///		Signal the possibly waiting threads about the interrupt.
	if ((NULL != s_ipspMtSemaphore) && s_ipspMtSemaphore->IsActive ())
	{
		if (TRUE == s_bIsIsrPrintoutEnabled)
		{
			sitalProcess_Log_PrintLine ("#StldTester.UserIsr.Mt.Semaphore.Release: name - %s\n", s_szMtSemaphoreName);
		}
		if ( ! s_ipspMtSemaphore->Release () )
		{
			if (TRUE == s_bIsIsrPrintoutEnabled)
			{
				sitalProcess_Log_PrintLine ("#StldTester.UserIsr.Mt.Semaphore.Release.Error: error - %i\n", GetLastError_NonOsDependent());
			}
		}
	}
}



/// <summary>
/// With given BC device create a minor frame that contains given command, then create a major frame that calls the created minor frame, run it, and wait for it to end.
/// Along test print suitable info and indications into the log.
///
/// @param swDeviceId (in)	Logical number of device (0-1)
/// @param swCommandIdCallMinor (in)	A unique ID designating a still unused command (0-(sitalBcCounter_COMMANDS-1))
/// @param swFrameIdMinor (in)	A unique ID designating a still unused frame (0-(sitalBcCounter_FRAMES-1))
/// @param swapCommandIdsMinorFrame (in)	A pointer to an array of already created command-IDs to include in the minor frame
/// @param wCommandIdCountMinorFrame (in)	The number of command IDs in the minor frame (1-sitalBcCounter_FRAME_COMMANDS)
/// @param swFrameIdMajor (in)	A unique ID designating a still unused frame (0-(sitalBcCounter_FRAMES-1)).
/// @param wFrameTime (in)	The time (in units of 100 us) that the frame will take to complete
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Bc_RunFrame (S16BIT swDeviceId, S16BIT swCommandIdCallMinor, S16BIT swFrameIdMinor, S16BIT* swapCommandIdsMinorFrame, U16BIT wCommandIdCountMinorFrame, S16BIT swFrameIdMajor, U16BIT wFrameTime)
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	/// Create for given device (BC) given minor-caller-command, a subroutine call command that executes given minor-frame.
	sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", swDeviceId, swCommandIdCallMinor, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, swFrameIdMinor, 0);
	S16BIT swResult; // Result of operation or function call.
	swResult = sitalBc_Command_Create (swDeviceId, swCommandIdCallMinor, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, (U16BIT)swFrameIdMinor, 0, 0);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
		return swResult;
	}

	/// Create for given device (BC) given minor-frame, a frame that contains given minor-frame-commands.
	sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i - %i), time - %i, flags - %i\n", swDeviceId, swFrameIdMinor, "minor", wCommandIdCountMinorFrame, swapCommandIdsMinorFrame[0], swapCommandIdsMinorFrame[wCommandIdCountMinorFrame - 1], 0, 0);
	swResult = sitalBc_Frame_Create (swDeviceId, swFrameIdMinor, sitalBcFrameType_MINOR, swapCommandIdsMinorFrame, wCommandIdCountMinorFrame, 0, 0);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
		return swResult;
	}

	/// Create for given device (BC) given major-frame, a frame that contains the minor-caller-command of given device (BC) and whose frame time is FrameTime us.
	sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", swDeviceId, swFrameIdMajor, "major", 1, swCommandIdCallMinor, (wFrameTime / 10), 0);
	swResult = sitalBc_Frame_Create (swDeviceId, swFrameIdMajor, sitalBcFrameType_MAJOR, &swCommandIdCallMinor, 1, wFrameTime, 0);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
		return swResult;
	}

	/// Configure given device (BC) to get IRQ-3 notifications using an ISR that collects interrupt data, counts ISR calls, and signals interested threads.
	sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Manipulate: unit - %i, enable - %i, irq - %08X, callback - %s\n", swDeviceId, TRUE, sitalInterruptRegister2_BC_IRQ3, "stldTester_Bc_UserIsr");
	swResult = sitalDevice_Irq_Manipulate (swDeviceId, TRUE, sitalInterruptRegister2_BC_IRQ3, stldTester_Bc_UserIsr);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Set.Error: error - %i\n", swResult);
		sitalDevice_Free (swDeviceId);
		return swResult;
	}

	/// Zero the global ISR counter of ended minor frames.
	s_dwMinorFrameCounter = 0;

	/// Start given device (BC) running given major-frame for FrameCount times.
	U16BIT wFrameCount; // The number of frames to run.
	wFrameCount = 1;
	sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", swDeviceId, swFrameIdMajor, wFrameCount);
	swResult = sitalBc_Start (swDeviceId, swFrameIdMajor, wFrameCount);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
		return swResult;
	}

	/// Wait for the frame run to end.
	U32BIT dwRemainingFrameRunTime; // The expected remaining frame run time (in ms).
	dwRemainingFrameRunTime = (U32BIT)(wFrameCount * (wFrameTime / 10));
	swResult = stldTester_Bc_WaitForEndOfFrameRun (swDeviceId, dwRemainingFrameRunTime, wFrameCount, &s_dwMinorFrameCounter);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Bc.WaitForEndOfFrameRun.Error: error - %i\n", swResult);
		return swResult;
	}
	sitalProcess_Log_PrintLine ("#StldTester.Wait.End: unit - %i, isr_invoked - %i\n", swDeviceId, (wFrameCount == s_dwMinorFrameCounter));

	/// Stop given device (BC).
	sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop: unit - %i\n", swDeviceId);
	swResult = sitalBc_Stop (swDeviceId);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
		return swResult;
	}

	return sitalReturnCode_SUCCESS;
}



// Exported functions.

/// <summary>
/// Open the log file, use default file name.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_OpenLog ()
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	return sitalProcess_Log_Start (NULL);
}



/// <summary>
/// Close the log file, and save its contents.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_CloseLog ()
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	return sitalProcess_Log_Stop (TRUE);
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to given value.
///
/// @param iBus (in)	Bus number (sitalBcControlWord_BUS_*)
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetBus1 (int iBus)
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	if ((sitalBcControlWord_BUS_A != iBus) && (sitalBcControlWord_BUS_B != iBus))
	{
		return sitalReturnCode_INVALID_PARAMETER;
	}

	/// Set test constant.
	s_wBus1 = sitalBcControlWord_BUS_A;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetBus1_A ()
{
	/// @pseudocode

	/// Set test constant.
	s_wBus1 = sitalBcControlWord_BUS_A;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetBus1_B ()
{
	/// @pseudocode

	/// Set test constant.
	s_wBus1 = sitalBcControlWord_BUS_B;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to given value.
///
/// @param iDevice (in)	Device number (0-31)
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice1 (int iDevice)
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	if ((0 > iDevice) && (31 < iDevice))
	{
		return sitalReturnCode_INVALID_PARAMETER;
	}

	/// Set test constant.
	s_swDevice1 = (S16BIT)iDevice;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice1_000 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice1 = 0;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice1_001 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice1 = 1;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice1_002 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice1 = 2;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice1_003 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice1 = 3;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice1_004 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice1 = 4;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice1_005 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice1 = 5;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice1_006 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice1 = 6;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice1_007 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice1 = 7;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice1_008 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice1 = 8;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice1_009 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice1 = 9;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice1_010 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice1 = 10;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice1_011 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice1 = 11;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice1_012 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice1 = 12;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice1_013 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice1 = 13;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice1_014 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice1 = 14;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice1_015 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice1 = 15;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice1_016 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice1 = 16;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice1_017 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice1 = 17;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice1_018 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice1 = 18;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice1_019 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice1 = 19;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice1_020 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice1 = 20;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice1_021 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice1 = 21;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice1_022 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice1 = 22;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice1_023 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice1 = 23;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice1_024 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice1 = 24;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice1_025 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice1 = 25;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice1_026 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice1 = 26;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice1_027 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice1 = 27;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice1_028 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice1 = 28;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice1_029 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice1 = 29;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice1_030 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice1 = 30;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice1_031 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice1 = 31;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to given value.
///
/// @param iDevice (in)	Device number (0-31)
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice2 (int iDevice)
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	if ((0 > iDevice) && (31 < iDevice))
	{
		return sitalReturnCode_INVALID_PARAMETER;
	}

	/// Set test constant.
	s_swDevice2 = (S16BIT)iDevice;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice2_000 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice2 = 0;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice2_001 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice2 = 1;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice2_002 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice2 = 2;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice2_003 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice2 = 3;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice2_004 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice2 = 4;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice2_005 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice2 = 5;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice2_006 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice2 = 6;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice2_007 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice2 = 7;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice2_008 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice2 = 8;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice2_009 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice2 = 9;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice2_010 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice2 = 10;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice2_011 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice2 = 11;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice2_012 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice2 = 12;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice2_013 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice2 = 13;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice2_014 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice2 = 14;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice2_015 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice2 = 15;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice2_016 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice2 = 16;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice2_017 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice2 = 17;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice2_018 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice2 = 18;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice2_019 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice2 = 19;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice2_020 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice2 = 20;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice2_021 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice2 = 21;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice2_022 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice2 = 22;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice2_023 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice2 = 23;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice2_024 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice2 = 24;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice2_025 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice2 = 25;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice2_026 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice2 = 26;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice2_027 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice2 = 27;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice2_028 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice2 = 28;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice2_029 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice2 = 29;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice2_030 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice2 = 30;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice2_031 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice2 = 31;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to given value.
///
/// @param iDevice (in)	Device number (0-31)
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice3 (int iDevice)
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	if ((0 > iDevice) && (31 < iDevice))
	{
		return sitalReturnCode_INVALID_PARAMETER;
	}

	/// Set test constant.
	s_swDevice3 = (S16BIT)iDevice;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice3_000 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice3 = 0;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice3_001 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice3 = 1;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice3_002 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice3 = 2;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice3_003 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice3 = 3;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice3_004 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice3 = 4;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice3_005 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice3 = 5;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice3_006 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice3 = 6;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice3_007 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice3 = 7;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice3_008 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice3 = 8;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice3_009 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice3 = 9;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice3_010 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice3 = 10;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice3_011 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice3 = 11;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice3_012 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice3 = 12;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice3_013 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice3 = 13;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice3_014 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice3 = 14;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice3_015 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice3 = 15;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice3_016 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice3 = 16;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice3_017 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice3 = 17;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice3_018 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice3 = 18;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice3_019 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice3 = 19;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice3_020 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice3 = 20;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice3_021 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice3 = 21;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice3_022 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice3 = 22;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice3_023 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice3 = 23;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice3_024 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice3 = 24;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice3_025 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice3 = 25;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice3_026 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice3 = 26;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice3_027 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice3 = 27;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice3_028 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice3 = 28;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice3_029 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice3 = 29;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice3_030 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice3 = 30;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice3_031 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice3 = 31;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to given value.
///
/// @param iDevice (in)	Device number (0-31)
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice4 (int iDevice)
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	if ((0 > iDevice) && (31 < iDevice))
	{
		return sitalReturnCode_INVALID_PARAMETER;
	}

	/// Set test constant.
	s_swDevice4 = (S16BIT)iDevice;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice4_000 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice4 = 0;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice4_001 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice4 = 1;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice4_002 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice4 = 2;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice4_003 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice4 = 3;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice4_004 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice4 = 4;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice4_005 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice4 = 5;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice4_006 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice4 = 6;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice4_007 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice4 = 7;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice4_008 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice4 = 8;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice4_009 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice4 = 9;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice4_010 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice4 = 10;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice4_011 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice4 = 11;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice4_012 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice4 = 12;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice4_013 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice4 = 13;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice4_014 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice4 = 14;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice4_015 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice4 = 15;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice4_016 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice4 = 16;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice4_017 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice4 = 17;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice4_018 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice4 = 18;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice4_019 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice4 = 19;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice4_020 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice4 = 20;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice4_021 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice4 = 21;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice4_022 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice4 = 22;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice4_023 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice4 = 23;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice4_024 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice4 = 24;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice4_025 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice4 = 25;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice4_026 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice4 = 26;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice4_027 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice4 = 27;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice4_028 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice4 = 28;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice4_029 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice4 = 29;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice4_030 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice4 = 30;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetDevice4_031 ()
{
	/// @pseudocode

	/// Set test constant.
	s_swDevice4 = 31;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to given value.
///
/// @param iRtAddress (in)	RT address number (0-31)
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetRtAddress1 (int iRtAddress)
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	if ((0 > iRtAddress) && (31 < iRtAddress))
	{
		return sitalReturnCode_INVALID_PARAMETER;
	}

	/// Set test constant.
	s_wRtAddress1 = (U16BIT)iRtAddress;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetRtAddress1_00 ()
{
	/// @pseudocode

	/// Set test constant.
	s_wRtAddress1 = sitalRtAddress_0;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetRtAddress1_01 ()
{
	/// @pseudocode

	/// Set test constant.
	s_wRtAddress1 = sitalRtAddress_1;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetRtAddress1_02 ()
{
	/// @pseudocode

	/// Set test constant.
	s_wRtAddress1 = sitalRtAddress_2;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetRtAddress1_03 ()
{
	/// @pseudocode

	/// Set test constant.
	s_wRtAddress1 = sitalRtAddress_3;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetRtAddress1_04 ()
{
	/// @pseudocode

	/// Set test constant.
	s_wRtAddress1 = sitalRtAddress_4;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetRtAddress1_05 ()
{
	/// @pseudocode

	/// Set test constant.
	s_wRtAddress1 = sitalRtAddress_5;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetRtAddress1_06 ()
{
	/// @pseudocode

	/// Set test constant.
	s_wRtAddress1 = sitalRtAddress_6;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetRtAddress1_07 ()
{
	/// @pseudocode

	/// Set test constant.
	s_wRtAddress1 = sitalRtAddress_7;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetRtAddress1_08 ()
{
	/// @pseudocode

	/// Set test constant.
	s_wRtAddress1 = sitalRtAddress_8;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetRtAddress1_09 ()
{
	/// @pseudocode

	/// Set test constant.
	s_wRtAddress1 = sitalRtAddress_9;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetRtAddress1_10 ()
{
	/// @pseudocode

	/// Set test constant.
	s_wRtAddress1 = sitalRtAddress_10;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetRtAddress1_11 ()
{
	/// @pseudocode

	/// Set test constant.
	s_wRtAddress1 = sitalRtAddress_11;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetRtAddress1_12 ()
{
	/// @pseudocode

	/// Set test constant.
	s_wRtAddress1 = sitalRtAddress_12;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetRtAddress1_13 ()
{
	/// @pseudocode

	/// Set test constant.
	s_wRtAddress1 = sitalRtAddress_13;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetRtAddress1_14 ()
{
	/// @pseudocode

	/// Set test constant.
	s_wRtAddress1 = sitalRtAddress_14;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetRtAddress1_15 ()
{
	/// @pseudocode

	/// Set test constant.
	s_wRtAddress1 = sitalRtAddress_15;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetRtAddress1_16 ()
{
	/// @pseudocode

	/// Set test constant.
	s_wRtAddress1 = sitalRtAddress_16;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetRtAddress1_17 ()
{
	/// @pseudocode

	/// Set test constant.
	s_wRtAddress1 = sitalRtAddress_17;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetRtAddress1_18 ()
{
	/// @pseudocode

	/// Set test constant.
	s_wRtAddress1 = sitalRtAddress_18;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetRtAddress1_19 ()
{
	/// @pseudocode

	/// Set test constant.
	s_wRtAddress1 = sitalRtAddress_19;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetRtAddress1_20 ()
{
	/// @pseudocode

	/// Set test constant.
	s_wRtAddress1 = sitalRtAddress_20;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetRtAddress1_21 ()
{
	/// @pseudocode

	/// Set test constant.
	s_wRtAddress1 = sitalRtAddress_21;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetRtAddress1_22 ()
{
	/// @pseudocode

	/// Set test constant.
	s_wRtAddress1 = sitalRtAddress_22;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetRtAddress1_23 ()
{
	/// @pseudocode

	/// Set test constant.
	s_wRtAddress1 = sitalRtAddress_23;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetRtAddress1_24 ()
{
	/// @pseudocode

	/// Set test constant.
	s_wRtAddress1 = sitalRtAddress_24;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetRtAddress1_25 ()
{
	/// @pseudocode

	/// Set test constant.
	s_wRtAddress1 = sitalRtAddress_25;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetRtAddress1_26 ()
{
	/// @pseudocode

	/// Set test constant.
	s_wRtAddress1 = sitalRtAddress_26;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetRtAddress1_27 ()
{
	/// @pseudocode

	/// Set test constant.
	s_wRtAddress1 = sitalRtAddress_27;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetRtAddress1_28 ()
{
	/// @pseudocode

	/// Set test constant.
	s_wRtAddress1 = sitalRtAddress_28;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetRtAddress1_29 ()
{
	/// @pseudocode

	/// Set test constant.
	s_wRtAddress1 = sitalRtAddress_29;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetRtAddress1_30 ()
{
	/// @pseudocode

	/// Set test constant.
	s_wRtAddress1 = sitalRtAddress_30;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetRtAddress1_31 ()
{
	/// @pseudocode

	/// Set test constant.
	s_wRtAddress1 = sitalRtAddress_31;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to given value.
///
/// @param iRtAddress (in)	RT address number (0-31)
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetRtAddress2 (int iRtAddress)
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	if ((0 > iRtAddress) && (31 < iRtAddress))
	{
		return sitalReturnCode_INVALID_PARAMETER;
	}

	/// Set test constant.
	s_wRtAddress2 = (U16BIT)iRtAddress;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetRtAddress2_00 ()
{
	/// @pseudocode

	/// Set test constant.
	s_wRtAddress2 = sitalRtAddress_0;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetRtAddress2_01 ()
{
	/// @pseudocode

	/// Set test constant.
	s_wRtAddress2 = sitalRtAddress_1;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetRtAddress2_02 ()
{
	/// @pseudocode

	/// Set test constant.
	s_wRtAddress2 = sitalRtAddress_2;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetRtAddress2_03 ()
{
	/// @pseudocode

	/// Set test constant.
	s_wRtAddress2 = sitalRtAddress_3;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetRtAddress2_04 ()
{
	/// @pseudocode

	/// Set test constant.
	s_wRtAddress2 = sitalRtAddress_4;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetRtAddress2_05 ()
{
	/// @pseudocode

	/// Set test constant.
	s_wRtAddress2 = sitalRtAddress_5;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetRtAddress2_06 ()
{
	/// @pseudocode

	/// Set test constant.
	s_wRtAddress2 = sitalRtAddress_6;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetRtAddress2_07 ()
{
	/// @pseudocode

	/// Set test constant.
	s_wRtAddress2 = sitalRtAddress_7;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetRtAddress2_08 ()
{
	/// @pseudocode

	/// Set test constant.
	s_wRtAddress2 = sitalRtAddress_8;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetRtAddress2_09 ()
{
	/// @pseudocode

	/// Set test constant.
	s_wRtAddress2 = sitalRtAddress_9;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetRtAddress2_10 ()
{
	/// @pseudocode

	/// Set test constant.
	s_wRtAddress2 = sitalRtAddress_10;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetRtAddress2_11 ()
{
	/// @pseudocode

	/// Set test constant.
	s_wRtAddress2 = sitalRtAddress_11;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetRtAddress2_12 ()
{
	/// @pseudocode

	/// Set test constant.
	s_wRtAddress2 = sitalRtAddress_12;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetRtAddress2_13 ()
{
	/// @pseudocode

	/// Set test constant.
	s_wRtAddress2 = sitalRtAddress_13;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetRtAddress2_14 ()
{
	/// @pseudocode

	/// Set test constant.
	s_wRtAddress2 = sitalRtAddress_14;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetRtAddress2_15 ()
{
	/// @pseudocode

	/// Set test constant.
	s_wRtAddress2 = sitalRtAddress_15;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetRtAddress2_16 ()
{
	/// @pseudocode

	/// Set test constant.
	s_wRtAddress2 = sitalRtAddress_16;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetRtAddress2_17 ()
{
	/// @pseudocode

	/// Set test constant.
	s_wRtAddress2 = sitalRtAddress_17;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetRtAddress2_18 ()
{
	/// @pseudocode

	/// Set test constant.
	s_wRtAddress2 = sitalRtAddress_18;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetRtAddress2_19 ()
{
	/// @pseudocode

	/// Set test constant.
	s_wRtAddress2 = sitalRtAddress_19;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetRtAddress2_20 ()
{
	/// @pseudocode

	/// Set test constant.
	s_wRtAddress2 = sitalRtAddress_20;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetRtAddress2_21 ()
{
	/// @pseudocode

	/// Set test constant.
	s_wRtAddress2 = sitalRtAddress_21;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetRtAddress2_22 ()
{
	/// @pseudocode

	/// Set test constant.
	s_wRtAddress2 = sitalRtAddress_22;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetRtAddress2_23 ()
{
	/// @pseudocode

	/// Set test constant.
	s_wRtAddress2 = sitalRtAddress_23;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetRtAddress2_24 ()
{
	/// @pseudocode

	/// Set test constant.
	s_wRtAddress2 = sitalRtAddress_24;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetRtAddress2_25 ()
{
	/// @pseudocode

	/// Set test constant.
	s_wRtAddress2 = sitalRtAddress_25;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetRtAddress2_26 ()
{
	/// @pseudocode

	/// Set test constant.
	s_wRtAddress2 = sitalRtAddress_26;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetRtAddress2_27 ()
{
	/// @pseudocode

	/// Set test constant.
	s_wRtAddress2 = sitalRtAddress_27;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetRtAddress2_28 ()
{
	/// @pseudocode

	/// Set test constant.
	s_wRtAddress2 = sitalRtAddress_28;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetRtAddress2_29 ()
{
	/// @pseudocode

	/// Set test constant.
	s_wRtAddress2 = sitalRtAddress_29;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetRtAddress2_30 ()
{
	/// @pseudocode

	/// Set test constant.
	s_wRtAddress2 = sitalRtAddress_30;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetRtAddress2_31 ()
{
	/// @pseudocode

	/// Set test constant.
	s_wRtAddress2 = sitalRtAddress_31;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to given value.
///
/// @param iRtMtModeTestFlag (in)	RT-MT mode test off (0) or on (otherwise)
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetRtMtModeTest (int iRtMtModeTestFlag)
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	/// Set test constant.
	if (0U == iRtMtModeTestFlag)
	{
		s_bIsRtMtModeTest = FALSE;
		s_wRtInitializationMode = sitalMode_RT;
		s_wMtInitializationMode = sitalMode_MT;
	}
	else
	{
		s_bIsRtMtModeTest = TRUE;
		s_wRtInitializationMode = sitalMode_RT_AND_MT;
		s_wMtInitializationMode = sitalMode_RT_AND_MT;
	}

	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetRtMtModeTest_True ()
{
	/// @pseudocode

	/// Set test constant.
	s_bIsRtMtModeTest = TRUE;
	s_wRtInitializationMode = sitalMode_RT_AND_MT;
	s_wMtInitializationMode = sitalMode_RT_AND_MT;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to the value designated right after.
///
/// Note:
/// - This function does not check in any way the validity of the new requested setting, so that a problematic setting will be rolled on to the following ATP tests.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetRtMtModeTest_False ()
{
	/// @pseudocode

	/// Set test constant.
	s_bIsRtMtModeTest = FALSE;
	s_wRtInitializationMode = sitalMode_RT;
	s_wMtInitializationMode = sitalMode_MT;
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to given value.
///
/// @param iFrameTime (in)	Frame time (>=0, in units of 100 us).
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetFrameTime (int iFrameTime)
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.
	if ((0 > iFrameTime) || (0xFFFF < iFrameTime))
	{
		return sitalReturnCode_INVALID_PARAMETER;
	}

	/// Set test constant.
	s_wFrameTime = iFrameTime;

	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to given value.
///
/// @param iIsrPrintoutEnabled (in)	ISR printouts off (0) or on (otherwise)
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetIsrPrintoutState (int iIsrPrintoutEnabled)
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	/// Set test constant.
	if (0U == iIsrPrintoutEnabled)
	{
		s_bIsIsrPrintoutEnabled = FALSE;
	}
	else
	{
		s_bIsIsrPrintoutEnabled = TRUE;
	}

	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Set the Acceptance Test Plan (ATP) test constant that is designated in this function's name to given value.
///
/// @param iTestIterationsCount (in)	Number of test iterations to perform (>=0, where 0 means infinite).
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Atp_SetTestIterationsCount (int iTestIterationsCount)
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.
	if ((0 > iTestIterationsCount) || (0xFFFFFFFFU < (U32BIT)iTestIterationsCount))
	{
		return sitalReturnCode_INVALID_PARAMETER;
	}

	/// Set test constant.
	s_dwTestIterationsCount = iTestIterationsCount;

	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Test the stld1553 library in the henceforth described scenario:
/// Participant[s]: BC0, RT1, RT2, MT3.
/// Configuration: All participating devices must be able to communicate with each other.
/// Short description: Test the Rx/Tx of a frame that contains all message types.
/// Along test print suitable info and indications into the log.
///
/// @returnsitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_BcRtRtMt_FrameContainingAllMessageTypes ()
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	sitalProcess_Log_PrintLine ("#StldTester.BcRtRtMt_FrameContainingAllMessageTypes.Start\n");

	/// Set test arguments.
	U32BIT dwMaximumFrameRunTime; // The maximum overall frame run time (in ms).
	dwMaximumFrameRunTime = 3000; // 3 seconds.
	U16BIT wRtAddress_0; // A RT address (0-31).
	wRtAddress_0 = 29;
	U16BIT wRtAddress_1; // A RT address (0-31).
	wRtAddress_1 = 30;
	U16BIT wRtSubaddressRx; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31) that's set for a RT for Rx.
	wRtSubaddressRx = 1;
	U16BIT wRtSubaddressTx; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31) that's set for a RT for Tx.
	wRtSubaddressTx = 2;
	U16BIT wWordCount; // Word count (1-32).
	wWordCount = 32;
	S16BIT swDevice_0; // Logical number of device (0-31).
	swDevice_0 = 0;
	S16BIT swDevice_1; // Logical number of device (0-31).
	swDevice_1 = 1;
	S16BIT swDevice_2; // Logical number of device (0-31).
	swDevice_2 = 2;
	S16BIT swDevice_3; // Logical number of device (0-31).
	swDevice_3 = 3;

	S16BIT swResult; // Result of operation or function call.
	swResult = sitalReturnCode_SUCCESS;

	/// Get device count.
	U16BIT wRequiredDeviceCount; // The number of devices required for this test.
	wRequiredDeviceCount = 4;
	U16BIT wAvailableDeviceCount; // The total number of devices available in the system.
	swResult = sitalDevice_GetCount (&wAvailableDeviceCount);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %i\n", swResult);
		return swResult;
	}
	sitalProcess_Log_PrintLine ("#StldTester.Device.Count: available - %i, required - %i\n", wAvailableDeviceCount, wRequiredDeviceCount);
	if (wAvailableDeviceCount < wRequiredDeviceCount)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %s\n", "not_enough_devices_installed");
		return sitalReturnCode_SUCCESS;
	}
	
	try
	{
		/// Initialize device-0 (BC) as a real BC.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-bc, alloc_mem-none\n", swDevice_0);
		swResult = sitalDevice_Initialize (swDevice_0, sitalAccess_CARD, sitalMode_BC, 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-1 (RT) as a real RT.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-rt, alloc_mem-none\n", swDevice_1);
		swResult = sitalDevice_Initialize (swDevice_1, (U16BIT)sitalAccess_CARD, ((s_bIsRtMtModeTest ? sitalMode_RT_AND_MT : sitalMode_RT) | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-2 (RT) as a real RT.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-rt, alloc_mem-none\n", swDevice_2);
		swResult = sitalDevice_Initialize (swDevice_2, sitalAccess_CARD, ((s_bIsRtMtModeTest ? sitalMode_RT_AND_MT : sitalMode_RT) | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-3 (MT) as a real MT.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access - card, mode - mt, alloc_mem - none\n", swDevice_3);
		swResult = sitalDevice_Initialize (swDevice_3, sitalAccess_CARD, ((s_bIsRtMtModeTest ? sitalMode_RT_AND_MT : sitalMode_MT) | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Set device-1 (RT) address to RtAddress0.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set: unit - %i, address - %i\n", swDevice_1, wRtAddress_0);
		swResult = sitalRt_Address_Set (swDevice_1, wRtAddress_0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Set device-2 (RT) address to RtAddress1.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set: unit - %i, address - %i\n", swDevice_1, wRtAddress_1);
		swResult = sitalRt_Address_Set (swDevice_1, wRtAddress_1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-0 (BC) data-block-0, a data block of 64 data words.
		S16BIT swDataBlockId_0; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId_0 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i\n", swDevice_0, swDataBlockId_0, 64, 0);
		swResult = sitalBc_DataBlock_Create (swDevice_0, swDataBlockId_0, sitalBcDataBlockSizeOption_DOUBLE, NULL, 0);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-0 (BC) message-0, a BC-to-RT(RtAddress1/RtSubAddressRx) message that uses data-block-0 to transmit WordCount data words. Message bus: bus-1.
		S16BIT swMessageId_0; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create: unit - %i, message - %i, block - %i, rx_rt - %i, rx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", swDevice_0, swMessageId_0, swDataBlockId_0, wRtAddress_1, wRtSubaddressRx, wWordCount, 0, s_wBus1);
		swResult = sitalBc_Message_CreateBcToRt (swDevice_0, swMessageId_0, swDataBlockId_0, wRtAddress_1, wRtSubaddressRx, wWordCount, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-0 (BC) data-block-1, a data block of 64 data words.
		S16BIT swDataBlockId_1; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId_1 = 2;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i\n", swDevice_0, swDataBlockId_1, 64, 0);
		swResult = sitalBc_DataBlock_Create (swDevice_0, swDataBlockId_1, sitalBcDataBlockSizeOption_DOUBLE, NULL, 0);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-0 (BC) message-1, a RT(RtAddress2/RtSubAddressTx)-to-BC message that uses data-block-1 to receive WordCount data words. Message bus: bus-1.
		S16BIT swMessageId_1; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_1 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBc.Create: unit - %i, message - %i, block - %i, tx_rt - %i, tx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", swDevice_0, swMessageId_1, swDataBlockId_1, wRtAddress_0, wRtSubaddressRx, wWordCount, 0, s_wBus1);
		swResult = sitalBc_Message_CreateRtToBc (swDevice_0, swMessageId_1, swDataBlockId_1, wRtAddress_0, wRtSubaddressTx, wWordCount, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBc.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-0 (BC) data-block-2, a data block of 64 data words.
		S16BIT swDataBlockId_2; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId_2 = 3;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i\n", swDevice_0, swDataBlockId_2, 64, 0);
		swResult = sitalBc_DataBlock_Create (swDevice_0, swDataBlockId_2, sitalBcDataBlockSizeOption_DOUBLE, NULL, 0);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-0 (BC) message-2, a RT(RtAddress/RtSubAddressTx)-to-RT(RtAddress/RtSubAddressRx) message that uses data-block-2 to receive WordCount data words. Message bus: bus-1.
		S16BIT swMessageId_2; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_2 = 2;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToRt.Create: unit - %i, message - %i, block - %i, tx_rt - %i, tx_rt_sa - %i, rx_rt - %i, rx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", swDevice_0, swMessageId_2, swDataBlockId_2, wRtAddress_0, wRtSubaddressTx, wRtAddress_1, wRtSubaddressRx, wWordCount, 0, s_wBus1);
		swResult = sitalBc_Message_CreateRtToRt (swDevice_0, swMessageId_2, swDataBlockId_2, wRtAddress_1, wRtSubaddressRx, wWordCount, wRtAddress_0, wRtSubaddressRx, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToRt.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-0 (BC) data-block-3, a data block of 64 data words.
		S16BIT swDataBlockId_3; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId_3 = 4;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i\n", swDevice_0, swDataBlockId_3, 64, 0);
		swResult = sitalBc_DataBlock_Create (swDevice_0, swDataBlockId_3, sitalBcDataBlockSizeOption_DOUBLE, NULL, 0);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-0 (BC) message-3, a BC-to-RT(RtAddress)-Mode(10001) message that uses data-block-3. Message bus: bus-1.
		S16BIT swMessageId_3; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_3 = 3;
		U16BIT wMessageDirection; // RT-related message direction.
		wMessageDirection = sitalMessageDirection_RX;
		U16BIT wModeCommand; // A mode command.
		wModeCommand = 0x11;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRtMode.Create: unit - %i, message - %i, block - %i, rx_rt - %i, direction - %i, mode - %i, gap - %i, opts-x%X\n", swDevice_0, swMessageId_3, swDataBlockId_3, wRtAddress_1, wMessageDirection, wModeCommand, 0, s_wBus1);
		swResult = sitalBc_Message_CreateBcToOrFromRtMode (swDevice_0, swMessageId_3, swDataBlockId_3, wRtAddress_1, wMessageDirection, wModeCommand, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRtMode.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-0 (BC) data-block-4, a data block of 64 data words.
		S16BIT swDataBlockId_4; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId_4 = 5;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i\n", swDevice_0, swDataBlockId_4, 64, 0);
		swResult = sitalBc_DataBlock_Create (swDevice_0, swDataBlockId_4, sitalBcDataBlockSizeOption_DOUBLE, NULL, 0);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-0 (BC) message-4, a BC-to-Broadcast(RtSubAddressRx) message that uses data-block-4 to transmit WordCount data words. Message bus: bus-1.
		S16BIT swMessageId_4; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_4 = 4;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToBroadcast.Create: unit - %i, message - %i, block - %i, rx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", swDevice_0, swMessageId_0, swDataBlockId_0, wRtSubaddressRx, wWordCount, 0, s_wBus1);
		swResult = sitalBc_Message_CreateBcToBroadcast (swDevice_0, swMessageId_4, swDataBlockId_4, wRtSubaddressRx, wWordCount, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToBroadcast.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-0 (BC) data-block-5, a data block of 64 data words.
		S16BIT swDataBlockId_5; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId_5 = 6;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i\n", swDevice_0, swDataBlockId_5, 64, 0);
		swResult = sitalBc_DataBlock_Create (swDevice_0, swDataBlockId_5, sitalBcDataBlockSizeOption_DOUBLE, NULL, 0);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-0 (BC) message-5, a RT(RtAddress/RtSubAddress)-to-Broadcast(RtSubAddress) message that transmits WordCount data words. BC side data-block: data-block-4. Message bus: Wrong-bus-1.
		// Note: This message is intentionally wrongly defined, what will fail the test unless the message will be fixed later on (see below).
		S16BIT swMessageId_5; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_5 = 5;
		U16BIT wWrongBus1; // A wrong bus (sitalBcControlWord_BUS_*) to use instead of bus-1.
		wWrongBus1 = ( (sitalBcControlWord_BUS_A == s_wBus1) ? sitalBcControlWord_BUS_B : sitalBcControlWord_BUS_A );
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBroadcast.Create: unit - %i, message - %i, block - %i, tx_rt - %i, tx_rt_sa - %i, rx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", swDevice_0, swMessageId_5, swDataBlockId_5, wRtAddress_0, wRtSubaddressTx, wRtSubaddressRx, wWordCount, 0, wWrongBus1);
		swResult = sitalBc_Message_CreateRtToBroadcast (swDevice_0, swMessageId_5, swDataBlockId_5, wRtSubaddressRx, wWordCount, wRtAddress_0, wRtSubaddressTx, 0, wWrongBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBroadcast.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Modify for device-2 (BC) message-5, into a RT(RtAddress/RtSubAddressTx)-to-Broadcast(RtSubAddressRx) message that uses data-block-4 to transmit WordCount data words. Message bus: bus-1.
		// Note: This message has been intentionally wrongly defined (see above), so it it therefore fixed here, and only if the fix actually succeeds will the test pass.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBroadcast.Modify: unit - %i, message - %i, block - %i, tx_rt - %i, tx_rt_sa - %i, rx_rt_sa - %i, wc - %i, gap - %i, opts-x%X, modified - x%X\n", swDevice_0, swMessageId_5, swDataBlockId_5, wRtAddress_0, wRtSubaddressTx, wRtSubaddressRx, wWordCount, 0, s_wBus1, sitalBcMessageModifier_MESSAGE1_BC_CONTROL);
		swResult = sitalBc_Message_ModifyRtToBroadcast (swDevice_0, swMessageId_5, swDataBlockId_5, wRtSubaddressRx, wWordCount, wRtAddress_0, wRtSubaddressTx, 0, s_wBus1, sitalBcMessageModifier_MESSAGE1_BC_CONTROL);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBroadcast.Modify.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-0 (BC) data-block-6, a data block of 64 data words.
		S16BIT swDataBlockId_6; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId_6 = 7;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i\n", swDevice_0, swDataBlockId_6, 64, 0);
		swResult = sitalBc_DataBlock_Create (swDevice_0, swDataBlockId_6, sitalBcDataBlockSizeOption_DOUBLE, NULL, 0);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-0 (BC) message-6, a BC-to-Broadcast-Mode(10001) message that uses data-block-6. Message bus: Wrong-bus-1.
		// Note: This message is intentionally wrongly defined, what will fail the test unless the message will be fixed later on (see below).
		S16BIT swMessageId_6; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_6 = 6;
		wMessageDirection = sitalMessageDirection_RX;
		wModeCommand = 0x11;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToBroadcastMode.Create: unit - %i, message - %i, block - %i, direction - %i, mode - %i, gap - %i, opts-x%X\n", swDevice_0, swMessageId_0, swDataBlockId_0, wMessageDirection, wModeCommand, 0, wWrongBus1);
		swResult = sitalBc_Message_CreateBcToOrFromBroadcastMode (swDevice_0, swMessageId_6, swDataBlockId_6, wMessageDirection, wModeCommand, 0, wWrongBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToOrFromBroadcastMode.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Modify for device-2 (BC) message-6, into a BC-to-Broadcast-Mode(10001) message that uses data-block-6. Message bus: bus-1.
		// Note: This message has been intentionally wrongly defined (see above), so it it therefore fixed here, and only if the fix actually succeeds will the test pass.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToBroadcastMode.Modify: unit - %i, message - %i, block - %i, direction - %i, mode - %i, gap - %i, opts-x%X\n", swDevice_0, swMessageId_0, swDataBlockId_0, wMessageDirection, wModeCommand, 0, s_wBus1, sitalBcMessageModifier_MESSAGE1_BC_CONTROL);
		swResult = sitalBc_Message_ModifyBcToOrFromBroadcastMode (swDevice_0, swMessageId_6, swDataBlockId_6, wMessageDirection, wModeCommand, 0, s_wBus1, sitalBcMessageModifier_MESSAGE1_BC_CONTROL);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToOrFromBroadcastMode.Modify.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-0 (BC) command-0, an XQF command that executes message-0.
		S16BIT swCommandId_0; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", swDevice_0, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0);
		swResult = sitalBc_Command_Create (swDevice_0, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-0 (BC) command-1, an XQF command that executes message-1.
		S16BIT swCommandId_1; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_1 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", swDevice_0, swCommandId_1, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_1, 0);
		swResult = sitalBc_Command_Create (swDevice_0, swCommandId_1, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_1, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-0 (BC) command-2, an XQF command that executes message-2.
		S16BIT swCommandId_2; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_2 = 2;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", swDevice_0, swCommandId_2, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_2, 0);
		swResult = sitalBc_Command_Create (swDevice_0, swCommandId_2, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_2, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-0 (BC) command-3, an XQF command that executes message-3.
		S16BIT swCommandId_3; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_3 = 3;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", swDevice_0, swCommandId_3, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_3, 0);
		swResult = sitalBc_Command_Create (swDevice_0, swCommandId_3, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_3, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-0 (BC) command-4, an XQF command that executes message-4.
		S16BIT swCommandId_4; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_4 = 4;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", swDevice_0, swCommandId_4, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_4, 0);
		swResult = sitalBc_Command_Create (swDevice_0, swCommandId_4, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_4, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-0 (BC) command-5, an XQF command that executes message-5.
		S16BIT swCommandId_5; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_5 = 5;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", swDevice_0, swCommandId_5, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_5, 0);
		swResult = sitalBc_Command_Create (swDevice_0, swCommandId_5, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_5, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-0 (BC) command-6, an XQF command that executes message-6.
		S16BIT swCommandId_6; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_6 = 6;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", swDevice_0, swCommandId_6, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_6, 0);
		swResult = sitalBc_Command_Create (swDevice_0, swCommandId_6, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_6, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-0 (BC) command-7, a subroutine call command that executes minor-frame-0. 
		S16BIT swCommandId_7; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_7 = 7;
		S16BIT swFrameId_0; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", swDevice_0, swCommandId_7, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, swFrameId_0, 0);
		swResult = sitalBc_Command_Create (swDevice_0, swCommandId_7, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, (U16BIT)swFrameId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-0 (BC) minor-frame-0, a frame that contains commands 0-6.
		S16BIT swaCommandIds[sitalBcCounter_COMMANDS]; // An array of command-IDs.
		swaCommandIds[0] = swCommandId_0;
		swaCommandIds[1] = swCommandId_1;
		swaCommandIds[2] = swCommandId_2;
		swaCommandIds[3] = swCommandId_3;
		swaCommandIds[4] = swCommandId_4;
		swaCommandIds[5] = swCommandId_5;
		swaCommandIds[6] = swCommandId_6;
		U16BIT wCommandIdCount; // The number of command IDs in the frame (1-sitalBcCounter_FRAME_COMMANDS).
		wCommandIdCount = 7;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i - %i), time - %i, flags - %i\n", swDevice_0, swFrameId_0, "minor", wCommandIdCount, swCommandId_0, swCommandId_6, 0, 0);
		swResult = sitalBc_Frame_Create (swDevice_0, swFrameId_0, sitalBcFrameType_MINOR, swaCommandIds, wCommandIdCount, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-0 (BC) major-frame-1, a frame that contains command-7 of device-0 (BC) and whose frame time is FrameTime us.
		S16BIT swFrameId_1; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_1 = 1;
		swaCommandIds[0] = swCommandId_7;
		wCommandIdCount = 1;
		U16BIT wFrameTime; // The time (in units of 100 us) that the frame will take to complete.
		wFrameTime = 1000; // Back-to-back.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", swDevice_0, swFrameId_1, "major", wCommandIdCount, swCommandId_7, (wFrameTime / 10), 0);
		swResult = sitalBc_Frame_Create (swDevice_0, swFrameId_1, sitalBcFrameType_MAJOR, swaCommandIds, wCommandIdCount, wFrameTime, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Start device-3 (MT).
		sitalProcess_Log_PrintLine ("#StldTester.Mt.Start: unit - %i\n", swDevice_3);
		swResult = sitalMt_Start (swDevice_3);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Start device-2 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Start: unit - %i\n", swDevice_2);
		swResult = sitalRt_Start (swDevice_2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Start device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Start: unit - %i\n", swDevice_1);
		swResult = sitalRt_Start (swDevice_1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Zero the global ISR counter of ended minor frames.
		s_dwMinorFrameCounter = 0;

		U32BIT dwMinorFrameCounter; // The overall number of transmitted minor frames.
		dwMinorFrameCounter = 0;

		/// Configure device-0 (BC) to get IRQ-3 notifications using an ISR that collects interrupt data, counts ISR calls, and signals interested threads.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Manipulate: unit - %i, enable - %i, irq - %08X, callback - %s\n", swDevice_0, TRUE, sitalInterruptRegister2_BC_IRQ3, "stldTester_Bc_UserIsr");
		swResult = sitalDevice_Irq_Manipulate (swDevice_0, TRUE, sitalInterruptRegister2_BC_IRQ3, stldTester_Bc_UserIsr);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Set.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Start device-0 (BC) running major-frame-1 for FrameCount times.
		U16BIT wFrameCount; // The number of frames to run.
		wFrameCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", swDevice_0, swFrameId_1, wFrameCount);
		swResult = sitalBc_Start (swDevice_0, swFrameId_1, wFrameCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		///	Keep waiting until the global ISR frame counter shows that the frame has ended OR the overall frame run timer expires (the earlier):
		PrecisionTimer pt; // A precision timer.
		U32BIT dwOverallFrameRunTime; // The expected overall frame run time (in ms).
		dwOverallFrameRunTime = 0;
		U32BIT dwSleepTime; // Sleeping period (in ms).
		dwSleepTime = 100; // 0.1 second.
		while ((dwMinorFrameCounter == s_dwMinorFrameCounter) && (dwMaximumFrameRunTime > dwOverallFrameRunTime))
		{
			///			Wait for a while.
			sitalProcess_Log_PrintLine ("#StldTester.Wait: time_passed - %ims, isr_calls - %i\n", dwOverallFrameRunTime, s_dwMinorFrameCounter);
			sitalProcess_Log_PrintLine ("\tWAIT_FOR %i us\n", (dwSleepTime * 1000));
			pt.AccuratelySleep (dwSleepTime);

			///			Update the overall frame run timer.
			dwOverallFrameRunTime += dwSleepTime;
		}
		sitalProcess_Log_PrintLine ("#StldTester.Wait.End: unit - %i, isr_invoked - %i\n", swDevice_0, (dwMinorFrameCounter != s_dwMinorFrameCounter));

		/// Stop device-0 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop: unit - %i\n", swDevice_0);
		swResult = sitalBc_Stop (swDevice_0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Stop device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Stop: unit - %i\n", swDevice_1);
		swResult = sitalRt_Stop (swDevice_1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Stop device-2 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Stop: unit - %i\n", swDevice_2);
		swResult = sitalRt_Stop (swDevice_2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Stop device-3 (MT).
		sitalProcess_Log_PrintLine ("#StldTester.Mt.Stop: unit - %i\n", swDevice_3);
		swResult = sitalMt_Stop (swDevice_3);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Verify that device-3 (MT) has detected all 7 messages on bus-1.
		// Note:
		// - In case participating devices are only connected to a single bus, the wrong number of messages will be received.
		// - In case participating devices are connected to two buses, messages received on the wrong bus will have it signed in their BSW.
		U16BIT waBuffer[10 * sitalMtMaximum_MESSAGE_SIZE]; // A buffer in which the raw messages are stored.
		sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackRaw: unit - %i\n", swDevice_3);
		swResult = sitalMt_Message_GetFromStackRaw (swDevice_3, waBuffer, (10 * sitalMtMaximum_MESSAGE_SIZE), sitalMtStack_ACTIVE);
		if (7 != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackRaw.Error: error - %i\n", swResult);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		U16BIT* wpRawMessage; // A pointer to a raw message.
		wpRawMessage = (U16BIT*)waBuffer;
		U16BIT wExpectedBusBitB; // The expected value for the bus-B bit in the block status word of all received messages.
		wExpectedBusBitB = ( (sitalBcControlWord_BUS_A == s_wBus1) ? 0 : sitalBcBlockStatusWord_BUS_B );
		for (U16BIT i=0; i<7; i++, wpRawMessage+=sitalMtMaximum_MESSAGE_SIZE)
		{
			U16BIT wBlockStatusWord; // Message block status word.
			wBlockStatusWord = (*wpRawMessage);
			if (wExpectedBusBitB != (sitalBcBlockStatusWord_BUS_B & wBlockStatusWord))
			{
				sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackRaw.Error: error - %s\n", "bad_bus");
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}
		}

		/// Free device-0 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", swDevice_0);
		swResult = sitalDevice_Free (swDevice_0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", swDevice_1);
		swResult = sitalDevice_Free (swDevice_1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-2 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", swDevice_2);
		swResult = sitalDevice_Free (swDevice_2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-3 (MT).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", swDevice_3);
		swResult = sitalDevice_Free (swDevice_3);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}
	}
	catch (int)
	{
		sitalDevice_Free (swDevice_0);
		sitalDevice_Free (swDevice_1);
		sitalDevice_Free (swDevice_2);
		sitalDevice_Free (swDevice_3);
	}

	sitalProcess_Log_PrintLine ("#StldTester.BcRtRtMt_FrameContainingAllMessageTypes.End: test_succeeded - %c\n", ((sitalReturnCode_SUCCESS == swResult) ? 'Y' : 'N'));
	return swResult;
}



/// <summary>
/// Test the stld1553 library in the henceforth described scenario:
/// Participant[s]: BC.
/// Configuration: -
/// Short description: Test the operation of the device open & close functions.
/// Along test print suitable info and indications into the log.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Bc_OpenCloseDevice ()
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	sitalProcess_Log_PrintLine ("#StldTester.Bc_OpenCloseDevice.Start\n");

	S16BIT swDevice_0; // Logical number of device (0-31).
	swDevice_0 = 0;

	S16BIT swResult; // Result of operation or function call.
	swResult = sitalReturnCode_SUCCESS;

	/// Get device count.
	U16BIT wRequiredDeviceCount; // The number of devices required for this test.
	wRequiredDeviceCount = (1 + swDevice_0);
	U16BIT wAvailableDeviceCount; // The total number of devices available in the system.
	swResult = sitalDevice_GetCount (&wAvailableDeviceCount);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %i\n", swResult);
		return swResult;
	}
	sitalProcess_Log_PrintLine ("#StldTester.Device.Count: available - %i, required - %i\n", wAvailableDeviceCount, wRequiredDeviceCount);
	if (wAvailableDeviceCount < wRequiredDeviceCount)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %s\n", "not_enough_devices_installed");
		return sitalReturnCode_SUCCESS;
	}

	try
	{
		/// Initialize device-0 (BC) as a real BC.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-bc, alloc_mem-none\n", swDevice_0);
		swResult = sitalDevice_Initialize (swDevice_0, sitalAccess_CARD, sitalMode_BC, 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-0 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", swDevice_0);
		swResult = sitalDevice_Free (swDevice_0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-0 (BC) as a real BC.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-bc, alloc_mem-none\n", swDevice_0);
		swResult = sitalDevice_Initialize (swDevice_0, sitalAccess_CARD, sitalMode_BC, 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-0 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", swDevice_0);
		swResult = sitalDevice_Free (swDevice_0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}
	}
	catch (int)
	{
		sitalDevice_Free (swDevice_0);
	}

	sitalProcess_Log_PrintLine ("#StldTester.Bc_OpenCloseDevice.End: test_succeeded - %c\n", ((sitalReturnCode_SUCCESS == swResult) ? 'Y' : 'N'));
	return swResult;
}



/// <summary>
/// Test the stld1553 library in the henceforth described scenario:
/// Participant[s]: BC.
/// Configuration: -
/// Short description: Test the operation of the ISR calls system.
/// Along test print suitable info and indications into the log.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Bc_Isr ()
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	sitalProcess_Log_PrintLine ("#StldTester.Bc_Isr.Start\n");

	/// Zero the global ISR counter of ended minor frames.
	s_dwMinorFrameCounter = 0;

	/// Zero the overall frame run timer.
	U32BIT dwOverallFrameRunTime; // The expected overall frame run time (in ms).
	dwOverallFrameRunTime = 0;

	/// Set test arguments.
	U16BIT wFrameCount; // The number of frames to run.
	wFrameCount = 10000;
	S16BIT swDevice_0; // Logical number of device (0-31).
	swDevice_0 = 0;

	S16BIT swResult; // Result of operation or function call.
	swResult = sitalReturnCode_SUCCESS;

	/// Get device count.
	U16BIT wRequiredDeviceCount; // The number of devices required for this test.
	wRequiredDeviceCount = (1 + swDevice_0);
	U16BIT wAvailableDeviceCount; // The total number of devices available in the system.
	swResult = sitalDevice_GetCount (&wAvailableDeviceCount);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %i\n", swResult);
		return swResult;
	}
	sitalProcess_Log_PrintLine ("#StldTester.Device.Count: available - %i, required - %i\n", wAvailableDeviceCount, wRequiredDeviceCount);
	if (wAvailableDeviceCount < wRequiredDeviceCount)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %s\n", "not_enough_devices_installed");
		return sitalReturnCode_SUCCESS;
	}

	try
	{
		/// Initialize device-0 (BC) as a real BC.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-bc, alloc_mem-none\n", swDevice_0);
		swResult = sitalDevice_Initialize (swDevice_0, sitalAccess_CARD, sitalMode_BC, 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-0 (BC) data-block-0, a data block of 64 data words.
		S16BIT swDataBlockId_0; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId_0 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i\n", swDevice_0, swDataBlockId_0, 64, 0);
		swResult = sitalBc_DataBlock_Create (swDevice_0, swDataBlockId_0, sitalBcDataBlockSizeOption_DOUBLE, NULL, 0);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-0 (BC) message-0, a BC-to-Broadcast(some RtSubAddressRx) message that uses data-block-0 to transmit some data words. Message bus: bus-1.
		S16BIT swMessageId_0; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToBroadcast.Create: unit - %i, message - %i, block - %i, rx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", swDevice_0, swMessageId_0, swDataBlockId_0, 1, 32, 0, s_wBus1);
		swResult = sitalBc_Message_CreateBcToBroadcast (swDevice_0, swMessageId_0, swDataBlockId_0, 1, 32, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToBroadcast.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-0 (BC) command-0, a XEQ command that executes message-0.
		S16BIT swCommandId_0; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", swDevice_0, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0);
		swResult = sitalBc_Command_Create (swDevice_0, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-0 (BC) command-1, a subroutine call command that executes minor-frame-0. 
		S16BIT swCommandId_1; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_1 = 1;
		S16BIT swFrameId_0; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", swDevice_0, swCommandId_1, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, swFrameId_0, 0);
		swResult = sitalBc_Command_Create (swDevice_0, swCommandId_1, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, (U16BIT)swFrameId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-0 (BC) minor-frame-0, a frame that contains command-0 and command-1.
		S16BIT swaCommandIds[sitalBcCounter_COMMANDS]; // An array of command-IDs.
		swaCommandIds[0] = swCommandId_0;
		U16BIT wCommandIdCount; // The number of command IDs in the frame (1-sitalBcCounter_FRAME_COMMANDS).
		wCommandIdCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %i, flags - %i\n", swDevice_0, swFrameId_0, "minor", wCommandIdCount, swCommandId_0, 0, 0);
		swResult = sitalBc_Frame_Create (swDevice_0, swFrameId_0, sitalBcFrameType_MINOR, swaCommandIds, wCommandIdCount, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-0 (BC) major-frame-1, a frame that contains command-1 of device-0 (BC) and whose frame time is FrameTime us.
		S16BIT swFrameId_1; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_1 = 1;
		swaCommandIds[0] = swCommandId_1;
		wCommandIdCount = 1;
		U16BIT wFrameTime; // The time (in units of 100 us) that the frame will take to complete.
		wFrameTime = 200; // 20ms.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", swDevice_0, swFrameId_1, "major", wCommandIdCount, swCommandId_1, (wFrameTime / 10), 0);
		swResult = sitalBc_Frame_Create (swDevice_0, swFrameId_1, sitalBcFrameType_MAJOR, swaCommandIds, wCommandIdCount, wFrameTime, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Configure device-0 (BC) to get IRQ-3 notifications using an ISR that collects interrupt data, counts ISR calls, and signals interested threads.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Manipulate: unit - %i, enable - %i, irq - %08X, callback - %s\n", swDevice_0, TRUE, sitalInterruptRegister2_BC_IRQ3, "stldTester_Bc_UserIsr");
		swResult = sitalDevice_Irq_Manipulate (swDevice_0, TRUE, sitalInterruptRegister2_BC_IRQ3, stldTester_Bc_UserIsr);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Set.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Start device-0 (BC) running major-frame-1 for FrameCount times.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i, isr_calls - %i\n", swDevice_0, swFrameId_1, wFrameCount, s_dwMinorFrameCounter);
		swResult = sitalBc_Start (swDevice_0, swFrameId_1, wFrameCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Keep waiting until the global ISR frame counter becomes equal to FrameCount OR the overall frame run timer gets greater-than (FrameTime * FrameCount) (the earlier):
		U32BIT dwExpectedOverallFrameRunTime; // The expected overall frame run time (in ms).
		dwExpectedOverallFrameRunTime = (((U32BIT)wFrameTime * wFrameCount) / 10);
		U32BIT dwSleepTime; // Sleeping period (in ms).
		dwSleepTime = 10000; // 10 seconds.
		PrecisionTimer pt; // A precision timer.
		while ((wFrameCount > s_dwMinorFrameCounter) && (dwExpectedOverallFrameRunTime > dwOverallFrameRunTime))
		{
			/// Wait for a while.
			sitalProcess_Log_PrintLine ("#StldTester.Wait: time_passed - %ims, isr_calls - %i\n", dwOverallFrameRunTime, s_dwMinorFrameCounter);
			sitalProcess_Log_PrintLine ("\tWAIT_FOR %i us\n", (dwSleepTime * 1000));
			pt.AccuratelySleep (dwSleepTime);

			/// Update the overall frame run timer.
			dwOverallFrameRunTime += dwSleepTime;
		}

		/// Stop device-0 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop: unit - %i\n", swDevice_0);
		swResult = sitalBc_Stop (swDevice_0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-0 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", swDevice_0);
		swResult = sitalDevice_Free (swDevice_0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}
	}
	catch (int)
	{
		sitalDevice_Free (swDevice_0);
	}

	swResult = ( ((sitalReturnCode_SUCCESS == swResult) && (wFrameCount != s_dwMinorFrameCounter)) ? sitalReturnCode_TEST_FAIL : swResult);
	sitalProcess_Log_PrintLine ("#StldTester.Bc_Isr.End: frame_count - %i, isr_calls - %i, test_succeeded - %c\n", wFrameCount, s_dwMinorFrameCounter, ((sitalReturnCode_SUCCESS == swResult) ? 'Y' : 'N'));
	return swResult;
}



/// <summary>
/// Test the stld1553 library in the henceforth described scenario:
/// Participant[s]: BC, RT.
/// Configuration: All participating devices must be able to communicate with each other.
/// Short description: Test the BC - RT bi-directional Rx/Tx of data.
/// Along test print suitable info and indications into the log.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_BcRt_DataRxTx ()
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	sitalProcess_Log_PrintLine ("#StldTester.BcRt_DataRxTx.Start\n");

	/// Set test arguments.
	U16BIT wRtAddress; // A RT address (0-31).
	wRtAddress = 30;
	U16BIT wRtSubaddressRx; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31) that's set for a RT for Rx.
	wRtSubaddressRx = 1;
	U16BIT wRtSubaddressTx; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31) that's set for a RT for Tx.
	wRtSubaddressTx = 2;
	U16BIT wWordCount; // Word count (1-32).
	wWordCount = 10;
	U16BIT wFrameCount; // The number of frames to run.
	wFrameCount = 1;
	S16BIT swDevice_0; // Logical number of device (0-31).
	swDevice_0 = 0;
	S16BIT swDevice_1; // Logical number of device (0-31).
	swDevice_1 = 1;

	S16BIT swResult; // Result of operation or function call.
	swResult = sitalReturnCode_SUCCESS;

	/// Get device count.
	U16BIT wRequiredDeviceCount; // The number of devices required for this test.
	wRequiredDeviceCount = (1 + max(swDevice_0, swDevice_1));
	U16BIT wAvailableDeviceCount; // The total number of devices available in the system.
	swResult = sitalDevice_GetCount (&wAvailableDeviceCount);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %i\n", swResult);
		return swResult;
	}
	sitalProcess_Log_PrintLine ("#StldTester.Device.Count: available - %i, required - %i\n", wAvailableDeviceCount, wRequiredDeviceCount);
	if (wAvailableDeviceCount < wRequiredDeviceCount)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %s\n", "not_enough_devices_installed");
		return sitalReturnCode_SUCCESS;
	}

	try
	{
		/// Initialize device-0 (BC) as a real BC.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-bc, alloc_mem-none\n", swDevice_0);
		swResult = sitalDevice_Initialize (swDevice_0, sitalAccess_CARD, (sitalMode_BC | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-1 (RT) as a real RT.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-rt, alloc_mem-none\n", swDevice_1);
		swResult = sitalDevice_Initialize (swDevice_1, sitalAccess_CARD, ((s_bIsRtMtModeTest ? sitalMode_RT_AND_MT : sitalMode_RT) | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Set device-1 (RT) address to RtAddress.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set: unit - %i, address - %i\n", swDevice_1, wRtAddress);
		swResult = sitalRt_Address_Set (swDevice_1, wRtAddress);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-0 (BC) data-block-0, a data block of 64 data words.
		S16BIT swDataBlockId_0; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId_0 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i\n", swDevice_0, swDataBlockId_0, 64, 0);
		swResult = sitalBc_DataBlock_Create (swDevice_0, swDataBlockId_0, sitalBcDataBlockSizeOption_DOUBLE, NULL, 0);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-0 (BC) data-block-1, a data block of 64 data words, and fill the first WordCount data words with ascending values.
		S16BIT swDataBlockId_1; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId_1 = 2;
		U16BIT waBuffer[32]; // A buffer to read-from/write-into along data-block write/read operations, respectively.
		for (int i=0; i<wWordCount; i++) waBuffer[i] = i;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i, buf_contents - %i...\n", swDevice_1, swDataBlockId_1, 64, wWordCount, 0);
		swResult = sitalBc_DataBlock_Create (swDevice_0, swDataBlockId_1, sitalBcDataBlockSizeOption_DOUBLE, waBuffer, wWordCount);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-0 (BC) message-0, a RT(RtAddress/RtSubAddressTx)-to-BC message that uses data-block-0 to receive WordCount data words. Message bus: bus-1.
		S16BIT swMessageId_0; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBc.Create: unit - %i, message - %i, block - %i, tx_rt - %i, tx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", swDevice_0, swMessageId_0, swDataBlockId_0, wRtAddress, wRtSubaddressTx, wWordCount, 0, s_wBus1);
		swResult = sitalBc_Message_CreateRtToBc (swDevice_0, swMessageId_0, swDataBlockId_0, wRtAddress, wRtSubaddressTx, wWordCount, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBc.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-0 (BC) message-1, a BC-to-RT(RtAddress/RtSubAddressRx) message that uses data-block-1 to transmit WordCount data words. Message bus: bus-1.
		S16BIT swMessageId_1; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_1 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create: unit - %i, message - %i, block - %i, rx_rt - %i, rx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", swDevice_0, swMessageId_1, swDataBlockId_1, wRtAddress, wRtSubaddressRx, wWordCount, 0, s_wBus1);
		swResult = sitalBc_Message_CreateBcToRt (swDevice_0, swMessageId_1, swDataBlockId_1, wRtAddress, wRtSubaddressRx, wWordCount, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-0 (BC) command-0, an XQF command that executes message-0.
		S16BIT swCommandId_0; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", swDevice_0, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0);
		swResult = sitalBc_Command_Create (swDevice_0, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-0 (BC) command-1, an XQF command that executes message-1.
		S16BIT swCommandId_1; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_1 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", swDevice_0, swCommandId_1, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_1, 0);
		swResult = sitalBc_Command_Create (swDevice_0, swCommandId_1, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_1, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-0 (BC) command-2, a subroutine call command that executes minor-frame-0. 
		S16BIT swCommandId_2; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_2 = 2;
		S16BIT swFrameId_0; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", swDevice_0, swCommandId_2, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, swFrameId_0, 0);
		swResult = sitalBc_Command_Create (swDevice_0, swCommandId_2, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, (U16BIT)swFrameId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-0 (BC) minor-frame-0, a frame that contains command-0 and command-1.
		S16BIT swaCommandIds[sitalBcCounter_COMMANDS]; // An array of command-IDs.
		swaCommandIds[0] = swCommandId_0;
		swaCommandIds[1] = swCommandId_1;
		U16BIT wCommandIdCount; // The number of command IDs in the frame (1-sitalBcCounter_FRAME_COMMANDS).
		wCommandIdCount = 2;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i, %i), time - %i, flags - %i\n", swDevice_0, swFrameId_0, "minor", wCommandIdCount, swCommandId_0, swCommandId_1, 0, 0);
		swResult = sitalBc_Frame_Create (swDevice_0, swFrameId_0, sitalBcFrameType_MINOR, swaCommandIds, wCommandIdCount, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-0 (BC) major-frame-1, a frame that contains command-2 of device-0 (BC) and whose frame time is FrameTime us.
		S16BIT swFrameId_1; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_1 = 1;
		swaCommandIds[0] = swCommandId_2;
		wCommandIdCount = 1;
		U16BIT wFrameTime; // The time (in units of 100 us) that the frame will take to complete.
		wFrameTime = 1000; // 100ms.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", swDevice_0, swFrameId_1, "major", wCommandIdCount, swCommandId_2, (wFrameTime / 10), 0);
		swResult = sitalBc_Frame_Create (swDevice_0, swFrameId_1, sitalBcFrameType_MAJOR, swaCommandIds, wCommandIdCount, wFrameTime, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-1 (RT) data-block-0, a data block of 64 data words.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i\n", swDevice_1, swDataBlockId_0, 64, 0);
		swResult = sitalRt_DataBlock_Create (swDevice_1, swDataBlockId_0, sitalRtDataBlockType_DOUBLE, NULL, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Map for device-1 (RT) data-block-0 with Rx subaddress RtSubAddressRx, and legalize this subaddress.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.MapToSa: unit - %i, block - %i, sa - %i, msg_types - x%X, irq_opts-x%X, legalize - %c\n", swDevice_1, swDataBlockId_0, wRtSubaddressRx, sitalRtMessageType_RX, sitalRtDataBlockIrq_NONE, 'Y');
		swResult = sitalRt_DataBlock_MapToSubaddress (swDevice_1, swDataBlockId_0, wRtSubaddressRx, sitalRtMessageType_RX, sitalRtDataBlockIrq_NONE, TRUE);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.MapToSa.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-1 (RT) data-block-1, a data block of 64 data words, and fill the first WordCount data words with descending values.
		for (int i=0; i<wWordCount; i++) waBuffer[i] = (wWordCount - 1 - i);
		sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i, buf_contents-...%i\n", swDevice_1, swDataBlockId_1, 64, wWordCount, (wWordCount - 1));
		swResult = sitalRt_DataBlock_Create (swDevice_1, swDataBlockId_1, sitalRtDataBlockType_DOUBLE, waBuffer, wWordCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Map for device-1 (RT) data-block-1 with Tx subaddress RtSubAddressTx, and legalize this subaddress.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.MapToSa: unit - %i, block - %i, sa - %i, msg_types - x%X, irq_opts-x%X, legalize - %c\n", swDevice_1, swDataBlockId_1, wRtSubaddressTx, sitalRtMessageType_TX, sitalRtDataBlockIrq_NONE, 'Y');
		swResult = sitalRt_DataBlock_MapToSubaddress (swDevice_1, swDataBlockId_1, wRtSubaddressTx, sitalRtMessageType_TX, sitalRtDataBlockIrq_NONE, TRUE);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.MapToSa.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Start device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Start: unit - %i\n", swDevice_1);
		swResult = sitalRt_Start (swDevice_1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Start device-0 (BC) running major-frame-1 for 1-shot.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", swDevice_0, swFrameId_1, 1);
		swResult = sitalBc_Start (swDevice_0, swFrameId_1, 1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Wait for FrameTime ms.
		sitalProcess_Log_PrintLine ("#StldTester.Wait: time - %ims\n", (wFrameTime / 10));
		sitalProcess_Log_PrintLine ("\tWAIT_FOR %i us\n", ((U32BIT)wFrameTime * 100));
		PrecisionTimer pt; // A precision timer.
		pt.AccuratelySleep (wFrameTime / 10);

		/// With device-0 (BC) get message-0.
		U16BIT waRawMessage[sitalBcMaximum_MESSAGE_SIZE]; // A buffer that stores a raw BC message.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetByIdRaw: unit - %i, message - %i, purge - %i\n", swDevice_0, swMessageId_0, TRUE);
		swResult = sitalBc_Message_GetByIdRaw (swDevice_0, swMessageId_0, waRawMessage, TRUE);
		if (1 != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetByIdRaw.Error: error - %i\n", swResult);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		///	With device-0 (BC) verify that message-0 contains the expected data.
		for (int i=0; i<wWordCount; i++)
		{
			if ((wWordCount - 1 - i) != waRawMessage[sitalBcMaximum_ACTUAL_MESSAGE_BLOCK + i])
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.ReceivedData.Error: error - %i\n", waBuffer[i]);
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}
		}

		/// With device-0 (BC) get message-1.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetByIdRaw: unit - %i, message - %i, purge - %i\n", swDevice_0, swMessageId_1, TRUE);
		swResult = sitalBc_Message_GetByIdRaw (swDevice_0, swMessageId_1, waRawMessage, TRUE);
		if (1 != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetByIdRaw.Error: error - %i\n", swResult);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		///	With device-0 (BC) verify that message-1 contains the expected data.
		for (int i=0; i<wWordCount; i++)
		{
			if (i != waRawMessage[sitalBcMaximum_ACTUAL_MESSAGE_BLOCK + i])
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.TransmittedData.Error: error - %i\n", waBuffer[i]);
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}
		}

		/// With device-0 (BC) read from data-block-0 WordCount data words.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Read: unit - %i, block - %i, buf_size - %i, offset - %i\n", swDevice_0, swDataBlockId_0, wWordCount, 0);
		swResult = sitalBc_DataBlock_Read (swDevice_0, swDataBlockId_0, waBuffer, wWordCount, 0);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Read.Error: error - %i\n", swResult);
			throw 0;
		}

		///	With device-0 (BC) verify that data-block-0 contains the expected data.
		for (int i=0; i<wWordCount; i++)
		{
			if ((wWordCount - 1 - i) != waBuffer[i])
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.ReceivedData.Error: error - %i\n", waBuffer[i]);
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}
		}

		/// With device-0 (BC) read from data-block-1 WordCount data words.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Read: unit - %i, block - %i, buf_size - %i, offset - %i\n", swDevice_0, swDataBlockId_1, wWordCount, 0);
		swResult = sitalBc_DataBlock_Read (swDevice_0, swDataBlockId_1, waBuffer, wWordCount, 0);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Read.Error: error - %i\n", swResult);
			throw 0;
		}

		///	With device-0 (BC) verify that data-block-1 contains the expected data.
		for (int i=0; i<wWordCount; i++)
		{
			if (i != waBuffer[i])
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.TransmittedData.Error: error - %i\n", waBuffer[i]);
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}
		}

		/// With device-1 (RT) verify there are two messages in stack now.
		U16BIT wRtCommandStackPointer; // The value of the RT's command stack pointer.
		wRtCommandStackPointer = sitalDevice_Memory_Read (swDevice_1, 0x0100);
		sitalProcess_Log_PrintLine ("#StldTester.Device.MemoryWord.Read: unit - %i, address - %i, initial_value - 0400, current_value - %04X\n", swDevice_0, 0x0100, wRtCommandStackPointer);
		if (0x408 != wRtCommandStackPointer)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.MemoryWord.Read.Error: error - %i\n", wRtCommandStackPointer);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// With device-1 (RT) get message-0.
		sitalDecodedMessageStructure dmsDecodedMessage; // A decoded message.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Message.GetFromStackDecoded: unit - %i, message - %i, purge - %i\n", swDevice_1, swMessageId_0, TRUE);
		swResult = sitalRt_Message_GetFromStackDecoded (swDevice_1, &dmsDecodedMessage, sitalMessageLocationAndRemoval_NEXT_PURGE);
		if (1 != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Message.GetDecoded.Error: error - %i\n", swResult);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		///	With device-1 (RT) verify that message-0 contains the expected data.
		for (int i=0; i<wWordCount; i++)
		{
			if ((wWordCount - 1 - i) != dmsDecodedMessage.waData[i])
			{
				sitalProcess_Log_PrintLine ("#StldTester.Rt.TransmittedData.Error: error - %i\n", waBuffer[i]);
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}
		}

		/// With device-1 (RT) get message-1.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Message.GetFromStackDecoded: unit - %i, message - %i, purge - %i\n", swDevice_0, swMessageId_1, TRUE);
		swResult = sitalRt_Message_GetFromStackDecoded (swDevice_1, &dmsDecodedMessage, sitalMessageLocationAndRemoval_NEXT_PURGE);
		if (1 != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Message.GetDecoded.Error: error - %i\n", swResult);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		///	With device-1 (RT) verify that message-1 contains the expected data.
		for (int i=0; i<wWordCount; i++)
		{
			if (i != dmsDecodedMessage.waData[i])
			{
				sitalProcess_Log_PrintLine ("#StldTester.Rt.ReceivedData.Error: error - %i\n", waBuffer[i]);
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}
		}

		/// With device-1 (RT) read from data-block-0 WordCount data words.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.Read: unit - %i, block - %i, buf_size - %i, offset - %i\n", swDevice_1, swDataBlockId_0, wWordCount, 0);
		swResult = sitalRt_DataBlock_Read (swDevice_1, swDataBlockId_0, waBuffer, wWordCount, 0);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.Read.Error: error - %i\n", swResult);
			throw 0;
		}

		///	With device-1 (RT) verify that data-block-0 contains the expected data.
		for (int i=0; i<wWordCount; i++)
		{
			if (i != waBuffer[i])
			{
				sitalProcess_Log_PrintLine ("#StldTester.Rt.ReceivedData.Error: error - %i\n", waBuffer[i]);
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}
		}

		/// With device-1 (RT) read from data-block-1 WordCount data words.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.Read: unit - %i, block - %i, buf_size - %i, offset - %i\n", swDevice_1, swDataBlockId_1, wWordCount, 0);
		swResult = sitalRt_DataBlock_Read (swDevice_1, swDataBlockId_1, waBuffer, wWordCount, 0);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.Read.Error: error - %i\n", swResult);
			throw 0;
		}

		///	With device-1 (RT) verify that data-block-1 contains the expected data.
		for (int i=0; i<wWordCount; i++)
		{
			if ((wWordCount - 1- i) != waBuffer[i])
			{
				sitalProcess_Log_PrintLine ("#StldTester.Rt.TransmittedData.Error: error - %i\n", waBuffer[i]);
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}
		}

		/// Stop device-0 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop: unit - %i\n", swDevice_0);
		swResult = sitalBc_Stop (swDevice_0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Stop device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Stop: unit - %i\n", swDevice_1);
		swResult = sitalRt_Stop (swDevice_1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-0 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", swDevice_0);
		swResult = sitalDevice_Free (swDevice_0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", swDevice_1);
		swResult = sitalDevice_Free (swDevice_1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}
	}
	catch (int)
	{
		sitalDevice_Free (swDevice_0);
		sitalDevice_Free (swDevice_1);
	}

	sitalProcess_Log_PrintLine ("#StldTester.BcRt_DataRxTx.End: test_succeeded - %c\n", ((sitalReturnCode_SUCCESS == swResult) ? 'Y' : 'N'));
	return swResult;
}



/// <summary>
/// Test the stld1553 library in the henceforth described scenario:
/// Participant[s]: BC.
/// Configuration: -
/// Short description: Test the operation of the high-priority asynchronous messages system.
/// Along test print suitable info and indications into the log.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Bc_HighPriorityAsynchronousMessages ()
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	sitalProcess_Log_PrintLine ("#StldTester.Bc_HighPriorityAsynchronousMessages.Start\n");

	/// Set test arguments.
	U16BIT wRtAddress; // A RT address (0-31).
	wRtAddress = 30;
	U16BIT wRtSubaddressRx; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31) that's set for a RT for Rx.
	wRtSubaddressRx = 1;
	U16BIT wWordCount; // Word count (1-32).
	wWordCount = 10;
	U16BIT wFrameCount; // The number of frames to run.
	wFrameCount = 5;
	S16BIT swDevice_0; // Logical number of device (0-31).
	swDevice_0 = 0;

	S16BIT swResult; // Result of operation or function call.
	swResult = sitalReturnCode_SUCCESS;

	/// Get device count.
	U16BIT wRequiredDeviceCount; // The number of devices required for this test.
	wRequiredDeviceCount = (1 + swDevice_0);
	U16BIT wAvailableDeviceCount; // The total number of devices available in the system.
	swResult = sitalDevice_GetCount (&wAvailableDeviceCount);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %i\n", swResult);
		return swResult;
	}
	sitalProcess_Log_PrintLine ("#StldTester.Device.Count: available - %i, required - %i\n", wAvailableDeviceCount, wRequiredDeviceCount);
	if (wAvailableDeviceCount < wRequiredDeviceCount)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %s\n", "not_enough_devices_installed");
		return sitalReturnCode_SUCCESS;
	}

	try
	{
		/// Initialize device-0 (BC) as a real BC.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-bc, alloc_mem-none\n", swDevice_0);
		swResult = sitalDevice_Initialize (swDevice_0, sitalAccess_CARD, sitalMode_BC, 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-0 (BC) as one that supports high priority asynchronous messages.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Init: unit - %i, options - %i\n", swDevice_0, sitalBcSetupOption_ASYNCHRONOUS_HIGH_PRIORITY_MODE);
		swResult = sitalBc_Initialize (swDevice_0, sitalBcSetupOption_ASYNCHRONOUS_HIGH_PRIORITY_MODE);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-0 (BC) data-block-0, a data block of 64 data words.
		S16BIT swDataBlockId_0; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId_0 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i\n", swDevice_0, swDataBlockId_0, 64, 0);
		swResult = sitalBc_DataBlock_Create (swDevice_0, swDataBlockId_0, sitalBcDataBlockSizeOption_DOUBLE, NULL, 0);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-0 (BC) message-0, a RT(RtAddress/RtSubAddressTx)-to-BC message that uses data-block-0 to receive WordCount data words. Message bus: bus-1.
		S16BIT swMessageId_0; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create: unit - %i, message - %i, block - %i, rx_rt - %i, rx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", swDevice_0, swMessageId_0, swDataBlockId_0, wRtAddress, wRtSubaddressRx, wWordCount, 0, s_wBus1);
		swResult = sitalBc_Message_CreateBcToRt (swDevice_0, swMessageId_0, swDataBlockId_0, wRtAddress, wRtSubaddressRx, wWordCount, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-0 (BC) command-0, a XEQ command that executes message-0.
		S16BIT swCommandId_0; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", swDevice_0, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0);
		swResult = sitalBc_Command_Create (swDevice_0, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-0 (BC) command-1, a subroutine call command that executes minor-frame-0. 
		S16BIT swCommandId_1; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_1 = 1;
		S16BIT swFrameId_0; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", swDevice_0, swCommandId_1, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, swFrameId_0, 0);
		swResult = sitalBc_Command_Create (swDevice_0, swCommandId_1, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, (U16BIT)swFrameId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-0 (BC) minor-frame-0, a frame that contains command-0 and command-1.
		S16BIT swaCommandIds[sitalBcCounter_COMMANDS]; // An array of command-IDs.
		swaCommandIds[0] = swCommandId_0;
		U16BIT wCommandIdCount; // The number of command IDs in the frame (1-sitalBcCounter_FRAME_COMMANDS).
		wCommandIdCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %i, flags - %i\n", swDevice_0, swFrameId_0, "minor", wCommandIdCount, swCommandId_0, 0, 0);
		swResult = sitalBc_Frame_Create (swDevice_0, swFrameId_0, sitalBcFrameType_MINOR, swaCommandIds, wCommandIdCount, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-0 (BC) major-frame-1, a frame that contains command-1 of device-0 (BC) and whose frame time is FrameTime us.
		S16BIT swFrameId_1; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_1 = 1;
		swaCommandIds[0] = swCommandId_1;
		wCommandIdCount = 1;
		U16BIT wFrameTime; // The time (in units of 100 us) that the frame will take to complete.
		wFrameTime = 1000; // 100ms.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", swDevice_0, swFrameId_1, "major", wCommandIdCount, swCommandId_1, (wFrameTime / 10), 0);
		swResult = sitalBc_Frame_Create (swDevice_0, swFrameId_1, sitalBcFrameType_MAJOR, swaCommandIds, wCommandIdCount, wFrameTime, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Start device-0 (BC) running major-frame-1 forever.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", swDevice_0, swFrameId_1, sitalBcFrameIterations_RUN_FOREVER);
		swResult = sitalBc_Start (swDevice_0, swFrameId_1, sitalBcFrameIterations_RUN_FOREVER);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-0 (BC) message-1, an asynchronous BC-to-RT(RtAddress/RtSubAddressRx) message that uses data-block-1, which is filled with ascending values, to transmit WordCount data words. Message bus: bus-1.
		S16BIT swMessageId_1; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_1 = 1;
		S16BIT swDataBlockId_1; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId_1 = 2;
		U16BIT waBuffer[64]; // A buffer to read-from/write-into along data-block write/read operations, respectively.
		for (int i=0; i<wWordCount; i++) waBuffer[i] = i;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.AsynchronousMessage.BcToRt.Create: unit - %i, message - %i, block - %i, rx_rt - %i, rx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", swDevice_0, swMessageId_1, swDataBlockId_1, wRtAddress, wRtSubaddressRx, wWordCount, 0, s_wBus1);
		swResult = sitalBc_AsynchronousMessage_CreateBcToRt (swDevice_0, swMessageId_1, swDataBlockId_1, wRtAddress, wRtSubaddressRx, wWordCount, 0, s_wBus1, waBuffer);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.AsynchronousMessage.BcToRt.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Request device-0 (BC) to transmit asynchronous message-1 at high priority.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.AsynchronousMessage.Tx.High: unit - %i, message - %i\n", swDevice_0, swMessageId_1);
		swResult = sitalBc_AsynchronousMessage_SendAtHighPriority (swDevice_0, swMessageId_1, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.AsynchronousMessage.Tx.High.Error: error - %i\n", swResult);
			throw 0;
		}

		sitalDecodedMessageStructure dmsDecodedMessage; // A decoded message.
		PrecisionTimer pt; // A precision timer.
		for (int iFrameNumber=1; iFrameNumber<=wFrameCount; iFrameNumber++)
		{
			/// Wait for FrameTime ms.
			sitalProcess_Log_PrintLine ("#StldTester.Wait: time - %ims\n", (wFrameTime / 10));
			sitalProcess_Log_PrintLine ("\tWAIT_FOR %i us\n", ((U32BIT)wFrameTime * 100));
			PrecisionTimer pt; // A precision timer.
			pt.AccuratelySleep (wFrameTime / 10);

			/// Get message-0 from device-0 (BC).
			///   *** Note: This message should be transmitted again and again with each frame!
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded: unit - %i, message - %i, purge-y\n", swDevice_0, swMessageId_0);
			swResult = sitalBc_Message_GetByIdDecoded (swDevice_0, swMessageId_0, &dmsDecodedMessage, TRUE);
			if (1 != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded.Error: error - %i\n", swResult);
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}

			/// Get message-1 from device-0 (BC).
			///   *** Note: This message should be transmitted only once!
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded: unit - %i, message - %i, purge-y\n", swDevice_0, swMessageId_1);
			swResult = sitalBc_Message_GetByIdDecoded (swDevice_0, swMessageId_1, &dmsDecodedMessage, TRUE);
			if (0U != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded.Error: error - %i\n", swResult);
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}
		}

		/// Stop device-0 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop: unit - %i\n", swDevice_0);
		swResult = sitalBc_Stop (swDevice_0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-0 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", swDevice_0);
		swResult = sitalDevice_Free (swDevice_0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}
	}
	catch (int)
	{
		sitalDevice_Free (swDevice_0);
	}

	sitalProcess_Log_PrintLine ("#StldTester.Bc_HighPriorityAsynchronousMessages.End: test_succeeded - %c\n", ((sitalReturnCode_SUCCESS == swResult) ? 'Y' : 'N'));
	return swResult;
}



/// <summary>
/// Test the stld1553 library in the henceforth described scenario:
/// Participant[s]: BC.
/// Configuration: -
/// Short description: Test the operation of the low-priority asynchronous messages system.
/// Along test print suitable info and indications into the log.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Bc_LowPriorityAsynchronousMessages ()
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	sitalProcess_Log_PrintLine ("#StldTester.Bc_LowPriorityAsynchronousMessages.Start\n");

	/// Set test arguments.
	U16BIT wRtAddress; // A RT address (0-31).
	wRtAddress = 30;
	U16BIT wRtSubaddressRx; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31) that's set for a RT for Rx.
	wRtSubaddressRx = 1;
	U16BIT wWordCount; // Word count (1-32).
	wWordCount = 10;
	U16BIT wFrameCount; // The number of frames to run.
	wFrameCount = 5;
	S16BIT swDevice_0; // Logical number of device (0-31).
	swDevice_0 = 0;

	S16BIT swResult; // Result of operation or function call.
	swResult = sitalReturnCode_SUCCESS;

	/// Get device count.
	U16BIT wRequiredDeviceCount; // The number of devices required for this test.
	wRequiredDeviceCount = (1 + swDevice_0);
	U16BIT wAvailableDeviceCount; // The total number of devices available in the system.
	swResult = sitalDevice_GetCount (&wAvailableDeviceCount);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %i\n", swResult);
		return swResult;
	}
	sitalProcess_Log_PrintLine ("#StldTester.Device.Count: available - %i, required - %i\n", wAvailableDeviceCount, wRequiredDeviceCount);
	if (wAvailableDeviceCount < wRequiredDeviceCount)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %s\n", "not_enough_devices_installed");
		return sitalReturnCode_SUCCESS;
	}

	try
	{
		/// Initialize device-0 (BC) as a real BC.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-bc, alloc_mem-none\n", swDevice_0);
		swResult = sitalDevice_Initialize (swDevice_0, sitalAccess_CARD, sitalMode_BC, 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-0 (BC) as one that supports low priority asynchronous messages.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Init: unit - %i, options - %i\n", swDevice_0, sitalBcSetupOption_ASYNCHRONOUS_LOW_PRIORITY_MODE);
		swResult = sitalBc_Initialize (swDevice_0, sitalBcSetupOption_ASYNCHRONOUS_LOW_PRIORITY_MODE);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-0 (BC) data-block-0, a data block of 64 data words.
		S16BIT swDataBlockId_0; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId_0 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i\n", swDevice_0, swDataBlockId_0, 64, 0);
		swResult = sitalBc_DataBlock_Create (swDevice_0, swDataBlockId_0, sitalBcDataBlockSizeOption_DOUBLE, NULL, 0);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-0 (BC) message-0, a RT(RtAddress/RtSubAddressTx)-to-BC message that uses data-block-0 to receive WordCount data words. Message bus: bus-1.
		S16BIT swMessageId_0; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create: unit - %i, message - %i, block - %i, rx_rt - %i, rx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", swDevice_0, swMessageId_0, swDataBlockId_0, wRtAddress, wRtSubaddressRx, wWordCount, 0, s_wBus1);
		swResult = sitalBc_Message_CreateBcToRt (swDevice_0, swMessageId_0, swDataBlockId_0, wRtAddress, wRtSubaddressRx, wWordCount, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-0 (BC) command-0, a XEQ command that executes message-0.
		S16BIT swCommandId_0; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", swDevice_0, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0);
		swResult = sitalBc_Command_Create (swDevice_0, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-0 (BC) command-1, a subroutine call command that executes minor-frame-0. 
		S16BIT swCommandId_1; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_1 = 1;
		S16BIT swFrameId_0; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", swDevice_0, swCommandId_1, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, swFrameId_0, 0);
		swResult = sitalBc_Command_Create (swDevice_0, swCommandId_1, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, (U16BIT)swFrameId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-0 (BC) minor-frame-0, a frame that contains command-0.
		S16BIT swaCommandIds[sitalBcCounter_COMMANDS]; // An array of command-IDs.
		swaCommandIds[0] = swCommandId_0;
		U16BIT wCommandIdCount; // The number of command IDs in the frame (1-sitalBcCounter_FRAME_COMMANDS).
		wCommandIdCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %i, flags - %i\n", swDevice_0, swFrameId_0, "minor", wCommandIdCount, swCommandId_0, 0, 0);
		swResult = sitalBc_Frame_Create (swDevice_0, swFrameId_0, sitalBcFrameType_MINOR, swaCommandIds, wCommandIdCount, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-0 (BC) major-frame-1, a frame that contains command-1 and whose frame time is FrameTime us.
		S16BIT swFrameId_1; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_1 = 1;
		swaCommandIds[0] = swCommandId_1;
		wCommandIdCount = 1;
		U16BIT wFrameTime; // The time (in units of 100 us) that the frame will take to complete.
		wFrameTime = 1000; // 100ms.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", swDevice_0, swFrameId_1, "major", wCommandIdCount, swCommandId_1, (wFrameTime / 10), 0);
		swResult = sitalBc_Frame_Create (swDevice_0, swFrameId_1, sitalBcFrameType_MAJOR, swaCommandIds, wCommandIdCount, wFrameTime, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Start device-0 (BC) running major-frame-1 forever.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", swDevice_0, swFrameId_1, sitalBcFrameIterations_RUN_FOREVER);
		swResult = sitalBc_Start (swDevice_0, swFrameId_1, sitalBcFrameIterations_RUN_FOREVER);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-0 (BC) message-1, an asynchronous BC-to-RT(RtAddress/RtSubAddressRx) message that uses data-block-1, which is filled with ascending values, to transmit WordCount data words. Message bus: bus-1.
		S16BIT swMessageId_1; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_1 = 1;
		S16BIT swDataBlockId_1; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId_1 = 2;
		U16BIT waBuffer[64]; // A buffer to read-from/write-into along data-block write/read operations, respectively.
		for (int i=0; i<wWordCount; i++) waBuffer[i] = i;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.AsynchronousMessage.BcToRt.Create: unit - %i, message - %i, block - %i, rx_rt - %i, rx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", swDevice_0, swMessageId_1, swDataBlockId_1, wRtAddress, wRtSubaddressRx, wWordCount, 0, s_wBus1);
		swResult = sitalBc_AsynchronousMessage_CreateBcToRt (swDevice_0, swMessageId_1, swDataBlockId_1, wRtAddress, wRtSubaddressRx, wWordCount, 0, s_wBus1, waBuffer);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.AsynchronousMessage.BcToRt.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		sitalDecodedMessageStructure dmsDecodedMessage; // A decoded message.
		U16BIT wRemainingMessageCount; // The count of asynchronous messages that still weren't assigned for Tx.
		wRemainingMessageCount = 1; // A mere initial value to force the first call to function sitalBc_AsynchronousMessage_SendAtLowPriority.
		PrecisionTimer pt; // A precision timer.
		for (int iFrameNumber=1; iFrameNumber<=wFrameCount; iFrameNumber++)
		{
			/// Wait for FrameTime ms.
			sitalProcess_Log_PrintLine ("#StldTester.Wait: time - %ims\n", (wFrameTime / 10));
			sitalProcess_Log_PrintLine ("\tWAIT_FOR %i us\n", ((U32BIT)wFrameTime * 100));
			pt.AccuratelySleep (wFrameTime / 10);

			/// Request device-0 (BC) to transmit asynchronous messages at low priority.
			///   Note: The following condition could be actually avoided, as function sitalBc_AsynchronousMessage_SendAtLowPriority will do nothing if no new
			///   asynchronous messages are created. This condition is nevertheless inserted in order to save the device i/o that's made by this function.
			if (0 < wRemainingMessageCount)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.AsynchronousMessage.Tx.Low: unit - %i\n", swDevice_0);
				swResult = sitalBc_AsynchronousMessage_SendAtLowPriority (swDevice_0, &wRemainingMessageCount, 0);
				if (0 > swResult)
				{
					sitalProcess_Log_PrintLine ("#StldTester.Bc.AsynchronousMessage.Tx.Low.Error: error - %i\n", swResult);
					throw 0;
				}
			}

			/// Get message-0 from device-0 (BC).
			///   *** Note: This message should be transmitted again and again with each frame!
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded: unit - %i, message - %i, purge-y\n", swDevice_0, swMessageId_0);
			swResult = sitalBc_Message_GetByIdDecoded (swDevice_0, swMessageId_0, &dmsDecodedMessage, TRUE);
			if (1 != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded.Error: error - %i\n", swResult);
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}

			/// Get message-1 from device-0 (BC).
			///   *** Note: This message should be transmitted only once!
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded: unit - %i, message - %i, purge-y\n", swDevice_0, swMessageId_1);
			swResult = sitalBc_Message_GetByIdDecoded (swDevice_0, swMessageId_1, &dmsDecodedMessage, TRUE);
			if (0U != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded.Error: error - %i\n", swResult);
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}
		}

		/// Stop device-0 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop: unit - %i\n", swDevice_0);
		swResult = sitalBc_Stop (swDevice_0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-0 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", swDevice_0);
		swResult = sitalDevice_Free (swDevice_0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}
	}
	catch (int)
	{
		sitalDevice_Free (swDevice_0);
	}

	sitalProcess_Log_PrintLine ("#StldTester.Bc_LowPriorityAsynchronousMessages.End: test_succeeded - %c\n", ((sitalReturnCode_SUCCESS == swResult) ? 'Y' : 'N'));
	return sitalReturnCode_SUCCESS;
}



/// <summary>
/// Test the stld1553 library in the henceforth described scenario:
/// Participant[s]: BC.
/// Configuration: -
/// Short description: Test the operation of the high-priority asynchronous messages system.
/// Along test print suitable info and indications into the log.
///
/// Note:
/// This test verifies its granular completion, i.e., that the operating system does not suspend its thread between the time tag set and the time tag get operations.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Bc_TimeTag ()
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	sitalProcess_Log_PrintLine ("#StldTester.Bc_TimeTag.Start\n");

	/// Set test arguments.
	S16BIT swDevice_0; // Logical number of device (0-31).
	swDevice_0 = 0;

	S16BIT swResult; // Result of operation or function call.
	swResult = sitalReturnCode_SUCCESS;

	/// Get device count.
	U16BIT wRequiredDeviceCount; // The number of devices required for this test.
	wRequiredDeviceCount = (1 + swDevice_0);
	U16BIT wAvailableDeviceCount; // The total number of devices available in the system.
	swResult = sitalDevice_GetCount (&wAvailableDeviceCount);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %i\n", swResult);
		return swResult;
	}
	sitalProcess_Log_PrintLine ("#StldTester.Device.Count: available - %i, required - %i\n", wAvailableDeviceCount, wRequiredDeviceCount);
	if (wAvailableDeviceCount < wRequiredDeviceCount)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %s\n", "not_enough_devices_installed");
		return sitalReturnCode_SUCCESS;
	}

	try
	{
		/// Initialize device-0 (BC) as a real BC with time tag control options.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-bc, alloc_mem-none\n", swDevice_0);
		swResult = sitalDevice_Initialize (swDevice_0, sitalAccess_CARD, (sitalMode_BC | sitalModeVariant_NO_TIME_TAG_RESET), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		///   (Retry the following sub-test until it is performed granularly or, otherwise, give up the test.)
		sitalProcess_Log_PrintLine ("#StldTester.Bc_TimeTag.SubTest00.Start\n");
		PrecisionTimer pt; // A precision timer.
		U16BIT wTestRetryCount; // The number of times that the relevant [sub-]test has been retried already.
		wTestRetryCount = 0;
		U16BIT wWrittenTimeTag; // Device time tag.
		U16BIT wReadTimeTag; // Device time tag.
		S32BIT sdwTimeTagDifference; // Device time tag difference.
		do 
		{
			///   Read the computer's current tick counter.
			pt.SetBaseTime ();

			/// Set for device-0 (BC) the time tag to 1111h.
			wWrittenTimeTag = 0x1111;
			sitalProcess_Log_PrintLine ("#StldTester.Bc.TimeTag.Set: unit - %i, value - %04X\n", swDevice_0, wWrittenTimeTag);
			swResult = sitalDevice_TimeTag_Set (swDevice_0, wWrittenTimeTag);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.TimeTag.Set.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Get for device-0 (BC) the current time tag, and verify that its value is 1111h or a bit higher.
			wReadTimeTag = 0;
			sitalProcess_Log_PrintLine ("#StldTester.Bc.TimeTag.Get: unit - %i\n", swDevice_0);
			swResult = sitalDevice_TimeTag_Get (swDevice_0, &wReadTimeTag);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.TimeTag.Get.Error: error - %i\n", swResult);
				throw 0;
			}

			///   Read the computer's current tick counter.
			pt.SetEndTime ();

			wTestRetryCount++;
		}
		while ((10 > wTestRetryCount) && (15 < pt.GetDuration ()));

		if (15 >= pt.GetDuration ())
		{
			sdwTimeTagDifference = ((S32BIT)wReadTimeTag - wWrittenTimeTag);
			sitalProcess_Log_PrintLine ("#StldTester.Bc.TimeTag.Get: unit - %i, value - %04X, difference - %i\n", swDevice_0, wReadTimeTag, sdwTimeTagDifference);
			if ((0 >= sdwTimeTagDifference) || (0x8000 <= sdwTimeTagDifference))
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.TimeTag.Promotion.Error: error - %i\n", sdwTimeTagDifference);
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}
		}
		else
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc_TimeTag.SubTest00.Warning: bad_duration - %f\n", pt.GetDuration ());
		}

		///   (Retry the following sub-test until it is performed granularly or, otherwise, give up the test.)
		sitalProcess_Log_PrintLine ("#StldTester.Bc_TimeTag.SubTest01.Start\n");
		wTestRetryCount = 0;
		do 
		{
			///   Read the computer's current tick counter.
			pt.SetBaseTime ();

			/// Set for device-0 (BC) the time tag to FFFFh.
			wWrittenTimeTag = 0xFFFF;
			sitalProcess_Log_PrintLine ("#StldTester.Bc.TimeTag.Set: unit - %i, value - %04X\n", swDevice_0, wWrittenTimeTag);
			swResult = sitalDevice_TimeTag_Set (swDevice_0, wWrittenTimeTag);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.TimeTag.Set.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Wait enough for the time tag to be cyclically promoted.
			sitalProcess_Log_PrintLine ("#StldTester.Wait: time - %ims\n", 1);
			sitalProcess_Log_PrintLine ("\tWAIT_FOR %i us\n", 1000);
			pt.AccuratelySleep (1);

			/// Get for device-0 (BC) the current time tag, and verify that its value is 0000h or a bit higher.
			wReadTimeTag = 0;
			sitalProcess_Log_PrintLine ("#StldTester.Bc.TimeTag.Get: unit - %i\n", swDevice_0);
			swResult = sitalDevice_TimeTag_Get (swDevice_0, &wReadTimeTag);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.TimeTag.Get.Error: error - %i\n", swResult);
				throw 0;
			}

			///   Read the computer's current tick counter.
			pt.SetEndTime ();

			wTestRetryCount++;
		}
		while ((10 > wTestRetryCount) && (15 < pt.GetDuration ()));

		if (15 >= pt.GetDuration ())
		{
			sdwTimeTagDifference = ((S32BIT)wWrittenTimeTag - wReadTimeTag);
			sitalProcess_Log_PrintLine ("#StldTester.Bc.TimeTag.Get: unit - %i, value - %04X, difference - %i\n", swDevice_0, wReadTimeTag, sdwTimeTagDifference);
			if (0x8000 >= sdwTimeTagDifference)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.TimeTag.Promotion.Error: error - %i\n", sdwTimeTagDifference);
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}
		}
		else
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc_TimeTag.SubTest01.Warning: bad_duration - %f\n", pt.GetDuration ());
		}

		///   (Retry the following sub-test until it is performed granularly or, otherwise, give up the test.)
		sitalProcess_Log_PrintLine ("#StldTester.Bc_TimeTag.SubTest02.Start\n");
		wTestRetryCount = 0;
		do 
		{
			///   Read the computer's current tick counter.
			pt.SetBaseTime ();

			/// Set for device-0 (BC) the time tag to 7FFFh.
			wWrittenTimeTag = 0x7FFF;
			sitalProcess_Log_PrintLine ("#StldTester.Bc.TimeTag.Set: unit - %i, value - %04X\n", swDevice_0, wWrittenTimeTag);
			swResult = sitalDevice_TimeTag_Set (swDevice_0, wWrittenTimeTag);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.TimeTag.Set.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Reset for device-0 (BC) the time tag.
			sitalProcess_Log_PrintLine ("#StldTester.Bc.TimeTag.Reset: unit - %i\n", swDevice_0);
			swResult = sitalDevice_TimeTag_Reset (swDevice_0);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.TimeTag.Reset.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Get for device-0 (BC) the current time tag, and verify that its value is 0000h or a bit higher.
			wReadTimeTag = 0;
			sitalProcess_Log_PrintLine ("#StldTester.Bc.TimeTag.Get: unit - %i\n", swDevice_0);
			swResult = sitalDevice_TimeTag_Get (swDevice_0, &wReadTimeTag);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.TimeTag.Get.Error: error - %i\n", swResult);
				throw 0;
			}

			///   Read the computer's current tick counter.
			pt.SetEndTime ();

			wTestRetryCount++;
		}
		while ((10 > wTestRetryCount) && (15 < pt.GetDuration ()));

		if (15 >= pt.GetDuration ())
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.TimeTag.Get: unit - %i, value - %04X, difference - %i\n", swDevice_0, wReadTimeTag, wReadTimeTag);
			if (0x8000 <= wReadTimeTag)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.TimeTag.Promotion.Error: error - %i\n", wReadTimeTag);
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}
		}
		else
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc_TimeTag.SubTest02.Warning: bad_duration - %f\n", pt.GetDuration ());
		}

		/// Repeat the following steps for each of the available time tag resolutions:
		sitalProcess_Log_PrintLine ("#StldTester.Bc_TimeTag.SubTest03.Start\n");
		U16BIT waTimeTagResolution[] // An array that contains the value into which the time tag resolution configuration bits are set per time tag resolution.
				= { sitalTimeTagResolution_2US, sitalTimeTagResolution_4US, sitalTimeTagResolution_8US, sitalTimeTagResolution_16US, sitalTimeTagResolution_32US, sitalTimeTagResolution_64US };
		U16BIT wTimeTagResolutionCount; // The count of tested time tag resolutions.
		wTimeTagResolutionCount = (sizeof(waTimeTagResolution) / sizeof(waTimeTagResolution[0]));
		double dSleepDuration; // The calculated/actual duration of sleep period.
		U16BIT wMinimumExpectedTimeTag; // The minimum expected new time tag.
		PrecisionTimer ptSleepMeasurer; // A precision timer.
		for (int i=0; i<wTimeTagResolutionCount; i++)
		{
			///   Set for device-0 (BC) the time tag n to the current one.
			sitalProcess_Log_PrintLine ("#StldTester.Bc.TimeTag.SetResolution: unit - %i, value - %i\n", swDevice_0, waTimeTagResolution[i]);
			swResult = sitalDevice_TimeTag_SetResolution (swDevice_0, waTimeTagResolution[i]);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.TimeTag.SetResolution.Error: error - %i\n", swResult);
				throw 0;
			}

			///   (Retry the following sub-test until it is performed granularly or, otherwise, give up the test.)
			wTestRetryCount = 0;
			wReadTimeTag = 0;
			do 
			{
				///   Read the computer's current tick counter.
				pt.SetBaseTime ();

				///   Reset for device-0 (BC) the time tag.
				sitalProcess_Log_PrintLine ("#StldTester.Bc.TimeTag.Reset: unit - %i\n", swDevice_0);
				swResult = sitalDevice_TimeTag_Reset (swDevice_0);
				if (sitalReturnCode_SUCCESS != swResult)
				{
					sitalProcess_Log_PrintLine ("#StldTester.Bc.TimeTag.Reset.Error: error - %i\n", swResult);
					throw 0;
				}

				///   Read the computer's current tick counter.
				ptSleepMeasurer.SetBaseTime ();

				///   Sleep for some period.
				sitalProcess_Log_PrintLine ("#StldTester.Wait: time - %ims\n", 1);
				sitalProcess_Log_PrintLine ("\tWAIT_FOR %i us\n", 1000);
				ptSleepMeasurer.AccuratelySleep (1);

				///   Read the computer's current tick counter.
				ptSleepMeasurer.SetEndTime ();

				///   Get for device-0 (BC) the current time tag, and verify that its value is the minimum expected one or a bit higher.
				sitalProcess_Log_PrintLine ("#StldTester.Bc.TimeTag.Get: unit - %i\n", swDevice_0);
				swResult = sitalDevice_TimeTag_Get (swDevice_0, &wReadTimeTag);
				if (sitalReturnCode_SUCCESS != swResult)
				{
					sitalProcess_Log_PrintLine ("#StldTester.Bc.TimeTag.Get.Error: error - %i\n", swResult);
					throw 0;
				}

				///   Read the computer's current tick counter.
				pt.SetEndTime ();

				wTestRetryCount++;
			}
			while ((10 > wTestRetryCount) && (15 < pt.GetDuration ()));

			double daTimeTagResolution[] // An array that contains the time tag resolutions (milliseconds).
					= { 0.002, 0.004, 0.008, 0.016, 0.032, 0.064 };
			if (15 >= pt.GetDuration ())
			{
				///   Calculate the exact sleeping duration.
				dSleepDuration = ptSleepMeasurer.GetDuration ();

				///   Calculate the minimum expected new time tag using the current time tag resolution.
				wMinimumExpectedTimeTag = (U16BIT)(dSleepDuration / daTimeTagResolution[i]);

				sitalProcess_Log_PrintLine ("#StldTester.Bc.TimeTag.Get: unit - %i, value - %04X, min_expected - %04X\n", swDevice_0, wReadTimeTag, wMinimumExpectedTimeTag);
				if (wMinimumExpectedTimeTag > wReadTimeTag)
				{
					sitalProcess_Log_PrintLine ("#StldTester.Bc.TimeTag.Promotion.Error: error - %i\n", wMinimumExpectedTimeTag);
					swResult = sitalReturnCode_TEST_FAIL;
					throw 0;
				}
			}
			else
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc_TimeTag.SubTest03/resolution%02i.Warning: %i, bad_duration - %f\n", waTimeTagResolution[i], pt.GetDuration ());
			}
		}

		/// Free device-0 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", swDevice_0);
		swResult = sitalDevice_Free (swDevice_0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}
	}
	catch (int)
	{
		sitalDevice_Free (swDevice_0);
	}

	sitalProcess_Log_PrintLine ("#StldTester.Bc_TimeTag.End: test_succeeded - %c\n", ((sitalReturnCode_SUCCESS == swResult) ? 'Y' : 'N'));
	return swResult;
}



/// <summary>
/// Test the stld1553 library in the henceforth described scenario:
/// Participant[s]: BC0, RT1[, RT2[...]] (RtCount>0).
/// Configuration:
/// - The single BC is defined on a separate card as device 0.
/// - All participating RTs must be numbered as devices 1 to N.
/// - All participating devices must be able to communicate with each other.
/// Short description: Test the BC - multi-RT bi-directional Rx/Tx of data with more than one card.
/// Along test print suitable info and indications into the log.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_BcMultiRt_DataRxTx ()
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	sitalProcess_Log_PrintLine ("#StldTester.BcMultiRt_DataRxTx.Start\n");

	/// Set test arguments.
	BOOLEAN bStopOnDataError; // A flag that says whether should stop in case a non-crucial error (e.g., bad/no data Rx) is detected.
	bStopOnDataError = TRUE;
	BOOLEAN bOperationalRtDevices; // A flag that says whether enough card units are available for the RT devices (otherwise, no RT is initialized and no RT is expected to respond the BC device).
	bOperationalRtDevices = true;
	U16BIT wRtCount; // The count of participating RT devices, which is the number of BC-to-RT messages being created (BOTH <=32 *AND* <(sitalMaximum_DEVICES - 1)).
	wRtCount = 7;
	U32BIT dwIterations; // The number of test iterations (>0).
	dwIterations = 1000;
	U32BIT dwMaximumFrameRunTime; // The maximum overall frame run time (in ms).
	dwMaximumFrameRunTime = 3000; // 3 seconds.
	U16BIT wRtSubaddress; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31) that's set for a RT for Rx, Tx, and broadcast.
	wRtSubaddress = 30;
	S16BIT swDevice_0; // Logical number of device (0-31).
	swDevice_0 = 0;
	S16BIT swResult; // Result of operation or function call.

	swResult = sitalReturnCode_SUCCESS;
	S16BIT swDeviceId; // Logical number of device (0-31).

	/// Get device count.
	U16BIT wRequiredDeviceCount; // The number of devices required for this test.
	wRequiredDeviceCount = (1 + wRtCount);
	U16BIT wAvailableDeviceCount; // The total number of devices available in the system.
	swResult = sitalDevice_GetCount (&wAvailableDeviceCount);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %i\n", swResult);
		return swResult;
	}
	sitalProcess_Log_PrintLine ("#StldTester.Device.Count: available - %i, required - %i\n", wAvailableDeviceCount, wRequiredDeviceCount);
	if (wAvailableDeviceCount < wRequiredDeviceCount)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %s\n", "not_enough_devices_installed");
		return sitalReturnCode_SUCCESS;
	}

	try
	{
		/// Initialize device-0 (BC) as a real BC.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-bc, alloc_mem-none\n", swDevice_0);
		swResult = sitalDevice_Initialize (swDevice_0, sitalAccess_CARD, (sitalMode_BC | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		///	Set the response timeout for device-0 (BC) to an appropriate one.
		sitalProcess_Log_PrintLine ("#StldTester.Device.ResponseTimeout.Set: unit - %i, timeout - %i\n", swDevice_0, sitalResponseTimeout_50US);
		swResult = sitalDevice_SetResponseTimeout (swDevice_0, sitalResponseTimeout_50US);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.ResponseTimeout.Set.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Loop over all RT devices (RTs).
		S16BIT swDataBlockId_0; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId_0 = 1;
		U16BIT wRtAddress; // A RT address (0-31).
		U16BIT wWordCount; // Word count (1-32).
		S16BIT swDataBlockId; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		S16BIT swMessageId; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		S16BIT swCommandId; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		S16BIT swFrameCommandIndex; // A unique index designating a command in its frame (0-(sitalBcCounter_FRAME_COMMANDS-1)).
		S16BIT swaBcToRtCommandIds[sitalMaximum_DEVICES]; // An array of command-IDs.
		S16BIT swaRtToBcCommandIds[sitalMaximum_DEVICES]; // An array of command-IDs.
		for (S16BIT i=1; i<=wRtCount; i++)
		{
			///		Create for device-0 (BC) data-block-i to use with device-i (RT), a data block of 32 data words.
			swDataBlockId = i;
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i\n", swDevice_0, swDataBlockId_0, 32, 0);
			swResult = sitalBc_DataBlock_Create (swDevice_0, swDataBlockId, sitalBcDataBlockSizeOption_SINGLE, NULL, 0);
			if (0 > swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
				throw 0;
			}

			///		Create for device-0 (BC) message-*, a BC-to-RT(device-i-RtAddress/RtSubAddress) message that uses device-i's corresponding data-block to transmit a RT-unique number of WordCount data words. Message bus: bus-1.
			swDeviceId = i; // device-1, device-2, device-3, ...
			swMessageId = ( ((i-1) * 2) + 0 ); // message-0 for device-1, message-2 for device-2, message-4 for device-3, ...
			wRtAddress = i; // RT-address-1 for device-1, RT-address-2 for device-2, RT-address-3 for device-3, ...
			wWordCount = (33 - i); // 32 words for device-1, 31 words for device-2, 30 words for device-3, ...
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create: unit - %i, message - %i, block - %i, rx_rt - %i, rx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", swDevice_0, swMessageId, swDataBlockId, wRtAddress, wRtSubaddress, wWordCount, 0, s_wBus1);
			swResult = sitalBc_Message_CreateBcToRt (swDevice_0, swMessageId, swDataBlockId, wRtAddress, wRtSubaddress, wWordCount, 0, s_wBus1);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create.Error: error - %i\n", swResult);
				throw 0;
			}

			///		Create for device-0 (BC) command-*, a XEQ command that executes the message-*.
			swFrameCommandIndex = (i - 1); // command#0 for for device-1, command#1 for device-2, command#2 for device-3, ...
			swCommandId = swMessageId; // command-0 for for device-1, command-2 for device-2, command-4 for device-3, ...
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", swDevice_0, swCommandId, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId, 0);
			swResult = sitalBc_Command_Create (swDevice_0, swCommandId, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId, 0, 0);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
				throw 0;
			}
			swaBcToRtCommandIds[swFrameCommandIndex] = swCommandId;

			///		Create for device-0 (BC) message-*, a RT(device-i-RtAddress/RtSubAddress)-to-BC message that uses device-i's corresponding data-block to receive a RT-unique number of WordCount data words. Message bus: bus-1.
			swMessageId = ( ((i-1) * 2) + 1 ); // message-1 for device-1, message-3 for device-2, message-5 for device-3, ...
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBc.Create: unit - %i, message - %i, block - %i, tx_rt - %i, tx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", swDevice_0, swMessageId, swDataBlockId, wRtAddress, wRtSubaddress, wWordCount, 0, s_wBus1);
			swResult = sitalBc_Message_CreateRtToBc (swDevice_0, swMessageId, swDataBlockId, wRtAddress, wRtSubaddress, wWordCount, 0, s_wBus1);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBc.Create.Error: error - %i\n", swResult);
				throw 0;
			}

			///		Create for device-0 (BC) command-*, a XEQ command that executes the message-*.
			swCommandId = swMessageId; // command-1 for for device-1, command-3 for device-2, command-5 for device-3, ...
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", swDevice_0, swCommandId, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId, 0);
			swResult = sitalBc_Command_Create (swDevice_0, swCommandId, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId, 0, 0);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
				throw 0;
			}
			swaRtToBcCommandIds[swFrameCommandIndex] = swCommandId;

			///		Initialize device-i (RT) as a real RT.
			if (TRUE == bOperationalRtDevices)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-rt, alloc_mem-none\n", swDeviceId);
				swResult = sitalDevice_Initialize (swDeviceId, sitalAccess_CARD, ((s_bIsRtMtModeTest ? sitalMode_RT_AND_MT : sitalMode_RT) | sitalModeVariant_ADVANCED), 0, 0, 0);
				if (sitalReturnCode_SUCCESS != swResult)
				{
					sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
					throw 0;
				}

				///		Set the response timeout for device-i (RT) to an appropriate one.
				sitalProcess_Log_PrintLine ("#StldTester.Device.ResponseTimeout.Set: unit - %i, timeout - %i\n", swDeviceId, sitalResponseTimeout_50US);
				swResult = sitalDevice_SetResponseTimeout (swDeviceId, sitalResponseTimeout_50US);
				if (sitalReturnCode_SUCCESS != swResult)
				{
					sitalProcess_Log_PrintLine ("#StldTester.Device.ResponseTimeout.Set.Error: error - %i\n", swResult);
					throw 0;
				}

				///		Set device-i (RT) address to be same as its device-ID.
				sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set: unit - %i, address - %i\n", swDeviceId, swDeviceId);
				swResult = sitalRt_Address_Set (swDeviceId, swDeviceId);
				if (sitalReturnCode_SUCCESS != swResult)
				{
					sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set.Error: error - %i\n", swResult);
					throw 0;
				}

				///		Create for device-i (RT) data-block-0, a data block of 32 data words.
				sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i\n", swDeviceId, swDataBlockId_0, 32, 0);
				swResult = sitalRt_DataBlock_Create (swDeviceId, swDataBlockId_0, sitalRtDataBlockType_SINGLE_32, NULL, 0);
				if (sitalReturnCode_SUCCESS != swResult)
				{
					sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.Create.Error: error - %i\n", swResult);
					throw 0;
				}

				///		Map for device-i (RT) data-block-0 with Rx/Tx/Broadcast subaddress RtSubAddressRx, and legalize this subaddress.
				sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.MapToSa: unit - %i, block - %i, sa - %i, msg_types - x%X, irq_opts-x%X, legalize - %c\n", swDeviceId, swDataBlockId_0, wRtSubaddress, sitalRtMessageType_ALL, sitalRtDataBlockIrq_NONE, 'Y');
				swResult = sitalRt_DataBlock_MapToSubaddress (swDeviceId, swDataBlockId_0, wRtSubaddress, sitalRtMessageType_ALL, sitalRtDataBlockIrq_NONE, TRUE);
				if (sitalReturnCode_SUCCESS != swResult)
				{
					sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.MapToSa.Error: error - %i\n", swResult);
					throw 0;
				}

				///		Start device-i (RT).
				sitalProcess_Log_PrintLine ("#StldTester.Rt.Start: unit - %i\n", swDeviceId);
				swResult = sitalRt_Start (swDeviceId);
				if (sitalReturnCode_SUCCESS != swResult)
				{
					sitalProcess_Log_PrintLine ("#StldTester.Rt.Start.Error: error - %i\n", swResult);
					throw 0;
				}
			}
		}

		/// Create for device-0 (BC) minor-frame-0, a frame that contains all the commands that execute BC-to-RT messages.
		S16BIT swFrameId_0; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_0 = 0;
		U16BIT wCommandIdCount; // The number of command IDs in the frame (1-sitalBcCounter_FRAME_COMMANDS).
		wCommandIdCount = wRtCount;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i[, ...]), time - %i, flags - %i\n", swDevice_0, swFrameId_0, "minor", wCommandIdCount, swaBcToRtCommandIds[0], 0, 0);
		swResult = sitalBc_Frame_Create (swDevice_0, swFrameId_0, sitalBcFrameType_MINOR, swaBcToRtCommandIds, wCommandIdCount, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-0 (BC) minor-frame-1, a frame that contains all the commands that execute RT-to-BC messages.
		S16BIT swFrameId_1; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_1 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i[, ...]), time - %i, flags - %i\n", swDevice_0, swFrameId_1, "minor", wCommandIdCount, swaRtToBcCommandIds[0], 0, 0);
		swResult = sitalBc_Frame_Create (swDevice_0, swFrameId_1, sitalBcFrameType_MINOR, swaRtToBcCommandIds, wCommandIdCount, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-0 (BC) command-*, a subroutine call command that executes minor-frame-0. 
		S16BIT swCommandId_2; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_2 = (2 * wRtCount);
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", swDevice_0, swCommandId_2, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, swFrameId_0, 0);
		swResult = sitalBc_Command_Create (swDevice_0, swCommandId_2, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, (U16BIT)swFrameId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-0 (BC) major-frame-2, a frame that contains a command that performs frame-0 of device-0 (BC) and whose frame time is FrameTime us.
		S16BIT swFrameId_2; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_2 = 2;
		wCommandIdCount = 1;
		U16BIT wFrameTime; // The time (in units of 100 us) that the frame will take to complete.
		wFrameTime = 0; // Back-to-back.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", swDevice_0, swFrameId_2, "major", wCommandIdCount, swCommandId_2, (wFrameTime / 10), 0);
		swResult = sitalBc_Frame_Create (swDevice_0, swFrameId_2, sitalBcFrameType_MAJOR, &swCommandId_2, wCommandIdCount, wFrameTime, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-0 (BC) command-*, a subroutine call command that executes minor-frame-1. 
		S16BIT swCommandId_3; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_3 = (swCommandId_2 + 1);
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", swDevice_0, swCommandId_3, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, swFrameId_1, 0);
		swResult = sitalBc_Command_Create (swDevice_0, swCommandId_3, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, (U16BIT)swFrameId_1, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-0 (BC) major-frame-3, a frame that contains a command that performs frame-1 of device-0 (BC) and whose frame time is FrameTime us.
		S16BIT swFrameId_3; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_3 = 3;
		wCommandIdCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", swDevice_0, swFrameId_3, "major", wCommandIdCount, swCommandId_3, (wFrameTime / 10), 0);
		swResult = sitalBc_Frame_Create (swDevice_0, swFrameId_3, sitalBcFrameType_MAJOR, &swCommandId_3, wCommandIdCount, wFrameTime, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Configure device-0 (BC) to get IRQ-3 notifications using an ISR that collects interrupt data, counts ISR calls, and signals interested threads.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Manipulate: unit - %i, enable - %i, irq - %08X, callback - %s\n", swDevice_0, TRUE, sitalInterruptRegister2_BC_IRQ3, "stldTester_Bc_UserIsr");
		swResult = sitalDevice_Irq_Manipulate (swDevice_0, TRUE, sitalInterruptRegister2_BC_IRQ3, stldTester_Bc_UserIsr);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Set.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Loop to perform the required number of test iterations.
		U16BIT waRtNextIncrementedDataWord[sitalMaximum_DEVICES]; // An array containing an entry per RT device, within which the next incremented data word to transmit to this RT device is recorded.
		memset (waRtNextIncrementedDataWord, 0, sizeof(waRtNextIncrementedDataWord));
		U32BIT dwMinorFrameCounter; // The overall number of transmitted minor frames.
		PrecisionTimer pt; // A precision timer.
		U32BIT dwOverallFrameRunTime; // The expected overall frame run time (in ms).
		U32BIT dwSleepTime; // Sleeping period (in ms).
		U16BIT* wpCurrentRtNextIncrementedDataWord; // A pointer to the entry for current RT in array waRtNextIncrementedDataWord.
		U16BIT waBuffer[32]; // A buffer to read-from/write-into along data-block write/read operations, respectively.
		for (U32BIT uiIteration=0; uiIteration<dwIterations; uiIteration++)
		{
			printf ("#StldTester.TestIteration.Start: iteration - %i\n", uiIteration);
			sitalProcess_Log_PrintLine ("#StldTester.TestIteration.Start: iteration - %i\n", uiIteration);

			///		Loop over all RT devices (RTs).
			wpCurrentRtNextIncrementedDataWord = waRtNextIncrementedDataWord;
			for (S16BIT i=1; i<=wRtCount; i++)
			{
				///			Write into device-0 (BC) data-block-i the suitable number of incremental data words used with device-i (RT).
				swDataBlockId = i;
				wWordCount = (33 - i); // 32 words for device-1, 31 words for device-2, 30 words for device-3, ...
				wpCurrentRtNextIncrementedDataWord++;
				for (int i=0; i<wWordCount; i++)
				{
					waBuffer[i] = (*wpCurrentRtNextIncrementedDataWord);
					(*wpCurrentRtNextIncrementedDataWord)++; // Cyclic incrementation: 0xFFFF is incremented to 0x0000.
				}
				sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Write: unit - %i, block - %i, size - %i, buf_size - %i (%04X...)\n", swDevice_0, swDataBlockId, 32, wWordCount, waBuffer[0]);
				swResult = sitalBc_DataBlock_Write (swDevice_0, swDataBlockId, waBuffer, wWordCount, 0);
				if (0 > swResult)
				{
					sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Write.Error: error - %i\n", swResult);
					throw 0;
				}
			}

			///		Start device-0 (BC) running major-frame-2 for 1-shot.
			dwMinorFrameCounter = s_dwMinorFrameCounter;
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", swDevice_0, swFrameId_2, 1);
			swResult = sitalBc_Start (swDevice_0, swFrameId_2, 1);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
				throw 0;
			}

			///		Keep waiting until the global ISR frame counter shows that the frame has ended OR the overall frame run timer expires (the earlier):
			dwSleepTime = 100; // 0.1 second.
			dwOverallFrameRunTime = 0;
			while ((dwMinorFrameCounter == s_dwMinorFrameCounter) && (dwMaximumFrameRunTime > dwOverallFrameRunTime))
			{
				///			Wait for a while.
				sitalProcess_Log_PrintLine ("#StldTester.Wait: time_passed - %ims, isr_calls - %i\n", dwOverallFrameRunTime, s_dwMinorFrameCounter);
				sitalProcess_Log_PrintLine ("\tWAIT_FOR %i us\n", (dwSleepTime * 1000));
				pt.AccuratelySleep (dwSleepTime);

				///			Update the overall frame run timer.
				dwOverallFrameRunTime += dwSleepTime;
			}
			sitalProcess_Log_PrintLine ("#StldTester.Wait.End: unit - %i, isr_invoked - %i\n", swDevice_0, (dwMinorFrameCounter != s_dwMinorFrameCounter));

			/// Stop device-0 (BC).
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop: unit - %i\n", swDevice_0);
			swResult = sitalBc_Stop (swDevice_0);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
				throw 0;
			}

			///		Loop over all RT devices (RTs).
			for (S16BIT i=1; i<=wRtCount; i++)
			{
				///			Zero in device-0 (BC) data-block-i the suitable number of data words as used with device-i (RT).
				swDataBlockId = i;
				wWordCount = (33 - i); // 32 words for device-1, 31 words for device-2, 30 words for device-3, ...
				for (int i=0; i<wWordCount; i++)
				{
					waBuffer[i] = 0;
				}
				sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Zero: unit - %i, block - %i, size - %i, buf_size - %i\n", swDevice_0, swDataBlockId, 32, wWordCount);
				swResult = sitalBc_DataBlock_Write (swDevice_0, swDataBlockId, waBuffer, wWordCount, 0);
				if (0 > swResult)
				{
					sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Write.Error: error - %i\n", swResult);
					throw 0;
				}
			}

			///		Start device-0 (BC) running major-frame-3 for 1-shot.
			dwMinorFrameCounter = s_dwMinorFrameCounter;
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", swDevice_0, swFrameId_3, 1);
			swResult = sitalBc_Start (swDevice_0, swFrameId_3, 1);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
				throw 0;
			}

			///		Keep waiting until the global ISR frame counter shows that the frame has ended OR the overall frame run timer expires (the earlier):
			dwSleepTime = 100; // 0.1 second.
			dwOverallFrameRunTime = 0;
			while ((dwMinorFrameCounter == s_dwMinorFrameCounter) && (dwMaximumFrameRunTime > dwOverallFrameRunTime))
			{
				///			Wait for a while.
				sitalProcess_Log_PrintLine ("#StldTester.Wait: time_passed - %ims, isr_calls - %i\n", dwOverallFrameRunTime, s_dwMinorFrameCounter);
				sitalProcess_Log_PrintLine ("\tWAIT_FOR %i us\n", (dwSleepTime * 1000));
				pt.AccuratelySleep (dwSleepTime);

				///			Update the overall frame run timer.
				dwOverallFrameRunTime += dwSleepTime;
			}
			sitalProcess_Log_PrintLine ("#StldTester.Wait.End: unit - %i, isr_invoked - %i\n", swDevice_0, (dwMinorFrameCounter != s_dwMinorFrameCounter));

			/// Stop device-0 (BC).
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop: unit - %i\n", swDevice_0);
			swResult = sitalBc_Stop (swDevice_0);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
				throw 0;
			}

			///		Loop over all RT devices (RTs).
			wpCurrentRtNextIncrementedDataWord = waRtNextIncrementedDataWord;
			U16BIT wExpectedDataWord; // An expected data word.
			for (S16BIT i=1; i<=wRtCount; i++)
			{
				///			Read from device-0 (BC) data-block-i the suitable number of incremental data words used with device-i (RT).
				swDataBlockId = i;
				wWordCount = (33 - i); // 32 words for device-1, 31 words for device-2, 30 words for device-3, ...
				sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Read: unit - %i, block - %i, buf_size - %i, offset - %i\n", swDevice_0, swDataBlockId, wWordCount, 0);
				swResult = sitalBc_DataBlock_Read (swDevice_0, swDataBlockId, waBuffer, wWordCount, 0);
				if (0 > swResult)
				{
					sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Read.Error: error - %i\n", swResult);
					throw 0;
				}

				///			Verify that received from device-i (RT) the same incremental data that has been transmitted to it.
				if (TRUE == bOperationalRtDevices)
				{
					wpCurrentRtNextIncrementedDataWord++;
					wExpectedDataWord = (*wpCurrentRtNextIncrementedDataWord);
					for (int i=(wWordCount-1); i>=0; i--)
					{
						wExpectedDataWord--;
						if (wExpectedDataWord != waBuffer[i])
						{
							sitalProcess_Log_PrintLine ("#StldTester.Rt.ReceivedData.Error: error - %04X\n", waBuffer[i]);
							if (bStopOnDataError)
							{
								swResult = sitalReturnCode_TEST_FAIL;
								throw 0;
							}
						}
					}
				}
			}
			sitalProcess_Log_PrintLine ("#StldTester.TestIteration.End: iteration - %i\n", uiIteration);
		}

		/// Free device-0 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", swDevice_0);
		swResult = sitalDevice_Free (swDevice_0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Loop over all RT devices (RTs).
		if (TRUE == bOperationalRtDevices)
		{
			for (S16BIT i=1; i<=wRtCount; i++)
			{
				/// Stop device-i (RT).
				swDeviceId = i;
				sitalProcess_Log_PrintLine ("#StldTester.Rt.Stop: unit - %i\n", swDeviceId);
				swResult = sitalRt_Stop (swDeviceId);
				if (sitalReturnCode_SUCCESS != swResult)
				{
					sitalProcess_Log_PrintLine ("#StldTester.Rt.Stop.Error: error - %i\n", swResult);
					throw 0;
				}

				/// Free device-i (RT).
				sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", swDeviceId);
				swResult = sitalDevice_Free (swDeviceId);
				if (sitalReturnCode_SUCCESS != swResult)
				{
					sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
					throw 0;
				}
			}
		}
	}
	catch (int)
	{
		sitalDevice_Free (swDevice_0);
		if (TRUE == bOperationalRtDevices)
		{
			for (S16BIT i=1; i<=wRtCount; i++)
			{
				swDeviceId = i;
				sitalDevice_Free (swDeviceId);
			}
		}
	}

	sitalProcess_Log_PrintLine ("#StldTester.BcMultiRt_DataRxTx.End: test_succeeded - %c\n", ((sitalReturnCode_SUCCESS == swResult) ? 'Y' : 'N'));
	return swResult;
}



/// <summary>
/// Test the stld1553 library in the henceforth described scenario:
/// Participant[s]: RT1, BC2.
/// Configuration: All participating devices must be able to communicate with each other.
/// Short description: Acceptance Test Plan (ATP) test 461_11a - "One shot" mode for simplified BC operation.
/// Along test print suitable info and indications into the log.
///
/// Note:
/// - This function may use any of above defined Acceptance Test Plan (ATP) tests variables.
///   The caller must therefore call the suitable stldTester_Atp_Set* functions prior to calling this function.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Atp_DoTest461_11a ()
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_11a.Start\n");

	/// Set test arguments.
	U16BIT wRtSubaddressRx; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31) that's set for a RT for Rx.
	wRtSubaddressRx = sitalRtSubaddress_1;
	U16BIT wRtSubaddressTx; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31) that's set for a RT for Tx.
	wRtSubaddressTx = sitalRtSubaddress_2;
	U16BIT wWordCount; // Word count (1-32).
	wWordCount = 32;
	U16BIT wFrameTime; // The time (in units of 100 us) that the frame will take to complete.
	wFrameTime = 0; // No time imposed by major frame on its minor frames.
	U16BIT wFrameOneShotCount; // Number of frame 1-shot.
	wFrameOneShotCount = 4;

	S16BIT swResult; // Result of operation or function call.
	swResult = sitalReturnCode_SUCCESS;

	/// Get device count.
	U16BIT wRequiredDeviceCount; // The number of devices required for this test.
	wRequiredDeviceCount = (1 + max(s_swDevice1, s_swDevice2));
	U16BIT wAvailableDeviceCount; // The total number of devices available in the system.
	swResult = sitalDevice_GetCount (&wAvailableDeviceCount);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %i\n", swResult);
		return swResult;
	}
	sitalProcess_Log_PrintLine ("#StldTester.Device.Count: available - %i, required - %i\n", wAvailableDeviceCount, wRequiredDeviceCount);
	if (wAvailableDeviceCount < wRequiredDeviceCount)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %s\n", "not_enough_devices_installed");
		return sitalReturnCode_SUCCESS;
	}
	if ( (s_swDevice1 == s_swDevice2) )
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Number.Error: error - %s\n", "same_device_used_twice");
		return sitalReturnCode_INVALID_DEVICE_NUMBER;
	}

	try
	{
		swResult = stldSyncStart (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncStart.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-1 (RT) as a real RT.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-rt, alloc_mem-none\n", s_swDevice1);
		swResult = sitalDevice_Initialize (s_swDevice1, sitalAccess_CARD, ((s_bIsRtMtModeTest ? sitalMode_RT_AND_MT : sitalMode_RT) | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}
		if (s_bIsRtMtModeTest)
		{
			// Accommodate with tests where the required amount of allocations of data blocks for RT devices is to big when these RT devices are initialized as RT-MT devices with a rather limited amount of memory.
			sitalProcess_Log_PrintLine ("#StldTester.RtMt.Init: unit - %i, rt_command_stack_size - %i, mt_stack_count - %i, mt_command_stack_size - %i, mt_data_stack_size - %i, options - %X\n", s_swDevice1, sitalRtCommandStackSize_256, sitalMtStackOption_SINGLE, sitalMtCommandStackSize_256, sitalMtDataStackSize_512, 0);
			swResult = sitalRtMt_Initialize (s_swDevice1, sitalRtCommandStackSize_256, sitalMtStackOption_SINGLE, sitalMtCommandStackSize_256, sitalMtDataStackSize_512, 0);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.RtMt.Init.Error: error - %i\n", swResult);
				throw 0;
			}
		}

		/// Verify that device-1 (RT) uses an internal address source.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.AddrSource.Get: unit - %i\n", s_swDevice1);
		U16BIT wRtAddressSource; // Address source (sitalRtAddressSource_*).
		swResult = sitalRt_AddressSource_Get (s_swDevice1, &wRtAddressSource);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.AddrSource.Get.Error: error - %i\n", swResult);
			throw 0;
		}
		sitalProcess_Log_PrintLine ("#StldTester.Rt.AddrSource.Get: unit - %i, address_source - %s\n", s_swDevice1, ( (sitalRtAddressSource_EXTERNAL == wRtAddressSource) ? "external" : "internal" ));

		/// Set device-1 (RT) address to RT-address-1.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set: unit - %i, address - %i\n", s_swDevice1, s_wRtAddress1);
		swResult = sitalRt_Address_Set (s_swDevice1, s_wRtAddress1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Verify that device-1 (RT) has its RT address set to RT-address-1.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Get: unit - %i\n", s_swDevice1);
		U16BIT wRtAddress; // A RT address (0-31).
		swResult = sitalRt_Address_Get (s_swDevice1, &wRtAddress);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Get.Error: error - %i\n", swResult);
			throw 0;
		}
		if (s_wRtAddress1 != wRtAddress)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Get.Error: error - %i\n", wRtAddress);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		U16BIT wRtSubaddress; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31).
		const U16BIT waMessageType[] // Array of message types.
						= { sitalRtMessageType_RX, sitalRtMessageType_TX, sitalRtMessageType_BROADCAST};
		U16BIT wMessageTypeCount; // Number of defined message types.
		wMessageTypeCount = ( sizeof(waMessageType) / sizeof(waMessageType[0]) );
		S16BIT swDataBlockId; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId = 1;
		U16BIT waBuffer[32]; // A buffer to read-from/write-into along data-block write/read operations, respectively.
		for (int i=0; i<wWordCount; i++) waBuffer[i] = (wWordCount - 1 - i);
		for (wRtSubaddress=sitalRtSubaddress_1; wRtSubaddress<=sitalRtSubaddress_30; wRtSubaddress++)
		{
			for (int i=0; i<wMessageTypeCount; i++)
			{
				/// Create for device-1 (RT) data-block-DataBlockId, a data block of 64 data words.
				sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i, buf_contents-...%i\n", s_swDevice1, swDataBlockId, 64, wWordCount, (wWordCount - 1));
				swResult = sitalRt_DataBlock_Create (s_swDevice1, swDataBlockId, sitalRtDataBlockType_SINGLE_32, waBuffer, wWordCount);
				if (sitalReturnCode_SUCCESS != swResult)
				{
					sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.Create.Error: error - %i\n", swResult);
					throw 0;
				}

				/// Map for device-1 (RT) data-block-DataBlockId with message-type-j for subaddress-i, and legalize this subaddress.
				sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.MapToSa: unit - %i, block - %i, sa - %i, msg_types - x%X, irq_opts-x%X, legalize - %c\n", s_swDevice1, swDataBlockId, wRtSubaddress, waMessageType[i], sitalRtDataBlockIrq_NONE, 'Y');
				swResult = sitalRt_DataBlock_MapToSubaddress (s_swDevice1, swDataBlockId, wRtSubaddress, waMessageType[i], sitalRtDataBlockIrq_NONE, TRUE);
				if (sitalReturnCode_SUCCESS != swResult)
				{
					sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.MapToSa.Error: error - %i\n", swResult);
					throw 0;
				}

				/// Move to next DataBlockId.
				swDataBlockId++;
			}
		}

		/// Start device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Start: unit - %i\n", s_swDevice1);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Start (s_swDevice1) : sitalRt_Start (s_swDevice1) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-2 (BC) as a real BC.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-bc, alloc_mem-none\n", s_swDevice2);
		swResult = sitalDevice_Initialize (s_swDevice2, sitalAccess_CARD, (sitalMode_BC | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) data-block-0, a data block of 64 data words.
		S16BIT swDataBlockId_0; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId_0 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i\n", s_swDevice2, swDataBlockId_0, 64, 0);
		swResult = sitalBc_DataBlock_Create (s_swDevice2, swDataBlockId_0, sitalBcDataBlockSizeOption_DOUBLE, NULL, 0);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) data-block-1, a data block of 64 data words, and fill the first WordCount data words with ascending values.
		S16BIT swDataBlockId_1; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId_1 = 2;
		for (int i=0; i<wWordCount; i++) waBuffer[i] = i;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i, buf_contents - %i...\n", s_swDevice2, swDataBlockId_1, 64, wWordCount, 0);
		swResult = sitalBc_DataBlock_Create (s_swDevice2, swDataBlockId_1, sitalBcDataBlockSizeOption_DOUBLE, waBuffer, wWordCount);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) message-0, a RT(RT-address-1/RtSubAddressTx)-to-BC message that uses data-block-0 to receive WordCount data words. Message bus: bus-1.
		S16BIT swMessageId_0; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBc.Create: unit - %i, message - %i, block - %i, tx_rt - %i, tx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddressTx, wWordCount, 0, s_wBus1);
		swResult = sitalBc_Message_CreateRtToBc (s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddressTx, wWordCount, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBc.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) message-1, a BC-to-RT(RT-address-1/RtSubAddressRx) message that uses data-block-1 to transmit WordCount data words. Message bus: bus-1.
		S16BIT swMessageId_1; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_1 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create: unit - %i, message - %i, block - %i, rx_rt - %i, rx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_1, swDataBlockId_1, s_wRtAddress1, wRtSubaddressRx, wWordCount, 0, s_wBus1);
		swResult = sitalBc_Message_CreateBcToRt (s_swDevice2, swMessageId_1, swDataBlockId_1, s_wRtAddress1, wRtSubaddressRx, wWordCount, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-0, an XQF command that executes message-0.
		S16BIT swCommandId_0; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-1, an XQF command that executes message-1.
		S16BIT swCommandId_1; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_1 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_1, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_1, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_1, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_1, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-2, a subroutine call command that executes minor-frame-0. 
		S16BIT swCommandId_2; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_2 = 2;
		S16BIT swFrameId_0; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_2, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, swFrameId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_2, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, (U16BIT)swFrameId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) minor-frame-0, a frame that contains command-0 and command-1.
		S16BIT swaCommandIds[sitalBcCounter_COMMANDS]; // An array of command-IDs.
		swaCommandIds[0] = swCommandId_0;
		swaCommandIds[1] = swCommandId_1;
		U16BIT wCommandIdCount; // The number of command IDs in the frame (1-sitalBcCounter_FRAME_COMMANDS).
		wCommandIdCount = 2;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i, %i), time - %i, flags - %i\n", s_swDevice2, swFrameId_0, "minor", wCommandIdCount, swCommandId_0, swCommandId_1, wFrameTime, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_0, sitalBcFrameType_MINOR, swaCommandIds, wCommandIdCount, wFrameTime, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) major-frame-1, a frame that contains command-2 and whose frame time is FrameTime us.
		S16BIT swFrameId_1; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_1 = 1;
		swaCommandIds[0] = swCommandId_2;
		wCommandIdCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_1, "major", wCommandIdCount, swCommandId_2, (wFrameTime / 10), 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_1, sitalBcFrameType_MAJOR, swaCommandIds, wCommandIdCount, wFrameTime, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		wFrameTime = 1000; // 100ms.
		for (int i=0; i<wFrameOneShotCount; i++)
		{
			/// Start device-2 (BC) running major-frame-1 for 1-shot.
			U16BIT wFrameCount; // The number of frames to run.
			wFrameCount = 1;
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", s_swDevice2, swFrameId_1, wFrameCount);
			swResult = sitalBc_Start (s_swDevice2, swFrameId_1, 1);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
				throw 0;
			}

			///	Wait for the frame run to end.
			U32BIT dwRemainingFrameRunTime; // The expected remaining frame run time (in ms).
			dwRemainingFrameRunTime = (wFrameCount * (wFrameTime / 10));
			swResult = stldTester_Bc_WaitForEndOfFrameRun (s_swDevice2, dwRemainingFrameRunTime, 0, NULL);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.WaitForEndOfFrameRun.Error: error - %i\n", swResult);
				throw 0;
			}

			/// With device-1 (RT) get message-0.
			sitalDecodedMessageStructure dmsDecodedMessage; // A decoded message.
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Message.GetFromStackDecoded: unit - %i, message - %i, purge - %i\n", s_swDevice1, swMessageId_0, TRUE);
			swResult = sitalRt_Message_GetFromStackDecoded (s_swDevice1, &dmsDecodedMessage, sitalMessageLocationAndRemoval_NEXT_PURGE);
			if (1 != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Rt.Message.GetDecoded.Error: error - %i\n", swResult);
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}

			///	With device-1 (RT) verify that message-0 contains the expected data.
			for (int i=0; i<wWordCount; i++)
			{
				if ((wWordCount - 1 - i) != dmsDecodedMessage.waData[i])
				{
					sitalProcess_Log_PrintLine ("#StldTester.Rt.TransmittedData.Error: error - %i\n", waBuffer[i]);
					swResult = sitalReturnCode_TEST_FAIL;
					throw 0;
				}
			}

			/// With device-1 (RT) get message-1.
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Message.GetFromStackDecoded: unit - %i, message - %i, purge - %i\n", s_swDevice1, swMessageId_1, TRUE);
			swResult = sitalRt_Message_GetFromStackDecoded (s_swDevice1, &dmsDecodedMessage, sitalMessageLocationAndRemoval_NEXT_PURGE);
			if (1 != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Rt.Message.GetDecoded.Error: error - %i\n", swResult);
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}

			///	With device-1 (RT) verify that message-1 contains the expected data.
			for (int i=0; i<wWordCount; i++)
			{
				if (i != dmsDecodedMessage.waData[i])
				{
					sitalProcess_Log_PrintLine ("#StldTester.Rt.ReceivedData.Error: error - %i\n", waBuffer[i]);
					swResult = sitalReturnCode_TEST_FAIL;
					throw 0;
				}
			}

			/// Stop device-2 (BC).
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop: unit - %i\n", s_swDevice2);
			swResult = sitalBc_Stop (s_swDevice2);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
				throw 0;
			}
		}

		/// Stop device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Stop: unit - %i\n", s_swDevice1);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Stop (s_swDevice1) : sitalRt_Stop (s_swDevice1) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice2);
		swResult = sitalDevice_Free (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice1);
		swResult = sitalDevice_Free (s_swDevice1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		swResult = stldSyncEnd (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncEnd.Error: error - %i\n", swResult);
			throw 0;
		}
	}
	catch (int)
	{
		sitalDevice_Free (s_swDevice2);
		sitalDevice_Free (s_swDevice1);
	}

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_11a.End: test_succeeded - %c\n", ((sitalReturnCode_SUCCESS == swResult) ? 'Y' : 'N'));
	return swResult;
}



/// <summary>
/// Test the stld1553 library in the henceforth described scenario:
/// Participant[s]: RT1, BC2.
/// Configuration: All participating devices must be able to communicate with each other.
/// Short description: Acceptance Test Plan (ATP) test 461_11b - "Loop" mode (Auto repeat) for simplified repeatable BC message operation.
/// Along test print suitable info and indications into the log.
///
/// Note:
/// - This function may use any of above defined Acceptance Test Plan (ATP) tests variables.
///   The caller must therefore call the suitable stldTester_Atp_Set* functions prior to calling this function.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Atp_DoTest461_11b ()
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_11b.Start\n");

	/// Set test arguments.
	U16BIT wRtSubaddressRx; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31) that's set for a RT for Rx.
	wRtSubaddressRx = sitalRtSubaddress_1;
	U16BIT wRtSubaddressTx; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31) that's set for a RT for Tx.
	wRtSubaddressTx = sitalRtSubaddress_2;
	U16BIT wWordCount; // Word count (1-32).
	wWordCount = 32;
	U16BIT wFrameTime; // The time (in units of 100 us) that the frame will take to complete.
	// for T5 20ms is 4ms.
	wFrameTime = 200; // 20ms. 
	double dFrameRunMeasureTolerance; // The tolerance (in units of ms) allowed in frame run time measures.
	dFrameRunMeasureTolerance = 40; // Notice: This tolerance MUST be constant, NO MATTER the SHOT-COUNT, in order to prove there's no drift in h/w frame length.
	U16BIT wFrameShotCount; // Number of frame shots.
	wFrameShotCount = 101;
	U16BIT wFrameShotStep; // The number of frame shots to add when proceeding to the next sub-test.
	wFrameShotStep = 400;
	U16BIT wTotalSubTests; // The overall number of successful sub-tests to perform.
	wTotalSubTests = 5;
	U16BIT wMaximumSubTestFailureCount; // The maximum number of trials enabled for the current sub-test, i.e., the number of faulty repetitions of a sub-test that is considered a failure of the whole test.
	wMaximumSubTestFailureCount = 10;

	S16BIT swResult; // Result of operation or function call.
	swResult = sitalReturnCode_SUCCESS;
	InterProcessSemaphore ipsSemaphore (s_szBcSemaphoreName, 1); // A signaling object that is used by an assigned ISR function to signal interrupts to waiting threads.

	/// Get device count.
	U16BIT wRequiredDeviceCount; // The number of devices required for this test.
	wRequiredDeviceCount = (1 + max(s_swDevice1, s_swDevice2));
	U16BIT wAvailableDeviceCount; // The total number of devices available in the system.
	swResult = sitalDevice_GetCount (&wAvailableDeviceCount);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %i\n", swResult);
		return swResult;
	}
	sitalProcess_Log_PrintLine ("#StldTester.Device.Count: available - %i, required - %i\n", wAvailableDeviceCount, wRequiredDeviceCount);
	if (wAvailableDeviceCount < wRequiredDeviceCount)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %s\n", "not_enough_devices_installed");
		return sitalReturnCode_SUCCESS;
	}
	if ( (s_swDevice1 == s_swDevice2) )
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Number.Error: error - %s\n", "same_device_used_twice");
		return sitalReturnCode_INVALID_DEVICE_NUMBER;
	}

	try
	{
		swResult = stldSyncStart (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncStart.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create a signaling object.
		sitalProcess_Log_PrintLine ("#StldTester.Semaphore.Create: name - %s\n", s_szBcSemaphoreName);
		if (0U != ipsSemaphore.Create ())
		{
			sitalProcess_Log_PrintLine ("#StldTester.Semaphore.Create.Error: error - %i\n", GetLastError_NonOsDependent());
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		U16BIT wSubTestCount; // The count of sub-tests that have been performed by now, each having a different number of frame shots, in which the count of minor frames in the ISR wasn't disturbed by the operating-system.
		wSubTestCount = 0;
		U16BIT wSubTestFailureCount; // The number repetitions of the current sub-test in which the ISR failed to count all transmitted minor frames.
		wSubTestFailureCount = 0;
		while (wTotalSubTests > wSubTestCount)
		{
			sitalProcess_Log_PrintLine ("#StldTester.SubTest.Next: sub_test - %i(%i), failed - %i(%i), frame_shots - %i\n", wSubTestCount, wTotalSubTests, wSubTestFailureCount, wMaximumSubTestFailureCount, wFrameShotCount);
			//printf ("#StldTester.SubTest.Next: sub_test - %i/%i, failed - %i/%i, frame_shots - %i\n", wSubTestCount, wTotalSubTests, wSubTestFailureCount, wMaximumSubTestFailureCount, wFrameShotCount);

			/// Initialize device-1 (RT) as a real RT.
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-rt, alloc_mem-none\n", s_swDevice1);
			swResult = sitalDevice_Initialize (s_swDevice1, sitalAccess_CARD, ((s_bIsRtMtModeTest ? sitalMode_RT_AND_MT : sitalMode_RT) | sitalModeVariant_ADVANCED), 0, 0, 0);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
				throw 0;
			}
			if (s_bIsRtMtModeTest)
			{
				// Accommodate with tests where the required amount of allocations of data blocks for RT devices is to big when these RT devices are initialized as RT-MT devices with a rather limited amount of memory.
				sitalProcess_Log_PrintLine ("#StldTester.RtMt.Init: unit - %i, rt_command_stack_size - %i, mt_stack_count - %i, mt_command_stack_size - %i, mt_data_stack_size - %i, options - %X\n", s_swDevice1, sitalRtCommandStackSize_256, sitalMtStackOption_SINGLE, sitalMtCommandStackSize_256, sitalMtDataStackSize_512, 0);
				swResult = sitalRtMt_Initialize (s_swDevice1, sitalRtCommandStackSize_256, sitalMtStackOption_SINGLE, sitalMtCommandStackSize_256, sitalMtDataStackSize_512, 0);
				if (sitalReturnCode_SUCCESS != swResult)
				{
					sitalProcess_Log_PrintLine ("#StldTester.RtMt.Init.Error: error - %i\n", swResult);
					throw 0;
				}
			}

			/// Set device-1 (RT) address to RT-address-1.
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set: unit - %i, address - %i\n", s_swDevice1, s_wRtAddress1);
			swResult = sitalRt_Address_Set (s_swDevice1, s_wRtAddress1);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set.Error: error - %i\n", swResult);
				throw 0;
			}

			U16BIT wRtSubaddress; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31).
			const U16BIT waMessageType[] // Array of message types.
					= { sitalRtMessageType_RX, sitalRtMessageType_TX, sitalRtMessageType_BROADCAST};
			U16BIT wMessageTypeCount; // Number of defined message types.
			wMessageTypeCount = ( sizeof(waMessageType) / sizeof(waMessageType[0]) );
			S16BIT swDataBlockId; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
			swDataBlockId = 1;
			U16BIT waBuffer[32]; // A buffer to read-from/write-into along data-block write/read operations, respectively.
			for (int i=0; i<wWordCount; i++) waBuffer[i] = (wWordCount - 1 - i);
			for (wRtSubaddress=sitalRtSubaddress_1; wRtSubaddress<=sitalRtSubaddress_30; wRtSubaddress++)
			{
				for (int i=0; i<wMessageTypeCount; i++)
				{
					/// Create for device-1 (RT) data-block-DataBlockId, a data block of 64 data words.
					sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i, buf_contents-...%i\n", s_swDevice1, swDataBlockId, 64, wWordCount, (wWordCount - 1));
					swResult = sitalRt_DataBlock_Create (s_swDevice1, swDataBlockId, sitalRtDataBlockType_SINGLE_32, waBuffer, wWordCount);
					if (sitalReturnCode_SUCCESS != swResult)
					{
						sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.Create.Error: error - %i\n", swResult);
						throw 0;
					}

					/// Map for device-1 (RT) data-block-DataBlockId with message-type-j for subaddress-i, and legalize this subaddress.
					sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.MapToSa: unit - %i, block - %i, sa - %i, msg_types - x%X, irq_opts-x%X, legalize - %c\n", s_swDevice1, swDataBlockId, wRtSubaddress, waMessageType[i], sitalRtDataBlockIrq_NONE, 'Y');
					swResult = sitalRt_DataBlock_MapToSubaddress (s_swDevice1, swDataBlockId, wRtSubaddress, waMessageType[i], sitalRtDataBlockIrq_NONE, TRUE);
					if (sitalReturnCode_SUCCESS != swResult)
					{
						sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.MapToSa.Error: error - %i\n", swResult);
						throw 0;
					}

					/// Move to next DataBlockId.
					swDataBlockId++;
				}
			}

			/// Start device-1 (RT).
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Start: unit - %i\n", s_swDevice1);
			swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Start (s_swDevice1) : sitalRt_Start (s_swDevice1) );
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Rt.Start.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Initialize device-2 (BC) as a real BC.
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-bc, alloc_mem-none\n", s_swDevice2);
			swResult = sitalDevice_Initialize (s_swDevice2, sitalAccess_CARD, (sitalMode_BC | sitalModeVariant_ADVANCED), 0, 0, 0);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Create for device-2 (BC) data-block-0, a data block of 64 data words.
			S16BIT swDataBlockId_0; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
			swDataBlockId_0 = 1;
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i\n", s_swDevice2, swDataBlockId_0, 64, 0);
			swResult = sitalBc_DataBlock_Create (s_swDevice2, swDataBlockId_0, sitalBcDataBlockSizeOption_DOUBLE, NULL, 0);
			if (0 > swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Create for device-2 (BC) data-block-1, a data block of 64 data words, and fill the first WordCount data words with ascending values.
			S16BIT swDataBlockId_1; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
			swDataBlockId_1 = 2;
			for (int i=0; i<wWordCount; i++) waBuffer[i] = i;
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i, buf_contents - %i...\n", s_swDevice2, swDataBlockId_1, 64, wWordCount, 0);
			swResult = sitalBc_DataBlock_Create (s_swDevice2, swDataBlockId_1, sitalBcDataBlockSizeOption_DOUBLE, waBuffer, wWordCount);
			if (0 > swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Create for device-2 (BC) message-0, a RT(RT-address-1/RtSubAddressTx)-to-BC message that uses data-block-0 to receive WordCount data words. Message bus: bus-1.
			S16BIT swMessageId_0; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
			swMessageId_0 = 0;
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBc.Create: unit - %i, message - %i, block - %i, tx_rt - %i, tx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddressTx, wWordCount, 0, s_wBus1);
			swResult = sitalBc_Message_CreateRtToBc (s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddressTx, wWordCount, 0, s_wBus1);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBc.Create.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Create for device-2 (BC) message-1, a BC-to-RT(RT-address-1/RtSubAddressRx) message that uses data-block-1 to transmit WordCount data words. Message bus: bus-1.
			S16BIT swMessageId_1; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
			swMessageId_1 = 1;
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create: unit - %i, message - %i, block - %i, rx_rt - %i, rx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_1, swDataBlockId_1, s_wRtAddress1, wRtSubaddressRx, wWordCount, 0, s_wBus1);
			swResult = sitalBc_Message_CreateBcToRt (s_swDevice2, swMessageId_1, swDataBlockId_1, s_wRtAddress1, wRtSubaddressRx, wWordCount, 0, s_wBus1);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Create for device-2 (BC) command-0, an XQF command that executes message-0.
			S16BIT swCommandId_0; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
			swCommandId_0 = 0;
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0);
			swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0, 0);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Create for device-2 (BC) command-1, an XQF command that executes message-1.
			S16BIT swCommandId_1; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
			swCommandId_1 = 1;
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_1, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_1, 0);
			swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_1, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_1, 0, 0);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Create for device-2 (BC) command-2, a subroutine call command that executes minor-frame-0. 
			S16BIT swCommandId_2; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
			swCommandId_2 = 2;
			S16BIT swFrameId_0; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
			swFrameId_0 = 0;
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_2, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, swFrameId_0, 0);
			swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_2, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, (U16BIT)swFrameId_0, 0, 0);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Create for device-2 (BC) minor-frame-0, a frame that contains command-0 and command-1.
			S16BIT swaCommandIds[sitalBcCounter_COMMANDS]; // An array of command-IDs.
			swaCommandIds[0] = swCommandId_0;
			swaCommandIds[1] = swCommandId_1;
			U16BIT wCommandIdCount; // The number of command IDs in the frame (1-sitalBcCounter_FRAME_COMMANDS).
			wCommandIdCount = 2;
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i, %i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_0, "minor", wCommandIdCount, swCommandId_0, swCommandId_1, (wFrameTime / 10), 0);
			swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_0, sitalBcFrameType_MINOR, swaCommandIds, wCommandIdCount, wFrameTime, 0);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Create for device-2 (BC) major-frame-1, a frame that contains command-2 and whose frame time is FrameTime us.
			S16BIT swFrameId_1; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
			swFrameId_1 = 1;
			swaCommandIds[0] = swCommandId_2;
			wCommandIdCount = 1;
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_1, "major", wCommandIdCount, swCommandId_2, (wFrameTime / 10), 0);
			swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_1, sitalBcFrameType_MAJOR, swaCommandIds, wCommandIdCount, wFrameTime, 0);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Configure device-2 (BC) to get IRQ-3 notifications using an ISR that collects interrupt data, counts ISR calls, and signals interested threads.
			sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Manipulate: unit - %i, enable - %i, irq - %08X, callback - %s\n", s_swDevice2, TRUE, sitalInterruptRegister2_BC_IRQ3, "stldTester_Bc_UserIsr");
			swResult = sitalDevice_Irq_Manipulate (s_swDevice2, TRUE, sitalInterruptRegister2_BC_IRQ3, stldTester_Bc_UserIsr);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Set.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Zero the global ISR counter of ended minor frames.
			s_dwMinorFrameCounter = 0;

			// Give the precision timers a chance to properly measure frame run.
			s_bIsPrecisionTimerRegressive = false;
			s_bIsPrecisionTimerHasty = false;
			s_wMaxFrameTime = ((wFrameTime / 10) + (wFrameTime / 20)); // 50% spare time added.

			/// Start device-2 (BC) running major-frame-1 for FrameShotCount-shots.
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", s_swDevice2, swFrameId_1, wFrameShotCount);
			swResult = sitalBc_Start (s_swDevice2, swFrameId_1, wFrameShotCount);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
				throw 0;
			}

			///	Wait for the first transmitted minor frame to end.
			U32BIT dwSleepTime; // Sleeping period (in ms).
			dwSleepTime = ( (1 *  (wFrameTime / 10)) + stldInterruptLatency_Get () );
		/*	if (0U != ipsSemaphore.Catch (dwSleepTime)) 
			{ 
				sitalProcess_Log_PrintLine ("#StldTester.Semaphore.Wait.Error: error - %s\n", "timeout");
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}
*/
			/// Read the computer's current tick counter into the global ISR frame run time measurer.
			s_ptMinorFrameRunTimeMeasurer.SetBaseTime ();
			s_bFirstFrame = true;

			sitalProcess_Log_PrintLine ("#StldTester.Wait: time - %ims\n", dwSleepTime);
			sitalProcess_Log_PrintLine ("\tWAIT_FOR %i us\n", (dwSleepTime * 1000));

			/// Wait for the frame run to end.
			U32BIT dwRemainingFrameRunTime; // The expected remaining frame run time (in ms).
			dwRemainingFrameRunTime = ((U32BIT)(wFrameShotCount - 1) * (wFrameTime / 10));
			// taking the IP frequency under account
			dwRemainingFrameRunTime = (U32BIT)(dwRemainingFrameRunTime / IP_FREQUENCY_FACTOR);
			swResult = stldTester_Bc_WaitForEndOfFrameRun (s_swDevice2, dwRemainingFrameRunTime, wFrameShotCount, &s_dwMinorFrameCounter);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.WaitForEndOfFrameRun.Error: error - %i\n", swResult);
				throw 0;
			}

			U32BIT dwMinorFrameCounter; // The overall number of transmitted minor frames, used instead variable s_dwMinorFrameCounter as the latter may be updated in a late ISR call, what may cause inconsistent behavior of below code.
			dwMinorFrameCounter = s_dwMinorFrameCounter;
			BOOLEAN bIsFrameCountOkay; // A flag that says whether all transmitted minor frames have been counted.
			bIsFrameCountOkay = (wFrameShotCount == dwMinorFrameCounter);

			double dTimeDifference; // The excess time (ms) it took for the overall frame run.
			dTimeDifference = ( (s_ptMinorFrameRunTimeMeasurer.GetDuration () >= dwRemainingFrameRunTime) ? (s_ptMinorFrameRunTimeMeasurer.GetDuration () - dwRemainingFrameRunTime) : (dwRemainingFrameRunTime - s_ptMinorFrameRunTimeMeasurer.GetDuration ()) );
			bool runIsFasterThanRequired = false;
			runIsFasterThanRequired = (s_ptMinorFrameRunTimeMeasurer.GetDuration () >= dwRemainingFrameRunTime) ? false : true;
			
			sitalProcess_Log_PrintLine ("#StldTester.Bc.FrameRun.Summary: unit - %i, count - %i, expected_count - %i, all_counted - %i, frame_time - %ims, overall_time - %fms, expected_time - %ims, excess_time - %f\n", s_swDevice2, dwMinorFrameCounter, wFrameShotCount, bIsFrameCountOkay, (wFrameTime / 10), s_ptMinorFrameRunTimeMeasurer.GetDuration (), dwRemainingFrameRunTime, dTimeDifference);
			if (FALSE == bIsFrameCountOkay)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.FrameRun.ActualCount.Error: error - wrong\n");
				wSubTestFailureCount++;

				if (wMaximumSubTestFailureCount <= wSubTestFailureCount)
				{
					swResult = sitalReturnCode_TEST_FAIL;
					throw 0;
				}
			}
			else
			{
				// If the system's precision timer functioned correctly, verify that frame transmission took around the expected time.
				if (s_bIsPrecisionTimerRegressive || s_bIsPrecisionTimerHasty)
				{
					sitalProcess_Log_PrintLine ("#StldTester.Bc.FrameRun.TimeMeasuring.Error: timer_regressive - %i, timer_hasty - %i, frame_time - %ims, max_accepted_frame_time - %ims, time_test_ignored\n", s_bIsPrecisionTimerRegressive, s_bIsPrecisionTimerHasty, (wFrameTime / 10), s_wMaxFrameTime);
				}
				else
				{
					if (dFrameRunMeasureTolerance < dTimeDifference)
					{
						sitalProcess_Log_PrintLine ("#StldTester.Bc.FrameRun.ActualOverallTime.Error: error - %f\n", dTimeDifference);
						swResult = sitalReturnCode_TEST_FAIL;
						throw 0;
					}
				}

				wFrameShotCount += wFrameShotStep;
				wSubTestFailureCount = 0;
				wSubTestCount++;
			}

			/// Stop device-2 (BC).
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop: unit - %i\n", s_swDevice2);
			swResult = sitalBc_Stop (s_swDevice2);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Stop device-1 (RT).
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Stop: unit - %i\n", s_swDevice1);
			swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Stop (s_swDevice1) : sitalRt_Stop (s_swDevice1) );
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Rt.Stop.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Free device-2 (BC).
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice2);
			swResult = sitalDevice_Free (s_swDevice2);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Free device-1 (RT).
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice1);
			swResult = sitalDevice_Free (s_swDevice1);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
				throw 0;
			}
		}

		/// Close the signaling object.
		sitalProcess_Log_PrintLine ("#StldTester.Semaphore.Close: name - %s\n", s_szBcSemaphoreName);
		if (0U != ipsSemaphore.Close ())
		{
			sitalProcess_Log_PrintLine ("#StldTester.Semaphore.Close.Error: error - %i\n", GetLastError_NonOsDependent());
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		if (0U != s_ipspBcSemaphore->Close ())
		{
			sitalProcess_Log_PrintLine ("#StldTester.Semaphore.Callback.Close.Error: error - %i\n", GetLastError_NonOsDependent());
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		delete (s_ipspBcSemaphore);
		s_ipspBcSemaphore = NULL;

		swResult = stldSyncEnd (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncEnd.Error: error - %i\n", swResult);
			throw 0;
		}
	}
	catch (int)
	{
		sitalDevice_Free (s_swDevice2);
		sitalDevice_Free (s_swDevice1);
		ipsSemaphore.Close ();
		if (NULL != s_ipspBcSemaphore)
		{
			s_ipspBcSemaphore->Close ();
			delete (s_ipspBcSemaphore);
			s_ipspBcSemaphore = NULL;
		}
	}

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_11b.End: test_succeeded - %c\n", ((sitalReturnCode_SUCCESS == swResult) ? 'Y' : 'N'));
	return swResult;
}



/// <summary>
/// Test the stld1553 library in the henceforth described scenario:
/// Participant[s]: BC2, MT3.
/// Configuration: All participating devices must be able to communicate with each other.
/// Short description: Acceptance Test Plan (ATP) test 461_11c - Multiple sequencing messages (multi frame setup) based on real-time schedule.
/// Along test print suitable info and indications into the log.
///
/// Note:
/// - This function may use any of above defined Acceptance Test Plan (ATP) tests variables.
///   The caller must therefore call the suitable stldTester_Atp_Set* functions prior to calling this function.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Atp_DoTest461_11c ()
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_11c.Start\n");

	/// Set test arguments.
	U16BIT wWordCount; // Word count (1-32).
	wWordCount = 32;
	U16BIT wRtSubaddress1; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31).
	wRtSubaddress1 = 1;
	U16BIT wRtSubaddress2; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31).
	wRtSubaddress2 = 2;
	U16BIT wFrameTime; // The time (in units of 100 us) that the frame will take to complete.
	wFrameTime = 100; // 10ms.
	U16BIT wMinorFrameCount; // The number of minor frames contained within the major frame.
	wMinorFrameCount = 4;

	S16BIT swResult; // Result of operation or function call.
	swResult = sitalReturnCode_SUCCESS;

	/// Get device count.
	U16BIT wRequiredDeviceCount; // The number of devices required for this test.
	wRequiredDeviceCount = (1 + max(s_swDevice2, s_swDevice3));
	U16BIT wAvailableDeviceCount; // The total number of devices available in the system.
	swResult = sitalDevice_GetCount (&wAvailableDeviceCount);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %i\n", swResult);
		return swResult;
	}
	sitalProcess_Log_PrintLine ("#StldTester.Device.Count: available - %i, required - %i\n", wAvailableDeviceCount, wRequiredDeviceCount);
	if (wAvailableDeviceCount < wRequiredDeviceCount)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %s\n", "not_enough_devices_installed");
		return sitalReturnCode_SUCCESS;
	}
	if ( (s_swDevice2 == s_swDevice3) )
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Number.Error: error - %s\n", "same_device_used_twice");
		return sitalReturnCode_INVALID_DEVICE_NUMBER;
	}

	try
	{
		swResult = stldSyncStart (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncStart.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-3 (MT) as a real MT.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access - card, mode - mt, alloc_mem - none\n", s_swDevice3);
		swResult = sitalDevice_Initialize (s_swDevice3, sitalAccess_CARD, ((s_bIsRtMtModeTest ? sitalMode_RT_AND_MT : sitalMode_MT) | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Start device-3 (MT).
		sitalProcess_Log_PrintLine ("#StldTester.Mt.Start: unit - %i\n", s_swDevice3);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Start (s_swDevice3) : sitalMt_Start (s_swDevice3) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		if ( ! s_bIsRtMtModeTest )
		{
			/// Pause device-3 (MT).
			// Note: It will be only proved in the second part of this test that function sitalMt_Pause really works, but it is nevertheless assumed here it really.
			sitalProcess_Log_PrintLine ("#StldTester.Mt.Pause: unit - %i\n", s_swDevice3);
			swResult = sitalMt_Pause (s_swDevice3);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Mt.Pause.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Make device-3 (MT) continue.
			// Note: As said above, it is assumed here that function sitalMt_Pause paused the operation of device-3 (MT), and therefore the following call to function sitalMt_Continue must really work in order for device-3 (MT) to record messages and for the test to succeed.
			sitalProcess_Log_PrintLine ("#StldTester.Mt.Continue: unit - %i\n", s_swDevice3);
			swResult = sitalMt_Continue (s_swDevice3);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Mt.Continue.Error: error - %i\n", swResult);
				throw 0;
			}
		}

		/// Initialize device-2 (BC) as a real BC.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-bc, alloc_mem-none\n", s_swDevice2);
		swResult = sitalDevice_Initialize (s_swDevice2, sitalAccess_CARD, (sitalMode_BC | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) data-block-0, a data block of 64 data words.
		S16BIT swDataBlockId_0; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId_0 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i\n", s_swDevice2, swDataBlockId_0, 64, 0);
		swResult = sitalBc_DataBlock_Create (s_swDevice2, swDataBlockId_0, sitalBcDataBlockSizeOption_DOUBLE, NULL, 0);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) message-0, a RT(RT-address-1/RtSubAddress1)-to-BC message that uses data-block-0 to receive WordCount data words. Message bus: bus-1.
		S16BIT swMessageId_0; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBc.Create: unit - %i, message - %i, block - %i, tx_rt - %i, tx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddress1, wWordCount, 0, s_wBus1);
		swResult = sitalBc_Message_CreateRtToBc (s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddress1, wWordCount, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBc.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) message-1, a BC-to-RT(RT-address-2/RtSubAddress2) message that uses data-block-0 to transmit WordCount data words. Message bus: bus-1.
		S16BIT swMessageId_1; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_1 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create: unit - %i, message - %i, block - %i, rx_rt - %i, rx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_1, swDataBlockId_0, s_wRtAddress1, wRtSubaddress2, wWordCount, 0, s_wBus1);
		swResult = sitalBc_Message_CreateBcToRt (s_swDevice2, swMessageId_1, swDataBlockId_0, s_wRtAddress2, wRtSubaddress2, wWordCount, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) message-2, a RT(RT-address-2/RtSubAddress2)-to-RT(RT-address-1/RtSubAddress1) message that uses data-block-0 to receive WordCount data words. Message bus: bus-1.
		S16BIT swMessageId_2; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_2 = 2;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToRt.Create: unit - %i, message - %i, block - %i, tx_rt - %i, tx_rt_sa - %i, rx_rt - %i, rx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_2, swDataBlockId_0, s_wRtAddress2, wRtSubaddress2, s_wRtAddress1, wRtSubaddress1, wWordCount, 0, s_wBus1);
		swResult = sitalBc_Message_CreateRtToRt (s_swDevice2, swMessageId_2, swDataBlockId_0, s_wRtAddress1, wRtSubaddress1, wWordCount, s_wRtAddress2, wRtSubaddress2, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToRt.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-0, a XEQ command that executes message-0.
		S16BIT swCommandId_0; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-1, a XEQ command that executes message-1.
		S16BIT swCommandId_1; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_1 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_1, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_1, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_1, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_1, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-2, a XEQ command that executes message-2.
		S16BIT swCommandId_2; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_2 = 2;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_2, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_2, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_2, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_2, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-3, a subroutine call command that executes minor-frame-0. 
		S16BIT swCommandId_3; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_3 = 3;
		S16BIT swFrameId_0; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_3, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, swFrameId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_3, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, (U16BIT)swFrameId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-4, a subroutine call command that executes minor-frame-1. 
		S16BIT swCommandId_4; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_4 = 4;
		S16BIT swFrameId_1; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_1 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_4, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, swFrameId_1, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_4, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, (U16BIT)swFrameId_1, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-5, a subroutine call command that executes minor-frame-2. 
		S16BIT swCommandId_5; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_5 = 5;
		S16BIT swFrameId_2; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_2 = 2;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_5, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, swFrameId_2, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_5, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, (U16BIT)swFrameId_2, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-6, a subroutine call command that executes minor-frame-3. 
		S16BIT swCommandId_6; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_6 = 6;
		S16BIT swFrameId_3; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_3 = 3;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_6, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, swFrameId_3, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_6, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, (U16BIT)swFrameId_3, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) minor-frame-0, a frame that contains command-0 and command-1.
		S16BIT swaCommandIds[sitalBcCounter_COMMANDS]; // An array of command-IDs.
		swaCommandIds[0] = swCommandId_0;
		swaCommandIds[1] = swCommandId_1;
		U16BIT wCommandIdCount; // The number of command IDs in the frame (1-sitalBcCounter_FRAME_COMMANDS).
		wCommandIdCount = 2;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i, %i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_0, "minor", wCommandIdCount, swCommandId_0, swCommandId_1, (wFrameTime / 10), 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_0, sitalBcFrameType_MINOR, swaCommandIds, wCommandIdCount, wFrameTime, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) minor-frame-1, a frame that contains command-0 and command-2.
		swaCommandIds[0] = swCommandId_0;
		swaCommandIds[1] = swCommandId_2;
		wCommandIdCount = 2;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i, %i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_1, "minor", wCommandIdCount, swCommandId_0, swCommandId_2, (wFrameTime / 10), 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_1, sitalBcFrameType_MINOR, swaCommandIds, wCommandIdCount, wFrameTime, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) minor-frame-2, a frame that contains command-0 and command-1.
		swaCommandIds[0] = swCommandId_0;
		swaCommandIds[1] = swCommandId_1;
		wCommandIdCount = 2;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i, %i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_2, "minor", wCommandIdCount, swCommandId_0, swCommandId_1, (wFrameTime / 10), 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_2, sitalBcFrameType_MINOR, swaCommandIds, wCommandIdCount, wFrameTime, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) minor-frame-3, a frame that contains command-0.
		swaCommandIds[0] = swCommandId_0;
		wCommandIdCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_3, "minor", wCommandIdCount, swCommandId_0, (wFrameTime / 10), 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_3, sitalBcFrameType_MINOR, swaCommandIds, wCommandIdCount, wFrameTime, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) major-frame-4, a frame that contains command-3 and command-4 and command-5 and command-6 and whose frame time is FrameTime us.
		S16BIT swFrameId_4; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_4 = 4;
		swaCommandIds[0] = swCommandId_3;
		swaCommandIds[1] = swCommandId_4;
		swaCommandIds[2] = swCommandId_5;
		swaCommandIds[3] = swCommandId_6;
		wCommandIdCount = 4;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i, %i, %i, %i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_4, "major", wCommandIdCount, swCommandId_3, swCommandId_4, swCommandId_5, swCommandId_6, (wFrameTime / 10), 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_4, sitalBcFrameType_MAJOR, swaCommandIds, wCommandIdCount, wFrameTime, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Start device-2 (BC) running major-frame-4 for 1-shot.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", s_swDevice2, swFrameId_4, 1);
		swResult = sitalBc_Start (s_swDevice2, swFrameId_4, 1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		///	Wait for the frame run to end.
		U32BIT dwRemainingFrameRunTime; // The expected remaining frame run time (in ms).
		dwRemainingFrameRunTime = (wMinorFrameCount * (wFrameTime / 10));
		swResult = stldTester_Bc_WaitForEndOfFrameRun (s_swDevice2, dwRemainingFrameRunTime, 0, NULL);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.WaitForEndOfFrameRun.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Stop device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop: unit - %i\n", s_swDevice2);
		swResult = sitalBc_Stop (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Use given message descriptors to code the [first] command word of above defined messages.
		// Note: Defined messages have been intentionally devised in a way that their [first] command word will differ.
		U16BIT wCommandWord0; // Command word for above defined RT-to-BC message-0.
		swResult = sitalBcStld1553_Command_GetCode (&wCommandWord0, s_wRtAddress1, sitalCommand_TX, wRtSubaddress1, wWordCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Command.GetCode.Error: error - %i\n", swResult);
			throw 0;
		}
		U16BIT wCommandWord1; // Command word for above defined BC-to-RT message-1.
		swResult = sitalBcStld1553_Command_GetCode (&wCommandWord1, s_wRtAddress2, sitalCommand_RX, wRtSubaddress2, wWordCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Command.GetCode.Error: error - %i\n", swResult);
			throw 0;
		}
		U16BIT wCommandWord2; // Command word for above defined RT-to-RT message-2.
		swResult = sitalBcStld1553_Command_GetCode (&wCommandWord2, s_wRtAddress1, sitalCommand_RX, wRtSubaddress1, wWordCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Command.GetCode.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Verify that device-3 (MT) has detected messages in the expected composition.
		sitalDecodedMessageStructure dmsDecodedMessage; // A decoded message.
		U16BIT wRtToBcMessageCount; // The total number of RT-to-BC messages that have been identified.
		wRtToBcMessageCount = 0;
		U16BIT wBcToRtMessageCount; // The total number of BC-to-RT messages that have been identified.
		wBcToRtMessageCount = 0;
		U16BIT wRtToRtMessageCount; // The total number of RT-to-RT messages that have been identified.
		wRtToRtMessageCount = 0;
		do
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded: unit - %i, purge - %i\n", s_swDevice3, TRUE);
			swResult = sitalMt_Message_GetFromStackDecoded (s_swDevice3, &dmsDecodedMessage, sitalMessageLocationAndRemoval_NEXT_PURGE, sitalMtStack_ACTIVE);
			if (1 == swResult)
			{
				if (wCommandWord0 == dmsDecodedMessage.wCommandWord1)
				{
					wRtToBcMessageCount++;
				}
				else
				if (wCommandWord1 == dmsDecodedMessage.wCommandWord1)
				{
					wBcToRtMessageCount++;
				}
				else
				if (wCommandWord2 == dmsDecodedMessage.wCommandWord1)
				{
					wRtToRtMessageCount++;
				}
				else
				{
					sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Message.Error: error - %04X\n", dmsDecodedMessage.wCommandWord1);
					swResult = sitalReturnCode_TEST_FAIL;
					throw 0;
				}
			}
		}
		while (1 == swResult);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Error: error - %i\n", swResult);
			throw 0;
		}
		sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.MessageComposition: unit - %i, RT2BC(%04X) - %i(expected-%i), BC2RT(%04X) - %i(expected-%i), RT2RT(%04X) - %i(expected-%i)\n", s_swDevice3, wCommandWord0, wRtToBcMessageCount, wMinorFrameCount, wCommandWord1, wBcToRtMessageCount, (wMinorFrameCount / 2), wCommandWord2, wRtToRtMessageCount, (wMinorFrameCount / 4));
		if ((wMinorFrameCount != wRtToBcMessageCount) || ((wMinorFrameCount / 2) != wBcToRtMessageCount) || ((wMinorFrameCount / 4) != wRtToRtMessageCount))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Error: error - %s\n", "unexpected_message_composition");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		if ( ! s_bIsRtMtModeTest )
		{
			/// Pause device-3 (MT).
			// Note: Here it will be proved that function sitalMt_Pause really works.
			sitalProcess_Log_PrintLine ("#StldTester.Mt.Pause: unit - %i\n", s_swDevice3);
			swResult = sitalMt_Pause (s_swDevice3);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Mt.Pause.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Start device-2 (BC) running major-frame-4 for 1-shot.
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", s_swDevice2, swFrameId_4, 1);
			swResult = sitalBc_Start (s_swDevice2, swFrameId_4, 1);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
				throw 0;
			}

			///	Wait for the frame run to end.
			dwRemainingFrameRunTime = (wMinorFrameCount * (wFrameTime / 10));
			swResult = stldTester_Bc_WaitForEndOfFrameRun (s_swDevice2, dwRemainingFrameRunTime, 0, NULL);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.WaitForEndOfFrameRun.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Stop device-2 (BC).
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop: unit - %i\n", s_swDevice2);
			swResult = sitalBc_Stop (s_swDevice2);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Make device-3 (MT) continue.
			// Note: The operation of device-3 (MT) is here resumed after the regular frames are transmitted, and it will be eventually verified that it really missed these frames.
			sitalProcess_Log_PrintLine ("#StldTester.Mt.Continue: unit - %i\n", s_swDevice3);
			swResult = sitalMt_Continue (s_swDevice3);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Mt.Continue.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Verify that device-3 (MT) has been actually paused, and didn't record any message.`
			swResult = sitalMt_Message_GetFromStackDecoded (s_swDevice3, &dmsDecodedMessage, sitalMessageLocationAndRemoval_NEXT_PURGE, sitalMtStack_ACTIVE);
			if ((0 > swResult) || (1 < swResult))
			{
				sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Error: error - %i\n", swResult);
				throw 0;
			}
			if (1 == swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Message.Error: error - %s\n", "mt_active");
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}

			/// Stop device-3 (MT).
			sitalProcess_Log_PrintLine ("#StldTester.Mt.Stop: unit - %i\n", s_swDevice3);
			swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Stop (s_swDevice3) : sitalMt_Stop (s_swDevice3) );
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Mt.Stop.Error: error - %i\n", swResult);
				throw 0;
			}
		}

		/// Free device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice2);
		swResult = sitalDevice_Free (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-3 (MT).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice3);
		swResult = sitalDevice_Free (s_swDevice3);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		swResult = stldSyncEnd (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncEnd.Error: error - %i\n", swResult);
			throw 0;
		}
	}
	catch (int)
	{
		sitalDevice_Free (s_swDevice2);
		sitalDevice_Free (s_swDevice3);
	}

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_11c.End: test_succeeded - %c\n", ((sitalReturnCode_SUCCESS == swResult) ? 'Y' : 'N'));
	return swResult;
}



/// <summary>
/// Test the stld1553 library in the henceforth described scenario:
/// Participant[s]: BC2, MT3.
/// Configuration: All participating devices must be able to communicate with each other.
/// Short description: Acceptance Test Plan (ATP) test 461_11d - Multi frame and auto repeat.
/// Along test print suitable info and indications into the log.
///
/// Note:
/// - This function may use any of above defined Acceptance Test Plan (ATP) tests variables.
///   The caller must therefore call the suitable stldTester_Atp_Set* functions prior to calling this function.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Atp_DoTest461_11d ()
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_11d.Start\n");

	/// Set test arguments.
	U16BIT wWordCount; // Word count (1-32).
	wWordCount = 32;
	U16BIT wRtAddress1; // A RT address (0-31).
	wRtAddress1 = 1;
	U16BIT wRtAddress2; // A RT address (0-31).
	wRtAddress2 = 2;
	U16BIT wRtSubaddress1; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31).
	wRtSubaddress1 = 1;
	U16BIT wRtSubaddress2; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31).
	wRtSubaddress2 = 2;
	U16BIT wFrameTime; // The time (in units of 100 us) that the frame will take to complete.
	wFrameTime = 100; // 10ms.
	U16BIT wFrameShotCount; // Number of frame shots.
	wFrameShotCount = 100;
	U16BIT wMinorFrameCount; // The number of minor frames contained within the major frame.
	wMinorFrameCount = 4;

	S16BIT swResult; // Result of operation or function call.
	swResult = sitalReturnCode_SUCCESS;

	/// Get device count.
	U16BIT wRequiredDeviceCount; // The number of devices required for this test.
	wRequiredDeviceCount = (1 + max(s_swDevice2, s_swDevice3));
	U16BIT wAvailableDeviceCount; // The total number of devices available in the system.
	swResult = sitalDevice_GetCount (&wAvailableDeviceCount);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %i\n", swResult);
		return swResult;
	}
	sitalProcess_Log_PrintLine ("#StldTester.Device.Count: available - %i, required - %i\n", wAvailableDeviceCount, wRequiredDeviceCount);
	if (wAvailableDeviceCount < wRequiredDeviceCount)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %s\n", "not_enough_devices_installed");
		return sitalReturnCode_SUCCESS;
	}
	if ( (s_swDevice2 == s_swDevice3) )
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Number.Error: error - %s\n", "same_device_used_twice");
		return sitalReturnCode_INVALID_DEVICE_NUMBER;
	}

	try
	{
		swResult = stldSyncStart (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncStart.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-3 (MT) as a real MT.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access - card, mode - mt, alloc_mem - none\n", s_swDevice3);
		swResult = sitalDevice_Initialize (s_swDevice3, sitalAccess_CARD, ((s_bIsRtMtModeTest ? sitalMode_RT_AND_MT : sitalMode_MT) | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Start device-3 (MT).
		sitalProcess_Log_PrintLine ("#StldTester.Mt.Start: unit - %i\n", s_swDevice3);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Start (s_swDevice3) : sitalMt_Start (s_swDevice3) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-2 (BC) as a real BC.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-bc, alloc_mem-none\n", s_swDevice2);
		swResult = sitalDevice_Initialize (s_swDevice2, sitalAccess_CARD, (sitalMode_BC | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) data-block-0, a data block of 64 data words.
		S16BIT swDataBlockId_0; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId_0 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i\n", s_swDevice2, swDataBlockId_0, 64, 0);
		swResult = sitalBc_DataBlock_Create (s_swDevice2, swDataBlockId_0, sitalBcDataBlockSizeOption_DOUBLE, NULL, 0);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) message-0, a RT(RT-address-1/RtSubAddress1)-to-BC message that uses data-block-0 to receive WordCount data words. Message bus: bus-1.
		S16BIT swMessageId_0; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBc.Create: unit - %i, message - %i, block - %i, tx_rt - %i, tx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddress1, wWordCount, 0, s_wBus1);
		swResult = sitalBc_Message_CreateRtToBc (s_swDevice2, swMessageId_0, swDataBlockId_0, wRtAddress1, wRtSubaddress1, wWordCount, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBc.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) message-1, a BC-to-RT(RT-address-1/RtSubAddress2) message that uses data-block-0 to transmit WordCount data words. Message bus: bus-1.
		S16BIT swMessageId_1; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_1 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create: unit - %i, message - %i, block - %i, rx_rt - %i, rx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_1, swDataBlockId_0, s_wRtAddress1, wRtSubaddress2, wWordCount, 0, s_wBus1);
		swResult = sitalBc_Message_CreateBcToRt (s_swDevice2, swMessageId_1, swDataBlockId_0, wRtAddress2, wRtSubaddress2, wWordCount, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) message-2, a RT(RT-Address-2/RtSubAddress2)-to-RT(RT-Address-1/RtSubAddress1) message that uses data-block-0 to receive OtherWordCount data words. Message bus: bus-1.
		// Note: This message is intentionally wrongly defined, what will fail the test unless the message will be fixed later on (see below).
		U16BIT wOtherWordCount; // Word count (1-32).
		wOtherWordCount = ((1 == wWordCount) ? 2 : 1); // A different word count.
		S16BIT swMessageId_2; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_2 = 2;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToRt.Create: unit - %i, message - %i, block - %i, tx_rt - %i, tx_rt_sa - %i, rx_rt - %i, rx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_2, swDataBlockId_0, wRtAddress1, wRtSubaddress2, wRtAddress2, wRtSubaddress1, wOtherWordCount, 0, s_wBus1);
		swResult = sitalBc_Message_CreateRtToRt (s_swDevice2, swMessageId_2, swDataBlockId_0, wRtAddress1, wRtSubaddress1, wOtherWordCount, wRtAddress2, wRtSubaddress2, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToRt.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Modify for device-2 (BC) message-2, into a RT(RT-Address-2/RtSubAddress2)-to-RT(RT-Address-1/RtSubAddress1) message that uses data-block-0 to receive WordCount data words. Message bus: bus-1.
		// Note: This message has been intentionally wrongly defined (see above), so it it therefore fixed here, and only if the fix actually succeeds will the test pass.
		swMessageId_2 = 2;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToRt.Modify: unit - %i, message - %i, block - %i, tx_rt - %i, tx_rt_sa - %i, rx_rt - %i, rx_rt_sa - %i, wc - %i, gap - %i, opts-x%X, modified - x%X\n", s_swDevice2, swMessageId_2, swDataBlockId_0, wRtAddress1, wRtSubaddress2, wRtAddress2, wRtSubaddress1, wWordCount, 0, s_wBus1, sitalBcMessageModifier_MESSAGE1_COMMAND1);
		swResult = sitalBc_Message_ModifyRtToRt (s_swDevice2, swMessageId_2, swDataBlockId_0, wRtAddress1, wRtSubaddress1, wWordCount, wRtAddress2, wRtSubaddress2, 0, s_wBus1, sitalBcMessageModifier_MESSAGE1_COMMAND1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToRt.Modify.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-0, an XQF command that executes message-0.
		S16BIT swCommandId_0; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-1, an XQF command that executes message-1.
		S16BIT swCommandId_1; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_1 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_1, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_1, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_1, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_1, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-2, an XQF command that executes message-2.
		S16BIT swCommandId_2; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_2 = 2;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_2, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_2, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_2, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_2, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-3, a subroutine call command that executes minor-frame-0. 
		S16BIT swCommandId_3; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_3 = 3;
		S16BIT swFrameId_0; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_3, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, swFrameId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_3, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, (U16BIT)swFrameId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-4, a subroutine call command that executes minor-frame-1. 
		S16BIT swCommandId_4; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_4 = 4;
		S16BIT swFrameId_1; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_1 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_4, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, swFrameId_1, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_4, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, (U16BIT)swFrameId_1, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-5, a subroutine call command that executes minor-frame-2. 
		S16BIT swCommandId_5; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_5 = 5;
		S16BIT swFrameId_2; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_2 = 2;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_5, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, swFrameId_2, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_5, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, (U16BIT)swFrameId_2, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-6, a subroutine call command that executes minor-frame-3. 
		S16BIT swCommandId_6; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_6 = 6;
		S16BIT swFrameId_3; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_3 = 3;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_6, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, swFrameId_3, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_6, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, (U16BIT)swFrameId_3, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) minor-frame-0, a frame that contains command-0 and command-1.
		S16BIT swaCommandIds[sitalBcCounter_COMMANDS]; // An array of command-IDs.
		swaCommandIds[0] = swCommandId_0;
		swaCommandIds[1] = swCommandId_1;
		U16BIT wCommandIdCount; // The number of command IDs in the frame (1-sitalBcCounter_FRAME_COMMANDS).
		wCommandIdCount = 2;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i, %i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_0, "minor", wCommandIdCount, swCommandId_0, swCommandId_1, (wFrameTime / 10), 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_0, sitalBcFrameType_MINOR, swaCommandIds, wCommandIdCount, wFrameTime, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) minor-frame-1, a frame that contains command-0 and command-2.
		swaCommandIds[0] = swCommandId_0;
		swaCommandIds[1] = swCommandId_2;
		wCommandIdCount = 2;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i, %i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_1, "minor", wCommandIdCount, swCommandId_0, swCommandId_2, (wFrameTime / 10), 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_1, sitalBcFrameType_MINOR, swaCommandIds, wCommandIdCount, wFrameTime, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) minor-frame-2, a frame that contains command-0 and command-1.
		swaCommandIds[0] = swCommandId_0;
		swaCommandIds[1] = swCommandId_1;
		wCommandIdCount = 2;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i, %i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_2, "minor", wCommandIdCount, swCommandId_0, swCommandId_1, (wFrameTime / 10), 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_2, sitalBcFrameType_MINOR, swaCommandIds, wCommandIdCount, wFrameTime, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) minor-frame-3, a frame that contains command-0.
		swaCommandIds[0] = swCommandId_0;
		wCommandIdCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_3, "minor", wCommandIdCount, swCommandId_0, (wFrameTime / 10), 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_3, sitalBcFrameType_MINOR, swaCommandIds, wCommandIdCount, wFrameTime, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) major-frame-4, a frame that contains command-3 and command-4 and command-5 and command-6 and whose frame time is FrameTime us.
		S16BIT swFrameId_4; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_4 = 4;
		swaCommandIds[0] = swCommandId_3;
		swaCommandIds[1] = swCommandId_4;
		swaCommandIds[2] = swCommandId_5;
		swaCommandIds[3] = swCommandId_6;
		wCommandIdCount = 4;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i, %i, %i, %i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_4, "major", wCommandIdCount, swCommandId_3, swCommandId_4, swCommandId_5, swCommandId_6, (wFrameTime / 10), 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_4, sitalBcFrameType_MAJOR, swaCommandIds, wCommandIdCount, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Use given message descriptors to code the [first] command word of above defined messages.
		// Note: Defined messages have been intentionally devised in a way that their [first] command word will differ.
		U16BIT wCommandWord0; // Command word for above defined RT-to-BC message-0.
		swResult = sitalBcStld1553_Command_GetCode (&wCommandWord0, wRtAddress1, sitalCommand_TX, wRtSubaddress1, wWordCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Command.GetCode.Error: error - %i\n", swResult);
			throw 0;
		}
		U16BIT wCommandWord1; // Command word for above defined BC-to-RT message-1.
		swResult = sitalBcStld1553_Command_GetCode (&wCommandWord1, wRtAddress2, sitalCommand_RX, wRtSubaddress2, wWordCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Command.GetCode.Error: error - %i\n", swResult);
			throw 0;
		}
		U16BIT wCommandWord2; // First command word for above defined RT-to-RT message-2.
		swResult = sitalBcStld1553_Command_GetCode (&wCommandWord2, wRtAddress1, sitalCommand_RX, wRtSubaddress1, wWordCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Command.GetCode.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Start device-2 (BC) running major-frame-4 for wFrameShotCount-shots.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", s_swDevice2, swFrameId_4, wFrameShotCount);
		swResult = sitalBc_Start (s_swDevice2, swFrameId_4, wFrameShotCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Read the computer's current tick counter into the global ISR frame run time measurer.
		s_ptMinorFrameRunTimeMeasurer.SetBaseTime ();

		/// Wait for all the frames to end, and in the meanwhile get all available messages from the device-3 (MT).
		PrecisionTimer ptSleepMeasurer; // A precision timer.
		ptSleepMeasurer.SetBaseTime ();
		bool bTimeEnded; // A flag that says whether the maximum time alloted for a certain operation has already elapsed.
		U32BIT dwOverallFrameRunTime; // The expected overall frame run time (in ms).
		dwOverallFrameRunTime = ((U32BIT)wFrameShotCount * wMinorFrameCount * (wFrameTime / 10));
		bTimeEnded = false;
		sitalDecodedMessageStructure dmsDecodedMessage; // A decoded message.
		U16BIT wRtToBcMessageCount; // The total number of RT-to-BC messages that have been identified.
		wRtToBcMessageCount = 0;
		U16BIT wBcToRtMessageCount; // The total number of BC-to-RT messages that have been identified.
		wBcToRtMessageCount = 0;
		U16BIT wRtToRtMessageCount; // The total number of RT-to-RT messages that have been identified.
		wRtToRtMessageCount = 0;
		while (!bTimeEnded)
		{
			ptSleepMeasurer.SetEndTime ();
			bTimeEnded = ( dwOverallFrameRunTime < ptSleepMeasurer.GetDuration () );
			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded: unit - %i, purge - %i\n", s_swDevice3, TRUE);

			do 
			{
				swResult = sitalMt_Message_GetFromStackDecoded (s_swDevice3, &dmsDecodedMessage, sitalMessageLocationAndRemoval_NEXT_PURGE, sitalMtStack_ACTIVE);
				if ((0 > swResult) || (1 < swResult))
				{
					sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Error: error - %i\n", swResult);
					throw 0;
				}
				if (1 == swResult)
				{
					sitalProcess_Log_PrintLine ("#StldTester.Mt.Message: unit - %i, command - %04X, data_count - %i, time_tag - %04X\n", s_swDevice3, dmsDecodedMessage.wCommandWord1, dmsDecodedMessage.wDataWordCount, dmsDecodedMessage.wTimeTag);
					if (wCommandWord0 == dmsDecodedMessage.wCommandWord1)
					{
						wRtToBcMessageCount++;
					}
					else
					if (wCommandWord1 == dmsDecodedMessage.wCommandWord1)
					{
						wBcToRtMessageCount++;
					}
					else
					if (wCommandWord2 == dmsDecodedMessage.wCommandWord1)
					{
						wRtToRtMessageCount++;
					}
					else
					{
						sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Message.Error: error - %04X\n", dmsDecodedMessage.wCommandWord1);
						swResult = sitalReturnCode_TEST_FAIL;
						throw 0;
					}
				}
			}
			while (1 == swResult);
		}

		sitalProcess_Log_PrintLine ("#StldTester.Wait: time - %ims\n", U32BIT(ptSleepMeasurer.GetDuration ()));
		sitalProcess_Log_PrintLine ("\tWAIT_FOR %i us\n", U32BIT(ptSleepMeasurer.GetDuration () * 1000));

		/// Stop device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop: unit - %i\n", s_swDevice2);
		swResult = sitalBc_Stop (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Stop device-3 (MT).
		sitalProcess_Log_PrintLine ("#StldTester.Mt.Stop: unit - %i\n", s_swDevice3);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Stop (s_swDevice3) : sitalMt_Stop (s_swDevice3) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Verify that device-3 (MT) has detected messages in the expected composition.
		sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.MessageComposition: unit - %i, RT2BC(%04X) - %i(expected-%i), BC2RT(%04X) - %i(expected-%i), RT2RT(%04X) - %i(expected-%i)\n", s_swDevice3, wCommandWord0, wRtToBcMessageCount, wMinorFrameCount, wCommandWord1, wBcToRtMessageCount, (wMinorFrameCount / 2), wCommandWord2, wRtToRtMessageCount, (wMinorFrameCount / 4));
		if (((wMinorFrameCount * wFrameShotCount) != wRtToBcMessageCount) || (((wMinorFrameCount / 2) * wFrameShotCount) != wBcToRtMessageCount) || (((wMinorFrameCount / 4) * wFrameShotCount) != wRtToRtMessageCount))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Error: error - %s\n", "unexpected_message_composition");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// Free device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice2);
		swResult = sitalDevice_Free (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-3 (MT).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice3);
		swResult = sitalDevice_Free (s_swDevice3);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		swResult = stldSyncEnd (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncEnd.Error: error - %i\n", swResult);
			throw 0;
		}
	}
	catch (int)
	{
		sitalDevice_Free (s_swDevice2);
		sitalDevice_Free (s_swDevice3);
	}

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_11d.End: test_succeeded - %c\n", ((sitalReturnCode_SUCCESS == swResult) ? 'Y' : 'N'));
	return swResult;
}



/// <summary>
/// Test the stld1553 library in the henceforth described scenario:
/// Participant[s]: RT1, BC2.
/// Configuration: All participating devices must be able to communicate with each other.
/// Short description: Acceptance Test Plan (ATP) test 461_11e - Selectable interrupt generation for ISR-1 Bit-00 End of Message.
/// Along test print suitable info and indications into the log.
///
/// Note:
/// - This function may use any of above defined Acceptance Test Plan (ATP) tests variables.
///   The caller must therefore call the suitable stldTester_Atp_Set* functions prior to calling this function.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Atp_DoTest461_11e_Isr1_Bit00 ()
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_11e_Isr1_Bit00.Start\n");

	/// Set test arguments.
	U16BIT wRtSubaddress; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31) that's set for a RT for Rx.
	wRtSubaddress = 1;
	U16BIT wWordCount; // Word count (1-32).
	wWordCount = 32;
	U16BIT wFrameTime; // The time (in units of 100 us) that the frame will take to complete.
	wFrameTime = 1000; // 100ms.

	S16BIT swResult; // Result of operation or function call.
	swResult = sitalReturnCode_SUCCESS;

	/// Get device count.
	U16BIT wRequiredDeviceCount; // The number of devices required for this test.
	wRequiredDeviceCount = (1 + max(s_swDevice1, s_swDevice2));
	U16BIT wAvailableDeviceCount; // The total number of devices available in the system.
	swResult = sitalDevice_GetCount (&wAvailableDeviceCount);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %i\n", swResult);
		return swResult;
	}
	sitalProcess_Log_PrintLine ("#StldTester.Device.Count: available - %i, required - %i\n", wAvailableDeviceCount, wRequiredDeviceCount);
	if (wAvailableDeviceCount < wRequiredDeviceCount)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %s\n", "not_enough_devices_installed");
		return sitalReturnCode_SUCCESS;
	}
	if ( (s_swDevice1 == s_swDevice2) )
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Number.Error: error - %s\n", "same_device_used_twice");
		return sitalReturnCode_INVALID_DEVICE_NUMBER;
	}

	try
	{
		swResult = stldSyncStart (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncStart.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-1 (RT) as a real RT in advanced mode.
		// Note: Advanced mode will enable r/w to the memory of device-1 (RT) while the device is running.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-rt, alloc_mem-none\n", s_swDevice1);
		swResult = sitalDevice_Initialize (s_swDevice1, sitalAccess_CARD, ((s_bIsRtMtModeTest ? sitalMode_RT_AND_MT : sitalMode_RT) | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Set device-1 (RT) address to RtAddress0.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set: unit - %i, address - %i\n", s_swDevice1, s_wRtAddress1);
		swResult = sitalRt_Address_Set (s_swDevice1, s_wRtAddress1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Legalize on device-1 (RT) all address-types in all directions for all subaddresses and all word counts.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.MessageLegality.Enable: unit - %i, address_type - %i, directions - %i, subaddress - %i, word_count - %08X\n", s_swDevice1, sitalRtAddressType_BOTH, sitalMessageDirection_BOTH, sitalRtSubaddress_ALL, 0xFFFFFFFF);
		swResult = sitalRt_MessageLegality_Enable (s_swDevice1, sitalRtAddressType_BOTH, sitalMessageDirection_BOTH, sitalRtSubaddress_ALL, 0xFFFFFFFF);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.MessageLegality.Enable.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Configure device-1 (RT) to issue desired interrupts.
		U16BIT wRegisterValue; // The current value of a target register, or the value to set for a target register.
		wRegisterValue = sitalInterruptMaskRegister1_END_OF_MESSAGE;
		sitalProcess_Log_PrintLine ("#StldTester.Device.Imr1.Set: unit - %i, imr1(post) - %04X\n", s_swDevice1, wRegisterValue);
		swResult = sitalDevice_Register_Write (s_swDevice1, sitalRegisterAddress_INTERRUPT_MASK_1, wRegisterValue);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Imr1.Set.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Get for device-1 (RT) the current interrupt configuration.
		U16BIT wInterruptMode; // Current configuration for type of interrupt signal (sitalIrqMode_*).
		U16BIT wAutoClear; // Current configuration for post-read auto-clearing of interrupt status (sitalIrqClear_*).
		swResult = sitalDevice_Irq_GetMode (s_swDevice1, &wInterruptMode, &wAutoClear);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Mode.Get.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Configure device-1 (RT) to issue suitable interrupts and perform post-read auto-clearing of interrupt status.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Configure: unit - %i, mode - %i, auto_clear - %i\n", s_swDevice1, wInterruptMode, sitalIrqClear_NO_AUTO_CLEAR);
		swResult = sitalDevice_Irq_Configure (s_swDevice1, wInterruptMode, sitalIrqClear_NO_AUTO_CLEAR);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Configure.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Configure device-1 (RT) to get desired notifications using an ISR that collects interrupt data, counts ISR calls, records the interrupt mask, and signals interested threads.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Manipulate: unit - %i, enable - %i, irq - %08X, callback - %s\n", s_swDevice1, TRUE, sitalInterruptRegister1_END_OF_MESSAGE, "stldTester_Rt_UserIsr");
		swResult = sitalDevice_Irq_Manipulate (s_swDevice1, TRUE, sitalInterruptRegister1_END_OF_MESSAGE, stldTester_Rt_UserIsr);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Set.Error: error - %i\n", swResult);
			throw 0;
		}
		s_dwRtIsrCallCounter = 0;
		s_dwRtIrqStatus = 0;

		/// Start device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Start: unit - %i\n", s_swDevice1);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Start (s_swDevice1) : sitalRt_Start (s_swDevice1) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-2 (BC) as a real BC.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-bc, alloc_mem-none\n", s_swDevice2);
		swResult = sitalDevice_Initialize (s_swDevice2, sitalAccess_CARD, (sitalMode_BC | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) data-block-0, a data block of 32 data words.
		S16BIT swDataBlockId_0; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId_0 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i\n", s_swDevice2, swDataBlockId_0, 32, 0);
		swResult = sitalBc_DataBlock_Create (s_swDevice2, swDataBlockId_0, sitalBcDataBlockSizeOption_SINGLE, NULL, wWordCount);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) message-0, a RT(RtAddress1/RtSubAddress)-to-BC message that uses data-block-0 to receive WordCount data words. Message bus: bus-1.
		S16BIT swMessageId_0; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBc.Create: unit - %i, message - %i, block - %i, tx_rt - %i, tx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddress, wWordCount, 0, s_wBus1);
		swResult = sitalBc_Message_CreateRtToBc (s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddress, wWordCount, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBc.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-0, a XEQ command that executes message-0.
		S16BIT swCommandId_0; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-1, a subroutine call command that executes minor-frame-0.
		S16BIT swCommandId_1; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_1 = 1;
		S16BIT swFrameId_0; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_1, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, swFrameId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_1, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, (U16BIT)swFrameId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) minor-frame-0, a frame that contains command-0.
		S16BIT swaCommandIds[sitalBcCounter_COMMANDS]; // An array of command-IDs.
		swaCommandIds[0] = swCommandId_0;
		U16BIT wCommandIdCount; // The number of command IDs in the frame (1-sitalBcCounter_FRAME_COMMANDS).
		wCommandIdCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_0, "minor", wCommandIdCount, swCommandId_0, 0, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_0, sitalBcFrameType_MINOR, swaCommandIds, wCommandIdCount, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) major-frame-1, a frame that contains command-1 and whose time is FrameTime.
		S16BIT swFrameId_1; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_1 = 1;
		swaCommandIds[0] = swCommandId_1;
		wCommandIdCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_1, "major", wCommandIdCount, swCommandId_1, wFrameTime, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_1, sitalBcFrameType_MAJOR, swaCommandIds, wCommandIdCount, wFrameTime, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Configure device-2 (BC) to issue desired interrupts.
		wRegisterValue = sitalInterruptMaskRegister1_END_OF_MESSAGE;
		sitalProcess_Log_PrintLine ("#StldTester.Device.Imr1.Set: unit - %i, imr1(post) - %04X\n", s_swDevice2, wRegisterValue);
		swResult = sitalDevice_Register_Write (s_swDevice2, sitalRegisterAddress_INTERRUPT_MASK_1, wRegisterValue);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Imr1.Set.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Get for device-2 (BC) the current interrupt configuration.
		swResult = sitalDevice_Irq_GetMode (s_swDevice2, &wInterruptMode, &wAutoClear);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Mode.Get.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Configure device-2 (BC) to issue suitable interrupts and perform post-read auto-clearing of interrupt status.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Configure: unit - %i, mode - %i, auto_clear - %i\n", s_swDevice2, wInterruptMode, sitalIrqClear_NO_AUTO_CLEAR);
		swResult = sitalDevice_Irq_Configure (s_swDevice2, wInterruptMode, sitalIrqClear_NO_AUTO_CLEAR);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Configure.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Configure device-2 (BC) to get desired notifications using an ISR that collects interrupt data, counts ISR calls, records the interrupt mask, and signals interested threads.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Manipulate: unit - %i, enable - %i, irq - %08X, callback - %s\n", s_swDevice2, TRUE, sitalInterruptRegister1_END_OF_MESSAGE, "stldTester_Bc_UserIsr");
		swResult = sitalDevice_Irq_Manipulate (s_swDevice2, TRUE, sitalInterruptRegister1_END_OF_MESSAGE, stldTester_Bc_UserIsr);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Set.Error: error - %i\n", swResult);
			throw 0;
		}
		s_dwBcIsrCallCounter = 0;
		s_dwBcIrqStatus = 0;

		///	Start device-2 (BC) running major-frame-1 for 1-shot.
		U16BIT wFrameCount; // The number of frames to run.
		wFrameCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", s_swDevice2, swFrameId_1, wFrameCount);
		swResult = sitalBc_Start (s_swDevice2, swFrameId_1, wFrameCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Wait for the frame run to end.
		U32BIT dwRemainingFrameRunTime; // The expected remaining frame run time (in ms).
		dwRemainingFrameRunTime = (wFrameCount * (wFrameTime / 10));
		swResult = stldTester_Bc_WaitForEndOfFrameRun (s_swDevice2, dwRemainingFrameRunTime, 0, NULL);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.WaitForEndOfFrameRun.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Stop device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop: unit - %i\n", s_swDevice2);
		swResult = sitalBc_Stop (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Stop device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Stop: unit - %i\n", s_swDevice1);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Stop (s_swDevice1) : sitalRt_Stop (s_swDevice1) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// As long a not all expected interrupts occurred AND the maximum possible interrupt latency still didn't pass:
		///		Wait.
		stldInterruptLatency_Wait (&s_dwBcIsrCallCounter, wFrameCount);

		/// Verify that a correct number of interrupts of a suitable type occurred.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Interrupts: unit - %i, count - %i, expected_count - %i, last_mask - %08X, expected_mask - %08X\n", s_swDevice2, s_dwBcIsrCallCounter, wFrameCount, s_dwBcIrqStatus, sitalInterruptRegister1_END_OF_MESSAGE);
		if (wFrameCount != s_dwBcIsrCallCounter)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Interrupt.CallCount.Error: error - %i\n", s_dwBcIsrCallCounter);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		if (0U == (sitalInterruptRegister1_END_OF_MESSAGE & s_dwBcIrqStatus))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Interrupt.Mask.Error: error - %08X\n", s_dwBcIrqStatus);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		sitalProcess_Log_PrintLine ("#StldTester.Device.Interrupts: unit - %i, count - %i, expected_count - %i, last_mask - %08X, expected_mask - %08X\n", s_swDevice1, s_dwRtIsrCallCounter, wFrameCount, s_dwRtIrqStatus, sitalInterruptRegister1_END_OF_MESSAGE);
		if (wFrameCount != s_dwRtIsrCallCounter)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Interrupt.CallCount.Error: error - %i\n", s_dwRtIsrCallCounter);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		if (0U == (sitalInterruptRegister1_END_OF_MESSAGE & s_dwRtIrqStatus))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Interrupt.Mask.Error: error - %08X\n", s_dwRtIrqStatus);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// Free device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice2);
		swResult = sitalDevice_Free (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice1);
		swResult = sitalDevice_Free (s_swDevice1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		swResult = stldSyncEnd (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncEnd.Error: error - %i\n", swResult);
			throw 0;
		}
	}
	catch (int)
	{
		sitalDevice_Free (s_swDevice2);
	}

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_11e_Isr1_Bit00.End: test_succeeded - %c\n", ((sitalReturnCode_SUCCESS == swResult) ? 'Y' : 'N'));
	return swResult;
}



/// <summary>
/// Test the stld1553 library in the henceforth described scenario:
/// Participant[s]: RT1, BC2.
/// Configuration: All participating devices must be able to communicate with each other.
/// Short description: Acceptance Test Plan (ATP) test 461_11e - Selectable interrupt generation for ISR-1 Bit-01 RT's status with status bit set high.
/// Along test print suitable info and indications into the log.
///
/// Note:
/// - This function may use any of above defined Acceptance Test Plan (ATP) tests variables.
///   The caller must therefore call the suitable stldTester_Atp_Set* functions prior to calling this function.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Atp_DoTest461_11e_Isr1_Bit01 ()
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_11e_Isr1_Bit01.Start\n");

	/// Set test arguments.
	U16BIT wRtSubaddress; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31) that's set for a RT for Rx.
	wRtSubaddress = 1;
	U16BIT wWordCount; // Word count (1-32).
	wWordCount = 32;
	U16BIT wFrameTime; // The time (in units of 100 us) that the frame will take to complete.
	wFrameTime = 1000; // 100ms.

	S16BIT swResult; // Result of operation or function call.
	swResult = sitalReturnCode_SUCCESS;

	/// Get device count.
	U16BIT wRequiredDeviceCount; // The number of devices required for this test.
	wRequiredDeviceCount = (1 + max(s_swDevice1, s_swDevice2));
	U16BIT wAvailableDeviceCount; // The total number of devices available in the system.
	swResult = sitalDevice_GetCount (&wAvailableDeviceCount);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %i\n", swResult);
		return swResult;
	}
	sitalProcess_Log_PrintLine ("#StldTester.Device.Count: available - %i, required - %i\n", wAvailableDeviceCount, wRequiredDeviceCount);
	if (wAvailableDeviceCount < wRequiredDeviceCount)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %s\n", "not_enough_devices_installed");
		return sitalReturnCode_SUCCESS;
	}
	if ( (s_swDevice1 == s_swDevice2) )
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Number.Error: error - %s\n", "same_device_used_twice");
		return sitalReturnCode_INVALID_DEVICE_NUMBER;
	}

	try
	{
		swResult = stldSyncStart (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncStart.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-1 (RT) as a real RT.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-rt, alloc_mem-none\n", s_swDevice1);
		swResult = sitalDevice_Initialize (s_swDevice1, sitalAccess_CARD, ((s_bIsRtMtModeTest ? sitalMode_RT_AND_MT : sitalMode_RT) | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Set device-1 (RT) address to RT-address-1.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set: unit - %i, address - %i\n", s_swDevice1, s_wRtAddress1);
		swResult = sitalRt_Address_Set (s_swDevice1, s_wRtAddress1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Set RT-flag for the response status bits of device-1 (RT).
		// Note: In regular (i.e., non-alternate) RT mode, this operation turns off the BUSY bit in configuration register #1.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.ResponseStatusBits.Set: unit - %i, mask - %04X\n", s_swDevice1, sitalConfigurationRegister1_RT_FLAG);
		swResult = sitalRt_ResponseStatusBits_Set (s_swDevice1, sitalConfigurationRegister1_RT_FLAG);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.ResponseStatusBits.Error: error - %i\n", swResult);
			throw 0;
		}

		/// 	Start device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Start: unit - %i\n", s_swDevice1);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Start (s_swDevice1) : sitalRt_Start (s_swDevice1) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-2 (BC) as a real BC.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-bc, alloc_mem-none\n", s_swDevice2);
		swResult = sitalDevice_Initialize (s_swDevice2, sitalAccess_CARD, (sitalMode_BC | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) data-block-0, a data block of 32 data words.
		S16BIT swDataBlockId_0; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId_0 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i\n", s_swDevice2, swDataBlockId_0, 32, 0);
		swResult = sitalBc_DataBlock_Create (s_swDevice2, swDataBlockId_0, sitalBcDataBlockSizeOption_SINGLE, NULL, wWordCount);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) message-0, a RT(RtAddress1/RtSubAddress)-to-BC message that uses data-block-0 to receive WordCount data words. Message bus: bus-1.
		S16BIT swMessageId_0; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBc.Create: unit - %i, message - %i, block - %i, tx_rt - %i, tx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddress, wWordCount, 0, s_wBus1);
		swResult = sitalBc_Message_CreateRtToBc (s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddress, wWordCount, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBc.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-0, a XEQ command that executes message-0.
		S16BIT swCommandId_0; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-1, a subroutine call command that executes minor-frame-0.
		S16BIT swCommandId_1; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_1 = 1;
		S16BIT swFrameId_0; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_1, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, swFrameId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_1, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, (U16BIT)swFrameId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) minor-frame-0, a frame that contains command-0.
		S16BIT swaCommandIds[sitalBcCounter_COMMANDS]; // An array of command-IDs.
		swaCommandIds[0] = swCommandId_0;
		U16BIT wCommandIdCount; // The number of command IDs in the frame (1-sitalBcCounter_FRAME_COMMANDS).
		wCommandIdCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_0, "minor", wCommandIdCount, swCommandId_0, 0, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_0, sitalBcFrameType_MINOR, swaCommandIds, wCommandIdCount, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) major-frame-1, a frame that contains command-1 and whose time is FrameTime.
		S16BIT swFrameId_1; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_1 = 1;
		swaCommandIds[0] = swCommandId_1;
		wCommandIdCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_1, "major", wCommandIdCount, swCommandId_1, wFrameTime, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_1, sitalBcFrameType_MAJOR, swaCommandIds, wCommandIdCount, wFrameTime, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Configure device-2 (BC) to issue desired interrupts.
		U16BIT wRegisterValue; // The current value of a target register, or the value to set for a target register.
		wRegisterValue = sitalInterruptMaskRegister1_BC_STATUS_SET;
		sitalProcess_Log_PrintLine ("#StldTester.Device.Imr1.Set: unit - %i, imr1(post) - %04X\n", s_swDevice2, wRegisterValue);
		swResult = sitalDevice_Register_Write (s_swDevice2, sitalRegisterAddress_INTERRUPT_MASK_1, wRegisterValue);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Imr1.Set.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Get for device-2 (BC) the current interrupt configuration.
		U16BIT wInterruptMode; // Current configuration for type of interrupt signal (sitalIrqMode_*).
		U16BIT wAutoClear; // Current configuration for post-read auto-clearing of interrupt status (sitalIrqClear_*).
		swResult = sitalDevice_Irq_GetMode (s_swDevice2, &wInterruptMode, &wAutoClear);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Mode.Get.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Configure device-2 (BC) to issue suitable interrupts and perform post-read auto-clearing of interrupt status.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Configure: unit - %i, mode - %i, auto_clear - %i\n", s_swDevice2, wInterruptMode, sitalIrqClear_NO_AUTO_CLEAR);
		swResult = sitalDevice_Irq_Configure (s_swDevice2, wInterruptMode, sitalIrqClear_NO_AUTO_CLEAR);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Configure.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Configure device-2 (BC) to get desired notifications using an ISR that collects interrupt data, counts ISR calls, records the interrupt mask, and signals interested threads.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Manipulate: unit - %i, enable - %i, irq - %08X, callback - %s\n", s_swDevice2, TRUE, sitalInterruptRegister1_END_OF_MESSAGE, "stldTester_Bc_UserIsr");
		swResult = sitalDevice_Irq_Manipulate (s_swDevice2, TRUE, sitalInterruptRegister1_BC_STATUS_SET, stldTester_Bc_UserIsr);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Set.Error: error - %i\n", swResult);
			throw 0;
		}
		s_dwBcIsrCallCounter = 0;
		s_dwBcIrqStatus = 0;

		///	Start device-2 (BC) running major-frame-1 for 1-shot.
		U16BIT wFrameCount; // The number of frames to run.
		wFrameCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", s_swDevice2, swFrameId_1, wFrameCount);
		swResult = sitalBc_Start (s_swDevice2, swFrameId_1, wFrameCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Wait for the frame run to end.
		U32BIT dwRemainingFrameRunTime; // The expected remaining frame run time (in ms).
		dwRemainingFrameRunTime = (wFrameCount * (wFrameTime / 10));
		swResult = stldTester_Bc_WaitForEndOfFrameRun (s_swDevice2, dwRemainingFrameRunTime, 0, NULL);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.WaitForEndOfFrameRun.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Stop device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop: unit - %i\n", s_swDevice2);
		swResult = sitalBc_Stop (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// As long a not all expected interrupts occurred AND the maximum possible interrupt latency still didn't pass:
		///		Wait.
		stldInterruptLatency_Wait (&s_dwBcIsrCallCounter, wFrameCount);

		/// Verify that a correct number of interrupts of a suitable type occurred.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Interrupts: unit - %i, count - %i, expected_count - %i, last_mask - %08X, expected_mask - %08X\n", s_swDevice2, s_dwBcIsrCallCounter, wFrameCount, s_dwBcIrqStatus, sitalInterruptRegister1_BC_STATUS_SET);
		if (wFrameCount != s_dwBcIsrCallCounter)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Interrupt.CallCount.Error: error - %i\n", s_dwBcIsrCallCounter);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		if (0U == (sitalInterruptRegister1_BC_STATUS_SET & s_dwBcIrqStatus))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Interrupt.Mask.Error: error - %08X\n", s_dwBcIrqStatus);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// Stop device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Stop: unit - %i\n", s_swDevice1);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Stop (s_swDevice1) : sitalRt_Stop (s_swDevice1) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice2);
		swResult = sitalDevice_Free (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice1);
		swResult = sitalDevice_Free (s_swDevice1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		swResult = stldSyncEnd (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncEnd.Error: error - %i\n", swResult);
			throw 0;
		}
	}
	catch (int)
	{
		sitalDevice_Free (s_swDevice1);
		sitalDevice_Free (s_swDevice2);
	}

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_11e_Isr1_Bit01.End: test_succeeded - %c\n", ((sitalReturnCode_SUCCESS == swResult) ? 'Y' : 'N'));
	return swResult;
}



/// <summary>
/// Test the stld1553 library in the henceforth described scenario:
/// Participant[s]: RT1, BC2.
/// Configuration: All participating devices must be able to communicate with each other.
/// Short description: Acceptance Test Plan (ATP) test 461_11e - Selectable interrupt generation for ISR-1 Bit-02 Response Time Out.
/// Along test print suitable info and indications into the log.
///
/// Note:
/// - This function may use any of above defined Acceptance Test Plan (ATP) tests variables.
///   The caller must therefore call the suitable stldTester_Atp_Set* functions prior to calling this function.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Atp_DoTest461_11e_Isr1_Bit02 ()
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_11e_Isr1_Bit02.Start\n");

	/// Set test arguments.
	U16BIT wRtSubaddress; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31) that's set for a RT for Rx.
	wRtSubaddress = 1;
	U16BIT wWordCount; // Word count (1-32).
	wWordCount = 32;
	U16BIT wFrameTime; // The time (in units of 100 us) that the frame will take to complete.
	wFrameTime = 1000; // 100ms.

	S16BIT swResult; // Result of operation or function call.
	swResult = sitalReturnCode_SUCCESS;

	/// Get device count.
	U16BIT wRequiredDeviceCount; // The number of devices required for this test.
	wRequiredDeviceCount = (1 + max(s_swDevice1, s_swDevice2));
	U16BIT wAvailableDeviceCount; // The total number of devices available in the system.
	swResult = sitalDevice_GetCount (&wAvailableDeviceCount);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %i\n", swResult);
		return swResult;
	}
	sitalProcess_Log_PrintLine ("#StldTester.Device.Count: available - %i, required - %i\n", wAvailableDeviceCount, wRequiredDeviceCount);
	if (wAvailableDeviceCount < wRequiredDeviceCount)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %s\n", "not_enough_devices_installed");
		return sitalReturnCode_SUCCESS;
	}
	if ( (s_swDevice1 == s_swDevice2) )
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Number.Error: error - %s\n", "same_device_used_twice");
		return sitalReturnCode_INVALID_DEVICE_NUMBER;
	}

	try
	{
		swResult = stldSyncStart (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncStart.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-1 (RT) as a real RT.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-rt, alloc_mem-none\n", s_swDevice1);
		swResult = sitalDevice_Initialize (s_swDevice1, sitalAccess_CARD, ((s_bIsRtMtModeTest ? sitalMode_RT_AND_MT : sitalMode_RT) | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Set device-1 (RT) address to RT-address-1.
		// Note: An RT with this address is initialized just in order to verify that a RT with this address does NOT already exist on the bus, and it isn't really started in order to force response timeout at the BC.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set: unit - %i, address - %i\n", s_swDevice1, s_wRtAddress1);
		swResult = sitalRt_Address_Set (s_swDevice1, s_wRtAddress1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-2 (BC) as a real BC.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-bc, alloc_mem-none\n", s_swDevice2);
		swResult = sitalDevice_Initialize (s_swDevice2, sitalAccess_CARD, (sitalMode_BC | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) data-block-0, a data block of 32 data words.
		S16BIT swDataBlockId_0; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId_0 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i\n", s_swDevice2, swDataBlockId_0, 32, 0);
		swResult = sitalBc_DataBlock_Create (s_swDevice2, swDataBlockId_0, sitalBcDataBlockSizeOption_SINGLE, NULL, wWordCount);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) message-0, a RT(RtAddress1/RtSubAddress)-to-BC message that uses data-block-0 to receive WordCount data words. Message bus: bus-1.
		S16BIT swMessageId_0; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBc.Create: unit - %i, message - %i, block - %i, tx_rt - %i, tx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddress, wWordCount, 0, s_wBus1);
		swResult = sitalBc_Message_CreateRtToBc (s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddress, wWordCount, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBc.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-0, a XEQ command that executes message-0.
		S16BIT swCommandId_0; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-1, a subroutine call command that executes minor-frame-0.
		S16BIT swCommandId_1; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_1 = 1;
		S16BIT swFrameId_0; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_1, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, swFrameId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_1, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, (U16BIT)swFrameId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) minor-frame-0, a frame that contains command-0.
		S16BIT swaCommandIds[sitalBcCounter_COMMANDS]; // An array of command-IDs.
		swaCommandIds[0] = swCommandId_0;
		U16BIT wCommandIdCount; // The number of command IDs in the frame (1-sitalBcCounter_FRAME_COMMANDS).
		wCommandIdCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_0, "minor", wCommandIdCount, swCommandId_0, 0, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_0, sitalBcFrameType_MINOR, swaCommandIds, wCommandIdCount, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) major-frame-1, a frame that contains command-1 and whose time is FrameTime.
		S16BIT swFrameId_1; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_1 = 1;
		swaCommandIds[0] = swCommandId_1;
		wCommandIdCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_1, "major", wCommandIdCount, swCommandId_1, wFrameTime, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_1, sitalBcFrameType_MAJOR, swaCommandIds, wCommandIdCount, wFrameTime, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Configure device-2 (BC) to issue desired interrupts.
		U16BIT wRegisterValue; // The current value of a target register, or the value to set for a target register.
		wRegisterValue = sitalInterruptMaskRegister1_FORMAT_ERROR;
		sitalProcess_Log_PrintLine ("#StldTester.Device.Imr1.Set: unit - %i, imr1(post) - %04X\n", s_swDevice2, wRegisterValue);
		swResult = sitalDevice_Register_Write (s_swDevice2, sitalRegisterAddress_INTERRUPT_MASK_1, wRegisterValue);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Imr1.Set.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Get for device-2 (BC) the current interrupt configuration.
		U16BIT wInterruptMode; // Current configuration for type of interrupt signal (sitalIrqMode_*).
		U16BIT wAutoClear; // Current configuration for post-read auto-clearing of interrupt status (sitalIrqClear_*).
		swResult = sitalDevice_Irq_GetMode (s_swDevice2, &wInterruptMode, &wAutoClear);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Mode.Get.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Configure device-2 (BC) to issue suitable interrupts and perform post-read auto-clearing of interrupt status.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Configure: unit - %i, mode - %i, auto_clear - %i\n", s_swDevice2, wInterruptMode, sitalIrqClear_NO_AUTO_CLEAR);
		swResult = sitalDevice_Irq_Configure (s_swDevice2, wInterruptMode, sitalIrqClear_NO_AUTO_CLEAR);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Configure.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Configure device-2 (BC) to get desired notifications using an ISR that collects interrupt data, counts ISR calls, records the interrupt mask, and signals interested threads.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Manipulate: unit - %i, enable - %i, irq - %08X, callback - %s\n", s_swDevice2, TRUE, sitalInterruptRegister1_FORMAT_ERROR, "stldTester_Bc_UserIsr");
		swResult = sitalDevice_Irq_Manipulate (s_swDevice2, TRUE, sitalInterruptRegister1_FORMAT_ERROR, stldTester_Bc_UserIsr);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Set.Error: error - %i\n", swResult);
			throw 0;
		}
		s_dwBcIsrCallCounter = 0;
		s_dwBcIrqStatus = 0;

		///	Start device-2 (BC) running major-frame-1 for 1-shot.
		U16BIT wFrameCount; // The number of frames to run.
		wFrameCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", s_swDevice2, swFrameId_1, wFrameCount);
		swResult = sitalBc_Start (s_swDevice2, swFrameId_1, wFrameCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Wait for the frame run to end.
		U32BIT dwRemainingFrameRunTime; // The expected remaining frame run time (in ms).
		dwRemainingFrameRunTime = (wFrameCount * (wFrameTime / 10));
		swResult = stldTester_Bc_WaitForEndOfFrameRun (s_swDevice2, dwRemainingFrameRunTime, 0, NULL);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.WaitForEndOfFrameRun.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Stop device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop: unit - %i\n", s_swDevice2);
		swResult = sitalBc_Stop (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// As long a not all expected interrupts occurred AND the maximum possible interrupt latency still didn't pass:
		///		Wait.
		stldInterruptLatency_Wait (&s_dwBcIsrCallCounter, wFrameCount);

		/// Verify that a correct number of interrupts of a suitable type occurred.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Interrupts: unit - %i, count - %i, expected_count - %i, last_mask - %08X, expected_mask - %08X\n", s_swDevice2, s_dwBcIsrCallCounter, wFrameCount, s_dwBcIrqStatus, sitalInterruptRegister1_FORMAT_ERROR);
		if (wFrameCount != s_dwBcIsrCallCounter)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Interrupt.CallCount.Error: error - %i\n", s_dwBcIsrCallCounter);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		if (0U == (sitalInterruptRegister1_FORMAT_ERROR & s_dwBcIrqStatus))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Interrupt.Mask.Error: error - %08X\n", s_dwBcIrqStatus);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// Free device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice2);
		swResult = sitalDevice_Free (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice1);
		swResult = sitalDevice_Free (s_swDevice1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		swResult = stldSyncEnd (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncEnd.Error: error - %i\n", swResult);
			throw 0;
		}
	}
	catch (int)
	{
		sitalDevice_Free (s_swDevice1);
		sitalDevice_Free (s_swDevice2);
	}

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_11e_Isr1_Bit02.End: test_succeeded - %c\n", ((sitalReturnCode_SUCCESS == swResult) ? 'Y' : 'N'));
	return swResult;
}



/// <summary>
/// Test the stld1553 library in the henceforth described scenario:
/// Participant[s]: BC2.
/// Configuration: All participating devices must be able to communicate with each other.
/// Short description: Acceptance Test Plan (ATP) test 461_11e - Selectable interrupt generation for ISR-1 Bit-03 BC Watchdog timeout occurred.
/// Along test print suitable info and indications into the log.
///
/// Note:
/// - This function may use any of above defined Acceptance Test Plan (ATP) tests variables.
///   The caller must therefore call the suitable stldTester_Atp_Set* functions prior to calling this function.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Atp_DoTest461_11e_Isr1_Bit03 ()
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_11e_Isr1_Bit03.Start\n");

	/// Set test arguments.
	U16BIT wRtSubaddress; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31) that's set for a RT for Rx.
	wRtSubaddress = 1;

	S16BIT swResult; // Result of operation or function call.
	swResult = sitalReturnCode_SUCCESS;

	/// Get device count.
	U16BIT wRequiredDeviceCount; // The number of devices required for this test.
	wRequiredDeviceCount = (1 + s_swDevice2);
	U16BIT wAvailableDeviceCount; // The total number of devices available in the system.
	swResult = sitalDevice_GetCount (&wAvailableDeviceCount);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %i\n", swResult);
		return swResult;
	}
	sitalProcess_Log_PrintLine ("#StldTester.Device.Count: available - %i, required - %i\n", wAvailableDeviceCount, wRequiredDeviceCount);
	if (wAvailableDeviceCount < wRequiredDeviceCount)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %s\n", "not_enough_devices_installed");
		return sitalReturnCode_SUCCESS;
	}

	try
	{
		swResult = stldSyncStart (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncStart.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-2 (BC) as a real BC.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-bc, alloc_mem-none\n", s_swDevice2);
		swResult = sitalDevice_Initialize (s_swDevice2, sitalAccess_CARD, (sitalMode_BC | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) data-block-0, a data block of 32 data words.
		S16BIT swDataBlockId_0; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId_0 = 1;
		U16BIT wWordCount; // Word count (1-32).
		wWordCount = 32; // See the remark for wFrameTime (below).
		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i\n", s_swDevice2, swDataBlockId_0, 32, 0);
		swResult = sitalBc_DataBlock_Create (s_swDevice2, swDataBlockId_0, sitalBcDataBlockSizeOption_SINGLE, NULL, wWordCount);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) message-0, a RT(RtAddress1/RtSubAddress)-to-BC message that uses data-block-0 to receive WordCount data words. Message bus: bus-1.
		S16BIT swMessageId_0; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBc.Create: unit - %i, message - %i, block - %i, tx_rt - %i, tx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddress, wWordCount, 0, s_wBus1);
		swResult = sitalBc_Message_CreateRtToBc (s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddress, wWordCount, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBc.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-0, a XEQ command that executes message-0.
		S16BIT swCommandId_0; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-1, a subroutine call command that executes minor-frame-0.
		S16BIT swCommandId_1; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_1 = 1;
		S16BIT swFrameId_0; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_1, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, swFrameId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_1, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, (U16BIT)swFrameId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) minor-frame-0, a frame that contains command-0.
		S16BIT swaCommandIds[sitalBcCounter_COMMANDS]; // An array of command-IDs.
		swaCommandIds[0] = swCommandId_0;
		U16BIT wCommandIdCount; // The number of command IDs in the frame (1-sitalBcCounter_FRAME_COMMANDS).
		wCommandIdCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_0, "minor", wCommandIdCount, swCommandId_0, 0, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_0, sitalBcFrameType_MINOR, swaCommandIds, wCommandIdCount, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) major-frame-1, a frame that contains command-1 and whose time is FrameTime.
		S16BIT swFrameId_1; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_1 = 1;
		swaCommandIds[0] = swCommandId_1;
		wCommandIdCount = 1;
		U16BIT wFrameTime; // The time (in units of 100 us) that the frame will take to complete.
		wFrameTime = 1; // 100us = 0.1ms, a timeout shorter than the time takes for a single BC-to-RT message of 32 data words.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_1, "major", wCommandIdCount, swCommandId_1, wFrameTime, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_1, sitalBcFrameType_MAJOR, swaCommandIds, wCommandIdCount, wFrameTime, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Configure device-2 (BC) to issue desired interrupts.
		U16BIT wRegisterValue; // The current value of a target register, or the value to set for a target register.
		wRegisterValue = sitalInterruptMaskRegister1_BC_END_OF_FRAME;
		sitalProcess_Log_PrintLine ("#StldTester.Device.Imr1.Set: unit - %i, imr1(post) - %04X\n", s_swDevice2, wRegisterValue);
		swResult = sitalDevice_Register_Write (s_swDevice2, sitalRegisterAddress_INTERRUPT_MASK_1, wRegisterValue);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Imr1.Set.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Get for device-2 (BC) the current interrupt configuration.
		U16BIT wInterruptMode; // Current configuration for type of interrupt signal (sitalIrqMode_*).
		U16BIT wAutoClear; // Current configuration for post-read auto-clearing of interrupt status (sitalIrqClear_*).
		swResult = sitalDevice_Irq_GetMode (s_swDevice2, &wInterruptMode, &wAutoClear);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Mode.Get.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Configure device-2 (BC) to issue suitable interrupts and perform post-read auto-clearing of interrupt status.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Configure: unit - %i, mode - %i, auto_clear - %i\n", s_swDevice2, wInterruptMode, sitalIrqClear_NO_AUTO_CLEAR);
		swResult = sitalDevice_Irq_Configure (s_swDevice2, wInterruptMode, sitalIrqClear_NO_AUTO_CLEAR);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Configure.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Configure device-2 (BC) to get desired notifications using an ISR that collects interrupt data, counts ISR calls, records the interrupt mask, and signals interested threads.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Manipulate: unit - %i, enable - %i, irq - %08X, callback - %s\n", s_swDevice2, TRUE, sitalInterruptRegister1_FORMAT_ERROR, "stldTester_Bc_UserIsr");
		swResult = sitalDevice_Irq_Manipulate (s_swDevice2, TRUE, sitalInterruptRegister1_BC_END_OF_FRAME, stldTester_Bc_UserIsr);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Set.Error: error - %i\n", swResult);
			throw 0;
		}
		s_dwBcIsrCallCounter = 0;
		s_dwBcIrqStatus = 0;

		///	Configure device-2 (BC) to have a watchdog timeout of 100 us.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.WatchdogTimeout: unit - %i, timeout - %c(%ius)\n", s_swDevice2, 'y', 100);
		swResult = sitalDevice_ConfigureWatchdogTimeout (s_swDevice2, TRUE, 1 /* 100 us */);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.WatchdogTimeout.Error: error - %i\n", swResult);
			throw 0;
		}

		///	Start device-2 (BC) running major-frame-1 for 1-shot.
		U16BIT wFrameCount; // The number of frames to run.
		wFrameCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", s_swDevice2, swFrameId_1, wFrameCount);
		swResult = sitalBc_Start (s_swDevice2, swFrameId_1, wFrameCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Wait for the frame run to end.
		U32BIT dwRemainingFrameRunTime; // The expected remaining frame run time (in ms).
		dwRemainingFrameRunTime = (wFrameCount * (wFrameTime / 10));
		swResult = stldTester_Bc_WaitForEndOfFrameRun (s_swDevice2, dwRemainingFrameRunTime, 0, NULL);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.WaitForEndOfFrameRun.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Stop device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop: unit - %i\n", s_swDevice2);
		swResult = sitalBc_Stop (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// As long a not all expected interrupts occurred AND the maximum possible interrupt latency still didn't pass:
		///		Wait.
		stldInterruptLatency_Wait (&s_dwBcIsrCallCounter, wFrameCount);

		/// Verify that a correct number of interrupts of a suitable type occurred.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Interrupts: unit - %i, count - %i, expected_count - %i, last_mask - %08X, expected_mask - %08X\n", s_swDevice2, s_dwBcIsrCallCounter, wFrameCount, s_dwBcIrqStatus, sitalInterruptRegister1_BC_END_OF_FRAME);
		if (wFrameCount != s_dwBcIsrCallCounter)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Interrupt.CallCount.Error: error - %i\n", s_dwBcIsrCallCounter);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		if (0U == (sitalInterruptRegister1_BC_END_OF_FRAME & s_dwBcIrqStatus))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Interrupt.Mask.Error: error - %08X\n", s_dwBcIrqStatus);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// Free device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice2);
		swResult = sitalDevice_Free (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		swResult = stldSyncEnd (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncEnd.Error: error - %i\n", swResult);
			throw 0;
		}
	}
	catch (int)
	{
		sitalDevice_Free (s_swDevice2);
	}

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_11e_Isr1_Bit03.End: test_succeeded - %c\n", ((sitalReturnCode_SUCCESS == swResult) ? 'Y' : 'N'));
	return swResult;
}



/// <summary>
/// Test the stld1553 library in the henceforth described scenario:
/// Participant[s]: RT1, BC2.
/// Configuration: All participating devices must be able to communicate with each other.
/// Short description: Acceptance Test Plan (ATP) test 461_11e - Selectable interrupt generation for ISR-1 Bit-04 Message was sent.
/// Along test print suitable info and indications into the log.
///
/// Note:
/// - This function may use any of above defined Acceptance Test Plan (ATP) tests variables.
///   The caller must therefore call the suitable stldTester_Atp_Set* functions prior to calling this function.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Atp_DoTest461_11e_Isr1_Bit04 ()
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_11e_Isr1_Bit04.Start\n");

	/// Set test arguments.
	U16BIT wRtSubaddress; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31) that's set for a RT for Rx.
	wRtSubaddress = 1;
	U16BIT wWordCount; // Word count (1-32).
	wWordCount = 32;
	U16BIT wFrameCount; // The number of frames to run.
	wFrameCount = 100;
	U16BIT wFrameTime; // The time (in units of 100 us) that the frame will take to complete.
	wFrameTime = 1000; // 100ms.

	S16BIT swResult; // Result of operation or function call.
	swResult = sitalReturnCode_SUCCESS;

	/// Get device count.
	U16BIT wRequiredDeviceCount; // The number of devices required for this test.
	wRequiredDeviceCount = (1 + max(s_swDevice1, s_swDevice2));
	U16BIT wAvailableDeviceCount; // The total number of devices available in the system.
	swResult = sitalDevice_GetCount (&wAvailableDeviceCount);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %i\n", swResult);
		return swResult;
	}
	sitalProcess_Log_PrintLine ("#StldTester.Device.Count: available - %i, required - %i\n", wAvailableDeviceCount, wRequiredDeviceCount);
	if (wAvailableDeviceCount < wRequiredDeviceCount)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %s\n", "not_enough_devices_installed");
		return sitalReturnCode_SUCCESS;
	}
	if ( (s_swDevice1 == s_swDevice2) )
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Number.Error: error - %s\n", "same_device_used_twice");
		return sitalReturnCode_INVALID_DEVICE_NUMBER;
	}

	try
	{
		swResult = stldSyncStart (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncStart.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-1 (RT) as a real RT in advanced mode.
		// Note: Advanced mode will enable r/w to the memory of device-1 (RT) while the device is running.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-rt, alloc_mem-none\n", s_swDevice1);
		swResult = sitalDevice_Initialize (s_swDevice1, sitalAccess_CARD, ((s_bIsRtMtModeTest ? sitalMode_RT_AND_MT : sitalMode_RT) | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Set device-1 (RT) address to RtAddress0.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set: unit - %i, address - %i\n", s_swDevice1, s_wRtAddress1);
		swResult = sitalRt_Address_Set (s_swDevice1, s_wRtAddress1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-1 (RT) data-block-0, a data block of 32 data words.
		S16BIT swRtDataBlockId_0; // A unique ID designating a data block (0-(sitalRtCounter_DATA_BLOCKS-1)).
		swRtDataBlockId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.Create: unit - %i, block - %i, type - %i, buf_size - %i\n", s_swDevice1, swRtDataBlockId_0, sitalRtDataBlockType_SINGLE_32, wWordCount);
		swResult = sitalRt_DataBlock_Create (s_swDevice1, swRtDataBlockId_0, sitalRtDataBlockType_SINGLE_32, NULL, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		///	Map for device-1 (RT) data-block-0 with Rx/Tx/Broadcast subaddress RtSubAddressRx, and legalize this subaddress.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.MapToSa: unit - %i, block - %i, sa - %i, msg_types - x%X, irq_opts-x%X, legalize - %c\n", s_swDevice1, swRtDataBlockId_0, wRtSubaddress, sitalRtMessageType_ALL, sitalRtDataBlockIrq_END_OF_MESSAGE, 'Y');
		swResult = sitalRt_DataBlock_MapToSubaddress (s_swDevice1, swRtDataBlockId_0, wRtSubaddress, sitalRtMessageType_ALL, sitalRtDataBlockIrq_END_OF_MESSAGE, TRUE);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.MapToSa.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Get for device-1 (RT) the current interrupt configuration.
		U16BIT wInterruptMode; // Current configuration for type of interrupt signal (sitalIrqMode_*).
		U16BIT wAutoClear; // Current configuration for post-read auto-clearing of interrupt status (sitalIrqClear_*).
		swResult = sitalDevice_Irq_GetMode (s_swDevice1, &wInterruptMode, &wAutoClear);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Mode.Get.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Configure device-1 (RT) to issue suitable interrupts and perform post-read auto-clearing of interrupt status.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Configure: unit - %i, mode - %i, auto_clear - %i\n", s_swDevice1, wInterruptMode, sitalIrqClear_NO_AUTO_CLEAR);
		swResult = sitalDevice_Irq_Configure (s_swDevice1, wInterruptMode, sitalIrqClear_NO_AUTO_CLEAR);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Configure.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Configure device-1 (RT) to get desired notifications using an ISR that collects interrupt data, counts ISR calls, records the interrupt mask, and signals interested threads.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Manipulate: unit - %i, enable - %i, irq - %08X, callback - %s\n", s_swDevice1, TRUE, sitalInterruptRegister1_RT_SUBADDRESS_END_OF_MESSAGE, "stldTester_Rt_UserIsr");
		swResult = sitalDevice_Irq_Manipulate (s_swDevice1, TRUE, sitalInterruptRegister1_RT_SUBADDRESS_END_OF_MESSAGE, stldTester_Rt_UserIsr);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Set.Error: error - %i\n", swResult);
			throw 0;
		}
		s_dwRtIsrCallCounter = 0;
		s_dwRtIrqStatus = 0;

		/// Start device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Start: unit - %i\n", s_swDevice1);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Start (s_swDevice1) : sitalRt_Start (s_swDevice1) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-2 (BC) as a real BC.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-bc, alloc_mem-none\n", s_swDevice2);
		swResult = sitalDevice_Initialize (s_swDevice2, sitalAccess_CARD, (sitalMode_BC | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) data-block-0, a data block of 32 data words.
		S16BIT swDataBlockId_0; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId_0 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i\n", s_swDevice2, swDataBlockId_0, 32, 0);
		swResult = sitalBc_DataBlock_Create (s_swDevice2, swDataBlockId_0, sitalBcDataBlockSizeOption_SINGLE, NULL, wWordCount);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) message-0, a RT(RtAddress1/RtSubAddress)-to-BC message that uses data-block-0 to receive WordCount data words and causes the issue of end-of-message interrupt. Message bus: bus-1.
		S16BIT swMessageId_0; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBc.Create: unit - %i, message - %i, block - %i, tx_rt - %i, tx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddress, wWordCount, 0, (sitalBcControlWord_END_OF_MESSAGE_INTERRUPT_ENABLED | s_wBus1));
		swResult = sitalBc_Message_CreateRtToBc (s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddress, wWordCount, 0, (sitalBcControlWord_END_OF_MESSAGE_INTERRUPT_ENABLED | s_wBus1));
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBc.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-0, a XEQ command that executes message-0.
		S16BIT swCommandId_0; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-1, a subroutine call command that executes minor-frame-0.
		S16BIT swCommandId_1; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_1 = 1;
		S16BIT swFrameId_0; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_1, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, swFrameId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_1, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, (U16BIT)swFrameId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) minor-frame-0, a frame that contains command-0.
		S16BIT swaCommandIds[sitalBcCounter_COMMANDS]; // An array of command-IDs.
		swaCommandIds[0] = swCommandId_0;
		U16BIT wCommandIdCount; // The number of command IDs in the frame (1-sitalBcCounter_FRAME_COMMANDS).
		wCommandIdCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_0, "minor", wCommandIdCount, swCommandId_0, 0, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_0, sitalBcFrameType_MINOR, swaCommandIds, wCommandIdCount, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) major-frame-1, a frame that contains command-1 and whose time is FrameTime.
		S16BIT swFrameId_1; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_1 = 1;
		swaCommandIds[0] = swCommandId_1;
		wCommandIdCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_1, "major", wCommandIdCount, swCommandId_1, wFrameTime, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_1, sitalBcFrameType_MAJOR, swaCommandIds, wCommandIdCount, wFrameTime, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Get for device-2 (BC) the current interrupt configuration.
		swResult = sitalDevice_Irq_GetMode (s_swDevice2, &wInterruptMode, &wAutoClear);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Mode.Get.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Configure device-2 (BC) to issue suitable interrupts and perform post-read auto-clearing of interrupt status.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Configure: unit - %i, mode - %i, auto_clear - %i\n", s_swDevice2, wInterruptMode, sitalIrqClear_NO_AUTO_CLEAR);
		swResult = sitalDevice_Irq_Configure (s_swDevice2, wInterruptMode, sitalIrqClear_NO_AUTO_CLEAR);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Configure.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Configure device-2 (BC) to get desired notifications using an ISR that collects interrupt data, counts ISR calls, records the interrupt mask, and signals interested threads.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Manipulate: unit - %i, enable - %i, irq - %08X, callback - %s\n", s_swDevice2, TRUE, sitalInterruptRegister1_BC_END_OF_MESSAGE, "stldTester_Bc_UserIsr");
		swResult = sitalDevice_Irq_Manipulate (s_swDevice2, TRUE, sitalInterruptRegister1_BC_END_OF_MESSAGE, stldTester_Bc_UserIsr);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Set.Error: error - %i\n", swResult);
			throw 0;
		}
		s_dwBcIsrCallCounter = 0;
		s_dwBcIrqStatus = 0;

		///	Start device-2 (BC) running major-frame-1 for 1-shot.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", s_swDevice2, swFrameId_1, wFrameCount);
		swResult = sitalBc_Start (s_swDevice2, swFrameId_1, wFrameCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Wait for the frame run to end.
		U32BIT dwRemainingFrameRunTime; // The expected remaining frame run time (in ms).
		dwRemainingFrameRunTime = (wFrameCount * (wFrameTime / 10));
		swResult = stldTester_Bc_WaitForEndOfFrameRun (s_swDevice2, dwRemainingFrameRunTime, 0, NULL);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.WaitForEndOfFrameRun.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Stop device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop: unit - %i\n", s_swDevice2);
		swResult = sitalBc_Stop (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// As long a not all expected interrupts occurred AND the maximum possible interrupt latency still didn't pass:
		///		Wait.
		stldInterruptLatency_Wait (&s_dwBcIsrCallCounter, wFrameCount);

		/// Verify that a correct number of interrupts of a suitable type occurred.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Interrupts: unit - %i, count - %i, expected_count - %i, last_mask - %08X, expected_mask - %08X\n", s_swDevice2, s_dwBcIsrCallCounter, wFrameCount, s_dwBcIrqStatus, sitalInterruptRegister1_BC_END_OF_MESSAGE);
		if (wFrameCount != s_dwBcIsrCallCounter)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Interrupt.CallCount.Error: error - %i\n", s_dwBcIsrCallCounter);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		if (0U == (sitalInterruptRegister1_BC_END_OF_MESSAGE & s_dwBcIrqStatus))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Interrupt.Mask.Error: error - %08X\n", s_dwBcIrqStatus);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// Free device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice2);
		swResult = sitalDevice_Free (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		swResult = stldSyncEnd (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncEnd.Error: error - %i\n", swResult);
			throw 0;
		}
	}
	catch (int)
	{
		sitalDevice_Free (s_swDevice2);
	}

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_11e_Isr1_Bit04.End: test_succeeded - %c\n", ((sitalReturnCode_SUCCESS == swResult) ? 'Y' : 'N'));
	return swResult;
}



/// <summary>
/// Test the stld1553 library in the henceforth described scenario:
/// Participant[s]: BC2.
/// Configuration: All participating devices must be able to communicate with each other.
/// Short description: Acceptance Test Plan (ATP) test 461_11e - Selectable interrupt generation for ISR-1 Bit-06 Time Tag Rollover.
/// Along test print suitable info and indications into the log.
///
/// Note:
/// - This function may use any of above defined Acceptance Test Plan (ATP) tests variables.
///   The caller must therefore call the suitable stldTester_Atp_Set* functions prior to calling this function.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Atp_DoTest461_11e_Isr1_Bit06 ()
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_11e_Isr1_Bit06.Start\n");

	S16BIT swResult; // Result of operation or function call.
	swResult = sitalReturnCode_SUCCESS;

	/// Get device count.
	U16BIT wRequiredDeviceCount; // The number of devices required for this test.
	wRequiredDeviceCount = (1 + s_swDevice2);
	U16BIT wAvailableDeviceCount; // The total number of devices available in the system.
	swResult = sitalDevice_GetCount (&wAvailableDeviceCount);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %i\n", swResult);
		return swResult;
	}
	sitalProcess_Log_PrintLine ("#StldTester.Device.Count: available - %i, required - %i\n", wAvailableDeviceCount, wRequiredDeviceCount);
	if (wAvailableDeviceCount < wRequiredDeviceCount)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %s\n", "not_enough_devices_installed");
		return sitalReturnCode_SUCCESS;
	}

	try
	{
		swResult = stldSyncStart (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncStart.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-2 (BC) as a real BC.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-bc, alloc_mem-none\n", s_swDevice2);
		swResult = sitalDevice_Initialize (s_swDevice2, sitalAccess_CARD, (sitalMode_BC | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Configure device-2 (BC) to issue desired interrupts.
		// Note: Here function sitalDevice_Register_Write may not be used, and function sitalDevice_AccessMemory must be used instead in order to directly update the target register in the device. That's because the target device isn't turned eventually into state RUN, so that the image of the registers will never be written into the target device.
		U16BIT wRegisterValue; // The current value of a target register, or the value to set for a target register.
		wRegisterValue = sitalInterruptMaskRegister1_TIME_TAG_ROLLOVER;
		sitalProcess_Log_PrintLine ("#StldTester.Device.Imr1.Set: unit - %i, imr1(post) - %04X\n", s_swDevice2, wRegisterValue);
		swResult = sitalDevice_AccessMemory (s_swDevice2, sitalDeviceAccessOperation_Write, sitalRegisterAddress_INTERRUPT_MASK_1, sitalDeviceMemorySection_Registers, 1, &wRegisterValue);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Imr1.Set.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Get for device-2 (BC) the current interrupt configuration.
		U16BIT wInterruptMode; // Current configuration for type of interrupt signal (sitalIrqMode_*).
		U16BIT wAutoClear; // Current configuration for post-read auto-clearing of interrupt status (sitalIrqClear_*).
		swResult = sitalDevice_Irq_GetMode (s_swDevice2, &wInterruptMode, &wAutoClear);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Mode.Get.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Configure device-2 (BC) to issue suitable interrupts and perform post-read auto-clearing of interrupt status.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Configure: unit - %i, mode - %i, auto_clear - %i\n", s_swDevice2, wInterruptMode, sitalIrqClear_NO_AUTO_CLEAR);
		swResult = sitalDevice_Irq_Configure (s_swDevice2, wInterruptMode, sitalIrqClear_NO_AUTO_CLEAR);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Configure.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Force the actual configuration of device-2 (BC).
		// Note: That's required in cases where no frame run takes place, so that configured registers aren't automatically written to the device by library stld1553.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Register.UpdateDevice: unit - %i, register - %i\n", s_swDevice2, sitalRegisterAddress_CONFIGURATION_2);
		swResult = sitalDevice_Register_UpdateDevice (s_swDevice2, sitalRegisterAddress_CONFIGURATION_2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Register.UpdateDevice.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Configure device-2 (BC) to get desired notifications using an ISR that collects interrupt data, counts ISR calls, records the interrupt mask, and signals interested threads.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Manipulate: unit - %i, enable - %i, irq - %08X, callback - %s\n", s_swDevice2, TRUE, sitalInterruptRegister1_TIME_TAG_ROLLOVER, "stldTester_Bc_UserIsr");
		swResult = sitalDevice_Irq_Manipulate (s_swDevice2, TRUE, sitalInterruptRegister1_TIME_TAG_ROLLOVER, stldTester_Bc_UserIsr);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Set.Error: error - %i\n", swResult);
			throw 0;
		}
		s_dwBcIsrCallCounter = 0;
		s_dwBcIrqStatus = 0;

		/// Set for device-2 (BC) the time tag to FFFFh.
		U16BIT wWrittenTimeTag; // Device time tag.
		wWrittenTimeTag = 0xFFFF;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.TimeTag.Set: unit - %i, value - %04X\n", s_swDevice2, wWrittenTimeTag);
		swResult = sitalDevice_TimeTag_Set (s_swDevice2, wWrittenTimeTag);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.TimeTag.Set.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Wait for the expected interrupt to be issued.
		U32BIT dwSleepTime; // Sleeping period (in ms).
		dwSleepTime = 1000; // A time that's much longer than really required is used in order to verify that no additional (an unexpected) interrupts occur.
		sitalProcess_Log_PrintLine ("#StldTester.Wait: time - %ims\n", dwSleepTime);
		sitalProcess_Log_PrintLine ("\tWAIT_FOR %i us\n", (dwSleepTime * 1000));
		PrecisionTimer pt; // A precision timer.
		pt.AccuratelySleep (dwSleepTime);

		/// As long a not all expected interrupts occurred AND the maximum possible interrupt latency still didn't pass:
		///		Wait.
		stldInterruptLatency_Wait (&s_dwBcIsrCallCounter, 1);

		/// Verify that a correct number of interrupts of a suitable type occurred.
		// Note: Some time tag interrupts must have occurred, but the exact number of depends on the sleeping time, the time tag resolution, and on some random component.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Interrupts: unit - %i, count - %i, expected_count - %i, last_mask - %08X, expected_mask - %08X\n", s_swDevice2, s_dwBcIsrCallCounter, 1, s_dwBcIrqStatus, sitalInterruptRegister1_TIME_TAG_ROLLOVER);
		if (0U == s_dwBcIsrCallCounter)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Interrupt.CallCount.Error: error - %i\n", s_dwBcIsrCallCounter);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		if (0U == (sitalInterruptRegister1_TIME_TAG_ROLLOVER & s_dwBcIrqStatus))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Interrupt.Mask.Error: error - %08X\n", s_dwBcIrqStatus);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// Free device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice2);
		swResult = sitalDevice_Free (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		swResult = stldSyncEnd (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncEnd.Error: error - %i\n", swResult);
			throw 0;
		}
	}
	catch (int)
	{
		sitalDevice_Free (s_swDevice2);
	}

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_11e_Isr1_Bit06.End: test_succeeded - %c\n", ((sitalReturnCode_SUCCESS == swResult) ? 'Y' : 'N'));
	return swResult;
}



/// <summary>
/// Test the stld1553 library in the henceforth described scenario:
/// Participant[s]: RT1.
/// Configuration: All participating devices must be able to communicate with each other.
/// Short description: Acceptance Test Plan (ATP) test 461_11e - Selectable interrupt generation for ISR-1 Bit-07 RT Address parity error found.
/// Along test print suitable info and indications into the log.
///
/// Note:
/// - This function may use any of above defined Acceptance Test Plan (ATP) tests variables.
///   The caller must therefore call the suitable stldTester_Atp_Set* functions prior to calling this function.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Atp_DoTest461_11e_Isr1_Bit07 ()
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_11e_Isr1_Bit07.Start\n");

	S16BIT swResult; // Result of operation or function call.
	swResult = sitalReturnCode_SUCCESS;

	/// Get device count.
	U16BIT wRequiredDeviceCount; // The number of devices required for this test.
	wRequiredDeviceCount = (1 + s_swDevice1);
	U16BIT wAvailableDeviceCount; // The total number of devices available in the system.
	swResult = sitalDevice_GetCount (&wAvailableDeviceCount);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %i\n", swResult);
		return swResult;
	}
	sitalProcess_Log_PrintLine ("#StldTester.Device.Count: available - %i, required - %i\n", wAvailableDeviceCount, wRequiredDeviceCount);
	if (wAvailableDeviceCount < wRequiredDeviceCount)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %s\n", "not_enough_devices_installed");
		return sitalReturnCode_SUCCESS;
	}

	try
	{
		swResult = stldSyncStart (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncStart.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-1 (RT) as a real RT.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-rt, alloc_mem-none\n", s_swDevice1);
		swResult = sitalDevice_Initialize (s_swDevice1, sitalAccess_CARD, ((s_bIsRtMtModeTest ? sitalMode_RT_AND_MT : sitalMode_RT) | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Verify for device-1 (RT) that its firmware version is >= 0x066*.
		// Note: This test fails with older versions.
		U16BIT wRegisterValue; // The current value of a target register, or the value to set for a target register.
		wRegisterValue = sitalDevice_Register_Read (s_swDevice1, sitalRegisterAddress_SITAL_FIRMWARE_VERSION);
		sitalProcess_Log_PrintLine ("#StldTester.Device.Firmaware: unit - %i, version - %04X\n", s_swDevice1, wRegisterValue);
		if (0x0660 > (0x0FF0 & wRegisterValue))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Firmaware.Error: error - %04X\n", 0x0660);
			swResult = sitalReturnCode_TEST_FAIL;
			return sitalReturnCode_SUCCESS;
		}

		/// Set device-1 (RT) address to RT-address-0.
		// Note: This test uses a predefined (and illegal) combination of RT-address and parity (address-0, odd-parity-0: see the next configuration below), not s_wRtAddress1 which may be dynamically set by the user app.
		U16BIT wRtAddress = sitalRtAddress_0; ///< RT address (sitalRtAddress_0-sitalRtAddress_31).
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set: unit - %i, address - %i\n", s_swDevice1, wRtAddress);
		swResult = sitalRt_Address_Set (s_swDevice1, wRtAddress);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Configure device-1 (RT) to have the wrong odd address parity LS bit.
		wRegisterValue = sitalDevice_Register_Read (s_swDevice1, sitalRegisterAddress_CONFIGURATION_5);
		sitalProcess_Log_PrintLine ("#StldTester.Device.SetBadOddAddressParityBit: unit - %i, configuration_register5(pre) - %04X, configuration_register5(post) - %04X\n", s_swDevice1, wRegisterValue, ((~sitalConfigurationRegister5_RT_ADDRESS_PARITY) & wRegisterValue));
		swResult = sitalDevice_Register_Write (s_swDevice1, sitalRegisterAddress_CONFIGURATION_5, ((~sitalConfigurationRegister5_RT_ADDRESS_PARITY) & wRegisterValue));
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.SetFrameStopOnError.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Configure device-1 (RT) to issue desired interrupts.
		wRegisterValue = sitalInterruptRegister1_RT_ADDRESS_PARITY_ERROR;
		sitalProcess_Log_PrintLine ("#StldTester.Device.Imr1.Set: unit - %i, imr1(post) - %04X\n", s_swDevice1, wRegisterValue);
		swResult = sitalDevice_Register_Write (s_swDevice1, sitalRegisterAddress_INTERRUPT_MASK_1, wRegisterValue);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Imr1.Set.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Get for device-1 (RT) the current interrupt configuration.
		U16BIT wInterruptMode; // Current configuration for type of interrupt signal (sitalIrqMode_*).
		U16BIT wAutoClear; // Current configuration for post-read auto-clearing of interrupt status (sitalIrqClear_*).
		swResult = sitalDevice_Irq_GetMode (s_swDevice1, &wInterruptMode, &wAutoClear);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Mode.Get.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Configure device-1 (RT) to issue suitable interrupts and perform post-read auto-clearing of interrupt status.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Configure: unit - %i, mode - %i, auto_clear - %i\n", s_swDevice1, wInterruptMode, sitalIrqClear_NO_AUTO_CLEAR);
		swResult = sitalDevice_Irq_Configure (s_swDevice1, wInterruptMode, sitalIrqClear_NO_AUTO_CLEAR);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Configure.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Configure device-1 (RT) to get desired notifications using an ISR that collects interrupt data, counts ISR calls, records the interrupt mask, and signals interested threads.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Manipulate: unit - %i, enable - %i, irq - %08X, callback - %s\n", s_swDevice1, TRUE, sitalInterruptRegister1_RT_ADDRESS_PARITY_ERROR, "stldTester_RT_UserIsr");
		swResult = sitalDevice_Irq_Manipulate (s_swDevice1, TRUE, sitalInterruptRegister1_RT_ADDRESS_PARITY_ERROR, stldTester_Rt_UserIsr);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Set.Error: error - %i\n", swResult);
			throw 0;
		}
		s_dwRtIsrCallCounter = 0;
		s_dwRtIrqStatus = 0;

		/// Start device-1 (RT).
		//wRegisterValue = sitalDevice_Register_Read (s_swDevice1, sitalRegisterAddress_INTERRUPT_STATUS_1);
		//sitalProcess_Log_PrintLine ("#StldTester.Device.Register.ISR1.Read.PreStart: unit - %i, value - %04X\n", s_swDevice1, wRegisterValue);
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Start: unit - %i\n", s_swDevice1);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Start (s_swDevice1) : sitalRt_Start (s_swDevice1) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Start.Error: error - %i\n", swResult);
			throw 0;
		}
		//wRegisterValue = sitalDevice_Register_Read (s_swDevice1, sitalRegisterAddress_INTERRUPT_STATUS_1);
		//sitalProcess_Log_PrintLine ("#StldTester.Device.Register.ISR1.Read.PostStart: unit - %i, value - %04X\n", s_swDevice1, wRegisterValue);

		/// As long as not all expected interrupts occurred AND the maximum possible interrupt latency still didn't pass:
		///		Wait.
		stldInterruptLatency_Wait (&s_dwRtIsrCallCounter, 1);

		/// Verify that a correct number of interrupts of a suitable type occurred.
		// Note: In this case, the exact number of interrupts isn't a deterministic one, all we know is that at least one interrupt should be issued.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Interrupts: unit - %i, count - %i, minimum_expected_count - %i, last_mask - %08X, expected_mask - %08X\n", s_swDevice1, s_dwRtIsrCallCounter, 1, s_dwRtIrqStatus, (sitalInterruptStatusRegister1_MASTER_INTERRUPT | sitalInterruptRegister1_RT_ADDRESS_PARITY_ERROR));
		if (0U == s_dwRtIsrCallCounter)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Interrupt.CallCount.Error: error - %i\n", s_dwRtIsrCallCounter);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		if (0U == (sitalInterruptRegister1_RT_ADDRESS_PARITY_ERROR & s_dwRtIrqStatus))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Interrupt.Mask.Error: error - %08X\n", s_dwRtIrqStatus);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		if (0U == (sitalInterruptStatusRegister1_MASTER_INTERRUPT & s_dwRtIrqStatus))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Interrupt.Mask.Error: error - %08X\n", s_dwRtIrqStatus);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// Stop device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Stop: unit - %i\n", s_swDevice1);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Stop (s_swDevice1) : sitalRt_Stop (s_swDevice1) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Wait between the tests.
		U32BIT dwSleepTime; // Sleeping period (in ms).
		dwSleepTime = 1000;
		sitalProcess_Log_PrintLine ("#StldTester.Wait: time - %ims\n", dwSleepTime);
		sitalProcess_Log_PrintLine ("\tWAIT_FOR %i us\n", (dwSleepTime * 1000));
		PrecisionTimer pt; // A precision timer.
		pt.AccuratelySleep (dwSleepTime);

		/// Set device-1 (RT) address to RT-address-0.
		// Note: Here a legal combination of RT-address and parity is set.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set: unit - %i, address - %i\n", s_swDevice1, wRtAddress);
		swResult = sitalRt_Address_Set (s_swDevice1, wRtAddress);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set.Error: error - %i\n", swResult);
			throw 0;
		}

		s_dwRtIsrCallCounter = 0;
		s_dwRtIrqStatus = 0;

		/// Start device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Start: unit - %i\n", s_swDevice1);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Start (s_swDevice1) : sitalRt_Start (s_swDevice1) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// As long a not all expected interrupts occurred AND the maximum possible interrupt latency still didn't pass:
		///		Wait.
		stldInterruptLatency_Wait (&s_dwRtIsrCallCounter, 1);

		/// Verify that a correct number of interrupts of a suitable type occurred.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Interrupts: unit - %i, count - %i, expected_count - %i, last_mask - %08X, expected_mask - %08X\n", s_swDevice1, s_dwRtIsrCallCounter, 0, s_dwRtIrqStatus, 0);
		if (0U != s_dwRtIsrCallCounter)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Interrupt.CallCount.Error: error - %i\n", s_dwRtIsrCallCounter);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// Stop device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Stop: unit - %i\n", s_swDevice1);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Stop (s_swDevice1) : sitalRt_Stop (s_swDevice1) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice1);
		swResult = sitalDevice_Free (s_swDevice1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		swResult = stldSyncEnd (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncEnd.Error: error - %i\n", swResult);
			throw 0;
		}
	}
	catch (int)
	{
		sitalDevice_Free (s_swDevice1);
	}

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_11e_Isr1_Bit07.End: test_succeeded - %c\n", ((sitalReturnCode_SUCCESS == swResult) ? 'Y' : 'N'));
	return swResult;
}



/// <summary>
/// Test the stld1553 library in the henceforth described scenario:
/// Participant[s]: BC2.
/// Configuration: All participating devices must be able to communicate with each other.
/// Short description: Acceptance Test Plan (ATP) test 461_11e - Selectable interrupt generation for ISR-1 Bit-08 BC retry following incorrect/no RT response.
/// Along test print suitable info and indications into the log.
///
/// Note:
/// - This function may use any of above defined Acceptance Test Plan (ATP) tests variables.
///   The caller must therefore call the suitable stldTester_Atp_Set* functions prior to calling this function.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Atp_DoTest461_11e_Isr1_Bit08 ()
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_11e_Isr1_Bit08.Start\n");

	/// Set test arguments.
	U16BIT wRtSubaddress; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31) that's set for a RT for Rx.
	wRtSubaddress = 1;
	U16BIT wWordCount; // Word count (1-32).
	wWordCount = 32;
	U16BIT wFrameTime; // The time (in units of 100 us) that the frame will take to complete.
	wFrameTime = 1000; // 100ms.

	S16BIT swResult; // Result of operation or function call.
	swResult = sitalReturnCode_SUCCESS;

	/// Get device count.
	U16BIT wRequiredDeviceCount; // The number of devices required for this test.
	wRequiredDeviceCount = (1 + s_swDevice2);
	U16BIT wAvailableDeviceCount; // The total number of devices available in the system.
	swResult = sitalDevice_GetCount (&wAvailableDeviceCount);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %i\n", swResult);
		return swResult;
	}
	sitalProcess_Log_PrintLine ("#StldTester.Device.Count: available - %i, required - %i\n", wAvailableDeviceCount, wRequiredDeviceCount);
	if (wAvailableDeviceCount < wRequiredDeviceCount)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %s\n", "not_enough_devices_installed");
		return sitalReturnCode_SUCCESS;
	}

	try
	{
		swResult = stldSyncStart (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncStart.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-2 (BC) as a real BC.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-bc, alloc_mem-none\n", s_swDevice2);
		swResult = sitalDevice_Initialize (s_swDevice2, sitalAccess_CARD, (sitalMode_BC | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) data-block-0, a data block of 32 data words.
		S16BIT swDataBlockId_0; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId_0 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i\n", s_swDevice2, swDataBlockId_0, 32, 0);
		swResult = sitalBc_DataBlock_Create (s_swDevice2, swDataBlockId_0, sitalBcDataBlockSizeOption_SINGLE, NULL, wWordCount);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) message-0, a RT(RtAddress1/RtSubAddress)-to-BC message that uses data-block-0 to receive WordCount data words and causes the issue of end-of-message interrupt. Message bus: bus-1.
		S16BIT swMessageId_0; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBc.Create: unit - %i, message - %i, block - %i, tx_rt - %i, tx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddress, wWordCount, 0, (sitalBcControlWord_RETRY_ENABLED | s_wBus1));
		swResult = sitalBc_Message_CreateRtToBc (s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddress, wWordCount, 0, (sitalBcControlWord_RETRY_ENABLED | s_wBus1));
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBc.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-0, a XEQ command that executes message-0.
		S16BIT swCommandId_0; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-1, a subroutine call command that executes minor-frame-0.
		S16BIT swCommandId_1; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_1 = 1;
		S16BIT swFrameId_0; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_1, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, swFrameId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_1, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, (U16BIT)swFrameId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) minor-frame-0, a frame that contains command-0.
		S16BIT swaCommandIds[sitalBcCounter_COMMANDS]; // An array of command-IDs.
		swaCommandIds[0] = swCommandId_0;
		U16BIT wCommandIdCount; // The number of command IDs in the frame (1-sitalBcCounter_FRAME_COMMANDS).
		wCommandIdCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_0, "minor", wCommandIdCount, swCommandId_0, 0, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_0, sitalBcFrameType_MINOR, swaCommandIds, wCommandIdCount, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) major-frame-1, a frame that contains command-1 and whose time is FrameTime.
		S16BIT swFrameId_1; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_1 = 1;
		swaCommandIds[0] = swCommandId_1;
		wCommandIdCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_1, "major", wCommandIdCount, swCommandId_1, wFrameTime, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_1, sitalBcFrameType_MAJOR, swaCommandIds, wCommandIdCount, wFrameTime, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Configure device-2 (BC) to retry badly responded messages.
		U16BIT wRegisterValue; // The current value of a target register, or the value to set for a target register.
		wRegisterValue = sitalDevice_Register_Read (s_swDevice2, sitalRegisterAddress_CONFIGURATION_1);
		sitalProcess_Log_PrintLine ("#StldTester.Device.SetFrameStopOnError: unit - %i, configuration_register1(pre) - %04X, configuration_register1(post) - %04X\n", s_swDevice2, wRegisterValue, (sitalConfigurationRegister1_BC_RETRY_ENABLED | wRegisterValue));
		swResult = sitalDevice_Register_Write (s_swDevice2, sitalRegisterAddress_CONFIGURATION_1, (sitalConfigurationRegister1_BC_RETRY_ENABLED | wRegisterValue));
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.SetFrameStopOnError.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Configure device-2 (BC) to issue desired interrupts.
		wRegisterValue = sitalInterruptMaskRegister1_BC_RETRY;
		sitalProcess_Log_PrintLine ("#StldTester.Device.Imr1.Set: unit - %i, imr1(post) - %04X\n", s_swDevice2, wRegisterValue);
		swResult = sitalDevice_Register_Write (s_swDevice2, sitalRegisterAddress_INTERRUPT_MASK_1, wRegisterValue);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Imr1.Set.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Get for device-2 (BC) the current interrupt configuration.
		U16BIT wInterruptMode; // Current configuration for type of interrupt signal (sitalIrqMode_*).
		U16BIT wAutoClear; // Current configuration for post-read auto-clearing of interrupt status (sitalIrqClear_*).
		swResult = sitalDevice_Irq_GetMode (s_swDevice2, &wInterruptMode, &wAutoClear);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Mode.Get.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Configure device-2 (BC) to issue suitable interrupts and perform post-read auto-clearing of interrupt status.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Configure: unit - %i, mode - %i, auto_clear - %i\n", s_swDevice2, wInterruptMode, sitalIrqClear_NO_AUTO_CLEAR);
		swResult = sitalDevice_Irq_Configure (s_swDevice2, wInterruptMode, sitalIrqClear_NO_AUTO_CLEAR);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Configure.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Configure device-2 (BC) to get desired notifications using an ISR that collects interrupt data, counts ISR calls, records the interrupt mask, and signals interested threads.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Manipulate: unit - %i, enable - %i, irq - %08X, callback - %s\n", s_swDevice2, TRUE, sitalInterruptRegister1_BC_RETRY, "stldTester_Bc_UserIsr");
		swResult = sitalDevice_Irq_Manipulate (s_swDevice2, TRUE, sitalInterruptRegister1_BC_RETRY, stldTester_Bc_UserIsr);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Set.Error: error - %i\n", swResult);
			throw 0;
		}
		s_dwBcIsrCallCounter = 0;
		s_dwBcIrqStatus = 0;

		///	Start device-2 (BC) running major-frame-1 for 1-shot.
		U16BIT wFrameCount; // The number of frames to run.
		wFrameCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", s_swDevice2, swFrameId_1, wFrameCount);
		swResult = sitalBc_Start (s_swDevice2, swFrameId_1, wFrameCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Wait for the frame run to end.
		U32BIT dwRemainingFrameRunTime; // The expected remaining frame run time (in ms).
		dwRemainingFrameRunTime = (wFrameCount * ((U16BIT)(wFrameTime / IP_FREQUENCY_FACTOR)/ 10));
		swResult = stldTester_Bc_WaitForEndOfFrameRun (s_swDevice2, dwRemainingFrameRunTime, 0, NULL);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.WaitForEndOfFrameRun.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Stop device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop: unit - %i\n", s_swDevice2);
		swResult = sitalBc_Stop (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// As long a not all expected interrupts occurred AND the maximum possible interrupt latency still didn't pass:
		///		Wait.
		stldInterruptLatency_Wait (&s_dwBcIsrCallCounter, wFrameCount);

		/// Verify that a correct number of interrupts of a suitable type occurred.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Interrupts: unit - %i, count - %i, expected_count - %i, last_mask - %08X, expected_mask - %08X\n", s_swDevice2, s_dwBcIsrCallCounter, wFrameCount, s_dwBcIrqStatus, (sitalInterruptRegister1_MASTER_INTERRUPT | sitalInterruptRegister1_BC_RETRY));
		if (wFrameCount != s_dwBcIsrCallCounter)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Interrupt.CallCount.Error: error - %i\n", s_dwBcIsrCallCounter);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		if (0U == (sitalInterruptRegister1_BC_RETRY & s_dwBcIrqStatus))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Interrupt.Mask.Error: error - %08X\n", s_dwBcIrqStatus);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		if (0U == (sitalInterruptRegister1_MASTER_INTERRUPT & s_dwBcIrqStatus))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Interrupt.Mask.Error: error - %08X\n", s_dwRtIrqStatus);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// Verify for device-2 (BC) that register ISR1 has been auto-cleared.
		wRegisterValue = sitalDevice_Register_Read (s_swDevice2, sitalRegisterAddress_INTERRUPT_STATUS_1);
		sitalProcess_Log_PrintLine ("#StldTester.Device.Register.Read: unit - %i, register_address - %04X, value - %04X\n", s_swDevice2, sitalRegisterAddress_INTERRUPT_STATUS_1, wRegisterValue);
		// Added ignore timetag rollover interrupt (0x40)
		if (0U != (wRegisterValue & 0xFFBF))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Register.Interrupt.AutoClear.Error: error - %i\n", s_dwRtIrqStatus);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// Free device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice2);
		swResult = sitalDevice_Free (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		swResult = stldSyncEnd (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncEnd.Error: error - %i\n", swResult);
			throw 0;
		}
	}
	catch (int)
	{
		sitalDevice_Free (s_swDevice2);
	}

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_11e_Isr1_Bit08.End: test_succeeded - %c\n", ((sitalReturnCode_SUCCESS == swResult) ? 'Y' : 'N'));
	return swResult;
}



/// <summary>
/// Test the stld1553 library in the henceforth described scenario:
/// Participant[s]: BC2.
/// Configuration: All participating devices must be able to communicate with each other.
/// Short description: Acceptance Test Plan (ATP) test 461_11e - Selectable interrupt generation for ISR-2 Bit-00 Chain from ISR1 bit 15.
/// Along test print suitable info and indications into the log.
///
/// Note:
/// - This function may use any of above defined Acceptance Test Plan (ATP) tests variables.
///   The caller must therefore call the suitable stldTester_Atp_Set* functions prior to calling this function.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Atp_DoTest461_11e_Isr2_Bit00 ()
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_11e_Isr2_Bit00.Start\n");

	S16BIT swResult; // Result of operation or function call.
	swResult = sitalReturnCode_SUCCESS;

	/// Get device count.
	U16BIT wRequiredDeviceCount; // The number of devices required for this test.
	wRequiredDeviceCount = (1 + s_swDevice2);
	U16BIT wAvailableDeviceCount; // The total number of devices available in the system.
	swResult = sitalDevice_GetCount (&wAvailableDeviceCount);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %i\n", swResult);
		return swResult;
	}
	sitalProcess_Log_PrintLine ("#StldTester.Device.Count: available - %i, required - %i\n", wAvailableDeviceCount, wRequiredDeviceCount);
	if (wAvailableDeviceCount < wRequiredDeviceCount)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %s\n", "not_enough_devices_installed");
		return sitalReturnCode_SUCCESS;
	}

	try
	{
		swResult = stldSyncStart (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncStart.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-2 (BC) as a real BC.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-bc, alloc_mem-none\n", s_swDevice2);
		swResult = sitalDevice_Initialize (s_swDevice2, sitalAccess_CARD, (sitalMode_BC | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Configure device-2 (BC) to issue desired interrupts.
		// Note: Here function sitalDevice_Register_Write may not be used, and function sitalDevice_AccessMemory must be used instead in order to directly update the target register in the device. That's because the target device isn't turned eventually into state RUN, so that the image of the registers will never be written into the target device.
		U16BIT wRegisterValue; // The current value of a target register, or the value to set for a target register.
		wRegisterValue = sitalInterruptMaskRegister1_TIME_TAG_ROLLOVER;
		sitalProcess_Log_PrintLine ("#StldTester.Device.Imr1.Set: unit - %i, imr1(post) - %04X\n", s_swDevice2, wRegisterValue);
		swResult = sitalDevice_AccessMemory (s_swDevice2, sitalDeviceAccessOperation_Write, sitalRegisterAddress_INTERRUPT_MASK_1, sitalDeviceMemorySection_Registers, 1, &wRegisterValue);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Imr1.Set.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Get for device-2 (BC) the current interrupt configuration.
		U16BIT wInterruptMode; // Current configuration for type of interrupt signal (sitalIrqMode_*).
		U16BIT wAutoClear; // Current configuration for post-read auto-clearing of interrupt status (sitalIrqClear_*).
		swResult = sitalDevice_Irq_GetMode (s_swDevice2, &wInterruptMode, &wAutoClear);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Mode.Get.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Configure device-2 (BC) to issue suitable interrupts and perform post-read auto-clearing of interrupt status.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Configure: unit - %i, mode - %i, auto_clear - %i\n", s_swDevice2, wInterruptMode, sitalIrqClear_NO_AUTO_CLEAR);
		swResult = sitalDevice_Irq_Configure (s_swDevice2, wInterruptMode, sitalIrqClear_NO_AUTO_CLEAR);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Configure.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Force the actual configuration of device-2 (BC).
		// Note: That's required in cases where no frame run takes place, so that configured registers aren't automatically written to the device by library stld1553.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Register.UpdateDevice: unit - %i, register - %i\n", s_swDevice2, sitalRegisterAddress_CONFIGURATION_2);
		swResult = sitalDevice_Register_UpdateDevice (s_swDevice2, sitalRegisterAddress_CONFIGURATION_2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Register.UpdateDevice.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Configure device-2 (BC) to get desired notifications using an ISR that collects interrupt data, counts ISR calls, records the interrupt mask, and signals interested threads.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Manipulate: unit - %i, enable - %i, irq - %08X, callback - %s\n", s_swDevice2, TRUE, sitalInterruptRegister1_TIME_TAG_ROLLOVER, "stldTester_Bc_UserIsr");
		swResult = sitalDevice_Irq_Manipulate (s_swDevice2, TRUE, sitalInterruptRegister1_TIME_TAG_ROLLOVER, stldTester_Bc_UserIsr);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Set.Error: error - %i\n", swResult);
			throw 0;
		}
		s_dwBcIsrCallCounter = 0;
		s_dwBcIrqStatus = 0;

		/// Set for device-2 (BC) the time tag to FFFFh.
		U16BIT wWrittenTimeTag; // Device time tag.
		wWrittenTimeTag = 0xFFFF;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.TimeTag.Set: unit - %i, value - %04X\n", s_swDevice2, wWrittenTimeTag);
		swResult = sitalDevice_TimeTag_Set (s_swDevice2, wWrittenTimeTag);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.TimeTag.Set.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Wait for the expected interrupt to be issued.
		U32BIT dwSleepTime; // Sleeping period (in ms).
		dwSleepTime = 1000; // A time that's much longer than really required is used in order to verify that no additional (an unexpected) interrupts occur.
		sitalProcess_Log_PrintLine ("#StldTester.Wait: time - %ims\n", dwSleepTime);
		sitalProcess_Log_PrintLine ("\tWAIT_FOR %i us\n", (dwSleepTime * 1000));
		PrecisionTimer pt; // A precision timer.
		pt.AccuratelySleep (dwSleepTime);

		/// As long as not all expected interrupts occurred AND the maximum possible interrupt latency still didn't pass:
		///		Wait.
		stldInterruptLatency_Wait (&s_dwBcIsrCallCounter, 1);

		/// Verify that a correct number of interrupts of a suitable type occurred.
		// Note: Some time tag interrupts must have occurred, but the exact number of depends on the sleeping time, the time tag resolution, and on some random component.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Interrupts: unit - %i, count - %i, expected_count - %i, last_mask - %08X, expected_mask - %08X\n", s_swDevice2, s_dwBcIsrCallCounter, 1, s_dwBcIrqStatus, sitalInterruptRegister1_TIME_TAG_ROLLOVER);
		if (0U == s_dwBcIsrCallCounter)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Interrupt.CallCount.Error: error - %i\n", s_dwBcIsrCallCounter);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		if (0U == (sitalInterruptRegister1_TIME_TAG_ROLLOVER & s_dwBcIrqStatus))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Interrupt.Mask.Error: error - %08X\n", s_dwBcIrqStatus);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		if (0U == (sitalInterruptRegister2_INTERRUPT_CHAIN & s_dwBcIrqStatus))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Interrupt.Mask.Error: error - %08X\n", s_dwBcIrqStatus);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// Verify for device-2 (BC) that register ISR1 has been auto-cleared.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Register.ISR1.Read: unit - %i, expected_value - %04X\n", s_swDevice2, 0x0000);
		wRegisterValue = sitalDevice_Register_Read (s_swDevice2, sitalRegisterAddress_INTERRUPT_STATUS_1);
		if (0x0000 != wRegisterValue)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Register.ISR1.Read.Error: error - %04X\n", wRegisterValue);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// Verify for device-2 (BC) that register ISR2 has been auto-cleared.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Register.ISR2.Read: unit - %i, expected_value - %04X\n", s_swDevice2, 0x0000);
		wRegisterValue = sitalDevice_Register_Read (s_swDevice2, sitalRegisterAddress_INTERRUPT_STATUS_2);
		if (0x0000 != wRegisterValue)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Register.ISR2.Read.Error: error - %04X\n", wRegisterValue);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// Free device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice2);
		swResult = sitalDevice_Free (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		swResult = stldSyncEnd (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncEnd.Error: error - %i\n", swResult);
			throw 0;
		}
	}
	catch (int)
	{
		sitalDevice_Free (s_swDevice2);
	}

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_11e_Isr2_Bit00.End: test_succeeded - %c\n", ((sitalReturnCode_SUCCESS == swResult) ? 'Y' : 'N'));
	return swResult;
}



/// <summary>
/// Test the stld1553 library in the henceforth described scenario:
/// Participant[s]: BC2.
/// Configuration: All participating devices must be able to communicate with each other.
/// Short description: Acceptance Test Plan (ATP) test 461_11e - Selectable interrupt generation for ISR-2 Bit-01 Built-in test complete.
/// Along test print suitable info and indications into the log.
///
/// Note:
/// - This function may use any of above defined Acceptance Test Plan (ATP) tests variables.
///   The caller must therefore call the suitable stldTester_Atp_Set* functions prior to calling this function.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Atp_DoTest461_11e_Isr2_Bit01 ()
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_11e_Isr2_Bit01.Start\n");

	S16BIT swResult; // Result of operation or function call.
	swResult = sitalReturnCode_SUCCESS;

	/// Get device count.
	U16BIT wRequiredDeviceCount; // The number of devices required for this test.
	wRequiredDeviceCount = (1 + s_swDevice2);
	U16BIT wAvailableDeviceCount; // The total number of devices available in the system.
	swResult = sitalDevice_GetCount (&wAvailableDeviceCount);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %i\n", swResult);
		return swResult;
	}
	sitalProcess_Log_PrintLine ("#StldTester.Device.Count: available - %i, required - %i\n", wAvailableDeviceCount, wRequiredDeviceCount);
	if (wAvailableDeviceCount < wRequiredDeviceCount)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %s\n", "not_enough_devices_installed");
		return sitalReturnCode_SUCCESS;
	}

	try
	{
		swResult = stldSyncStart (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncStart.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-2 (BC) as a real BC.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-bc, alloc_mem-none\n", s_swDevice2);
		swResult = sitalDevice_Initialize (s_swDevice2, sitalAccess_CARD, (sitalMode_BC | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Configure device-2 (BC) to get desired notifications using an ISR that collects interrupt data, counts ISR calls, records the interrupt mask, and signals interested threads.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Manipulate: unit - %i, enable - %i, irq - %08X, callback - %s\n", s_swDevice2, TRUE, sitalInterruptRegister2_BUILT_IN_TEST_COMPLETE, "stldTester_Bc_UserIsr");
		swResult = sitalDevice_Irq_Manipulate (s_swDevice2, TRUE, sitalInterruptRegister2_BUILT_IN_TEST_COMPLETE, stldTester_Bc_UserIsr);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Set.Error: error - %i\n", swResult);
			throw 0;
		}
		s_dwBcIsrCallCounter = 0;
		s_dwBcIrqStatus = 0;

		/// Get device-2 information.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Information.Get: unit - %i\n", s_swDevice2);
		sitalDeviceCapabilitiesStructure dcsDeviceCapabilities;
		dcsDeviceCapabilities.dwStructureSize = sizeof(sitalDeviceCapabilitiesStructure);
		swResult = sitalDevice_GetInformation (s_swDevice2, &dcsDeviceCapabilities);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.GetInformation.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Get device-2 card's information.
		sitalProcess_Log_PrintLine ("#StldTester.Card.Information.Get: card - %i\n", dcsDeviceCapabilities.swCard);
		sitalCardInformationStructure cisCardInformation;
		cisCardInformation.dwStructureSize = sizeof(sitalCardInformationStructure);
		swResult = sitalCard_GetInformation (dcsDeviceCapabilities.swCard, &cisCardInformation);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.GetInformation.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Reset device-2 via its own card.
		// Note: In case of a target card technology for which function sitalCard_Ip_Reset isn't supported (e.g., PCI), this test cannot be performed.
		U16BIT uiDeviceIndexInCard; // The index of a specific device among the devices of its card.
		uiDeviceIndexInCard = (s_swDevice2 - cisCardInformation.wFirstUnitId);
		sitalProcess_Log_PrintLine ("#StldTester.Card.Device.Reset: card - %i, unit - %i, first_unit - %i, reset_mask - %04X\n", dcsDeviceCapabilities.swCard, s_swDevice2, cisCardInformation.wFirstUnitId, (1 << uiDeviceIndexInCard));
		swResult = sitalCard_Ip_Reset (dcsDeviceCapabilities.swCard, (1 << uiDeviceIndexInCard));
		if (sitalReturnCode_NOT_SUPPORTED == swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Card.Device.Reset.Error: error - %s\n", "card_ip_reset_unsupported");
			return sitalReturnCode_SUCCESS;
		}
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Card.Device.Reset.Error: error - %i\n", swResult);
			throw 0;
		}
		swResult = sitalCard_Ip_Reset (dcsDeviceCapabilities.swCard, 0/*clear reset lines*/);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Card.Device.Reset.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Configure device-2 to the desired type of interrupt signal.
		U16BIT wInterruptMode; // Proper configuration for type of interrupt signal (sitalIrqMode_*) in the light of the operating system and card type.
		wInterruptMode = ( (BRMC_CAP_IRQ_PULSE == (BRMC_CAP_IRQ_PULSE & cisCardInformation.wFlags)) ? sitalIrqMode_PULSE: sitalIrqMode_LEVEL );
		U16BIT wRegisterValue; // The current value of a target register, or the value to set for a target register.
		wRegisterValue =  ((sitalIrqMode_LEVEL == wInterruptMode) ? 0xFFFFU : 0x0000U);
		sitalProcess_Log_PrintLine ("#StldTester.Device.Config2.Set.Masked: unit - %i, ir_mode - %04X, ir_mask - %04X\n", s_swDevice2, wInterruptMode, wRegisterValue);
		swResult = sitalDevice_AccessMemory	(s_swDevice2, sitalDeviceAccessOperation_WriteMasked, sitalRegisterAddress_CONFIGURATION_2, sitalDeviceMemorySection_Registers, sitalConfigurationRegister2_LEVEL_OR_PULSE_INTERRUPTS, &wRegisterValue);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Config2.Set.Masked.Error: error - %i\n", swResult);
			return swResult;
		}

		/// Configure device-2 to issue desired interrupts.
		// Note: Here function sitalDevice_Register_Write may not be used, and function sitalDevice_AccessMemory must be used instead in order to directly update the target register in the device. That's because the target device isn't turned eventually into state RUN, so that the image of the registers will never be written into the target device.
		wRegisterValue = sitalInterruptMaskRegister2_BUILT_IN_TEST_COMPLETE;
		sitalProcess_Log_PrintLine ("#StldTester.Device.Imr2.Set: unit - %i, imr2(post) - %04X\n", s_swDevice2, wRegisterValue);
		
		swResult = sitalDevice_AccessMemory (s_swDevice2, sitalDeviceAccessOperation_Write, sitalRegisterAddress_INTERRUPT_MASK_2, sitalDeviceMemorySection_Registers, 1, &wRegisterValue);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Imr2.Set.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Wait for the expected interrupt to be issued.
		U32BIT dwSleepTime; // Sleeping period (in ms).
		dwSleepTime = 1; // The device should set the expected interrupt within this time.
		sitalProcess_Log_PrintLine ("#StldTester.Wait: time - %ims\n", dwSleepTime);
		sitalProcess_Log_PrintLine ("\tWAIT_FOR %i us\n", (dwSleepTime * 1000));
		PrecisionTimer pt; // A precision timer.
		pt.AccuratelySleep (dwSleepTime);

		/// As long a not all expected interrupts occurred AND the maximum possible interrupt latency still didn't pass:
		///		Wait.
		stldInterruptLatency_Wait (&s_dwBcIsrCallCounter, 1);

		/// Verify that a correct number of interrupts of a suitable type occurred.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Interrupts: unit - %i, count - %i, expected_count - %i, last_mask - %08X, expected_mask - %08X\n", s_swDevice2, s_dwBcIsrCallCounter, 1, s_dwBcIrqStatus, (sitalInterruptRegister2_MASTER_INTERRUPT | sitalInterruptRegister2_BUILT_IN_TEST_COMPLETE));
		if (1 != s_dwBcIsrCallCounter)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Interrupt.CallCount.Error: error - %i\n", s_dwBcIsrCallCounter);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		if (0U == (sitalInterruptRegister2_BUILT_IN_TEST_COMPLETE & s_dwBcIrqStatus))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Interrupt.Mask.Error: error - %08X\n", s_dwBcIrqStatus);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		if (0U == (sitalInterruptRegister2_MASTER_INTERRUPT & s_dwBcIrqStatus))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Interrupt.Mask.Error: error - %08X\n", s_dwBcIrqStatus);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// Verify for device-2 (BC) that register ISR1 has been auto-cleared.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Register.ISR1.Read: unit - %i, expected_value - %04X\n", s_swDevice2, 0x0000);
		wRegisterValue = sitalDevice_Register_Read (s_swDevice2, sitalRegisterAddress_INTERRUPT_STATUS_1);
		if (0x0000 != wRegisterValue)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Register.ISR1.Read.Error: error - %04X\n", wRegisterValue);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// Verify for device-2 (BC) that register ISR2 has been auto-cleared.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Register.ISR2.Read: unit - %i, expected_value - %04X\n", s_swDevice2, 0x0000);
		wRegisterValue = sitalDevice_Register_Read (s_swDevice2, sitalRegisterAddress_INTERRUPT_STATUS_2);
		if (0x0000 != wRegisterValue)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Register.ISR2.Read.Error: error - %04X\n", wRegisterValue);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// Free device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice2);
		swResult = sitalDevice_Free (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		swResult = stldSyncEnd (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncEnd.Error: error - %i\n", swResult);
			throw 0;
		}
	}
	catch (int)
	{
		sitalDevice_Free (s_swDevice2);
	}

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_11e_Isr2_Bit01.End: test_succeeded - %c\n", ((sitalReturnCode_SUCCESS == swResult) ? 'Y' : 'N'));
	return swResult;
}



/// <summary>
/// Test the stld1553 library in the henceforth described scenario:
/// Participant[s]: BC2.
/// Configuration: All participating devices must be able to communicate with each other.
/// Short description: Acceptance Test Plan (ATP) test 461_11e - Selectable interrupt generation for ISR-2 Bits 02-05 eBC sets IRQ 0-3, respectively.
/// Along test print suitable info and indications into the log.
///
/// Note:
/// - This function may use any of above defined Acceptance Test Plan (ATP) tests variables.
///   The caller must therefore call the suitable stldTester_Atp_Set* functions prior to calling this function.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Atp_DoTest461_11e_Isr2_Bits02To05 ()
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_11e_Isr2_Bits02To05.Start\n");

	S16BIT swResult; // Result of operation or function call.
	swResult = sitalReturnCode_SUCCESS;

	/// Get device count.
	U16BIT wRequiredDeviceCount; // The number of devices required for this test.
	wRequiredDeviceCount = (1 + s_swDevice2);
	U16BIT wAvailableDeviceCount; // The total number of devices available in the system.
	swResult = sitalDevice_GetCount (&wAvailableDeviceCount);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %i\n", swResult);
		return swResult;
	}
	sitalProcess_Log_PrintLine ("#StldTester.Device.Count: available - %i, required - %i\n", wAvailableDeviceCount, wRequiredDeviceCount);
	if (wAvailableDeviceCount < wRequiredDeviceCount)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %s\n", "not_enough_devices_installed");
		return sitalReturnCode_SUCCESS;
	}

	try
	{
		swResult = stldSyncStart (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncStart.Error: error - %i\n", swResult);
			throw 0;
		}

		// Loop over all sub-tests:
		for (int iSubTest=0; iSubTest<4; iSubTest++)
		{
			/// Initialize device-2 (BC) as a real BC.
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-bc, alloc_mem-none\n", s_swDevice2);
			swResult = sitalDevice_Initialize (s_swDevice2, sitalAccess_CARD, (sitalMode_BC | sitalModeVariant_ADVANCED), 0, 0, 0);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Initialize sub-test variables using suitable constants.
			U16BIT wRegisterValue; // The current value of a target register, or the value to set for a target register.
			U32BIT dwIrqMask; // A mask designating a set of interrupts that should be enabled/disabled as indicated by parameter bIsInterruptEnabled (An or-ed combination of sitalInterruptRegister*).
			switch (iSubTest)
			{
			case 0: wRegisterValue = sitalInterruptMaskRegister2_BC_INTERRUPT_0; dwIrqMask = sitalInterruptRegister2_BC_IRQ0; break;
			case 1: wRegisterValue = sitalInterruptMaskRegister2_BC_INTERRUPT_1; dwIrqMask = sitalInterruptRegister2_BC_IRQ1; break;
			case 2: wRegisterValue = sitalInterruptMaskRegister2_BC_INTERRUPT_2; dwIrqMask = sitalInterruptRegister2_BC_IRQ2; break;
			case 3: wRegisterValue = sitalInterruptMaskRegister2_BC_INTERRUPT_3; dwIrqMask = sitalInterruptRegister2_BC_IRQ3; break;
			default:
				{
					sitalProcess_Log_PrintLine ("#StldTester.SubTest.Error: error - %i\n", iSubTest);
					swResult = sitalReturnCode_TEST_FAIL;
					throw 0;
				}
			}

			/// Configure device-2 (BC) to issue desired interrupts.
			sitalProcess_Log_PrintLine ("#StldTester.Device.Imr2.Set: unit - %i, imr2(post) - %04X\n", s_swDevice2, wRegisterValue);
			swResult = sitalDevice_Register_Write (s_swDevice2, sitalRegisterAddress_INTERRUPT_MASK_2, wRegisterValue);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Device.Imr2.Set.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Get for device-2 (BC) the current interrupt configuration.
			U16BIT wInterruptMode; // Current configuration for type of interrupt signal (sitalIrqMode_*).
			U16BIT wAutoClear; // Current configuration for post-read auto-clearing of interrupt status (sitalIrqClear_*).
			swResult = sitalDevice_Irq_GetMode (s_swDevice2, &wInterruptMode, &wAutoClear);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Mode.Get.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Configure device-2 (BC) to issue suitable interrupts and perform post-read auto-clearing of interrupt status.
			sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Configure: unit - %i, mode - %i, auto_clear - %i\n", s_swDevice2, wInterruptMode, sitalIrqClear_NO_AUTO_CLEAR);
			swResult = sitalDevice_Irq_Configure (s_swDevice2, wInterruptMode, sitalIrqClear_NO_AUTO_CLEAR);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Configure.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Configure device-2 (BC) to get desired notifications using an ISR that collects interrupt data, counts ISR calls, records the interrupt mask, and signals interested threads.
			sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Manipulate: unit - %i, enable - %i, irq - %08X, callback - %s\n", s_swDevice2, TRUE, dwIrqMask, "stldTester_Bc_UserIsr");
			swResult = sitalDevice_Irq_Manipulate (s_swDevice2, TRUE, dwIrqMask, stldTester_Bc_UserIsr);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Set.Error: error - %i\n", swResult);
				throw 0;
			}
			s_dwBcIsrCallCounter = 0;
			s_dwBcIrqStatus = 0;

			/// Create for device-2 (BC) command-0, an IRQ-0 request command.
			S16BIT swCommandId_0; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
			swCommandId_0 = 0;
			U32BIT dwParameter1; // Command definition parameter #1.
			dwParameter1 = (1 << iSubTest); // If bit #n, where n=0...3, is set, then the device raises register ISR2 bit #m, where m=2....5, respectively.
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_0, sitalOpcode_INTERRUPT_REQUEST, sitalOpcodeCondition_ALWAYS, dwParameter1, 0);
			swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_0, sitalOpcode_INTERRUPT_REQUEST, sitalOpcodeCondition_ALWAYS, dwParameter1, 0, 0);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Create for device-2 (BC) command-1, a halt command.
			S16BIT swCommandId_1; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
			swCommandId_1 = 1;
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_1, sitalOpcode_HALT, sitalOpcodeCondition_ALWAYS, 0, 0);
			swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_1, sitalOpcode_HALT, sitalOpcodeCondition_ALWAYS, 0, 0, 0);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Create for device-2 (BC) command-2, a subroutine call command that executes minor-frame-0. 
			S16BIT swCommandId_2; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
			swCommandId_2 = 2;
			S16BIT swFrameId_0; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
			swFrameId_0 = 0;
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_2, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, swFrameId_0, 0);
			swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_2, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, (U16BIT)swFrameId_0, 0, 0);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Create for device-2 (BC) minor-frame-0, a frame that contains command-0 and command-1.
			S16BIT swaCommandIds[sitalBcCounter_COMMANDS]; // An array of command-IDs.
			swaCommandIds[0] = swCommandId_0;
			swaCommandIds[1] = swCommandId_1;
			U16BIT wCommandIdCount; // The number of command IDs in the frame (1-sitalBcCounter_FRAME_COMMANDS).
			wCommandIdCount = 2;
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i, %i), time - %i, flags - %i\n", s_swDevice2, swFrameId_0, "minor", wCommandIdCount, swCommandId_0, swCommandId_1, 0, 0);
			swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_0, sitalBcFrameType_MINOR, swaCommandIds, wCommandIdCount, 0, 0);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Create for device-2 (BC) major-frame-1, a frame that contains command-2.
			S16BIT swFrameId_1; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
			swFrameId_1 = 1;
			swaCommandIds[0] = swCommandId_2;
			wCommandIdCount = 1;
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_1, "major", wCommandIdCount, swCommandId_2, 0, 0);
			swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_1, sitalBcFrameType_MAJOR, swaCommandIds, wCommandIdCount, 0, 0);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Start device-2 (BC) running major-frame-1 for 1-shot.
			U16BIT wFrameShotCount; // Number of frame shots.
			wFrameShotCount = 1;
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", s_swDevice2, swFrameId_1, wFrameShotCount);
			swResult = sitalBc_Start (s_swDevice2, swFrameId_1, wFrameShotCount);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Wait for the expected interrupt to be issued.
			U32BIT dwSleepTime; // Sleeping period (in ms).
			dwSleepTime = 10; // The device should set the expected interrupt within this time.
			sitalProcess_Log_PrintLine ("#StldTester.Wait: time - %ims\n", dwSleepTime);
			sitalProcess_Log_PrintLine ("\tWAIT_FOR %i us\n", (dwSleepTime * 1000));
			PrecisionTimer pt; // A precision timer.
			pt.AccuratelySleep (dwSleepTime);

			///	Stop device-2 (BC).
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop: unit - %i\n", s_swDevice2);
			swResult = sitalBc_Stop (s_swDevice2);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
				throw 0;
			}

			/// As long a not all expected interrupts occurred AND the maximum possible interrupt latency still didn't pass:
			///		Wait.
			stldInterruptLatency_Wait (&s_dwBcIsrCallCounter, 1);

			/// Verify that a correct number of interrupts of a suitable type occurred.
			sitalProcess_Log_PrintLine ("#StldTester.Device.Interrupts: unit - %i, count - %i, expected_count - %i, last_mask - %08X, expected_mask - %08X\n", s_swDevice2, s_dwBcIsrCallCounter, 1, s_dwBcIrqStatus, (sitalInterruptRegister2_MASTER_INTERRUPT | dwIrqMask));
			if (1 != s_dwBcIsrCallCounter)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Device.Interrupt.CallCount.Error: error - %i\n", s_dwBcIsrCallCounter);
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}
			if (dwIrqMask != (dwIrqMask & s_dwBcIrqStatus))
			{
				sitalProcess_Log_PrintLine ("#StldTester.Device.Interrupt.Mask.Error: error - %08X\n", s_dwBcIrqStatus);
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}
			if (0U == (sitalInterruptRegister2_MASTER_INTERRUPT & s_dwBcIrqStatus))
			{
				sitalProcess_Log_PrintLine ("#StldTester.Device.Interrupt.Mask.Error: error - %08X\n", s_dwBcIrqStatus);
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}

			/// Verify for device-2 (BC) that register ISR1 has been auto-cleared.
			sitalProcess_Log_PrintLine ("#StldTester.Device.Register.ISR1.Read: unit - %i, expected_value - %04X\n", s_swDevice2, 0x0000);
			wRegisterValue = sitalDevice_Register_Read (s_swDevice2, sitalRegisterAddress_INTERRUPT_STATUS_1);
			// Added ignore timetag rollover interrupt (0x40)
			if (0U != (wRegisterValue & 0xFFBF))
			{
				sitalProcess_Log_PrintLine ("#StldTester.Device.Register.ISR1.Read.Error: error - %04X\n", wRegisterValue);
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}

			/// Verify for device-2 (BC) that register ISR2 has been auto-cleared.
			sitalProcess_Log_PrintLine ("#StldTester.Device.Register.ISR2.Read: unit - %i, expected_value - %04X\n", s_swDevice2, 0x0000);
			wRegisterValue = sitalDevice_Register_Read (s_swDevice2, sitalRegisterAddress_INTERRUPT_STATUS_2);
			// Added ignore timetag rollover interrupt (0x40)
			if (0U != (wRegisterValue & 0xFFBF))
			{
				sitalProcess_Log_PrintLine ("#StldTester.Device.Register.ISR2.Read.Error: error - %04X\n", wRegisterValue);
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}

			/// Free device-2 (BC).
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice2);
			swResult = sitalDevice_Free (s_swDevice2);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
				throw 0;
			}
		}

		swResult = stldSyncEnd (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncEnd.Error: error - %i\n", swResult);
			throw 0;
		}
	}
	catch (int)
	{
		sitalDevice_Free (s_swDevice2);
	}

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_11e_Isr2_Bits02To05.End: test_succeeded - %c\n", ((sitalReturnCode_SUCCESS == swResult) ? 'Y' : 'N'));
	return swResult;
}



/// <summary>
/// Test the stld1553 library in the henceforth described scenario:
/// Participant[s]: BC2.
/// Configuration: All participating devices must be able to communicate with each other.
/// Short description: Acceptance Test Plan (ATP) test 461_11e - Selectable interrupt generation for ISR-2 Bit 10 eBC illegal Op-code or illegal parity or illegal constant.
/// Along test print suitable info and indications into the log.
///
/// Note:
/// - This function may use any of above defined Acceptance Test Plan (ATP) tests variables.
///   The caller must therefore call the suitable stldTester_Atp_Set* functions prior to calling this function.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Atp_DoTest461_11e_Isr2_Bit10 ()
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_11e_Isr2_Bit10.Start\n");

#ifdef USE_DDC_LIBRARY
	sitalProcess_Log_PrintLine ("#StldTester.TestRelevancy.Error: error - %s\n", "function_sitalBc_Command_DistortWord_undefined _by_DDC_API");
	return sitalReturnCode_SUCCESS;
#else // ! USE_DDC_LIBRARY

	S16BIT swResult; // Result of operation or function call.
	swResult = sitalReturnCode_SUCCESS;

	/// Get device count.
	U16BIT wRequiredDeviceCount; // The number of devices required for this test.
	wRequiredDeviceCount = (1 + s_swDevice2);
	U16BIT wAvailableDeviceCount; // The total number of devices available in the system.
	swResult = sitalDevice_GetCount (&wAvailableDeviceCount);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %i\n", swResult);
		return swResult;
	}
	sitalProcess_Log_PrintLine ("#StldTester.Device.Count: available - %i, required - %i\n", wAvailableDeviceCount, wRequiredDeviceCount);
	if (wAvailableDeviceCount < wRequiredDeviceCount)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %s\n", "not_enough_devices_installed");
		return sitalReturnCode_SUCCESS;
	}

	try
	{
		swResult = stldSyncStart (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncStart.Error: error - %i\n", swResult);
			throw 0;
		}

		// Loop over all sub-tests:
		for (int iSubTest=0; iSubTest<3; iSubTest++)
		{
			/// Initialize device-2 (BC) as a real BC.
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-bc, alloc_mem-none\n", s_swDevice2);
			swResult = sitalDevice_Initialize (s_swDevice2, sitalAccess_CARD, (sitalMode_BC | sitalModeVariant_ADVANCED), 0, 0, 0);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Initialize sub-test variables using suitable constants.
			/// Configure the stld1553 library to insert a sub-test specific distortion of the HALT command-0 (defined below).
			// Note: All below requested command distortions result in halt commands with illegal parity bit.
			U16BIT wRegisterValue; // The current value of a target register, or the value to set for a target register.
			U32BIT dwIrqMask; // A mask designating a set of interrupts that should be enabled/disabled as indicated by parameter bIsInterruptEnabled (An or-ed combination of sitalInterruptRegister*).
			switch (iSubTest)
			{
			case 0: swResult = sitalBc_Command_DistortWord (sitalOpcode_HALT, sitalOpcodeDistortion_FlipBit, 15); wRegisterValue = (sitalInterruptMaskRegister2_BC_TRAP | sitalInterruptMaskRegister2_BC_OPCODE_PARITY_ERROR); dwIrqMask = (sitalInterruptRegister2_BC_TRAP | sitalInterruptRegister2_BC_OPCODE_PARITY_ERROR); break;
			case 1: swResult = sitalBc_Command_DistortWord (sitalOpcode_HALT, sitalOpcodeDistortion_FlipBit, 07); wRegisterValue = (sitalInterruptMaskRegister2_BC_TRAP | sitalInterruptMaskRegister2_BC_OPCODE_PARITY_ERROR); dwIrqMask = (sitalInterruptRegister2_BC_TRAP | sitalInterruptRegister2_BC_OPCODE_PARITY_ERROR); break;
			case 2: swResult = sitalBc_Command_DistortWord (sitalOpcode_HALT, sitalOpcodeDistortion_SetCommand, 0x8000); wRegisterValue = sitalInterruptMaskRegister2_BC_TRAP; dwIrqMask = sitalInterruptRegister2_BC_TRAP; break;
			default:
				{
					sitalProcess_Log_PrintLine ("#StldTester.SubTest.Error: error - %i\n", iSubTest);
					swResult = sitalReturnCode_TEST_FAIL;
					throw 0;
				}
			}
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Distortion.On.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Configure device-2 (BC) to issue desired interrupts.
			sitalProcess_Log_PrintLine ("#StldTester.Device.Imr2.Set: unit - %i, imr2(post) - %04X\n", s_swDevice2, wRegisterValue);
			swResult = sitalDevice_Register_Write (s_swDevice2, sitalRegisterAddress_INTERRUPT_MASK_2, wRegisterValue);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Device.Imr2.Set.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Get for device-2 (BC) the current interrupt configuration.
			U16BIT wInterruptMode; // Current configuration for type of interrupt signal (sitalIrqMode_*).
			U16BIT wAutoClear; // Current configuration for post-read auto-clearing of interrupt status (sitalIrqClear_*).
			swResult = sitalDevice_Irq_GetMode (s_swDevice2, &wInterruptMode, &wAutoClear);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Mode.Get.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Configure device-2 (BC) to issue suitable interrupts and perform post-read auto-clearing of interrupt status.
			sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Configure: unit - %i, mode - %i, auto_clear - %i\n", s_swDevice2, wInterruptMode, sitalIrqClear_NO_AUTO_CLEAR);
			swResult = sitalDevice_Irq_Configure (s_swDevice2, wInterruptMode, sitalIrqClear_NO_AUTO_CLEAR);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Configure.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Configure device-2 (BC) to get desired notifications using an ISR that collects interrupt data, counts ISR calls, records the interrupt mask, and signals interested threads.
			sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Manipulate: unit - %i, enable - %i, irq - %08X, callback - %s\n", s_swDevice2, TRUE, dwIrqMask, "stldTester_Bc_UserIsr");
			swResult = sitalDevice_Irq_Manipulate (s_swDevice2, TRUE, dwIrqMask, stldTester_Bc_UserIsr);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Set.Error: error - %i\n", swResult);
				throw 0;
			}
			s_dwBcIsrCallCounter = 0;
			s_dwBcIrqStatus = 0;

			/// Create for device-2 (BC) command-0, a halt command.
			S16BIT swCommandId_0; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
			swCommandId_0 = 0;
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_0, sitalOpcode_HALT, sitalOpcodeCondition_ALWAYS, 0, 0);
			swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_0, sitalOpcode_HALT, sitalOpcodeCondition_ALWAYS, 0, 0, 0);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Create for device-2 (BC) command-1, a subroutine call command that executes minor-frame-0. 
			S16BIT swCommandId_1; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
			swCommandId_1 = 1;
			S16BIT swFrameId_0; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
			swFrameId_0 = 0;
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_1, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, swFrameId_0, 0);
			swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_1, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, (U16BIT)swFrameId_0, 0, 0);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Create for device-2 (BC) minor-frame-0, a frame that contains command-0.
			S16BIT swaCommandIds[sitalBcCounter_COMMANDS]; // An array of command-IDs.
			swaCommandIds[0] = swCommandId_0;
			U16BIT wCommandIdCount; // The number of command IDs in the frame (1-sitalBcCounter_FRAME_COMMANDS).
			wCommandIdCount = 1;
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %i, flags - %i\n", s_swDevice2, swFrameId_0, "minor", wCommandIdCount, swCommandId_0, 0, 0);
			swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_0, sitalBcFrameType_MINOR, swaCommandIds, wCommandIdCount, 0, 0);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Create for device-2 (BC) major-frame-1, a frame that contains command-1.
			S16BIT swFrameId_1; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
			swFrameId_1 = 1;
			swaCommandIds[0] = swCommandId_1;
			wCommandIdCount = 1;
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_1, "major", wCommandIdCount, swCommandId_1, 0, 0);
			swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_1, sitalBcFrameType_MAJOR, swaCommandIds, wCommandIdCount, 0, 0);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Start device-2 (BC) running major-frame-1 for 1-shot.
			U16BIT wFrameShotCount; // Number of frame shots.
			wFrameShotCount = 1;
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", s_swDevice2, swFrameId_1, wFrameShotCount);
			swResult = sitalBc_Start (s_swDevice2, swFrameId_1, wFrameShotCount);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Wait for the expected interrupt to be issued.
			U32BIT dwSleepTime; // Sleeping period (in ms).
			dwSleepTime = 10; // The device should set the expected interrupt within this time.
			sitalProcess_Log_PrintLine ("#StldTester.Wait: time - %ims\n", dwSleepTime);
			sitalProcess_Log_PrintLine ("\tWAIT_FOR %i us\n", (dwSleepTime * 1000));
			PrecisionTimer pt; // A precision timer.
			pt.AccuratelySleep (dwSleepTime);

			///	Stop device-2 (BC).
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop: unit - %i\n", s_swDevice2);
			swResult = sitalBc_Stop (s_swDevice2);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Cancel the previously configured command distortion.
			swResult = sitalBc_Command_DistortWord (sitalOpcode_HALT, sitalOpcodeDistortion_None, 0);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Distortion.Off.Error: error - %i\n", swResult);
				throw 0;
			}

			/// As long a not all expected interrupts occurred AND the maximum possible interrupt latency still didn't pass:
			///		Wait.
			stldInterruptLatency_Wait (&s_dwBcIsrCallCounter, 1);

			/// Verify that a correct number of interrupts of a suitable type occurred.
			sitalProcess_Log_PrintLine ("#StldTester.Device.Interrupts: unit - %i, count - %i, expected_count - %i, last_mask - %08X, expected_mask - %08X\n", s_swDevice2, s_dwBcIsrCallCounter, 1, s_dwBcIrqStatus, (sitalInterruptRegister2_MASTER_INTERRUPT | dwIrqMask));
			if (1 != s_dwBcIsrCallCounter)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Device.Interrupt.CallCount.Error: error - %i\n", s_dwBcIsrCallCounter);
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}
			if (dwIrqMask != (dwIrqMask & s_dwBcIrqStatus))
			{
				sitalProcess_Log_PrintLine ("#StldTester.Device.Interrupt.Mask.Error: error - %08X\n", s_dwBcIrqStatus);
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}
			if (0U == (sitalInterruptRegister2_MASTER_INTERRUPT & s_dwBcIrqStatus))
			{
				sitalProcess_Log_PrintLine ("#StldTester.Device.Interrupt.Mask.Error: error - %08X\n", s_dwBcIrqStatus);
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}

			/// Verify for device-2 (BC) that register ISR1 has been auto-cleared.
			sitalProcess_Log_PrintLine ("#StldTester.Device.Register.ISR1.Read: unit - %i, expected_value - %04X\n", s_swDevice2, 0x0000);
			wRegisterValue = sitalDevice_Register_Read (s_swDevice2, sitalRegisterAddress_INTERRUPT_STATUS_1);
			// Added ignore timetag rollover interrupt (0x40)
			if (0U != (wRegisterValue & 0xFFBF))
			{
				sitalProcess_Log_PrintLine ("#StldTester.Device.Register.ISR1.Read.Error: error - %04X\n", wRegisterValue);
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}

			/// Verify for device-2 (BC) that register ISR2 has been auto-cleared.
			sitalProcess_Log_PrintLine ("#StldTester.Device.Register.ISR2.Read: unit - %i, expected_value - %04X\n", s_swDevice2, 0x0000);
			wRegisterValue = sitalDevice_Register_Read (s_swDevice2, sitalRegisterAddress_INTERRUPT_STATUS_2);
			// Added ignore timetag rollover interrupt (0x40)
			if (0U != (wRegisterValue & 0xFFBF))
			{
				sitalProcess_Log_PrintLine ("#StldTester.Device.Register.ISR2.Read.Error: error - %04X\n", wRegisterValue);
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}

			/// Free device-2 (BC).
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice2);
			swResult = sitalDevice_Free (s_swDevice2);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
				throw 0;
			}
		}

		swResult = stldSyncEnd (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncEnd.Error: error - %i\n", swResult);
			throw 0;
		}
	}
	catch (int)
	{
		sitalDevice_Free (s_swDevice2);
	}

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_11e_Isr2_Bit10.End: test_succeeded - %c\n", ((sitalReturnCode_SUCCESS == swResult) ? 'Y' : 'N'));
	return swResult;

#endif // USE_DDC_LIBRARY
}



/// <summary>
/// Test the stld1553 library in the henceforth described scenario:
/// Participant[s]: BC2.
/// Configuration: All participating devices must be able to communicate with each other.
/// Short description: Acceptance Test Plan (ATP) test 461_11e - Selectable interrupt generation for ISR-2 Bit 11 eBC � Call stack overflow or underflow. Max depth is 16.
/// Along test print suitable info and indications into the log.
///
/// Note:
/// - This function may use any of above defined Acceptance Test Plan (ATP) tests variables.
///   The caller must therefore call the suitable stldTester_Atp_Set* functions prior to calling this function.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Atp_DoTest461_11e_Isr2_Bit11 ()
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_11e_Isr2_Bit11.Start\n");

#ifdef USE_DDC_LIBRARY

	sitalProcess_Log_PrintLine ("#StldTester.TestRelevancy.Error: error - %s\n", "function_sitalBc_Command_DistortWord_undefined _by_DDC_API");
	return sitalReturnCode_SUCCESS;

#else // ! USE_DDC_LIBRARY

	S16BIT swResult; // Result of operation or function call.
	swResult = sitalReturnCode_SUCCESS;

	/// Get device count.
	U16BIT wRequiredDeviceCount; // The number of devices required for this test.
	wRequiredDeviceCount = (1 + s_swDevice2);
	U16BIT wAvailableDeviceCount; // The total number of devices available in the system.
	swResult = sitalDevice_GetCount (&wAvailableDeviceCount);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %i\n", swResult);
		return swResult;
	}
	sitalProcess_Log_PrintLine ("#StldTester.Device.Count: available - %i, required - %i\n", wAvailableDeviceCount, wRequiredDeviceCount);
	if (wAvailableDeviceCount < wRequiredDeviceCount)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %s\n", "not_enough_devices_installed");
		return sitalReturnCode_SUCCESS;
	}

	try
	{
		swResult = stldSyncStart (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncStart.Error: error - %i\n", swResult);
			throw 0;
		}

		// Sub-test #0: Verify that function sitalBc_Start rejects a major-frame that contains an underflow error (i.e., return command with no previous subroutine call command).
		sitalProcess_Log_PrintLine ("#StldTester.SubTest0\n");

		/// Initialize device-2 (BC) as a real BC.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-bc, alloc_mem-none\n", s_swDevice2);
		swResult = sitalDevice_Initialize (s_swDevice2, sitalAccess_CARD, (sitalMode_BC | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-0, a return command.
		// Note: This return command has no preceding subroutine-call command, and must therefore cause a condition of underflow.
		S16BIT swCommandId_0; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_0, sitalOpcode_RETURN_FROM_SUBROUTINE, sitalOpcodeCondition_ALWAYS, 0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_0, sitalOpcode_RETURN_FROM_SUBROUTINE, sitalOpcodeCondition_ALWAYS, 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) major-frame-0, a frame that contains command-0.
		S16BIT swFrameId_0; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_0 = 0;
		S16BIT swaCommandIds[sitalBcCounter_COMMANDS]; // An array of command-IDs.
		swaCommandIds[0] = swCommandId_0;
		U16BIT wCommandIdCount; // The number of command IDs in the frame (1-sitalBcCounter_FRAME_COMMANDS).
		wCommandIdCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_0, "major", wCommandIdCount, swCommandId_0, 0, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_0, sitalBcFrameType_MAJOR, swaCommandIds, wCommandIdCount, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Start device-2 (BC) running major-frame-0 for 1-shot.
		// As major-frame-0 contains an underflow error (i.e., return command with no previous subroutine call command), this frame must be rejected by function sitalBc_Start!
		U16BIT wFrameShotCount; // Number of frame shots.
		wFrameShotCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", s_swDevice2, swFrameId_0, wFrameShotCount);
		swResult = sitalBc_Start (s_swDevice2, swFrameId_0, wFrameShotCount);
		if (sitalReturnCode_ILLEGAL_FRAME != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// Free device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice2);
		swResult = sitalDevice_Free (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		// Sub-test #1: Verify that a BC device rejects with an interrupt a major-frame that contains an underflow error (i.e., return command with no previous subroutine call command).
		sitalProcess_Log_PrintLine ("#StldTester.SubTest1\n");

		/// Initialize device-2 (BC) as a real BC.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-bc, alloc_mem-none\n", s_swDevice2);
		swResult = sitalDevice_Initialize (s_swDevice2, sitalAccess_CARD, (sitalMode_BC | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Configure device-2 (BC) to issue desired interrupts.
		U16BIT wRegisterValue; // The current value of a target register, or the value to set for a target register.
		wRegisterValue = sitalInterruptMaskRegister2_BC_CALL_STACK_ERROR;
		sitalProcess_Log_PrintLine ("#StldTester.Device.Imr2.Set: unit - %i, imr2(post) - %04X\n", s_swDevice2, wRegisterValue);
		swResult = sitalDevice_Register_Write (s_swDevice2, sitalRegisterAddress_INTERRUPT_MASK_2, wRegisterValue);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Imr2.Set.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Get for device-2 (BC) the current interrupt configuration.
		U16BIT wInterruptMode; // Current configuration for type of interrupt signal (sitalIrqMode_*).
		U16BIT wAutoClear; // Current configuration for post-read auto-clearing of interrupt status (sitalIrqClear_*).
		swResult = sitalDevice_Irq_GetMode (s_swDevice2, &wInterruptMode, &wAutoClear);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Mode.Get.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Configure device-2 (BC) to issue suitable interrupts and perform post-read auto-clearing of interrupt status.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Configure: unit - %i, mode - %i, auto_clear - %i\n", s_swDevice2, wInterruptMode, sitalIrqClear_NO_AUTO_CLEAR);
		swResult = sitalDevice_Irq_Configure (s_swDevice2, wInterruptMode, sitalIrqClear_NO_AUTO_CLEAR);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Configure.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Configure device-2 (BC) to get desired notifications using an ISR that collects interrupt data, counts ISR calls, records the interrupt mask, and signals interested threads.
		U32BIT dwIrqMask; // A mask designating a set of interrupts that should be enabled/disabled as indicated by parameter bIsInterruptEnabled (An or-ed combination of sitalInterruptRegister*).
		dwIrqMask = sitalInterruptRegister2_BC_CALL_STACK_ERROR;
		sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Manipulate: unit - %i, enable - %i, irq - %08X, callback - %s\n", s_swDevice2, TRUE, dwIrqMask, "stldTester_Bc_UserIsr");
		swResult = sitalDevice_Irq_Manipulate (s_swDevice2, TRUE, dwIrqMask, stldTester_Bc_UserIsr);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Set.Error: error - %i\n", swResult);
			throw 0;
		}
		s_dwBcIsrCallCounter = 0;
		s_dwBcIrqStatus = 0;

		/// Create for device-2 (BC) command-0, a halt command.
		// Note: This halt command will be eventually distorted into a return command (see further explanation below).
		swCommandId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_0, sitalOpcode_RETURN_FROM_SUBROUTINE, sitalOpcodeCondition_ALWAYS, 0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_0, sitalOpcode_HALT, sitalOpcodeCondition_ALWAYS, 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) major-frame-0, a frame that contains command-0.
		swFrameId_0 = 0;
		swaCommandIds[0] = swCommandId_0;
		wCommandIdCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_0, "major", wCommandIdCount, swCommandId_0, 0, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_0, sitalBcFrameType_MAJOR, swaCommandIds, wCommandIdCount, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Configure the stld1553 library to insert such a distortion of the HALT command-0 that will turn it into a return command.
		// Since function sitalBc_Start rejects major-frames that contain an underflow error, some bypass must be done in order to force the creation and execution of such a major-frame.
		// The easiest way will be to exploit the command distortion mechanism, which operates beneath the hoods so that function sitalBc_Start isn't aware of it, in a way that will turn each halt command to a return from subroutine command.
		U16BIT wCommandWord; // Command word.
		swResult = sitalBc_Command_GetWord (sitalOpcode_RETURN_FROM_SUBROUTINE, sitalOpcodeCondition_ALWAYS, &wCommandWord);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.GetWord.Error: error - %i\n", swResult);
			throw 0;
		}
		swResult = sitalBc_Command_DistortWord (sitalOpcode_HALT, sitalOpcodeDistortion_SetCommand, wCommandWord);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Distortion.On.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Start device-2 (BC) running major-frame-0 for 1-shot.
		wFrameShotCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", s_swDevice2, swFrameId_0, wFrameShotCount);
		swResult = sitalBc_Start (s_swDevice2, swFrameId_0, wFrameShotCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Wait for the expected interrupt to be issued.
		U32BIT dwSleepTime; // Sleeping period (in ms).
		dwSleepTime = 10; // The device should set the expected interrupt within this time.
		sitalProcess_Log_PrintLine ("#StldTester.Wait: time - %ims\n", dwSleepTime);
		sitalProcess_Log_PrintLine ("\tWAIT_FOR %i us\n", (dwSleepTime * 1000));
		PrecisionTimer pt; // A precision timer.
		pt.AccuratelySleep (dwSleepTime);

		///	Stop device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop: unit - %i\n", s_swDevice2);
		swResult = sitalBc_Stop (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Cancel the previously configured command distortion.
		swResult = sitalBc_Command_DistortWord (sitalOpcode_HALT, sitalOpcodeDistortion_None, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Distortion.Off.Error: error - %i\n", swResult);
			throw 0;
		}

		/// As long a not all expected interrupts occurred AND the maximum possible interrupt latency still didn't pass:
		///		Wait.
		stldInterruptLatency_Wait (&s_dwBcIsrCallCounter, 1);

		/// Verify that a correct number of interrupts of a suitable type occurred.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Interrupts: unit - %i, count - %i, expected_count - %i, last_mask - %08X, expected_mask - %08X\n", s_swDevice2, s_dwBcIsrCallCounter, 1, s_dwBcIrqStatus, (sitalInterruptRegister2_MASTER_INTERRUPT | dwIrqMask));
		if (1 != s_dwBcIsrCallCounter)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Interrupt.CallCount.Error: error - %i\n", s_dwBcIsrCallCounter);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		if (dwIrqMask != (dwIrqMask & s_dwBcIrqStatus))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Interrupt.Mask.Error: error - %08X\n", s_dwBcIrqStatus);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		if (0U == (sitalInterruptRegister2_MASTER_INTERRUPT & s_dwBcIrqStatus))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Interrupt.Mask.Error: error - %08X\n", s_dwBcIrqStatus);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// Verify for device-2 (BC) that register ISR1 has been auto-cleared.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Register.ISR1.Read: unit - %i, expected_value - %04X\n", s_swDevice2, 0x0000);
		wRegisterValue = sitalDevice_Register_Read (s_swDevice2, sitalRegisterAddress_INTERRUPT_STATUS_1);
		// Added ignore timetag rollover interrupt (0x40)
		if (0U != (wRegisterValue & 0xFFBF))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Register.ISR1.Read.Error: error - %04X\n", wRegisterValue);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// Verify for device-2 (BC) that register ISR2 has been auto-cleared.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Register.ISR2.Read: unit - %i, expected_value - %04X\n", s_swDevice2, 0x0000);
		wRegisterValue = sitalDevice_Register_Read (s_swDevice2, sitalRegisterAddress_INTERRUPT_STATUS_2);
		// Added ignore timetag rollover interrupt (0x40)
		if (0U != (wRegisterValue & 0xFFBF))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Register.ISR2.Read.Error: error - %04X\n", wRegisterValue);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// Free device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice2);
		swResult = sitalDevice_Free (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		// Sub-test #2: Verify that function sitalBc_Start rejects a major-frame that contains an overflow error (i.e., a too deep nesting series of subroutine call commands).
		sitalProcess_Log_PrintLine ("#StldTester.SubTest2\n");

		/// Initialize device-2 (BC) as a real BC.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-bc, alloc_mem-none\n", s_swDevice2);
		swResult = sitalDevice_Initialize (s_swDevice2, sitalAccess_CARD, (sitalMode_BC | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) commands 0...(sitalBcMaximum_FRAME_NESTING_STACK - 1), subroutine call commands, where command-(N) executes minor-frame-(N+1). 
		/// Create for device-2 (BC) frames 0...(sitalBcMaximum_FRAME_NESTING_STACK - 1), where frame-(N) contains command-(N). 
		S16BIT swCommandId; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		S16BIT swCallerFrameId; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		S16BIT swCalledFrameId; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		for (swCommandId=0, swCallerFrameId = 0, swCalledFrameId = 1; swCommandId<sitalBcMaximum_FRAME_NESTING_STACK; swCommandId++, swCallerFrameId++, swCalledFrameId++)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, swCalledFrameId, 0);
			swResult = sitalBc_Command_Create (s_swDevice2, swCommandId, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, (U16BIT)swCalledFrameId, 0, 0);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
				throw 0;
			}

			swaCommandIds[0] = swCommandId;
			wCommandIdCount = 1;
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %i, flags - %i\n", s_swDevice2, swCallerFrameId, "minor", wCommandIdCount, swaCommandIds[0], 0, 0);
			swResult = sitalBc_Frame_Create (s_swDevice2, swCallerFrameId, sitalBcFrameType_MINOR, swaCommandIds, wCommandIdCount, 0, 0);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
				throw 0;
			}
		}

		/// Create for device-2 (BC) command-(sitalBcMaximum_FRAME_NESTING_STACK), a return from subroutine command.
		swCommandId = sitalBcMaximum_FRAME_NESTING_STACK;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId, sitalOpcode_RETURN_FROM_SUBROUTINE, sitalOpcodeCondition_ALWAYS, 0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId, sitalOpcode_RETURN_FROM_SUBROUTINE, sitalOpcodeCondition_ALWAYS, 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) minor-frame-(sitalBcMaximum_FRAME_NESTING_STACK), a frame that contains command-(sitalBcMaximum_FRAME_NESTING_STACK).
		S16BIT swFrameId; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId = sitalBcMaximum_FRAME_NESTING_STACK;
		swaCommandIds[0] = swCommandId;
		wCommandIdCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %i, flags - %i\n", s_swDevice2, swFrameId, "minor", wCommandIdCount, swaCommandIds[0], 0, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId, sitalBcFrameType_MINOR, swaCommandIds, wCommandIdCount, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-(sitalBcMaximum_FRAME_NESTING_STACK+1), a subroutine call command that executes minor-frame-0.
		swCommandId = (sitalBcMaximum_FRAME_NESTING_STACK + 1);
		swFrameId = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, swFrameId, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, (U16BIT)swFrameId, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) major-frame-(sitalBcMaximum_FRAME_NESTING_STACK + 1), a frame that contains command-(sitalBcMaximum_FRAME_NESTING_STACK + 1).
		swFrameId = (sitalBcMaximum_FRAME_NESTING_STACK + 1);
		swaCommandIds[0] = (sitalBcMaximum_FRAME_NESTING_STACK + 1);
		wCommandIdCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId, "major", wCommandIdCount, swaCommandIds[0], 0, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId, sitalBcFrameType_MAJOR, swaCommandIds, wCommandIdCount, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Start device-2 (BC) running major-frame-(sitalBcMaximum_FRAME_NESTING_STACK + 1) for 1-shot.
		swFrameId = (sitalBcMaximum_FRAME_NESTING_STACK + 1);
		wFrameShotCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", s_swDevice2, swFrameId, wFrameShotCount);
		swResult = sitalBc_Start (s_swDevice2, swFrameId, wFrameShotCount);
		if (sitalReturnCode_FRAME_NESTING_STACK_OVERFLOW != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// Free device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice2);
		swResult = sitalDevice_Free (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		// Sub-test #3: Verify that a BC device rejects with an interrupt a major-frame that contains an overflow error (i.e., a too deep nesting series of subroutine call commands).
		sitalProcess_Log_PrintLine ("#StldTester.SubTest3\n");

		/// Initialize device-2 (BC) as a real BC.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-bc, alloc_mem-none\n", s_swDevice2);
		swResult = sitalDevice_Initialize (s_swDevice2, sitalAccess_CARD, (sitalMode_BC | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Configure device-2 (BC) to issue desired interrupts.
		wRegisterValue = sitalInterruptMaskRegister2_BC_CALL_STACK_ERROR;
		sitalProcess_Log_PrintLine ("#StldTester.Device.Imr2.Set: unit - %i, imr2(post) - %04X\n", s_swDevice2, wRegisterValue);
		swResult = sitalDevice_Register_Write (s_swDevice2, sitalRegisterAddress_INTERRUPT_MASK_2, wRegisterValue);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Imr2.Set.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Get for device-2 (BC) the current interrupt configuration.
		swResult = sitalDevice_Irq_GetMode (s_swDevice2, &wInterruptMode, &wAutoClear);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Mode.Get.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Configure device-2 (BC) to issue suitable interrupts and perform post-read auto-clearing of interrupt status.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Configure: unit - %i, mode - %i, auto_clear - %i\n", s_swDevice2, wInterruptMode, sitalIrqClear_NO_AUTO_CLEAR);
		swResult = sitalDevice_Irq_Configure (s_swDevice2, wInterruptMode, sitalIrqClear_NO_AUTO_CLEAR);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Configure.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Configure device-2 (BC) to get desired notifications using an ISR that collects interrupt data, counts ISR calls, records the interrupt mask, and signals interested threads.
		dwIrqMask = sitalInterruptRegister2_BC_CALL_STACK_ERROR;
		sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Manipulate: unit - %i, enable - %i, irq - %08X, callback - %s\n", s_swDevice2, TRUE, dwIrqMask, "stldTester_Bc_UserIsr");
		swResult = sitalDevice_Irq_Manipulate (s_swDevice2, TRUE, dwIrqMask, stldTester_Bc_UserIsr);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Set.Error: error - %i\n", swResult);
			throw 0;
		}
		s_dwBcIsrCallCounter = 0;
		s_dwBcIrqStatus = 0;

		/// Create for device-2 (BC) commands 0...(sitalBcMaximum_FRAME_NESTING_STACK - 3), subroutine call commands, where command-(N) executes minor-frame-(N+1). 
		/// Create for device-2 (BC) frames 0...(sitalBcMaximum_FRAME_NESTING_STACK - 3), where frame-(N) contains command-(N). 
		for (swCommandId=0, swCallerFrameId = 0, swCalledFrameId = 1; swCommandId<=(sitalBcMaximum_FRAME_NESTING_STACK - 3); swCommandId++, swCallerFrameId++, swCalledFrameId++)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, swCalledFrameId, 0);
			swResult = sitalBc_Command_Create (s_swDevice2, swCommandId, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, (U16BIT)swCalledFrameId, 0, 0);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
				throw 0;
			}

			swaCommandIds[0] = swCommandId;
			wCommandIdCount = 1;
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %i, flags - %i\n", s_swDevice2, swCallerFrameId, "minor", wCommandIdCount, swaCommandIds[0], 0, 0);
			swResult = sitalBc_Frame_Create (s_swDevice2, swCallerFrameId, sitalBcFrameType_MINOR, swaCommandIds, wCommandIdCount, 0, 0);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
				throw 0;
			}
		}

		/// Create for device-2 (BC) command-(sitalBcMaximum_FRAME_NESTING_STACK - 2), a halt command.
		// Note: This halt command will be eventually distorted into an additional subroutine call command (see further explanation below).
		swCommandId = (sitalBcMaximum_FRAME_NESTING_STACK - 2);
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId, sitalOpcode_HALT, sitalOpcodeCondition_ALWAYS, 0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId, sitalOpcode_HALT, sitalOpcodeCondition_ALWAYS, 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) minor-frame-(sitalBcMaximum_FRAME_NESTING_STACK - 2), a frame that contains command-(sitalBcMaximum_FRAME_NESTING_STACK - 1).
		swFrameId = (sitalBcMaximum_FRAME_NESTING_STACK - 2);
		swaCommandIds[0] = swCommandId;
		wCommandIdCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %i, flags - %i\n", s_swDevice2, swFrameId, "minor", wCommandIdCount, swaCommandIds[0], 0, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId, sitalBcFrameType_MINOR, swaCommandIds, wCommandIdCount, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-(sitalBcMaximum_FRAME_NESTING_STACK - 1), a subroutine call command that executes minor-frame-0.
		swCommandId = (sitalBcMaximum_FRAME_NESTING_STACK - 1);
		swFrameId = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, swFrameId, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, (U16BIT)swFrameId, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) major-frame-(sitalBcMaximum_FRAME_NESTING_STACK - 1), a frame that contains command-(sitalBcMaximum_FRAME_NESTING_STACK).
		swFrameId = (sitalBcMaximum_FRAME_NESTING_STACK - 1);
		swaCommandIds[0] = (sitalBcMaximum_FRAME_NESTING_STACK - 1);
		wCommandIdCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId, "major", wCommandIdCount, swaCommandIds[0], 0, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId, sitalBcFrameType_MAJOR, swaCommandIds, wCommandIdCount, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Configure the stld1553 library to insert such a distortion of the HALT command-(sitalBcMaximum_FRAME_NESTING_STACK - 1) that will turn it into a subroutine call command.
		// Since function sitalBc_Start rejects major-frames that contain a subroutine call overflow error, some bypass must be done in order to force the creation and execution of such a major-frame.
		// The easiest way will be to exploit the command distortion mechanism, which operates beneath the hoods so that function sitalBc_Start isn't aware of it, in a way that will turn each halt command to a subroutine call command.
		swResult = sitalBc_Command_GetWord (sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, &wCommandWord);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.GetWord.Error: error - %i\n", swResult);
			throw 0;
		}
		swResult = sitalBc_Command_DistortWord (sitalOpcode_HALT, sitalOpcodeDistortion_SetCommand, wCommandWord);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Distortion.On.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Start device-2 (BC) running major-frame-(sitalBcMaximum_FRAME_NESTING_STACK) for 1-shot.
		swFrameId = (sitalBcMaximum_FRAME_NESTING_STACK - 1);
		wFrameShotCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", s_swDevice2, swFrameId, wFrameShotCount);
		swResult = sitalBc_Start (s_swDevice2, swFrameId, wFrameShotCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// Wait for the expected interrupt to be issued.
		dwSleepTime = 10; // The device should set the expected interrupt within this time.
		sitalProcess_Log_PrintLine ("#StldTester.Wait: time - %ims\n", dwSleepTime);
		sitalProcess_Log_PrintLine ("\tWAIT_FOR %i us\n", (dwSleepTime * 1000));
		pt.AccuratelySleep (dwSleepTime);

		///	Stop device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop: unit - %i\n", s_swDevice2);
		swResult = sitalBc_Stop (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Cancel the previously configured command distortion.
		swResult = sitalBc_Command_DistortWord (sitalOpcode_HALT, sitalOpcodeDistortion_None, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Distortion.Off.Error: error - %i\n", swResult);
			throw 0;
		}

		/// As long a not all expected interrupts occurred AND the maximum possible interrupt latency still didn't pass:
		///		Wait.
		stldInterruptLatency_Wait (&s_dwBcIsrCallCounter, 1);

		/// Verify that a correct number of interrupts of a suitable type occurred.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Interrupts: unit - %i, count - %i, expected_count - %i, last_mask - %08X, expected_mask - %08X\n", s_swDevice2, s_dwBcIsrCallCounter, 1, s_dwBcIrqStatus, (sitalInterruptRegister2_MASTER_INTERRUPT | dwIrqMask));
		if (1 != s_dwBcIsrCallCounter)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Interrupt.CallCount.Error: error - %i\n", s_dwBcIsrCallCounter);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		if (dwIrqMask != (dwIrqMask & s_dwBcIrqStatus))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Interrupt.Mask.Error: error - %08X\n", s_dwBcIrqStatus);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		if (0U == (sitalInterruptRegister2_MASTER_INTERRUPT & s_dwBcIrqStatus))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Interrupt.Mask.Error: error - %08X\n", s_dwBcIrqStatus);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// Verify for device-2 (BC) that register ISR1 has been auto-cleared.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Register.ISR1.Read: unit - %i, expected_value - %04X\n", s_swDevice2, 0x0000);
		wRegisterValue = sitalDevice_Register_Read (s_swDevice2, sitalRegisterAddress_INTERRUPT_STATUS_1);
		// Added ignore timetag rollover interrupt (0x40)
		if (0U != (wRegisterValue & 0xFFBF))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Register.ISR1.Read.Error: error - %04X\n", wRegisterValue);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// Verify for device-2 (BC) that register ISR2 has been auto-cleared.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Register.ISR2.Read: unit - %i, expected_value - %04X\n", s_swDevice2, 0x0000);
		wRegisterValue = sitalDevice_Register_Read (s_swDevice2, sitalRegisterAddress_INTERRUPT_STATUS_2);
		// Added ignore timetag rollover interrupt (0x40)
		if (0U != (wRegisterValue & 0xFFBF))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Register.ISR2.Read.Error: error - %04X\n", wRegisterValue);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// Free device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice2);
		swResult = sitalDevice_Free (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		swResult = stldSyncEnd (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncEnd.Error: error - %i\n", swResult);
			throw 0;
		}
	}
	catch (int)
	{
		sitalDevice_Free (s_swDevice2);
	}

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_11e_Isr2_Bit11.End: test_succeeded - %c\n", ((sitalReturnCode_SUCCESS == swResult) ? 'Y' : 'N'));
	return swResult;

#endif // USE_DDC_LIBRARY
}



/// <summary>
/// Test the stld1553 library in the henceforth described scenario:
/// Participant[s]: BC2.
/// Configuration: All participating devices must be able to communicate with each other.
/// Short description: Acceptance Test Plan (ATP) test 461_11e - Selectable interrupt generation for ISR-2 Bit-12 General Purpose queue rollover.
/// Along test print suitable info and indications into the log.
///
/// Note:
/// - This function may use any of above defined Acceptance Test Plan (ATP) tests variables.
///   The caller must therefore call the suitable stldTester_Atp_Set* functions prior to calling this function.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Atp_DoTest461_11e_Isr2_Bit12 ()
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_11e_Isr2_Bit12.Start\n");

	S16BIT swResult; // Result of operation or function call.
	swResult = sitalReturnCode_SUCCESS;

	/// Get device count.
	U16BIT wRequiredDeviceCount; // The number of devices required for this test.
	wRequiredDeviceCount = (1 + s_swDevice2);
	U16BIT wAvailableDeviceCount; // The total number of devices available in the system.
	swResult = sitalDevice_GetCount (&wAvailableDeviceCount);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %i\n", swResult);
		return swResult;
	}
	sitalProcess_Log_PrintLine ("#StldTester.Device.Count: available - %i, required - %i\n", wAvailableDeviceCount, wRequiredDeviceCount);
	if (wAvailableDeviceCount < wRequiredDeviceCount)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %s\n", "not_enough_devices_installed");
		return sitalReturnCode_SUCCESS;
	}

	try
	{
		swResult = stldSyncStart (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncStart.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-2 (BC) as a real BC.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-bc, alloc_mem-none\n", s_swDevice2);
		swResult = sitalDevice_Initialize (s_swDevice2, sitalAccess_CARD, (sitalMode_BC | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Configure device-2 (BC) to issue desired interrupts.
		U16BIT wRegisterValue; // The current value of a target register, or the value to set for a target register.
		wRegisterValue = sitalInterruptMaskRegister2_BC_GENERAL_PURPOSE_QUEUE_ROLLOVER;
		sitalProcess_Log_PrintLine ("#StldTester.Device.Imr2.Set: unit - %i, imr2(post) - %04X\n", s_swDevice2, wRegisterValue);
		swResult = sitalDevice_Register_Write (s_swDevice2, sitalRegisterAddress_INTERRUPT_MASK_2, wRegisterValue);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Imr2.Set.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Get for device-2 (BC) the current interrupt configuration.
		U16BIT wInterruptMode; // Current configuration for type of interrupt signal (sitalIrqMode_*).
		U16BIT wAutoClear; // Current configuration for post-read auto-clearing of interrupt status (sitalIrqClear_*).
		swResult = sitalDevice_Irq_GetMode (s_swDevice2, &wInterruptMode, &wAutoClear);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Mode.Get.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Configure device-2 (BC) to issue suitable interrupts and perform post-read auto-clearing of interrupt status.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Configure: unit - %i, mode - %i, auto_clear - %i\n", s_swDevice2, wInterruptMode, sitalIrqClear_NO_AUTO_CLEAR);
		swResult = sitalDevice_Irq_Configure (s_swDevice2, wInterruptMode, sitalIrqClear_NO_AUTO_CLEAR);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Configure.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Configure device-2 (BC) to get desired notifications using an ISR that collects interrupt data, counts ISR calls, records the interrupt mask, and signals interested threads.
		U32BIT dwIrqMask; // A mask designating a set of interrupts that should be enabled/disabled as indicated by parameter bIsInterruptEnabled (An or-ed combination of sitalInterruptRegister*).
		dwIrqMask = sitalInterruptRegister2_BC_GENERAL_PURPOSE_QUEUE_ROLLOVER;
		sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Manipulate: unit - %i, enable - %i, irq - %08X, callback - %s\n", s_swDevice2, TRUE, dwIrqMask, "stldTester_Bc_UserIsr");
		swResult = sitalDevice_Irq_Manipulate (s_swDevice2, TRUE, dwIrqMask, stldTester_Bc_UserIsr);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Set.Error: error - %i\n", swResult);
			throw 0;
		}
		s_dwBcIsrCallCounter = 0;
		s_dwBcIrqStatus = 0;

		/// Create for device-2 (BC) command-0, a push-value-to-GPQ command.
		S16BIT swCommandId_0; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_0, sitalOpcode_PUSH_VALUE, sitalOpcodeCondition_ALWAYS, 0xFFFF, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_0, sitalOpcode_PUSH_VALUE, sitalOpcodeCondition_ALWAYS, 0xFFFF, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) major-frame-0, a frame that contains command-0.
		S16BIT swFrameId_0; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_0 = 0;
		S16BIT swaCommandIds[sitalBcCounter_COMMANDS]; // An array of command-IDs.
		swaCommandIds[0] = swCommandId_0;
		U16BIT wCommandIdCount; // The number of command IDs in the frame (1-sitalBcCounter_FRAME_COMMANDS).
		wCommandIdCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_0, "major", wCommandIdCount, swCommandId_0, 0, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_0, sitalBcFrameType_MAJOR, swaCommandIds, wCommandIdCount, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Start device-2 (BC) running major-frame-0 for 1-shot.
		U16BIT wFrameShotCount; // Number of frame shots.
		wFrameShotCount = (sitalBcCounter_GPQ_ENTRIES * bcMemoryObjectSize_GPQ_ENTRY);
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", s_swDevice2, swFrameId_0, wFrameShotCount);
		swResult = sitalBc_Start (s_swDevice2, swFrameId_0, wFrameShotCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Wait for the expected interrupt to be issued.
		U32BIT dwSleepTime; // Sleeping period (in ms).
		dwSleepTime = 10; // The device should set the expected interrupt within this time.
		sitalProcess_Log_PrintLine ("#StldTester.Wait: time - %ims\n", dwSleepTime);
		sitalProcess_Log_PrintLine ("\tWAIT_FOR %i us\n", (dwSleepTime * 1000));
		PrecisionTimer pt; // A precision timer.
		pt.AccuratelySleep (dwSleepTime);

		///	Stop device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop: unit - %i\n", s_swDevice2);
		swResult = sitalBc_Stop (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// As long a not all expected interrupts occurred AND the maximum possible interrupt latency still didn't pass:
		///		Wait.
		stldInterruptLatency_Wait (&s_dwBcIsrCallCounter, 1);

		/// Verify that a correct number of interrupts of a suitable type occurred.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Interrupts: unit - %i, count - %i, expected_count - %i, last_mask - %08X, expected_mask - %08X\n", s_swDevice2, s_dwBcIsrCallCounter, 1, s_dwBcIrqStatus, (sitalInterruptRegister2_MASTER_INTERRUPT | dwIrqMask));
		if (1 != s_dwBcIsrCallCounter)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Interrupt.CallCount.Error: error - %i\n", s_dwBcIsrCallCounter);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		if (dwIrqMask != (dwIrqMask & s_dwBcIrqStatus))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Interrupt.Mask.Error: error - %08X\n", s_dwBcIrqStatus);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		if (0U == (sitalInterruptRegister2_MASTER_INTERRUPT & s_dwBcIrqStatus))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Interrupt.Mask.Error: error - %08X\n", s_dwBcIrqStatus);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// Verify for device-2 (BC) that register ISR1 has been auto-cleared.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Register.ISR1.Read: unit - %i, expected_value - %04X\n", s_swDevice2, 0x0000);
		wRegisterValue = sitalDevice_Register_Read (s_swDevice2, sitalRegisterAddress_INTERRUPT_STATUS_1);
		// Added ignore timetag rollover interrupt (0x40)
		if (0U != (wRegisterValue & 0xFFBF))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Register.ISR1.Read.Error: error - %04X\n", wRegisterValue);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// Verify for device-2 (BC) that register ISR2 has been auto-cleared.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Register.ISR2.Read: unit - %i, expected_value - %04X\n", s_swDevice2, 0x0000);
		wRegisterValue = sitalDevice_Register_Read (s_swDevice2, sitalRegisterAddress_INTERRUPT_STATUS_2);
		// Added ignore timetag rollover interrupt (0x40)
		if (0U != (wRegisterValue & 0xFFBF))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Register.ISR2.Read.Error: error - %04X\n", wRegisterValue);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// Free device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice2);
		swResult = sitalDevice_Free (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		swResult = stldSyncEnd (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncEnd.Error: error - %i\n", swResult);
			throw 0;
		}
	}
	catch (int)
	{
		sitalDevice_Free (s_swDevice2);
	}

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_11e_Isr2_Bit12.End: test_succeeded - %c\n", ((sitalReturnCode_SUCCESS == swResult) ? 'Y' : 'N'));
	return swResult;
}



/// <summary>
/// Test the stld1553 library in the henceforth described scenario:
/// Participant[s]: BC2.
/// Configuration: All participating devices must be able to communicate with each other.
/// Short description: Acceptance Test Plan (ATP) test 461_11e - Selectable interrupt generation for ISR-2 Bit-14 Enhanced BC instruction parity error detected.
/// Along test print suitable info and indications into the log.
///
/// Note:
/// - This function may use any of above defined Acceptance Test Plan (ATP) tests variables.
///   The caller must therefore call the suitable stldTester_Atp_Set* functions prior to calling this function.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Atp_DoTest461_11e_Isr2_Bit14 ()
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	S16BIT swResult; // Result of operation or function call.
	swResult = sitalReturnCode_SUCCESS;

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_11e_Isr2_Bit14.Start\n");

	swResult = stldTester_Atp_DoTest461_11e_Isr2_Bit10 ();

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_11e_Isr2_Bit14.End: test_succeeded - %c\n", ((sitalReturnCode_SUCCESS == swResult) ? 'Y' : 'N'));
	return swResult;
}



/// <summary>
/// Test the stld1553 library in the henceforth described scenario:
/// Participant[s]: BC2, MT3.
/// Configuration: All participating devices must be able to communicate with each other.
/// Short description: Acceptance Test Plan (ATP) test 461_11e - Selectable interrupt generation for ISR-1 Bits 10-11 and ISR-2 Bits 06-07 MT data stack 50% and 100% fullness.
/// Along test print suitable info and indications into the log.
///
/// Note:
/// - This function may use any of above defined Acceptance Test Plan (ATP) tests variables.
///   The caller must therefore call the suitable stldTester_Atp_Set* functions prior to calling this function.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Atp_DoTest461_11e_Isr1_Bits10To11_And_Isr2_Bits06To07 ()
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_11e_Isr1_Bits10To11_And_Isr2_Bits06To07.Start\n");

	/// Set test arguments.
	U16BIT wRtSubaddress; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31) that's set for a RT for Rx.
	wRtSubaddress = 1;
	U16BIT wWordCount; // Word count (1-32).
	wWordCount = 32;
	U16BIT wFrameTime; // The time (in units of 100 us) that the frame will take to complete.
	wFrameTime = 200; // 20ms.

	S16BIT swResult; // Result of operation or function call.
	swResult = sitalReturnCode_SUCCESS;

	/// Get device count.
	U16BIT wRequiredDeviceCount; // The number of devices required for this test.
	wRequiredDeviceCount = (1 + max(s_swDevice2, s_swDevice3));
	U16BIT wAvailableDeviceCount; // The total number of devices available in the system.
	swResult = sitalDevice_GetCount (&wAvailableDeviceCount);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %i\n", swResult);
		return swResult;
	}
	sitalProcess_Log_PrintLine ("#StldTester.Device.Count: available - %i, required - %i\n", wAvailableDeviceCount, wRequiredDeviceCount);
	if (wAvailableDeviceCount < wRequiredDeviceCount)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %s\n", "not_enough_devices_installed");
		return sitalReturnCode_SUCCESS;
	}
	if ( (s_swDevice2 == s_swDevice3) )
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Number.Error: error - %s\n", "same_device_used_twice");
		return sitalReturnCode_INVALID_DEVICE_NUMBER;
	}

	try
	{
		swResult = stldSyncStart (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncStart.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Get device-3 (MT) information.
		sitalDeviceCapabilitiesStructure dcsDeviceCapabilities;
		dcsDeviceCapabilities.dwStructureSize = sizeof(sitalDeviceCapabilitiesStructure);
		swResult = sitalDevice_GetInformation (s_swDevice3, &dcsDeviceCapabilities);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.GetInformation.Error: error - %i\n", swResult);
			throw 0;
		}
		sitalProcess_Log_PrintLine ("#StldTester.Device.Information: unit - %i, ram_size - %X\n", s_swDevice3, dcsDeviceCapabilities.dwRamSize);

		/// Initialize device-2 (BC) as a real BC.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-bc, alloc_mem-none\n", s_swDevice2);
		swResult = sitalDevice_Initialize (s_swDevice2, sitalAccess_CARD, (sitalMode_BC | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) data-block-0, a data block of 32 data words.
		S16BIT swDataBlockId_0; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId_0 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i\n", s_swDevice2, swDataBlockId_0, 32, 0);
		swResult = sitalBc_DataBlock_Create (s_swDevice2, swDataBlockId_0, sitalBcDataBlockSizeOption_SINGLE, NULL, wWordCount);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) message-0, a BC-to-Broadcast(RtSubAddress) message that uses data-block-1 to transmit WordCount data words. Message bus: bus-1.
		// A BC-to-Broadcast message with 32 data words is created, as such a message requires exactly 32 words in the MT data stack, what makes the calculations of MT data stack fullness simpler due to the fact that the sizes possible for a MT data stack are all multiplications of 32 (see below).
		S16BIT swMessageId_0; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToBroadcast.Create: unit - %i, message - %i, block - %i, rx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_0, swDataBlockId_0, wRtSubaddress, wWordCount, 0, s_wBus1);
		swResult = sitalBc_Message_CreateBcToBroadcast (s_swDevice2, swMessageId_0, swDataBlockId_0, wRtSubaddress, wWordCount, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToBroadcast.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-0, a XEQ command that executes message-0.
		S16BIT swCommandId_0; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-1, a subroutine call command that executes minor-frame-0.
		S16BIT swCommandId_1; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_1 = 1;
		S16BIT swFrameId_0; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_1, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, swFrameId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_1, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, (U16BIT)swFrameId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) minor-frame-0, a frame that contains command-0.
		S16BIT swaCommandIds[sitalBcCounter_COMMANDS]; // An array of command-IDs.
		swaCommandIds[0] = swCommandId_0;
		U16BIT wCommandIdCount; // The number of command IDs in the frame (1-sitalBcCounter_FRAME_COMMANDS).
		wCommandIdCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_0, "minor", wCommandIdCount, swCommandId_0, 0, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_0, sitalBcFrameType_MINOR, swaCommandIds, wCommandIdCount, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) major-frame-1, a frame that contains command-1 and whose time is FrameTime.
		S16BIT swFrameId_1; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_1 = 1;
		swaCommandIds[0] = swCommandId_1;
		wCommandIdCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_1, "major", wCommandIdCount, swCommandId_1, wFrameTime, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_1, sitalBcFrameType_MAJOR, swaCommandIds, wCommandIdCount, wFrameTime, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Loop over both MT stack types (i.e., command and data stacks):
		for (S16BIT swIndexOfTestedStackType=0; swIndexOfTestedStackType<2; swIndexOfTestedStackType++)
		{
			sitalProcess_Log_PrintLine ("#StldTester.TestIteration: stack_type - %i/%i\n", swIndexOfTestedStackType, 2);

			static const U16BIT s_waMtCommandStackSize[] // Array of MT command stack size constants.
								= { sitalMtCommandStackSize_256, sitalMtCommandStackSize_1024, sitalMtCommandStackSize_4096, sitalMtCommandStackSize_16384 };
			static const U16BIT s_waMtCommandStackWords[] // Array of MT command stack sizes (in words).
								= { 256, 1024, 4096, 16384 };
			static const U16BIT s_waMtCommandStackMessageCount[] // Array of MT command stack sizes (in messages), where each entry relates the corresponding entry in array s_waMtCommandStackSize.
								= { 64, 256, 1024, 4096 }; // Note: This size is calculated based on the fact that each message requires 4 words in the MT command stack.
			static const U16BIT s_wMtCommandStackSizeCount // The number of sizes possible for a MT command stack.
								= ( sizeof(s_waMtCommandStackSize) / sizeof(s_waMtCommandStackSize[0]) );

			static const U16BIT s_waMtDataStackSize[] // Array of MT data stack size constants.
								= { sitalMtDataStackSize_512, sitalMtDataStackSize_1024, sitalMtDataStackSize_2048, sitalMtDataStackSize_4096, sitalMtDataStackSize_8192, sitalMtDataStackSize_16384, sitalMtDataStackSize_32768 };
			static const U16BIT s_waMtDataStackWords[] // Array of MT data stack sizes (in words).
								= { 512, 1024, 2048, 4096, 8192, 16384, 32768 };
			static const U16BIT s_waMtDataStackMessageCount[] // Array of MT data stack sizes (in messages), where each entry relates the corresponding entry in array s_waMtDataStackSize.
								= { 16, 32, 64, 128, 256, 512, 1024 }; // Note: This size is calculated based on the assumption that all transmitted messages are BC-to-BCST messages of 32 data words; each such a message requires exactly 32 words in the MT data stack.
			static const U16BIT s_wMtDataStackSizeCount // The number of sizes possible for a MT data stack.
								= ( sizeof(s_waMtDataStackSize) / sizeof(s_waMtDataStackSize[0]) );

			U16BIT* wapMtStackSize; // A pointer to the array of stack size constants for the currently tested type of MT stack.
			U16BIT* wapMtStackWords; // A pointer to the array of stack sizes (in words) for the currently tested type of MT stack.
			U16BIT* wapMtStackMessageCount; // A pointer to the array of stack sizes (in messages) for the currently tested type of MT stack.
			U16BIT wMtStackSizeCount; // The number of sizes possible for the currently tested type of MT stack.
			if (0U == swIndexOfTestedStackType)
			{
				wapMtStackSize = (U16BIT*)s_waMtCommandStackSize;
				wapMtStackWords = (U16BIT*)s_waMtCommandStackWords;
				wapMtStackMessageCount = (U16BIT*)s_waMtCommandStackMessageCount;
				wMtStackSizeCount = s_wMtCommandStackSizeCount;
			}
			else
			{
				wapMtStackSize = (U16BIT*)s_waMtDataStackSize;
				wapMtStackWords = (U16BIT*)s_waMtDataStackWords;
				wapMtStackMessageCount = (U16BIT*)s_waMtDataStackMessageCount;
				wMtStackSizeCount = s_wMtDataStackSizeCount;
			}

			///		Loop over all possible sizes of the currently tested type of MT stack:
			for (U16BIT wMtStackSizeIndex=0; wMtStackSizeIndex<wMtStackSizeCount; wMtStackSizeIndex++)
			{
				sitalProcess_Log_PrintLine ("#StldTester.TestIteration: stack_type - %i/%i, stack_size - %i/%i\n", swIndexOfTestedStackType, 2, wMtStackSizeIndex, wMtStackSizeCount);

				///			Initialize device-3 (MT) as a real MT.
				sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access - card, mode - mt, alloc_mem - none\n", s_swDevice3);
				swResult = sitalDevice_Initialize (s_swDevice3, sitalAccess_CARD, ((s_bIsRtMtModeTest ? sitalMode_RT_AND_MT : sitalMode_MT) | sitalModeVariant_ADVANCED), 0, 0, 0);
				if (sitalReturnCode_SUCCESS != swResult)
				{
					sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
					throw 0;
				}

				///			Initialize device-3 (MT) to have the currently tested combination of stack sizes.
				// Note:
				// - When the command / data stack is tested, the other stack (data / command, respectively), is set to its minimum possible size, so that the maximum amount of device memory will be left for the allocation of the tested stack.
				// - This function should not care whether the currently tested combination of stack sizes may be allocated in device-3 (MT); Function sitalMt_Initialize will reject a request that exceeds the capacity of given device.
				sitalProcess_Log_PrintLine ("#StldTester.Mt.Init: unit - %i, stack_count - %i, command_stack_size - %i, data_stack_size - %i, options - %X\n", s_swDevice3, sitalMtStackOption_SINGLE, sitalMtCommandStackSize_256, wapMtStackSize[wMtStackSizeIndex], 0);
				if (0U == swIndexOfTestedStackType)
				{
					swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Initialize (s_swDevice3, sitalRtCommandStackSize_256, sitalMtStackOption_SINGLE, wapMtStackSize[wMtStackSizeIndex], sitalMtDataStackSize_512, 0) : sitalMt_Initialize (s_swDevice3, sitalMtStackOption_SINGLE, wapMtStackSize[wMtStackSizeIndex], sitalMtDataStackSize_512, 0) );
				}
				else
				{
					swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Initialize (s_swDevice3, sitalRtCommandStackSize_256, sitalMtStackOption_SINGLE, sitalMtCommandStackSize_256, wapMtStackSize[wMtStackSizeIndex], 0) : sitalMt_Initialize (s_swDevice3, sitalMtStackOption_SINGLE, sitalMtCommandStackSize_256, wapMtStackSize[wMtStackSizeIndex], 0) );
				}
				if (sitalReturnCode_ALLOCATION_FAIL != swResult)
				{
					if (sitalReturnCode_SUCCESS != swResult)
					{
						sitalProcess_Log_PrintLine ("#StldTester.Mt.Initialize.Error: error - %i\n", swResult);
						throw 0;
					}

					///			Configure device-3 (MT) to issue desired interrupts.
					U16BIT wRegisterValue; // The current value of a target register, or the value to set for a target register.
					wRegisterValue = ( (0U == swIndexOfTestedStackType) ? sitalInterruptMaskRegister1_MT_COMMAND_STACK_ROLLOVER : sitalInterruptMaskRegister1_MT_DATA_STACK_ROLLOVER );
					sitalProcess_Log_PrintLine ("#StldTester.Device.Imr1.Set: unit - %i, imr1(post) - %04X\n", s_swDevice3, wRegisterValue);
					swResult = sitalDevice_Register_Write (s_swDevice3, sitalRegisterAddress_INTERRUPT_MASK_1, wRegisterValue);
					if (sitalReturnCode_SUCCESS != swResult)
					{
						sitalProcess_Log_PrintLine ("#StldTester.Device.Imr1.Set.Error: error - %i\n", swResult);
						throw 0;
					}
					wRegisterValue = ( (0U == swIndexOfTestedStackType) ? sitalInterruptMaskRegister2_MT_COMMAND_STACK_HALF_ROLLOVER : sitalInterruptMaskRegister2_MT_DATA_STACK_HALF_ROLLOVER );
					sitalProcess_Log_PrintLine ("#StldTester.Device.Imr2.Set: unit - %i, imr2(post) - %04X\n", s_swDevice3, wRegisterValue);
					swResult = sitalDevice_Register_Write (s_swDevice3, sitalRegisterAddress_INTERRUPT_MASK_2, wRegisterValue);
					if (sitalReturnCode_SUCCESS != swResult)
					{
						sitalProcess_Log_PrintLine ("#StldTester.Device.Imr2.Set.Error: error - %i\n", swResult);
						throw 0;
					}

					/// Get for device-3 (MT) the current interrupt configuration.
					U16BIT wInterruptMode; // Current configuration for type of interrupt signal (sitalIrqMode_*).
					U16BIT wAutoClear; // Current configuration for post-read auto-clearing of interrupt status (sitalIrqClear_*).
					swResult = sitalDevice_Irq_GetMode (s_swDevice3, &wInterruptMode, &wAutoClear);
					if (sitalReturnCode_SUCCESS != swResult)
					{
						sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Mode.Get.Error: error - %i\n", swResult);
						throw 0;
					}

					///			Configure device-3 (MT) to issue suitable interrupts and perform post-read auto-clearing of interrupt status.
					sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Configure: unit - %i, mode - %i, auto_clear - %i\n", s_swDevice3, wInterruptMode, sitalIrqClear_NO_AUTO_CLEAR);
					swResult = sitalDevice_Irq_Configure (s_swDevice3, wInterruptMode, sitalIrqClear_NO_AUTO_CLEAR);
					if (sitalReturnCode_SUCCESS != swResult)
					{
						sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Configure.Error: error - %i\n", swResult);
						throw 0;
					}

					///			Configure device-3 (MT) to get desired notifications using an ISR that collects interrupt data, counts ISR calls, records the interrupt mask, and signals interested threads.
					U32BIT dwIrqMask; // A mask designating a set of interrupts that should be enabled/disabled as indicated by parameter bIsInterruptEnabled (An or-ed combination of sitalInterruptRegister*).
					dwIrqMask = ( (0U == swIndexOfTestedStackType) ? (sitalInterruptRegister1_MT_COMMAND_STACK_ROLLOVER | sitalInterruptRegister2_MT_COMMAND_STACK_HALF_ROLLOVER) : (sitalInterruptRegister1_MT_DATA_STACK_ROLLOVER | sitalInterruptRegister2_MT_DATA_STACK_HALF_ROLLOVER) );
					sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Manipulate: unit - %i, enable - %i, irq - %08X, callback - %s\n", s_swDevice3, TRUE, dwIrqMask, "stldTester_Mt_UserIsr");
					swResult = sitalDevice_Irq_Manipulate (s_swDevice3, TRUE, dwIrqMask, stldTester_Mt_UserIsr);
					if (sitalReturnCode_SUCCESS != swResult)
					{
						sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Set.Error: error - %i\n", swResult);
						throw 0;
					}

					///			Start device-3 (MT).
					sitalProcess_Log_PrintLine ("#StldTester.Mt.Start: unit - %i\n", s_swDevice3);
					swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Start (s_swDevice3) : sitalMt_Start (s_swDevice3) );
					if (sitalReturnCode_SUCCESS != swResult)
					{
						sitalProcess_Log_PrintLine ("#StldTester.Mt.Start.Error: error - %i\n", swResult);
						throw 0;
					}

					///			Loop over all the defined MT stack fullness interrupts (i.e., the first / second iteration tests the half-full / full stack interrupt, respectively):
					for (S16BIT swIndexOfTestedInterrupt=0; swIndexOfTestedInterrupt<2; swIndexOfTestedInterrupt++)
					{
						sitalProcess_Log_PrintLine ("#StldTester.TestIteration: stack_type - %i/%i, stack_size - %i/%i, interrupt - %i/%i\n", swIndexOfTestedStackType, 2, wMtStackSizeIndex, wMtStackSizeCount, swIndexOfTestedInterrupt, 2);

						s_dwMtIsrCallCounter = 0;
						s_dwMtIrqStatus = 0;

						U16BIT wMessageCountInHalfStack; // Half MT stack size (in messages).
						wMessageCountInHalfStack = (wapMtStackMessageCount[wMtStackSizeIndex] >> 1);

						U16BIT wFrameCount; // The number of frames to run.
						wFrameCount = (wMessageCountInHalfStack - 1); // Each frame contains a single message, so a transmission of FrameCount-shots is expected to ONLY ALMOST fill half / fully fill (depending on the currently tested interrupt) the currently tested MT stack.

						///				Start device-2 (BC) running major-frame-1 for FrameCount-shot.
						sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", s_swDevice2, swFrameId_1, wFrameCount);
						swResult = sitalBc_Start (s_swDevice2, swFrameId_1, wFrameCount);
						if (sitalReturnCode_SUCCESS != swResult)
						{
							sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
							throw 0;
						}

						///				Wait for the frame run to end.
						U32BIT dwRemainingFrameRunTime; // The expected remaining frame run time (in ms).
						dwRemainingFrameRunTime = (wFrameCount * (wFrameTime / 10));
						swResult = stldTester_Bc_WaitForEndOfFrameRun (s_swDevice2, dwRemainingFrameRunTime, 0, NULL);
						if (sitalReturnCode_SUCCESS != swResult)
						{
							sitalProcess_Log_PrintLine ("#StldTester.Bc.WaitForEndOfFrameRun.Error: error - %i\n", swResult);
							throw 0;
						}

						///				Stop device-2 (BC).
						sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop: unit - %i\n", s_swDevice2);
						swResult = sitalBc_Stop (s_swDevice2);
						if (sitalReturnCode_SUCCESS != swResult)
						{
							sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
							throw 0;
						}

						///				Verify that the MT did not issue any interrupt.
						if (0U != s_dwMtIsrCallCounter)
						{
							sitalProcess_Log_PrintLine ("#StldTester.TestIteration.Error: stack_type - %i/%i, stack_size - %i/%i, interrupt - %i/%i\n", swIndexOfTestedStackType, 2, wMtStackSizeIndex, wMtStackSizeCount, swIndexOfTestedInterrupt, 2);
							sitalProcess_Log_PrintLine ("#StldTester.Device.Interrupt.CallCount.Error: error - %i\n", s_dwMtIsrCallCounter);
							swResult = sitalReturnCode_TEST_FAIL;
							throw 0;
						}

						s_dwMtIsrCallCounter = 0;
						s_dwMtIrqStatus = 0;

						wFrameCount = 1; // Each frame contains a single message, so a transmission of one more shot is expected to ACCOMPLISH the half / fully filling (depending on the currently tested interrupt) of the currently tested MT stack.

						///				Start device-2 (BC) running major-frame-1 for FrameCount-shot.
						sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", s_swDevice2, swFrameId_1, wFrameCount);
						swResult = sitalBc_Start (s_swDevice2, swFrameId_1, wFrameCount);
						if (sitalReturnCode_SUCCESS != swResult)
						{
							sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
							throw 0;
						}

						///				Wait for the frame run to end.
						dwRemainingFrameRunTime = (wFrameCount * (wFrameTime / 10));
						swResult = stldTester_Bc_WaitForEndOfFrameRun (s_swDevice2, dwRemainingFrameRunTime, 0, NULL);
						if (sitalReturnCode_SUCCESS != swResult)
						{
							sitalProcess_Log_PrintLine ("#StldTester.Bc.WaitForEndOfFrameRun.Error: error - %i\n", swResult);
							throw 0;
						}

						///				Stop device-2 (BC).
						sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop: unit - %i\n", s_swDevice2);
						swResult = sitalBc_Stop (s_swDevice2);
						if (sitalReturnCode_SUCCESS != swResult)
						{
							sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
							throw 0;
						}

						///				Verify that the MT issued a single interrupt of the currently tested type.
						if (1 != s_dwMtIsrCallCounter)
						{
							sitalProcess_Log_PrintLine ("#StldTester.TestIteration.Error: stack_type - %i/%i, stack_size - %i/%i, interrupt - %i/%i\n", swIndexOfTestedStackType, 2, wMtStackSizeIndex, wMtStackSizeCount, swIndexOfTestedInterrupt, 2);
							sitalProcess_Log_PrintLine ("#StldTester.Device.Interrupt.CallCount.Error: error - %i\n", s_dwMtIsrCallCounter);
							swResult = sitalReturnCode_TEST_FAIL;
							throw 0;
						}
						U32BIT dwCurrentlyTestedIrq; // The currently tested interrupt request.
						dwCurrentlyTestedIrq = ( (0U == swIndexOfTestedStackType) ? ((0U == swIndexOfTestedInterrupt) ? sitalInterruptRegister2_MT_COMMAND_STACK_HALF_ROLLOVER : sitalInterruptRegister1_MT_COMMAND_STACK_ROLLOVER ) : ((0U == swIndexOfTestedInterrupt) ? sitalInterruptRegister2_MT_DATA_STACK_HALF_ROLLOVER : sitalInterruptRegister1_MT_DATA_STACK_ROLLOVER ) );
						if (0U == (dwCurrentlyTestedIrq & s_dwMtIrqStatus))
						{
							sitalProcess_Log_PrintLine ("#StldTester.TestIteration.Error: stack_type - %i/%i, stack_size - %i/%i, interrupt - %i/%i\n", swIndexOfTestedStackType, 2, wMtStackSizeIndex, wMtStackSizeCount, swIndexOfTestedInterrupt, 2);
							sitalProcess_Log_PrintLine ("#StldTester.Device.Interrupt.Mask.Error: error - %08X\n", s_dwMtIrqStatus);
							swResult = sitalReturnCode_TEST_FAIL;
							throw 0;
						}
						sitalProcess_Log_PrintLine ("#StldTester.Device.Interrupts: unit - %i, count - %i, expected_count - %i, last_mask - %08X, expected_mask - %08X\n", s_swDevice3, s_dwMtIsrCallCounter, 1, s_dwMtIrqStatus, dwCurrentlyTestedIrq);
					}

					/// 		Stop device-3 (MT).
					sitalProcess_Log_PrintLine ("#StldTester.Mt.Stop: unit - %i\n", s_swDevice3);
					swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Stop (s_swDevice3) : sitalMt_Stop (s_swDevice3) );
					if (sitalReturnCode_SUCCESS != swResult)
					{
						sitalProcess_Log_PrintLine ("#StldTester.Mt.Stop.Error: error - %i\n", swResult);
						throw 0;
					}
				}
				else
				{
					sitalProcess_Log_PrintLine ("#StldTester.TestIteration.StackSizeTooBigForDeviceRamSize: stack_type - %i/%i, stack_size - %i/%i\n", swIndexOfTestedStackType, 2, wMtStackSizeIndex, wMtStackSizeCount);
				}

				/// 		Free device-3 (MT).
				sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice3);
				swResult = sitalDevice_Free (s_swDevice3);
				if (sitalReturnCode_SUCCESS != swResult)
				{
					sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
					throw 0;
				}
			}
		}

		/// Free device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice2);
		swResult = sitalDevice_Free (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		swResult = stldSyncEnd (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncEnd.Error: error - %i\n", swResult);
			throw 0;
		}
	}
	catch (int)
	{
		sitalDevice_Free (s_swDevice3);
		sitalDevice_Free (s_swDevice2);
	}

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_11e_Isr1_Bits10To11_And_Isr2_Bits06To07.End: test_succeeded - %c\n", ((sitalReturnCode_SUCCESS == swResult) ? 'Y' : 'N'));
	return swResult;
}



/// <summary>
/// Test the stld1553 library in the henceforth described scenario:
/// Participant[s]: RT1, BC2, MT3.
/// Configuration: All participating devices must be able to communicate with each other.
/// Short description: Acceptance Test Plan (ATP) test 461_11f - Stop on error/status bit.
/// Along test print suitable info and indications into the log.
///
/// Note:
/// - This function may use any of above defined Acceptance Test Plan (ATP) tests variables.
///   The caller must therefore call the suitable stldTester_Atp_Set* functions prior to calling this function.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Atp_DoTest461_11f ()
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_11f.Start\n");

	/// Set test arguments.
	U16BIT wRtSubaddressRx; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31) that's set for a RT for Rx.
	wRtSubaddressRx = sitalRtSubaddress_1;
	U16BIT wRtSubaddressTx; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31) that's set for a RT for Tx.
	wRtSubaddressTx = sitalRtSubaddress_2;
	U16BIT wWordCount; // Word count (1-32).
	wWordCount = 32;
	U16BIT wFrameTime; // The time (in units of 100 us) that the frame will take to complete.
	wFrameTime = 100; // 10ms.
	U16BIT wFrameShotCount; // Number of frame shots.
	wFrameShotCount = 10;

	S16BIT swResult; // Result of operation or function call.
	swResult = sitalReturnCode_SUCCESS;

	/// Get device count.
	U16BIT wRequiredDeviceCount; // The number of devices required for this test.
	wRequiredDeviceCount = (1 + max(max(s_swDevice1, s_swDevice2), s_swDevice3));
	U16BIT wAvailableDeviceCount; // The total number of devices available in the system.
	swResult = sitalDevice_GetCount (&wAvailableDeviceCount);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %i\n", swResult);
		return swResult;
	}
	sitalProcess_Log_PrintLine ("#StldTester.Device.Count: available - %i, required - %i\n", wAvailableDeviceCount, wRequiredDeviceCount);
	if (wAvailableDeviceCount < wRequiredDeviceCount)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %s\n", "not_enough_devices_installed");
		return sitalReturnCode_SUCCESS;
	}
	if ( (s_swDevice1 == s_swDevice2) || (s_swDevice1 == s_swDevice3) || (s_swDevice2 == s_swDevice3))
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Number.Error: error - %s\n", "same_device_used_twice");
		return sitalReturnCode_INVALID_DEVICE_NUMBER;
	}

	try
	{
		swResult = stldSyncStart (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncStart.Error: error - %i\n", swResult);
			throw 0;
		}

		// Loop over all sub-tests:
		for (int iSubTest=1; iSubTest<=2; iSubTest++)
		{
			sitalProcess_Log_PrintLine ("#StldTester.TestLoop: sub_test - %i\n", iSubTest);

			/// 	Initialize device-1 (RT) as a real RT.
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-rt, alloc_mem-none\n", s_swDevice1);
			swResult = sitalDevice_Initialize (s_swDevice1, sitalAccess_CARD, ((s_bIsRtMtModeTest ? sitalMode_RT_AND_MT : sitalMode_RT) | sitalModeVariant_ADVANCED), 0, 0, 0);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
				throw 0;
			}
			if (s_bIsRtMtModeTest)
			{
				// Accommodate with tests where the required amount of allocations of data blocks for RT devices is to big when these RT devices are initialized as RT-MT devices with a rather limited amount of memory.
				sitalProcess_Log_PrintLine ("#StldTester.RtMt.Init: unit - %i, rt_command_stack_size - %i, mt_stack_count - %i, mt_command_stack_size - %i, mt_data_stack_size - %i, options - %X\n", s_swDevice1, sitalRtCommandStackSize_256, sitalMtStackOption_SINGLE, sitalMtCommandStackSize_256, sitalMtDataStackSize_512, 0);
				swResult = sitalRtMt_Initialize (s_swDevice1, sitalRtCommandStackSize_256, sitalMtStackOption_SINGLE, sitalMtCommandStackSize_256, sitalMtDataStackSize_512, 0);
				if (sitalReturnCode_SUCCESS != swResult)
				{
					sitalProcess_Log_PrintLine ("#StldTester.RtMt.Init.Error: error - %i\n", swResult);
					throw 0;
				}
			}

			/// 	Set device-1 (RT) address to RT-address-1.
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set: unit - %i, address - %i\n", s_swDevice1, s_wRtAddress1);
			swResult = sitalRt_Address_Set (s_swDevice1, s_wRtAddress1);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set.Error: error - %i\n", swResult);
				throw 0;
			}

			U16BIT wRtSubaddress; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31).
			const U16BIT waMessageType[] // Array of message types.
							= { sitalRtMessageType_RX, sitalRtMessageType_TX, sitalRtMessageType_BROADCAST};
			U16BIT wMessageTypeCount; // Number of defined message types.
			wMessageTypeCount = ( sizeof(waMessageType) / sizeof(waMessageType[0]) );
			S16BIT swDataBlockId; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
			swDataBlockId = 1;
			U16BIT waBuffer[32]; // A buffer to read-from/write-into along data-block write/read operations, respectively.
			for (int i=0; i<wWordCount; i++) waBuffer[i] = (wWordCount - 1 - i);
			for (wRtSubaddress=sitalRtSubaddress_1; wRtSubaddress<=sitalRtSubaddress_30; wRtSubaddress++)
			{
				for (int i=0; i<wMessageTypeCount; i++)
				{
					/// 	Create for device-1 (RT) data-block-DataBlockId, a data block of 64 data words.
					sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i, buf_contents-...%i\n", s_swDevice1, swDataBlockId, 64, wWordCount, (wWordCount - 1));
					swResult = sitalRt_DataBlock_Create (s_swDevice1, swDataBlockId, sitalRtDataBlockType_SINGLE_32, waBuffer, wWordCount);
					if (sitalReturnCode_SUCCESS != swResult)
					{
						sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.Create.Error: error - %i\n", swResult);
						throw 0;
					}

					/// 	Map for device-1 (RT) data-block-DataBlockId with message-type-j for subaddress-i, and legalize this subaddress.
					sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.MapToSa: unit - %i, block - %i, sa - %i, msg_types - x%X, irq_opts-x%X, legalize - %c\n", s_swDevice1, swDataBlockId, wRtSubaddress, waMessageType[i], sitalRtDataBlockIrq_NONE, 'Y');
					swResult = sitalRt_DataBlock_MapToSubaddress (s_swDevice1, swDataBlockId, wRtSubaddress, waMessageType[i], sitalRtDataBlockIrq_NONE, TRUE);
					if (sitalReturnCode_SUCCESS != swResult)
					{
						sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.MapToSa.Error: error - %i\n", swResult);
						throw 0;
					}

					/// 	Move to next DataBlockId.
					swDataBlockId++;
				}
			}

			/// 	Start device-1 (RT).
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Start: unit - %i\n", s_swDevice1);
			swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Start (s_swDevice1) : sitalRt_Start (s_swDevice1) );
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Rt.Start.Error: error - %i\n", swResult);
				throw 0;
			}

			/// 	Initialize device-3 (MT) as a real MT.
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access - card, mode - mt, alloc_mem - none\n", s_swDevice3);
			swResult = sitalDevice_Initialize (s_swDevice3, sitalAccess_CARD, ((s_bIsRtMtModeTest ? sitalMode_RT_AND_MT : sitalMode_MT) | sitalModeVariant_ADVANCED), 0, 0, 0);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
				throw 0;
			}

			/// 	Start device-3 (MT).
			sitalProcess_Log_PrintLine ("#StldTester.Mt.Start: unit - %i\n", s_swDevice3);
			swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Start (s_swDevice3) : sitalMt_Start (s_swDevice3) );
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Mt.Start.Error: error - %i\n", swResult);
				throw 0;
			}

			/// 	Initialize device-2 (BC) as a real BC.
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-bc, alloc_mem-none\n", s_swDevice2);
			swResult = sitalDevice_Initialize (s_swDevice2, sitalAccess_CARD, (sitalMode_BC | sitalModeVariant_ADVANCED), 0, 0, 0);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
				throw 0;
			}

			/// 	Configure device-2 (BC) to stop frame running on error.
			U16BIT wRegisterValue; // The current value of a target register, or the value to set for a target register.
			wRegisterValue = sitalDevice_Register_Read (s_swDevice2, sitalRegisterAddress_CONFIGURATION_1);
			U16BIT wNewRegisterValue; // The current value of a target register, or the value to set for a target register.
			wNewRegisterValue = ( wRegisterValue | ( (1 == iSubTest) ? sitalConfigurationRegister1_BC_FRAME_STOP_ON_ERROR : (sitalConfigurationRegister1_BC_STATUS_SET_STOP_ON_MESSAGE | sitalConfigurationRegister1_BC_STATUS_SET_STOP_ON_FRAME) ) );
			sitalProcess_Log_PrintLine ("#StldTester.Device.SetFrameStopOnError: unit - %i, configuration_register1(pre) - %04X, configuration_register1(post) - %04X\n", s_swDevice2, wRegisterValue, wNewRegisterValue);
			swResult = sitalDevice_Register_Write (s_swDevice2, sitalRegisterAddress_CONFIGURATION_1, wNewRegisterValue);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Device.SetFrameStopOnError.Error: error - %i\n", swResult);
				throw 0;
			}

			/// 	Create for device-2 (BC) data-block-0, a data block of 64 data words.
			S16BIT swDataBlockId_0; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
			swDataBlockId_0 = 1;
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i\n", s_swDevice2, swDataBlockId_0, 64, 0);
			swResult = sitalBc_DataBlock_Create (s_swDevice2, swDataBlockId_0, sitalBcDataBlockSizeOption_DOUBLE, NULL, 0);
			if (0 > swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
				throw 0;
			}

			/// 	Create for device-2 (BC) message-0, a RT(RT-address-1/RtSubAddressTx)-to-BC message that uses data-block-0 to receive WordCount data words. Message bus: bus-1.
			S16BIT swMessageId_0; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
			swMessageId_0 = 0;
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBc.Create: unit - %i, message - %i, block - %i, tx_rt - %i, tx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddressTx, wWordCount, 0, s_wBus1);
			swResult = sitalBc_Message_CreateRtToBc (s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddressTx, wWordCount, 0, s_wBus1);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBc.Create.Error: error - %i\n", swResult);
				throw 0;
			}

			/// 	Create for device-2 (BC) command-0, an XQF command that executes message-0.
			S16BIT swCommandId_0; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
			swCommandId_0 = 0;
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0);
			swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0, 0);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
				throw 0;
			}

			/// 	Create for device-2 (BC) command-1, a subroutine call command that executes minor-frame-0. 
			S16BIT swCommandId_1; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
			swCommandId_1 = 1;
			S16BIT swFrameId_0; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
			swFrameId_0 = 0;
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_1, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, swFrameId_0, 0);
			swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_1, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, (U16BIT)swFrameId_0, 0, 0);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
				throw 0;
			}

			/// 	Create for device-2 (BC) minor-frame-0, a frame that contains command-0.
			S16BIT swaCommandIds[sitalBcCounter_COMMANDS]; // An array of command-IDs.
			swaCommandIds[0] = swCommandId_0;
			U16BIT wCommandIdCount; // The number of command IDs in the frame (1-sitalBcCounter_FRAME_COMMANDS).
			wCommandIdCount = 1;
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %i, flags - %i\n", s_swDevice2, swFrameId_0, "minor", wCommandIdCount, swCommandId_0, wFrameTime, 0);
			swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_0, sitalBcFrameType_MINOR, swaCommandIds, wCommandIdCount, wFrameTime, 0);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
				throw 0;
			}

			/// 	Create for device-2 (BC) major-frame-1, a frame that contains command-1 and whose frame time is FrameTime us.
			S16BIT swFrameId_1; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
			swFrameId_1 = 1;
			swaCommandIds[0] = swCommandId_1;
			wCommandIdCount = 1;
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_1, "major", wCommandIdCount, swCommandId_1, (wFrameTime / 10), 0);
			swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_1, sitalBcFrameType_MAJOR, swaCommandIds, wCommandIdCount, wFrameTime, 0);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
				throw 0;
			}

			/// 	Start device-2 (BC) running major-frame-1 forever.
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", s_swDevice2, swFrameId_1, sitalBcFrameIterations_RUN_FOREVER);
			swResult = sitalBc_Start (s_swDevice2, swFrameId_1, sitalBcFrameIterations_RUN_FOREVER);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
				throw 0;
			}

			/// 	Wait for FrameShotCount * FrameTime ms.
			U32BIT dwSleepTime; // Sleeping period (in ms).
			dwSleepTime = ((U32BIT)wFrameShotCount * ((U16BIT)(wFrameTime / IP_FREQUENCY_FACTOR) / 10));
			sitalProcess_Log_PrintLine ("#StldTester.Wait: time - %ims\n", dwSleepTime);
			sitalProcess_Log_PrintLine ("\tWAIT_FOR %i us\n", (dwSleepTime * 1000));
			PrecisionTimer pt; // A precision timer.
			pt.AccuratelySleep (dwSleepTime);

			///		Depending on current sub-test:
			//			Stop device-1 (RT).
			//		Or:
			//			Configure device-1 (RT) to respond with bad status.
			if (1 == iSubTest)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Rt.Stop: unit - %i\n", s_swDevice1);
				swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Stop (s_swDevice1) : sitalRt_Stop (s_swDevice1) );
				if (sitalReturnCode_SUCCESS != swResult)
				{
					sitalProcess_Log_PrintLine ("#StldTester.Rt.Stop.Error: error - %i\n", swResult);
					throw 0;
				}
			}
			else
			{
				sitalProcess_Log_PrintLine ("#StldTester.Rt.ResponseStatusBits.Set: unit - %i, mask - %04X\n", s_swDevice1, sitalConfigurationRegister1_RT_FLAG);
				swResult = sitalRt_ResponseStatusBits_Set (s_swDevice1, sitalConfigurationRegister1_RT_FLAG);
				if (sitalReturnCode_SUCCESS != swResult)
				{
					sitalProcess_Log_PrintLine ("#StldTester.Rt.ResponseStatusBits.Error: error - %i\n", swResult);
					throw 0;
				}
			}

			/// 	Wait for FrameTime ms to let device-2 (BC) stop frame running on error.
			//dwSleepTime = ( (wFrameTime / 10) + 1 ); // +1 just for safe, to compensate for a bit to short sleep or a bit to slow reaction of RT or BC device.
			// considering IP core frequency
			dwSleepTime = (((U16BIT)(wFrameTime / IP_FREQUENCY_FACTOR)/ 10) + 1); // +1 just for safe, to compensate for a bit to short sleep or a bit to slow reaction of RT or BC device.
			sitalProcess_Log_PrintLine ("#StldTester.Wait: time - %ims\n", dwSleepTime);
			sitalProcess_Log_PrintLine ("\tWAIT_FOR %i us\n", (dwSleepTime * 1000));
			pt.AccuratelySleep (dwSleepTime);

			/// 	Verify that device-2 (BC) has stopped the frame running on due error.
			sitalProcess_Log_PrintLine ("#StldTester.Bc.GetActivationState: unit - %i\n", s_swDevice2);
			U16BIT wCurrentState; // The current state of a BC device (sitalBcActivationState_*).
			swResult = sitalBc_GetActivationState (s_swDevice2, &wCurrentState);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.GetActivationState.Error: error - %i\n", swResult);
				throw 0;
			}
			if (sitalBcActivationState_IDLE != wCurrentState)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.GetActivationState.CurrentState.Error: error - %i\n", wCurrentState);
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}
			sitalDecodedMessageStructure dmsDecodedMessage; // A decoded message.
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded: unit - %i, message - %i, purge-y\n", s_swDevice2, swMessageId_0);
			swResult = sitalBc_Message_GetByIdDecoded (s_swDevice2, swMessageId_0, &dmsDecodedMessage, TRUE);
			if (0 > swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded.Error: error - %i\n", swResult);
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded: unit - %i, message - %i, purge-y, message_count - %i\n", s_swDevice2, swMessageId_0, swResult);
			if (1 != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded.Error: error - %i\n", swResult);
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message: BSW - %04X\n", dmsDecodedMessage.wBlockStatus);
			if (1 == iSubTest)
			{
				if (0U == ((sitalBcBlockStatusWord_ERROR_FLAG | sitalBcBlockStatusWord_FORMAT_ERROR | sitalBcBlockStatusWord_NO_RESPONSE) & dmsDecodedMessage.wBlockStatus))
				{
					sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BlockStatus.Error: error - %s\n", "NoErrorInBsw");
					swResult = sitalReturnCode_TEST_FAIL;
					throw 0;
				}
			}
			else
			{
				if	((sitalBcBlockStatusWord_END_OF_MESSAGE | sitalBcBlockStatusWord_STATUS_SET | sitalBcBlockStatusWord_MASKED_STATUS | sitalBcBlockStatusWord_GOOD_DATA_BLOCK_RECEIVED) != dmsDecodedMessage.wBlockStatus)
				{
					sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BlockStatus.Error: error - %s\n", "ExpectedBswIs0x8890");
					swResult = sitalReturnCode_TEST_FAIL;
					throw 0;
				}
			}

			/// 	Use given message descriptors to code the command word of above defined message.
			U16BIT wCommandWord0; // Command word for above defined RT-to-BC message-0.
			swResult = sitalBcStld1553_Command_GetCode (&wCommandWord0, s_wRtAddress1, sitalCommand_TX, wRtSubaddressTx, wWordCount);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Command.GetCode.Error: error - %i\n", swResult);
				throw 0;
			}

			/// 	Verify that device-3 (MT) has detected messages in the expected composition.
			U16BIT wRtToBcMessageCount; // The total number of RT-to-BC messages that have been identified.
			wRtToBcMessageCount = 0;
			do
			{
				sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded: unit - %i, purge - %i\n", s_swDevice3, TRUE);
				swResult = sitalMt_Message_GetFromStackDecoded (s_swDevice3, &dmsDecodedMessage, sitalMessageLocationAndRemoval_NEXT_PURGE, sitalMtStack_ACTIVE);
				if (1 == swResult)
				{
					if (wCommandWord0 == dmsDecodedMessage.wCommandWord1)
					{
						wRtToBcMessageCount++;
					}
					else
					{
						sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Message.Error: error - %04X\n", dmsDecodedMessage.wCommandWord1);
						swResult = sitalReturnCode_TEST_FAIL;
						throw 0;
					}
				}
			}
			while (1 == swResult);
			if (0 > swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Error: error - %i\n", swResult);
				throw 0;
			}
			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.MessageComposition: unit - %i, RT2BC(%04X) - %i, expected - %i-%i\n", s_swDevice3, wCommandWord0, wRtToBcMessageCount, wFrameShotCount, (wFrameShotCount + 2));
			if ((wFrameShotCount > wRtToBcMessageCount) || ((wFrameShotCount + 4) < wRtToBcMessageCount))
			{
				sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Error: error - %s\n", "unexpected_message_composition");
				printf("\tExpected 10:14 got %i probably because OS not RTime", wRtToBcMessageCount);
				swResult = sitalReturnCode_TEST_FAIL -1;
				throw 0;
			}
			/*else {
				printf("\tExpected 10:14 got %i", wRtToBcMessageCount);
			}*/

			/// 	Stop device-2 (BC).
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop: unit - %i\n", s_swDevice2);
			swResult = sitalBc_Stop (s_swDevice2);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
				throw 0;
			}

			/// 	If device-1 (RT) was not stopped yet:
			///			Stop device-1 (RT).
			if (2 == iSubTest)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Rt.Stop: unit - %i\n", s_swDevice1);
				swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Stop (s_swDevice1) : sitalRt_Stop (s_swDevice1) );
				if (sitalReturnCode_SUCCESS != swResult)
				{
					sitalProcess_Log_PrintLine ("#StldTester.Rt.Stop.Error: error - %i\n", swResult);
					throw 0;
				}
			}

			/// 	Stop device-3 (MT).
			sitalProcess_Log_PrintLine ("#StldTester.Mt.Stop: unit - %i\n", s_swDevice3);
			swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Stop (s_swDevice3) : sitalMt_Stop (s_swDevice3) );
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Mt.Stop.Error: error - %i\n", swResult);
				throw 0;
			}

			/// 	Free device-2 (BC).
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice2);
			swResult = sitalDevice_Free (s_swDevice2);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
				throw 0;
			}

			/// 	Free device-1 (RT).
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice1);
			swResult = sitalDevice_Free (s_swDevice1);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
				throw 0;
			}

			/// 	Free device-3 (MT).
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice3);
			swResult = sitalDevice_Free (s_swDevice3);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
				throw 0;
			}
		}

		swResult = stldSyncEnd (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncEnd.Error: error - %i\n", swResult);
			throw 0;
		}
	}
	catch (int)
	{
		sitalDevice_Free (s_swDevice3);
		sitalDevice_Free (s_swDevice2);
		sitalDevice_Free (s_swDevice1);
	}

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_11f.End: test_succeeded - %c\n", ((sitalReturnCode_SUCCESS == swResult) ? 'Y' : 'N'));
	return swResult;
}



/// <summary>
/// Test the stld1553 library in the henceforth described scenario:
/// Participant[s]: BC2, MT3.
/// Configuration: All participating devices must be able to communicate with each other.
/// Short description: Acceptance Test Plan (ATP) test 461_11g - Multiple store buffers with 16 bit time tag.
/// Along test print suitable info and indications into the log.
///
/// Note:
/// - This function may use any of above defined Acceptance Test Plan (ATP) tests variables.
///   The caller must therefore call the suitable stldTester_Atp_Set* functions prior to calling this function.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Atp_DoTest461_11g ()
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_11g.Start\n");

	/// Set test arguments.
	U16BIT wWordCount; // Word count (1-32).
	wWordCount = 32;
	U16BIT wRtSubaddress; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31).
	wRtSubaddress = 1;
	U16BIT wFrameTime; // The time (in units of 100 us) that the frame will take to complete.
	wFrameTime = 20; // 2ms.
	U16BIT wFrameCount; // The number of frames to run.
	wFrameCount = 128; // Note: This value assumes full-blown device memory space; in cases that's not the case, this value is properly adapted below.
	U16BIT wTimeTagCalculationInaccuracy; // The maximum tolerated difference between the expected time tag and the actual one.
	wTimeTagCalculationInaccuracy = 25;

	S16BIT swResult; // Result of operation or function call.
	swResult = sitalReturnCode_SUCCESS;

	/// Get device count.
	U16BIT wRequiredDeviceCount; // The number of devices required for this test.
	wRequiredDeviceCount = (1 + max(s_swDevice2, s_swDevice3));
	U16BIT wAvailableDeviceCount; // The total number of devices available in the system.
	swResult = sitalDevice_GetCount (&wAvailableDeviceCount);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %i\n", swResult);
		return swResult;
	}
	sitalProcess_Log_PrintLine ("#StldTester.Device.Count: available - %i, required - %i\n", wAvailableDeviceCount, wRequiredDeviceCount);
	if (wAvailableDeviceCount < wRequiredDeviceCount)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %s\n", "not_enough_devices_installed");
		return sitalReturnCode_SUCCESS;
	}
	if ( (s_swDevice2 == s_swDevice3) )
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Number.Error: error - %s\n", "same_device_used_twice");
		return sitalReturnCode_INVALID_DEVICE_NUMBER;
	}

	/// Get device-2 (BC) information.
	sitalDeviceCapabilitiesStructure dcsDeviceCapabilities;
	dcsDeviceCapabilities.dwStructureSize = sizeof(sitalDeviceCapabilitiesStructure);
	swResult = sitalDevice_GetInformation (s_swDevice2, &dcsDeviceCapabilities);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.GetInformation.Error: error - %i\n", swResult);
		throw 0;
	}
	sitalProcess_Log_PrintLine ("#StldTester.Device.Information: unit - %i, ram_size - %X\n", s_swDevice2, dcsDeviceCapabilities.dwRamSize);
	if (0x2000 /* 8K */ >= dcsDeviceCapabilities.dwRamSize)
	{
		wFrameCount = 24;
	}
	else
	if (0x4000 /* 16K */ >= dcsDeviceCapabilities.dwRamSize)
	{
		wFrameCount = 48;
	}

	try
	{
		swResult = stldSyncStart (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncStart.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-3 (MT) as a real MT.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access - card, mode - mt, alloc_mem - none\n", s_swDevice3);
		swResult = sitalDevice_Initialize (s_swDevice3, sitalAccess_CARD, ((s_bIsRtMtModeTest ? sitalMode_RT_AND_MT : sitalMode_MT) | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Start device-3 (MT).
		sitalProcess_Log_PrintLine ("#StldTester.Mt.Start: unit - %i\n", s_swDevice3);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Start (s_swDevice3) : sitalMt_Start (s_swDevice3) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-2 (BC) as a real BC.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-bc, alloc_mem-none\n", s_swDevice2);
		swResult = sitalDevice_Initialize (s_swDevice2, sitalAccess_CARD, (sitalMode_BC | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Set for device-2 (BC) a time tag resolution of 2 us.
		U16BIT wTimeTagResolution; // Time tag resolution (in units of 1 us).
		wTimeTagResolution = 2;
		sitalProcess_Log_PrintLine ("#StldTester.Device.TimeTagResolution.Set: unit - %i, resolution - %ius\n", s_swDevice2);
		swResult = sitalDevice_TimeTag_SetResolution (s_swDevice2, sitalTimeTagResolution_2US);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.TimeTagResolution.Set.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Loop to create for device-2 (BC) a number of FrameCount data blocks, messages, message execution commands, minor frames, and minor frame execution commands:
		// The data block, message, message execution command, and frame IDs will all be 1-FrameCount.
		// The minor frame execution command IDs will be (FrameCount+1)-(2*FrameCount).
		U16BIT waBuffer[32]; // A buffer to read-from/write-into along data-block write/read operations, respectively.
		S16BIT swDataBlockId; // Data block ID;
		S16BIT swMessageId; // Message ID;
		S16BIT swCommandId; // Command ID;
		U16BIT wCommandWord; // A 1553 command word.
		S16BIT swaCommandIds[sitalBcCounter_COMMANDS]; // An array of command-IDs.
		U16BIT wCommandIdCount; // The number of command IDs in a frame (1-sitalBcCounter_FRAME_COMMANDS).
		wCommandIdCount = 1;
		S16BIT swFrameId; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		for (S16BIT i=1; i<=wFrameCount; i++)
		{
			///		Create for device-2 (BC) data-block-DataBlockId, a data block of 64 data words, and fill it with unique data.
			swDataBlockId = i;
			for (int j=0; j<wWordCount; j++) waBuffer[j] = i;
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i, buf_contents - %i...%i\n", s_swDevice2, swDataBlockId, 64, wWordCount, i, i);
			swResult = sitalBc_DataBlock_Create (s_swDevice2, swDataBlockId, sitalBcDataBlockSizeOption_DOUBLE, waBuffer, wWordCount);
			if (0 > swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
				throw 0;
			}

			///		Create for device-2 (BC) message-MessageId, a BC-to-RT(RtAddress1/RtSubAddress) message that uses data-block-DataBlockId to transmit WordCount data words. Message bus: bus-1.
			swMessageId = i;
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create: unit - %i, message - %i, block - %i, tx_rt - %i, tx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId, swDataBlockId, s_wRtAddress1, wRtSubaddress, wWordCount, 0, s_wBus1);
			swResult = sitalBc_Message_CreateBcToRt (s_swDevice2, swMessageId, swDataBlockId, s_wRtAddress1, wRtSubaddress, wWordCount, 0, s_wBus1);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create.Error: error - %i\n", swResult);
				throw 0;
			}

			///		Use given message descriptors to code the command word of above defined message.
			// Note: Defined messages have been intentionally devised in a way that their [first] command word will differ.
			swResult = sitalBcStld1553_Command_GetCode (&wCommandWord, s_wRtAddress1, sitalCommand_RX, wRtSubaddress, wWordCount);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Command.GetCode.Error: error - %i\n", swResult);
				throw 0;
			}
			sitalProcess_Log_PrintLine ("#StldTester.Command.GetCode: rt_address - %i, direction - %i, subaddress_or_mode - %i, wc/mc - %i, code - %04X\n", s_wRtAddress1, sitalCommand_RX, wRtSubaddress, wWordCount, wCommandWord);

			///		Create for device-2 (BC) command-CommandId, a XEQ command that executes message-MessageId.
			swCommandId = i;
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId, 0);
			swResult = sitalBc_Command_Create (s_swDevice2, swCommandId, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId, 0, 0);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
				throw 0;
			}

			///		Create for device-2 (BC) minor-frame-FrameId, a frame that contains command-CommandId.
			swFrameId = i;
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId, "minor", wCommandIdCount, swaCommandIds[0], swCommandId, (wFrameTime / 10), 0);
			swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId, sitalBcFrameType_MINOR, &swCommandId, wCommandIdCount, wFrameTime, 0);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
				throw 0;
			}

			///		Create for device-2 (BC) command-CommandId, a subroutine call command that executes minor-frame-FrameId.
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, (wFrameCount + i), sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, swFrameId, 0);
			swResult = sitalBc_Command_Create (s_swDevice2, (wFrameCount + i), sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, (U16BIT)swFrameId, 0, 0);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
				throw 0;
			}
			swaCommandIds[i - 1] = (wFrameCount + i);
		}

		/// Create for device-2 (BC) major-frame-0, a frame that contains all above created minor frame execution commands and whose frame time is FrameTime us.
		swFrameId = 0;
		wCommandIdCount = wFrameCount;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i...%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId, "major", wCommandIdCount, (wFrameCount + 1), (2 * wFrameCount), (wFrameTime / 10), 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId, sitalBcFrameType_MAJOR, swaCommandIds, wCommandIdCount, wFrameTime, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Start device-2 (BC) running major-frame-0 for 1-shot.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", s_swDevice2, swFrameId, 1);
		swResult = sitalBc_Start (s_swDevice2, swFrameId, 1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Wait for the frame run to end.
		U32BIT dwRemainingFrameRunTime; // The expected remaining frame run time (in ms).
		dwRemainingFrameRunTime = (wFrameCount * (wFrameTime / 10));
		swResult = stldTester_Bc_WaitForEndOfFrameRun (s_swDevice2, dwRemainingFrameRunTime, 0, NULL);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.WaitForEndOfFrameRun.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Stop device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop: unit - %i\n", s_swDevice2);
		swResult = sitalBc_Stop (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Stop device-3 (MT).
		sitalProcess_Log_PrintLine ("#StldTester.Mt.Stop: unit - %i\n", s_swDevice3);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Stop (s_swDevice3) : sitalMt_Stop (s_swDevice3) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Verify that device-2 (BC) reports suitable incremental time tags for all messages.
		sitalDecodedMessageStructure dmsDecodedMessage; // A decoded message.
		U16BIT wExpectedTimeTag; // The next expected time tag.
		wExpectedTimeTag = 0;
		for (S16BIT i=1; i<=wFrameCount; i++)
		{
			swMessageId = i;
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded: unit - %i, message - %i, purge-y\n", s_swDevice2, swMessageId);
			swResult = sitalBc_Message_GetByIdDecoded (s_swDevice2, swMessageId, &dmsDecodedMessage, TRUE);
			if (1 != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded.Error: error - %i\n", swResult);
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}

			if (1 == i)
			{
				wExpectedTimeTag = dmsDecodedMessage.wTimeTag;
			}
			U16BIT wTimeTagDifference; // The difference between the expected time tag and the actual one.
			wTimeTagDifference = (wExpectedTimeTag - dmsDecodedMessage.wTimeTag);
			wTimeTagDifference = ( (0x8000 < wTimeTagDifference) ? (0x10000 - wTimeTagDifference) : wTimeTagDifference); // To handle a case of time tag rollover.
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message: unit - %i, time_tag - %04X, expected_time_tag - ~%04X\n", s_swDevice2, dmsDecodedMessage.wTimeTag, wExpectedTimeTag);
			if (wTimeTagCalculationInaccuracy < wTimeTagDifference)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.Error: error - %s\n", "bad_timing");
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}
			wExpectedTimeTag = ( dmsDecodedMessage.wTimeTag + (wFrameTime * 100 / wTimeTagResolution) ); // Frame time is expressed in units of 100us.
		}

		/// Verify that device-3 (MT) has detected messages in the expected composition containing the expected data.
		for (S16BIT i=1; i<=wFrameCount; i++)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded: unit - %i, purge - %i\n", s_swDevice3, TRUE);
			swResult = sitalMt_Message_GetFromStackDecoded (s_swDevice3, &dmsDecodedMessage, sitalMessageLocationAndRemoval_NEXT_PURGE, sitalMtStack_ACTIVE);
			if ((0 > swResult) || (1 < swResult))
			{
				sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Error: error - %i\n", swResult);
				throw 0;
			}
			else
			if (0U == swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Error: error - %s\n", "no_message");
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}
			else
			{
				sitalProcess_Log_PrintLine ("#StldTester.Mt.Message: unit - %i, message - %i, command - %04X, expected_command - %04X, data_words - %i(%04X...), expected_data_words - %i(%04X...)\n", s_swDevice3, i, dmsDecodedMessage.wCommandWord1, wCommandWord, dmsDecodedMessage.wDataWordCount, dmsDecodedMessage.waData[0], wWordCount, i);
				if (wCommandWord != dmsDecodedMessage.wCommandWord1)
				{
					sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Message.Error: error - %s\n", "bad_command");
					swResult = sitalReturnCode_TEST_FAIL;
					throw 0;
				}
				else
				if (wWordCount != dmsDecodedMessage.wDataWordCount)
				{
					sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Message.Error: error - %s\n", "bad_word_count");
					swResult = sitalReturnCode_TEST_FAIL;
					throw 0;
				}
				else
				if (i != dmsDecodedMessage.waData[0])
				{
					sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Message.Error: error - %s\n", "bad_data");
					swResult = sitalReturnCode_TEST_FAIL;
					throw 0;
				}
			}
		}

		/// Free device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice2);
		swResult = sitalDevice_Free (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-3 (MT).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice3);
		swResult = sitalDevice_Free (s_swDevice3);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		swResult = stldSyncEnd (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncEnd.Error: error - %i\n", swResult);
			throw 0;
		}
	}
	catch (int)
	{
		sitalDevice_Free (s_swDevice2);
		sitalDevice_Free (s_swDevice3);
	}

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_11g.End: test_succeeded - %c\n", ((sitalReturnCode_SUCCESS == swResult) ? 'Y' : 'N'));
	return swResult;
}



/// <summary>
/// Test the stld1553 library in the henceforth described scenario:
/// Participant[s]: BC2, MT3.
/// Configuration: All participating devices must be able to communicate with each other.
/// Short description: Acceptance Test Plan (ATP) test 461_11h - Message table capacity.
/// Along test print suitable info and indications into the log.
///
/// Note:
/// - This function may use any of above defined Acceptance Test Plan (ATP) tests variables.
///   The caller must therefore call the suitable stldTester_Atp_Set* functions prior to calling this function.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Atp_DoTest461_11h ()
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_11h.Start\n");

	/// Set test arguments.
	U16BIT wWordCount; // Word count (1-32).
	wWordCount = 32;
	S16BIT swMessageIdStep; // Message-ID-step to use in message creation loop.
	swMessageIdStep = 50;
	U16BIT wRtSubaddress; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31).
	wRtSubaddress = 1;
	U16BIT wFrameTime; // The time (in units of 100 us) that the frame will take to complete.
	wFrameTime = 100; // 10ms.

	S16BIT swResult; // Result of operation or function call.
	swResult = sitalReturnCode_SUCCESS;

	/// Get device count.
	U16BIT wRequiredDeviceCount; // The number of devices required for this test.
	wRequiredDeviceCount = (1 + max(s_swDevice2, s_swDevice3));
	U16BIT wAvailableDeviceCount; // The total number of devices available in the system.
	swResult = sitalDevice_GetCount (&wAvailableDeviceCount);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %i\n", swResult);
		return swResult;
	}
	sitalProcess_Log_PrintLine ("#StldTester.Device.Count: available - %i, required - %i\n", wAvailableDeviceCount, wRequiredDeviceCount);
	if (wAvailableDeviceCount < wRequiredDeviceCount)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %s\n", "not_enough_devices_installed");
		return sitalReturnCode_SUCCESS;
	}
	if ( (s_swDevice2 == s_swDevice3) )
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Number.Error: error - %s\n", "same_device_used_twice");
		return sitalReturnCode_INVALID_DEVICE_NUMBER;
	}

	try
	{
		swResult = stldSyncStart (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncStart.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-3 (MT) as a real MT.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access - card, mode - mt, alloc_mem - none\n", s_swDevice3);
		swResult = sitalDevice_Initialize (s_swDevice3, sitalAccess_CARD, ((s_bIsRtMtModeTest ? sitalMode_RT_AND_MT : sitalMode_MT) | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Start device-3 (MT).
		sitalProcess_Log_PrintLine ("#StldTester.Mt.Start: unit - %i\n", s_swDevice3);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Start (s_swDevice3) : sitalMt_Start (s_swDevice3) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-2 (BC) as a real BC.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-bc, alloc_mem-none\n", s_swDevice2);
		swResult = sitalDevice_Initialize (s_swDevice2, sitalAccess_CARD, (sitalMode_BC | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Loop over the range of possible message IDs starting with ID-0, ending with top possible ID, and moving in steps that suit the required number of messages:
		S16BIT swDataBlockId; // Data block ID;
		S16BIT swMessageId; // Message ID;
		swMessageId = 0;
		U16BIT wRxRtAddress; // RT address (sitalRtAddress_0-sitalRtAddress_31).
		wRxRtAddress = 0;
		S16BIT swCommandId; // Command ID;
		S16BIT swMessageCount; // The overall number of created messages.
		swMessageCount = 0;
		U16BIT waBuffer[32]; // A buffer to read-from/write-into along data-block write/read operations, respectively.
		U16BIT waCommandWords[sitalBcCounter_COMMANDS]; // An array of 1553 command words.
		S16BIT swaCommandIds[sitalBcCounter_COMMANDS]; // An array of command-IDs.
		do
		{
			///		Create for device-2 (BC) data-block-DataBlockId, a data block of 64 data words, and fill it with unique data.
			swDataBlockId = (swMessageCount + 1);
			for (int i=0; i<wWordCount; i++) waBuffer[i] = (swMessageCount + i);
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i, buf_contents - %i...\n", s_swDevice2, swDataBlockId, 64, wWordCount, swMessageCount);
			swResult = sitalBc_DataBlock_Create (s_swDevice2, swDataBlockId, sitalBcDataBlockSizeOption_DOUBLE, waBuffer, wWordCount);
			if (0 > swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
				throw 0;
			}

			///		Create for device-2 (BC) message-MessageId, a BC-to-RT(RxRtAddress/RtSubAddress) message that uses data-block-DataBlockId to transmit WordCount data words. Message bus: bus-1.
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create: unit - %i, message - %i, block - %i, tx_rt - %i, tx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId, swDataBlockId, wRxRtAddress, wRtSubaddress, wWordCount, 0, s_wBus1);
			swResult = sitalBc_Message_CreateBcToRt (s_swDevice2, swMessageId, swDataBlockId, wRxRtAddress, wRtSubaddress, wWordCount, 0, s_wBus1);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create.Error: error - %i\n", swResult);
				throw 0;
			}

			///		Use given message descriptors to code the command word of above defined message.
			// Note: Defined messages have been intentionally devised in a way that their [first] command word will differ.
			swResult = sitalBcStld1553_Command_GetCode (&(waCommandWords[swMessageCount]), wRxRtAddress, sitalCommand_RX, wRtSubaddress, wWordCount);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Command.GetCode.Error: error - %i\n", swResult);
				throw 0;
			}
			sitalProcess_Log_PrintLine ("#StldTester.Command.GetCode: rt_address - %i, direction - %i, subaddress_or_mode - %i, wc/mc - %i, code - %04X\n", wRxRtAddress, sitalCommand_RX, wRtSubaddress, wWordCount, waCommandWords[swMessageCount]);

			///		Create for device-2 (BC) command-CommandId, a XEQ command that executes message-MessageId.
			swCommandId = swMessageCount;
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId, 0);
			swResult = sitalBc_Command_Create (s_swDevice2, swCommandId, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId, 0, 0);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
				throw 0;
			}
			swaCommandIds[swMessageCount] = swCommandId;

			swMessageCount++;
			swMessageId = ( ((sitalBcCounter_MESSAGES <= (swMessageId + swMessageIdStep)) && ((sitalBcCounter_MESSAGES - 1) != swMessageId)) ? (sitalBcCounter_MESSAGES - 1) : (swMessageId + swMessageIdStep) );
			wRxRtAddress++;
			if (rtAddress_BORDER <= wRxRtAddress)
			{
				sitalProcess_Log_PrintLine ("#StldTester.RtAddress.Error: error - %s\n", "exhausted"); // Must configure swMessageIdStep into lower value!
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}
		}
		while (swMessageId < sitalBcCounter_MESSAGES);

		/// Create for device-2 (BC) minor-frame-0, a frame that contains all created commands.
		S16BIT swFrameId_0; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_0 = 0;
		U16BIT wCommandIdCount; // The number of command IDs in the frame (1-sitalBcCounter_FRAME_COMMANDS).
		wCommandIdCount = swMessageCount;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i...%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_0, "minor", wCommandIdCount, swaCommandIds[0], swaCommandIds[wCommandIdCount - 1], (wFrameTime / 10), 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_0, sitalBcFrameType_MINOR, swaCommandIds, wCommandIdCount, wFrameTime, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}
		
		/// Create for device-2 (BC) command-MessageCount, a subroutine call command that executes minor-frame-0. 
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swMessageCount, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, swFrameId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swMessageCount, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, (U16BIT)swFrameId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) major-frame-1, a frame that contains command-MessageCount and whose frame time is FrameTime us.
		S16BIT swFrameId_1; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_1 = 1;
		swaCommandIds[0] = swMessageCount;
		wCommandIdCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_1, "major", wCommandIdCount, swMessageCount, (wFrameTime / 10), 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_1, sitalBcFrameType_MAJOR, swaCommandIds, wCommandIdCount, wFrameTime, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Start device-2 (BC) running major-frame-1 for 1-shot.
		U16BIT wFrameCount; // The number of frames to run.
		wFrameCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", s_swDevice2, swFrameId_1, wFrameCount);
		swResult = sitalBc_Start (s_swDevice2, swFrameId_1, wFrameCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Wait for the frame run to end.
		U32BIT dwRemainingFrameRunTime; // The expected remaining frame run time (in ms).
		dwRemainingFrameRunTime = (wFrameCount * (wFrameTime / 10));
		swResult = stldTester_Bc_WaitForEndOfFrameRun (s_swDevice2, dwRemainingFrameRunTime, 0, NULL);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.WaitForEndOfFrameRun.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Stop device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop: unit - %i\n", s_swDevice2);
		swResult = sitalBc_Stop (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Stop device-3 (MT).
		sitalProcess_Log_PrintLine ("#StldTester.Mt.Stop: unit - %i\n", s_swDevice3);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Stop (s_swDevice3) : sitalMt_Stop (s_swDevice3) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Verify that device-3 (MT) has detected messages in the expected composition.
		sitalDecodedMessageStructure dmsDecodedMessage; // A decoded message.
		for (int i=0; i<swMessageCount; i++)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded: unit - %i, purge - %i\n", s_swDevice3, TRUE);
			swResult = sitalMt_Message_GetFromStackDecoded (s_swDevice3, &dmsDecodedMessage, sitalMessageLocationAndRemoval_NEXT_PURGE, sitalMtStack_ACTIVE);
			if ((0 > swResult) || (1 < swResult))
			{
				sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Error: error - %i\n", swResult);
				throw 0;
			}
			else
			if (0U == swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Error: error - %s\n", "no_message");
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}
			else
			{
				sitalProcess_Log_PrintLine ("#StldTester.Mt.Message: unit - %i, message - %i, command - %04X, expected_command - %04X, data_words - %i(%04X...), expected_data_words - %i(%04X...)\n", s_swDevice3, i, dmsDecodedMessage.wCommandWord1, waCommandWords[i], dmsDecodedMessage.wDataWordCount, dmsDecodedMessage.waData[0], wWordCount, i);
				if (waCommandWords[i] != dmsDecodedMessage.wCommandWord1)
				{
					sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Message.Error: error - %s\n", "bad_command");
					swResult = sitalReturnCode_TEST_FAIL;
					throw 0;
				}
				else
				if (wWordCount != dmsDecodedMessage.wDataWordCount)
				{
					sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Message.Error: error - %s\n", "bad_word_count");
					swResult = sitalReturnCode_TEST_FAIL;
					throw 0;
				}
				else
				if (i != dmsDecodedMessage.waData[0])
				{
					sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Message.Error: error - %s\n", "bad_data");
					swResult = sitalReturnCode_TEST_FAIL;
					throw 0;
				}
			}
		}

		/// Free device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice2);
		swResult = sitalDevice_Free (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-3 (MT).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice3);
		swResult = sitalDevice_Free (s_swDevice3);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		swResult = stldSyncEnd (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncEnd.Error: error - %i\n", swResult);
			throw 0;
		}
	}
	catch (int)
	{
		sitalDevice_Free (s_swDevice2);
		sitalDevice_Free (s_swDevice3);
	}

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_11h.End: test_succeeded - %c\n", ((sitalReturnCode_SUCCESS == swResult) ? 'Y' : 'N'));
	return swResult;
}



/// <summary>
/// Test the stld1553 library in the henceforth described scenario:
/// Participant[s]: RT1, BC2.
/// Configuration: All participating devices must be able to communicate with each other.
/// Short description: Acceptance Test Plan (ATP) test 461_11j - All mode codes shall be supported.
/// Along test print suitable info and indications into the log.
///
/// Note:
/// - This function may use any of above defined Acceptance Test Plan (ATP) tests variables.
///   The caller must therefore call the suitable stldTester_Atp_Set* functions prior to calling this function.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Atp_DoTest461_11j ()
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_11j.Start\n");

	/// Set test arguments.
	U32BIT dwMaximumFrameRunTime; // The maximum overall frame run time (in ms).
	dwMaximumFrameRunTime = 3000; // 3 seconds.
	U16BIT wRtSubaddressRx; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31) that's set for a RT for Rx.
	wRtSubaddressRx = 1;
	U16BIT wRtSubaddressTx; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31) that's set for a RT for Tx.
	wRtSubaddressTx = 2;

	S16BIT swResult; // Result of operation or function call.
	swResult = sitalReturnCode_SUCCESS;

	/// Get device count.
	U16BIT wRequiredDeviceCount; // The number of devices required for this test.
	wRequiredDeviceCount = (1 + max(s_swDevice1, s_swDevice2));
	U16BIT wAvailableDeviceCount; // The total number of devices available in the system.
	swResult = sitalDevice_GetCount (&wAvailableDeviceCount);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %i\n", swResult);
		return swResult;
	}
	sitalProcess_Log_PrintLine ("#StldTester.Device.Count: available - %i, required - %i\n", wAvailableDeviceCount, wRequiredDeviceCount);
	if (wAvailableDeviceCount < wRequiredDeviceCount)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %s\n", "not_enough_devices_installed");
		return sitalReturnCode_SUCCESS;
	}
	if ( (s_swDevice1 == s_swDevice2) )
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Number.Error: error - %s\n", "same_device_used_twice");
		return sitalReturnCode_INVALID_DEVICE_NUMBER;
	}

	try
	{
		swResult = stldSyncStart (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncStart.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-2 (BC) as a real BC.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-bc, alloc_mem-none\n", s_swDevice2);
		swResult = sitalDevice_Initialize (s_swDevice2, sitalAccess_CARD, sitalMode_BC, 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-1 (RT) as a real RT in advanced mode.
		// Note: Advanced mode will enable r/w to the memory of device-1 (RT) while the device is running.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-rt, alloc_mem-none\n", s_swDevice1);
		swResult = sitalDevice_Initialize (s_swDevice1, sitalAccess_CARD, ((s_bIsRtMtModeTest ? sitalMode_RT_AND_MT : sitalMode_RT) | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		///// Initialize device-1 (RT) and configure it to load its time tag when receiving synchronize-with-data mode code messages.
		//sitalProcess_Log_PrintLine ("#StldTester.Rt.Init: unit - %i, command_stack_size - %i, options - %08X\n", swDevice_1, 0x4000, sitalRtSetupOption_LOAD_TIME_TAG);
		//swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Initialize (s_swDevice1, 0x4000, sitalMtStackOption_SINGLE, sitalMtCommandStackSize_256, sitalMtDataStackSize_512, sitalRtSetupOption_LOAD_TIME_TAG) : sitalRt_Initialize (s_swDevice1, 0x4000, sitalRtSetupOption_LOAD_TIME_TAG) );
		//if (sitalReturnCode_SUCCESS != swResult)
		//{
		//	sitalProcess_Log_PrintLine ("#StldTester.Rt.Init.Error: error - %i\n", swResult);
		//	throw 0;
		//}

		/// Set device-1 (RT) address to RtAddress0.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set: unit - %i, address - %i\n", s_swDevice1, s_wRtAddress1);
		swResult = sitalRt_Address_Set (s_swDevice1, s_wRtAddress1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Legalize on device-1 (RT) all address-types in all directions for all subaddresses and all word counts.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.MessageLegality.Enable: unit - %i, address_type - %i, directions - %i, subaddress - %i, word_count - %08X\n", s_swDevice1, sitalRtAddressType_BOTH, sitalMessageDirection_BOTH, sitalRtSubaddress_ALL, 0xFFFFFFFF);
		swResult = sitalRt_MessageLegality_Enable (s_swDevice1, sitalRtAddressType_BOTH, sitalMessageDirection_BOTH, sitalRtSubaddress_ALL, 0xFFFFFFFF);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.MessageLegality.Enable.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Configure device-1 (RT) to read from RAM address 0x123 the mode data word to transmit in response to mode code 19, "Transmit Bit Word".
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Bit.Configure: unit - %i, source - %i, permit_on_busy - %i\n", s_swDevice1, sitalRtBuiltInTestWordSource_MEMORY, sitalRtBuiltInTestPermitOnBusy_ENABLED);
		swResult = sitalRt_BuiltInTestWord_Configure (s_swDevice1, sitalRtBuiltInTestWordSource_MEMORY, sitalRtBuiltInTestPermitOnBusy_ENABLED);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Bit.Configure.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Configure device-1 (RT) to get mode-code notifications using an ISR that counts ISR calls.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Manipulate: unit - %i, enable - %i, irq - %08X, callback - %s\n", s_swDevice1, TRUE, sitalInterruptRegister1_RT_MODE_CODE, "stldTester_Rt_UserIsr");
		swResult = sitalDevice_Irq_Manipulate (s_swDevice1, TRUE, sitalInterruptRegister1_RT_MODE_CODE, stldTester_Rt_UserIsr);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Set.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Start device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Start: unit - %i\n", s_swDevice1);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Start (s_swDevice1) : sitalRt_Start (s_swDevice1) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Loop to test the operation of mode-code messages.
		S16BIT swDataBlockId_0; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId_0 = 1;
		S16BIT swMessageId_0; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_0 = 0;
		S16BIT swCommandId_0; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_0 = 0;
		U32BIT dwOptions; // Initialization options (An or-ed combination of sitalBcSetupOption_*).
		dwOptions = 0;
		U16BIT wTransmittedModeDataWord; // A mode data word.
		U32BIT wReceivedMessageDataWord; // A message data word, or -1 if irrelevant.
		U16BIT wReceivedModeCodeDataWord; // A mode code data word.
		BOOLEAN bBadModeCodeDataWordReceived; // A flag that says whether a bad mode code data word has been received.
		U16BIT* wapRtModeDataAddress; // A pointer to an array of RT memory addresses to read from or write in the mode data word of the corresponding regular or broadcast mode code messages, as suits the current mode code message, where zero designates that no data word is read from or written into RT memory (entry i relates mode code i).
		sitalDecodedMessageStructure dmsDecodedMessage; // A decoded MIL-STD-1553 message.
		char szLogMessage[512]; // A log message string.
		U16BIT waModeCommand[] // An array that specifies the mode command used by each test (entry i relates test i).
		// Command index: 0,       1,       2,       3,       4,       5,       6,       7,       8,       9,      10,      11,      12,      13,      14,      15,      16,      17,      18,      19,      20,      21,      22,      23,      24,      25,      26,      27,      28,      29,      30,      31
				= {       0,       1,       2,       3,       4,       5,       6,       7,       8,      16,      18,      19,      17,      20,      21,       1,       3,       4,       5,       6,       7,       8,      17,      20,      21 };
		U16BIT baBroadcastMessage[] // An array of flags that say whether the corresponding tested message should be transmitted as a broadcast (entry i relates test i). A message may always be regularly transmitted, and if-and-only-if protocol IEEE-1553 allows it, it may be also broadcasted. Therefore, when broadcast is an option, this array says whether the referred test requires a broadcast.
		// Command index: 0,       1,       2,       3,       4,       5,       6,       7,       8,       9,      10,      11,      12,      13,      14,      15,      16,      17,      18,      19,      20,      21,      22,      23,      24,      25,      26,      27,      28,      29,      30,      31
				= {   false,   false,   false,   false,   false,   false,   false,   false,   false,   false,   false,   false,   false,   false,   false,    true,    true,    true,    true,    true,    true,    true,    true,    true,    true };
		U16BIT baIsModeWithData[] // An array of flags that say whether the corresponding mode code messages contain data (entry i relates mode code i). Each specific mode code is defined to either contain a data word or not, and that's strictly and non-optionally defined by protocol IEEE-1553 for that mode code.
		// Mode-code:     0,       1,       2,       3,       4,       5,       6,       7,       8,       9,      10,      11,      12,      13,      14,      15,      16,      17,      18,      19,      20,      21,      22,      23,      24,      25,      26,      27,      28,      29,      30,      31
				= {   false,   false,   false,   false,   false,   false,   false,   false,   false,   false,   false,   false,   false,   false,   false,   false,    true,    true,    true,    true,    true,    true,    true,    true,    true,    true,    true,    true,    true,    true,    true,    true };
		U16BIT baIsAlsoBroadcast[] // An array of flags that say whether the corresponding mode code messages may be also broadcasted (entry i relates mode code i). Each specific mode code is defined to have a broadcast option or not, and that's strictly and non-optionally defined by protocol IEEE-1553 for that mode code.
		// Mode-code:     0,       1,       2,       3,       4,       5,       6,       7,       8,       9,      10,      11,      12,      13,      14,      15,      16,      17,      18,      19,      20,      21,      22,      23,      24,      25,      26,      27,      28,      29,      30,      31
				= {   false,    true,   false,    true,    true,    true,    true,    true,    true,   false,   false,   false,   false,   false,   false,   false,   false,    true,   false,   false,    true,    true,   false,   false,   false,   false,   false,   false,   false,   false,   false,   false };
		U16BIT baIsRtModeDataTransmitter[] // An array of flags that say whether the RT serves as the transmitter of the mode code data word in the corresponding mode code messages (entry i relates mode code i). Each specific mode code is defined to have the Transmit bit on/off, and that's strictly and non-optionally defined by protocol IEEE-1553 for that mode code.
		// Mode-code:     0,       1,       2,       3,       4,       5,       6,       7,       8,       9,      10,      11,      12,      13,      14,      15,      16,      17,      18,      19,      20,      21,      22,      23,      24,      25,      26,      27,      28,      29,      30,      31
				= {    true,    true,    true,    true,    true,    true,    true,    true,    true,   false,   false,   false,   false,   false,   false,   false,    true,   false,    true,    true,   false,   false,   false,   false,   false,   false,   false,   false,   false,   false,   false,    false };
		U16BIT baIsRtInterruptOnRx[] // An array of flags that say whether the RT may interrupt whenever it receives the corresponding mode code messages (entry i relates mode code i). The receiving RT actually interrupts only if interrupts have been enabled for the corresponding mode code.
		// Mode-code:     0,       1,       2,       3,       4,       5,       6,       7,       8,       9,      10,      11,      12,      13,      14,      15,      16,      17,      18,      19,      20,      21,      22,      23,      24,      25,      26,      27,      28,      29,      30,      31
				= {   false,    true,    true,    true,    true,    true,    true,    true,    true,   false,   false,   false,   false,   false,   false,   false,    true,    true,    true,    true,    true,    true,   false,   false,   false,   false,   false,   false,   false,   false,   false,    false };
		U16BIT waRtRegularModeDataAddress[] // An array of RT memory addresses to read from or write in the mode data word of the corresponding regular mode code messages, where zero designates that no data word is read from or written into RT memory (entry i relates mode code i). Each relevant specific mode code has a predefined memory address of its own, and that's strictly and non-optionally defined by protocol IEEE-1553 for that mode code.
		// Mode-code:     0,       1,       2,       3,       4,       5,       6,       7,       8,       9,      10,      11,      12,      13,      14,      15,      16,      17,      18,      19,      20,      21,      22,      23,      24,      25,      26,      27,      28,      29,      30,      31
				= {  0x0000,  0x0000,  0x0000,  0x0000,  0x0000,  0x0000,  0x0000,  0x0000,  0x0000,  0x0000,  0x0000,  0x0000,  0x0000,  0x0000,  0x0000,  0x0000,  0x0120,  0x0111,  0x0122,  0x0123,  0x0114,  0x0115,  0x0000,  0x0000,  0x0000,  0x0000,  0x0111,  0x0000,  0x0000,  0x0000,  0x0000,  0x0000 };
		U16BIT waRtRegularModeRegisterAddress[] // An array of RT register addresses where the device records the mode data word to transmit, if so configured, in response to the of the corresponding regular mode code messages, where zero designates that there's no such register (entry i relates mode code i). Each relevant specific mode code has a predefined register address of its own, and that's strictly and non-optionally defined by protocol IEEE-1553 for that mode code.
		// Mode-code:     0,       1,       2,       3,       4,       5,       6,       7,       8,       9,      10,      11,      12,      13,      14,      15,      16,      17,      18,      19,      20,      21,      22,      23,      24,      25,      26,      27,      28,      29,      30,      31
				= {  0x0000,  0x0000,  0x0000,  0x0000,  0x0000,  0x0000,  0x0000,  0x0000,  0x0000,  0x0000,  0x0000,  0x0000,  0x0000,  0x0000,  0x0000,  0x0000,  0x0000,  0x0000,  0x000D,  0x000F,  0x0000,  0x0000,  0x0000,  0x0000,  0x0000,  0x0000,  0x0000,  0x0000,  0x0000,  0x0000,  0x0000,  0x0000 };
		U16BIT waRtBroadcastModeDataAddress[] // An array of RT memory addresses to read from or write in the mode data word of the corresponding broadcast mode code messages, where zero designates that no data word is read from or written into RT memory (entry i relates mode code i). Each relevant specific mode code has a predefined memory address of its own, and that's strictly and non-optionally defined by protocol IEEE-1553 for that mode code.
		// Mode-code:     0,       1,       2,       3,       4,       5,       6,       7,       8,       9,      10,      11,      12,      13,      14,      15,      16,      17,      18,      19,      20,      21,      22,      23,      24,      25,      26,      27,      28,      29,      30,      31
				= {  0x0000,  0x0000,  0x0000,  0x0000,  0x0000,  0x0000,  0x0000,  0x0000,  0x0000,  0x0000,  0x0000,  0x0000,  0x0000,  0x0000,  0x0000,  0x0000,  0x0000,  0x0131,  0x0000,  0x0000,  0x0134,  0x0135,  0x0000,  0x0000,  0x0000,  0x0000,  0x0000,  0x0000,  0x0000,  0x0000,  0x0000,  0x0000 };
		U16BIT waModeDataSource[] // An array that specifies the source of the mode data word that's read from or written into RT memory for the corresponding mode code messages, where zero designates mode code messages with no data (entry i relates mode code i). Each relevant specific mode code has a predefined source of its own, and that's strictly and non-optionally defined by protocol IEEE-1553 for that mode code.
		// Mode-code:     0,       1,       2,       3,       4,       5,       6,       7,       8,       9,      10,      11,      12,      13,      14,      15,      16,      17,      18,      19,      20,      21,      22,      23,      24,      25,      26,      27,      28,      29,      30,      31
				= {       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0,  RtHost,  BcHost,  RtCore,  RtHost,  BcHost,  BcHost,       0,       0,       0,       0,       0,       0,       0,       0,       0,       0 };
		U16BIT wModeCommandCount; // The number of mode commands stored in array waModeCommand.
		wModeCommandCount = ( sizeof(waModeCommand) / sizeof(waModeCommand[0]) );
		for (U16BIT i=0; i<wModeCommandCount; i++, wTransmittedModeDataWord=(0xAA00+i))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Test.Iteration.Start: iteration_index - %i\n", i);

			///		If the current mode code may cause RT interrupts:
			///			Enable the relevant mode code interrupt.
			///			Verify that the relevant mode code interrupt is indeed enabled.
			///			Zero the global ISR call counter.
			U16BIT wModeCodeType; // A type of mode code (sitalRtModeCodeType_*).
			wModeCodeType = 0;
			U16BIT wModeCodeIrqSet; // A set of mode code interrupts (An or-ed combination of sitalRtModeCodeIrq_*).
			wModeCodeIrqSet = 0;
			U16BIT wModeCodeIrqRetrieved; // A set of mode code interrupts (An or-ed combination of sitalRtModeCodeIrq_*).
			wModeCodeIrqRetrieved = 0;
			if (baIsRtInterruptOnRx[waModeCommand[i]])
			{
				wModeCodeType = 0;
				wModeCodeType += ( (baBroadcastMessage[i]) ? sitalRtModeCodeTypeBase_BROADCAST : sitalRtModeCodeTypeBase_NOT_BROADCAST );
				wModeCodeType += ( (baIsRtModeDataTransmitter[waModeCommand[i]]) ? sitalRtModeCodeTypeBase_TX : sitalRtModeCodeTypeBase_RX );
				wModeCodeType += ( (baIsModeWithData[waModeCommand[i]]) ? sitalRtModeCodeTypeBase_DATA : sitalRtModeCodeTypeBase_NO_DATA );
				wModeCodeIrqSet = (  1  <<  ( (16 <= waModeCommand[i]) ? (waModeCommand[i] - 16) : waModeCommand[i] )  );
				sitalProcess_Log_PrintLine ("#StldTester.Rt.Interrupt.ModeCode.Enable: unit - %i, mode_command - %i, is_broadcast - %i, is_tx - %i, is_data - %i, type - %i, irq - %04X\n", s_swDevice1, waModeCommand[i], baBroadcastMessage[waModeCommand[i]], baIsRtModeDataTransmitter[waModeCommand[i]], baIsModeWithData[waModeCommand[i]], wModeCodeType, wModeCodeIrqSet);
				swResult = sitalRt_ModeCode_EnableIrq (s_swDevice1, wModeCodeType, wModeCodeIrqSet);
				if (0 > swResult)
				{
					sitalProcess_Log_PrintLine ("#StldTester.Rt.ModeCode.Irq.Enable.Error: error - %i\n", swResult);
					throw 0;
				}
				sitalProcess_Log_PrintLine ("#StldTester.Rt.Interrupt.ModeCode.Get: unit - %i, type - %i\n", s_swDevice1, wModeCodeType);
				swResult = sitalRt_ModeCode_GetIrq (s_swDevice1, wModeCodeType, &wModeCodeIrqRetrieved);
				if (0 > swResult)
				{
					sitalProcess_Log_PrintLine ("#StldTester.Rt.ModeCode.Irq.Get.Error: error - %i\n", swResult);
					throw 0;
				}
				if (wModeCodeIrqSet != wModeCodeIrqRetrieved)
				{
					sitalProcess_Log_PrintLine ("#StldTester.Rt.ModeCode.Irq.Get.Error: error - %i\n", wModeCodeIrqRetrieved);
					swResult = sitalReturnCode_TEST_FAIL;
					throw 0;
				}

				s_dwRtIsrCallCounter = 0;
			}

			///		Re-initialize device-2 (BC) as a real BC.
			sitalProcess_Log_PrintLine ("#StldTester.Bc.ReInit: unit - %i, options - %X\n", s_swDevice2, dwOptions);
			swResult = sitalBc_Initialize (s_swDevice2, dwOptions);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.ReInit.Error: error - %i\n", swResult);
				throw 0;
			}

			///		Create for device-2 (BC) data-block-0, a data block of 32 data words.
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i\n", s_swDevice2, swDataBlockId_0, 32, 0);
			swResult = sitalBc_DataBlock_Create (s_swDevice2, swDataBlockId_0, sitalBcDataBlockSizeOption_SINGLE, NULL, 0);
			if (0 > swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
				throw 0;
			}

			if (baBroadcastMessage[i] && (!baIsAlsoBroadcast[waModeCommand[i]]))
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.Create.Error: error - %s\n", "irrelevant_bcst");
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}

			if (baIsRtModeDataTransmitter[waModeCommand[i]] && (!baBroadcastMessage[i]))
			{
				///		Create for device-2 (BC) message-0, a RT(RtAddress)-to-BC-Mode(*) message. Message bus: bus-1.
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBcMode.Create: unit - %i, message - %i, block - %i, rt - %i, direction - %i, mode - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, (baIsRtModeDataTransmitter[waModeCommand[i]] ? sitalMessageDirection_TX : sitalMessageDirection_RX), waModeCommand[i], 0, s_wBus1);
				swResult = sitalBc_Message_CreateBcToOrFromRtMode (s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, (baIsRtModeDataTransmitter[waModeCommand[i]] ? sitalMessageDirection_TX : sitalMessageDirection_RX), waModeCommand[i], 0, s_wBus1);
				if (sitalReturnCode_SUCCESS != swResult)
				{
					sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBcMode.Create.Error: error - %i\n", swResult);
					throw 0;
				}
				wapRtModeDataAddress = waRtRegularModeDataAddress;
			}
			else
			if ((!baIsRtModeDataTransmitter[waModeCommand[i]]) && (!baBroadcastMessage[i]))
			{
				///		Create for device-2 (BC) message-0, a BC-to-RT(RtAddress)-Mode(*) message. Message bus: bus-1.
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRtMode.Create: unit - %i, message - %i, block - %i, rt - %i, direction - %i, mode - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, (baIsRtModeDataTransmitter[i] ? sitalMessageDirection_TX : sitalMessageDirection_RX), waModeCommand[i], 0, s_wBus1);
				swResult = sitalBc_Message_CreateBcToOrFromRtMode (s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, (baIsRtModeDataTransmitter[waModeCommand[i]] ? sitalMessageDirection_TX : sitalMessageDirection_RX), waModeCommand[i], 0, s_wBus1);
				if (sitalReturnCode_SUCCESS != swResult)
				{
					sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBcMode.Create.Error: error - %i\n", swResult);
					throw 0;
				}
				wapRtModeDataAddress = waRtRegularModeDataAddress;
			}
			else
			if (baIsRtModeDataTransmitter[waModeCommand[i]] && baBroadcastMessage[i])
			{
				///		Create for device-2 (BC) message-0, a Broadcast-to-BC-Mode(*) message. Message bus: bus-1.
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BroadcastToBcMode.Create: unit - %i, message - %i, block - %i, direction - %i, mode - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_0, swDataBlockId_0, (baIsRtModeDataTransmitter[waModeCommand[i]] ? sitalMessageDirection_TX : sitalMessageDirection_RX), waModeCommand[i], 0, s_wBus1);
				swResult = sitalBc_Message_CreateBcToOrFromBroadcastMode (s_swDevice2, swMessageId_0, swDataBlockId_0, (baIsRtModeDataTransmitter[waModeCommand[i]] ? sitalMessageDirection_TX : sitalMessageDirection_RX), waModeCommand[i], 0, s_wBus1);
				if (sitalReturnCode_SUCCESS != swResult)
				{
					sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToBroadcastMode.Create.Error: error - %i\n", swResult);
					throw 0;
				}
				wapRtModeDataAddress = waRtBroadcastModeDataAddress;
			}
			else
			if ((!baIsRtModeDataTransmitter[waModeCommand[i]]) && baBroadcastMessage[i])
			{
				///		Create for device-2 (BC) message-0, a BC-to-Broadcast-Mode(*) message. Message bus: bus-1.
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToBroadcastMode.Create: unit - %i, message - %i, block - %i, direction - %i, mode - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_0, swDataBlockId_0, (baIsRtModeDataTransmitter[waModeCommand[i]] ? sitalMessageDirection_TX : sitalMessageDirection_RX), waModeCommand[i], 0, s_wBus1);
				swResult = sitalBc_Message_CreateBcToOrFromBroadcastMode (s_swDevice2, swMessageId_0, swDataBlockId_0, (baIsRtModeDataTransmitter[waModeCommand[i]] ? sitalMessageDirection_TX : sitalMessageDirection_RX), waModeCommand[i], 0, s_wBus1);
				if (sitalReturnCode_SUCCESS != swResult)
				{
					sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToBroadcastMode.Create.Error: error - %i\n", swResult);
					throw 0;
				}
				wapRtModeDataAddress = waRtBroadcastModeDataAddress;
			}
			else
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.Create.Error: error - %s\n", "bad_type");
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}

			///		Create for device-2 (BC) command-0, a XEQ command that executes message-0.
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0);
			swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0, 0);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
				throw 0;
			}

			///		If the current mode code message contains data:
			///			If the BC plays the transmitter of the current mode code message:
			///				Write into device-2 (BC) data-block-0 a (single unique) mode data word.
			///			Else:
			///				If with the current mode code message the host of device-1 (RT) is the source of a mode data word that's written in the memory of the device-1 (RT):
			///					Update the relevant mode data word in the memory of device-1 (RT).
			///				Else:
			///					If device-1 (RT) core has written in the memory of device-1 (RT) the mode data word that the RT has just transmitted:
			///						With device-1 (RT) get the transmitted mode data word.
			if (baIsModeWithData[waModeCommand[i]])
			{
				if (!baIsRtModeDataTransmitter[waModeCommand[i]])
				{
					sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Write: unit - %i, block - %i, offset - %i, buf_size - %i, buf_contents - %04X\n", s_swDevice2, swDataBlockId_0, 0, 1, wTransmittedModeDataWord);
					swResult = sitalBc_DataBlock_Write (s_swDevice2, swDataBlockId_0, &wTransmittedModeDataWord, 1, 0);
					if (sitalReturnCode_SUCCESS != swResult)
					{
						sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Write.Error: error - %i\n", swResult);
						throw 0;
					}
				}
				else
				if (RtHost == waModeDataSource[waModeCommand[i]])
				{
					sitalProcess_Log_PrintLine ("#StldTester.Rt.ModeCode.Data.Write: unit - %i, mode_code_address - %04X, value - %04X\n", s_swDevice1, wapRtModeDataAddress[waModeCommand[i]], wTransmittedModeDataWord);
					swResult = sitalRt_ModeCode_WriteData (s_swDevice1, (wapRtModeDataAddress[waModeCommand[i]] - 0x0110), wTransmittedModeDataWord);
					if (sitalReturnCode_SUCCESS != swResult)
					{
						sitalProcess_Log_PrintLine ("#StldTester.Rt.ModeCode.Data.Write.Error: error - %i\n", swResult);
						throw 0;
					}
				}
				else
				if (RtCore == waModeDataSource[waModeCommand[i]])
				{
					switch (waModeCommand[i])
					{
					case 18 /* "Transmit Last Command Word" */ :
						// Verify that this isn't the first message to be transmitted in this test, as otherwise there won't exist a last command word that the RT will be able to transmit as the expected mode code data word.
						if (0U == i)
						{
							sitalProcess_Log_PrintLine ("#StldTester.Bc.ModeCommand.Error: error - %s\n", "bad_sequence");
							swResult = sitalReturnCode_TEST_FAIL;
							throw 0;
						}

						// With mode code "Transmit Last Command Word", the mode data word is the last command word that was received by the RT.
						wTransmittedModeDataWord = dmsDecodedMessage.wCommandWord1; // Mode code messages all contain a single command word.
						break;
					default:
						sitalProcess_Log_PrintLine ("#StldTester.Bc.ModeCommand.Error: error - %i\n", waModeCommand[i]);
						swResult = sitalReturnCode_TEST_FAIL;
						throw 0;
					}
				}
				else
				{
					sitalProcess_Log_PrintLine ("#StldTester.Bc.ModeCommand.Error: error - %i\n", waModeCommand[i]);
					swResult = sitalReturnCode_TEST_FAIL;
					throw 0;
				}
			}

			///		Create and run a frame that contains command-0 of device-2 (BC).
			swResult = stldTester_Bc_RunFrame (s_swDevice2, 1, 0, &swCommandId_0, 1, 1, 0);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				throw 0;
			}

			///		Get message-0 from device-2 (BC).
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded: unit - %i, message - %i, purge-y\n", s_swDevice2, swMessageId_0);
			swResult = sitalBc_Message_GetByIdDecoded (s_swDevice2, swMessageId_0, &dmsDecodedMessage, TRUE);
			if (1 != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded.Error: error - %i\n", swResult);
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}

			///		If device-1 (RT) should have responded with status:
			///			Verify that device-1 (RT) has responded with a proper status.
			// For a status word to be correct it must fulfill the following conditions:
			// - The terminal address must that of the target RT.
			// - All the error flag bits must be zeroed.
			// - The broadcast command received field must be set only in case of a respond to a "Transmit Status Word" or "Transmit Last Command Word" mode code messages, if these are received by a remote terminal right after it received a broadcast message.
			if (baIsRtModeDataTransmitter[waModeCommand[i]] || (!baBroadcastMessage[i]))
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded: unit - %i, message - %i, message_block_status - %04X, response_received - %i, status_1_relevant - %i, status_1 - %04X, status_1_string - \"%s\", rt_address - %i, mode_command - %i\n", s_swDevice2, swMessageId_0, dmsDecodedMessage.wBlockStatus, (0U == (sitalRtBlockStatusWord_NO_RESPONSE & dmsDecodedMessage.wBlockStatus)), dmsDecodedMessage.bIsStatus1Relevant, dmsDecodedMessage.wStatus1, sitalStld1553_GetBlockStatusWordErrorString (sitalMode_BC, dmsDecodedMessage.wStatus1), ((sitalIeee1553_Status_TERMINAL_ADDRESS & dmsDecodedMessage.wStatus1) >> sitalIeee1553_Status_OFFSET_OF_TERMINAL_ADDRESS), waModeCommand[i]);
				if	(
						(sitalRtBlockStatusWord_NO_RESPONSE == (sitalRtBlockStatusWord_NO_RESPONSE & dmsDecodedMessage.wBlockStatus))
						||
						(
							(FALSE == baBroadcastMessage[i])
							&&
							(
								(FALSE == dmsDecodedMessage.bIsStatus1Relevant)
								||
								(s_wRtAddress1 != ((sitalIeee1553_Status_TERMINAL_ADDRESS & dmsDecodedMessage.wStatus1) >> sitalIeee1553_Status_OFFSET_OF_TERMINAL_ADDRESS))
							)
						)
						||
						(0U != (sitalIeee1553_Status_ERROR_SECTION & dmsDecodedMessage.wStatus1))
						||
						(
							(0U != (sitalIeee1553_Status_BROADCAST_COMMAND_RECEIVED & dmsDecodedMessage.wStatus1))
							&&
							(
								( (2 /* Transmit Status Word */ != waModeCommand[i]) && (18 /* Transmit Last Command Word */ != waModeCommand[i]) ) || // Broadcast indication wasn't required at all.
								(0U == i) || // No previous message has been sent.
								//(0U == (MessageType_DestinationIsBroadcast && waMessageType[i - 1]))) // Previous message wasn't a broadcast message.
								(FALSE == baBroadcastMessage[i - 1]) // Previous message wasn't a broadcast message.
							)
						)
					)
				{
					sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded.Error: error - %s\n", "bad response");
					swResult = sitalReturnCode_TEST_FAIL;
					throw 0;
				}
			}

			///		If the current mode code message contains data:
			///			If the BC plays the transmitter of the current mode code message:
			///				Read the relevant mode data word from the memory of device-1 (RT).
			///			Else:
			///				Read device-2 (BC) data-block-0 first mode data word.
			///			Verify that the transmitted mode code data has been received.
			// Note: If the current mode code message contains data and the BC does not play the transmitter, then with the current mode code message [either the core/host of] device-1 (RT) must be the source of a mode data word that's written in the memory of the device-1 (RT).
			if (baIsModeWithData[waModeCommand[i]])
			{
				bBadModeCodeDataWordReceived = false;
				if (!baIsRtModeDataTransmitter[waModeCommand[i]])
				{
					sitalProcess_Log_PrintLine ("#StldTester.Rt.Message.GetFromStackDecoded: unit - %i, order-latest, removal-purge\n", s_swDevice1);
					swResult = sitalRt_Message_GetFromStackDecoded (s_swDevice1, &dmsDecodedMessage, (sitalMessageLocation_LATEST | sitalMessageRemoval_PURGE));
					if (1 != swResult)
					{
						sitalProcess_Log_PrintLine ("#StldTester.Rt.Message.GetFromStackDecoded.Error: error - %i\n", swResult);
						throw 0;
					}
					wReceivedMessageDataWord = dmsDecodedMessage.waData[0];

					sitalProcess_Log_PrintLine ("#StldTester.Rt.ModeCode.Data.Read: unit - %i, mode_code_address - %04X\n", s_swDevice1, wapRtModeDataAddress[waModeCommand[i]]);
					swResult = sitalRt_ModeCode_ReadData (s_swDevice1, (wapRtModeDataAddress[waModeCommand[i]] - 0x0110), &wReceivedModeCodeDataWord);
					if (sitalReturnCode_SUCCESS != swResult)
					{
						sitalProcess_Log_PrintLine ("#StldTester.Rt.ModeCode.Data.Read.Error: error - %i\n", swResult);
						throw 0;
					}

					bBadModeCodeDataWordReceived = ((wTransmittedModeDataWord != wReceivedModeCodeDataWord) || (wTransmittedModeDataWord != wReceivedMessageDataWord));
					sprintf (szLogMessage, "#StldTester.ModeCode.DataWord.Compare.Error: transmitted_mode_data - %04X, received_data_word - %04X, received_mode_data - %04X\n", wTransmittedModeDataWord, wReceivedMessageDataWord, wReceivedModeCodeDataWord);
				}
				else
				{
					sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Read: unit - %i, block - %i, buf_size - %i, offset - %i\n", s_swDevice2, swDataBlockId_0, 1, 0);
					swResult = sitalBc_DataBlock_Read (s_swDevice2, swDataBlockId_0, &wReceivedModeCodeDataWord, 1, 0);
					if (0 > swResult)
					{
						sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Read.Error: error - %i\n", swResult);
						throw 0;
					}

					bBadModeCodeDataWordReceived = (wTransmittedModeDataWord != wReceivedModeCodeDataWord);
					sprintf (szLogMessage, "#StldTester.ModeCode.DataWord.Compare.Error: transmitted_mode_data - %04X, received_mode_data - %04X\n", wTransmittedModeDataWord, wReceivedModeCodeDataWord);
				}

				if (bBadModeCodeDataWordReceived)
				{
					if (0U != waRtRegularModeRegisterAddress[waModeCommand[i]])
					{
						U16BIT wRegisterValue; // The current value of a target register, or the value to set for a target register.
						wRegisterValue = sitalDevice_Register_Read (s_swDevice1, waRtRegularModeRegisterAddress[waModeCommand[i]]);
						sitalProcess_Log_PrintLine ("#StldTester.Device.Reg.%04X.Read: value - %04X\n", waRtRegularModeRegisterAddress[waModeCommand[i]], wRegisterValue);
					}
					sitalProcess_Log_PrintLine (szLogMessage);
					swResult = sitalReturnCode_TEST_FAIL;
					throw 0;
				}
			}

			///		If the current mode code may cause RT interrupts:
			///			Verify that an interrupt occurred using the global ISR call counter.
			///			Disable the previously enabled mode code interrupt.
			if (baIsRtInterruptOnRx[waModeCommand[i]])
			{
				/// As long a not all expected interrupts occurred AND the maximum possible interrupt latency still didn't pass:
				///		Wait.
				stldInterruptLatency_Wait (&s_dwRtIsrCallCounter, 1);

				sitalProcess_Log_PrintLine ("#StldTester.Rt.Interrupt.ModeCode: unit - %i, iteration - %i, count - %i\n", s_swDevice1, i, s_dwRtIsrCallCounter);
				if (1 != s_dwRtIsrCallCounter)
				{
					sitalProcess_Log_PrintLine ("#StldTester.Rt.ModeCode.Irq.Error: error - %s\n", "no_interrupt");
					swResult = sitalReturnCode_TEST_FAIL;
					throw 0;
				}

				sitalProcess_Log_PrintLine ("#StldTester.Rt.ModeCode.Irq.Disable: unit - %i, type - %i, irq - %i\n", s_swDevice1, wModeCodeType, wModeCodeIrqSet);
				swResult = sitalRt_ModeCode_DisableIrq (s_swDevice1, wModeCodeType, wModeCodeIrqSet);
				if (0 > swResult)
				{
					sitalProcess_Log_PrintLine ("#StldTester.Rt.ModeCode.Irq.Disable.Error: error - %i\n", swResult);
					throw 0;
				}
			}
		}

		/// Stop device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Stop: unit - %i\n", s_swDevice1);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Stop (s_swDevice1) : sitalRt_Stop (s_swDevice1) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice2);
		swResult = sitalDevice_Free (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice1);
		swResult = sitalDevice_Free (s_swDevice1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		swResult = stldSyncEnd (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncEnd.Error: error - %i\n", swResult);
			throw 0;
		}
	}
	catch (int)
	{
		sitalDevice_Free (s_swDevice2);
		sitalDevice_Free (s_swDevice1);
	}

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_11j.End: test_succeeded - %c\n", ((sitalReturnCode_SUCCESS == swResult) ? 'Y' : 'N'));
	return swResult;
}



/// <summary>
/// Test the stld1553 library in the henceforth described scenario:
/// Participant[s]: BC2, MT3.
/// Configuration: All participating devices must be able to communicate with each other.
/// Short description: Acceptance Test Plan (ATP) test 461_11k_1 - Programmable control of major and minor frame content and timing.
/// Along test print suitable info and indications into the log.
///
/// Note:
/// - This function may use any of above defined Acceptance Test Plan (ATP) tests variables.
///   The caller must therefore call the suitable stldTester_Atp_Set* functions prior to calling this function.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Atp_DoTest461_11k_1 ()
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_11k_1.Start\n");

	/// Set test arguments.
	U16BIT wWordCount; // Word count (1-32).
	wWordCount = 32;
	S16BIT swMessageIdStep; // Message-ID-step to use in message creation loop.
	swMessageIdStep = 50;
	U16BIT wRtSubaddressRx; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31).
	wRtSubaddressRx = 1;
	U16BIT wRtSubaddressTx; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31).
	wRtSubaddressTx = 2;
	U16BIT wMessageTime; // The basic time (in units of 100 us) that the message will take to complete.
	wMessageTime = 100; // 10ms.
	U16BIT wIterations; // The number of test iterations.
	wIterations = 10;
	U16BIT wTimeTagCalculationInaccuracy; // The maximum tolerated difference between the expected time tag and the actual one.
	wTimeTagCalculationInaccuracy = 25;

	S16BIT swResult; // Result of operation or function call.
	swResult = sitalReturnCode_SUCCESS;

	/// Get device count.
	U16BIT wRequiredDeviceCount; // The number of devices required for this test.
	wRequiredDeviceCount = (1 + max(s_swDevice2, s_swDevice3));
	U16BIT wAvailableDeviceCount; // The total number of devices available in the system.
	swResult = sitalDevice_GetCount (&wAvailableDeviceCount);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %i\n", swResult);
		return swResult;
	}
	sitalProcess_Log_PrintLine ("#StldTester.Device.Count: available - %i, required - %i\n", wAvailableDeviceCount, wRequiredDeviceCount);
	if (wAvailableDeviceCount < wRequiredDeviceCount)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %s\n", "not_enough_devices_installed");
		return sitalReturnCode_SUCCESS;
	}
	if ( (s_swDevice2 == s_swDevice3) )
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Number.Error: error - %s\n", "same_device_used_twice");
		return sitalReturnCode_INVALID_DEVICE_NUMBER;
	}

	try
	{
		swResult = stldSyncStart (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncStart.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-3 (MT) as a real MT.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access - card, mode - mt, alloc_mem - none\n", s_swDevice3);
		swResult = sitalDevice_Initialize (s_swDevice3, sitalAccess_CARD, ((s_bIsRtMtModeTest ? sitalMode_RT_AND_MT : sitalMode_MT) | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Set for device-3 (MT) a time tag resolution of 2 us.
		U16BIT wTimeTagResolution; // Time tag resolution (in units of 1 us).
		wTimeTagResolution = 2;
		sitalProcess_Log_PrintLine ("#StldTester.Device.TimeTagResolution.Set: unit - %i, resolution - %ius\n", s_swDevice3, wTimeTagResolution);
		swResult = sitalDevice_TimeTag_SetResolution (s_swDevice3, sitalTimeTagResolution_2US);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.TimeTagResolution.Set.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Start device-3 (MT).
		sitalProcess_Log_PrintLine ("#StldTester.Mt.Start: unit - %i\n", s_swDevice3);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Start (s_swDevice3) : sitalMt_Start (s_swDevice3) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-2 (BC) as a real BC.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-bc, alloc_mem-none\n", s_swDevice2);
		swResult = sitalDevice_Initialize (s_swDevice2, sitalAccess_CARD, (sitalMode_BC | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) data-block-0, a data block of 64 data words.
		S16BIT swDataBlockId_0; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId_0 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i\n", s_swDevice2, swDataBlockId_0, 64, 0);
		swResult = sitalBc_DataBlock_Create (s_swDevice2, swDataBlockId_0, sitalBcDataBlockSizeOption_DOUBLE, NULL, 0);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) data-block-1, a data block of 64 data words.
		S16BIT swDataBlockId_1; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId_1 = 2;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i\n", s_swDevice2, swDataBlockId_1, 64, 0);
		swResult = sitalBc_DataBlock_Create (s_swDevice2, swDataBlockId_1, sitalBcDataBlockSizeOption_DOUBLE, NULL, 0);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) message-0, a RT(RT-address-1/RtSubAddressTx)-to-BC message that uses data-block-0 to receive WordCount data words. Message bus: bus-1.
		S16BIT swMessageId_0; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBc.Create: unit - %i, message - %i, block - %i, tx_rt - %i, tx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddressTx, wWordCount, 0, s_wBus1);
		swResult = sitalBc_Message_CreateRtToBc (s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddressTx, wWordCount, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBc.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) message-1, a BC-to-RT(RT-address-1/RtSubAddressTx message that uses data-block-1 to receive WordCount data words. Message bus: bus-1.
		S16BIT swMessageId_1; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_1 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create: unit - %i, message - %i, block - %i, rx_rt - %i, rx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_1, swDataBlockId_1, s_wRtAddress1, wRtSubaddressRx, wWordCount, 0, s_wBus1);
		swResult = sitalBc_Message_CreateBcToRt (s_swDevice2, swMessageId_1, swDataBlockId_1, s_wRtAddress1, wRtSubaddressRx, wWordCount, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Use given message descriptors to code the command word of above defined messages.
		// Note: Defined messages have been intentionally devised in a way that their [first] command word will differ.
		U16BIT wCommandWord0; // Command word for above defined RT-to-BC message-0.
		swResult = sitalBcStld1553_Command_GetCode (&wCommandWord0, s_wRtAddress1, sitalCommand_TX, wRtSubaddressTx, wWordCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Command.GetCode.Error: error - %i\n", swResult);
			throw 0;
		}
		sitalProcess_Log_PrintLine ("#StldTester.Command.GetCode: rt_address - %i, direction - %i, subaddress_or_mode - %i, wc/mc - %i, code - %04X\n", s_wRtAddress1, sitalCommand_TX, wRtSubaddressTx, wWordCount, wCommandWord0);
		U16BIT wCommandWord1; // Command word for above defined BC-to-RT message-1.
		swResult = sitalBcStld1553_Command_GetCode (&wCommandWord1, s_wRtAddress1, sitalCommand_RX, wRtSubaddressRx, wWordCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Command.GetCode.Error: error - %i\n", swResult);
			throw 0;
		}
		sitalProcess_Log_PrintLine ("#StldTester.Command.GetCode: rt_address - %i, direction - %i, subaddress_or_mode - %i, wc/mc - %i, code - %04X\n", s_wRtAddress1, sitalCommand_RX, wRtSubaddressRx, wWordCount, wCommandWord1);

		/// Create for device-2 (BC) command-0, a XEQ command that executes message-0.
		S16BIT swCommandId_0; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-1, a XEQ command that executes message-1.
		S16BIT swCommandId_1; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_1 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_1, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_1, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_1, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_1, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-2, a subroutine call command that executes minor-frame-0. 
		S16BIT swCommandId_2; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_2 = 2;
		S16BIT swFrameId_0; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_2, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, swFrameId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_2, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, (U16BIT)swFrameId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) minor-frame-0, a frame that contains command-0 and command-1.
		S16BIT swaCommandIds[sitalBcCounter_COMMANDS]; // An array of command-IDs.
		swaCommandIds[0] = swCommandId_0;
		swaCommandIds[1] = swCommandId_1;
		U16BIT wCommandIdCount; // The number of command IDs in the frame (1-sitalBcCounter_FRAME_COMMANDS).
		wCommandIdCount = 2;
		U16BIT wFrameMessageCount; // The number of messages in the frame.
		wFrameMessageCount = wCommandIdCount;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i, %i), time - %i, flags - %i\n", s_swDevice2, swFrameId_0, "minor", wCommandIdCount, swCommandId_0, swCommandId_1, 0, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_0, sitalBcFrameType_MINOR, swaCommandIds, wCommandIdCount, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Loop to test various major frames and frame times:
		for (S16BIT i=1; i<=wIterations; i++)
		{
			U16BIT wFrameCount; // The number of frames to run.
			wFrameCount = 2;
			wCommandIdCount = i;
			U16BIT wOverallMessageCount; // The overall number of messages.
			wOverallMessageCount = (wFrameMessageCount * wCommandIdCount * wFrameCount);
			sitalProcess_Log_PrintLine ("#StldTester.SubTest: iteration - %i, message_count - %i\n", i, wOverallMessageCount);

			/// 	Create for device-2 (BC) major-frame-1, a frame that contains i x command-2 and whose frame time is (i x FrameTime) us.
			S16BIT swFrameId_1; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
			swFrameId_1 = 1;
			for (S16BIT j=0; j<i; j++)	swaCommandIds[j] = swCommandId_2;
			U16BIT wFrameTime; // The time (in units of 100 us) that the current/actual frame will take to complete.
			wFrameTime = (wFrameMessageCount * wMessageTime);
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(all %i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_1, "major", wCommandIdCount, swCommandId_2, (wFrameTime / 10), 0);
			swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_1, sitalBcFrameType_MAJOR, swaCommandIds, wCommandIdCount, wFrameTime, 0);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
				throw 0;
			}

			/// 	Start device-2 (BC) running major-frame-1 for 2-shots.
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", s_swDevice2, swFrameId_1, wFrameCount);
			swResult = sitalBc_Start (s_swDevice2, swFrameId_1, wFrameCount);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Wait for the frame run to end.
			U32BIT dwRemainingFrameRunTime; // The expected remaining frame run time (in ms).
			dwRemainingFrameRunTime = (wFrameCount * wCommandIdCount * (wFrameTime / 10));
			swResult = stldTester_Bc_WaitForEndOfFrameRun (s_swDevice2, dwRemainingFrameRunTime, 0, NULL);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.WaitForEndOfFrameRun.Error: error - %i\n", swResult);
				throw 0;
			}

			///		Stop device-2 (BC).
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop: unit - %i\n", s_swDevice2);
			swResult = sitalBc_Stop (s_swDevice2);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
				throw 0;
			}

			/// 	Delete for device-2 (BC) major-frame-1.
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Delete: unit - %i, frame - %i\n", s_swDevice2, swFrameId_1);
			swResult = sitalBc_Frame_Delete (s_swDevice2, swFrameId_1);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Delete.Error: error - %i\n", swResult);
				throw 0;
			}

			///		Verify that device-3 (MT) has detected messages in the expected composition and timing.
			U16BIT wExpectedCommandWord; // The command word expected for the message to be detected next by device-3 (MT).
			wExpectedCommandWord = wCommandWord0;
			U32BIT dwExpectedMessage0TimeTag; // The time tag expected for the RT-to-BC message to be detected next by device-3 (MT), or 0xFFFFFFFF if still didn't get the first message.
			dwExpectedMessage0TimeTag = 0xFFFFFFFF;
			U16BIT wRtToBcMessageCount; // The total number of RT-to-BC messages that have been identified.
			wRtToBcMessageCount = 0;
			U16BIT wBcToRtMessageCount; // The total number of BC-to-RT messages that have been identified.
			wBcToRtMessageCount = 0;
			U16BIT wMessageNumber; // The serial number (1, 2, ...) of the currently received message.
			wMessageNumber = 0;
			do 
			{
				sitalDecodedMessageStructure dmsDecodedMessage; // A decoded message.
				sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded: unit - %i, purge - %i\n", s_swDevice3, TRUE);
				swResult = sitalMt_Message_GetFromStackDecoded (s_swDevice3, &dmsDecodedMessage, sitalMessageLocationAndRemoval_NEXT_PURGE, sitalMtStack_ACTIVE);
				if ((0 > swResult) || (1 < swResult))
				{
					sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Error: error - %i\n", swResult);
					throw 0;
				}
				else
				if (1 == swResult)
				{
					wMessageNumber++;
					char szLogMessage[64]; // A log message string.
					sprintf (szLogMessage, "expected_time_tag - %08X", dwExpectedMessage0TimeTag);
					sitalProcess_Log_PrintLine ("#StldTester.Mt.Message: unit - %i, message_number - %i/%i, command - %04X, expected_command - %04X, time_tag - %04X %s\n", s_swDevice3, wMessageNumber, wOverallMessageCount, dmsDecodedMessage.wCommandWord1, wExpectedCommandWord, dmsDecodedMessage.wTimeTag, ((wCommandWord0 == wExpectedCommandWord) ? szLogMessage : ""));
					if (wExpectedCommandWord != dmsDecodedMessage.wCommandWord1)
					{
						sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Message.Error: error - %s\n", "bad_command");
						swResult = sitalReturnCode_TEST_FAIL;
						throw 0;
					}
					else
					{
						if (wCommandWord0 == wExpectedCommandWord)
						{
							if (0xFFFFFFFF == dwExpectedMessage0TimeTag)
							{
								dwExpectedMessage0TimeTag = dmsDecodedMessage.wTimeTag;
							}
							else
							if (0x0000FFFF < dwExpectedMessage0TimeTag)
							{
								dwExpectedMessage0TimeTag = (0x0000FFFF & dwExpectedMessage0TimeTag);
							}

							U16BIT wTimeTagDifference; // The difference between the expected time tag and the actual one.
							wTimeTagDifference = (U16BIT)(dwExpectedMessage0TimeTag - dmsDecodedMessage.wTimeTag);
							wTimeTagDifference = (U16BIT)( (0x8000 < wTimeTagDifference) ? (0x10000 - wTimeTagDifference) : wTimeTagDifference); // To handle a case of time tag rollover.
							if (wTimeTagCalculationInaccuracy < wTimeTagDifference)
							{
								sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Message.Error: error - %s\n", "bad_timing");
								swResult = sitalReturnCode_TEST_FAIL;
								throw 0;
							}

							wRtToBcMessageCount++;
							wExpectedCommandWord = wCommandWord1;
							dwExpectedMessage0TimeTag = ( dmsDecodedMessage.wTimeTag + (wFrameTime * 100 / wTimeTagResolution) ); // Frame time is expressed in units of 100us.
						}
						else
						{
							wBcToRtMessageCount++;
							wExpectedCommandWord = wCommandWord0;
						}
					}
				}
			}
			while (1 == swResult);

			U16BIT wExpectedMessageCount; // The expected total number of messages that have been identified for each of one above defined message.
			wExpectedMessageCount = (wCommandIdCount * wFrameCount);
			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.MessageComposition: unit - %i, RT2BC(%04X) - %i(expected-%i), BC2RT(%04X) - %i(expected-%i)\n", s_swDevice3, wCommandWord0, wRtToBcMessageCount, wExpectedMessageCount, wCommandWord1, wBcToRtMessageCount, wExpectedMessageCount);
			if ((wExpectedMessageCount != wRtToBcMessageCount) || (wExpectedMessageCount != wBcToRtMessageCount))
			{
				sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Error: error - %s\n", "unexpected_message_composition");
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}
		}

		/// Stop device-3 (MT).
		sitalProcess_Log_PrintLine ("#StldTester.Mt.Stop: unit - %i\n", s_swDevice3);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Stop (s_swDevice3) : sitalMt_Stop (s_swDevice3) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice2);
		swResult = sitalDevice_Free (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-3 (MT).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice3);
		swResult = sitalDevice_Free (s_swDevice3);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		swResult = stldSyncEnd (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncEnd.Error: error - %i\n", swResult);
			throw 0;
		}
	}
	catch (int)
	{
		sitalDevice_Free (s_swDevice2);
		sitalDevice_Free (s_swDevice3);
	}

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_11k_1.End: test_succeeded - %c\n", ((sitalReturnCode_SUCCESS == swResult) ? 'Y' : 'N'));
	return swResult;
}



/// <summary>
/// Test the stld1553 library in the henceforth described scenario:
/// Participant[s]: BC2, MT3.
/// Configuration: All participating devices must be able to communicate with each other.
/// Short description: Acceptance Test Plan (ATP) test 461_11k_2 - Inter-message gap times (cycle mode).
/// Along test print suitable info and indications into the log.
///
/// Note:
/// - This function may use any of above defined Acceptance Test Plan (ATP) tests variables.
///   The caller must therefore call the suitable stldTester_Atp_Set* functions prior to calling this function.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Atp_DoTest461_11k_2 ()
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_11k_2.Start\n");

	/// Set test arguments.
	U16BIT wWordCount; // Word count (1-32).
	wWordCount = 32;
	S16BIT swMessageIdStep; // Message-ID-step to use in message creation loop.
	swMessageIdStep = 50;
	U16BIT wRtSubaddressRx; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31).
	wRtSubaddressRx = 1;
	U16BIT wRtSubaddressTx; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31).
	wRtSubaddressTx = 2;
	U16BIT wBasicMessageGap; // The basic message gap time (in units of 1 us) to test.
	wBasicMessageGap = 1024;
	U16BIT wMessageGapStep; // The step to make in message gap time (in units of 1 us) between the various sub-tests.
	wMessageGapStep = 100;
	U16BIT wIterations; // The number of test iterations.
	wIterations = 10;
	U16BIT wTimeTagCalculationInaccuracy; // The maximum tolerated difference between the expected time tag and the actual one.
	wTimeTagCalculationInaccuracy = 25;

	S16BIT swResult; // Result of operation or function call.
	swResult = sitalReturnCode_SUCCESS;

	/// Get device count.
	U16BIT wRequiredDeviceCount; // The number of devices required for this test.
	wRequiredDeviceCount = (1 + max(s_swDevice2, s_swDevice3));
	U16BIT wAvailableDeviceCount; // The total number of devices available in the system.
	swResult = sitalDevice_GetCount (&wAvailableDeviceCount);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %i\n", swResult);
		return swResult;
	}
	sitalProcess_Log_PrintLine ("#StldTester.Device.Count: available - %i, required - %i\n", wAvailableDeviceCount, wRequiredDeviceCount);
	if (wAvailableDeviceCount < wRequiredDeviceCount)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %s\n", "not_enough_devices_installed");
		return sitalReturnCode_SUCCESS;
	}
	if ( (s_swDevice2 == s_swDevice3) )
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Number.Error: error - %s\n", "same_device_used_twice");
		return sitalReturnCode_INVALID_DEVICE_NUMBER;
	}

	try
	{
		swResult = stldSyncStart (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncStart.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-3 (MT) as a real MT.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access - card, mode - mt, alloc_mem - none\n", s_swDevice3);
		swResult = sitalDevice_Initialize (s_swDevice3, sitalAccess_CARD, ((s_bIsRtMtModeTest ? sitalMode_RT_AND_MT : sitalMode_MT) | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Set for device-3 (MT) a time tag resolution of 2 us.
		U16BIT wTimeTagResolution; // Time tag resolution (in units of 1 us).
		wTimeTagResolution = 2;
		sitalProcess_Log_PrintLine ("#StldTester.Device.TimeTagResolution.Set: unit - %i, resolution - %ius\n", s_swDevice3);
		swResult = sitalDevice_TimeTag_SetResolution (s_swDevice3, sitalTimeTagResolution_2US);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.TimeTagResolution.Set.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Start device-3 (MT).
		sitalProcess_Log_PrintLine ("#StldTester.Mt.Start: unit - %i\n", s_swDevice3);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Start (s_swDevice3) : sitalMt_Start (s_swDevice3) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-2 (BC) as a real BC.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-bc, alloc_mem-none\n", s_swDevice2);
		swResult = sitalDevice_Initialize (s_swDevice2, sitalAccess_CARD, (sitalMode_BC | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Configure device-2 (BC) to enable the gap time field.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.MessageGapTime.Configure: unit - %i, policy - enable\n", s_swDevice2);
		swResult = sitalBc_ConfigureMessageGapTimeFieldPolicy (s_swDevice2, TRUE);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.MessageGapTime.Configure.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) data-block-0, a data block of 64 data words.
		S16BIT swDataBlockId_0; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId_0 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i\n", s_swDevice2, swDataBlockId_0, 64, 0);
		swResult = sitalBc_DataBlock_Create (s_swDevice2, swDataBlockId_0, sitalBcDataBlockSizeOption_DOUBLE, NULL, 0);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) data-block-1, a data block of 64 data words.
		S16BIT swDataBlockId_1; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId_1 = 2;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i\n", s_swDevice2, swDataBlockId_1, 64, 0);
		swResult = sitalBc_DataBlock_Create (s_swDevice2, swDataBlockId_1, sitalBcDataBlockSizeOption_DOUBLE, NULL, 0);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) message-0, a RT(RT-address-1/RtSubAddressTx)-to-BC message that uses data-block-0 to receive WordCount data words. Message bus: bus-1.
		S16BIT swMessageId_0; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBc.Create: unit - %i, message - %i, block - %i, tx_rt - %i, tx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddressTx, wWordCount, 0, s_wBus1);
		swResult = sitalBc_Message_CreateRtToBc (s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddressTx, wWordCount, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBc.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) message-1, a BC-to-RT(RT-address-1/RtSubAddressTx message that uses data-block-1 to receive WordCount data words. Message bus: bus-1.
		S16BIT swMessageId_1; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_1 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create: unit - %i, message - %i, block - %i, rx_rt - %i, rx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_1, swDataBlockId_1, s_wRtAddress1, wRtSubaddressRx, wWordCount, 0, s_wBus1);
		swResult = sitalBc_Message_CreateBcToRt (s_swDevice2, swMessageId_1, swDataBlockId_1, s_wRtAddress1, wRtSubaddressRx, wWordCount, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Use given message descriptors to code the command word of above defined messages.
		// Note: Defined messages have been intentionally devised in a way that their [first] command word will differ.
		U16BIT wCommandWord0; // Command word for above defined RT-to-BC message-0.
		swResult = sitalBcStld1553_Command_GetCode (&wCommandWord0, s_wRtAddress1, sitalCommand_TX, wRtSubaddressTx, wWordCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Command.GetCode.Error: error - %i\n", swResult);
			throw 0;
		}
		sitalProcess_Log_PrintLine ("#StldTester.Command.GetCode: rt_address - %i, direction - %i, subaddress_or_mode - %i, wc/mc - %i, code - %04X\n", s_wRtAddress1, sitalCommand_TX, wRtSubaddressTx, wWordCount, wCommandWord0);
		U16BIT wCommandWord1; // Command word for above defined BC-to-RT message-1.
		swResult = sitalBcStld1553_Command_GetCode (&wCommandWord1, s_wRtAddress1, sitalCommand_RX, wRtSubaddressRx, wWordCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Command.GetCode.Error: error - %i\n", swResult);
			throw 0;
		}
		sitalProcess_Log_PrintLine ("#StldTester.Command.GetCode: rt_address - %i, direction - %i, subaddress_or_mode - %i, wc/mc - %i, code - %04X\n", s_wRtAddress1, sitalCommand_RX, wRtSubaddressRx, wWordCount, wCommandWord1);

		/// Create for device-2 (BC) command-0, a XEQ command that executes message-0.
		S16BIT swCommandId_0; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-1, a XEQ command that executes message-1.
		S16BIT swCommandId_1; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_1 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_1, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_1, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_1, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_1, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-2, a subroutine call command that executes minor-frame-0. 
		S16BIT swCommandId_2; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_2 = 2;
		S16BIT swFrameId_0; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_2, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, swFrameId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_2, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, (U16BIT)swFrameId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) minor-frame-0, a frame that contains command-0 and command-1.
		S16BIT swaCommandIds[sitalBcCounter_COMMANDS]; // An array of command-IDs.
		swaCommandIds[0] = swCommandId_0;
		swaCommandIds[1] = swCommandId_1;
		U16BIT wCommandIdCount; // The number of command IDs in the frame (1-sitalBcCounter_FRAME_COMMANDS).
		wCommandIdCount = 2;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i, %i), time - %i, flags - %i\n", s_swDevice2, swFrameId_0, "minor", wCommandIdCount, swCommandId_0, swCommandId_1, 0, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_0, sitalBcFrameType_MINOR, swaCommandIds, wCommandIdCount, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) major-frame-1, a frame that contains command-2 of device-2 (BC).
		S16BIT swFrameId_1; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_1 = 1;
		swaCommandIds[0] = swCommandId_2;
		wCommandIdCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_1, "major", wCommandIdCount, swCommandId_2, 0, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_1, sitalBcFrameType_MAJOR, swaCommandIds, wCommandIdCount, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Loop to test various message gap times:
		U16BIT wMessageGap; // The message gap time (in units of 1 us) that is currently tested.
		wMessageGap = wBasicMessageGap;
		for (S16BIT i=0; i<wIterations; i++, wMessageGap+=wMessageGapStep)
		{
			sitalProcess_Log_PrintLine ("#StldTester.SubTest: iteration - %i\n", i);

			///		Modify for device-2 (BC) message-0 to have a gap of MessageGap.
			S16BIT swMessageId_0; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
			swMessageId_0 = 0;
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBc.Modify: unit - %i, message - %i, block - %i, tx_rt - %i, tx_rt_sa - %i, wc - %i, gap - %i, opts-x%X, modified - x%X\n", s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddressTx, wWordCount, wMessageGap, s_wBus1, sitalBcMessageModifier_MESSAGE1_GAPTIME);
			swResult = sitalBc_Message_ModifyRtToBc (s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddressTx, wWordCount, wMessageGap, s_wBus1, sitalBcMessageModifier_MESSAGE1_GAPTIME);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBc.Modify.Error: error - %i\n", swResult);
				throw 0;
			}

			/// 	Start device-2 (BC) running major-frame-1 for 1-shot.
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", s_swDevice2, swFrameId_1, 1);
			swResult = sitalBc_Start (s_swDevice2, swFrameId_1, 1);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
				throw 0;
			}

			///		Wait for the frame run to end.
			U32BIT dwRemainingFrameRunTime; // The expected remaining frame run time (in ms).
			dwRemainingFrameRunTime = 100; // An exaggerated time.
			swResult = stldTester_Bc_WaitForEndOfFrameRun (s_swDevice2, dwRemainingFrameRunTime, 0, NULL);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.WaitForEndOfFrameRun.Error: error - %i\n", swResult);
				throw 0;
			}

			///		Stop device-2 (BC).
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop: unit - %i\n", s_swDevice2);
			swResult = sitalBc_Stop (s_swDevice2);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
				throw 0;
			}

			///		Verify that device-3 (MT) has detected messages in the expected composition and timing.
			sitalDecodedMessageStructure dmsDecodedMessage; // A decoded message.
			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded: unit - %i, purge - %i\n", s_swDevice3, TRUE);
			swResult = sitalMt_Message_GetFromStackDecoded (s_swDevice3, &dmsDecodedMessage, sitalMessageLocationAndRemoval_NEXT_PURGE, sitalMtStack_ACTIVE);
			if ((0 > swResult) || (1 < swResult))
			{
				sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Error: error - %i\n", swResult);
				throw 0;
			}
			if (0U == swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Error: error - %s\n", "no_message");
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}
			sitalProcess_Log_PrintLine ("#StldTester.Mt.Message: unit - %i, command - %04X, expected_command - %04X, time_tag - %04X\n", s_swDevice3, dmsDecodedMessage.wCommandWord1, wCommandWord0, dmsDecodedMessage.wTimeTag);
			if (wCommandWord0 != dmsDecodedMessage.wCommandWord1)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Message.Error: error - %s\n", "bad_command");
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}
			U16BIT wExpectedMessage1TimeTag; // The time tag expected for the BC-to-RT message to be detected next by device-3 (MT).
			wExpectedMessage1TimeTag = (dmsDecodedMessage.wTimeTag + (wMessageGap / wTimeTagResolution));

			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded: unit - %i, purge - %i\n", s_swDevice3, TRUE);
			swResult = sitalMt_Message_GetFromStackDecoded (s_swDevice3, &dmsDecodedMessage, sitalMessageLocationAndRemoval_NEXT_PURGE, sitalMtStack_ACTIVE);
			if ((0 > swResult) || (1 < swResult))
			{
				sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Error: error - %i\n", swResult);
				throw 0;
			}
			if (0U == swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Error: error - %s\n", "no_message");
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}
			sitalProcess_Log_PrintLine ("#StldTester.Mt.Message: unit - %i, command - %04X, expected_command - %04X, time_tag - %04X, expected_time_tag - %04X\n", s_swDevice3, dmsDecodedMessage.wCommandWord1, wCommandWord1, dmsDecodedMessage.wTimeTag, wExpectedMessage1TimeTag);
			if (wCommandWord1 != dmsDecodedMessage.wCommandWord1)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Message.Error: error - %s\n", "bad_command");
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}
			U16BIT wTimeTagDifference; // The difference between the expected time tag and the actual one.
			wTimeTagDifference = (wExpectedMessage1TimeTag - dmsDecodedMessage.wTimeTag);
			wTimeTagDifference = ( (0x8000 < wTimeTagDifference) ? (0x10000 - wTimeTagDifference) : wTimeTagDifference); // To handle a case of time tag rollover.
			if (wTimeTagCalculationInaccuracy < wTimeTagDifference)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Message.Error: error - %s\n", "bad_timing");
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}

			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded: unit - %i, purge - %i\n", s_swDevice3, TRUE);
			swResult = sitalMt_Message_GetFromStackDecoded (s_swDevice3, &dmsDecodedMessage, sitalMessageLocationAndRemoval_NEXT_PURGE, sitalMtStack_ACTIVE);
			if ((0 > swResult) || (1 < swResult))
			{
				sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Error: error - %i\n", swResult);
				throw 0;
			}
			if (1 == swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Error: error - %s\n", "extra_message");
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}
		}

		/// Stop device-3 (MT).
		sitalProcess_Log_PrintLine ("#StldTester.Mt.Stop: unit - %i\n", s_swDevice3);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Stop (s_swDevice3) : sitalMt_Stop (s_swDevice3) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice2);
		swResult = sitalDevice_Free (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-3 (MT).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice3);
		swResult = sitalDevice_Free (s_swDevice3);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		swResult = stldSyncEnd (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncEnd.Error: error - %i\n", swResult);
			throw 0;
		}
	}
	catch (int)
	{
		sitalDevice_Free (s_swDevice2);
		sitalDevice_Free (s_swDevice3);
	}

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_11k_2.End: test_succeeded - %c\n", ((sitalReturnCode_SUCCESS == swResult) ? 'Y' : 'N'));
	return swResult;
}



/// <summary>
/// Test the stld1553 library in the henceforth described scenario:
/// Participant[s]: RT1, BC2, MT3, RT4.
/// Configuration:
/// - The participating BC must be able to communicate with the participating RT devices.
/// - RT4 must be 2 hubs away from the BC in order to delay its responses to the BC.
///   This way, in case the BC is configured to have response timeout of 18 us, the BC won't accept responses from RT4.
/// Short description: Acceptance Test Plan (ATP) test 461_11k_3 - Response time-out and late response.
/// Along test print suitable info and indications into the log.
///
/// Note:
/// - This function may use any of above defined Acceptance Test Plan (ATP) tests variables.
///   The caller must therefore call the suitable stldTester_Atp_Set* functions prior to calling this function.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Atp_DoTest461_11k_3 ()
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_11k_3.Start\n");

	/// Set test arguments.
	U16BIT wWordCount; // Word count (1-32).
	wWordCount = 32;
	S16BIT swMessageIdStep; // Message-ID-step to use in message creation loop.
	swMessageIdStep = 50;
	U16BIT wRtSubaddressRx; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31).
	wRtSubaddressRx = 1;
	U16BIT wRtSubaddressTx; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31).
	wRtSubaddressTx = 2;
	U16BIT wBasicMessageGap; // The basic message gap time (in units of 1 us) to test.
	wBasicMessageGap = 1024;
	U16BIT wMessageGapStep; // The step to make in message gap time (in units of 1 us) between the various sub-tests.
	wMessageGapStep = 100;
	U16BIT wIterations; // The number of test iterations.
	wIterations = 10;

	S16BIT swResult; // Result of operation or function call.
	swResult = sitalReturnCode_SUCCESS;

	/// Get device count.
	U16BIT wRequiredDeviceCount; // The number of devices required for this test.
	wRequiredDeviceCount = (1 + max(max(max(s_swDevice1, s_swDevice2), s_swDevice3), s_swDevice4));
	U16BIT wAvailableDeviceCount; // The total number of devices available in the system.
	swResult = sitalDevice_GetCount (&wAvailableDeviceCount);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %i\n", swResult);
		return swResult;
	}
	sitalProcess_Log_PrintLine ("#StldTester.Device.Count: available - %i, required - %i\n", wAvailableDeviceCount, wRequiredDeviceCount);
	if (wAvailableDeviceCount < wRequiredDeviceCount)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %s\n", "not_enough_devices_installed");
		return sitalReturnCode_SUCCESS;
	}
	if ( (s_swDevice1 == s_swDevice2) || (s_swDevice1 == s_swDevice3) || (s_swDevice1 == s_swDevice4) || (s_swDevice2 == s_swDevice3) || (s_swDevice2 == s_swDevice4) || (s_swDevice3 == s_swDevice4))
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Number.Error: error - %s\n", "same_device_used_twice");
		return sitalReturnCode_INVALID_DEVICE_NUMBER;
	}

	try
	{
		swResult = stldSyncStart (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncStart.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-3 (MT) as a real MT.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access - card, mode - mt, alloc_mem - none\n", s_swDevice3);
		swResult = sitalDevice_Initialize (s_swDevice3, sitalAccess_CARD, ((s_bIsRtMtModeTest ? sitalMode_RT_AND_MT : sitalMode_MT) | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-1 (RT) as a real RT in advanced mode.
		// Note: Advanced mode will enable r/w to the memory of device-1 (RT) while the device is running.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-rt, alloc_mem-none\n", s_swDevice1);
		swResult = sitalDevice_Initialize (s_swDevice1, sitalAccess_CARD, ((s_bIsRtMtModeTest ? sitalMode_RT_AND_MT : sitalMode_RT) | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Set device-1 (RT) address to RtAddress0.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set: unit - %i, address - %i\n", s_swDevice1, s_wRtAddress1);
		swResult = sitalRt_Address_Set (s_swDevice1, s_wRtAddress1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Legalize on device-1 (RT) all address-types in all directions for all subaddresses and all word counts.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.MessageLegality.Enable: unit - %i, address_type - %i, directions - %i, subaddress - %i, word_count - %08X\n", s_swDevice1, sitalRtAddressType_BOTH, sitalMessageDirection_BOTH, sitalRtSubaddress_ALL, 0xFFFFFFFF);
		swResult = sitalRt_MessageLegality_Enable (s_swDevice1, sitalRtAddressType_BOTH, sitalMessageDirection_BOTH, sitalRtSubaddress_ALL, 0xFFFFFFFF);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.MessageLegality.Enable.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Start device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Start: unit - %i\n", s_swDevice1);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Start (s_swDevice1) : sitalRt_Start (s_swDevice1) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-4 (RT) as a real RT in advanced mode.
		// Note: Advanced mode will enable r/w to the memory of device-1 (RT) while the device is running.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-rt, alloc_mem-none\n", s_swDevice4);
		swResult = sitalDevice_Initialize (s_swDevice4, sitalAccess_CARD, ((s_bIsRtMtModeTest ? sitalMode_RT_AND_MT : sitalMode_RT) | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Set device-4 (RT) address to RtAddress0.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set: unit - %i, address - %i\n", s_swDevice4, s_wRtAddress2);
		swResult = sitalRt_Address_Set (s_swDevice4, s_wRtAddress2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Legalize on device-4 (RT) all address-types in all directions for all subaddresses and all word counts.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.MessageLegality.Enable: unit - %i, address_type - %i, directions - %i, subaddress - %i, word_count - %08X\n", s_swDevice4, sitalRtAddressType_BOTH, sitalMessageDirection_BOTH, sitalRtSubaddress_ALL, 0xFFFFFFFF);
		swResult = sitalRt_MessageLegality_Enable (s_swDevice4, sitalRtAddressType_BOTH, sitalMessageDirection_BOTH, sitalRtSubaddress_ALL, 0xFFFFFFFF);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.MessageLegality.Enable.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Start device-4 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Start: unit - %i\n", s_swDevice4);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Start (s_swDevice4) : sitalRt_Start (s_swDevice4) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-2 (BC) as a real BC.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-bc, alloc_mem-none\n", s_swDevice2);
		swResult = sitalDevice_Initialize (s_swDevice2, sitalAccess_CARD, (sitalMode_BC | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) data-block-0, a data block of 64 data words.
		S16BIT swDataBlockId_0; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId_0 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i\n", s_swDevice2, swDataBlockId_0, 64, 0);
		swResult = sitalBc_DataBlock_Create (s_swDevice2, swDataBlockId_0, sitalBcDataBlockSizeOption_DOUBLE, NULL, 0);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		///	Create for device-2 (BC) message-0, a RT(RtAddress)-to-BC-Mode(*) message. Message bus: bus-1.
		S16BIT swMessageId_0; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBcMode.Create: unit - %i, message - %i, block - %i, rt - %i, direction - %i, mode - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, sitalMessageDirection_TX, 0x0002, 0, s_wBus1);
		swResult = sitalBc_Message_CreateBcToOrFromRtMode (s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, sitalMessageDirection_TX, 0x0002, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBcMode.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		///	Create for device-2 (BC) message-1, a RT(RtAddress)-to-BC-Mode(*) message. Message bus: bus-1.
		S16BIT swMessageId_1; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_1 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBcMode.Create: unit - %i, message - %i, block - %i, rt - %i, direction - %i, mode - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_1, swDataBlockId_0, s_wRtAddress2, sitalMessageDirection_TX, 0x0002, 0, s_wBus1);
		swResult = sitalBc_Message_CreateBcToOrFromRtMode (s_swDevice2, swMessageId_1, swDataBlockId_0, s_wRtAddress2, sitalMessageDirection_TX, 0x0002, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBcMode.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) message-2, a RT(RtAddress1/RtSubAddressTx)-to-RT(RtAddress2/RtSubAddressRx) message that uses data-block-0 to receive WordCount data words. Message bus: bus-1.
		S16BIT swMessageId_2; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_2 = 2;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToRt.Create: unit - %i, message - %i, block - %i, tx_rt - %i, tx_rt_sa - %i, rx_rt - %i, rx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_2, swDataBlockId_0, s_wRtAddress1, wRtSubaddressTx, s_wRtAddress2, wRtSubaddressRx, wWordCount, 0, s_wBus1);
		swResult = sitalBc_Message_CreateRtToRt (s_swDevice2, swMessageId_2, swDataBlockId_0, s_wRtAddress2, wRtSubaddressRx, wWordCount, s_wRtAddress1, wRtSubaddressTx, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToRt.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-0, a XEQ command that executes message-0.
		S16BIT swCommandId_0; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-1, a XEQ command that executes message-1.
		S16BIT swCommandId_1; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_1 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_1, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_1, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_1, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_1, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-2, a XEQ command that executes message-2.
		S16BIT swCommandId_2; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_2 = 2;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_2, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_2, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_2, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_2, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-3, a subroutine call command that executes minor-frame-0. 
		S16BIT swCommandId_3; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_3 = 3;
		S16BIT swFrameId_0; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_3, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, swFrameId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_3, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, (U16BIT)swFrameId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) minor-frame-0, a frame that contains command-0, command-1, and command-2.
		S16BIT swaCommandIds[sitalBcCounter_COMMANDS]; // An array of command-IDs.
		swaCommandIds[0] = swCommandId_0;
		swaCommandIds[1] = swCommandId_1;
		swaCommandIds[1] = swCommandId_2;
		U16BIT wCommandIdCount; // The number of command IDs in the frame (1-sitalBcCounter_FRAME_COMMANDS).
		wCommandIdCount = 3;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i, %i, %i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_0, "minor", wCommandIdCount, swCommandId_0, swCommandId_1, swCommandId_2, 0, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_0, sitalBcFrameType_MINOR, swaCommandIds, wCommandIdCount, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) major-frame-1, a frame that contains command-2.
		S16BIT swFrameId_1; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_1 = 1;
		swaCommandIds[0] = swCommandId_2;
		wCommandIdCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_1, "major", wCommandIdCount, swCommandId_2, 0, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_1, sitalBcFrameType_MAJOR, swaCommandIds, wCommandIdCount, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Loop over all possible response timeouts:
		static const U16BIT s_waResponseTimeout[] // An array that contains the list of tested response timeouts.
							= { sitalResponseTimeout_18US, sitalResponseTimeout_22US, sitalResponseTimeout_50US, sitalResponseTimeout_130US };
		U16BIT wResponseTimeoutCount; // The overall number of tested response timeouts.
		wResponseTimeoutCount = ( sizeof(s_waResponseTimeout) / sizeof(s_waResponseTimeout[0]) );
		for (S16BIT i=0; i<wResponseTimeoutCount; i++)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Iteration: iteration - %i\n", i);

			///		Configure device-3 (MT) to have the next tested response timeout.
			sitalProcess_Log_PrintLine ("#StldTester.Device.ResponseTimeout.Set: unit - %i, timeout - %i\n", s_swDevice3, s_waResponseTimeout[i]);
			swResult = sitalDevice_SetResponseTimeout (s_swDevice3, s_waResponseTimeout[i]);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Device.ResponseTimeout.Set.Error: error - %i\n", swResult);
				throw 0;
			}

			///		Start device-3 (MT).
			sitalProcess_Log_PrintLine ("#StldTester.Mt.Start: unit - %i\n", s_swDevice3);
			swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Start (s_swDevice3) : sitalMt_Start (s_swDevice3) );
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Mt.Start.Error: error - %i\n", swResult);
				throw 0;
			}

			///		Configure device-2 (BC) to have the next tested response timeout.
			sitalProcess_Log_PrintLine ("#StldTester.Device.ResponseTimeout.Set: unit - %i, timeout - %i\n", s_swDevice2, s_waResponseTimeout[i]);
			swResult = sitalDevice_SetResponseTimeout (s_swDevice2, s_waResponseTimeout[i]);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Device.ResponseTimeout.Set.Error: error - %i\n", swResult);
				throw 0;
			}

			///		Start device-2 (BC) running major-frame-1 for 1-shot.
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", s_swDevice2, swFrameId_1, 1);
			swResult = sitalBc_Start (s_swDevice2, swFrameId_1, 1);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
				throw 0;
			}

			///		Wait for the frame run to end.
			U32BIT dwRemainingFrameRunTime; // The expected remaining frame run time (in ms).
			dwRemainingFrameRunTime = 10; // An exaggerated time.
			swResult = stldTester_Bc_WaitForEndOfFrameRun (s_swDevice2, dwRemainingFrameRunTime, 0, NULL);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.WaitForEndOfFrameRun.Error: error - %i\n", swResult);
				throw 0;
			}

			///		Stop device-2 (BC).
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop: unit - %i\n", s_swDevice2);
			swResult = sitalBc_Stop (s_swDevice2);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
				throw 0;
			}

			///		Verify that a response has been received by device-2 (BC) from device-1 (RT) for message-0.
			sitalDecodedMessageStructure dmsDecodedMessage; // A decoded message.
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded: unit - %i, message - %i, purge-y\n", s_swDevice2, swMessageId_0);
			swResult = sitalBc_Message_GetByIdDecoded (s_swDevice2, swMessageId_0, &dmsDecodedMessage, TRUE);
			if (1 != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded.Error: error - %i\n", swResult);
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message: unit - %i, message - %i, command - %04X, data_count - %i, data - %i...%i, time_tag - %04X, bsw - %04X, status_1_relevant - %i, status_1 - %04X\n", s_swDevice2, swMessageId_1, dmsDecodedMessage.wCommandWord1, dmsDecodedMessage.wDataWordCount, dmsDecodedMessage.waData[0], dmsDecodedMessage.waData[dmsDecodedMessage.wDataWordCount - 1], dmsDecodedMessage.wTimeTag, dmsDecodedMessage.wBlockStatus, dmsDecodedMessage.bIsStatus1Relevant, dmsDecodedMessage.wStatus1);
			if (sitalBcBlockStatusWord_NO_RESPONSE == (sitalBcBlockStatusWord_NO_RESPONSE & dmsDecodedMessage.wBlockStatus))
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.FromRt1.Message.BlockStatus.Error: error - %04X\n", dmsDecodedMessage.wBlockStatus);
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}

			///		If the currently tested response timeout is 18 us (the shortest possible):
			///			Verify that a response has not been received by device-2 (BC) from device-4 (RT) for message-1.
			///		Else:
			///			Verify that a response has been received by device-2 (BC) from device-4 (RT) for message-1.
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded: unit - %i, message - %i, purge-y\n", s_swDevice2, swMessageId_1);
			swResult = sitalBc_Message_GetByIdDecoded (s_swDevice2, swMessageId_1, &dmsDecodedMessage, TRUE);
			if (1 != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded.Error: error - %i\n", swResult);
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message: unit - %i, message - %i, command - %04X, data_count - %i, data - %i...%i, time_tag - %04X, bsw - %04X, status_1_relevant - %i, status_1 - %04X\n", s_swDevice2, swMessageId_1, dmsDecodedMessage.wCommandWord1, dmsDecodedMessage.wDataWordCount, dmsDecodedMessage.waData[0], dmsDecodedMessage.waData[dmsDecodedMessage.wDataWordCount - 1], dmsDecodedMessage.wTimeTag, dmsDecodedMessage.wBlockStatus, dmsDecodedMessage.bIsStatus1Relevant, dmsDecodedMessage.wStatus1);
			if (  ( (sitalResponseTimeout_18US == s_waResponseTimeout[i]) ? 0 : sitalBcBlockStatusWord_NO_RESPONSE )  ==  (sitalBcBlockStatusWord_NO_RESPONSE & dmsDecodedMessage.wBlockStatus)  )
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.FromRt4.Message.BlockStatus.Error: error - %04X\n", dmsDecodedMessage.wBlockStatus);
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}

			///		If the currently tested response timeout is 18 us (the shortest possible):
			///			Verify that a response has not been received by device-1 (RT) from device-4 (RT) for message-2.
			///		Else:
			///			Verify that a response has been received by device-1 (RT) from device-4 (RT) for message-2.
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded: unit - %i, message - %i, purge-y\n", s_swDevice2, swMessageId_2);
			swResult = sitalBc_Message_GetByIdDecoded (s_swDevice2, swMessageId_2, &dmsDecodedMessage, TRUE);
			if (1 != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded.Error: error - %i\n", swResult);
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message: unit - %i, message - %i, command1 - %04X, command2 - %04X, data_count - %i, data - %i...%i, time_tag - %04X, bsw - %04X, status_1_relevant - %i, status_1 - %04X\n", s_swDevice2, swMessageId_2, dmsDecodedMessage.wCommandWord1, dmsDecodedMessage.wCommandWord2, dmsDecodedMessage.wDataWordCount, dmsDecodedMessage.waData[0], dmsDecodedMessage.waData[dmsDecodedMessage.wDataWordCount - 1], dmsDecodedMessage.wTimeTag, dmsDecodedMessage.wBlockStatus, dmsDecodedMessage.bIsStatus1Relevant, dmsDecodedMessage.wStatus1);
			if (  ( (sitalResponseTimeout_18US == s_waResponseTimeout[i]) ? 0 : sitalBcBlockStatusWord_NO_RESPONSE )  ==  (sitalBcBlockStatusWord_NO_RESPONSE & dmsDecodedMessage.wBlockStatus)  )
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.FromRt4.Message.BlockStatus.Error: error - %04X\n", dmsDecodedMessage.wBlockStatus);
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}

			///		Stop device-3 (MT).
			sitalProcess_Log_PrintLine ("#StldTester.Mt.Stop: unit - %i\n", s_swDevice3);
			swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Stop (s_swDevice3) : sitalMt_Stop (s_swDevice3) );
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Mt.Stop.Error: error - %i\n", swResult);
				throw 0;
			}

			///		Verify that a response from device-1 (RT) has been identified by device-3 (MT).
			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded: unit - %i, purge - %i\n", s_swDevice3, TRUE);
			swResult = sitalMt_Message_GetFromStackDecoded (s_swDevice3, &dmsDecodedMessage, sitalMessageLocationAndRemoval_NEXT_PURGE, sitalMtStack_ACTIVE);
			if ((0 > swResult) || (1 < swResult))
			{
				sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Error: error - %i\n", swResult);
				throw 0;
			}
			if (0U == swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Message.Error: error - %s\n", "no_message");
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}
			sitalProcess_Log_PrintLine ("#StldTester.Mt.Message: unit - %i, command - %04X, data_count - %i, data - %i...%i, time_tag - %04X, bsw - %04X, status_1_relevant - %i, status_1 - %04X\n", s_swDevice3, dmsDecodedMessage.wCommandWord1, dmsDecodedMessage.wDataWordCount, dmsDecodedMessage.waData[0], dmsDecodedMessage.waData[dmsDecodedMessage.wDataWordCount - 1], dmsDecodedMessage.wTimeTag, dmsDecodedMessage.wBlockStatus, dmsDecodedMessage.bIsStatus1Relevant, dmsDecodedMessage.wStatus1);
			if (sitalMtBlockStatusWord_NO_RESPONSE == (sitalMtBlockStatusWord_NO_RESPONSE & dmsDecodedMessage.wBlockStatus))
			{
				sitalProcess_Log_PrintLine ("#StldTester.Mt.Message.BlockStatus.Error: error - %04X\n", dmsDecodedMessage.wBlockStatus);
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}

			///		If the currently tested response timeout is 18 us (the shortest possible):
			///			Verify that a response from device-4 (RT) has not been identified by device-3 (MT).
			///		Else:
			///			Verify that a response has been received by device-2 (BC) from device-4 (RT).
			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded: unit - %i, purge - %i\n", s_swDevice3, TRUE);
			swResult = sitalMt_Message_GetFromStackDecoded (s_swDevice3, &dmsDecodedMessage, sitalMessageLocationAndRemoval_NEXT_PURGE, sitalMtStack_ACTIVE);
			if ((0 > swResult) || (1 < swResult))
			{
				sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Error: error - %i\n", swResult);
				throw 0;
			}
			if (0U == swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Message.Error: error - %s\n", "no_message");
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}
			sitalProcess_Log_PrintLine ("#StldTester.Mt.Message: unit - %i, command - %04X, data_count - %i, data - %i...%i, time_tag - %04X, bsw - %04X, status_1_relevant - %i, status_1 - %04X\n", s_swDevice3, dmsDecodedMessage.wCommandWord1, dmsDecodedMessage.wDataWordCount, dmsDecodedMessage.waData[0], dmsDecodedMessage.waData[dmsDecodedMessage.wDataWordCount - 1], dmsDecodedMessage.wTimeTag, dmsDecodedMessage.wBlockStatus, dmsDecodedMessage.bIsStatus1Relevant, dmsDecodedMessage.wStatus1);
			if (  ( (sitalResponseTimeout_18US == s_waResponseTimeout[i]) ? 0 : sitalMtBlockStatusWord_NO_RESPONSE )  ==  (sitalMtBlockStatusWord_NO_RESPONSE & dmsDecodedMessage.wBlockStatus)  )
			{
				sitalProcess_Log_PrintLine ("#StldTester.Mt.Message.BlockStatus.Error: error - %04X\n", dmsDecodedMessage.wBlockStatus);
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}
		}

		/// Stop device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Stop: unit - %i\n", s_swDevice1);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Stop (s_swDevice1) : sitalRt_Stop (s_swDevice1) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Stop device-4 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Stop: unit - %i\n", s_swDevice4);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Stop (s_swDevice4) : sitalRt_Stop (s_swDevice4) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice1);
		swResult = sitalDevice_Free (s_swDevice1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice2);
		swResult = sitalDevice_Free (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-3 (MT).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice3);
		swResult = sitalDevice_Free (s_swDevice3);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-4 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice4);
		swResult = sitalDevice_Free (s_swDevice4);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		swResult = stldSyncEnd (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncEnd.Error: error - %i\n", swResult);
			throw 0;
		}
	}
	catch (int)
	{
		sitalDevice_Free (s_swDevice1);
		sitalDevice_Free (s_swDevice2);
		sitalDevice_Free (s_swDevice3);
		sitalDevice_Free (s_swDevice4);
	}

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_11k_3.End: test_succeeded - %c\n", ((sitalReturnCode_SUCCESS == swResult) ? 'Y' : 'N'));
	return swResult;
}



/// <summary>
/// Test the stld1553 library in the henceforth described scenario:
/// Participant[s]: RT1, BC2, MT3.
/// Configuration: All participating devices must be able to communicate with each other. If device BC2 has two bus sockets, it must be actually connected to two buses, or otherwise the test will fail.
/// Short description: Acceptance Test Plan (ATP) test 461_11k_4 - Automatic retries.
/// Along test print suitable info and indications into the log.
///
/// Note:
/// - This function may use any of above defined Acceptance Test Plan (ATP) tests variables.
///   The caller must therefore call the suitable stldTester_Atp_Set* functions prior to calling this function.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Atp_DoTest461_11k_4 ()
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_11k_4.Start\n");

	/// Set test arguments.
	U16BIT wTheOtherBus; // The alternate bus (sitalBcControlWord_BUS_*) assuming that bus-1 is the first-chance bus.
	wTheOtherBus = ( (sitalBcControlWord_BUS_A == s_wBus1) ? sitalBcControlWord_BUS_B : sitalBcControlWord_BUS_A );
	U16BIT wWordCount; // Word count (1-32).
	wWordCount = 32;
	U16BIT wRtSubaddressRx; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31).
	wRtSubaddressRx = 1;

	S16BIT swResult; // Result of operation or function call.
	swResult = sitalReturnCode_SUCCESS;

	/// Get device count.
	U16BIT wRequiredDeviceCount; // The number of devices required for this test.
	wRequiredDeviceCount = (1 + max(max(s_swDevice1, s_swDevice2), s_swDevice3));
	U16BIT wAvailableDeviceCount; // The total number of devices available in the system.
	swResult = sitalDevice_GetCount (&wAvailableDeviceCount);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %i\n", swResult);
		return swResult;
	}
	sitalProcess_Log_PrintLine ("#StldTester.Device.Count: available - %i, required - %i\n", wAvailableDeviceCount, wRequiredDeviceCount);
	if (wAvailableDeviceCount < wRequiredDeviceCount)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %s\n", "not_enough_devices_installed");
		return sitalReturnCode_SUCCESS;
	}
	if ( (s_swDevice1 == s_swDevice2) || (s_swDevice1 == s_swDevice3) || (s_swDevice2 == s_swDevice3))
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Number.Error: error - %s\n", "same_device_used_twice");
		return sitalReturnCode_INVALID_DEVICE_NUMBER;
	}

	try
	{
		swResult = stldSyncStart (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncStart.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-3 (MT) as a real MT.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access - card, mode - mt, alloc_mem - none\n", s_swDevice3);
		swResult = sitalDevice_Initialize (s_swDevice3, sitalAccess_CARD, ((s_bIsRtMtModeTest ? sitalMode_RT_AND_MT : sitalMode_MT) | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Start device-3 (MT).
		sitalProcess_Log_PrintLine ("#StldTester.Mt.Start: unit - %i\n", s_swDevice3);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Start (s_swDevice3) : sitalMt_Start (s_swDevice3) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-1 (RT) as a real RT in advanced mode.
		// Note: Advanced mode will enable r/w to the memory of device-1 (RT) while the device is running.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-rt, alloc_mem-none\n", s_swDevice1);
		swResult = sitalDevice_Initialize (s_swDevice1, sitalAccess_CARD, ((s_bIsRtMtModeTest ? sitalMode_RT_AND_MT : sitalMode_RT) | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Set device-1 (RT) address to RtAddress0.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set: unit - %i, address - %i\n", s_swDevice1, s_wRtAddress1);
		swResult = sitalRt_Address_Set (s_swDevice1, s_wRtAddress1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Legalize on device-1 (RT) all address-types in all directions for all subaddresses and all word counts.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.MessageLegality.Enable: unit - %i, address_type - %i, directions - %i, subaddress - %i, word_count - %08X\n", s_swDevice1, sitalRtAddressType_BOTH, sitalMessageDirection_BOTH, sitalRtSubaddress_ALL, 0xFFFFFFFF);
		swResult = sitalRt_MessageLegality_Enable (s_swDevice1, sitalRtAddressType_BOTH, sitalMessageDirection_BOTH, sitalRtSubaddress_ALL, 0xFFFFFFFF);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.MessageLegality.Enable.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Start device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Start: unit - %i\n", s_swDevice1);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Start (s_swDevice1) : sitalRt_Start (s_swDevice1) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-2 (BC) as a real BC.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-bc, alloc_mem-none\n", s_swDevice2);
		swResult = sitalDevice_Initialize (s_swDevice2, sitalAccess_CARD, (sitalMode_BC | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Find out whether device-2 (BC) is connected to two buses.
		sitalProcess_Log_PrintLine ("#StldTester.Device.GetInfo: unit - %i\n", s_swDevice2);
		sitalDeviceCapabilitiesStructure dcsDeviceCapabilities; // A description of the capabilities of a device.
		dcsDeviceCapabilities.dwStructureSize = sizeof(sitalDeviceCapabilitiesStructure);
		swResult = sitalDevice_GetInformation (s_swDevice2, &dcsDeviceCapabilities);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.GetInfo.Error: error - %i\n", swResult);
			throw 0;
		}
		BOOLEAN bBcHasTwoBusSockets; // A flag that says whether device BC2 has two bus sockets. Recall: In case it has, it must be actually connected to two buses as part of the required configuration, or otherwise the test will fail.
		bBcHasTwoBusSockets = dcsDeviceCapabilities.bIsDualBus;
		sitalProcess_Log_PrintLine ("#StldTester.Device.GetInfo: unit - %i, 2_bus_sockets - %i\n", s_swDevice2, bBcHasTwoBusSockets);

		/// Create for device-2 (BC) data-block-0, a data block of 64 data words.
		S16BIT swDataBlockId_0; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId_0 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i\n", s_swDevice2, swDataBlockId_0, 64, 0);
		swResult = sitalBc_DataBlock_Create (s_swDevice2, swDataBlockId_0, sitalBcDataBlockSizeOption_DOUBLE, NULL, 0);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) message-0, a BC-to-RT(RT-address-1/RtSubAddressRx message that uses data-block-0 to transmit WordCount data words. Message bus: bus-1.
		S16BIT swMessageId_0; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create: unit - %i, message - %i, block - %i, rx_rt - %i, rx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddressRx, wWordCount, 0, (sitalBcControlWord_RETRY_ENABLED | s_wBus1));
		swResult = sitalBc_Message_CreateBcToRt (s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddressRx, wWordCount, 0, (sitalBcControlWord_RETRY_ENABLED | s_wBus1));
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) message-1, a BC-to-RT(RT-address-2/RtSubAddressRx message that uses data-block-0 to transmit WordCount data words. Message bus: bus-1.
		S16BIT swMessageId_1; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_1 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create: unit - %i, message - %i, block - %i, rx_rt - %i, rx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_1, swDataBlockId_0, s_wRtAddress2, wRtSubaddressRx, wWordCount, 0, (sitalBcControlWord_RETRY_ENABLED | s_wBus1));
		swResult = sitalBc_Message_CreateBcToRt (s_swDevice2, swMessageId_1, swDataBlockId_0, s_wRtAddress2, wRtSubaddressRx, wWordCount, 0, (sitalBcControlWord_RETRY_ENABLED | s_wBus1));
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Use given message descriptors to code the command word of above defined messages.
		// Note: Defined messages have been intentionally devised in a way that their [first] command word will differ.
		U16BIT wCommandWord0; // Command word for above defined BC-to-RT message-0.
		swResult = sitalBcStld1553_Command_GetCode (&wCommandWord0, s_wRtAddress1, sitalCommand_RX, wRtSubaddressRx, wWordCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Command.GetCode.Error: error - %i\n", swResult);
			throw 0;
		}
		sitalProcess_Log_PrintLine ("#StldTester.Command.GetCode: rt_address - %i, direction - %i, subaddress_or_mode - %i, wc/mc - %i, code - %04X\n", s_wRtAddress1, sitalCommand_RX, wRtSubaddressRx, wWordCount, wCommandWord0);
		U16BIT wCommandWord1; // Command word for above defined BC-to-RT message-1.
		swResult = sitalBcStld1553_Command_GetCode (&wCommandWord1, s_wRtAddress2, sitalCommand_RX, wRtSubaddressRx, wWordCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Command.GetCode.Error: error - %i\n", swResult);
			throw 0;
		}
		sitalProcess_Log_PrintLine ("#StldTester.Command.GetCode: rt_address - %i, direction - %i, subaddress_or_mode - %i, wc/mc - %i, code - %04X\n", s_wRtAddress2, sitalCommand_RX, wRtSubaddressRx, wWordCount, wCommandWord1);

		/// Create for device-2 (BC) command-0, a XEQ command that executes message-0.
		S16BIT swCommandId_0; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-1, a XEQ command that executes message-1.
		S16BIT swCommandId_1; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_1 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_1, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_1, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_1, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_1, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-2, a subroutine call command that executes minor-frame-0. 
		S16BIT swCommandId_2; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_2 = 2;
		S16BIT swFrameId_0; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_2, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, swFrameId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_2, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, (U16BIT)swFrameId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) minor-frame-0, a frame that contains command-0 and command-1.
		S16BIT swaCommandIds[sitalBcCounter_COMMANDS]; // An array of command-IDs.
		swaCommandIds[0] = swCommandId_0;
		swaCommandIds[1] = swCommandId_1;
		U16BIT wCommandIdCount; // The number of command IDs in the frame (1-sitalBcCounter_FRAME_COMMANDS).
		wCommandIdCount = 2;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i, %i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_0, "minor", wCommandIdCount, swCommandId_0, swCommandId_1, 0, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_0, sitalBcFrameType_MINOR, swaCommandIds, wCommandIdCount, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) major-frame-1, a frame that contains command-2.
		S16BIT swFrameId_1; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_1 = 1;
		swaCommandIds[0] = swCommandId_2;
		wCommandIdCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_1, "major", wCommandIdCount, swCommandId_2, 0, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_1, sitalBcFrameType_MAJOR, swaCommandIds, wCommandIdCount, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Loop over all possible retry and retry buses combinations:
		static const U16BIT s_waRetries[] // An array that contains the list of tested retry counts.
							= { sitalMessageRetryCount_NONE, sitalMessageRetryCount_ONE, sitalMessageRetryCount_TWO };
		U16BIT wRetriesCount; // The overall number of tested retry counts.
		wRetriesCount = ( sizeof(s_waRetries) / sizeof(s_waRetries[0]) );
		static const U16BIT s_waRetryBus[] // An array that contains the list of tested retry buses.
							= { sitalMessageRetryBus_SAME, sitalMessageRetryBus_OTHER };
		U16BIT wRetryBusCount; // The overall number of tested retry buses.
		wRetryBusCount = ( sizeof(s_waRetryBus) / sizeof(s_waRetryBus[0]) );
		for (S16BIT iRetryCountIndex=0; iRetryCountIndex<wRetriesCount; iRetryCountIndex++)
		{
			for (S16BIT iFirstRetryBus=0; iFirstRetryBus<wRetryBusCount; iFirstRetryBus++)
			{
				///		If the currently tested first retry bus option is "the other bus" but there's no "other bus":
				///			Skip this sub-test.
				if ((sitalMessageRetryBus_OTHER == s_waRetryBus[iFirstRetryBus]) && (FALSE == bBcHasTwoBusSockets))
				{
					continue;
				}

				for (S16BIT iSecondRetryBus=0; iSecondRetryBus<wRetryBusCount; iSecondRetryBus++)
				{
					///		If the currently tested second retry bus option is "the other bus" but there's no "other bus":
					///			Skip this sub-test.
					if ((sitalMessageRetryBus_OTHER == s_waRetryBus[iSecondRetryBus]) && (FALSE == bBcHasTwoBusSockets))
					{
						continue;
					}

					///		Configure device-2 (BC) to have the next tested retry and retry buses combination.
					sitalProcess_Log_PrintLine ("#StldTester.Bc.MessageRetryPolicy.Set: unit - %i, retry_count - %i(%i), 1st_retry_bus - %i(%i), 2nd_retry_bus - %i(%i)\n", s_swDevice2, s_waRetries[iRetryCountIndex], iRetryCountIndex, s_waRetryBus[iFirstRetryBus], iFirstRetryBus, s_waRetryBus[iSecondRetryBus], iSecondRetryBus);
					swResult = sitalBc_MessageRetryPolicy_Set (s_swDevice2, s_waRetries[iRetryCountIndex], s_waRetryBus[iFirstRetryBus], s_waRetryBus[iSecondRetryBus], 0);
					if (sitalReturnCode_SUCCESS != swResult)
					{
						sitalProcess_Log_PrintLine ("#StldTester.Bc.MessageRetryPolicy.Set.Error: error - %i\n", swResult);
						throw 0;
					}

					///		Start device-2 (BC) running major-frame-1 for 1-shot.
					sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", s_swDevice2, swFrameId_1, 1);
					swResult = sitalBc_Start (s_swDevice2, swFrameId_1, 1);
					if (sitalReturnCode_SUCCESS != swResult)
					{
						sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
						throw 0;
					}

					///		Wait for the frame run to end.
					U32BIT dwRemainingFrameRunTime; // The expected remaining frame run time (in ms).
					dwRemainingFrameRunTime = 10; // An exaggerated time.
					swResult = stldTester_Bc_WaitForEndOfFrameRun (s_swDevice2, dwRemainingFrameRunTime, 0, NULL);
					if (sitalReturnCode_SUCCESS != swResult)
					{
						sitalProcess_Log_PrintLine ("#StldTester.Bc.WaitForEndOfFrameRun.Error: error - %i\n", swResult);
						throw 0;
					}

					///		Stop device-2 (BC).
					sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop: unit - %i\n", s_swDevice2);
					swResult = sitalBc_Stop (s_swDevice2);
					if (sitalReturnCode_SUCCESS != swResult)
					{
						sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
						throw 0;
					}

					///		Verify that device-1 (RT) has responded message-0.
					sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded: unit - %i, purge - %i\n", s_swDevice3, TRUE);
					sitalDecodedMessageStructure dmsDecodedMessage; // A decoded message.
					swResult = sitalMt_Message_GetFromStackDecoded (s_swDevice3, &dmsDecodedMessage, sitalMessageLocationAndRemoval_NEXT_PURGE, sitalMtStack_ACTIVE);
					if ((0 > swResult) || (1 < swResult))
					{
						sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Error: error - %i\n", swResult);
						throw 0;
					}
					if (0U == swResult)
					{
						sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Error: error - %s\n", "no_message");
						swResult = sitalReturnCode_TEST_FAIL;
						throw 0;
					}
					sitalProcess_Log_PrintLine ("#StldTester.Mt.Message: unit - %i, command - %04X, expected_command - %04X, time_tag - %04X\n", s_swDevice3, dmsDecodedMessage.wCommandWord1, wCommandWord0, dmsDecodedMessage.wTimeTag);
					if (wCommandWord0 != dmsDecodedMessage.wCommandWord1)
					{
						sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Message.Error: error - %s\n", "bad_command");
						swResult = sitalReturnCode_TEST_FAIL;
						throw 0;
					}
					sitalProcess_Log_PrintLine ("#StldTester.Mt.Message: unit - %i, message - %i, message_block_status - %04X, response_received - %i, status_1_relevant - %i, status_1 - %04X, rt_address - %i, expected_rt_address - %i\n", s_swDevice2, swMessageId_0, dmsDecodedMessage.wBlockStatus, (0U == (sitalRtBlockStatusWord_NO_RESPONSE & dmsDecodedMessage.wBlockStatus)), dmsDecodedMessage.bIsStatus1Relevant, dmsDecodedMessage.wStatus1, ((sitalIeee1553_Status_TERMINAL_ADDRESS & dmsDecodedMessage.wStatus1) >> sitalIeee1553_Status_OFFSET_OF_TERMINAL_ADDRESS), s_wRtAddress1);
					if ((sitalRtBlockStatusWord_NO_RESPONSE == (sitalRtBlockStatusWord_NO_RESPONSE & dmsDecodedMessage.wBlockStatus)) || (((sitalIeee1553_Status_TERMINAL_ADDRESS & dmsDecodedMessage.wStatus1) >> sitalIeee1553_Status_OFFSET_OF_TERMINAL_ADDRESS) != s_wRtAddress1))
					{
						sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Message.Error: error - %s\n", "bad_response");
						swResult = sitalReturnCode_TEST_FAIL;
						throw 0;
					}

					///		Verify that message-1 has been transmitted once and then retried at the configured number of retry times and on the configured retry buses.
					/// Note: As message-1 is targeted at RT-address-2 and that RT-address isn't assigned to any active RT, message-1 won't be responded, and must be therefore retried, if so configured.
					for (S16BIT iRetry=0; iRetry<=s_waRetries[iRetryCountIndex]; iRetry++)
					{
						sitalProcess_Log_PrintLine ("#StldTester.MessageRetryLoop: retry - %i\n", iRetry);

						sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded: unit - %i, purge - %i\n", s_swDevice3, TRUE);
						sitalDecodedMessageStructure dmsDecodedMessage; // A decoded message.
						swResult = sitalMt_Message_GetFromStackDecoded (s_swDevice3, &dmsDecodedMessage, sitalMessageLocationAndRemoval_NEXT_PURGE, sitalMtStack_ACTIVE);
						if ((0 > swResult) || (1 < swResult))
						{
							sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Error: error - %i\n", swResult);
							throw 0;
						}
						if (0U == swResult)
						{
							sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Error: error - %s\n", "no_message"); // Note: If device BC2 may be connected to 2 buses, it must be so connected, or otherwise this error will occur!
							swResult = sitalReturnCode_TEST_FAIL;
							throw 0;
						}
						sitalProcess_Log_PrintLine ("#StldTester.Mt.Message: unit - %i, command - %04X, expected_command - %04X, time_tag - %04X\n", s_swDevice3, dmsDecodedMessage.wCommandWord1, wCommandWord1, dmsDecodedMessage.wTimeTag);
						if (wCommandWord1 != dmsDecodedMessage.wCommandWord1)
						{
							sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Message.Error: error - %s\n", "bad_command");
							swResult = sitalReturnCode_TEST_FAIL;
							throw 0;
						}
						sitalProcess_Log_PrintLine ("#StldTester.Mt.Message: unit - %i, message - %i, message_block_status - %04X, response_received - %i, status_1_relevant - %i, status_1 - %04X, rt_address - %i, expected_rt_address - %i\n", s_swDevice2, swMessageId_0, dmsDecodedMessage.wBlockStatus, (0U == (sitalRtBlockStatusWord_NO_RESPONSE & dmsDecodedMessage.wBlockStatus)), dmsDecodedMessage.bIsStatus1Relevant, dmsDecodedMessage.wStatus1, ((sitalIeee1553_Status_TERMINAL_ADDRESS & dmsDecodedMessage.wStatus1) >> sitalIeee1553_Status_OFFSET_OF_TERMINAL_ADDRESS), s_wRtAddress2);
						if (sitalRtBlockStatusWord_NO_RESPONSE != (sitalRtBlockStatusWord_NO_RESPONSE & dmsDecodedMessage.wBlockStatus))
						{
							sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Message.Error: error - %s\n", "unexpected_response"); // A response shouldn't have been detected: This may happen in case some unexpected RT device, whose RT address is 2, has responded this message.
							swResult = sitalReturnCode_TEST_FAIL;
							throw 0;
						}
						U16BIT wExpectedBus; // The bus on which the current message should have been transmitted.
						wExpectedBus = ( ((0U == iRetry) || ((1 == iRetry) && (sitalMessageRetryBus_SAME == s_waRetryBus[iFirstRetryBus])) || ((2 == iRetry) && (sitalMessageRetryBus_SAME == s_waRetryBus[iSecondRetryBus]))) ? s_wBus1 : wTheOtherBus ); // Note: The identity of the retry bus (same/other) relates the identity of the bus of the original message, not that of the previous (and faulty) retry!
						wExpectedBus = ( (sitalBcControlWord_BUS_A == wExpectedBus) ? 0 : 1 );
						U16BIT wActualBus; // The bus on which the current message have been actually transmitted.
						wActualBus = ( (sitalMtBlockStatusWord_BUS_B == (sitalMtBlockStatusWord_BUS_B & dmsDecodedMessage.wBlockStatus)) ? 1 : 0 );
						sitalProcess_Log_PrintLine ("#StldTester.Mt.Message.Buses: unit - %i, message - %i, message_block_status - %04X, original_bus - %i, actual_bus - %i, expected_bus - %i\n", s_swDevice2, swMessageId_1, dmsDecodedMessage.wBlockStatus, ((sitalBcControlWord_BUS_A == s_wBus1) ? 0 : 1), wActualBus, wExpectedBus);
						if (wActualBus != wExpectedBus)
						{
							sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Message.Error: error - %s\n", "bad_bus");
							swResult = sitalReturnCode_TEST_FAIL;
							throw 0;
						}
					}

					///		Verify that no additional messages have been transmitted.
					sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded: unit - %i, purge - %i\n", s_swDevice3, TRUE);
					swResult = sitalMt_Message_GetFromStackDecoded (s_swDevice3, &dmsDecodedMessage, sitalMessageLocationAndRemoval_NEXT_PURGE, sitalMtStack_ACTIVE);
					if ((0 > swResult) || (1 < swResult))
					{
						sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Error: error - %i\n", swResult);
						throw 0;
					}
					if (1 == swResult)
					{
						sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Error: error - %s\n", "extra_message");
						swResult = sitalReturnCode_TEST_FAIL;
						throw 0;
					}
				}
			}
		}

		/// Stop device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Stop: unit - %i\n", s_swDevice1);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Stop (s_swDevice1) : sitalRt_Stop (s_swDevice1) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Stop device-3 (MT).
		sitalProcess_Log_PrintLine ("#StldTester.Mt.Stop: unit - %i\n", s_swDevice3);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Stop (s_swDevice3) : sitalMt_Stop (s_swDevice3) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice1);
		swResult = sitalDevice_Free (s_swDevice1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice2);
		swResult = sitalDevice_Free (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-3 (MT).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice3);
		swResult = sitalDevice_Free (s_swDevice3);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		swResult = stldSyncEnd (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncEnd.Error: error - %i\n", swResult);
			throw 0;
		}
	}
	catch (int)
	{
		sitalDevice_Free (s_swDevice1);
		sitalDevice_Free (s_swDevice2);
		sitalDevice_Free (s_swDevice3);
	}

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_11k_4.End: test_succeeded - %c\n", ((sitalReturnCode_SUCCESS == swResult) ? 'Y' : 'N'));
	return swResult;
}



/// <summary>
/// Test the stld1553 library in the henceforth described scenario:
/// Participant[s]: BC2, MT3.
/// Configuration: All participating devices must be able to communicate with each other.
/// Short description: Acceptance Test Plan (ATP) test 461_11k_5 - Frame auto-repeat.
/// Along test print suitable info and indications into the log.
///
/// Note:
/// - This function may use any of above defined Acceptance Test Plan (ATP) tests variables.
///   The caller must therefore call the suitable stldTester_Atp_Set* functions prior to calling this function.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Atp_DoTest461_11k_5 ()
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_11k_5.Start\n");

	/// Set test arguments.
	U16BIT wWordCount; // Word count (1-32).
	wWordCount = 32;
	U16BIT wRtSubaddressRx; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31).
	wRtSubaddressRx = 1;
	U16BIT wInitialFrameCount; // The initial number of frames to test.
	wInitialFrameCount = 2;
	U16BIT wFinalFrameCount; // The final number of frames to test.
	wFinalFrameCount = 11;
	U16BIT wFrameTime; // The time (in units of 100 us) that the frame will take to complete.
	wFrameTime = 20; // 2ms.

	S16BIT swResult; // Result of operation or function call.
	swResult = sitalReturnCode_SUCCESS;

	/// Get device count.
	U16BIT wRequiredDeviceCount; // The number of devices required for this test.
	wRequiredDeviceCount = (1 + max(s_swDevice2, s_swDevice3));
	U16BIT wAvailableDeviceCount; // The total number of devices available in the system.
	swResult = sitalDevice_GetCount (&wAvailableDeviceCount);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %i\n", swResult);
		return swResult;
	}
	sitalProcess_Log_PrintLine ("#StldTester.Device.Count: available - %i, required - %i\n", wAvailableDeviceCount, wRequiredDeviceCount);
	if (wAvailableDeviceCount < wRequiredDeviceCount)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %s\n", "not_enough_devices_installed");
		return sitalReturnCode_SUCCESS;
	}
	if ( (s_swDevice2 == s_swDevice3) )
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Number.Error: error - %s\n", "same_device_used_twice");
		return sitalReturnCode_INVALID_DEVICE_NUMBER;
	}

	try
	{
		swResult = stldSyncStart (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncStart.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-3 (MT) as a real MT.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access - card, mode - mt, alloc_mem - none\n", s_swDevice3);
		swResult = sitalDevice_Initialize (s_swDevice3, sitalAccess_CARD, ((s_bIsRtMtModeTest ? sitalMode_RT_AND_MT : sitalMode_MT) | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Start device-3 (MT).
		sitalProcess_Log_PrintLine ("#StldTester.Mt.Start: unit - %i\n", s_swDevice3);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Start (s_swDevice3) : sitalMt_Start (s_swDevice3) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-2 (BC) as a real BC.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-bc, alloc_mem-none\n", s_swDevice2);
		swResult = sitalDevice_Initialize (s_swDevice2, sitalAccess_CARD, (sitalMode_BC | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) data-block-0, a data block of 64 data words.
		S16BIT swDataBlockId_0; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId_0 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i\n", s_swDevice2, swDataBlockId_0, 64, 0);
		swResult = sitalBc_DataBlock_Create (s_swDevice2, swDataBlockId_0, sitalBcDataBlockSizeOption_DOUBLE, NULL, 0);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) message-0, a BC-to-RT(RT-address-1/RtSubAddressRx message that uses data-block-0 to transmit WordCount data words. Message bus: bus-1.
		S16BIT swMessageId_0; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create: unit - %i, message - %i, block - %i, rx_rt - %i, rx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddressRx, wWordCount, 0, s_wBus1);
		swResult = sitalBc_Message_CreateBcToRt (s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddressRx, wWordCount, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) message-1, a BC-to-RT(RT-address-2/RtSubAddressRx message that uses data-block-0 to transmit WordCount data words. Message bus: bus-1.
		S16BIT swMessageId_1; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_1 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create: unit - %i, message - %i, block - %i, rx_rt - %i, rx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_1, swDataBlockId_0, s_wRtAddress2, wRtSubaddressRx, wWordCount, 0, s_wBus1);
		swResult = sitalBc_Message_CreateBcToRt (s_swDevice2, swMessageId_1, swDataBlockId_0, s_wRtAddress2, wRtSubaddressRx, wWordCount, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Use given message descriptors to code the command word of above defined messages.
		// Note: Defined messages have been intentionally devised in a way that their [first] command word will differ.
		U16BIT wCommandWord0; // Command word for above defined BC-to-RT message-0.
		swResult = sitalBcStld1553_Command_GetCode (&wCommandWord0, s_wRtAddress1, sitalCommand_RX, wRtSubaddressRx, wWordCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Command.GetCode.Error: error - %i\n", swResult);
			throw 0;
		}
		sitalProcess_Log_PrintLine ("#StldTester.Command.GetCode: rt_address - %i, direction - %i, subaddress_or_mode - %i, wc/mc - %i, code - %04X\n", s_wRtAddress1, sitalCommand_RX, wRtSubaddressRx, wWordCount, wCommandWord0);
		U16BIT wCommandWord1; // Command word for above defined BC-to-RT message-1.
		swResult = sitalBcStld1553_Command_GetCode (&wCommandWord1, s_wRtAddress2, sitalCommand_RX, wRtSubaddressRx, wWordCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Command.GetCode.Error: error - %i\n", swResult);
			throw 0;
		}
		sitalProcess_Log_PrintLine ("#StldTester.Command.GetCode: rt_address - %i, direction - %i, subaddress_or_mode - %i, wc/mc - %i, code - %04X\n", s_wRtAddress2, sitalCommand_RX, wRtSubaddressRx, wWordCount, wCommandWord1);

		/// Create for device-2 (BC) command-0, a XEQ command that executes message-0.
		S16BIT swCommandId_0; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-1, a XEQ command that executes message-1.
		S16BIT swCommandId_1; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_1 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_1, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_1, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_1, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_1, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-2, a subroutine call command that executes minor-frame-0. 
		S16BIT swCommandId_2; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_2 = 2;
		S16BIT swFrameId_0; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_2, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, swFrameId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_2, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, (U16BIT)swFrameId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) minor-frame-0, a frame that contains command-0 and command-1.
		S16BIT swaCommandIds[sitalBcCounter_COMMANDS]; // An array of command-IDs.
		swaCommandIds[0] = swCommandId_0;
		swaCommandIds[1] = swCommandId_1;
		U16BIT wCommandIdCount; // The number of command IDs in the frame (1-sitalBcCounter_FRAME_COMMANDS).
		wCommandIdCount = 2;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i, %i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_0, "minor", wCommandIdCount, swCommandId_0, swCommandId_1, 0, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_0, sitalBcFrameType_MINOR, swaCommandIds, wCommandIdCount, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) major-frame-1, a frame that contains command-2.
		S16BIT swFrameId_1; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_1 = 1;
		swaCommandIds[0] = swCommandId_2;
		wCommandIdCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_1, "major", wCommandIdCount, swCommandId_2, wFrameTime, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_1, sitalBcFrameType_MAJOR, swaCommandIds, wCommandIdCount, wFrameTime, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Loop over all required series of N-shots:
		for (U16BIT wFrameCount=wInitialFrameCount; wFrameCount<=wFinalFrameCount; wFrameCount++)
		{
			///		Start device-2 (BC) running major-frame-1 for FrameCount-shots.
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", s_swDevice2, swFrameId_1, wFrameCount);
			swResult = sitalBc_Start (s_swDevice2, swFrameId_1, wFrameCount);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
				throw 0;
			}

			///		Wait for the frame run to end.
			U32BIT dwRemainingFrameRunTime; // The expected remaining frame run time (in ms).
			dwRemainingFrameRunTime = (wFrameCount * (wFrameTime / 10));
			swResult = stldTester_Bc_WaitForEndOfFrameRun (s_swDevice2, dwRemainingFrameRunTime, 0, NULL);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.WaitForEndOfFrameRun.Error: error - %i\n", swResult);
				throw 0;
			}

			///		Stop device-2 (BC).
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop: unit - %i\n", s_swDevice2);
			swResult = sitalBc_Stop (s_swDevice2);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
				throw 0;
			}

			///		Verify that device-3 (MT) has detected messages in the expected composition.
			sitalDecodedMessageStructure dmsDecodedMessage; // A decoded message.
			U16BIT wBcToRt1MessageCount; // The total number of BC-to-RT1 messages that have been identified.
			wBcToRt1MessageCount = 0;
			U16BIT wBcToRt2MessageCount; // The total number of BC-to-RT2 messages that have been identified.
			wBcToRt2MessageCount = 0;
			do
			{
				sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded: unit - %i, purge - %i\n", s_swDevice3, TRUE);
				swResult = sitalMt_Message_GetFromStackDecoded (s_swDevice3, &dmsDecodedMessage, sitalMessageLocationAndRemoval_NEXT_PURGE, sitalMtStack_ACTIVE);
				if (1 == swResult)
				{
					if (wCommandWord0 == dmsDecodedMessage.wCommandWord1)
					{
						wBcToRt1MessageCount++;
					}
					else
					if (wCommandWord1 == dmsDecodedMessage.wCommandWord1)
					{
						wBcToRt2MessageCount++;
					}
					else
					{
						sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Message.Error: error - %04X\n", dmsDecodedMessage.wCommandWord1);
						swResult = sitalReturnCode_TEST_FAIL;
						throw 0;
					}
				}
			}
			while (1 == swResult);
			if (0 > swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Error: error - %i\n", swResult);
				throw 0;
			}
			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.MessageComposition: unit - %i, BC2RT1(%04X) - %i(expected-%i), BC2RT2(%04X) - %i(expected-%i)\n", s_swDevice3, wCommandWord0, wBcToRt1MessageCount, wFrameCount, wCommandWord1, wBcToRt2MessageCount, wFrameCount);
			if ((wFrameCount != wBcToRt1MessageCount) || (wFrameCount != wBcToRt2MessageCount))
			{
				sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Error: error - %s\n", "unexpected_message_composition");
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}

			///		Verify that no additional messages have been transmitted.
			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded: unit - %i, purge - %i\n", s_swDevice3, TRUE);
			swResult = sitalMt_Message_GetFromStackDecoded (s_swDevice3, &dmsDecodedMessage, sitalMessageLocationAndRemoval_NEXT_PURGE, sitalMtStack_ACTIVE);
			if ((0 > swResult) || (1 < swResult))
			{
				sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Error: error - %i\n", swResult);
				throw 0;
			}
			if (1 == swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Error: error - %s\n", "extra_message");
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}
		}

		/// Stop device-3 (MT).
		sitalProcess_Log_PrintLine ("#StldTester.Mt.Stop: unit - %i\n", s_swDevice3);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Stop (s_swDevice3) : sitalMt_Stop (s_swDevice3) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice2);
		swResult = sitalDevice_Free (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-3 (MT).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice3);
		swResult = sitalDevice_Free (s_swDevice3);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		swResult = stldSyncEnd (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncEnd.Error: error - %i\n", swResult);
			throw 0;
		}
	}
	catch (int)
	{
		sitalDevice_Free (s_swDevice2);
		sitalDevice_Free (s_swDevice3);
	}

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_11k_5.End: test_succeeded - %c\n", ((sitalReturnCode_SUCCESS == swResult) ? 'Y' : 'N'));
	return swResult;
}



/// <summary>
/// Test the stld1553 library in the henceforth described scenario:
/// Participant[s]: BC2, MT3.
/// Configuration: All participating devices must be able to communicate with each other.
/// Short description: Acceptance Test Plan (ATP) test 461_11k_6 - Bus A/B selection for transmission.
/// Along test print suitable info and indications into the log.
///
/// Note:
/// - This function may use any of above defined Acceptance Test Plan (ATP) tests variables.
///   The caller must therefore call the suitable stldTester_Atp_Set* functions prior to calling this function.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Atp_DoTest461_11k_6 ()
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_11k_6.Start\n");

	/// Set test arguments.
	U16BIT wTheOtherBus; // The alternate bus (sitalBcControlWord_BUS_*) assuming that bus-1 is the first-chance bus.
	wTheOtherBus = ( (sitalBcControlWord_BUS_A == s_wBus1) ? sitalBcControlWord_BUS_B : sitalBcControlWord_BUS_A );
	U16BIT wWordCount; // Word count (1-32).
	wWordCount = 32;
	U16BIT wRtSubaddressRx; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31).
	wRtSubaddressRx = 1;
	U16BIT wInitialFrameCount; // The initial number of frames to test.
	wInitialFrameCount = 1;
	U16BIT wFinalFrameCount; // The final number of frames to test.
	wFinalFrameCount = 1;

	S16BIT swResult; // Result of operation or function call.
	swResult = sitalReturnCode_SUCCESS;

	/// Get device count.
	U16BIT wRequiredDeviceCount; // The number of devices required for this test.
	wRequiredDeviceCount = (1 + max(s_swDevice2, s_swDevice3));
	U16BIT wAvailableDeviceCount; // The total number of devices available in the system.
	swResult = sitalDevice_GetCount (&wAvailableDeviceCount);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %i\n", swResult);
		return swResult;
	}
	sitalProcess_Log_PrintLine ("#StldTester.Device.Count: available - %i, required - %i\n", wAvailableDeviceCount, wRequiredDeviceCount);
	if (wAvailableDeviceCount < wRequiredDeviceCount)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %s\n", "not_enough_devices_installed");
		return sitalReturnCode_SUCCESS;
	}
	if ( (s_swDevice2 == s_swDevice3) )
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Number.Error: error - %s\n", "same_device_used_twice");
		return sitalReturnCode_INVALID_DEVICE_NUMBER;
	}

	try
	{
		swResult = stldSyncStart (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncStart.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-3 (MT) as a real MT.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access - card, mode - mt, alloc_mem - none\n", s_swDevice3);
		swResult = sitalDevice_Initialize (s_swDevice3, sitalAccess_CARD, ((s_bIsRtMtModeTest ? sitalMode_RT_AND_MT : sitalMode_MT) | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Start device-3 (MT).
		sitalProcess_Log_PrintLine ("#StldTester.Mt.Start: unit - %i\n", s_swDevice3);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Start (s_swDevice3) : sitalMt_Start (s_swDevice3) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-2 (BC) as a real BC.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-bc, alloc_mem-none\n", s_swDevice2);
		swResult = sitalDevice_Initialize (s_swDevice2, sitalAccess_CARD, (sitalMode_BC | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Find out whether device-2 (BC) is connected to two buses.
		sitalProcess_Log_PrintLine ("#StldTester.Device.GetInfo: unit - %i\n", s_swDevice2);
		sitalDeviceCapabilitiesStructure dcsDeviceCapabilities; // A description of the capabilities of a device.
		dcsDeviceCapabilities.dwStructureSize = sizeof(sitalDeviceCapabilitiesStructure);
		swResult = sitalDevice_GetInformation (s_swDevice2, &dcsDeviceCapabilities);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.GetInfo.Error: error - %i\n", swResult);
			throw 0;
		}
		BOOLEAN bIsSecondBus; // A flag that says whether a second bus is available, or not.
		bIsSecondBus = dcsDeviceCapabilities.bIsDualBus;

		/// Create for device-2 (BC) data-block-0, a data block of 64 data words.
		S16BIT swDataBlockId_0; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId_0 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i\n", s_swDevice2, swDataBlockId_0, 64, 0);
		swResult = sitalBc_DataBlock_Create (s_swDevice2, swDataBlockId_0, sitalBcDataBlockSizeOption_DOUBLE, NULL, 0);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) message-0, a BC-to-RT(RT-address-1/RtSubAddressRx message that uses data-block-0 to transmit WordCount data words. Message bus: bus-1.
		S16BIT swMessageId_0; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create: unit - %i, message - %i, block - %i, rx_rt - %i, rx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddressRx, wWordCount, 0, s_wBus1);
		swResult = sitalBc_Message_CreateBcToRt (s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddressRx, wWordCount, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) message-1, a BC-to-RT(RT-address-2/RtSubAddressRx message that uses data-block-0 to transmit WordCount data words. Message bus: The other bus (in regard with bus-1).
		S16BIT swMessageId_1; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_1 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create: unit - %i, message - %i, block - %i, rx_rt - %i, rx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_1, swDataBlockId_0, s_wRtAddress2, wRtSubaddressRx, wWordCount, 0, wTheOtherBus);
		swResult = sitalBc_Message_CreateBcToRt (s_swDevice2, swMessageId_1, swDataBlockId_0, s_wRtAddress2, wRtSubaddressRx, wWordCount, 0, wTheOtherBus);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Use given message descriptors to code the command word of above defined messages.
		// Note: Defined messages have been intentionally devised in a way that their [first] command word will differ.
		U16BIT wCommandWord0; // Command word for above defined BC-to-RT message-0.
		swResult = sitalBcStld1553_Command_GetCode (&wCommandWord0, s_wRtAddress1, sitalCommand_RX, wRtSubaddressRx, wWordCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Command.GetCode.Error: error - %i\n", swResult);
			throw 0;
		}
		U16BIT wCommandWord1; // Command word for above defined BC-to-RT message-1.
		swResult = sitalBcStld1553_Command_GetCode (&wCommandWord1, s_wRtAddress2, sitalCommand_RX, wRtSubaddressRx, wWordCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Command.GetCode.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-0, a XEQ command that executes message-0.
		S16BIT swCommandId_0; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-1, a XEQ command that executes message-1.
		S16BIT swCommandId_1; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_1 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_1, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_1, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_1, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_1, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-2, a subroutine call command that executes minor-frame-0. 
		S16BIT swCommandId_2; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_2 = 2;
		S16BIT swFrameId_0; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_2, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, swFrameId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_2, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, (U16BIT)swFrameId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) minor-frame-0, a frame that contains command-0 and command-1.
		S16BIT swaCommandIds[sitalBcCounter_COMMANDS]; // An array of command-IDs.
		swaCommandIds[0] = swCommandId_0;
		swaCommandIds[1] = swCommandId_1;
		U16BIT wCommandIdCount; // The number of command IDs in the frame (1-sitalBcCounter_FRAME_COMMANDS).
		wCommandIdCount = 2;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i, %i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_0, "minor", wCommandIdCount, swCommandId_0, swCommandId_1, 0, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_0, sitalBcFrameType_MINOR, swaCommandIds, wCommandIdCount, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) major-frame-1, a frame that contains command-2.
		S16BIT swFrameId_1; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_1 = 1;
		swaCommandIds[0] = swCommandId_2;
		wCommandIdCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_1, "major", wCommandIdCount, swCommandId_2, 0, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_1, sitalBcFrameType_MAJOR, swaCommandIds, wCommandIdCount, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		///	Start device-2 (BC) running major-frame-1 for 1-shot.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", s_swDevice2, swFrameId_1, 1);
		swResult = sitalBc_Start (s_swDevice2, swFrameId_1, 1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		///		Wait for the frame run to end.
		U32BIT dwRemainingFrameRunTime; // The expected remaining frame run time (in ms).
		dwRemainingFrameRunTime = 10; // Anexaggerated time.
		swResult = stldTester_Bc_WaitForEndOfFrameRun (s_swDevice2, dwRemainingFrameRunTime, 0, NULL);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.WaitForEndOfFrameRun.Error: error - %i\n", swResult);
			throw 0;
		}

		///	Stop device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop: unit - %i\n", s_swDevice2);
		swResult = sitalBc_Stop (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		///	Verify that device-3 (MT) has detected messages in the expected composition and carrier buses.
		sitalDecodedMessageStructure dmsDecodedMessage; // A decoded message.
		sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded: unit - %i, purge - %i\n", s_swDevice3, TRUE);
		swResult = sitalMt_Message_GetFromStackDecoded (s_swDevice3, &dmsDecodedMessage, sitalMessageLocationAndRemoval_NEXT_PURGE, sitalMtStack_ACTIVE);
		if ((0 > swResult) || (1 < swResult))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Error: error - %i\n", swResult);
			throw 0;
		}
		if (0U == swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Message.Error: error - %s\n", "no_message");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		sitalProcess_Log_PrintLine ("#StldTester.Mt.Message: unit - %i, command - %04X, expected_command - %04X, time_tag - %04X\n", s_swDevice3, dmsDecodedMessage.wCommandWord1, wCommandWord0, dmsDecodedMessage.wTimeTag);
		if (wCommandWord0 != dmsDecodedMessage.wCommandWord1)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Message.Error: error - %s\n", "bad_command");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		U16BIT wExpectedBus; // The bus on which the current message should have been transmitted.
		wExpectedBus = s_wBus1;
		wExpectedBus = ( (sitalBcControlWord_BUS_A == wExpectedBus) ? 0 : 1 );
		sitalProcess_Log_PrintLine ("#StldTester.Mt.Message: unit - %i, message - %i, message_block_status - %04X, bus - %i, expected_bus - %i\n", s_swDevice2, swMessageId_0, dmsDecodedMessage.wBlockStatus, ( (sitalMtBlockStatusWord_BUS_B == (sitalMtBlockStatusWord_BUS_B & dmsDecodedMessage.wBlockStatus)) ? 1 : 0 ), wExpectedBus);
		if (( (sitalMtBlockStatusWord_BUS_B == (sitalMtBlockStatusWord_BUS_B & dmsDecodedMessage.wBlockStatus)) ? 1 : 0 ) != wExpectedBus)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Message.Error: error - %s\n", "bad_bus");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded: unit - %i, purge - %i\n", s_swDevice3, TRUE);
		swResult = sitalMt_Message_GetFromStackDecoded (s_swDevice3, &dmsDecodedMessage, sitalMessageLocationAndRemoval_NEXT_PURGE, sitalMtStack_ACTIVE);
		if ((0 > swResult) || (1 < swResult))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Error: error - %i\n", swResult);
			throw 0;
		}
		if (0U == swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Message.Error: error - %s\n", "no_message");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		sitalProcess_Log_PrintLine ("#StldTester.Mt.Message: unit - %i, command - %04X, expected_command - %04X, time_tag - %04X\n", s_swDevice3, dmsDecodedMessage.wCommandWord1, wCommandWord1, dmsDecodedMessage.wTimeTag);
		if (wCommandWord1 != dmsDecodedMessage.wCommandWord1)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Message.Error: error - %s\n", "bad_command");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		wExpectedBus = wTheOtherBus;
		wExpectedBus = ( (sitalBcControlWord_BUS_A == wExpectedBus) ? 0 : 1 );
		sitalProcess_Log_PrintLine ("#StldTester.Mt.Message: unit - %i, message - %i, message_block_status - %04X, bus - %i, expected_bus - %i\n", s_swDevice2, swMessageId_1, dmsDecodedMessage.wBlockStatus, ( (sitalMtBlockStatusWord_BUS_B == (sitalMtBlockStatusWord_BUS_B & dmsDecodedMessage.wBlockStatus)) ? 1 : 0 ), wExpectedBus);
		if (( (sitalMtBlockStatusWord_BUS_B == (sitalMtBlockStatusWord_BUS_B & dmsDecodedMessage.wBlockStatus)) ? 1 : 0 ) != wExpectedBus)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Message.Error: error - %s\n", "bad_bus");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		///	Modify for device-2 (BC) message-0 to use the opposite bus.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Modify: unit - %i, message - %i, block - %i, tx_rt - %i, tx_rt_sa - %i, wc - %i, gap - %i, opts-x%X, modified - x%X\n", s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddressRx, wWordCount, 0, wTheOtherBus, sitalBcMessageModifier_MESSAGE1_BC_CONTROL);
		swResult = sitalBc_Message_ModifyBcToRt (s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddressRx, wWordCount, 0, wTheOtherBus, sitalBcMessageModifier_MESSAGE1_BC_CONTROL);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Modify.Error: error - %i\n", swResult);
			throw 0;
		}

		///	Modify for device-2 (BC) message-0 to use the opposite bus.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Modify: unit - %i, message - %i, block - %i, tx_rt - %i, tx_rt_sa - %i, wc - %i, gap - %i, opts-x%X, modified - x%X\n", s_swDevice2, swMessageId_1, swDataBlockId_0, s_wRtAddress2, wRtSubaddressRx, wWordCount, 0, s_wBus1, sitalBcMessageModifier_MESSAGE1_BC_CONTROL);
		swResult = sitalBc_Message_ModifyBcToRt (s_swDevice2, swMessageId_1, swDataBlockId_0, s_wRtAddress2, wRtSubaddressRx, wWordCount, 0, s_wBus1, sitalBcMessageModifier_MESSAGE1_BC_CONTROL);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Modify.Error: error - %i\n", swResult);
			throw 0;
		}

		///	Start device-2 (BC) running major-frame-1 for 1-shot.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", s_swDevice2, swFrameId_1, 1);
		swResult = sitalBc_Start (s_swDevice2, swFrameId_1, 1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		///		Wait for the frame run to end.
		dwRemainingFrameRunTime = 10; // An exaggerated time.
		swResult = stldTester_Bc_WaitForEndOfFrameRun (s_swDevice2, dwRemainingFrameRunTime, 0, NULL);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.WaitForEndOfFrameRun.Error: error - %i\n", swResult);
			throw 0;
		}

		///	Stop device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop: unit - %i\n", s_swDevice2);
		swResult = sitalBc_Stop (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		///	Verify that device-3 (MT) has detected messages in the expected composition and carrier buses.
		sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded: unit - %i, purge - %i\n", s_swDevice3, TRUE);
		swResult = sitalMt_Message_GetFromStackDecoded (s_swDevice3, &dmsDecodedMessage, sitalMessageLocationAndRemoval_NEXT_PURGE, sitalMtStack_ACTIVE);
		if ((0 > swResult) || (1 < swResult))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Error: error - %i\n", swResult);
			throw 0;
		}
		if (0U == swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Message.Error: error - %s\n", "no_message");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		sitalProcess_Log_PrintLine ("#StldTester.Mt.Message: unit - %i, command - %04X, expected_command - %04X, time_tag - %04X\n", s_swDevice3, dmsDecodedMessage.wCommandWord1, wCommandWord0, dmsDecodedMessage.wTimeTag);
		if (wCommandWord0 != dmsDecodedMessage.wCommandWord1)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Message.Error: error - %s\n", "bad_command");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		wExpectedBus = wTheOtherBus;
		wExpectedBus = ( (sitalBcControlWord_BUS_A == wExpectedBus) ? 0 : 1 );
		sitalProcess_Log_PrintLine ("#StldTester.Mt.Message: unit - %i, message - %i, message_block_status - %04X, bus - %i, expected_bus - %i\n", s_swDevice2, swMessageId_0, dmsDecodedMessage.wBlockStatus, ( (sitalMtBlockStatusWord_BUS_B == (sitalMtBlockStatusWord_BUS_B & dmsDecodedMessage.wBlockStatus)) ? 1 : 0 ), wExpectedBus);
		if (( (sitalMtBlockStatusWord_BUS_B == (sitalMtBlockStatusWord_BUS_B & dmsDecodedMessage.wBlockStatus)) ? 1 : 0 ) != wExpectedBus)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Message.Error: error - %s\n", "bad_bus");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded: unit - %i, purge - %i\n", s_swDevice3, TRUE);
		swResult = sitalMt_Message_GetFromStackDecoded (s_swDevice3, &dmsDecodedMessage, sitalMessageLocationAndRemoval_NEXT_PURGE, sitalMtStack_ACTIVE);
		if ((0 > swResult) || (1 < swResult))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Error: error - %i\n", swResult);
			throw 0;
		}
		if (0U == swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Message.Error: error - %s\n", "no_message");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		sitalProcess_Log_PrintLine ("#StldTester.Mt.Message: unit - %i, command - %04X, expected_command - %04X, time_tag - %04X\n", s_swDevice3, dmsDecodedMessage.wCommandWord1, wCommandWord1, dmsDecodedMessage.wTimeTag);
		if (wCommandWord1 != dmsDecodedMessage.wCommandWord1)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Message.Error: error - %s\n", "bad_command");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		wExpectedBus = s_wBus1;
		wExpectedBus = ( (sitalBcControlWord_BUS_A == wExpectedBus) ? 0 : 1 );
		sitalProcess_Log_PrintLine ("#StldTester.Mt.Message: unit - %i, message - %i, message_block_status - %04X, bus - %i, expected_bus - %i\n", s_swDevice2, swMessageId_1, dmsDecodedMessage.wBlockStatus, ( (sitalMtBlockStatusWord_BUS_B == (sitalMtBlockStatusWord_BUS_B & dmsDecodedMessage.wBlockStatus)) ? 1 : 0 ), wExpectedBus);
		if (( (sitalMtBlockStatusWord_BUS_B == (sitalMtBlockStatusWord_BUS_B & dmsDecodedMessage.wBlockStatus)) ? 1 : 0 ) != wExpectedBus)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Message.Error: error - %s\n", "bad_bus");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// Stop device-3 (MT).
		sitalProcess_Log_PrintLine ("#StldTester.Mt.Stop: unit - %i\n", s_swDevice3);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Stop (s_swDevice3) : sitalMt_Stop (s_swDevice3) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice2);
		swResult = sitalDevice_Free (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-3 (MT).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice3);
		swResult = sitalDevice_Free (s_swDevice3);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		swResult = stldSyncEnd (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncEnd.Error: error - %i\n", swResult);
			throw 0;
		}
	}
	catch (int)
	{
		sitalDevice_Free (s_swDevice2);
		sitalDevice_Free (s_swDevice3);
	}

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_11k_6.End: test_succeeded - %c\n", ((sitalReturnCode_SUCCESS == swResult) ? 'Y' : 'N'));
	return swResult;
}



/// <summary>
/// Test the stld1553 library in the henceforth described scenario:
/// Participant[s]: BC2, MT3.
/// Configuration: All participating devices must be able to communicate with each other.
/// Short description: Acceptance Test Plan (ATP) test 461_11l - Modify messages, data, or setup on fly.
/// Along test print suitable info and indications into the log.
///
/// Note:
/// - This function may use any of above defined Acceptance Test Plan (ATP) tests variables.
///   The caller must therefore call the suitable stldTester_Atp_Set* functions prior to calling this function.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Atp_DoTest461_11l ()
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_11l.Start\n");

	/// Set test arguments.
	U16BIT wWordCount; // Word count (1-32).
	wWordCount = 32;
	U16BIT wRtSubaddressRx; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31).
	wRtSubaddressRx = 1;
	U16BIT wFrameTime; // The time (in units of 100 us) that the frame will take to complete.
	wFrameTime = 1000; // 100ms.

	S16BIT swResult; // Result of operation or function call.
	swResult = sitalReturnCode_SUCCESS;
	InterProcessSemaphore ipsSemaphore (s_szBcSemaphoreName, 1); // A signaling object that is used by an assigned ISR function to signal interrupts to waiting threads.

	/// Get device count.
	U16BIT wRequiredDeviceCount; // The number of devices required for this test.
	wRequiredDeviceCount = (1 + max(s_swDevice2, s_swDevice3));
	U16BIT wAvailableDeviceCount; // The total number of devices available in the system.
	swResult = sitalDevice_GetCount (&wAvailableDeviceCount);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %i\n", swResult);
		return swResult;
	}
	sitalProcess_Log_PrintLine ("#StldTester.Device.Count: available - %i, required - %i\n", wAvailableDeviceCount, wRequiredDeviceCount);
	if (wAvailableDeviceCount < wRequiredDeviceCount)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %s\n", "not_enough_devices_installed");
		return sitalReturnCode_SUCCESS;
	}
	if ( (s_swDevice2 == s_swDevice3) )
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Number.Error: error - %s\n", "same_device_used_twice");
		return sitalReturnCode_INVALID_DEVICE_NUMBER;
	}

	try
	{
		swResult = stldSyncStart (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncStart.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-3 (MT) as a real MT.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access - card, mode - mt, alloc_mem - none\n", s_swDevice3);
		swResult = sitalDevice_Initialize (s_swDevice3, sitalAccess_CARD, ((s_bIsRtMtModeTest ? sitalMode_RT_AND_MT : sitalMode_MT) | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Start device-3 (MT).
		sitalProcess_Log_PrintLine ("#StldTester.Mt.Start: unit - %i\n", s_swDevice3);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Start (s_swDevice3) : sitalMt_Start (s_swDevice3) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-2 (BC) as a real BC.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-bc, alloc_mem-none\n", s_swDevice2);
		swResult = sitalDevice_Initialize (s_swDevice2, sitalAccess_CARD, (sitalMode_BC | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) data-block-0, a data block of 64 data words, and fill the first WordCount data words with ascending values.
		S16BIT swDataBlockId_0; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId_0 = 1;
		U16BIT waBuffer[32]; // A buffer to read-from/write-into along data-block write/read operations, respectively.
		U16BIT wAscendingDataValue; // An ascending data value.
		wAscendingDataValue = 0;
		for (int i=0; i<wWordCount; i++, wAscendingDataValue++) waBuffer[i] = wAscendingDataValue;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i, buf_contents - %i...\n", s_swDevice2, swDataBlockId_0, 64, wWordCount, waBuffer[0]);
		swResult = sitalBc_DataBlock_Create (s_swDevice2, swDataBlockId_0, sitalBcDataBlockSizeOption_DOUBLE, waBuffer, wWordCount);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) data-block-1, a data block of 64 data words, and fill the first WordCount data words with ascending values.
		S16BIT swDataBlockId_1; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId_1 = 2;
		for (int i=0; i<wWordCount; i++, wAscendingDataValue++) waBuffer[i] = wAscendingDataValue;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i, buf_contents - %i...\n", s_swDevice2, swDataBlockId_1, 64, wWordCount, waBuffer[0]);
		swResult = sitalBc_DataBlock_Create (s_swDevice2, swDataBlockId_1, sitalBcDataBlockSizeOption_DOUBLE, waBuffer, wWordCount);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) message-0, a BC-to-RT(RT-address-1/RtSubAddressRx message that uses data-block-0 to transmit WordCount data words. Message bus: bus-1.
		S16BIT swMessageId_0; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create: unit - %i, message - %i, block - %i, rx_rt - %i, rx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddressRx, wWordCount, 0, s_wBus1);
		swResult = sitalBc_Message_CreateBcToRt (s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddressRx, wWordCount, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) message-1, a BC-to-RT(RT-address-1/RtSubAddressRx message that uses data-block-0 to transmit WordCount data words. Message bus: bus-1.
		S16BIT swMessageId_1; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_1 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create: unit - %i, message - %i, block - %i, rx_rt - %i, rx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_1, swDataBlockId_1, s_wRtAddress1, wRtSubaddressRx, wWordCount, 0, s_wBus1);
		swResult = sitalBc_Message_CreateBcToRt (s_swDevice2, swMessageId_1, swDataBlockId_1, s_wRtAddress1, wRtSubaddressRx, wWordCount, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-0, a XEQ command that executes message-0.
		S16BIT swCommandId_0; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-1, a XEQ command that executes message-1.
		S16BIT swCommandId_1; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_1 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_1, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_1, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_1, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_1, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-2, a subroutine call command that executes minor-frame-0.
		S16BIT swCommandId_2; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_2 = 2;
		S16BIT swFrameId_0; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_2, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, swFrameId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_2, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, (U16BIT)swFrameId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-3, a subroutine call command that executes minor-frame-1.
		S16BIT swCommandId_3; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_3 = 3;
		S16BIT swFrameId_1; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_1 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_3, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, swFrameId_1, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_3, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, (U16BIT)swFrameId_1, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) minor-frame-0, a frame that contains command-0.
		S16BIT swaCommandIds[sitalBcCounter_COMMANDS]; // An array of command-IDs.
		swaCommandIds[0] = swCommandId_0;
		U16BIT wCommandIdCount; // The number of command IDs in the frame (1-sitalBcCounter_FRAME_COMMANDS).
		wCommandIdCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_0, "minor", wCommandIdCount, swCommandId_0, 0, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_0, sitalBcFrameType_MINOR, swaCommandIds, wCommandIdCount, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) minor-frame-1, a frame that contains command-1.
		swaCommandIds[0] = swCommandId_1;
		wCommandIdCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_1, "minor", wCommandIdCount, swCommandId_1, 0, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_1, sitalBcFrameType_MINOR, swaCommandIds, wCommandIdCount, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) major-frame-2, a frame that contains command-2 and command-3.
		S16BIT swFrameId_2; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_2 = 2;
		swaCommandIds[0] = swCommandId_2;
		swaCommandIds[1] = swCommandId_3;
		wCommandIdCount = 2;
		U16BIT wNumberOfMinorFramesPerMajorFrame; // The number of minor frames to run.
		wNumberOfMinorFramesPerMajorFrame = wCommandIdCount;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i, %i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_2, "major", wCommandIdCount, swCommandId_2, swCommandId_3, wFrameTime, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_2, sitalBcFrameType_MAJOR, swaCommandIds, wCommandIdCount, wFrameTime, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Configure device-2 (BC) to get IRQ-3 notifications using an ISR that collects interrupt data, counts ISR calls, and signals interested threads.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Manipulate: unit - %i, enable - %i, irq - %08X, callback - %s\n", s_swDevice2, TRUE, sitalInterruptRegister2_BC_IRQ3, "stldTester_Bc_UserIsr");
		swResult = sitalDevice_Irq_Manipulate (s_swDevice2, TRUE, sitalInterruptRegister2_BC_IRQ3, stldTester_Bc_UserIsr);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Set.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create a signaling object.
		sitalProcess_Log_PrintLine ("#StldTester.Semaphore.Create: name - %s\n", s_szBcSemaphoreName);
		if (0U != ipsSemaphore.Create ())
		{
			sitalProcess_Log_PrintLine ("#StldTester.Semaphore.Create.Error: error - %i\n", GetLastError_NonOsDependent());
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		///	Start device-2 (BC) running major-frame-2 for 1-shot.
		U16BIT wFrameCount; // The number of frames to run.
		wFrameCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", s_swDevice2, swFrameId_2, wFrameCount);
		swResult = sitalBc_Start (s_swDevice2, swFrameId_2, wFrameCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		///	Loop over all transmitted minor frames:
		S16BIT swTransmittedDataBlockId; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)) to be transmitted in the next transmitted message/frame.
		swTransmittedDataBlockId = swDataBlockId_0;
		U32BIT dwSleepTime; // Sleeping period (in ms).
		dwSleepTime = ( (1 *  (wFrameTime / 10)) + stldInterruptLatency_Get () );
		U16BIT wTotalMinorFrameCount; // The total number of minor frames to run.
		wTotalMinorFrameCount = (wNumberOfMinorFramesPerMajorFrame * wFrameCount);
		for (U16BIT iMinorFrame=0; iMinorFrame<wTotalMinorFrameCount; iMinorFrame++)
		{
			///		Wait for the currently transmitted minor frame to end.
			if (0U != ipsSemaphore.Catch (dwSleepTime)) 
			{ 
				sitalProcess_Log_PrintLine ("#StldTester.Semaphore.Wait.Error: error - %s\n", "timeout");
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}
			sitalProcess_Log_PrintLine ("#StldTester.Wait: time - %ims\n", dwSleepTime);
			sitalProcess_Log_PrintLine ("\tWAIT_FOR %i us\n", (dwSleepTime * 1000));

			///		Update the data in the buffer of the data block of the message of the minor frame that ended.
			for (int i=0; i<wWordCount; i++, wAscendingDataValue++) waBuffer[i] = wAscendingDataValue;
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Write: unit - %i, block - %i, offset - %i, buf_size - %i, buf_contents - %04X...\n", s_swDevice2, swTransmittedDataBlockId, 0, wWordCount, waBuffer[0]);
			swResult = sitalBc_DataBlock_Write (s_swDevice2, swDataBlockId_0, waBuffer, wWordCount, 0);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Write.Error: error - %i\n", swResult);
				throw 0;
			}

			swTransmittedDataBlockId = ( (swDataBlockId_0 == swTransmittedDataBlockId) ? swDataBlockId_1 : swDataBlockId_0 );
		}

		/// Wait for the frame run to end.
		U32BIT dwRemainingFrameRunTime; // The expected remaining frame run time (in ms).
		dwRemainingFrameRunTime = (wTotalMinorFrameCount * (wFrameTime / 10)); // Total frame run time; less time has actually left.
		swResult = stldTester_Bc_WaitForEndOfFrameRun (s_swDevice2, dwRemainingFrameRunTime, 0, NULL);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.WaitForEndOfFrameRun.Error: error - %i\n", swResult);
			throw 0;
		}

		///	Stop device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop: unit - %i\n", s_swDevice2);
		swResult = sitalBc_Stop (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice2);
		swResult = sitalDevice_Free (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Close the signaling object.
		sitalProcess_Log_PrintLine ("#StldTester.Semaphore.Close: name - %s\n", s_szBcSemaphoreName);
		if (0U != ipsSemaphore.Close ())
		{
			sitalProcess_Log_PrintLine ("#StldTester.Semaphore.Close.Error: error - %i\n", GetLastError_NonOsDependent());
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		if (0U != s_ipspBcSemaphore->Close ())
		{
			sitalProcess_Log_PrintLine ("#StldTester.Semaphore.Callback.Close.Error: error - %i\n", GetLastError_NonOsDependent());
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		delete (s_ipspBcSemaphore);
		s_ipspBcSemaphore = NULL;

		///	Verify that device-3 (MT) has detected messages with the expected data.
		wAscendingDataValue = 0;
		for (U16BIT iMinorFrame=0; iMinorFrame<wTotalMinorFrameCount; iMinorFrame++, wAscendingDataValue+=wWordCount)
		{
			sitalDecodedMessageStructure dmsDecodedMessage; // A decoded message.
			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded: unit - %i, purge - %i\n", s_swDevice3, TRUE);
			swResult = sitalMt_Message_GetFromStackDecoded (s_swDevice3, &dmsDecodedMessage, sitalMessageLocationAndRemoval_NEXT_PURGE, sitalMtStack_ACTIVE);
			if ((0 > swResult) || (1 < swResult))
			{
				sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Error: error - %i\n", swResult);
				throw 0;
			}
			if (0U == swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Message.Error: error - %s\n", "no_message");
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}
			sitalProcess_Log_PrintLine ("#StldTester.Mt.Message: unit - %i, command - %04X, data_count - %i, data - %i...%i, expected_data - %i..., time_tag - %04X\n", s_swDevice3, dmsDecodedMessage.wCommandWord1, dmsDecodedMessage.wDataWordCount, dmsDecodedMessage.waData[0], dmsDecodedMessage.waData[dmsDecodedMessage.wDataWordCount - 1], wAscendingDataValue, dmsDecodedMessage.wTimeTag);
			if ((wAscendingDataValue != dmsDecodedMessage.waData[0]) || ((wAscendingDataValue + wWordCount - 1) != dmsDecodedMessage.waData[dmsDecodedMessage.wDataWordCount - 1]))
			{
				sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Message.Error: error - %s\n", "bad_data");
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}
		}

		/// Stop device-3 (MT).
		sitalProcess_Log_PrintLine ("#StldTester.Mt.Stop: unit - %i\n", s_swDevice3);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Stop (s_swDevice3) : sitalMt_Stop (s_swDevice3) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-3 (MT).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice3);
		swResult = sitalDevice_Free (s_swDevice3);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		swResult = stldSyncEnd (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncEnd.Error: error - %i\n", swResult);
			throw 0;
		}
	}
	catch (int)
	{
		sitalDevice_Free (s_swDevice2);
		sitalDevice_Free (s_swDevice3);
		ipsSemaphore.Close ();
		if (NULL != s_ipspBcSemaphore)
		{
			s_ipspBcSemaphore->Close ();
			delete (s_ipspBcSemaphore);
			s_ipspBcSemaphore = NULL;
		}
	}

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_11l.End: test_succeeded - %c\n", ((sitalReturnCode_SUCCESS == swResult) ? 'Y' : 'N'));
	return swResult;
}



/// <summary>
/// Test the stld1553 library in the henceforth described scenario:
/// Participant[s]: RT1, BC2, MT3.
/// Configuration: All participating devices must be able to communicate with each other.
/// Short description: Acceptance Test Plan (ATP) test 461_11m - Insert aperiodic messages into a running BC list - Low priority asynchronous messages.
/// Along test print suitable info and indications into the log.
///
/// Note:
/// - This function may use any of above defined Acceptance Test Plan (ATP) tests variables.
///   The caller must therefore call the suitable stldTester_Atp_Set* functions prior to calling this function.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Atp_DoTest461_11m_Low ()
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_11m_Low.Start\n");

	/// Set test arguments.
	U16BIT wWordCount; // Word count (1-32).
	wWordCount = 32;
	U16BIT wRtSubaddressRx_1; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31).
	wRtSubaddressRx_1 = 1;
	U16BIT wRtSubaddressRx_2; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31).
	wRtSubaddressRx_2 = 2;
	U16BIT wFrameTime; // The time (in units of 100 us) that the frame will take to complete.
	wFrameTime = 100; // 10ms.
	U16BIT wFrameCount; // The number of frames to run.
	wFrameCount = 25;
	U16BIT wRegularFrameCount; // The number of frames to run regularly, i.e., before starting the transmission of asynchronous messages.
	wRegularFrameCount = 10;
	U16BIT wAsynchronousMessageCount; // The number of asynchronous messages.
	wAsynchronousMessageCount = 20;
	U16BIT wTimeTagCalculationInaccuracy; // The maximum tolerated difference between the expected time tag and the actual one.
	wTimeTagCalculationInaccuracy = 25;

	S16BIT swResult; // Result of operation or function call.
	swResult = sitalReturnCode_SUCCESS;

	/// Get device count.
	U16BIT wRequiredDeviceCount; // The number of devices required for this test.
	wRequiredDeviceCount = (1 + max(max(s_swDevice1, s_swDevice2), s_swDevice3));
	U16BIT wAvailableDeviceCount; // The total number of devices available in the system.
	swResult = sitalDevice_GetCount (&wAvailableDeviceCount);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %i\n", swResult);
		return swResult;
	}
	sitalProcess_Log_PrintLine ("#StldTester.Device.Count: available - %i, required - %i\n", wAvailableDeviceCount, wRequiredDeviceCount);
	if (wAvailableDeviceCount < wRequiredDeviceCount)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %s\n", "not_enough_devices_installed");
		return sitalReturnCode_SUCCESS;
	}
	if ( (s_swDevice1 == s_swDevice2) || (s_swDevice1 == s_swDevice3) || (s_swDevice2 == s_swDevice3))
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Number.Error: error - %s\n", "same_device_used_twice");
		return sitalReturnCode_INVALID_DEVICE_NUMBER;
	}

	try
	{
		swResult = stldSyncStart (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncStart.Error: error - %i\n", swResult);
			throw 0;
		}

		if (wFrameCount <= wRegularFrameCount)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Configuration.Error: error - %i\n", "bad_frame_counts");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		if (MAXIMUM_SLEEP_INACCURACY > ((wFrameCount - wRegularFrameCount) * (wFrameTime / 10)))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Configuration.Error: error - %i\n", "bad_timing");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// Initialize device-3 (MT) as a real MT.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access - card, mode - mt, alloc_mem - none\n", s_swDevice3);
		swResult = sitalDevice_Initialize (s_swDevice3, sitalAccess_CARD, ((s_bIsRtMtModeTest ? sitalMode_RT_AND_MT : sitalMode_MT) | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Set for device-3 (MT) a time tag resolution of 2 us.
		U16BIT wTimeTagResolution; // Time tag resolution (in units of 1 us).
		wTimeTagResolution = 2;
		sitalProcess_Log_PrintLine ("#StldTester.Device.TimeTagResolution.Set: unit - %i, resolution - %ius\n", s_swDevice3);
		swResult = sitalDevice_TimeTag_SetResolution (s_swDevice3, sitalTimeTagResolution_2US);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.TimeTagResolution.Set.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Start device-3 (MT).
		sitalProcess_Log_PrintLine ("#StldTester.Mt.Start: unit - %i\n", s_swDevice3);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Start (s_swDevice3) : sitalMt_Start (s_swDevice3) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-1 (RT) as a real RT in advanced mode.
		// Note:
		// - Advanced mode will enable r/w to the memory of device-1 (RT) while the device is running.
		// - An RT device that responds below created and transmitted BC asynchronous messages is required here, as each of these messages is transmitted again and again as long as possible until a condition of "good message" is fulfilled.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-rt, alloc_mem-none\n", s_swDevice1);
		swResult = sitalDevice_Initialize (s_swDevice1, sitalAccess_CARD, ((s_bIsRtMtModeTest ? sitalMode_RT_AND_MT : sitalMode_RT) | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Set device-1 (RT) address to RtAddress1.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set: unit - %i, address - %i\n", s_swDevice1, s_wRtAddress1);
		swResult = sitalRt_Address_Set (s_swDevice1, s_wRtAddress1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Legalize on device-1 (RT) all address-types in all directions for all subaddresses and all word counts.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.MessageLegality.Enable: unit - %i, address_type - %i, directions - %i, subaddress - %i, word_count - %08X\n", s_swDevice1, sitalRtAddressType_BOTH, sitalMessageDirection_BOTH, sitalRtSubaddress_ALL, 0xFFFFFFFF);
		swResult = sitalRt_MessageLegality_Enable (s_swDevice1, sitalRtAddressType_BOTH, sitalMessageDirection_BOTH, sitalRtSubaddress_ALL, 0xFFFFFFFF);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.MessageLegality.Enable.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Start device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Start: unit - %i\n", s_swDevice1);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Start (s_swDevice1) : sitalRt_Start (s_swDevice1) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-2 (BC) as a real BC.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-bc, alloc_mem-none\n", s_swDevice2);
		swResult = sitalDevice_Initialize (s_swDevice2, sitalAccess_CARD, (sitalMode_BC | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-2 (BC) as one that supports low priority asynchronous messages.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Init: unit - %i, options - %i\n", s_swDevice2, sitalBcSetupOption_ASYNCHRONOUS_LOW_PRIORITY_MODE);
		swResult = sitalBc_Initialize (s_swDevice2, sitalBcSetupOption_ASYNCHRONOUS_LOW_PRIORITY_MODE);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) data-block-0, a data block of 64 data words, and fill the first WordCount data words with zeros.
		S16BIT swDataBlockId_0; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId_0 = 1;
		U16BIT waBuffer[32]; // A buffer to read-from/write-into along data-block write/read operations, respectively.
		U16BIT wAscendingDataValue; // An ascending data value.
		wAscendingDataValue = 0;
		for (int i=0; i<wWordCount; i++) waBuffer[i] = wAscendingDataValue;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i, buf_contents - %i...%i\n", s_swDevice2, swDataBlockId_0, 64, wWordCount, waBuffer[0], waBuffer[0]);
		swResult = sitalBc_DataBlock_Create (s_swDevice2, swDataBlockId_0, sitalBcDataBlockSizeOption_DOUBLE, waBuffer, wWordCount);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) message-0, a BC-to-RT(RT-address-1/RtSubAddressRx-1 message that uses data-block-0 to transmit WordCount data words. Message bus: bus-1.
		S16BIT swMessageId_0; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create: unit - %i, message - %i, block - %i, rx_rt - %i, rx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddressRx_1, wWordCount, 0, s_wBus1);
		swResult = sitalBc_Message_CreateBcToRt (s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddressRx_1, wWordCount, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-0, a XEQ command that executes message-0.
		S16BIT swCommandId_0; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-1, a subroutine call command that executes minor-frame-0.
		S16BIT swCommandId_1; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_1 = 1;
		S16BIT swFrameId_0; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_1, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, swFrameId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_1, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, (U16BIT)swFrameId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) minor-frame-0, a frame that contains command-0.
		S16BIT swaCommandIds[sitalBcCounter_COMMANDS]; // An array of command-IDs.
		swaCommandIds[0] = swCommandId_0;
		U16BIT wCommandIdCount; // The number of command IDs in the frame (1-sitalBcCounter_FRAME_COMMANDS).
		wCommandIdCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_0, "minor", wCommandIdCount, swCommandId_0, 0, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_0, sitalBcFrameType_MINOR, swaCommandIds, wCommandIdCount, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) major-frame-1, a frame that contains command-1 and whose time is FrameTime.
		S16BIT swFrameId_1; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_1 = 1;
		swaCommandIds[0] = swCommandId_1;
		wCommandIdCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_1, "major", wCommandIdCount, swCommandId_1, wFrameTime, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_1, sitalBcFrameType_MAJOR, swaCommandIds, wCommandIdCount, wFrameTime, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) message-1 to message-AsynchronousMessageCount, asynchronous BC-to-RT(RtAddress1/RtSubAddressRx-2) messages that each of them uses a message-unique data-block that is filled with a number of WordCount data-block-unique data words.
		S16BIT swMessageId; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		S16BIT swDataBlockId; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		for (swMessageId=1, swDataBlockId=2; swMessageId<=wAsynchronousMessageCount; swMessageId++, swDataBlockId++)
		{
			wAscendingDataValue = swMessageId;
			for (int i=0; i<wWordCount; i++) waBuffer[i] = wAscendingDataValue;
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i, buf_contents - %i...%i\n", s_swDevice2, swDataBlockId, 64, wWordCount, waBuffer[0], waBuffer[0]);
			sitalProcess_Log_PrintLine ("#StldTester.Bc.AsynchronousMessage.BcToRt.Create: unit - %i, message - %i, block - %i, rx_rt - %i, rx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId, swDataBlockId, s_wRtAddress1, wRtSubaddressRx_2, wWordCount, 0, s_wBus1);
			swResult = sitalBc_AsynchronousMessage_CreateBcToRt (s_swDevice2, swMessageId, swDataBlockId, s_wRtAddress1, wRtSubaddressRx_2, wWordCount, 0, s_wBus1, waBuffer);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.AsynchronousMessage.BcToRt.Create.Error: error - %i\n", swResult);
				throw 0;
			}
		}

		///	Start device-2 (BC) running major-frame-1 for FrameCount-shot.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", s_swDevice2, swFrameId_1, wFrameCount);
		swResult = sitalBc_Start (s_swDevice2, swFrameId_1, wFrameCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Wait for the regular frame run to end.
		U32BIT dwSleepTime; // Sleeping period (in ms).
		dwSleepTime = (2/*sleep for 2 frames*/ *  ((U16BIT)(wFrameTime / IP_FREQUENCY_FACTOR) / 10));
		sitalProcess_Log_PrintLine ("#StldTester.Wait: time - %ims\n", dwSleepTime);
		sitalProcess_Log_PrintLine ("\tWAIT_FOR %i us\n", (dwSleepTime * 1000));
		PrecisionTimer pt; // A precision timer.
		pt.AccuratelySleep (dwSleepTime);

		/// Request device-2 (BC) to transmit the asynchronous messages at low priority.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.AsynchronousMessage.Tx.Low: unit - %i\n", s_swDevice2);
		U16BIT wRemainingMessageCount; // The count of asynchronous messages that still weren't assigned for Tx.
		wRemainingMessageCount = 0;
		swResult = sitalBc_AsynchronousMessage_SendAtLowPriority (s_swDevice2, &wRemainingMessageCount, 0);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.AsynchronousMessage.Tx.Low.Error: error - %i\n", swResult);
			throw 0;
		}
		sitalProcess_Log_PrintLine ("#StldTester.Bc.AsynchronousMessage.Tx.Low: unit - %i, tx_messages_count - %i, expected_tx_messages_count - %i\n", s_swDevice2, swResult, wAsynchronousMessageCount);
		if (wAsynchronousMessageCount != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.AsynchronousMessage.Tx.Low.Error: error - %i\n", "bad_tx_message_count");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// Wait for the regular frame run to end.
		//U32BIT dwSleepTime; // Sleeping period (in ms).
		dwSleepTime = ((wRegularFrameCount - 2) *  ((U16BIT)(wFrameTime / IP_FREQUENCY_FACTOR) / 10));
		sitalProcess_Log_PrintLine("#StldTester.Wait: time - %ims\n", dwSleepTime);
		sitalProcess_Log_PrintLine("\tWAIT_FOR %i us\n", (dwSleepTime * 1000));
		//PrecisionTimer pt; // A precision timer.
		pt.AccuratelySleep(dwSleepTime);

		/// Get for device-2 (BC) the current condition state for GPF7-set.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.ConditionState.Get: unit - %i, condition - %i\n", s_swDevice2, sitalOpcodeCondition_GP7_1);
		U16BIT wCurrentState; // The current state, idle (sitalBcActivationState_IDLE) or busy (sitalBcActivationState_BUSY).
		swResult = sitalBc_GetConditionState (s_swDevice2, sitalOpcodeCondition_GP7_1, &wCurrentState);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.ConditionState.Get.Error: error - %i\n", swResult);
			printf("\tframe should have been running, since its 5MHz and non RTOS, the delay may be too long and test fails");
			throw 0;
		}
		sitalProcess_Log_PrintLine ("#StldTester.Bc.ConditionState.Get: unit - %i, condition - %i, state - %i\n", s_swDevice2, sitalOpcodeCondition_GP7_1, wCurrentState);
		/// Verify for device-2 (BC) the correctness of the current condition state for GPF7-set.
		if (sitalBcActivationState_BUSY != wCurrentState)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.ConditionState.Get.Error: error - %s\n", "unexpected_state");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// Wait for the irregular frame run to end.
		dwSleepTime = ((wFrameCount - wRegularFrameCount) *  ((U16BIT)(wFrameTime / IP_FREQUENCY_FACTOR) / 10));
		sitalProcess_Log_PrintLine ("#StldTester.Wait: time - %ims\n", dwSleepTime);
		sitalProcess_Log_PrintLine ("\tWAIT_FOR %i us\n", (dwSleepTime * 1000));
		pt.AccuratelySleep (dwSleepTime);
		
		///	Stop device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop: unit - %i\n", s_swDevice2);
		swResult = sitalBc_Stop (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		///	Verify that:
		/// - Device-3 (MT) has detected all the regular and all the asynchronous messages.
		/// - Each regular message was transmitted a time of FrameTime after the previous regular message.
		///	- The asynchronous messages were transmitted only after the regular frames.
		///	- The asynchronous messages were transmitted at the correct order (see note below).
		/// - Each asynchronous message was transmitted exactly once.
		/// - Each asynchronous message was transmitted a suitable time after the preceding [regular/asynchronous] message (see note below).
		// Note:
		// - The order of transmission of asynchronous messages at low priority may principally vary, as a shorter later message may be transmitted prior to a former longer one.
		// - Yet, in this test, all asynchronous messages are identical (they merely differ in their data contents), so they should be transmitted in the expected order.
		// - After transmitting the regular message[s] of a minor frame, the device jumps to the asynchronous message routine.
		//   Within this routine, it has to skip the asynchronous messages that have already been transmitted after previous minor frames, and this skipping takes some additional time.
		//   As already said, in our case the asynchronous messages are transmitted in order, to that such a skip takes place only right after the end of a minor frame, right before the transmission of the first asynchronous message.
		//   Therefore, the time tag of the first asynchronous message that follows a minor frame may be greater than the normally expected.
		sitalDecodedMessageStructure dmsDecodedMessage; // A decoded message.
		wAscendingDataValue = 1;
		U16BIT wLeadingRegularMessagesCount; // The number of regular messages detected before the first asynchronous message.
		wLeadingRegularMessagesCount = 0;
		BOOLEAN bIsAnyPreviousRegularMessage; // A flag that says whether the first regular message has been received already.
		bIsAnyPreviousRegularMessage = false;
		BOOLEAN bIsPreviousMessageRegularOne; // A flag that says whether the previously detected message is a regular one.
		bIsPreviousMessageRegularOne = false;
		U16BIT wTimeTagOfPreviousRegularMessage; // The time stamp of the previously received regular message.
		wTimeTagOfPreviousRegularMessage = 0;
		U16BIT wTimeTagOfPreviousMessage; // The time stamp of the previously received message.
		wTimeTagOfPreviousMessage = 0;
		U16BIT wTimeTagDifference; // The difference between the expected time tag and the actual one.
		wTimeTagDifference = 0;
		U16BIT wExpectedTimeTag; // The next expected time tag.
		wExpectedTimeTag = 0;
		U16BIT wOverallMessageTxCount; // The overall number of transmitted messages (includes both regular and asynchronous transmitted messages).
		wOverallMessageTxCount = (wFrameCount + wAsynchronousMessageCount);
		U16BIT wTimeOfAsynchronousMessage; // The estimated time required for the transmission of each of the asynchronous messages (in units of 1 us).
		wTimeOfAsynchronousMessage = 700;
		for (U16BIT iTransmittedMessage=0; iTransmittedMessage<wOverallMessageTxCount; iTransmittedMessage++)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded: unit - %i, purge - %i\n", s_swDevice3, TRUE);
			swResult = sitalMt_Message_GetFromStackDecoded (s_swDevice3, &dmsDecodedMessage, sitalMessageLocationAndRemoval_NEXT_PURGE, sitalMtStack_ACTIVE);
			if ((0 > swResult) || (1 < swResult))
			{
				sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Error: error - %i\n", swResult);
				throw 0;
			}
			if (0U == swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Message.Error: error - %s\n", "no_message");
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}
			sitalProcess_Log_PrintLine ("#StldTester.Mt.Message: unit - %i, command - %04X, data_count - %i, data - %i..., time_tag - %04X\n", s_swDevice3, dmsDecodedMessage.wCommandWord1, dmsDecodedMessage.wDataWordCount, dmsDecodedMessage.waData[0], dmsDecodedMessage.wTimeTag);
			if (0U == dmsDecodedMessage.waData[0]) // Regular message
			{
				if (bIsAnyPreviousRegularMessage)
				{
					wExpectedTimeTag = ( wTimeTagOfPreviousRegularMessage + (wFrameTime * 100 / wTimeTagResolution) ); // Frame time is expressed in units of 100us.
					wTimeTagDifference = (dmsDecodedMessage.wTimeTag - wExpectedTimeTag);
					wTimeTagDifference = ( (0x8000 < wTimeTagDifference) ? (0x10000 - wTimeTagDifference) : wTimeTagDifference); // To handle a case of time tag rollover.
					sitalProcess_Log_PrintLine ("#StldTester.Time.Difference: unit - %i, permitted - %04X, actual - %04X\n", s_swDevice3, wTimeTagCalculationInaccuracy, wTimeTagDifference);
					if (wTimeTagCalculationInaccuracy < wTimeTagDifference)
					{
						sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Message.Error: error - %s\n", "bad_framing");
						swResult = sitalReturnCode_TEST_FAIL;
						throw 0;
					}
				}

				wTimeTagOfPreviousRegularMessage = dmsDecodedMessage.wTimeTag;
				bIsAnyPreviousRegularMessage = true;
				bIsPreviousMessageRegularOne = true;
				wLeadingRegularMessagesCount++;
			}
			else
			if ((wAscendingDataValue == dmsDecodedMessage.waData[0]) && (wAscendingDataValue <= wAsynchronousMessageCount)) // Expected asynchronous message
			{
				if (2 > wLeadingRegularMessagesCount)
				{
					sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Message.Error: error - %s\n", "bad_order");
					swResult = sitalReturnCode_TEST_FAIL;
					throw 0;
				}

				if ( ! bIsPreviousMessageRegularOne )
				{
					wExpectedTimeTag = ( wTimeTagOfPreviousMessage + (wTimeOfAsynchronousMessage / wTimeTagResolution) ); // Message time is expressed in units of 1us.
					wTimeTagDifference = (dmsDecodedMessage.wTimeTag - wExpectedTimeTag);
					wTimeTagDifference = ( (0x8000 < wTimeTagDifference) ? (0x10000 - wTimeTagDifference) : wTimeTagDifference); // To handle a case of time tag rollover.
					if (wTimeTagCalculationInaccuracy < wTimeTagDifference)
					{
						sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Message.Error: error - %s\n", "bad_timing");
						swResult = sitalReturnCode_TEST_FAIL;
						throw 0;
					}
				}

				bIsPreviousMessageRegularOne = false;
				wAscendingDataValue++;
			}
			else
			{
				sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Message.Error: error - %s\n", "bad_data");
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}

			wTimeTagOfPreviousMessage = dmsDecodedMessage.wTimeTag;
		}

		///	Verify that no additional messages have been transmitted.
		sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded: unit - %i, purge - %i\n", s_swDevice3, TRUE);
		swResult = sitalMt_Message_GetFromStackDecoded (s_swDevice3, &dmsDecodedMessage, sitalMessageLocationAndRemoval_NEXT_PURGE, sitalMtStack_ACTIVE);
		if ((0 > swResult) || (1 < swResult))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Error: error - %i\n", swResult);
			throw 0;
		}
		if (1 == swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Error: error - %s\n", "extra_message");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// Stop device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Stop: unit - %i\n", s_swDevice1);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Stop (s_swDevice1) : sitalRt_Stop (s_swDevice1) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Stop device-3 (MT).
		sitalProcess_Log_PrintLine ("#StldTester.Mt.Stop: unit - %i\n", s_swDevice3);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Stop (s_swDevice3) : sitalMt_Stop (s_swDevice3) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice1);
		swResult = sitalDevice_Free (s_swDevice1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice2);
		swResult = sitalDevice_Free (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-3 (MT).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice3);
		swResult = sitalDevice_Free (s_swDevice3);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		swResult = stldSyncEnd (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncEnd.Error: error - %i\n", swResult);
			throw 0;
		}
	}
	catch (int)
	{
		sitalDevice_Free (s_swDevice1);
		sitalDevice_Free (s_swDevice2);
		sitalDevice_Free (s_swDevice3);
	}

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_11m_Low.End: test_succeeded - %c\n", ((sitalReturnCode_SUCCESS == swResult) ? 'Y' : 'N'));
	return swResult;
}



/// <summary>
/// Test the stld1553 library in the henceforth described scenario:
/// Participant[s]: RT1, BC2, MT3, RT4.
/// Configuration: All participating devices must be able to communicate with each other.
/// Short description: Acceptance Test Plan (ATP) test 461_11m - Insert aperiodic messages into a running BC list - High priority asynchronous messages.
/// Along test print suitable info and indications into the log.
///
/// Note:
/// - This function may use any of above defined Acceptance Test Plan (ATP) tests variables.
///   The caller must therefore call the suitable stldTester_Atp_Set* functions prior to calling this function.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Atp_DoTest461_11m_High ()
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_11m_High.Start\n");

	/// Set test arguments.
	U16BIT wWordCount; // Word count (1-32).
	wWordCount = 32;
	U16BIT wRtSubaddressRx_1; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31).
	wRtSubaddressRx_1 = 1;
	U16BIT wRtSubaddressRx_2; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31).
	wRtSubaddressRx_2 = 2;
	U16BIT wMinorFrameCommandIdCount; // The number of command IDs in the frame (1-sitalBcCounter_FRAME_COMMANDS).
	wMinorFrameCommandIdCount = 35; // The number of frame messages that's required in order to force the transmission of a high-priority asynchronous message, whose transmission is requested right after the frame starts running, among the messages of the first frame, even before the first frame enters its passive phase.
	U16BIT wFrameCount; // The number of frames to run.
	wFrameCount = 7; // Must be > 1. Note: The MT command stack must be big enough to contain 4-word-entries for each of the messages that are transmitted in this test, and whose overall number is ( (wFrameCount * wMinorFrameCommandIdCount) +  ).
	U16BIT wRegularFrameCount; // The number of frames to run regularly, i.e., before starting the transmission of asynchronous messages.
	wRegularFrameCount = 2; // Must be smaller than wFrameCount.
	U16BIT wTimeTagCalculationInaccuracy; // The maximum tolerated difference between the expected time tag and the actual one.
	wTimeTagCalculationInaccuracy = 25;

	S16BIT swResult; // Result of operation or function call.
	swResult = sitalReturnCode_SUCCESS;

	/// Get device count.
	U16BIT wRequiredDeviceCount; // The number of devices required for this test.
	wRequiredDeviceCount = (1 + max(max(max(s_swDevice1, s_swDevice2), s_swDevice3), s_swDevice4));
	U16BIT wAvailableDeviceCount; // The total number of devices available in the system.
	swResult = sitalDevice_GetCount (&wAvailableDeviceCount);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %i\n", swResult);
		return swResult;
	}
	sitalProcess_Log_PrintLine ("#StldTester.Device.Count: available - %i, required - %i\n", wAvailableDeviceCount, wRequiredDeviceCount);
	if (wAvailableDeviceCount < wRequiredDeviceCount)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %s\n", "not_enough_devices_installed");
		return sitalReturnCode_SUCCESS;
	}
	if ( (s_swDevice1 == s_swDevice2) || (s_swDevice1 == s_swDevice3) || (s_swDevice1 == s_swDevice4) || (s_swDevice2 == s_swDevice3) || (s_swDevice2 == s_swDevice4) || (s_swDevice3 == s_swDevice4))
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Number.Error: error - %s\n", "same_device_used_twice");
		return sitalReturnCode_INVALID_DEVICE_NUMBER;
	}

	/// Get device-3 (MT) information.
	sitalDeviceCapabilitiesStructure dcsDeviceCapabilities;
	dcsDeviceCapabilities.dwStructureSize = sizeof(sitalDeviceCapabilitiesStructure);
	swResult = sitalDevice_GetInformation (s_swDevice3, &dcsDeviceCapabilities);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.GetInformation.Error: error - %i\n", swResult);
		throw 0;
	}
	sitalProcess_Log_PrintLine ("#StldTester.Device.Information: unit - %i, ram_size - %X\n", s_swDevice3, dcsDeviceCapabilities.dwRamSize);
	if (0x1000 /* 4K */ >= dcsDeviceCapabilities.dwRamSize)
	{
		wFrameCount = 3; // Must be > 1. Note: The MT command stack must be big enough to contain 4-word-entries for each of the messages that are transmitted in this test, and whose overall number is ( (wFrameCount * wMinorFrameCommandIdCount) +  ).
		wRegularFrameCount = 1; // Must be smaller than wFrameCount.
	}

	try
	{
		swResult = stldSyncStart (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncStart.Error: error - %i\n", swResult);
			throw 0;
		}

		if ((2 > wFrameCount) || (wFrameCount <= wRegularFrameCount))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Configuration.Error: error - %i\n", "bad_frame_counts");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		if (wRtSubaddressRx_1 == wRtSubaddressRx_2)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Configuration.Error: error - %i\n", "bad_rt_subaddresses");
			swResult = sitalReturnCode_TEST_FAIL - 1;
			throw 0;
		}

		U16BIT wFrameTime; // The time (in units of 100 us) that the frame will take to complete.
		wFrameTime = 10000; // 1000ms -- must be long enough to have a passive phase long enough to enable repeated transmission of high-priority asynchronous messages even if a non-real-time operating-system is used.
		if (MAXIMUM_SLEEP_INACCURACY > ((wFrameCount - wRegularFrameCount) * (wFrameTime / 10)))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Configuration.Error: error - %i\n", "bad_timing");
			swResult = sitalReturnCode_TEST_FAIL - 2;
			throw 0;
		}

		/// Initialize device-3 (MT) as a real MT.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access - card, mode - mt, alloc_mem - none\n", s_swDevice3);
		swResult = sitalDevice_Initialize (s_swDevice3, sitalAccess_CARD, ((s_bIsRtMtModeTest ? sitalMode_RT_AND_MT : sitalMode_MT) | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Set for device-3 (MT) a time tag resolution of 2 us.
		U16BIT wTimeTagResolution; // Time tag resolution (in units of 1 us).
		wTimeTagResolution = 2;
		sitalProcess_Log_PrintLine ("#StldTester.Device.TimeTagResolution.Set: unit - %i, resolution - %ius\n", s_swDevice3);
		swResult = sitalDevice_TimeTag_SetResolution (s_swDevice3, sitalTimeTagResolution_2US);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.TimeTagResolution.Set.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Start device-3 (MT).
		sitalProcess_Log_PrintLine ("#StldTester.Mt.Start: unit - %i\n", s_swDevice3);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Start (s_swDevice3) : sitalMt_Start (s_swDevice3) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-1 (RT) as a real RT in advanced mode.
		// Note:
		// - Advanced mode will enable r/w to the memory of device-1 (RT) while the device is running.
		// - An RT device that responds below created and transmitted BC asynchronous messages is required here, as each of these messages is transmitted again and again as long as possible until a condition of "good message" is fulfilled.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-rt, alloc_mem-none\n", s_swDevice1);
		swResult = sitalDevice_Initialize (s_swDevice1, sitalAccess_CARD, ((s_bIsRtMtModeTest ? sitalMode_RT_AND_MT : sitalMode_RT) | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Set device-1 (RT) address to RtAddress1.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set: unit - %i, address - %i\n", s_swDevice1, s_wRtAddress1);
		swResult = sitalRt_Address_Set (s_swDevice1, s_wRtAddress1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Legalize on device-1 (RT) all address-types in all directions for all subaddresses and all word counts.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.MessageLegality.Enable: unit - %i, address_type - %i, directions - %i, subaddress - %i, word_count - %08X\n", s_swDevice1, sitalRtAddressType_BOTH, sitalMessageDirection_BOTH, sitalRtSubaddress_ALL, 0xFFFFFFFF);
		swResult = sitalRt_MessageLegality_Enable (s_swDevice1, sitalRtAddressType_BOTH, sitalMessageDirection_BOTH, sitalRtSubaddress_ALL, 0xFFFFFFFF);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.MessageLegality.Enable.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Start device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Start: unit - %i\n", s_swDevice1);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Start (s_swDevice1) : sitalRt_Start (s_swDevice1) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-4 (RT) as a real RT in advanced mode.
		// Note:
		// - Advanced mode will enable r/w to the memory of device-1 (RT) while the device is running.
		// - An RT device that responds below created and transmitted BC asynchronous messages is required here, as each of these messages is transmitted again and again as long as possible until a condition of "good message" is fulfilled.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-rt, alloc_mem-none\n", s_swDevice4);
		swResult = sitalDevice_Initialize (s_swDevice4, sitalAccess_CARD, ((s_bIsRtMtModeTest ? sitalMode_RT_AND_MT : sitalMode_RT) | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Set device-4 (RT) address to RtAddress2.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set: unit - %i, address - %i\n", s_swDevice4, s_wRtAddress2);
		swResult = sitalRt_Address_Set (s_swDevice4, s_wRtAddress2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Legalize on device-4 (RT) all address-types in all directions for all subaddresses and all word counts.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.MessageLegality.Enable: unit - %i, address_type - %i, directions - %i, subaddress - %i, word_count - %08X\n", s_swDevice4, sitalRtAddressType_BOTH, sitalMessageDirection_BOTH, sitalRtSubaddress_ALL, 0xFFFFFFFF);
		swResult = sitalRt_MessageLegality_Enable (s_swDevice4, sitalRtAddressType_BOTH, sitalMessageDirection_BOTH, sitalRtSubaddress_ALL, 0xFFFFFFFF);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.MessageLegality.Enable.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Start device-4 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Start: unit - %i\n", s_swDevice4);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Start (s_swDevice4) : sitalRt_Start (s_swDevice4) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-2 (BC) as a real BC.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-bc, alloc_mem-none\n", s_swDevice2);
		swResult = sitalDevice_Initialize (s_swDevice2, sitalAccess_CARD, (sitalMode_BC | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-2 (BC) as one that supports high priority asynchronous messages.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Init: unit - %i, options - %i\n", s_swDevice2, sitalBcSetupOption_ASYNCHRONOUS_HIGH_PRIORITY_MODE);
		swResult = sitalBc_Initialize (s_swDevice2, sitalBcSetupOption_ASYNCHRONOUS_HIGH_PRIORITY_MODE);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) data-block-0, a data block of 64 data words, and fill the first WordCount data words with zeros.
		S16BIT swDataBlockId_0; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId_0 = 1;
		U16BIT waBuffer[32]; // A buffer to read-from/write-into along data-block write/read operations, respectively.
		U16BIT wAscendingDataValue; // An ascending data value.
		wAscendingDataValue = 0;
		for (int i=0; i<wWordCount; i++) waBuffer[i] = wAscendingDataValue;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i, buf_contents - %i...%i\n", s_swDevice2, swDataBlockId_0, 64, wWordCount, waBuffer[0], waBuffer[0]);
		swResult = sitalBc_DataBlock_Create (s_swDevice2, swDataBlockId_0, sitalBcDataBlockSizeOption_DOUBLE, waBuffer, wWordCount);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) message-0, a BC-to-RT(RT-address-1/RtSubAddressRx-1 message that uses data-block-0 to transmit WordCount data words. Message bus: bus-1.
		S16BIT swMessageId_0; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create: unit - %i, message - %i, block - %i, rx_rt - %i, rx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddressRx_1, wWordCount, 0, s_wBus1);
		swResult = sitalBc_Message_CreateBcToRt (s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddressRx_1, wWordCount, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Use given message descriptors to code the command word[s] of below defined message.
		// Note:
		// - Defined messages have been intentionally devised in a way that their [first] command word will differ.
		// - Here are their definitions:
		//		0-(R )BC    >RT1/1
		//		1-(R )BC    >RT1/2
		//		2-(R )RT1/1 >BC
		//		3-(R )RT1/2 >RT2/1
		//		4-(M2)RT1/1 >BC
		//		5-(R )BC    >BCST/1
		//		6-(R )RT2/1 >BCST/2
		//		7-(M2)BCST/0>BC
		U16BIT wCommandWord; // Command word.
		swResult = sitalBcStld1553_Command_GetCode (&wCommandWord, s_wRtAddress1, sitalCommand_RX, wRtSubaddressRx_1, wWordCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Command.GetCode.Error: error - %i\n", swResult);
			throw 0;
		}
#define NUMBER_OF_MESSAGE_TYPES					7
		U32BIT dwaFirstCommandWord[NUMBER_OF_MESSAGE_TYPES + 1]; // First command word for all defined messages (entry 0 for the single regular message, entries 1-7 for the asynchronous messages of all the 7 possible types).
		dwaFirstCommandWord[0] = wCommandWord;

		/// Create for device-2 (BC) command-0, a XEQ command that executes message-0.
		S16BIT swCommandId_0; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-1, a subroutine call command that executes minor-frame-0.
		S16BIT swCommandId_1; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_1 = 1;
		S16BIT swFrameId_0; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_1, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, swFrameId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_1, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, (U16BIT)swFrameId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) minor-frame-0, a frame that contains 5 occurrences of command-0.
		S16BIT swaCommandIds[sitalBcCounter_COMMANDS]; // An array of command-IDs.
		for (U16BIT i=0; i<wMinorFrameCommandIdCount; i++)
		{
			swaCommandIds[i] = swCommandId_0;
		}
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i...), time - %ims, flags - %i\n", s_swDevice2, swFrameId_0, "minor", wMinorFrameCommandIdCount, swCommandId_0, 0, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_0, sitalBcFrameType_MINOR, swaCommandIds, wMinorFrameCommandIdCount, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) major-frame-1, a frame that contains command-1 and whose time is FrameTime.
		S16BIT swFrameId_1; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_1 = 1;
		swaCommandIds[0] = swCommandId_1;
		U16BIT wCommandIdCount; // The number of command IDs in the frame (1-sitalBcCounter_FRAME_COMMANDS).
		wCommandIdCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_1, "major", wCommandIdCount, swCommandId_1, wFrameTime, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_1, sitalBcFrameType_MAJOR, swaCommandIds, wCommandIdCount, wFrameTime, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) message-1, asynchronous BC-to-RT(RT-address-1/RtSubAddressRx-2) message that uses data-block-1 to transmit WordCount unique data words. Message bus: bus-1.
		S16BIT swDataBlockId_1; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId_1 = 2;
		wAscendingDataValue++;
		for (int i=0; i<wWordCount; i++) waBuffer[i] = wAscendingDataValue;
		S16BIT swMessageId_1; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_1 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i, buf_contents - %i...%i\n", s_swDevice2, swDataBlockId_1, 64, wWordCount, waBuffer[0], waBuffer[0]);
		sitalProcess_Log_PrintLine ("#StldTester.Bc.AsynchronousMessage.BcToRt.Create: unit - %i, message - %i, block - %i, rx_rt - %i, rx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_1, swDataBlockId_1, s_wRtAddress1, wRtSubaddressRx_2, wWordCount, 0, s_wBus1);
		swResult = sitalBc_AsynchronousMessage_CreateBcToRt (s_swDevice2, swMessageId_1, swDataBlockId_1, s_wRtAddress1, wRtSubaddressRx_2, wWordCount, 0, s_wBus1, waBuffer);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Use given message descriptors to code the command word[s] of above defined message.
		// Note: Defined messages have been intentionally devised in a way that their [first] command word will differ.
		swResult = sitalBcStld1553_Command_GetCode (&wCommandWord, s_wRtAddress1, sitalCommand_RX, wRtSubaddressRx_2, wWordCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Command.GetCode.Error: error - %i\n", swResult);
			throw 0;
		}
		dwaFirstCommandWord[1] = wCommandWord;

		/// Create for device-2 (BC) message-2, asynchronous RT(RT-address-1/RtSubAddressRx-1)-to-Bc message that uses data-block-2 to transmit WordCount unique data words. Message bus: bus-1.
		S16BIT swDataBlockId_2; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId_2 = 3;
		wAscendingDataValue++;
		for (int i=0; i<wWordCount; i++) waBuffer[i] = wAscendingDataValue;
		S16BIT swMessageId_2; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_2 = 2;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i, buf_contents - %i...%i\n", s_swDevice2, swDataBlockId_2, 64, wWordCount, waBuffer[0], waBuffer[0]);
		sitalProcess_Log_PrintLine ("#StldTester.Bc.AsynchronousMessage.RtToBc.Create: unit - %i, message - %i, block - %i, tx_rt - %i, tx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_2, swDataBlockId_2, s_wRtAddress1, wRtSubaddressRx_1, wWordCount, 0, s_wBus1);
		swResult = sitalBc_AsynchronousMessage_CreateRtToBc (s_swDevice2, swMessageId_2, swDataBlockId_2, s_wRtAddress1, wRtSubaddressRx_1, wWordCount, 0, s_wBus1, waBuffer);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBc.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Use given message descriptors to code the command word[s] of above defined message.
		// Note: Defined messages have been intentionally devised in a way that their [first] command word will differ.
		swResult = sitalBcStld1553_Command_GetCode (&wCommandWord, s_wRtAddress1, sitalCommand_TX, wRtSubaddressRx_1, wWordCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Command.GetCode.Error: error - %i\n", swResult);
			throw 0;
		}
		dwaFirstCommandWord[2] = wCommandWord;

		/// Create for device-2 (BC) message-3, asynchronous RT(RT-address-1/RtSubAddressRx-2)-to-RT(RT-address-2/RtSubAddressRx-1) message that uses data-block-3 to transmit WordCount unique data words. Message bus: bus-1.
		S16BIT swDataBlockId_3; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId_3 = 4;
		wAscendingDataValue++;
		for (int i=0; i<wWordCount; i++) waBuffer[i] = wAscendingDataValue;
		S16BIT swMessageId_3; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_3 = 3;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i, buf_contents - %i...%i\n", s_swDevice2, swDataBlockId_3, 64, wWordCount, waBuffer[0], waBuffer[0]);
		sitalProcess_Log_PrintLine ("#StldTester.Bc.AsynchronousMessage.RtToRt.Create: unit - %i, message - %i, block - %i, tx_rt - %i, tx_rt_sa - %i, rx_rt - %i, rx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_3, swDataBlockId_3, s_wRtAddress1, wRtSubaddressRx_2, s_wRtAddress2, wRtSubaddressRx_1, wWordCount, 0, s_wBus1);
		swResult = sitalBc_AsynchronousMessage_CreateRtToRt (s_swDevice2, swMessageId_3, swDataBlockId_3, s_wRtAddress2, wRtSubaddressRx_1, wWordCount, s_wRtAddress1, wRtSubaddressRx_2, 0, s_wBus1, waBuffer);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToRt.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Use given message descriptors to code the command word[s] of above defined message.
		// Note: Defined messages have been intentionally devised in a way that their [first] command word will differ.
		swResult = sitalBcStld1553_Command_GetCode (&wCommandWord, s_wRtAddress2, sitalCommand_RX, wRtSubaddressRx_1, wWordCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Command.GetCode.Error: error - %i\n", swResult);
			throw 0;
		}
		dwaFirstCommandWord[3] = wCommandWord;

		/// Create for device-2 (BC) message-4, asynchronous RT(RT-address-1/RtSubAddressRx-1)-to-Bc-Mode(code=2) message. Message bus: bus-1.
		S16BIT swDataBlockId_4; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId_4 = 5;
		wAscendingDataValue++;
		for (int i=0; i<wWordCount; i++) waBuffer[i] = wAscendingDataValue;
		S16BIT swMessageId_4; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_4 = 4;
		U16BIT wModeCommand; // Mode command (1-32).
		wModeCommand = 0x0002;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i, buf_contents - %i...%i\n", s_swDevice2, swDataBlockId_4, 64, wWordCount, waBuffer[0], waBuffer[0]);
		sitalProcess_Log_PrintLine ("#StldTester.Bc.AsynchronousMessage.RtToBcMode.Create: unit - %i, message - %i, block - %i, rt - %i, , direction - %i, mode - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_4, swDataBlockId_4, s_wRtAddress1, sitalMessageDirection_TX, wModeCommand, 0, s_wBus1);
		swResult = sitalBc_AsynchronousMessage_CreateBcToOrFromRtMode (s_swDevice2, swMessageId_4, swDataBlockId_4, s_wRtAddress1, sitalMessageDirection_TX, wModeCommand, 0, s_wBus1, waBuffer);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBcMode.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Use given message descriptors to code the command word[s] of above defined message.
		// Note: Defined messages have been intentionally devised in a way that their [first] command word will differ.
		swResult = sitalBcStld1553_Command_GetCode (&wCommandWord, s_wRtAddress1, sitalCommand_TX, sitalRtSubaddress_0, wModeCommand);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Command.GetCode.Error: error - %i\n", swResult);
			throw 0;
		}
		dwaFirstCommandWord[4] = wCommandWord;

		/// Create for device-2 (BC) message-5, asynchronous BC-to-Broadcast(RtSubAddressRx-1) message that uses data-block-5 to transmit WordCount unique data words. Message bus: bus-1.
		S16BIT swDataBlockId_5; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId_5 = 6;
		wAscendingDataValue++;
		for (int i=0; i<wWordCount; i++) waBuffer[i] = wAscendingDataValue;
		S16BIT swMessageId_5; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_5 = 5;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i, buf_contents - %i...%i\n", s_swDevice2, swDataBlockId_5, 64, wWordCount, waBuffer[0], waBuffer[0]);
		sitalProcess_Log_PrintLine ("#StldTester.Bc.AsynchronousMessage.BcToBroadcast.Create: unit - %i, message - %i, block - %i, rx_rt - %i, rx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_1, swDataBlockId_1, s_wRtAddress1, wRtSubaddressRx_2, wWordCount, 0, s_wBus1);
		swResult = sitalBc_AsynchronousMessage_CreateBcToBroadcast (s_swDevice2, swMessageId_5, swDataBlockId_5, wRtSubaddressRx_1, wWordCount, 0, s_wBus1, waBuffer);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToBroadcast.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Use given message descriptors to code the command word[s] of above defined message.
		// Note: Defined messages have been intentionally devised in a way that their [first] command word will differ.
		swResult = sitalBcStld1553_Command_GetCode (&wCommandWord, sital1553_BROADCAST, sitalCommand_RX, wRtSubaddressRx_1, wWordCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Command.GetCode.Error: error - %i\n", swResult);
			throw 0;
		}
		dwaFirstCommandWord[5] = wCommandWord;

		/// Create for device-2 (BC) message-6, asynchronous RT(RT-address-2/RtSubAddressRx-1)-to-Broadcast(RtSubAddressRx-2) message that uses data-block-6 to transmit WordCount unique data words. Message bus: bus-1.
		S16BIT swDataBlockId_6; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId_6 = 7;
		wAscendingDataValue++;
		for (int i=0; i<wWordCount; i++) waBuffer[i] = wAscendingDataValue;
		S16BIT swMessageId_6; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_6 = 6;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i, buf_contents - %i...%i\n", s_swDevice2, swDataBlockId_6, 64, wWordCount, waBuffer[0], waBuffer[0]);
		sitalProcess_Log_PrintLine ("#StldTester.Bc.AsynchronousMessage.RtToBroadcast.Create: unit - %i, message - %i, block - %i, tx_rt - %i, tx_rt_sa - %i, rx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_6, swDataBlockId_6, s_wRtAddress2, wRtSubaddressRx_1, wRtSubaddressRx_2, wWordCount, 0, s_wBus1);
		swResult = sitalBc_AsynchronousMessage_CreateRtToBroadcast (s_swDevice2, swMessageId_6, swDataBlockId_6, wRtSubaddressRx_2, wWordCount, s_wRtAddress2, wRtSubaddressRx_1, 0, s_wBus1, waBuffer);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBroadcast.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Use given message descriptors to code the command word[s] of above defined message.
		// Note: Defined messages have been intentionally devised in a way that their [first] command word will differ.
		swResult = sitalBcStld1553_Command_GetCode (&wCommandWord, sital1553_BROADCAST, sitalCommand_RX, wRtSubaddressRx_2, wWordCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Command.GetCode.Error: error - %i\n", swResult);
			throw 0;
		}
		dwaFirstCommandWord[6] = wCommandWord;

		/// Create for device-2 (BC) message-7, asynchronous BC-to-Broadcast-Mode(code=2) message. Message bus: bus-1.
		S16BIT swDataBlockId_7; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId_7 = 8;
		wAscendingDataValue++;
		for (int i=0; i<wWordCount; i++) waBuffer[i] = wAscendingDataValue;
		S16BIT swMessageId_7; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_7 = 7;
		wModeCommand = 0x0001;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i, buf_contents - %i...%i\n", s_swDevice2, swDataBlockId_7, 64, wWordCount, waBuffer[0], waBuffer[0]);
		sitalProcess_Log_PrintLine ("#StldTester.Bc.AsynchronousMessage.BcToBroadcastMode.Create: unit - %i, message - %i, block - %i, direction - %i, mode - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_7, swDataBlockId_7, sitalMessageDirection_RX, wModeCommand, 0, s_wBus1);
		swResult = sitalBc_AsynchronousMessage_CreateBcToOrFromBroadcastMode (s_swDevice2, swMessageId_7, swDataBlockId_7, sitalMessageDirection_TX, wModeCommand, 0, s_wBus1, waBuffer);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToBroadcastMode.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Use given message descriptors to code the command word[s] of above defined message.
		// Note: Defined messages have been intentionally devised in a way that their [first] command word will differ.
		swResult = sitalBcStld1553_Command_GetCode (&wCommandWord, sital1553_BROADCAST, sitalCommand_TX, sitalRtSubaddress_0, wModeCommand);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Command.GetCode.Error: error - %i\n", swResult);
			throw 0;
		}
		dwaFirstCommandWord[7] = wCommandWord;

		for (U16BIT i=0; i<8; i++)
		{
			sitalProcess_Log_PrintLine ("#StldTester.MessageList: message_index - %i, first_command_word - %04X\n", i, dwaFirstCommandWord[i]);
		}

		///	Start device-2 (BC) running major-frame-1 for FrameCount-shot.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", s_swDevice2, swFrameId_1, wFrameCount);
		swResult = sitalBc_Start (s_swDevice2, swFrameId_1, wFrameCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		///	Request device-2 (BC) to transmit the first asynchronous message at high priority.
		S16BIT swMessageId; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.AsynchronousMessage.Tx.High: unit - %i, message - %i\n", s_swDevice2, swMessageId);
		swResult = sitalBc_AsynchronousMessage_SendAtHighPriority (s_swDevice2, swMessageId, 0);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.AsynchronousMessage.Tx.High.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Wait for the rest of the regular frame run to end.
		U32BIT dwSleepTime; // Sleeping period (in ms).
		dwSleepTime = (wRegularFrameCount *  ((U16BIT)(wFrameTime / IP_FREQUENCY_FACTOR) / 10));
		sitalProcess_Log_PrintLine ("#StldTester.Wait: time - %ims\n", dwSleepTime);
		sitalProcess_Log_PrintLine ("\tWAIT_FOR %i us\n", (dwSleepTime * 1000));
		PrecisionTimer pt; // A precision timer.
		pt.AccuratelySleep (dwSleepTime);

		/// Loop over all the rest of the asynchronous messages.
		for (swMessageId=2; swMessageId<=NUMBER_OF_MESSAGE_TYPES; swMessageId++)
		{
			///		Request device-2 (BC) to transmit the current asynchronous message at high priority.
			sitalProcess_Log_PrintLine ("#StldTester.Bc.AsynchronousMessage.Tx.High: unit - %i, message - %i\n", s_swDevice2, swMessageId);
			swResult = sitalBc_AsynchronousMessage_SendAtHighPriority (s_swDevice2, swMessageId, 0);
			if (0 > swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.AsynchronousMessage.Tx.High.Error: error - %i\n", swResult);
				throw 0;
			}
		}

		/// Wait for the irregular frame run to end.
		dwSleepTime = ((wFrameCount - wRegularFrameCount) * ((U16BIT)(wFrameTime / IP_FREQUENCY_FACTOR) / 10));
		sitalProcess_Log_PrintLine ("#StldTester.Wait: time - %ims\n", dwSleepTime);
		sitalProcess_Log_PrintLine ("\tWAIT_FOR %i us\n", (dwSleepTime * 1000));
		pt.AccuratelySleep (dwSleepTime);
		
		///	Stop device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop: unit - %i\n", s_swDevice2);
		swResult = sitalBc_Stop (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		///	Verify that:
		/// - Device-3 (MT) has detected all the regular and all the asynchronous messages.
		/// - The last message was a regular one.
		/// - The first asynchronous message has been transmitted before the last message of the first frame.
		///	- The rest of the asynchronous messages were transmitted only after the regular frames.
		///	- The asynchronous messages were transmitted at the correct order (see note below).
		/// - Each asynchronous message was transmitted exactly once.
		// Note:
		// - A high-priority asynchronous message may be transmitted either immediately after some regular message or along the passive phase of a frame.
		// - As said above, the minor frame has been designed so that the first high-priority asynchronous message will be transmitted somewhere among its regular messages.
		// - The rest of the high-priority asynchronous messages may have been principally transmitted in either location, but it's most reasonable that they'll be actually transmitted along the passive phase of a frame.
		sitalDecodedMessageStructure dmsDecodedMessage; // A decoded message.
		U16BIT wLeadingRegularMessagesCount; // The number of regular messages detected before the first asynchronous message.
		wLeadingRegularMessagesCount = 0;
		BOOLEAN bIsAnyPreviousRegularMessage; // A flag that says whether the first regular message has been received already.
		bIsAnyPreviousRegularMessage = false;
		BOOLEAN bIsPreviousMessageRegularOne; // A flag that says whether the previously detected message is a regular one.
		bIsPreviousMessageRegularOne = false;
		U16BIT wAsynchronousMessageCount; // The number of asynchronous messages.
		wAsynchronousMessageCount = NUMBER_OF_MESSAGE_TYPES;
		U16BIT wOverallMessageTxCount; // The overall number of transmitted messages (includes both regular and asynchronous transmitted messages).
		wOverallMessageTxCount = ((wMinorFrameCommandIdCount * wFrameCount) + wAsynchronousMessageCount);
		U16BIT wRecordedMessageCount; // The number of messages that were received by device-3 (MT) by now.
		wRecordedMessageCount = 0;
		S16BIT swaMessageCount[NUMBER_OF_MESSAGE_TYPES + 1]; // An array in which each entry stores the count of recorded messages whose first command word is stored at the corresponding entry of array dwaFirstCommandWord.
		S16BIT swaMessageIndex[NUMBER_OF_MESSAGE_TYPES + 1]; // An array in which each entry stores the index of the last recorded message whose first command word is stored at the corresponding entry of array dwaFirstCommandWord (-1 designates that such a message wasn't received yet).
		for (U16BIT i=0; i<=NUMBER_OF_MESSAGE_TYPES; i++)
		{
			swaMessageCount[i] = 0;
			swaMessageIndex[i] = -1;
		}
		do
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded: unit - %i, purge - %i\n", s_swDevice3, TRUE);
			swResult = sitalMt_Message_GetFromStackDecoded (s_swDevice3, &dmsDecodedMessage, sitalMessageLocationAndRemoval_NEXT_PURGE, sitalMtStack_ACTIVE);
			if ((0 > swResult) || (1 < swResult))
			{
				sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Error: error - %i\n", swResult);
				throw 0;
			}
			if (1 == swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Mt.Message: unit - %i, message_index - %i, command - %04X, data_count - %i, data - %i..., time_tag - %04X\n", s_swDevice3, wRecordedMessageCount, dmsDecodedMessage.wCommandWord1, dmsDecodedMessage.wDataWordCount, dmsDecodedMessage.waData[0], dmsDecodedMessage.wTimeTag);
				for (U16BIT i=0; i<=NUMBER_OF_MESSAGE_TYPES; i++)
				{
					if (dwaFirstCommandWord[i] == dmsDecodedMessage.wCommandWord1)
					{
						swaMessageCount[i]++;
						swaMessageIndex[i] = wRecordedMessageCount;
					}
				}
				wRecordedMessageCount++;
			}
		}
		while (1 == swResult);

		for (U16BIT i=0; i<=NUMBER_OF_MESSAGE_TYPES; i++)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.Message: unit - %i, message_type_index - %i, first_command_word - %04X, count - %i, last_message_index - %i\n", s_swDevice3, i, dwaFirstCommandWord[i], swaMessageCount[i], swaMessageIndex[i]);
			if (0U != i)
			{
				if (1 == i)
				{
					if (wMinorFrameCommandIdCount <= swaMessageIndex[i])
					{
						sitalProcess_Log_PrintLine ("#StldTester.Mt.Message.Error: error - %s\n", "bad_first_index");
						swResult = sitalReturnCode_TEST_FAIL - 3;
						throw 0;
					}
					/*else {
						printf("transmitted at message ID %i", swaMessageIndex[i]);
					}*/
				}
				else
				if (1 < i)
				{
					if ((wMinorFrameCommandIdCount * wRegularFrameCount) > swaMessageIndex[i])
					{
						sitalProcess_Log_PrintLine ("#StldTester.Mt.Message.Error: error - %s\n", "bad_index");
						swResult = sitalReturnCode_TEST_FAIL - 4;
						throw 0;
					}

					if (swaMessageIndex[i - 1] >= swaMessageIndex[i])
					{
						sitalProcess_Log_PrintLine ("#StldTester.Mt.Message.Error: error - %s\n", "bad_order");
						swResult = sitalReturnCode_TEST_FAIL - 5;
						throw 0;
					}
				}

				if (1 != swaMessageCount[i])
				{
					sitalProcess_Log_PrintLine ("#StldTester.Mt.Message.Error: error - %s\n", "bad_count");
					swResult = sitalReturnCode_TEST_FAIL - 6;
					throw 0;
				}
			}
		}

		if ((wRecordedMessageCount - 1) != swaMessageIndex[0])
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.Message.Error: error - %s\n", "bad_last_message");
			swResult = sitalReturnCode_TEST_FAIL - 7;
			throw 0;
		}

		if (wOverallMessageTxCount != wRecordedMessageCount)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.Message.Error: error - %s\n", "bad_total_count");
			swResult = sitalReturnCode_TEST_FAIL -8;
			throw 0;
		}

		/// Stop device-4 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Stop: unit - %i\n", s_swDevice4);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Stop (s_swDevice4) : sitalRt_Stop (s_swDevice4) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Stop device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Stop: unit - %i\n", s_swDevice1);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Stop (s_swDevice1) : sitalRt_Stop (s_swDevice1) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Stop device-3 (MT).
		sitalProcess_Log_PrintLine ("#StldTester.Mt.Stop: unit - %i\n", s_swDevice3);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Stop (s_swDevice3) : sitalMt_Stop (s_swDevice3) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice1);
		swResult = sitalDevice_Free (s_swDevice1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice2);
		swResult = sitalDevice_Free (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-3 (MT).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice3);
		swResult = sitalDevice_Free (s_swDevice3);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-4 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice4);
		swResult = sitalDevice_Free (s_swDevice4);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		swResult = stldSyncEnd (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncEnd.Error: error - %i\n", swResult);
			throw 0;
		}
	}
	catch (int)
	{
		sitalDevice_Free (s_swDevice1);
		sitalDevice_Free (s_swDevice2);
		sitalDevice_Free (s_swDevice3);
		sitalDevice_Free (s_swDevice4);
	}

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_11m_High.End: test_succeeded - %c\n", ((sitalReturnCode_SUCCESS == swResult) ? 'Y' : 'N'));
	return swResult;
}



/// <summary>
/// Test the stld1553 library in the henceforth described scenario:
/// Participant[s]: RT1, BC2, MT3.
/// Configuration: All participating devices must be able to communicate with each other.
/// Short description: Acceptance Test Plan (ATP) test 461_11n - Frame time tracking.
/// Along test print suitable info and indications into the log.
///
/// Note:
/// - This function may use any of above defined Acceptance Test Plan (ATP) tests variables.
///   The caller must therefore call the suitable stldTester_Atp_Set* functions prior to calling this function.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Atp_DoTest461_11n ()
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_11n.Start\n");

	/// Set test arguments.
	U16BIT wRtSubaddressRx; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31) that's set for a RT for Rx.
	wRtSubaddressRx = sitalRtSubaddress_1;
	U16BIT wRtSubaddressTx; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31) that's set for a RT for Tx.
	wRtSubaddressTx = sitalRtSubaddress_2;
	U16BIT wWordCount; // Word count (1-32).
	wWordCount = 32;
	U16BIT wFrameTime; // The time (in units of 100 us) that the frame will take to complete.
	wFrameTime = 200; // 20ms.
	U16BIT wFrameShotCount; // Number of frame shots.
	wFrameShotCount = 10;
	U16BIT wTimeTagCalculationInaccuracy; // The maximum tolerated difference between the expected time tag and the actual one.
	wTimeTagCalculationInaccuracy = 25;

	S16BIT swResult; // Result of operation or function call.
	swResult = sitalReturnCode_SUCCESS;
	InterProcessSemaphore ipsSemaphore (s_szBcSemaphoreName, 1); // A signaling object that is used by an assigned ISR function to signal interrupts to waiting threads.

	/// Get device count.
	U16BIT wRequiredDeviceCount; // The number of devices required for this test.
	wRequiredDeviceCount = (1 + max(max(s_swDevice1, s_swDevice2), s_swDevice3));
	U16BIT wAvailableDeviceCount; // The total number of devices available in the system.
	swResult = sitalDevice_GetCount (&wAvailableDeviceCount);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %i\n", swResult);
		return swResult;
	}
	sitalProcess_Log_PrintLine ("#StldTester.Device.Count: available - %i, required - %i\n", wAvailableDeviceCount, wRequiredDeviceCount);
	if (wAvailableDeviceCount < wRequiredDeviceCount)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %s\n", "not_enough_devices_installed");
		return sitalReturnCode_SUCCESS;
	}
	if ( (s_swDevice1 == s_swDevice2) || (s_swDevice1 == s_swDevice3) || (s_swDevice2 == s_swDevice3))
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Number.Error: error - %s\n", "same_device_used_twice");
		return sitalReturnCode_INVALID_DEVICE_NUMBER;
	}

	try
	{
		swResult = stldSyncStart (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncStart.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-3 (MT) as a real MT.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access - card, mode - mt, alloc_mem - none\n", s_swDevice3);
		swResult = sitalDevice_Initialize (s_swDevice3, sitalAccess_CARD, ((s_bIsRtMtModeTest ? sitalMode_RT_AND_MT : sitalMode_MT) | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Start device-3 (MT).
		sitalProcess_Log_PrintLine ("#StldTester.Mt.Start: unit - %i\n", s_swDevice3);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Start (s_swDevice3) : sitalMt_Start (s_swDevice3) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-1 (RT) as a real RT.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-rt, alloc_mem-none\n", s_swDevice1);
		swResult = sitalDevice_Initialize (s_swDevice1, sitalAccess_CARD, ((s_bIsRtMtModeTest ? sitalMode_RT_AND_MT : sitalMode_RT) | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}
		if (s_bIsRtMtModeTest)
		{
			// Accommodate with tests where the required amount of allocations of data blocks for RT devices is to big when these RT devices are initialized as RT-MT devices with a rather limited amount of memory.
			sitalProcess_Log_PrintLine ("#StldTester.RtMt.Init: unit - %i, rt_command_stack_size - %i, mt_stack_count - %i, mt_command_stack_size - %i, mt_data_stack_size - %i, options - %X\n", s_swDevice1, sitalRtCommandStackSize_256, sitalMtStackOption_SINGLE, sitalMtCommandStackSize_256, sitalMtDataStackSize_512, 0);
			swResult = sitalRtMt_Initialize (s_swDevice1, sitalRtCommandStackSize_256, sitalMtStackOption_SINGLE, sitalMtCommandStackSize_256, sitalMtDataStackSize_512, 0);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.RtMt.Init.Error: error - %i\n", swResult);
				throw 0;
			}
		}

		/// Set device-1 (RT) address to RT-address-1.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set: unit - %i, address - %i\n", s_swDevice1, s_wRtAddress1);
		swResult = sitalRt_Address_Set (s_swDevice1, s_wRtAddress1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set.Error: error - %i\n", swResult);
			throw 0;
		}

		U16BIT wRtSubaddress; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31).
		const U16BIT waMessageType[] // Array of message types.
				= { sitalRtMessageType_RX, sitalRtMessageType_TX, sitalRtMessageType_BROADCAST};
		U16BIT wMessageTypeCount; // Number of defined message types.
		wMessageTypeCount = ( sizeof(waMessageType) / sizeof(waMessageType[0]) );
		S16BIT swDataBlockId; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId = 1;
		U16BIT waBuffer[32]; // A buffer to read-from/write-into along data-block write/read operations, respectively.
		for (int i=0; i<wWordCount; i++) waBuffer[i] = (wWordCount - 1 - i);
		for (wRtSubaddress=sitalRtSubaddress_1; wRtSubaddress<=sitalRtSubaddress_30; wRtSubaddress++)
		{
			for (int i=0; i<wMessageTypeCount; i++)
			{
				/// Create for device-1 (RT) data-block-DataBlockId, a data block of 64 data words.
				sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i, buf_contents-...%i\n", s_swDevice1, swDataBlockId, 64, wWordCount, (wWordCount - 1));
				swResult = sitalRt_DataBlock_Create (s_swDevice1, swDataBlockId, sitalRtDataBlockType_SINGLE_32, waBuffer, wWordCount);
				if (sitalReturnCode_SUCCESS != swResult)
				{
					sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.Create.Error: error - %i\n", swResult);
					throw 0;
				}

				/// Map for device-1 (RT) data-block-DataBlockId with message-type-j for subaddress-i, and legalize this subaddress.
				sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.MapToSa: unit - %i, block - %i, sa - %i, msg_types - x%X, irq_opts-x%X, legalize - %c\n", s_swDevice1, swDataBlockId, wRtSubaddress, waMessageType[i], sitalRtDataBlockIrq_NONE, 'Y');
				swResult = sitalRt_DataBlock_MapToSubaddress (s_swDevice1, swDataBlockId, wRtSubaddress, waMessageType[i], sitalRtDataBlockIrq_NONE, TRUE);
				if (sitalReturnCode_SUCCESS != swResult)
				{
					sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.MapToSa.Error: error - %i\n", swResult);
					throw 0;
				}

				/// Move to next DataBlockId.
				swDataBlockId++;
			}
		}

		/// Start device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Start: unit - %i\n", s_swDevice1);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Start (s_swDevice1) : sitalRt_Start (s_swDevice1) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-2 (BC) as a real BC.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-bc, alloc_mem-none\n", s_swDevice2);
		swResult = sitalDevice_Initialize (s_swDevice2, sitalAccess_CARD, (sitalMode_BC | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Set for device-2 (BC) a time tag resolution of 2 us.
		U16BIT wTimeTagResolution; // Time tag resolution (in units of 1 us).
		wTimeTagResolution = 2;
		sitalProcess_Log_PrintLine ("#StldTester.Device.TimeTagResolution.Set: unit - %i, resolution - %ius\n", s_swDevice2);
		swResult = sitalDevice_TimeTag_SetResolution (s_swDevice2, sitalTimeTagResolution_2US);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.TimeTagResolution.Set.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) data-block-0, a data block of 64 data words.
		S16BIT swDataBlockId_0; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId_0 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i\n", s_swDevice2, swDataBlockId_0, 64, 0);
		swResult = sitalBc_DataBlock_Create (s_swDevice2, swDataBlockId_0, sitalBcDataBlockSizeOption_DOUBLE, NULL, 0);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) data-block-1, a data block of 64 data words, and fill the first WordCount data words with ascending values.
		S16BIT swDataBlockId_1; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId_1 = 2;
		for (int i=0; i<wWordCount; i++) waBuffer[i] = i;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i, buf_contents - %i...\n", s_swDevice2, swDataBlockId_1, 64, wWordCount, 0);
		swResult = sitalBc_DataBlock_Create (s_swDevice2, swDataBlockId_1, sitalBcDataBlockSizeOption_DOUBLE, waBuffer, wWordCount);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) message-0, a RT(RT-address-1/RtSubAddressTx)-to-BC message that uses data-block-0 to receive WordCount data words. Message bus: bus-1.
		S16BIT swMessageId_0; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBc.Create: unit - %i, message - %i, block - %i, tx_rt - %i, tx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddressTx, wWordCount, 0, s_wBus1);
		swResult = sitalBc_Message_CreateRtToBc (s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddressTx, wWordCount, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBc.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) message-1, a BC-to-RT(RT-address-1/RtSubAddressRx) message that uses data-block-1 to transmit WordCount data words. Message bus: bus-1.
		S16BIT swMessageId_1; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_1 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create: unit - %i, message - %i, block - %i, rx_rt - %i, rx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_1, swDataBlockId_1, s_wRtAddress1, wRtSubaddressRx, wWordCount, 0, s_wBus1);
		swResult = sitalBc_Message_CreateBcToRt (s_swDevice2, swMessageId_1, swDataBlockId_1, s_wRtAddress1, wRtSubaddressRx, wWordCount, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-0, an XQF command that executes message-0.
		S16BIT swCommandId_0; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-1, an XQF command that executes message-1.
		S16BIT swCommandId_1; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_1 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_1, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_1, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_1, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_1, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-2, a subroutine call command that executes minor-frame-0. 
		S16BIT swCommandId_2; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_2 = 2;
		S16BIT swFrameId_0; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_2, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, swFrameId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_2, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, (U16BIT)swFrameId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) minor-frame-0, a frame that contains command-0 and command-1.
		S16BIT swaCommandIds[sitalBcCounter_COMMANDS]; // An array of command-IDs.
		swaCommandIds[0] = swCommandId_0;
		swaCommandIds[1] = swCommandId_1;
		U16BIT wCommandIdCount; // The number of command IDs in the frame (1-sitalBcCounter_FRAME_COMMANDS).
		wCommandIdCount = 2;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i, %i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_0, "minor", wCommandIdCount, swCommandId_0, swCommandId_1, (wFrameTime / 10), 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_0, sitalBcFrameType_MINOR, swaCommandIds, wCommandIdCount, wFrameTime, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) major-frame-1, a frame that contains command-2 and whose frame time is FrameTime us.
		S16BIT swFrameId_1; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_1 = 1;
		swaCommandIds[0] = swCommandId_2;
		wCommandIdCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_1, "major", wCommandIdCount, swCommandId_2, (wFrameTime / 10), 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_1, sitalBcFrameType_MAJOR, swaCommandIds, wCommandIdCount, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Start device-2 (BC) running major-frame-1 for FrameShotCount-shots.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", s_swDevice2, swFrameId_1, wFrameShotCount);
		swResult = sitalBc_Start (s_swDevice2, swFrameId_1, wFrameShotCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Wait for the frame run to end.
		U32BIT dwRemainingFrameRunTime; // The expected remaining frame run time (in ms).
		dwRemainingFrameRunTime = (((U32BIT)wFrameShotCount) * (wFrameTime / 10));
		swResult = stldTester_Bc_WaitForEndOfFrameRun (s_swDevice2, dwRemainingFrameRunTime, wFrameShotCount, &s_dwMinorFrameCounter);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.WaitForEndOfFrameRun.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Stop device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop: unit - %i\n", s_swDevice2);
		swResult = sitalBc_Stop (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Stop device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Stop: unit - %i\n", s_swDevice1);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Stop (s_swDevice1) : sitalRt_Stop (s_swDevice1) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Stop device-3 (MT).
		sitalProcess_Log_PrintLine ("#StldTester.Mt.Stop: unit - %i\n", s_swDevice3);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Stop (s_swDevice3) : sitalMt_Stop (s_swDevice3) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Verify that device-3 (MT) detects suitable incremental time tags for all messages.
		U16BIT wExpectedTimeTag; // The next expected time tag.
		wExpectedTimeTag = 0;
		for (S16BIT i=1; i<=wFrameShotCount; i++)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded: unit - %i, purge - %i\n", s_swDevice3, TRUE);
			sitalDecodedMessageStructure dmsDecodedMessage0; // A decoded message.
			swResult = sitalMt_Message_GetFromStackDecoded (s_swDevice3, &dmsDecodedMessage0, sitalMessageLocationAndRemoval_NEXT_PURGE, sitalMtStack_ACTIVE);
			if ((0 > swResult) || (1 < swResult))
			{
				sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Error: error - %i\n", swResult);
				throw 0;
			}
			if (0U == swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Message.Error: error - %s\n", "no_message");
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}
			sitalProcess_Log_PrintLine ("#StldTester.Mt.Message: unit - %i, frame_number - %i, message_number - %i, command - %04X, data_count - %i, status_1_relevant - %i, status_1 - %04X, time_tag - %04X\n", s_swDevice3, i, 1, dmsDecodedMessage0.wCommandWord1, dmsDecodedMessage0.wDataWordCount, dmsDecodedMessage0.bIsStatus1Relevant, dmsDecodedMessage0.wStatus1, dmsDecodedMessage0.wTimeTag);

			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded: unit - %i, purge - %i\n", s_swDevice3, TRUE);
			sitalDecodedMessageStructure dmsDecodedMessage1; // A decoded message.
			swResult = sitalMt_Message_GetFromStackDecoded (s_swDevice3, &dmsDecodedMessage1, sitalMessageLocationAndRemoval_NEXT_PURGE, sitalMtStack_ACTIVE);
			if ((0 > swResult) || (1 < swResult))
			{
				sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Error: error - %i\n", swResult);
				throw 0;
			}
			if (0U == swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Message.Error: error - %s\n", "no_message");
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}
			sitalProcess_Log_PrintLine ("#StldTester.Mt.Message: unit - %i, frame_number - %i, message_number - %i, command - %04X, data_count - %i, status_1_relevant - %i, status_1 - %04X, time_tag - %04X\n", s_swDevice3, i, 2, dmsDecodedMessage1.wCommandWord1, dmsDecodedMessage1.wDataWordCount, dmsDecodedMessage1.bIsStatus1Relevant, dmsDecodedMessage1.wStatus1, dmsDecodedMessage1.wTimeTag);

			if (1 == i)
			{
				wExpectedTimeTag = dmsDecodedMessage0.wTimeTag;
			}
			U16BIT wTimeTagDifference; // The difference between the expected time tag and the actual one.
			wTimeTagDifference = (wExpectedTimeTag - dmsDecodedMessage0.wTimeTag);
			wTimeTagDifference = ( (0x8000 < wTimeTagDifference) ? (0x10000 - wTimeTagDifference) : wTimeTagDifference); // To handle a case of time tag rollover.
			sitalProcess_Log_PrintLine ("#StldTester.Mt.Message.TimeTag.Test: unit - %i, time_tag - %04X, expected_time_tag - ~%04X\n", s_swDevice3, dmsDecodedMessage0.wTimeTag, wExpectedTimeTag);
			if (wTimeTagCalculationInaccuracy < wTimeTagDifference)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Mt.Message.TimeTag.Test.Error: error - %s\n", "bad_timing");
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}
			wExpectedTimeTag = ( dmsDecodedMessage0.wTimeTag + (wFrameTime * 100 / wTimeTagResolution) ); // Frame time is expressed in units of 100us.
		}

		/// Free device-3 (MT).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice3);
		swResult = sitalDevice_Free (s_swDevice3);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice2);
		swResult = sitalDevice_Free (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice1);
		swResult = sitalDevice_Free (s_swDevice1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		swResult = stldSyncEnd (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncEnd.Error: error - %i\n", swResult);
			throw 0;
		}
	}
	catch (int)
	{
		sitalDevice_Free (s_swDevice3);
		sitalDevice_Free (s_swDevice2);
		sitalDevice_Free (s_swDevice1);
	}

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_11n.End: test_succeeded - %c\n", ((sitalReturnCode_SUCCESS == swResult) ? 'Y' : 'N'));
	return swResult;
}



/// <summary>
/// Test the stld1553 library in the henceforth described scenario:
/// Participant[s]: RT1, BC2.
/// Configuration: All participating devices must be able to communicate with each other.
/// Short description: Acceptance Test Plan (ATP) test 461_13a - RT features: a) Programmable command illegalization.
/// Along test print suitable info and indications into the log.
///
/// Note:
/// - This function may use any of above defined Acceptance Test Plan (ATP) tests variables.
///   The caller must therefore call the suitable stldTester_Atp_Set* functions prior to calling this function.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Atp_DoTest461_13a ()
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_13a.Start\n");

	/// Set test arguments.
	U16BIT wWordCount; // Word count (1-32).
	wWordCount = 32;
	U16BIT wLegalRtSubaddress; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31).
	wLegalRtSubaddress = sitalRtSubaddress_1;
	U16BIT wIllegalRtSubaddress; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31).
	wIllegalRtSubaddress = sitalRtSubaddress_2;
	U16BIT wFrameTime; // The time (in units of 100 us) that the frame will take to complete.
	wFrameTime = 330; // 33ms.

	S16BIT swResult; // Result of operation or function call.
	swResult = sitalReturnCode_SUCCESS;

	/// Get device count.
	U16BIT wRequiredDeviceCount; // The number of devices required for this test.
	wRequiredDeviceCount = (1 + max(s_swDevice1, s_swDevice2));
	U16BIT wAvailableDeviceCount; // The total number of devices available in the system.
	swResult = sitalDevice_GetCount (&wAvailableDeviceCount);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %i\n", swResult);
		return swResult;
	}
	sitalProcess_Log_PrintLine ("#StldTester.Device.Count: available - %i, required - %i\n", wAvailableDeviceCount, wRequiredDeviceCount);
	if (wAvailableDeviceCount < wRequiredDeviceCount)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %s\n", "not_enough_devices_installed");
		return sitalReturnCode_SUCCESS;
	}
	if ( (s_swDevice1 == s_swDevice2) )
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Number.Error: error - %s\n", "same_device_used_twice");
		return sitalReturnCode_INVALID_DEVICE_NUMBER;
	}

	try
	{
		swResult = stldSyncStart (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncStart.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-1 (RT) as a real RT in advanced mode.
		// Note: Advanced mode will enable r/w to the memory of device-1 (RT) while the device is running.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-rt, alloc_mem-none\n", s_swDevice1);
		swResult = sitalDevice_Initialize (s_swDevice1, sitalAccess_CARD, ((s_bIsRtMtModeTest ? sitalMode_RT_AND_MT : sitalMode_RT) | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Set device-1 (RT) address to RtAddress1.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set: unit - %i, address - %i\n", s_swDevice1, s_wRtAddress1);
		swResult = sitalRt_Address_Set (s_swDevice1, s_wRtAddress1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Legalize on device-1 (RT) all address-types in all directions for subaddress-LegalRtSubaddress and all word counts.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.MessageLegality.Enable: unit - %i, address_type - %04X, directions - %04X, subaddress - %04X, word_count - %08X\n", s_swDevice1, sitalRtAddressType_BOTH, sitalMessageDirection_BOTH, wLegalRtSubaddress, 0xFFFFFFFF);
		swResult = sitalRt_MessageLegality_Enable (s_swDevice1, sitalRtAddressType_BOTH, sitalMessageDirection_BOTH, wLegalRtSubaddress, 0xFFFFFFFF);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.MessageLegality.Enable.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Illegalize on device-1 (RT) all address-types in all directions for subaddress-IllegalRtSubaddress and all word counts.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.MessageLegality.Disable: unit - %i, address_type - %04X, directions - %04X, subaddress - %04X, word_count - %08X\n", s_swDevice1, sitalRtAddressType_BOTH, sitalMessageDirection_BOTH, wIllegalRtSubaddress, 0xFFFFFFFF);
		swResult = sitalRt_MessageLegality_Disable (s_swDevice1, sitalRtAddressType_BOTH, sitalMessageDirection_BOTH, wIllegalRtSubaddress, 0xFFFFFFFF);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.MessageLegality.Disable.Error: error - %i\n", swResult);
			throw 0;
		}

		U32BIT dwMessageLegality; // Message legality.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.MessageLegality.Get: unit - %i, address_type - %04X, directions - %04X, subaddress - %04X\n", s_swDevice1, sitalRtAddressType_OWN, sitalMessageDirection_RX, wLegalRtSubaddress);
		swResult = sitalRt_MessageLegality_GetStatus (s_swDevice1, sitalRtAddressType_OWN, sitalMessageDirection_RX, wLegalRtSubaddress, &dwMessageLegality);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.MessageLegality.Get.Error: error - %i\n", swResult);
			throw 0;
		}
		sitalProcess_Log_PrintLine ("#StldTester.Rt.MessageLegality.Get: unit - %i, address_type - %04X, directions - %04X, subaddress - %04X, legality - %08X\n", s_swDevice1, sitalRtAddressType_OWN, sitalMessageDirection_RX, wLegalRtSubaddress, dwMessageLegality);
		if (0U != dwMessageLegality)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.MessageLegality.Get.Error: error - %s\n", "bad_legality");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// Start device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Start: unit - %i\n", s_swDevice1);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Start (s_swDevice1) : sitalRt_Start (s_swDevice1) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-2 (BC) as a real BC.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-bc, alloc_mem-none\n", s_swDevice2);
		swResult = sitalDevice_Initialize (s_swDevice2, sitalAccess_CARD, (sitalMode_BC | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) data-block-0, a data block of 64 data words, and fill the first WordCount data words with zeros.
		S16BIT swDataBlockId_0; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId_0 = 1;
		U16BIT waBuffer[32]; // A buffer to read-from/write-into along data-block write/read operations, respectively.
		for (int i=0; i<wWordCount; i++) waBuffer[i] = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i, buf_contents - %i...%i\n", s_swDevice2, swDataBlockId_0, 64, wWordCount, waBuffer[0], waBuffer[0]);
		swResult = sitalBc_DataBlock_Create (s_swDevice2, swDataBlockId_0, sitalBcDataBlockSizeOption_DOUBLE, waBuffer, wWordCount);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) message-0, a BC-to-RT(RT-address-1/LegalRtSubAddress message that uses data-block-0 to transmit WordCount data words. Message bus: bus-1.
		S16BIT swMessageId_0; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create: unit - %i, message - %i, block - %i, rx_rt - %i, rx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wLegalRtSubaddress, wWordCount, 0, s_wBus1);
		swResult = sitalBc_Message_CreateBcToRt (s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wLegalRtSubaddress, wWordCount, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) message-1, a BC-to-RT(RT-address-1/IllegalRtSubAddress message that uses data-block-0 to transmit WordCount data words. Message bus: bus-1.
		S16BIT swMessageId_1; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_1 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create: unit - %i, message - %i, block - %i, rx_rt - %i, rx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_1, swDataBlockId_0, s_wRtAddress1, wIllegalRtSubaddress, wWordCount, 0, s_wBus1);
		swResult = sitalBc_Message_CreateBcToRt (s_swDevice2, swMessageId_1, swDataBlockId_0, s_wRtAddress1, wIllegalRtSubaddress, wWordCount, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) message-2, a RT(RtAddress1/LegalRtSubAddress)-to-BC message that uses data-block-0 to receive WordCount data words. Message bus: bus-1.
		S16BIT swMessageId_2; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_2 = 2;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBc.Create: unit - %i, message - %i, block - %i, tx_rt - %i, tx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_2, swDataBlockId_0, s_wRtAddress1, wLegalRtSubaddress, wWordCount, 0, s_wBus1);
		swResult = sitalBc_Message_CreateRtToBc (s_swDevice2, swMessageId_2, swDataBlockId_0, s_wRtAddress1, wLegalRtSubaddress, wWordCount, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBc.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) message-3, a RT(RtAddress1/IllegalRtSubAddress)-to-BC message that uses data-block-0 to receive WordCount data words. Message bus: bus-1.
		S16BIT swMessageId_3; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_3 = 3;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBc.Create: unit - %i, message - %i, block - %i, tx_rt - %i, tx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_3, swDataBlockId_0, s_wRtAddress1, wIllegalRtSubaddress, wWordCount, 0, s_wBus1);
		swResult = sitalBc_Message_CreateRtToBc (s_swDevice2, swMessageId_3, swDataBlockId_0, s_wRtAddress1, wIllegalRtSubaddress, wWordCount, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBc.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Use given message descriptors to code the [first] command word of above defined messages.
		// Note: Defined messages have been intentionally devised in a way that their [first] command word will differ.
		U16BIT wCommandWord0; // Command word for above defined BC-to-RT message-0.
		swResult = sitalBcStld1553_Command_GetCode (&wCommandWord0, s_wRtAddress1, sitalCommand_RX, wLegalRtSubaddress, wWordCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Command.GetCode.Error: error - %i\n", swResult);
			throw 0;
		}
		U16BIT wCommandWord1; // Command word for above defined BC-to-RT message-1.
		swResult = sitalBcStld1553_Command_GetCode (&wCommandWord1, s_wRtAddress1, sitalCommand_RX, wIllegalRtSubaddress, wWordCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Command.GetCode.Error: error - %i\n", swResult);
			throw 0;
		}
		U16BIT wCommandWord2; // Command word for above defined RT-to-BC message-2.
		swResult = sitalBcStld1553_Command_GetCode (&wCommandWord2, s_wRtAddress1, sitalCommand_TX, wLegalRtSubaddress, wWordCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Command.GetCode.Error: error - %i\n", swResult);
			throw 0;
		}
		U16BIT wCommandWord3; // Command word for above defined RT-to-BC message-3.
		swResult = sitalBcStld1553_Command_GetCode (&wCommandWord3, s_wRtAddress1, sitalCommand_TX, wIllegalRtSubaddress, wWordCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Command.GetCode.Error: error - %i\n", swResult);
			throw 0;
		}
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.Create.Summary: unit - %i, command0 - %04X, command1 - %04X, command2 - %04X, command3 - %04X\n", s_swDevice2, wCommandWord0, wCommandWord1, wCommandWord2, wCommandWord3);

		/// Create for device-2 (BC) command-0, a XEQ command that executes message-0.
		S16BIT swCommandId_0; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-1, a XEQ command that executes message-1.
		S16BIT swCommandId_1; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_1 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_1, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_1, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_1, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_1, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-2, a XEQ command that executes message-2.
		S16BIT swCommandId_2; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_2 = 2;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_2, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_2, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_2, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_2, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-3, a XEQ command that executes message-3.
		S16BIT swCommandId_3; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_3 = 3;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_3, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_3, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_3, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_3, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-4, a subroutine call command that executes minor-frame-0.
		S16BIT swCommandId_4; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_4 = 4;
		S16BIT swFrameId_0; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_4, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, swFrameId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_4, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, (U16BIT)swFrameId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) minor-frame-0, a frame that contains commands 0-3.
		S16BIT swaCommandIds[sitalBcCounter_COMMANDS]; // An array of command-IDs.
		swaCommandIds[0] = swCommandId_0;
		swaCommandIds[1] = swCommandId_1;
		swaCommandIds[2] = swCommandId_2;
		swaCommandIds[3] = swCommandId_3;
		U16BIT wCommandIdCount; // The number of command IDs in the frame (1-sitalBcCounter_FRAME_COMMANDS).
		wCommandIdCount = 4;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i %i %i %i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_0, "minor", wCommandIdCount, swCommandId_0, swCommandId_1, swCommandId_2, swCommandId_3, 0, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_0, sitalBcFrameType_MINOR, swaCommandIds, wCommandIdCount, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) major-frame-1, a frame that contains command-4 and whose time is FrameTime.
		S16BIT swFrameId_1; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_1 = 1;
		swaCommandIds[0] = swCommandId_4;
		wCommandIdCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_1, "major", wCommandIdCount, swCommandId_4, wFrameTime, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_1, sitalBcFrameType_MAJOR, swaCommandIds, wCommandIdCount, wFrameTime, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		///	Start device-2 (BC) running major-frame-1 for 1-shot.
		U16BIT wFrameCount; // The number of frames to run.
		wFrameCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", s_swDevice2, swFrameId_1, wFrameCount);
		swResult = sitalBc_Start (s_swDevice2, swFrameId_1, wFrameCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Wait for the frame run to end.
		U32BIT dwRemainingFrameRunTime; // The expected remaining frame run time (in ms).
		dwRemainingFrameRunTime = (wFrameCount * (wFrameTime / 10));
		swResult = stldTester_Bc_WaitForEndOfFrameRun (s_swDevice2, dwRemainingFrameRunTime, 0, NULL);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.WaitForEndOfFrameRun.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Stop device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop: unit - %i\n", s_swDevice2);
		swResult = sitalBc_Stop (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Stop device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Stop: unit - %i\n", s_swDevice1);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Stop (s_swDevice1) : sitalRt_Stop (s_swDevice1) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// With device-1 (RT) verify detection of message-0 with expected status.
		sitalDecodedMessageStructure dmsDecodedMessage; // A decoded message.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Message.GetFromStackDecoded: unit - %i, message - %i, purge - %i\n", s_swDevice1, swMessageId_0, TRUE);
		swResult = sitalRt_Message_GetFromStackDecoded (s_swDevice1, &dmsDecodedMessage, sitalMessageLocationAndRemoval_NEXT_PURGE);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Message.GetDecoded.Error: error - %i\n", swResult);
			throw 0;
		}
		if (1 != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Message.GetDecoded.Error: error - %i\n", swResult);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Message.GetDecoded: unit - %i, message - %i, command1 - %04X, expected_command1 - %04X, message_block_status - %04X\n", s_swDevice1, swMessageId_0, dmsDecodedMessage.wCommandWord1, wCommandWord0, dmsDecodedMessage.wBlockStatus);
		if (wCommandWord0 != dmsDecodedMessage.wCommandWord1)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Message.GetDecoded.Error: error - %s\n", "bad_command");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		if (sitalRtBlockStatusWord_END_OF_MESSAGE != (sitalRtBlockStatusWord_END_OF_MESSAGE & dmsDecodedMessage.wBlockStatus))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Message.GetDecoded.Error: error - %s\n", "bad_bsw");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// With device-1 (RT) verify detection of message-2 with expected status.
		// Note: A RT device isn't expected to record in its stack messages to illegal subaddresses, and message-1 shouldn't be therefore found in the stack; instead, message-2 should be found now.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Message.GetFromStackRaw: unit - %i, message - %i, purge - %i\n", s_swDevice1, swMessageId_2, TRUE);
		U16BIT waMessageBuffer[sitalRtMaximum_MESSAGE_SIZE];
		swResult = sitalRt_Message_GetFromStackRaw (s_swDevice1, waMessageBuffer, sitalRtMaximum_MESSAGE_SIZE);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Message.GetRaw.Error: error - %i\n", swResult);
			throw 0;
		}
		if (1 != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Message.GetRaw.Error: error - %i\n", swResult);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Message.DecodeRaw: unit - %i, message - %i\n", s_swDevice1, swMessageId_2);
		swResult = sitalRt_Message_DecodeRaw (s_swDevice1, waMessageBuffer, &dmsDecodedMessage);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Message.DecodeRaw.Error: error - %i\n", swResult);
			throw 0;
		}
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Message.GetDecoded: unit - %i, message - %i, command1 - %04X, expected_command1 - %04X, message_block_status - %04X\n", s_swDevice1, swMessageId_2, dmsDecodedMessage.wCommandWord1, wCommandWord3, dmsDecodedMessage.wBlockStatus);
		if (wCommandWord2 != dmsDecodedMessage.wCommandWord1)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Message.GetDecoded.Error: error - %s\n", "bad_command");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		if (sitalRtBlockStatusWord_END_OF_MESSAGE != (sitalRtBlockStatusWord_END_OF_MESSAGE & dmsDecodedMessage.wBlockStatus))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Message.GetDecoded.Error: error - %s\n", "bad_bsw");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// With device-1 (RT) verify that the stack is empty.
		// Note: A RT device isn't expected to record in its stack messages to illegal subaddresses, and message-3 shouldn't be therefore found in the stack, so the stack must be empty now.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Message.GetFromStackDecoded: unit - %i, purge - %i\n", s_swDevice1, TRUE);
		swResult = sitalRt_Message_GetFromStackDecoded (s_swDevice1, &dmsDecodedMessage, sitalMessageLocationAndRemoval_NEXT_PURGE);
		if (0 > swResult) 
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Message.GetDecoded.Error: error - %i\n", swResult);
			throw 0;
		}
		if (1 == swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Message.GetDecoded: unit - %i, command1 - %04X, message_block_status - %04X\n", s_swDevice1, dmsDecodedMessage.wCommandWord1, dmsDecodedMessage.wBlockStatus);
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Message.GetDecoded.Error: error - %s\n", "unexpected_message_in_stack");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// With device-2 (BC) verify detection of message-0 with expected status.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetByIdDecoded: unit - %i, message - %i, purge-y\n", s_swDevice2, swMessageId_0);
		swResult = sitalBc_Message_GetByIdDecoded (s_swDevice2, swMessageId_0, &dmsDecodedMessage, TRUE);
		if (1 != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded.Error: error - %i\n", swResult);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded: unit - %i, message - %i, command1 - %04X, message_block_status - %04X, status_1_relevant - %i, status_1 - %04X\n", s_swDevice1, swMessageId_0, dmsDecodedMessage.wCommandWord1, dmsDecodedMessage.wBlockStatus, dmsDecodedMessage.bIsStatus1Relevant, dmsDecodedMessage.wStatus1);
		if (sitalBcBlockStatusWord_END_OF_MESSAGE != (sitalBcBlockStatusWord_END_OF_MESSAGE & dmsDecodedMessage.wBlockStatus))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded.Error: error - %s\n", "bad_bsw");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// With device-2 (BC) verify detection of message-1 with expected status.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetByIdDecoded: unit - %i, message - %i, purge-y\n", s_swDevice2, swMessageId_1);
		swResult = sitalBc_Message_GetByIdDecoded (s_swDevice2, swMessageId_1, &dmsDecodedMessage, TRUE);
		if (1 != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded.Error: error - %i\n", swResult);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded: unit - %i, message - %i, command1 - %04X, message_block_status - %04X, status_1_relevant - %i, status_1 - %04X\n", s_swDevice1, swMessageId_1, dmsDecodedMessage.wCommandWord1, dmsDecodedMessage.wBlockStatus, dmsDecodedMessage.bIsStatus1Relevant, dmsDecodedMessage.wStatus1);
		if ((sitalBcBlockStatusWord_END_OF_MESSAGE | sitalBcBlockStatusWord_STATUS_SET | sitalBcBlockStatusWord_MASKED_STATUS) != ((sitalBcBlockStatusWord_END_OF_MESSAGE | sitalBcBlockStatusWord_STATUS_SET | sitalBcBlockStatusWord_MASKED_STATUS) & dmsDecodedMessage.wBlockStatus))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded.Error: error - %s\n", "bad_bsw");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// With device-2 (BC) verify detection of message-2 with expected status.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetByIdDecoded: unit - %i, message - %i, purge-y\n", s_swDevice2, swMessageId_2);
		swResult = sitalBc_Message_GetByIdDecoded (s_swDevice2, swMessageId_2, &dmsDecodedMessage, TRUE);
		if (1 != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded.Error: error - %i\n", swResult);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded: unit - %i, message - %i, command1 - %04X, message_block_status - %04X, status_1_relevant - %i, status_1 - %04X\n", s_swDevice1, swMessageId_2, dmsDecodedMessage.wCommandWord1, dmsDecodedMessage.wBlockStatus, dmsDecodedMessage.bIsStatus1Relevant, dmsDecodedMessage.wStatus1);
		if ((sitalBcBlockStatusWord_END_OF_MESSAGE | sitalBcBlockStatusWord_GOOD_DATA_BLOCK_RECEIVED) != ((sitalBcBlockStatusWord_END_OF_MESSAGE | sitalBcBlockStatusWord_GOOD_DATA_BLOCK_RECEIVED) & dmsDecodedMessage.wBlockStatus))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded.Error: error - %s\n", "bad_bsw");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// With device-2 (BC) verify detection of message-3 with expected status.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetByIdDecoded: unit - %i, message - %i, purge-y\n", s_swDevice2, swMessageId_3);
		swResult = sitalBc_Message_GetByIdDecoded (s_swDevice2, swMessageId_3, &dmsDecodedMessage, TRUE);
		if (1 != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded.Error: error - %i\n", swResult);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded: unit - %i, message - %i, command1 - %04X, message_block_status - %04X, status_1_relevant - %i, status_1 - %04X\n", s_swDevice1, swMessageId_3, dmsDecodedMessage.wCommandWord1, dmsDecodedMessage.wBlockStatus, dmsDecodedMessage.bIsStatus1Relevant, dmsDecodedMessage.wStatus1);
		if ((sitalBcBlockStatusWord_END_OF_MESSAGE | sitalBcBlockStatusWord_ERROR_FLAG | sitalBcBlockStatusWord_STATUS_SET | sitalBcBlockStatusWord_FORMAT_ERROR | sitalBcBlockStatusWord_MASKED_STATUS | sitalBcBlockStatusWord_WORD_COUNT_ERROR) != ((sitalBcBlockStatusWord_END_OF_MESSAGE | sitalBcBlockStatusWord_ERROR_FLAG | sitalBcBlockStatusWord_STATUS_SET | sitalBcBlockStatusWord_FORMAT_ERROR | sitalBcBlockStatusWord_MASKED_STATUS | sitalBcBlockStatusWord_WORD_COUNT_ERROR) & dmsDecodedMessage.wBlockStatus))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded.Error: error - %s\n", "bad_bsw");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// Free device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice1);
		swResult = sitalDevice_Free (s_swDevice1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice2);
		swResult = sitalDevice_Free (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		swResult = stldSyncEnd (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncEnd.Error: error - %i\n", swResult);
			throw 0;
		}
	}
	catch (int)
	{
		sitalDevice_Free (s_swDevice1);
		sitalDevice_Free (s_swDevice2);
	}

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_13a.End: test_succeeded - %c\n", ((sitalReturnCode_SUCCESS == swResult) ? 'Y' : 'N'));
	return swResult;
}



/// <summary>
/// Test the stld1553 library in the henceforth described scenario:
/// Participant[s]: RT1, BC2.
/// Configuration: All participating devices must be able to communicate with each other.
/// Short description: Acceptance Test Plan (ATP) test 461_13b - RT features: b) Choice of single buffer, double buffering, and circular buffering.
/// Along test print suitable info and indications into the log.
///
/// Note:
/// - This function may use any of above defined Acceptance Test Plan (ATP) tests variables.
///   The caller must therefore call the suitable stldTester_Atp_Set* functions prior to calling this function.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Atp_DoTest461_13b ()
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_13b.Start\n");

	// When using the DDC (or a DDC-like) library, un-include code that is unique to Sital and isn't supported by DDC.
#ifdef USE_DDC_LIBRARY

	sitalProcess_Log_PrintLine ("#StldTester.Library.Error: error - %s\n", "test_may_run_only_when_using_Sital_stld1553_library");
	return sitalReturnCode_SUCCESS;
	
#else // ! USE_DDC_LIBRARY

	/// Set test arguments.
	U16BIT wWordCount; // Word count (1-32).
	wWordCount = 30; // Would be best to set the count of message data words to a number that isn't a divider of any of the sizes of RT circular data blocks.
	U16BIT wRtSubaddress; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31).
	wRtSubaddress = sitalRtSubaddress_1;
	U16BIT wFrameTime; // The time (in units of 100 us) that the frame will take to complete.
	wFrameTime = 100; // 10ms.
	U16BIT wTimeTagCalculationInaccuracy; // The maximum tolerated difference between the expected time tag and the actual one.
	wTimeTagCalculationInaccuracy = 25;

	S16BIT swResult; // Result of operation or function call.
	swResult = sitalReturnCode_SUCCESS;

	/// Get device count.
	U16BIT wRequiredDeviceCount; // The number of devices required for this test.
	wRequiredDeviceCount = (1 + max(s_swDevice1, s_swDevice2));
	U16BIT wAvailableDeviceCount; // The total number of devices available in the system.
	swResult = sitalDevice_GetCount (&wAvailableDeviceCount);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %i\n", swResult);
		return swResult;
	}
	sitalProcess_Log_PrintLine ("#StldTester.Device.Count: available - %i, required - %i\n", wAvailableDeviceCount, wRequiredDeviceCount);
	if (wAvailableDeviceCount < wRequiredDeviceCount)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %s\n", "not_enough_devices_installed");
		return sitalReturnCode_SUCCESS;
	}
	if ( (s_swDevice1 == s_swDevice2) )
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Number.Error: error - %s\n", "same_device_used_twice");
		return sitalReturnCode_INVALID_DEVICE_NUMBER;
	}

	try
	{
		swResult = stldSyncStart (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncStart.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-1 (RT) as a real RT in advanced mode.
		// Note: Advanced mode will enable r/w to the memory of device-1 (RT) while the device is running.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-rt, alloc_mem-none\n", s_swDevice1);
		swResult = sitalDevice_Initialize (s_swDevice1, sitalAccess_CARD, ((s_bIsRtMtModeTest ? sitalMode_RT_AND_MT : sitalMode_RT) | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Set device-1 (RT) address to RtAddress1.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set: unit - %i, address - %i\n", s_swDevice1, s_wRtAddress1);
		swResult = sitalRt_Address_Set (s_swDevice1, s_wRtAddress1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-2 (BC) as a real BC.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-bc, alloc_mem-none\n", s_swDevice2);
		swResult = sitalDevice_Initialize (s_swDevice2, sitalAccess_CARD, (sitalMode_BC | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Set for device-2 (BC) a time tag resolution of 2 us.
		U16BIT wTimeTagResolution; // Time tag resolution (in units of 1 us).
		wTimeTagResolution = 2;
		sitalProcess_Log_PrintLine ("#StldTester.Device.TimeTagResolution.Set: unit - %i, resolution - %ius\n", s_swDevice2);
		swResult = sitalDevice_TimeTag_SetResolution (s_swDevice2, sitalTimeTagResolution_2US);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.TimeTagResolution.Set.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) data-block-0, a data block of 32 data words.
		S16BIT swDataBlockId_0; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId_0 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i\n", s_swDevice2, swDataBlockId_0, 32, 0);
		swResult = sitalBc_DataBlock_Create (s_swDevice2, swDataBlockId_0, sitalBcDataBlockSizeOption_SINGLE, NULL, 0);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) message-0, a BC-to-RT(RT-address-1/RtSubAddress message that uses data-block-0 to transmit WordCount data words. Message bus: bus-1.
		S16BIT swMessageId_0; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create: unit - %i, message - %i, block - %i, rx_rt - %i, rx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddress, wWordCount, 0, s_wBus1);
		swResult = sitalBc_Message_CreateBcToRt (s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddress, wWordCount, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create.Error: error - %i\n", swResult);
			throw 0;
		}
		U16BIT wTimeOfMessage0; // The estimated time required for the transmission of message-0 (in units of 1 us).
		wTimeOfMessage0 = 700;

		/// Use given message descriptors to code the command word of above defined message.
		U16BIT wCommandWord0; // Command word for above defined BC-to-RT message-0.
		swResult = sitalBcStld1553_Command_GetCode (&wCommandWord0, s_wRtAddress1, sitalCommand_RX, wRtSubaddress, wWordCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Command.GetCode.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) message-1, a RT(RtAddress1/RtSubAddress)-to-BC message that uses data-block-0 to receive WordCount data words. Message bus: bus-1.
		S16BIT swMessageId_1; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_1 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBc.Create: unit - %i, message - %i, block - %i, tx_rt - %i, tx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_1, swDataBlockId_0, s_wRtAddress1, wRtSubaddress, wWordCount, 0, s_wBus1);
		swResult = sitalBc_Message_CreateRtToBc (s_swDevice2, swMessageId_1, swDataBlockId_0, s_wRtAddress1, wRtSubaddress, wWordCount, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBc.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Use given message descriptors to code the command word of above defined message.
		U16BIT wCommandWord1; // Command word for above defined RT-to-BC message-1.
		swResult = sitalBcStld1553_Command_GetCode (&wCommandWord1, s_wRtAddress1, sitalCommand_TX, wRtSubaddress, wWordCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Command.GetCode.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-0, a XEQ command that executes message-0.
		S16BIT swCommandId_0; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-1, a XEQ command that executes message-1.
		S16BIT swCommandId_1; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_1 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_1, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_1, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_1, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_1, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-2, a subroutine call command that executes minor-frame-0.
		S16BIT swCommandId_2; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_2 = 2;
		S16BIT swFrameId_0; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_2, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, swFrameId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_2, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, (U16BIT)swFrameId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) minor-frame-0, a frame that contains command-0 and command-1.
		S16BIT swaCommandIds[sitalBcCounter_COMMANDS]; // An array of command-IDs.
		swaCommandIds[0] = swCommandId_0;
		swaCommandIds[1] = swCommandId_1;
		U16BIT wCommandIdCount; // The number of command IDs in the frame (1-sitalBcCounter_FRAME_COMMANDS).
		wCommandIdCount = 2;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i %i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_0, "minor", wCommandIdCount, swCommandId_0, swCommandId_1, 0, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_0, sitalBcFrameType_MINOR, swaCommandIds, wCommandIdCount, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) major-frame-1, a frame that contains command-4 and whose time is FrameTime.
		S16BIT swFrameId_1; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_1 = 1;
		swaCommandIds[0] = swCommandId_2;
		wCommandIdCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_1, "major", wCommandIdCount, swCommandId_2, wFrameTime, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_1, sitalBcFrameType_MAJOR, swaCommandIds, wCommandIdCount, wFrameTime, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Loop over all tested types of RT data blocks:
		const U16BIT waRtDataBlockType[] // An array of types of RT data blocks (sitalRtDataBlockType_*).
						= { sitalRtDataBlockType_SINGLE_32, sitalRtDataBlockType_DOUBLE, sitalRtDataBlockType_CIRCULAR_128, sitalRtDataBlockType_CIRCULAR_256, sitalRtDataBlockType_CIRCULAR_512, sitalRtDataBlockType_CIRCULAR_1024, sitalRtDataBlockType_CIRCULAR_2048, sitalRtDataBlockType_CIRCULAR_4096, sitalRtDataBlockType_CIRCULAR_8192 };
		U16BIT wRtDataBlockTypeCount; // Number of defined RT data block types.
		wRtDataBlockTypeCount = ( sizeof(waRtDataBlockType) / sizeof(waRtDataBlockType[0]) );
		S16BIT swRtDataBlockTypeIndex; // The index of the current type of RT data block.
		for (swRtDataBlockTypeIndex=0; swRtDataBlockTypeIndex<wRtDataBlockTypeCount; swRtDataBlockTypeIndex++)
		{
			S16BIT swRtDataBlockId; // A unique ID designating a data block (0-(sitalRtCounter_DATA_BLOCKS-1)).
			swRtDataBlockId = swRtDataBlockTypeIndex;
			U16BIT wRtDataBlockType; // A type of RT data block (sitalRtDataBlockType_*).
			wRtDataBlockType = waRtDataBlockType[swRtDataBlockTypeIndex];

			sitalProcess_Log_PrintLine ("#StldTester.SubTest.Iteration: iteration - %i, block_type - %i\n", swRtDataBlockTypeIndex, wRtDataBlockType);

			/// 	Create for device-1 (RT) data-block-DataBlockId, a data block of currently tested type.
			sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.Create: unit - %i, block - %i, type - %i, buf_size - %i\n", s_swDevice1, swRtDataBlockId, wRtDataBlockType, 0);
			swResult = sitalRt_DataBlock_Create (s_swDevice1, swRtDataBlockId, wRtDataBlockType, NULL, 0);
			if (sitalReturnCode_ALLOCATION_FAIL == swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.Create.Error: error - %i\n", swResult);
			}
			else
			{
				if (sitalReturnCode_SUCCESS != swResult)
				{
					sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.Create.Error: error - %i\n", swResult);
					throw 0;
				}

				/// 	Map for device-1 (RT) data-block-DataBlockId with message-types Rx and Tx for subaddress-RtSubaddress, and legalize this subaddress.
				sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.MapToSa: unit - %i, block - %i, sa - %i, msg_types - x%X, irq_opts-x%X, legalize - %c\n", s_swDevice1, swRtDataBlockId, wRtSubaddress, (sitalRtMessageType_RX | sitalRtMessageType_TX), sitalRtDataBlockIrq_NONE, 'Y');
				swResult = sitalRt_DataBlock_MapToSubaddress (s_swDevice1, swRtDataBlockId, wRtSubaddress, (sitalRtMessageType_RX | sitalRtMessageType_TX), sitalRtDataBlockIrq_NONE, TRUE);
				if (sitalReturnCode_SUCCESS != swResult)
				{
					sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.MapToSa.Error: error - %i\n", swResult);
					throw 0;
				}

				///		Start device-1 (RT).
				sitalProcess_Log_PrintLine ("#StldTester.Rt.Start: unit - %i\n", s_swDevice1);
				swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Start (s_swDevice1) : sitalRt_Start (s_swDevice1) );
				if (sitalReturnCode_SUCCESS != swResult)
				{
					sitalProcess_Log_PrintLine ("#StldTester.Rt.Start.Error: error - %i\n", swResult);
					throw 0;
				}

				/// Loop over such a number of rounds of data transmission/reception to/from the device-1 (RT) so that a cyclic wrap will have to take place.
				U16BIT wAscendingDataValue; // An ascending data value.
				wAscendingDataValue = 0;
				BOOLEAN bIsEvenRound; // A flag that says whether the current round has an even index.
				U16BIT wExpectedDataValue; // The value expected to be received from device-1 (RT) in result of the second message (RT-to-BC) of the frame whose run has just ended.
				wExpectedDataValue = wAscendingDataValue;
				U16BIT wExpectedTimeTagForMessage1; // The next expected time tag.
				S16BIT swRoundIndex; // The index of the current round.
				S16BIT swRound; // The number of the current round.
				U16BIT wSizeOfAllocatedDeviceMemory; // The size (in words) of device memory allocated for the data block.
				U16BIT wActualSizeOfDataBlock; // The actual size (in words) of the data block.
				swResult = sitalRt_DataBlock_GetSize (wRtDataBlockType, &wSizeOfAllocatedDeviceMemory, &wActualSizeOfDataBlock);
				if (sitalReturnCode_SUCCESS != swResult)
				{
					sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Size.Get.Error: error - %i\n", swResult);
					throw 0;
				}
				sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Size.Get: block_type - %i, allocated_size - %i, actual_size - %i, minimum_required_size - %i\n", wRtDataBlockType, wSizeOfAllocatedDeviceMemory, wActualSizeOfDataBlock, wWordCount);
				if (wWordCount > wActualSizeOfDataBlock)
				{
					sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Size.Error: error - %s\n", "too_small");
					throw 0;
				}
				sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.Address.Get: unit - %i, block - %i\n", s_swDevice1, swRtDataBlockId);
				U16BIT wDeviceMemoryAddressOfDataBlock; // The address (in words) of device memory allocated for the currently tested RT data block.
				swResult = sitalRt_DataBlock_GetAddress (s_swDevice1, swRtDataBlockId, &wDeviceMemoryAddressOfDataBlock);
				if (sitalReturnCode_SUCCESS != swResult)
				{
					sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.Address.Get.Error: error - %i\n", swResult);
					throw 0;
				}
				sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.Address.Get: unit - %i, block - %i, address - %04X\n", s_swDevice1, swRtDataBlockId, wDeviceMemoryAddressOfDataBlock);
				U16BIT wMinimumShotsToInsureCyclicWrap; // The minimum number of frame shots required in order to insure the occurrance of a cyclic wrap with the currently tested type of RT data block.
				wMinimumShotsToInsureCyclicWrap = ( ( wActualSizeOfDataBlock / wWordCount ) + 1 ); // +1 to ensure cyclic wrap with circular buffers.
				U16BIT wCountOfOneShotsPerRtDataBlock; // The number of rounds per tested type of RT data block in which a data block with incremental data is transmitted/received to/from the RT.
				wCountOfOneShotsPerRtDataBlock = ( wMinimumShotsToInsureCyclicWrap + 0 ); // May be wMinimumShotsToInsureCyclicWrap or higher, not less!
				for (swRoundIndex=0, swRound=1, bIsEvenRound=TRUE; swRoundIndex<wCountOfOneShotsPerRtDataBlock; swRoundIndex++, swRound++, bIsEvenRound=(~bIsEvenRound), wAscendingDataValue++)
				{
					///		Update for device-2 (BC) the data in the buffer of data-block-0.
					U16BIT waBuffer[32]; // A buffer to read-from/write-into along data-block write/read operations, respectively.
					for (int i=0; i<wWordCount; i++) waBuffer[i] = wAscendingDataValue;
					sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Write: unit - %i, block - %i, offset - %i, buf_size - %i, buf_contents - %04X...\n", s_swDevice2, swDataBlockId_0, 0, wWordCount, waBuffer[0]);
					swResult = sitalBc_DataBlock_Write (s_swDevice2, swDataBlockId_0, waBuffer, wWordCount, 0);
					if (sitalReturnCode_SUCCESS != swResult)
					{
						sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Write.Error: error - %i\n", swResult);
						throw 0;
					}

					///	Start device-2 (BC) running major-frame-1 for 1-shot.
					sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", s_swDevice2, swFrameId_1, 1);
					swResult = sitalBc_Start (s_swDevice2, swFrameId_1, 1);
					if (sitalReturnCode_SUCCESS != swResult)
					{
						sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
						throw 0;
					}

					/// Wait for the frame run to end.
					U32BIT dwRemainingFrameRunTime; // The expected remaining frame run time (in ms).
					dwRemainingFrameRunTime = (1 *  (wFrameTime / 10));
					swResult = stldTester_Bc_WaitForEndOfFrameRun (s_swDevice2, dwRemainingFrameRunTime, 0, NULL);
					if (sitalReturnCode_SUCCESS != swResult)
					{
						sitalProcess_Log_PrintLine ("#StldTester.Bc.WaitForEndOfFrameRun.Error: error - %i\n", swResult);
						throw 0;
					}

					/// Stop device-2 (BC).
					sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop: unit - %i\n", s_swDevice2);
					swResult = sitalBc_Stop (s_swDevice2);
					if (sitalReturnCode_SUCCESS != swResult)
					{
						sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
						throw 0;
					}

					///		Get for device-2 (BC) the data in the buffer of data-block-0.
					sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Read: unit - %i, block - %i, buf_size - %i, offset - %i\n", s_swDevice2, swDataBlockId_0, wWordCount, 0);
					swResult = sitalBc_DataBlock_Read (s_swDevice2, swDataBlockId_0, waBuffer, wWordCount, 0);
					if (0 > swResult)
					{
						sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Read.Error: error - %i\n", swResult);
						throw 0;
					}

					///		Verify that device-2 (BC) received the expected data.
					// Note:
					// - In case of a regular or circular RT data block:
					//   - The data transmitted by the BC-to-RT message will be automatically returned by the RT-to-BC message only in all rounds.
					// - In in case of a double RT data block:
					//   - The RT device writes the data it receives intermittently in the data block's first and second 32-word sections.
					//   - Yet, when the RT device transmits data from such a block, it always sends the data in the first 32-word section.
					//   - As a result, the data transmitted by the BC-to-RT message will be automatically returned by the RT-to-BC message only in even rounds (0, 2, 4, ...).
					//   - In the odd rounds the expected data is the data that has been transmitted in the previous (even) round.
					if ((sitalRtDataBlockType_DOUBLE != wRtDataBlockType) || (TRUE == bIsEvenRound))
					{
						wExpectedDataValue = wAscendingDataValue;
					}
					BOOLEAN bIsExpectedData; // A flag that says whether received data is the expected data, or not.
					bIsExpectedData = ( (wExpectedDataValue == waBuffer[0]) && (wExpectedDataValue == waBuffer[wWordCount - 1]) );
					sitalProcess_Log_PrintLine ("#StldTester.Bc.Data.Compare: transmitted_data - %i, expected_data - %i, received_data - %i, data_okay - %i\n", wAscendingDataValue, wExpectedDataValue, waBuffer[0], bIsExpectedData);
					if (FALSE == bIsExpectedData)
					{
						sitalProcess_Log_PrintLine ("#StldTester.Bc.Data.Compare.Error: error - %s\n", "bad_data");
						swResult = sitalReturnCode_TEST_FAIL;
						throw 0;
					}

					///		Get message-0 from device-2 (BC).
					sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded: unit - %i, message - %i, purge-y\n", s_swDevice2, swMessageId_0);
					sitalDecodedMessageStructure dmsDecodedMessage0; // A decoded message.
					swResult = sitalBc_Message_GetByIdDecoded (s_swDevice2, swMessageId_0, &dmsDecodedMessage0, TRUE);
					if (0 > swResult)
					{
						sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded.Error: error - %i\n", swResult);
						throw 0;
					}
					if (1 != swResult)
					{
						sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded.Error: error - %i\n", swResult);
						swResult = sitalReturnCode_TEST_FAIL;
						throw 0;
					}

					///		Get message-1 from device-2 (BC).
					sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded: unit - %i, message - %i, purge-y\n", s_swDevice2, swMessageId_1);
					sitalDecodedMessageStructure dmsDecodedMessage1; // A decoded message.
					swResult = sitalBc_Message_GetByIdDecoded (s_swDevice2, swMessageId_1, &dmsDecodedMessage1, TRUE);
					if (0 > swResult)
					{
						sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded.Error: error - %i\n", swResult);
						throw 0;
					}
					if (1 != swResult)
					{
						sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded.Error: error - %i\n", swResult);
						swResult = sitalReturnCode_TEST_FAIL;
						throw 0;
					}

					///		Verify that the time tag is properly advanced.
					wExpectedTimeTagForMessage1 = ( dmsDecodedMessage0.wTimeTag + (wTimeOfMessage0 / wTimeTagResolution) );
					U16BIT wTimeTagDifference; // The difference between the expected time tag and the actual one.
					wTimeTagDifference = (wExpectedTimeTagForMessage1 - dmsDecodedMessage1.wTimeTag);
					wTimeTagDifference = ( (0x8000 < wTimeTagDifference) ? (0x10000 - wTimeTagDifference) : wTimeTagDifference); // To handle a case of time tag rollover.
					sitalProcess_Log_PrintLine ("#StldTester.BC.Message: unit - %i, time_tag_0 - %04X, time_tag_1 - %04X, expected_time_tag_1 - ~%04X, difference - %i, maximum_difference - %i\n", s_swDevice2, dmsDecodedMessage0.wTimeTag, dmsDecodedMessage1.wTimeTag, wExpectedTimeTagForMessage1, wTimeTagDifference, wTimeTagCalculationInaccuracy);
					if (wTimeTagCalculationInaccuracy < wTimeTagDifference)
					{
						sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.Error: error - %s\n", "bad_timing");
						swResult = sitalReturnCode_TEST_FAIL;
						throw 0;
					}

					///		Get message-0 from device-1 (RT).
					sitalProcess_Log_PrintLine ("#StldTester.Rt.Message.GetFromStackDecoded: unit - %i, message - %i, purge - %i\n", s_swDevice1, swMessageId_0, TRUE);
					swResult = sitalRt_Message_GetFromStackDecoded (s_swDevice1, &dmsDecodedMessage0, sitalMessageLocationAndRemoval_NEXT_PURGE);
					if (0 > swResult)
					{
						sitalProcess_Log_PrintLine ("#StldTester.Rt.Message.GetDecoded.Error: error - %i\n", swResult);
						throw 0;
					}
					if (1 != swResult)
					{
						sitalProcess_Log_PrintLine ("#StldTester.Rt.Message.GetDecoded.Error: error - %i\n", swResult);
						swResult = sitalReturnCode_TEST_FAIL;
						throw 0;
					}
					sitalProcess_Log_PrintLine ("#StldTester.Rt.Message.GetDecoded: unit - %i, message - %i, command1 - %04X, command2 - %04X, is_command2 - %i, expected_command1 - %04X, message_block_status - %04X, status1 - %04X, status2 - %04X, data_count - %i(%i), data_words - %04X...%04X(%04X)\n", s_swDevice1, swMessageId_0, dmsDecodedMessage0.wCommandWord1, dmsDecodedMessage0.wCommandWord2, dmsDecodedMessage0.bIsCommandWord2Relevant, wCommandWord0, dmsDecodedMessage0.wBlockStatus, dmsDecodedMessage0.wStatus1, dmsDecodedMessage0.wStatus2, dmsDecodedMessage0.wDataWordCount, wWordCount, dmsDecodedMessage0.waData[0], dmsDecodedMessage0.waData[wWordCount - 1], wAscendingDataValue);

					///		Verify that device-1 (RT) properly recorded the message data words it received.
					// Note:
					// - It has already been verified above that device-1 (RT) has received and transmitted the expected data.
					// - See above notes in regard with the expected data words in case of regular, double, and circular RT data blocks.
					bIsExpectedData = ( (wAscendingDataValue == dmsDecodedMessage0.waData[0]) && (wAscendingDataValue == dmsDecodedMessage0.waData[wWordCount - 1]) );
					sitalProcess_Log_PrintLine ("#StldTester.Rt.Data.Compare.Received: iteration - %i, round - %i, message - %i, first_data_word - %04X, last_data_word - %04X, expected_data - %04X, data_okay - %i\n", swRtDataBlockTypeIndex, swRoundIndex, swMessageId_0, dmsDecodedMessage0.waData[0], dmsDecodedMessage0.waData[wWordCount - 1], wAscendingDataValue, bIsExpectedData);
					if (FALSE == bIsExpectedData)
					{
						sitalProcess_Log_PrintLine ("#StldTester.Rt.Data.Compare.Received.Error: error - %s\n", "bad_data");
						swResult = sitalReturnCode_TEST_FAIL;
						throw 0;
					}

					///		Get message-1 from device-1 (RT).
					sitalProcess_Log_PrintLine ("#StldTester.Rt.Message.GetFromStackDecoded: unit - %i, message - %i, purge - %i\n", s_swDevice1, swMessageId_1, TRUE);
					swResult = sitalRt_Message_GetFromStackDecoded (s_swDevice1, &dmsDecodedMessage1, sitalMessageLocationAndRemoval_NEXT_PURGE);
					if (0 > swResult)
					{
						sitalProcess_Log_PrintLine ("#StldTester.Rt.Message.GetDecoded.Error: error - %i\n", swResult);
						throw 0;
					}
					if (1 != swResult)
					{
						sitalProcess_Log_PrintLine ("#StldTester.Rt.Message.GetDecoded.Error: error - %i\n", swResult);
						swResult = sitalReturnCode_TEST_FAIL;
						throw 0;
					}
					sitalProcess_Log_PrintLine ("#StldTester.Rt.Message.GetDecoded: unit - %i, message - %i, command1 - %04X, command2 - %04X, is_command2 - %i, expected_command1 - %04X, message_block_status - %04X, status1 - %04X, status2 - %04X, data_count - %i(%i), data_words - %04X...%04X(%04X)\n", s_swDevice1, swMessageId_0, dmsDecodedMessage1.wCommandWord1, dmsDecodedMessage1.wCommandWord2, dmsDecodedMessage1.bIsCommandWord2Relevant, wCommandWord0, dmsDecodedMessage1.wBlockStatus, dmsDecodedMessage1.wStatus1, dmsDecodedMessage1.wStatus2, dmsDecodedMessage1.wDataWordCount, wWordCount, dmsDecodedMessage1.waData[0], dmsDecodedMessage1.waData[wWordCount - 1], wExpectedDataValue);

					///		Verify that device-1 (RT) properly recorded the message data words it transmitted.
					// Note:
					// - It has already been verified above that device-1 (RT) has received and transmitted the expected data.
					// - See above notes in regard with the expected data words in case of regular, double, and circular RT data blocks.
					bIsExpectedData = ( (wExpectedDataValue == dmsDecodedMessage1.waData[0]) && (wExpectedDataValue == dmsDecodedMessage1.waData[wWordCount - 1]) );
					sitalProcess_Log_PrintLine ("#StldTester.Rt.Data.Compare.Received: iteration - %i, round - %i, message - %i, first_data_word - %04X, last_data_word - %04X, expected_data - %04X, data_okay - %i\n", swRtDataBlockTypeIndex, swRoundIndex, swMessageId_1, dmsDecodedMessage1.waData[0], dmsDecodedMessage1.waData[wWordCount - 1], wExpectedDataValue, bIsExpectedData);
					if (FALSE == bIsExpectedData)
					{
						sitalProcess_Log_PrintLine ("#StldTester.Rt.Data.Compare.Received.Error: error - %s\n", "bad_data");
						swResult = sitalReturnCode_TEST_FAIL;
						throw 0;
					}

					// Here it is verified that circular data blocks have the expected size, i.e., that device memory in a suitable size has been dedicated for them AND that the device makes a cyclic wrap exactly when they are filled in this size.
					// This may be checked by constantly checking the first word of the data buffer: A cyclic wrap must change this word, which shouldn't change otherwise.
					// This is verified only until the first cyclic wrap, of course.
					///		If the currently tested RT data block is a circular data block AND a cyclic wrap either still didn't occur OR had just occurred:
					///			Read the current value of the first word in data-block-DataBlockId of device-1 (RT).
					///			If the first frame was just transmitted:
					///				Store the current value of the first word in data-block-DataBlockId of device-1 (RT) as its original value.
					///			Else:
					///				If a cyclic wrap shouldn't had yet occurred with data-block-DataBlockId of device-1 (RT):
					///					Verify that the first word in data-block-DataBlockId of device-1 (RT) has retained its original value.
					///				Else:
					///					If a cyclic wrap should have just occurred with data-block-DataBlockId of device-1 (RT):
					///						Verify that the first word in data-block-DataBlockId of device-1 (RT) has changed its value.
					U16BIT wCurrentValueOfFirstWord; // The current value of the first word in the currently tested RT data block, which is the value it had received after the transmission of the last frame.
					wCurrentValueOfFirstWord = 0;
					U16BIT wOriginalValueOfFirstWord; // The original value of the first word in the currently tested RT data block, which is the value it had after the transmission of the first frame.
					wOriginalValueOfFirstWord = 0;
					if ( (sitalRtDataBlockType_DOUBLE < wRtDataBlockType) && (swRound <= wMinimumShotsToInsureCyclicWrap) )
					{
						wCurrentValueOfFirstWord = sitalDevice_Memory_Read (s_swDevice1, wDeviceMemoryAddressOfDataBlock);
						if (1 == swRound)
						{
							wOriginalValueOfFirstWord = wCurrentValueOfFirstWord;
						}
						else
						{
							sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.Size.Check: original_first_word - %04X, current_first_word - %04X\n", wOriginalValueOfFirstWord, wCurrentValueOfFirstWord);
							if (wMinimumShotsToInsureCyclicWrap > swRound)
							{
								if (wOriginalValueOfFirstWord != wCurrentValueOfFirstWord)
								{
									sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.Size.Check.Error: error - %s\n", "early_cyclic_wrap");
									swResult = sitalReturnCode_TEST_FAIL;
									throw 0;
								}
							}
							else
								// (wMinimumShotsToInsureCyclicWrap == swRound)
							{
								if (wOriginalValueOfFirstWord == wCurrentValueOfFirstWord)
								{
									if (wOriginalValueOfFirstWord != wCurrentValueOfFirstWord)
									{
										sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.Size.Check.Error: error - %s\n", "no_cyclic_wrap");
										swResult = sitalReturnCode_TEST_FAIL;
										throw 0;
									}
								}
							}
						}
					}
				}

				///		Stop device-1 (RT).
				sitalProcess_Log_PrintLine ("#StldTester.Rt.Stop: unit - %i\n", s_swDevice1);
				swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Stop (s_swDevice1) : sitalRt_Stop (s_swDevice1) );
				if (sitalReturnCode_SUCCESS != swResult)
				{
					sitalProcess_Log_PrintLine ("#StldTester.Rt.Stop.Error: error - %i\n", swResult);
					throw 0;
				}

				/// 	Unmap for device-1 (RT) data-block-DataBlockId with message-types Rx and Tx for subaddress-RtSubaddress.
				sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.UnMapToSa: unit - %i, block - %i, sa - %i, msg_types - x%X\n", s_swDevice1, swRtDataBlockId, wRtSubaddress, (sitalRtMessageType_RX | sitalRtMessageType_TX));
				swResult = sitalRt_DataBlock_UnmapFromSubaddress (s_swDevice1, swRtDataBlockId, wRtSubaddress, (sitalRtMessageType_RX | sitalRtMessageType_TX));
				if (sitalReturnCode_SUCCESS != swResult)
				{
					sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.UnMapToSa.Error: error - %i\n", swResult);
					throw 0;
				}

				///		Delete device-1 (RT) data-block-0.
				sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.Delete: unit - %i, block - %i\n", s_swDevice1, swRtDataBlockId);
				swResult = sitalRt_DataBlock_Delete (s_swDevice1, swRtDataBlockId);
				if (sitalReturnCode_SUCCESS != swResult)
				{
					sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.Delete.Error: error - %i\n", swResult);
					throw 0;
				}
			}
		}

		/// Free device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice1);
		swResult = sitalDevice_Free (s_swDevice1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice2);
		swResult = sitalDevice_Free (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		swResult = stldSyncEnd (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncEnd.Error: error - %i\n", swResult);
			throw 0;
		}
	}
	catch (int)
	{
		sitalDevice_Free (s_swDevice1);
		sitalDevice_Free (s_swDevice2);
	}

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_13b.End: test_succeeded - %c\n", ((sitalReturnCode_SUCCESS == swResult) ? 'Y' : 'N'));
	return swResult;

#endif // USE_DDC_LIBRARY
}



/// <summary>
/// Test the stld1553 library in the henceforth described scenario:
/// Participant[s]: RT1, BC2.
/// Configuration: All participating devices must be able to communicate with each other.
/// Short description: Acceptance Test Plan (ATP) test 461_13c - RT features: c) Interrupts on individual mode codes.
/// Along test print suitable info and indications into the log.
///
/// Note:
/// - This function may use any of above defined Acceptance Test Plan (ATP) tests variables.
///   The caller must therefore call the suitable stldTester_Atp_Set* functions prior to calling this function.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Atp_DoTest461_13c ()
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	S16BIT swResult; // Result of operation or function call.
	swResult = sitalReturnCode_SUCCESS;

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_13c.Start\n");

	swResult = stldTester_Atp_DoTest461_11j ();

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_13c.End: test_succeeded - %c\n", ((sitalReturnCode_SUCCESS == swResult) ? 'Y' : 'N'));
	return swResult;
}



/// <summary>
/// Test the stld1553 library in the henceforth described scenario:
/// Participant[s]: RT1, BC2.
/// Configuration: All participating devices must be able to communicate with each other.
/// Special configuration variables: s_dwTestIterationsCount, s_wFrameTime.
/// Short description: Acceptance Test Plan (ATP) test 461_13e - Programmable message content.
/// Along test print suitable info and indications into the log.
///
/// Note:
/// - This function may use any of above defined Acceptance Test Plan (ATP) tests variables.
///   The caller must therefore call the suitable stldTester_Atp_Set* functions prior to calling this function.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Atp_DoTest461_13e ()
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_13e.Start\n");

	/// Set test arguments.
	U16BIT wWordCount; // Word count (1-32).
	wWordCount = 32;
	U16BIT wRtSubaddressTx; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31).
	wRtSubaddressTx = 1;
	U32BIT dwTestIterationsCount; // The number of test iterations to perform.
	dwTestIterationsCount = s_dwTestIterationsCount;
	U16BIT wFrameTime; // The time (in units of 100 us) that the frame will take to complete.
	wFrameTime = s_wFrameTime; // 1000; // 100ms.

	S16BIT swResult; // Result of operation or function call.
	swResult = sitalReturnCode_SUCCESS;
	InterProcessSemaphore ipsSemaphore (s_szRtSemaphoreName, 1); // A signaling object that is used by an assigned ISR function to signal interrupts to waiting threads.

	/// Get device count.
	U16BIT wRequiredDeviceCount; // The number of devices required for this test.
	wRequiredDeviceCount = (1 + max(s_swDevice1, s_swDevice2));
	U16BIT wAvailableDeviceCount; // The total number of devices available in the system.
	swResult = sitalDevice_GetCount (&wAvailableDeviceCount);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %i\n", swResult);
		return swResult;
	}
	sitalProcess_Log_PrintLine ("#StldTester.Device.Count: available - %i, required - %i\n", wAvailableDeviceCount, wRequiredDeviceCount);
	if (wAvailableDeviceCount < wRequiredDeviceCount)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %s\n", "not_enough_devices_installed");
		return sitalReturnCode_SUCCESS;
	}
	if ( (s_swDevice1 == s_swDevice2) )
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Number.Error: error - %s\n", "same_device_used_twice");
		return sitalReturnCode_INVALID_DEVICE_NUMBER;
	}

	try
	{
		swResult = stldSyncStart (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncStart.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-1 (RT) as a real RT in advanced mode.
		// Note: Advanced mode will enable r/w to the memory of device-1 (RT) while the device is running.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-rt, alloc_mem-none\n", s_swDevice1);
		swResult = sitalDevice_Initialize (s_swDevice1, sitalAccess_CARD, ((s_bIsRtMtModeTest ? sitalMode_RT_AND_MT : sitalMode_RT) | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Set device-1 (RT) address to RtAddress1.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set: unit - %i, address - %i\n", s_swDevice1, s_wRtAddress1);
		swResult = sitalRt_Address_Set (s_swDevice1, s_wRtAddress1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-1 (RT) data-block-0, a data block of 32 data words.
		S16BIT swRtDataBlockId; // A unique ID designating a data block (0-(sitalRtCounter_DATA_BLOCKS-1)).
		swRtDataBlockId = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.Create: unit - %i, block - %i, type - %i, buf_size - %i\n", s_swDevice1, swRtDataBlockId, sitalRtDataBlockType_SINGLE_32, 0);
		swResult = sitalRt_DataBlock_Create (s_swDevice1, swRtDataBlockId, sitalRtDataBlockType_SINGLE_32, NULL, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Map for device-1 (RT) data-block-0 with message-type Tx for subaddress-RtSubaddress, legalize this subaddress, and request end-of-message interrupts for it.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.MapToSa: unit - %i, block - %i, sa - %i, msg_types - x%X, irq_opts-x%X, legalize - %c\n", s_swDevice1, swRtDataBlockId, wRtSubaddressTx, sitalRtMessageType_TX, sitalRtDataBlockIrq_END_OF_MESSAGE, 'Y');
		swResult = sitalRt_DataBlock_MapToSubaddress (s_swDevice1, swRtDataBlockId, wRtSubaddressTx, sitalRtMessageType_TX, sitalRtDataBlockIrq_END_OF_MESSAGE, TRUE);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.MapToSa.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Configure device-1 (RT) to get mode-code notifications using an ISR that counts ISR calls.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Manipulate: unit - %i, enable - %i, irq - %08X, callback - %s\n", s_swDevice1, TRUE, sitalInterruptRegister1_RT_MODE_CODE, "stldTester_Rt_UserIsr");
		swResult = sitalDevice_Irq_Manipulate (s_swDevice1, TRUE, sitalInterruptRegister1_RT_SUBADDRESS_END_OF_MESSAGE, stldTester_Rt_UserIsr);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Set.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create a signaling object.
		sitalProcess_Log_PrintLine ("#StldTester.Semaphore.Create: name - %s\n", s_szRtSemaphoreName);
		if (0U != ipsSemaphore.Create ())
		{
			sitalProcess_Log_PrintLine ("#StldTester.Semaphore.Create.Error: error - %i\n", GetLastError_NonOsDependent());
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		///	Start device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Start: unit - %i\n", s_swDevice1);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Start (s_swDevice1) : sitalRt_Start (s_swDevice1) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-2 (BC) as a real BC.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-bc, alloc_mem-none\n", s_swDevice2);
		swResult = sitalDevice_Initialize (s_swDevice2, sitalAccess_CARD, (sitalMode_BC | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		PrecisionTimer ptBenchmarkTimer; // A precision timer.
		ptBenchmarkTimer.SetBaseTime ();

		// Loop over all test iterations:
		for (U32BIT dwIteration=0; (INFINITE_ITERATIONS || (dwIteration<dwTestIterationsCount)); dwIteration++)
		{
			/// Create for device-2 (BC) data-block-0, a data block of 32 data words.
			S16BIT swDataBlockId_0; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
			swDataBlockId_0 = 1;
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i\n", s_swDevice2, swDataBlockId_0, 32, 0);
			swResult = sitalBc_DataBlock_Create (s_swDevice2, swDataBlockId_0, sitalBcDataBlockSizeOption_SINGLE, NULL, 0);
			if (0 > swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Create for device-2 (BC) message-0, a RT(RT-address-1/RtSubAddressTx)-to_BC message that uses data-block-0 to receive WordCount data words. Message bus: bus-1.
			S16BIT swMessageId_0; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
			swMessageId_0 = 0;
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBc.Create: unit - %i, message - %i, block - %i, tx_rt - %i, tx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddressTx, wWordCount, 0, s_wBus1);
			swResult = sitalBc_Message_CreateRtToBc (s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddressTx, wWordCount, 0, s_wBus1);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBc.Create.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Create for device-2 (BC) command-0, a XEQ command that executes message-0.
			S16BIT swCommandId_0; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
			swCommandId_0 = 0;
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0);
			swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0, 0);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Create for device-2 (BC) command-1, a subroutine call command that executes minor-frame-0.
			S16BIT swCommandId_1; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
			swCommandId_1 = 1;
			S16BIT swFrameId_0; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
			swFrameId_0 = 0;
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_1, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, swFrameId_0, 0);
			swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_1, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, (U16BIT)swFrameId_0, 0, 0);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Create for device-2 (BC) minor-frame-0, a frame that contains command-0.
			S16BIT swaCommandIds[sitalBcCounter_COMMANDS]; // An array of command-IDs.
			swaCommandIds[0] = swCommandId_0;
			U16BIT wCommandIdCount; // The number of command IDs in the frame (1-sitalBcCounter_FRAME_COMMANDS).
			wCommandIdCount = 1;
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_0, "minor", wCommandIdCount, swCommandId_0, 0, 0);
			swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_0, sitalBcFrameType_MINOR, swaCommandIds, wCommandIdCount, 0, 0);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Create for device-2 (BC) major-frame-1, a frame that contains command-1.
			S16BIT swFrameId_1; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
			swFrameId_1 = 1;
			swaCommandIds[0] = swCommandId_1;
			wCommandIdCount = 1;
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_1, "major", wCommandIdCount, swCommandId_1, wFrameTime, 0);
			swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_1, sitalBcFrameType_MAJOR, swaCommandIds, wCommandIdCount, wFrameTime, 0);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
				throw 0;
			}

			///		Fill device-1 (RT) data-block-0 with current ascending data value.
			U16BIT wAscendingDataValue; // An ascending data value.
			wAscendingDataValue = (U32BIT)dwIteration;
			U16BIT waBuffer[32]; // A buffer to read-from/write-into along data-block write/read operations, respectively.
			for (int i=0; i<wWordCount; i++) waBuffer[i] = wAscendingDataValue;
			sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.Write: unit - %i, block - %i, size - %i, buf_size - %i (%04X...)\n", s_swDevice1, swRtDataBlockId, 32, wWordCount, waBuffer[0]);
			swResult = sitalRt_DataBlock_Write (s_swDevice1, swRtDataBlockId, waBuffer, wWordCount, 0);
			if (0 > swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.Write.Error: error - %i\n", swResult);
				throw 0;
			}

			///		Start device-2 (BC) running major-frame-1 for 1-shot.
			U16BIT wFrameCount; // The number of frames to run.
			wFrameCount = 1;
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", s_swDevice2, swFrameId_1, wFrameCount);
			swResult = sitalBc_Start (s_swDevice2, swFrameId_1, wFrameCount);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
				throw 0;
			}

			///		Wait for the currently transmitted message to end.
			U16BIT wActualFrameTime; // The time (in units of 100 us) that the actual transmission of all the messages in the frame will take to complete.
			wActualFrameTime = 10; // = 1 ms; Here we have a single RT-to-BC message with 32 data words in the frame, which takes ~700 us to transmit.
			U32BIT dwSleepTime; // Sleeping period (in ms).
			dwSleepTime = ( (wFrameCount * (( (0U == wFrameTime) ? wActualFrameTime : wFrameTime ) / 10)) + stldInterruptLatency_Get () );
			if (0U != ipsSemaphore.Catch (dwSleepTime)) 
			{ 
				sitalProcess_Log_PrintLine ("#StldTester.Semaphore.Wait.Error: error - %s\n", "timeout");
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}
			sitalProcess_Log_PrintLine ("#StldTester.Wait: time - %ims\n", dwSleepTime);
			sitalProcess_Log_PrintLine ("\tWAIT_FOR %i us\n", (dwSleepTime * 1000));

			///		Wait for the frame run to end.
			U32BIT dwRemainingFrameRunTime; // The expected remaining frame run time (in ms).
			dwRemainingFrameRunTime = (wFrameCount * (( (0U == wFrameTime) ? wActualFrameTime : wFrameTime ) / 10)); // Total frame run time; less time has actually left.
			swResult = stldTester_Bc_WaitForEndOfFrameRun (s_swDevice2, dwRemainingFrameRunTime, 0, NULL);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.WaitForEndOfFrameRun.Error: error - %i\n", swResult);
				throw 0;
			}

			///		Stop device-2 (BC).
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop: unit - %i\n", s_swDevice2);
			swResult = sitalBc_Stop (s_swDevice2);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
				throw 0;
			}

			///		Get for device-2 (BC) the data in the buffer of data-block-0.
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Read: unit - %i, block - %i, buf_size - %i, offset - %i\n", s_swDevice2, swDataBlockId_0, wWordCount, 0);
			swResult = sitalBc_DataBlock_Read (s_swDevice2, swDataBlockId_0, waBuffer, wWordCount, 0);
			if (0 > swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Read.Error: error - %i\n", swResult);
				throw 0;
			}

			///		Verify that device-2 (BC) received the expected data.
			BOOLEAN bIsExpectedData; // A flag that says whether received data is the expected data, or not.
			bIsExpectedData = (wAscendingDataValue == waBuffer[0]);
			sitalProcess_Log_PrintLine ("#StldTester.Data.Compare: iteration - %i, expected_data - x%04X, received_data - x%04X, data_okay - %i\n", dwIteration, wAscendingDataValue, waBuffer[0], bIsExpectedData);
			if (FALSE == bIsExpectedData)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Data.Compare.Error: error - %s\n", "bad_data");
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}

			/// Delete device-2 (BC) frame-1.
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Delete: unit - %i, frame - %i\n", s_swDevice2, swFrameId_1);
			swResult = sitalBc_Frame_Delete (s_swDevice2, swFrameId_1);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Delete.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Delete device-2 (BC) frame-0.
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Delete: unit - %i, frame - %i\n", s_swDevice2, swFrameId_0);
			swResult = sitalBc_Frame_Delete (s_swDevice2, swFrameId_0);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Delete.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Delete device-2 (BC) command-0.
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Delete: unit - %i, command - %i\n", s_swDevice2, swCommandId_0);
			swResult = sitalBc_Command_Delete (s_swDevice2, swCommandId_0);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Delete.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Delete device-2 (BC) command-1.
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Delete: unit - %i, command - %i\n", s_swDevice2, swCommandId_1);
			swResult = sitalBc_Command_Delete (s_swDevice2, swCommandId_1);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Delete.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Delete device-2 (BC) message-0.
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.Delete: unit - %i, frame - %i\n", s_swDevice2, swMessageId_0);
			swResult = sitalBc_Message_Delete (s_swDevice2, swMessageId_0);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.Delete.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Delete device-2 (BC) data-block-0.
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Delete: unit - %i, block - %i\n", s_swDevice2, swDataBlockId_0);
			swResult = sitalBc_DataBlock_Delete (s_swDevice2, swDataBlockId_0);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Delete.Error: error - %i\n", swResult);
				throw 0;
			}
		}

		ptBenchmarkTimer.SetEndTime ();
		double dOverallFrameRunDuration = ptBenchmarkTimer.GetDuration ();
		double dAverageActualFrameTime = ( dOverallFrameRunDuration / (double)dwTestIterationsCount );
		sitalProcess_Log_PrintLine ("#StldTester.Benchmark: iterations - %i, total_time - %.0f ms, average_frame - %.2f ms\n", dwTestIterationsCount, dOverallFrameRunDuration, dAverageActualFrameTime);

		///	Stop device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Stop: unit - %i\n", s_swDevice1);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Stop (s_swDevice1) : sitalRt_Stop (s_swDevice1) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice1);
		swResult = sitalDevice_Free (s_swDevice1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice2);
		swResult = sitalDevice_Free (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Close the signaling object.
		sitalProcess_Log_PrintLine ("#StldTester.Semaphore.Close: name - %s\n", s_szRtSemaphoreName);
		if (0U != ipsSemaphore.Close ())
		{
			sitalProcess_Log_PrintLine ("#StldTester.Semaphore.Close.Error: error - %i\n", GetLastError_NonOsDependent());
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		if (0U != s_ipspRtSemaphore->Close ())
		{
			sitalProcess_Log_PrintLine ("#StldTester.Semaphore.Callback.Close.Error: error - %i\n", GetLastError_NonOsDependent());
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		delete (s_ipspRtSemaphore);
		s_ipspRtSemaphore = NULL;

		swResult = stldSyncEnd (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncEnd.Error: error - %i\n", swResult);
			throw 0;
		}
	}
	catch (int)
	{
		sitalDevice_Free (s_swDevice1);
		sitalDevice_Free (s_swDevice2);
		ipsSemaphore.Close ();
		if (NULL != s_ipspRtSemaphore)
		{
			s_ipspRtSemaphore->Close ();
			delete (s_ipspRtSemaphore);
			s_ipspRtSemaphore = NULL;
		}
	}

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_13e.End: test_succeeded - %c\n", ((sitalReturnCode_SUCCESS == swResult) ? 'Y' : 'N'));
	return swResult;
}



/// <summary>
/// Test the stld1553 library in the henceforth described scenario:
/// Participant[s]: BC2.
/// External participants: A RT device that responds RT-to-BC messages that BC2 transmits on bus A with cyclically incremented data words.
/// Configuration: All participating devices must be able to communicate with each other.
/// Special configuration variables: s_dwTestIterationsCount, s_wFrameTime.
/// Short description: Acceptance Test Plan (ATP) test 461_13e_ExternalRt - Local BC properly getting message content being transmitted from an external RT.
/// Along test print suitable info and indications into the log.
///
/// Note:
/// - This function may use any of above defined Acceptance Test Plan (ATP) tests variables.
///   The caller must therefore call the suitable stldTester_Atp_Set* functions prior to calling this function.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Atp_DoTest461_13e_ExternRt ()
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_13e_ExternRt.Start\n");

	/// Set test arguments.
	U16BIT wWordCount; // Word count (1-32).
	wWordCount = 32;
	U16BIT wRtSubaddressTx; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31).
	wRtSubaddressTx = 1;
	U32BIT dwTestIterationsCount; // The number of test iterations to perform.
	dwTestIterationsCount = s_dwTestIterationsCount;
	U16BIT wFrameTime; // The time (in units of 100 us) that the frame will take to complete.
	wFrameTime = s_wFrameTime; // 1000; // 100ms.

	S16BIT swResult; // Result of operation or function call.
	swResult = sitalReturnCode_SUCCESS;
	InterProcessSemaphore ipsSemaphore (s_szBcSemaphoreName, 1); // A signaling object that is used by an assigned ISR function to signal interrupts to waiting threads.

	/// Get device count.
	U16BIT wRequiredDeviceCount; // The number of devices required for this test.
	wRequiredDeviceCount = 1;
	U16BIT wAvailableDeviceCount; // The total number of devices available in the system.
	swResult = sitalDevice_GetCount (&wAvailableDeviceCount);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %i\n", swResult);
		return swResult;
	}
	sitalProcess_Log_PrintLine ("#StldTester.Device.Count: available - %i, required - %i\n", wAvailableDeviceCount, wRequiredDeviceCount);
	if (wAvailableDeviceCount < wRequiredDeviceCount)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %s\n", "not_enough_devices_installed");
		return sitalReturnCode_SUCCESS;
	}

	try
	{
		swResult = stldSyncStart (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncStart.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create a signaling object.
		sitalProcess_Log_PrintLine ("#StldTester.Semaphore.Create: name - %s\n", s_szBcSemaphoreName);
		if (0U != ipsSemaphore.Create ())
		{
			sitalProcess_Log_PrintLine ("#StldTester.Semaphore.Create.Error: error - %i\n", GetLastError_NonOsDependent());
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// Initialize device-2 (BC) as a real BC.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-bc, alloc_mem-none\n", s_swDevice2);
		swResult = sitalDevice_Initialize (s_swDevice2, sitalAccess_CARD, (sitalMode_BC | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Configure device-2 (BC) to get IRQ-3 notifications using an ISR that collects interrupt data, counts ISR calls, and signals interested threads.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Manipulate: unit - %i, enable - %i, irq - %08X, callback - %s\n", s_swDevice2, TRUE, sitalInterruptRegister2_BC_IRQ3, "stldTester_Bc_UserIsr");
		swResult = sitalDevice_Irq_Manipulate (s_swDevice2, TRUE, sitalInterruptRegister2_BC_IRQ3, stldTester_Bc_UserIsr);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Set.Error: error - %i\n", swResult);
			throw 0;
		}
		s_dwBcIsrCallCounter = 0;
		s_dwBcIrqStatus = sitalInterruptRegister2_BC_IRQ3; // So that below conditions won't fail the test right before the first frame is transmitted.

		U16BIT waBuffer[32]; // A buffer to read-from/write-into along data-block write/read operations, respectively.
		U16BIT wAscendingDataValue; // An ascending data value.
		PrecisionTimer ptBenchmarkTimer; // A precision timer.
		ptBenchmarkTimer.SetBaseTime ();

		// Loop over all test iterations:
		for (U32BIT dwIteration=0; (INFINITE_ITERATIONS || (dwIteration<dwTestIterationsCount)); dwIteration++)
		{
			///		Create for device-2 (BC) data-block-0, a data block of 32 data words.
			S16BIT swDataBlockId_0; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
			swDataBlockId_0 = 1;
#ifndef CRITICAL_TIMING
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i\n", s_swDevice2, swDataBlockId_0, 32, 0);
#endif // CRITICAL_TIMING
			swResult = sitalBc_DataBlock_Create (s_swDevice2, swDataBlockId_0, sitalBcDataBlockSizeOption_SINGLE, NULL, 0);
			if (0 > swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
				throw 0;
			}

			///		Create for device-2 (BC) message-0, a RT(RT-address-1/RtSubAddressTx)-to_BC message that uses data-block-0 to receive WordCount data words. Message bus: bus-1.
			S16BIT swMessageId_0; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
			swMessageId_0 = 0;
#ifndef CRITICAL_TIMING
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBc.Create: unit - %i, message - %i, block - %i, tx_rt - %i, tx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddressTx, wWordCount, 0, s_wBus1);
#endif // CRITICAL_TIMING
			swResult = sitalBc_Message_CreateRtToBc (s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddressTx, wWordCount, 0, s_wBus1);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBc.Create.Error: error - %i\n", swResult);
				throw 0;
			}

			///		Create for device-2 (BC) command-0, a XEQ command that executes message-0.
			S16BIT swCommandId_0; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
			swCommandId_0 = 0;
#ifndef CRITICAL_TIMING
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0);
#endif // CRITICAL_TIMING
			swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0, 0);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
				throw 0;
			}

			///		Create for device-2 (BC) command-1, a subroutine call command that executes minor-frame-0.
			S16BIT swCommandId_1; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
			swCommandId_1 = 1;
			S16BIT swFrameId_0; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
			swFrameId_0 = 0;
#ifndef CRITICAL_TIMING
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_1, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, swFrameId_0, 0);
#endif // CRITICAL_TIMING
			swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_1, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, (U16BIT)swFrameId_0, 0, 0);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
				throw 0;
			}

			///		Create for device-2 (BC) minor-frame-0, a frame that contains command-0.
			S16BIT swaCommandIds[sitalBcCounter_COMMANDS]; // An array of command-IDs.
			swaCommandIds[0] = swCommandId_0;
			U16BIT wCommandIdCount; // The number of command IDs in the frame (1-sitalBcCounter_FRAME_COMMANDS).
			wCommandIdCount = 1;
#ifndef CRITICAL_TIMING
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_0, "minor", wCommandIdCount, swCommandId_0, 0, 0);
#endif // CRITICAL_TIMING
			swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_0, sitalBcFrameType_MINOR, swaCommandIds, wCommandIdCount, 0, 0);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
				throw 0;
			}

			///		Create for device-2 (BC) major-frame-1, a frame that contains command-1.
			S16BIT swFrameId_1; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
			swFrameId_1 = 1;
			swaCommandIds[0] = swCommandId_1;
			wCommandIdCount = 1;
#ifndef CRITICAL_TIMING
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_1, "major", wCommandIdCount, swCommandId_1, wFrameTime, 0);
#endif // CRITICAL_TIMING
			swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_1, sitalBcFrameType_MAJOR, swaCommandIds, wCommandIdCount, wFrameTime, 0);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
				throw 0;
			}

			///		Start device-2 (BC) running major-frame-1 for 1-shot.
			U16BIT wFrameCount; // The number of frames to run.
			wFrameCount = 1;
#ifndef CRITICAL_TIMING
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", s_swDevice2, swFrameId_1, wFrameCount);
#endif // CRITICAL_TIMING
			swResult = sitalBc_Start (s_swDevice2, swFrameId_1, wFrameCount);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
				throw 0;
			}

			///		Wait for the currently transmitted message to end.
			U16BIT wActualFrameTime; // The time (in units of 100 us) that the actual transmission of all the messages in the frame will take to complete.
			wActualFrameTime = 10; // = 1 ms; Here we have a single RT-to-BC message with 32 data words in the frame, which takes ~700 us to transmit.
			U32BIT dwSleepTime; // Sleeping period (in ms).
			dwSleepTime = ( (wFrameCount * (( (0U == wFrameTime) ? wActualFrameTime : wFrameTime ) / 10)) + stldInterruptLatency_Get () );
			if (0U != ipsSemaphore.Catch (dwSleepTime)) 
			{ 
				sitalProcess_Log_PrintLine ("#StldTester.Semaphore.Wait.Error: error - %s\n", "timeout");
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}
#ifndef CRITICAL_TIMING
			sitalProcess_Log_PrintLine ("#StldTester.Wait: time - %ims\n", dwSleepTime);
			sitalProcess_Log_PrintLine ("\tWAIT_FOR %i us\n", (dwSleepTime * 1000));
#endif // CRITICAL_TIMING

			/// Verify that a correct number of interrupts of a suitable type occurred.
			if (0U == (sitalInterruptRegister2_BC_IRQ3 & s_dwBcIrqStatus))
			{
				sitalProcess_Log_PrintLine ("#StldTester.Device.Interrupt.Mask.Error: error - %08X\n", s_dwBcIrqStatus);
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}

			///		Wait for the frame run to end.
			U32BIT dwRemainingFrameRunTime; // The expected remaining frame run time (in ms).
			dwRemainingFrameRunTime = (wFrameCount * (( (0U == wFrameTime) ? wActualFrameTime : wFrameTime ) / 10)); // Total frame run time; less time has actually left.
			swResult = stldTester_Bc_WaitForEndOfFrameRun (s_swDevice2, dwRemainingFrameRunTime, 0, NULL);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.WaitForEndOfFrameRun.Error: error - %i\n", swResult);
				throw 0;
			}

			///		Stop device-2 (BC).
#ifndef CRITICAL_TIMING
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop: unit - %i\n", s_swDevice2);
#endif // CRITICAL_TIMING
			swResult = sitalBc_Stop (s_swDevice2);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Verify that a response has been received by device-2 (BC) from device-1 (RT) for message-0.
			sitalDecodedMessageStructure dmsDecodedMessage; // A decoded message.
#ifndef CRITICAL_TIMING
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded: unit - %i, message - %i, purge-y\n", s_swDevice2, swMessageId_0);
#endif // CRITICAL_TIMING
			swResult = sitalBc_Message_GetByIdDecoded (s_swDevice2, swMessageId_0, &dmsDecodedMessage, TRUE);
			if (0 > swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded.Error: error - %i\n", swResult);
				throw 0;
			}
			if (1 != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded.Error: error - %i\n", swResult);
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}
#ifndef CRITICAL_TIMING
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message: message - %i, bsw - %04X\n", swMessageId_0, dmsDecodedMessage.wBlockStatus);
#endif // CRITICAL_TIMING
			if (sitalBcBlockStatusWord_NO_RESPONSE == (sitalBcBlockStatusWord_NO_RESPONSE & dmsDecodedMessage.wBlockStatus))
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BlockStatus.Error: error - %s\n", "bad_bsw");
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}

			///		Get for device-2 (BC) the data in the buffer of data-block-0.
#ifndef CRITICAL_TIMING
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Read: unit - %i, block - %i, buf_size - %i, offset - %i\n", s_swDevice2, swDataBlockId_0, wWordCount, 0);
#endif // CRITICAL_TIMING
			swResult = sitalBc_DataBlock_Read (s_swDevice2, swDataBlockId_0, waBuffer, wWordCount, 0);
			if (0 > swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Read.Error: error - %i\n", swResult);
				throw 0;
			}
			if (0U == dwIteration)	wAscendingDataValue = waBuffer[0]; // This test uses an external RT which responds BC-to-RT messages with incremental data. As its starter data word isn't known from advance, it is taken from the actual transmission.

			///		Verify that device-2 (BC) received the expected data.
			BOOLEAN bIsExpectedData; // A flag that says whether received data is the expected data, or not.
			bIsExpectedData = TRUE;
			U16BIT* wpCurrentlyCheckedDataWord; // A pointer to the received data word that is now being checked.
			wpCurrentlyCheckedDataWord = waBuffer;
			U16BIT wExpectedStarterData; // The value expected for the first data word in the now checked message.
			wExpectedStarterData = wAscendingDataValue;
			int iDataWordIndex; // Index of checked data word.
			for (iDataWordIndex=0; iDataWordIndex<32; iDataWordIndex++, wpCurrentlyCheckedDataWord++, wAscendingDataValue++)
			{
				if (wAscendingDataValue != (*wpCurrentlyCheckedDataWord))
				{
					bIsExpectedData = FALSE;
					break;
				}
			}
#ifndef CRITICAL_TIMING
			sitalProcess_Log_PrintLine ("#StldTester.Data.Compare: iteration - %i, expected_starter_data - x%04X, starter_data - x%04X, data_okay - %i\n", dwIteration, wExpectedStarterData, waBuffer[0], bIsExpectedData);
#endif // CRITICAL_TIMING
			if (FALSE == bIsExpectedData)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Data.Compare: iteration - %i, data_index - %i, expected_data - x%04X, received_data - x%04X, data_okay - %i\n", dwIteration, iDataWordIndex, wAscendingDataValue, (*wpCurrentlyCheckedDataWord), bIsExpectedData);
				sitalProcess_Log_PrintLine ("#StldTester.Data.Compare.Error: error - %s\n", "bad_data");
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}

			/// Delete device-2 (BC) frame-1.
#ifndef CRITICAL_TIMING
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Delete: unit - %i, frame - %i\n", s_swDevice2, swFrameId_1);
#endif // CRITICAL_TIMING
			swResult = sitalBc_Frame_Delete (s_swDevice2, swFrameId_1);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Delete.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Delete device-2 (BC) frame-0.
#ifndef CRITICAL_TIMING
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Delete: unit - %i, frame - %i\n", s_swDevice2, swFrameId_0);
#endif // CRITICAL_TIMING
			swResult = sitalBc_Frame_Delete (s_swDevice2, swFrameId_0);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Delete.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Delete device-2 (BC) command-0.
#ifndef CRITICAL_TIMING
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Delete: unit - %i, command - %i\n", s_swDevice2, swCommandId_0);
#endif // CRITICAL_TIMING
			swResult = sitalBc_Command_Delete (s_swDevice2, swCommandId_0);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Delete.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Delete device-2 (BC) command-1.
#ifndef CRITICAL_TIMING
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Delete: unit - %i, command - %i\n", s_swDevice2, swCommandId_1);
#endif // CRITICAL_TIMING
			swResult = sitalBc_Command_Delete (s_swDevice2, swCommandId_1);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Delete.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Delete device-2 (BC) message-0.
#ifndef CRITICAL_TIMING
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.Delete: unit - %i, frame - %i\n", s_swDevice2, swMessageId_0);
#endif // CRITICAL_TIMING
			swResult = sitalBc_Message_Delete (s_swDevice2, swMessageId_0);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.Delete.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Delete device-2 (BC) data-block-0.
#ifndef CRITICAL_TIMING
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Delete: unit - %i, block - %i\n", s_swDevice2, swDataBlockId_0);
#endif // CRITICAL_TIMING
			swResult = sitalBc_DataBlock_Delete (s_swDevice2, swDataBlockId_0);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Delete.Error: error - %i\n", swResult);
				throw 0;
			}
		}

		ptBenchmarkTimer.SetEndTime ();
		double dOverallFrameRunDuration = ptBenchmarkTimer.GetDuration ();
		double dAverageActualFrameTime = ( dOverallFrameRunDuration / (double)dwTestIterationsCount );
		sitalProcess_Log_PrintLine ("#StldTester.Benchmark: iterations - %i, total_time - %.0f ms, average_frame - %.2f ms\n", dwTestIterationsCount, dOverallFrameRunDuration, dAverageActualFrameTime);

		/// Free device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice2);
		swResult = sitalDevice_Free (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Close the signaling object.
		sitalProcess_Log_PrintLine ("#StldTester.Semaphore.Close: name - %s\n", s_szBcSemaphoreName);
		if (0U != ipsSemaphore.Close ())
		{
			sitalProcess_Log_PrintLine ("#StldTester.Semaphore.Close.Error: error - %i\n", GetLastError_NonOsDependent());
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		if (0U != s_ipspBcSemaphore->Close ())
		{
			sitalProcess_Log_PrintLine ("#StldTester.Semaphore.Callback.Close.Error: error - %i\n", GetLastError_NonOsDependent());
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		delete (s_ipspBcSemaphore);
		s_ipspBcSemaphore = NULL;

		swResult = stldSyncEnd (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncEnd.Error: error - %i\n", swResult);
			throw 0;
		}
	}
	catch (int)
	{
		sitalDevice_Free (s_swDevice2);
		ipsSemaphore.Close ();
		if (NULL != s_ipspBcSemaphore)
		{
			s_ipspBcSemaphore->Close ();
			delete (s_ipspBcSemaphore);
			s_ipspRtSemaphore = NULL;
		}
	}

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_13e_ExternRt.End: test_succeeded - %c\n", ((sitalReturnCode_SUCCESS == swResult) ? 'Y' : 'N'));
	return swResult;
}



/// <summary>
/// Test the stld1553 library in the henceforth described scenario:
/// Participant[s]: RT1, BC2.
/// Configuration: All participating devices must be able to communicate with each other.
/// Short description: Acceptance Test Plan (ATP) test 461_13f - Multiple store buffers with 16 bit time tag.
/// Along test print suitable info and indications into the log.
///
/// Note:
/// - This function may use any of above defined Acceptance Test Plan (ATP) tests variables.
///   The caller must therefore call the suitable stldTester_Atp_Set* functions prior to calling this function.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Atp_DoTest461_13f ()
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	S16BIT swResult; // Result of operation or function call.
	swResult = sitalReturnCode_SUCCESS;

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_13f.Start\n");

	swResult = stldTester_Atp_DoTest461_13b ();

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_13f.End: test_succeeded - %c\n", ((sitalReturnCode_SUCCESS == swResult) ? 'Y' : 'N'));
	return swResult;
}



/// <summary>
/// Test the stld1553 library in the henceforth described scenario:
/// Participant[s]: RT1, BC2.
/// Configuration: All participating devices must be able to communicate with each other.
/// Short description: Acceptance Test Plan (ATP) test 461_13g - Programmable sub-address selection - verify transmit SA can swap buffers.
/// Along test print suitable info and indications into the log.
///
/// Note:
/// - This function may use any of above defined Acceptance Test Plan (ATP) tests variables.
///   The caller must therefore call the suitable stldTester_Atp_Set* functions prior to calling this function.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Atp_DoTest461_13g ()
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_13g.Start\n");

	/// Set test arguments.
	U16BIT wWordCount; // Word count (1-32).
	wWordCount = 32;
	U16BIT wRtSubaddress; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31).
	wRtSubaddress = sitalRtSubaddress_1;
	U16BIT wFrameTime; // The time (in units of 100 us) that the frame will take to complete.
	wFrameTime = 1000; // 100ms.
	U16BIT wFrameCount; // The number of frames to run.
	wFrameCount = 10;

	S16BIT swResult; // Result of operation or function call.
	swResult = sitalReturnCode_SUCCESS;
	InterProcessSemaphore ipsSemaphore (s_szRtSemaphoreName, 1); // A signaling object that is used by an assigned ISR function to signal interrupts to waiting threads.

	/// Get device count.
	U16BIT wRequiredDeviceCount; // The number of devices required for this test.
	wRequiredDeviceCount = (1 + max(s_swDevice1, s_swDevice2));
	U16BIT wAvailableDeviceCount; // The total number of devices available in the system.
	swResult = sitalDevice_GetCount (&wAvailableDeviceCount);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %i\n", swResult);
		return swResult;
	}
	sitalProcess_Log_PrintLine ("#StldTester.Device.Count: available - %i, required - %i\n", wAvailableDeviceCount, wRequiredDeviceCount);
	if (wAvailableDeviceCount < wRequiredDeviceCount)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %s\n", "not_enough_devices_installed");
		return sitalReturnCode_SUCCESS;
	}
	if ( (s_swDevice1 == s_swDevice2) )
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Number.Error: error - %s\n", "same_device_used_twice");
		return sitalReturnCode_INVALID_DEVICE_NUMBER;
	}

	try
	{
		swResult = stldSyncStart (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncStart.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create a signaling object.
		sitalProcess_Log_PrintLine ("#StldTester.Semaphore.Create: name - %s\n", s_szRtSemaphoreName);
		if (0U != ipsSemaphore.Create ())
		{
			sitalProcess_Log_PrintLine ("#StldTester.Semaphore.Create.Error: error - %i\n", GetLastError_NonOsDependent());
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// Initialize device-1 (RT) as a real RT in advanced mode.
		// Note: Advanced mode will enable r/w to the memory of device-1 (RT) while the device is running.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-rt, alloc_mem-none\n", s_swDevice1);
		swResult = sitalDevice_Initialize (s_swDevice1, sitalAccess_CARD, ((s_bIsRtMtModeTest ? sitalMode_RT_AND_MT : sitalMode_RT) | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Set device-1 (RT) address to RtAddress1.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set: unit - %i, address - %i\n", s_swDevice1, s_wRtAddress1);
		swResult = sitalRt_Address_Set (s_swDevice1, s_wRtAddress1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Configure device-1 (RT) to get subaddress end-of-message notifications using an ISR that counts ISR calls, and signals interested threads.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Manipulate: unit - %i, enable - %i, irq - %08X, callback - %s\n", s_swDevice1, TRUE, sitalInterruptRegister1_RT_MODE_CODE, "stldTester_Rt_UserIsr");
		swResult = sitalDevice_Irq_Manipulate (s_swDevice1, TRUE, sitalInterruptRegister1_RT_SUBADDRESS_END_OF_MESSAGE, stldTester_Rt_UserIsr);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Set.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Loop to create RT data blocks for all requested swap iterations:
		// Note: This cannot be performed within the test iteration loop itself, as function sitalRt_DataBlock_Create may not be called while the target RT is running.
		S16BIT swRtDataBlockSwapIterationIndex; // The index of the current RT data block swap iteration.
		for (swRtDataBlockSwapIterationIndex=0; swRtDataBlockSwapIterationIndex<wFrameCount; swRtDataBlockSwapIterationIndex++)
		{
			S16BIT swRtDataBlockId; // A unique ID designating a data block (0-(sitalRtCounter_DATA_BLOCKS-1)).
			swRtDataBlockId = swRtDataBlockSwapIterationIndex;

			/// 	Create for device-1 (RT) data-block-DataBlockId, a data block of 32 data words, and fill it with ascending values.
			U16BIT wAscendingDataValue; // An ascending data value.
			wAscendingDataValue = swRtDataBlockSwapIterationIndex;
			U16BIT waBuffer[32]; // A buffer to read-from/write-into along data-block write/read operations, respectively.
			for (int i=0; i<wWordCount; i++) waBuffer[i] = wAscendingDataValue;
			sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.Create: unit - %i, block - %i, type - %i, buf_size - %i, buf_contents-%i...\n", s_swDevice1, swRtDataBlockId, sitalRtDataBlockType_SINGLE_32, wWordCount, wAscendingDataValue);
			swResult = sitalRt_DataBlock_Create (s_swDevice1, swRtDataBlockId, sitalRtDataBlockType_SINGLE_32, waBuffer, wWordCount);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.Create.Error: error - %i\n", swResult);
				throw 0;
			}
		}

		/// Start device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Start: unit - %i\n", s_swDevice1);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Start (s_swDevice1) : sitalRt_Start (s_swDevice1) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-2 (BC) as a real BC.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-bc, alloc_mem-none\n", s_swDevice2);
		swResult = sitalDevice_Initialize (s_swDevice2, sitalAccess_CARD, (sitalMode_BC | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) data-block-0, a data block of 32 data words.
		S16BIT swDataBlockId_0; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId_0 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i\n", s_swDevice2, swDataBlockId_0, 32, 0);
		swResult = sitalBc_DataBlock_Create (s_swDevice2, swDataBlockId_0, sitalBcDataBlockSizeOption_SINGLE, NULL, 0);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) message-0, a RT(RtAddress1/RtSubAddress)-to-BC message that uses data-block-0 to receive WordCount data words. Message bus: bus-1.
		S16BIT swMessageId_0; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBc.Create: unit - %i, message - %i, block - %i, tx_rt - %i, tx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddress, wWordCount, 0, s_wBus1);
		swResult = sitalBc_Message_CreateRtToBc (s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddress, wWordCount, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBc.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-0, a XEQ command that executes message-0.
		S16BIT swCommandId_0; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-1, a subroutine call command that executes minor-frame-0.
		S16BIT swCommandId_1; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_1 = 1;
		S16BIT swFrameId_0; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_1, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, swFrameId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_1, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, (U16BIT)swFrameId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) minor-frame-0, a frame that contains command-0.
		S16BIT swaCommandIds[sitalBcCounter_COMMANDS]; // An array of command-IDs.
		swaCommandIds[0] = swCommandId_0;
		U16BIT wCommandIdCount; // The number of command IDs in the frame (1-sitalBcCounter_FRAME_COMMANDS).
		wCommandIdCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_0, "minor", wCommandIdCount, swCommandId_0, 0, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_0, sitalBcFrameType_MINOR, swaCommandIds, wCommandIdCount, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) major-frame-1, a frame that contains command-1 and whose time is FrameTime.
		S16BIT swFrameId_1; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_1 = 1;
		swaCommandIds[0] = swCommandId_1;
		wCommandIdCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_1, "major", wCommandIdCount, swCommandId_1, wFrameTime, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_1, sitalBcFrameType_MAJOR, swaCommandIds, wCommandIdCount, wFrameTime, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Loop over all RT data block swap iterations:
		for (swRtDataBlockSwapIterationIndex=0; swRtDataBlockSwapIterationIndex<wFrameCount; swRtDataBlockSwapIterationIndex++)
		{
			S16BIT swRtDataBlockId; // A unique ID designating a data block (0-(sitalRtCounter_DATA_BLOCKS-1)).
			swRtDataBlockId = swRtDataBlockSwapIterationIndex;

			/// 	Map for device-1 (RT) data-block-DataBlockId with message-type Tx for subaddress-RtSubaddress, enable end-of-message interrupts for this subaddress, and legalize this subaddress.
			sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.MapToSa: unit - %i, block - %i, sa - %i, msg_types - x%X, irq_opts-x%X, legalize - %c\n", s_swDevice1, swRtDataBlockId, wRtSubaddress, sitalRtMessageType_TX, sitalRtDataBlockIrq_END_OF_MESSAGE, 'Y');
			swResult = sitalRt_DataBlock_MapToSubaddress (s_swDevice1, swRtDataBlockId, wRtSubaddress, sitalRtMessageType_TX, sitalRtDataBlockIrq_END_OF_MESSAGE, TRUE);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.MapToSa.Error: error - %i\n", swResult);
				throw 0;
			}

			///		If the test loop just began its first iteration:
			///			Start device-2 (BC) running major-frame-1 for FrameCount-shots.
			if (0U == swRtDataBlockSwapIterationIndex)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", s_swDevice2, swFrameId_1, wFrameCount);
				swResult = sitalBc_Start (s_swDevice2, swFrameId_1, wFrameCount);
				if (sitalReturnCode_SUCCESS != swResult)
				{
					sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
					throw 0;
				}
			}

			///		Wait for the currently transmitted message (which is the only message in the whole of the transmitted frame) to end.
			U32BIT dwSleepTime; // Sleeping period (in ms).
			dwSleepTime = ( (1 *  ((wFrameTime / IP_FREQUENCY_FACTOR) / 10)) + stldInterruptLatency_Get () );
			if (0U != ipsSemaphore.Catch (dwSleepTime)) 
			{ 
				sitalProcess_Log_PrintLine ("#StldTester.Semaphore.Wait.Error: error - %s\n", "timeout");
				swResult = sitalReturnCode_TEST_FAIL;
				printf("\tDepends on non RTOS thread pool resources -> sleep command failed and timed out");
				throw 0;
			}
			sitalProcess_Log_PrintLine ("#StldTester.Wait: time - %ims\n", dwSleepTime);
			sitalProcess_Log_PrintLine ("\tWAIT_FOR %i us\n", (dwSleepTime * 1000));

			///		Get for device-2 (BC) the data in the buffer of data-block-0.
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Read: unit - %i, block - %i, buf_size - %i, offset - %i\n", s_swDevice2, swDataBlockId_0, wWordCount, 0);
			U16BIT waBuffer[32]; // A buffer to read-from/write-into along data-block write/read operations, respectively.
			swResult = sitalBc_DataBlock_Read (s_swDevice2, swDataBlockId_0, waBuffer, wWordCount, 0);
			if (0 > swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Read.Error: error - %i\n", swResult);
				throw 0;
			}

			///		Verify that device-2 (BC) received the expected data.
			U16BIT wAscendingDataValue; // An ascending data value.
			wAscendingDataValue = swRtDataBlockSwapIterationIndex;
			BOOLEAN bIsExpectedData; // A flag that says whether received data is the expected data, or not.
			bIsExpectedData = (wAscendingDataValue == waBuffer[0]);
			sitalProcess_Log_PrintLine ("#StldTester.Data.Compare: expected_data - %i, received_data - %i, data_okay - %i\n", wAscendingDataValue, waBuffer[0], bIsExpectedData);
			if (FALSE == bIsExpectedData)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Data.Compare.Error: error - %s\n", "bad_data");
				swResult = sitalReturnCode_TEST_FAIL -1;
				printf("\texpected_data - %i, received_data - %i => ", wAscendingDataValue, waBuffer[0]);
				throw 0;
			}

			/// 	Unmap for device-1 (RT) data-block-DataBlockId with message-types Rx and Tx for subaddress-RtSubaddress.
			sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.UnMapToSa: unit - %i, block - %i, sa - %i, msg_types - x%X\n", s_swDevice1, swRtDataBlockId, wRtSubaddress, sitalRtMessageType_TX);
			swResult = sitalRt_DataBlock_UnmapFromSubaddress (s_swDevice1, swRtDataBlockId, wRtSubaddress, sitalRtMessageType_TX);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.UnMapToSa.Error: error - %i\n", swResult);
				throw 0;
			}
		}

		/// Wait for the frame run to end.
		U32BIT dwRemainingFrameRunTime; // The expected remaining frame run time (in ms).
		dwRemainingFrameRunTime = 10000; // The maximum time to wait.
		swResult = stldTester_Bc_WaitForEndOfFrameRun (s_swDevice2, dwRemainingFrameRunTime, 0, NULL);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.WaitForEndOfFrameRun.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Stop device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop: unit - %i\n", s_swDevice2);
		swResult = sitalBc_Stop (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		///		Stop device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Stop: unit - %i\n", s_swDevice1);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Stop (s_swDevice1) : sitalRt_Stop (s_swDevice1) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice1);
		swResult = sitalDevice_Free (s_swDevice1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice2);
		swResult = sitalDevice_Free (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Close the signaling object.
		sitalProcess_Log_PrintLine ("#StldTester.Semaphore.Close: name - %s\n", s_szRtSemaphoreName);
		if (0U != ipsSemaphore.Close ())
		{
			sitalProcess_Log_PrintLine ("#StldTester.Semaphore.Close.Error: error - %i\n", GetLastError_NonOsDependent());
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		if (0U != s_ipspRtSemaphore->Close ())
		{
			sitalProcess_Log_PrintLine ("#StldTester.Semaphore.Callback.Close.Error: error - %i\n", GetLastError_NonOsDependent());
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		delete (s_ipspRtSemaphore);
		s_ipspRtSemaphore = NULL;

		swResult = stldSyncEnd (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncEnd.Error: error - %i\n", swResult);
			throw 0;
		}
	}
	catch (int)
	{
		sitalDevice_Free (s_swDevice1);
		sitalDevice_Free (s_swDevice2);
		ipsSemaphore.Close ();
		if (NULL != s_ipspRtSemaphore)
		{
			s_ipspRtSemaphore->Close ();
			delete (s_ipspRtSemaphore);
			s_ipspRtSemaphore = NULL;
		}
	}

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_13g.End: test_succeeded - %c\n", ((sitalReturnCode_SUCCESS == swResult) ? 'Y' : 'N'));
	return swResult;
}



/// <summary>
/// Test the stld1553 library in the henceforth described scenario:
/// Participant[s]: RT1, BC2.
/// Configuration: All participating devices must be able to communicate with each other.
/// Short description: Acceptance Test Plan (ATP) test 461_13h - RT address setting.
/// Along test print suitable info and indications into the log.
///
/// Note:
/// - This function may use any of above defined Acceptance Test Plan (ATP) tests variables.
///   The caller must therefore call the suitable stldTester_Atp_Set* functions prior to calling this function.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Atp_DoTest461_13h ()
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_13h.Start\n");

	/// Set test arguments.
	U16BIT wWordCount; // Word count (1-32).
	wWordCount = 32;
	U16BIT wRtSubaddress; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31).
	wRtSubaddress = sitalRtSubaddress_1;
	U16BIT wFrameTime; // The time (in units of 100 us) that the frame will take to complete.
	wFrameTime = 1000; // 100ms.

	S16BIT swResult; // Result of operation or function call.
	swResult = sitalReturnCode_SUCCESS;

	/// Get device count.
	U16BIT wRequiredDeviceCount; // The number of devices required for this test.
	wRequiredDeviceCount = (1 + max(s_swDevice1, s_swDevice2));
	U16BIT wAvailableDeviceCount; // The total number of devices available in the system.
	swResult = sitalDevice_GetCount (&wAvailableDeviceCount);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %i\n", swResult);
		return swResult;
	}
	sitalProcess_Log_PrintLine ("#StldTester.Device.Count: available - %i, required - %i\n", wAvailableDeviceCount, wRequiredDeviceCount);
	if (wAvailableDeviceCount < wRequiredDeviceCount)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %s\n", "not_enough_devices_installed");
		return sitalReturnCode_SUCCESS;
	}
	if ( (s_swDevice1 == s_swDevice2) )
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Number.Error: error - %s\n", "same_device_used_twice");
		return sitalReturnCode_INVALID_DEVICE_NUMBER;
	}

	try
	{
		swResult = stldSyncStart (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncStart.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-2 (BC) as a real BC.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-bc, alloc_mem-none\n", s_swDevice2);
		swResult = sitalDevice_Initialize (s_swDevice2, sitalAccess_CARD, (sitalMode_BC | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) data-block-0, a data block of 32 data words.
		S16BIT swDataBlockId_0; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId_0 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i\n", s_swDevice2, swDataBlockId_0, 32, 0);
		swResult = sitalBc_DataBlock_Create (s_swDevice2, swDataBlockId_0, sitalBcDataBlockSizeOption_SINGLE, NULL, 0);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) message-0, a RT(RtAddress1/RtSubAddress)-to-BC message that uses data-block-0 to receive WordCount data words. Message bus: bus-1.
		S16BIT swMessageId_0; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBc.Create: unit - %i, message - %i, block - %i, tx_rt - %i, tx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddress, wWordCount, 0, s_wBus1);
		swResult = sitalBc_Message_CreateRtToBc (s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddress, wWordCount, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBc.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) message-1, a RT(RtAddress2/RtSubAddress)-to-BC message that uses data-block-0 to receive WordCount data words. Message bus: bus-1.
		S16BIT swMessageId_1; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_1 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBc.Create: unit - %i, message - %i, block - %i, tx_rt - %i, tx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_1, swDataBlockId_0, s_wRtAddress2, wRtSubaddress, wWordCount, 0, s_wBus1);
		swResult = sitalBc_Message_CreateRtToBc (s_swDevice2, swMessageId_1, swDataBlockId_0, s_wRtAddress2, wRtSubaddress, wWordCount, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBc.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-0, a XEQ command that executes message-0.
		S16BIT swCommandId_0; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-1, a XEQ command that executes message-1.
		S16BIT swCommandId_1; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_1 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_1, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_1, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_1, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_1, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-2, a subroutine call command that executes minor-frame-0.
		S16BIT swCommandId_2; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_2 = 2;
		S16BIT swFrameId_0; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_2, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, swFrameId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_2, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, (U16BIT)swFrameId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) minor-frame-0, a frame that contains command-0 and command-1.
		S16BIT swaCommandIds[sitalBcCounter_COMMANDS]; // An array of command-IDs.
		swaCommandIds[0] = swCommandId_0;
		swaCommandIds[1] = swCommandId_1;
		U16BIT wCommandIdCount; // The number of command IDs in the frame (1-sitalBcCounter_FRAME_COMMANDS).
		wCommandIdCount = 2;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i,%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_0, "minor", wCommandIdCount, swCommandId_0, swCommandId_1, 0, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_0, sitalBcFrameType_MINOR, swaCommandIds, wCommandIdCount, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) major-frame-1, a frame that contains command-2 and whose time is FrameTime.
		S16BIT swFrameId_1; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_1 = 1;
		swaCommandIds[0] = swCommandId_2;
		wCommandIdCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_1, "major", wCommandIdCount, swCommandId_2, wFrameTime, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_1, sitalBcFrameType_MAJOR, swaCommandIds, wCommandIdCount, wFrameTime, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-1 (RT) as a real RT in advanced mode.
		// Note: Advanced mode will enable r/w to the memory of device-1 (RT) while the device is running.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-rt, alloc_mem-none\n", s_swDevice1);
		swResult = sitalDevice_Initialize (s_swDevice1, sitalAccess_CARD, ((s_bIsRtMtModeTest ? sitalMode_RT_AND_MT : sitalMode_RT) | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Set device-1 (RT) address to RtAddress1.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set: unit - %i, address - %i\n", s_swDevice1, s_wRtAddress1);
		swResult = sitalRt_Address_Set (s_swDevice1, s_wRtAddress1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Start device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Start: unit - %i\n", s_swDevice1);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Start (s_swDevice1) : sitalRt_Start (s_swDevice1) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		///	Start device-2 (BC) running major-frame-1 for 1-shot.
		U16BIT wFrameCount; // The number of frames to run.
		wFrameCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", s_swDevice2, swFrameId_1, wFrameCount);
		swResult = sitalBc_Start (s_swDevice2, swFrameId_1, wFrameCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Wait for the frame run to end.
		U32BIT dwRemainingFrameRunTime; // The expected remaining frame run time (in ms).
		dwRemainingFrameRunTime = (wFrameCount * (wFrameTime / 10));
		swResult = stldTester_Bc_WaitForEndOfFrameRun (s_swDevice2, dwRemainingFrameRunTime, 0, NULL);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.WaitForEndOfFrameRun.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Stop device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop: unit - %i\n", s_swDevice2);
		swResult = sitalBc_Stop (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Verify that a response has been received by device-2 (BC) from device-1 (RT) for message-0.
		sitalDecodedMessageStructure dmsDecodedMessage; // A decoded message.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded: unit - %i, message - %i, purge-y\n", s_swDevice2, swMessageId_0);
		swResult = sitalBc_Message_GetByIdDecoded (s_swDevice2, swMessageId_0, &dmsDecodedMessage, TRUE);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded.Error: error - %i\n", swResult);
			throw 0;
		}
		if (1 != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded.Error: error - %i\n", swResult);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message: message - %i, bsw - %04X\n", swMessageId_0, dmsDecodedMessage.wBlockStatus);
		if (sitalBcBlockStatusWord_NO_RESPONSE == (sitalBcBlockStatusWord_NO_RESPONSE & dmsDecodedMessage.wBlockStatus))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BlockStatus.Error: error - %s\n", "bad_bsw");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// Verify that no response has been received by device-2 (BC) from device-1 (RT) for message-1.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded: unit - %i, message - %i, purge-y\n", s_swDevice2, swMessageId_1);
		swResult = sitalBc_Message_GetByIdDecoded (s_swDevice2, swMessageId_1, &dmsDecodedMessage, TRUE);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded.Error: error - %i\n", swResult);
			throw 0;
		}
		if (1 != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded.Error: error - %i\n", swResult);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message: message - %i, bsw - %04X\n", swMessageId_1, dmsDecodedMessage.wBlockStatus);
		if (sitalBcBlockStatusWord_NO_RESPONSE != (sitalBcBlockStatusWord_NO_RESPONSE & dmsDecodedMessage.wBlockStatus))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BlockStatus.Error: error - %s\n", "bad_bsw");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		///		Stop device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Stop: unit - %i\n", s_swDevice1);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Stop (s_swDevice1) : sitalRt_Stop (s_swDevice1) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Set device-1 (RT) address to RtAddress1.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set: unit - %i, address - %i\n", s_swDevice1, s_wRtAddress1);
		swResult = sitalRt_Address_Set (s_swDevice1, s_wRtAddress2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Start device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Start: unit - %i\n", s_swDevice1);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Start (s_swDevice1) : sitalRt_Start (s_swDevice1) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		///	Start device-2 (BC) running major-frame-1 for 1-shot.
		dwRemainingFrameRunTime = (wFrameCount * (wFrameTime / 10));
		wFrameCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", s_swDevice2, swFrameId_1, wFrameCount);
		swResult = sitalBc_Start (s_swDevice2, swFrameId_1, wFrameCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Wait for the frame run to end.
		dwRemainingFrameRunTime = (wFrameCount * (wFrameTime / 10));
		swResult = stldTester_Bc_WaitForEndOfFrameRun (s_swDevice2, dwRemainingFrameRunTime, 0, NULL);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.WaitForEndOfFrameRun.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Stop device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop: unit - %i\n", s_swDevice2);
		swResult = sitalBc_Stop (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Verify that no response has been received by device-2 (BC) from device-1 (RT) for message-0.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded: unit - %i, message - %i, purge-y\n", s_swDevice2, swMessageId_0);
		swResult = sitalBc_Message_GetByIdDecoded (s_swDevice2, swMessageId_0, &dmsDecodedMessage, TRUE);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded.Error: error - %i\n", swResult);
			throw 0;
		}
		if (1 != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded.Error: error - %i\n", swResult);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message: message - %i, bsw - %04X\n", swMessageId_0, dmsDecodedMessage.wBlockStatus);
		if (sitalBcBlockStatusWord_NO_RESPONSE != (sitalBcBlockStatusWord_NO_RESPONSE & dmsDecodedMessage.wBlockStatus))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BlockStatus.Error: error - %s\n", "bad_bsw");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// Verify that a response has been received by device-2 (BC) from device-1 (RT) for message-1.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded: unit - %i, message - %i, purge-y\n", s_swDevice2, swMessageId_1);
		swResult = sitalBc_Message_GetByIdDecoded (s_swDevice2, swMessageId_1, &dmsDecodedMessage, TRUE);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded.Error: error - %i\n", swResult);
			throw 0;
		}
		if (1 != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded.Error: error - %i\n", swResult);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message: message - %i, bsw - %04X\n", swMessageId_1, dmsDecodedMessage.wBlockStatus);
		if (sitalBcBlockStatusWord_NO_RESPONSE == (sitalBcBlockStatusWord_NO_RESPONSE & dmsDecodedMessage.wBlockStatus))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BlockStatus.Error: error - %s\n", "bad_bsw");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		///		Stop device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Stop: unit - %i\n", s_swDevice1);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Stop (s_swDevice1) : sitalRt_Stop (s_swDevice1) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice1);
		swResult = sitalDevice_Free (s_swDevice1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice2);
		swResult = sitalDevice_Free (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		swResult = stldSyncEnd (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncEnd.Error: error - %i\n", swResult);
			throw 0;
		}
	}
	catch (int)
	{
		sitalDevice_Free (s_swDevice1);
		sitalDevice_Free (s_swDevice2);
	}

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_13h.End: test_succeeded - %c\n", ((sitalReturnCode_SUCCESS == swResult) ? 'Y' : 'N'));
	return swResult;
}



/// <summary>
/// Test the stld1553 library in the henceforth described scenario:
/// Participant[s]: RT1, BC2.
/// Configuration: All participating devices must be able to communicate with each other.
/// Short description: Acceptance Test Plan (ATP) test 461_13i - RT shall be capable of distinguishing between a broadcast and non-broadcast message to the same sub-address.
/// Along test print suitable info and indications into the log.
///
/// Note:
/// - This function may use any of above defined Acceptance Test Plan (ATP) tests variables.
///   The caller must therefore call the suitable stldTester_Atp_Set* functions prior to calling this function.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Atp_DoTest461_13i ()
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_13i.Start\n");

	/// Set test arguments.
	U16BIT wWordCount; // Word count (1-32).
	wWordCount = 32;
	U16BIT wRtSubaddress; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31).
	wRtSubaddress = sitalRtSubaddress_3;
	U16BIT wFrameTime; // The time (in units of 100 us) that the frame will take to complete.
	wFrameTime = 1000; // 100ms.
	U16BIT wRxUniqueDataValue; // A unique data value that's used in filling data blocks.
	wRxUniqueDataValue = 1;
	U16BIT wBroadcastUniqueDataValue; // A unique data value that's used in filling data blocks.
	wBroadcastUniqueDataValue = 2;

	S16BIT swResult; // Result of operation or function call.
	swResult = sitalReturnCode_SUCCESS;

	/// Get device count.
	U16BIT wRequiredDeviceCount; // The number of devices required for this test.
	wRequiredDeviceCount = (1 + max(s_swDevice1, s_swDevice2));
	U16BIT wAvailableDeviceCount; // The total number of devices available in the system.
	swResult = sitalDevice_GetCount (&wAvailableDeviceCount);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %i\n", swResult);
		return swResult;
	}
	sitalProcess_Log_PrintLine ("#StldTester.Device.Count: available - %i, required - %i\n", wAvailableDeviceCount, wRequiredDeviceCount);
	if (wAvailableDeviceCount < wRequiredDeviceCount)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %s\n", "not_enough_devices_installed");
		return sitalReturnCode_SUCCESS;
	}
	if ( (s_swDevice1 == s_swDevice2) )
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Number.Error: error - %s\n", "same_device_used_twice");
		return sitalReturnCode_INVALID_DEVICE_NUMBER;
	}

	try
	{
		swResult = stldSyncStart (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncStart.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Verify legal test configuration.
		if (wRxUniqueDataValue == wBroadcastUniqueDataValue)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Configuration.Error: error - %s\n", "no_unique_data");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// Initialize device-1 (RT) as a real RT in advanced mode.
		// Note:
		// - The default configuration of initialized RTs is "separate broadcast data", so no special configuration shall be made in favor of this test.
		// - Advanced mode will enable r/w to the memory of device-1 (RT) while the device is running.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-rt, alloc_mem-none\n", s_swDevice1);
		swResult = sitalDevice_Initialize (s_swDevice1, sitalAccess_CARD, ((s_bIsRtMtModeTest ? sitalMode_RT_AND_MT : sitalMode_RT) | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Set device-1 (RT) address to RtAddress1.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set: unit - %i, address - %i\n", s_swDevice1, s_wRtAddress1);
		swResult = sitalRt_Address_Set (s_swDevice1, s_wRtAddress1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-1 (RT) data-block-0, a data block of 32 data words.
		S16BIT swRtDataBlockId_0; // A unique ID designating a data block (0-(sitalRtCounter_DATA_BLOCKS-1)).
		swRtDataBlockId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.Create: unit - %i, block - %i, type - %i, buf_size - %i\n", s_swDevice1, swRtDataBlockId_0, sitalRtDataBlockType_SINGLE_32, wWordCount);
		swResult = sitalRt_DataBlock_Create (s_swDevice1, swRtDataBlockId_0, sitalRtDataBlockType_SINGLE_32, NULL, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-1 (RT) data-block-1, a data block of 32 data words.
		S16BIT swRtDataBlockId_1; // A unique ID designating a data block (0-(sitalRtCounter_DATA_BLOCKS-1)).
		swRtDataBlockId_1 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.Create: unit - %i, block - %i, type - %i, buf_size - %i\n", s_swDevice1, swRtDataBlockId_1, sitalRtDataBlockType_SINGLE_32, wWordCount);
		swResult = sitalRt_DataBlock_Create (s_swDevice1, swRtDataBlockId_1, sitalRtDataBlockType_SINGLE_32, NULL, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Map for device-1 (RT) data-block-0 with subaddress-RtSubAddress for Rx, and legalize this subaddress.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.MapToSa: unit - %i, block - %i, sa - %i, msg_types - x%X, irq_opts-x%X, legalize - %c\n", s_swDevice1, swRtDataBlockId_0, wRtSubaddress, sitalRtMessageType_RX, sitalRtDataBlockIrq_NONE, 'Y');
		swResult = sitalRt_DataBlock_MapToSubaddress (s_swDevice1, swRtDataBlockId_0, wRtSubaddress, sitalRtMessageType_RX, sitalRtDataBlockIrq_NONE, TRUE);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.MapToSa.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Map for device-1 (RT) data-block-1 with subaddress-RtSubAddress for broadcast, and legalize this subaddress.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.MapToSa: unit - %i, block - %i, sa - %i, msg_types - x%X, irq_opts-x%X, legalize - %c\n", s_swDevice1, swRtDataBlockId_1, wRtSubaddress, sitalRtMessageType_BROADCAST, sitalRtDataBlockIrq_NONE, 'Y');
		swResult = sitalRt_DataBlock_MapToSubaddress (s_swDevice1, swRtDataBlockId_1, wRtSubaddress, sitalRtMessageType_BROADCAST, sitalRtDataBlockIrq_NONE, TRUE);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.MapToSa.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Start device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Start: unit - %i\n", s_swDevice1);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Start (s_swDevice1) : sitalRt_Start (s_swDevice1) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-2 (BC) as a real BC.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-bc, alloc_mem-none\n", s_swDevice2);
		swResult = sitalDevice_Initialize (s_swDevice2, sitalAccess_CARD, (sitalMode_BC | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) data-block-0, a data block of 32 data words, and fill it with unique data.
		S16BIT swDataBlockId_0; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId_0 = 1;
		U16BIT waBuffer[32]; // A buffer to read-from/write-into along data-block write/read operations, respectively.
		for (int i=0; i<wWordCount; i++) waBuffer[i] = wRxUniqueDataValue;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i, buf_contents - %i...\n", s_swDevice2, swDataBlockId_0, 32, 0, waBuffer[0]);
		swResult = sitalBc_DataBlock_Create (s_swDevice2, swDataBlockId_0, sitalBcDataBlockSizeOption_SINGLE, waBuffer, wWordCount);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) data-block-1, a data block of 32 data words, and fill it with unique data.
		S16BIT swDataBlockId_1; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId_1 = 2;
		for (int i=0; i<wWordCount; i++) waBuffer[i] = wBroadcastUniqueDataValue;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i, buf_contents - %i...\n", s_swDevice2, swDataBlockId_1, 32, 0, waBuffer[0]);
		swResult = sitalBc_DataBlock_Create (s_swDevice2, swDataBlockId_1, sitalBcDataBlockSizeOption_SINGLE, waBuffer, wWordCount);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) message-0, a BC-to-RT(RtAddress1/RtSubAddress) message that uses data-block-0 to transmit WordCount data words. Message bus: bus-1.
		S16BIT swMessageId_0; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create: unit - %i, message - %i, block - %i, rx_rt - %i, rx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddress, wWordCount, 0, s_wBus1);
		swResult = sitalBc_Message_CreateBcToRt (s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddress, wWordCount, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) message-1, a BC-to-Broadcast(RtSubAddress) message that uses data-block-1 to transmit WordCount data words. Message bus: bus-1.
		S16BIT swMessageId_1; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_1 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToBroadcast.Create: unit - %i, message - %i, block - %i, rx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_1, swDataBlockId_1, wRtSubaddress, wWordCount, 0, s_wBus1);
		swResult = sitalBc_Message_CreateBcToBroadcast (s_swDevice2, swMessageId_1, swDataBlockId_1, wRtSubaddress, wWordCount, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToBroadcast.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-0, a XEQ command that executes message-0.
		S16BIT swCommandId_0; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-1, a XEQ command that executes message-1.
		S16BIT swCommandId_1; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_1 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_1, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_1, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_1, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_1, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-2, a subroutine call command that executes minor-frame-0.
		S16BIT swCommandId_2; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_2 = 2;
		S16BIT swFrameId_0; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_2, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, swFrameId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_2, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, (U16BIT)swFrameId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) minor-frame-0, a frame that contains command-0 and command-1.
		S16BIT swaCommandIds[sitalBcCounter_COMMANDS]; // An array of command-IDs.
		swaCommandIds[0] = swCommandId_0;
		swaCommandIds[1] = swCommandId_1;
		U16BIT wCommandIdCount; // The number of command IDs in the frame (1-sitalBcCounter_FRAME_COMMANDS).
		wCommandIdCount = 2;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i,%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_0, "minor", wCommandIdCount, swCommandId_0, swCommandId_1, 0, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_0, sitalBcFrameType_MINOR, swaCommandIds, wCommandIdCount, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) major-frame-1, a frame that contains command-2 and whose time is FrameTime.
		S16BIT swFrameId_1; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_1 = 1;
		swaCommandIds[0] = swCommandId_2;
		wCommandIdCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_1, "major", wCommandIdCount, swCommandId_2, wFrameTime, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_1, sitalBcFrameType_MAJOR, swaCommandIds, wCommandIdCount, wFrameTime, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		///	Start device-2 (BC) running major-frame-1 for 1-shot.
		U16BIT wFrameCount; // The number of frames to run.
		wFrameCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", s_swDevice2, swFrameId_1, wFrameCount);
		swResult = sitalBc_Start (s_swDevice2, swFrameId_1, wFrameCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		///		Wait for the frame run to end.
		U32BIT dwRemainingFrameRunTime; // The expected remaining frame run time (in ms).
		dwRemainingFrameRunTime = (wFrameCount * (wFrameTime / 10));
		swResult = stldTester_Bc_WaitForEndOfFrameRun (s_swDevice2, dwRemainingFrameRunTime, 0, NULL);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.WaitForEndOfFrameRun.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Stop device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop: unit - %i\n", s_swDevice2);
		swResult = sitalBc_Stop (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		///	Stop device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Stop: unit - %i\n", s_swDevice1);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Stop (s_swDevice1) : sitalRt_Stop (s_swDevice1) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// With device-1 (RT) read from data-block-0 WordCount data words.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.Read: unit - %i, block - %i, buf_size - %i, offset - %i\n", s_swDevice1, swRtDataBlockId_0, wWordCount, 0);
		memset (waBuffer, 0, sizeof(waBuffer));
		swResult = sitalRt_DataBlock_Read (s_swDevice1, swRtDataBlockId_0, waBuffer, wWordCount, 0);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.Read.Error: error - %i\n", swResult);
			throw 0;
		}

		///	With device-1 (RT) verify that data-block-0 contains the expected data.
		for (int i=0; i<wWordCount; i++)
		{
			if (wRxUniqueDataValue != waBuffer[i])
			{
				sitalProcess_Log_PrintLine ("#StldTester.Rt.ReceivedData.Error: error - %i\n", waBuffer[i]);
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}
		}

		/// With device-1 (RT) read from data-block-1 WordCount data words.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.Read: unit - %i, block - %i, buf_size - %i, offset - %i\n", s_swDevice1, swRtDataBlockId_1, wWordCount, 0);
		memset (waBuffer, 0, sizeof(waBuffer));
		swResult = sitalRt_DataBlock_Read (s_swDevice1, swRtDataBlockId_1, waBuffer, wWordCount, 0);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.Read.Error: error - %i\n", swResult);
			throw 0;
		}

		///	With device-1 (RT) verify that data-block-0 contains the expected data.
		for (int i=0; i<wWordCount; i++)
		{
			if (wBroadcastUniqueDataValue != waBuffer[i])
			{
				sitalProcess_Log_PrintLine ("#StldTester.Rt.ReceivedData.Error: error - %i\n", waBuffer[i]);
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}
		}

		/// Free device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice1);
		swResult = sitalDevice_Free (s_swDevice1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice2);
		swResult = sitalDevice_Free (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		swResult = stldSyncEnd (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncEnd.Error: error - %i\n", swResult);
			throw 0;
		}
	}
	catch (int)
	{
		sitalDevice_Free (s_swDevice1);
		sitalDevice_Free (s_swDevice2);
	}

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_13i.End: test_succeeded - %c\n", ((sitalReturnCode_SUCCESS == swResult) ? 'Y' : 'N'));
	return swResult;
}



/// <summary>
/// Test the stld1553 library in the henceforth described scenario:
/// Participant[s]: RT1, BC2, MT3.
/// Configuration: All participating devices must be able to communicate with each other.
/// Short description: Acceptance Test Plan (ATP) test 461_13j - It shall be possible for the application software to Set/Clear Busy bit.
/// Along test print suitable info and indications into the log.
///
/// Note:
/// - This function may use any of above defined Acceptance Test Plan (ATP) tests variables.
///   The caller must therefore call the suitable stldTester_Atp_Set* functions prior to calling this function.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Atp_DoTest461_13j ()
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_13j.Start\n");

	/// Set test arguments.
	U16BIT wWordCount; // Word count (1-32).
	wWordCount = 32;
	U16BIT wRtSubaddress; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31).
	wRtSubaddress = sitalRtSubaddress_1;
	U16BIT wFrameTime; // The time (in units of 100 us) that the frame will take to complete.
	wFrameTime = 10; // 1ms (to enable simulation by Ofer Hofman).
	//wFrameTime = 1000; // 100ms (normal test).
	U16BIT wFrameCount; // The number of frames to run.
	wFrameCount = 3;

	S16BIT swResult; // Result of operation or function call.
	swResult = sitalReturnCode_SUCCESS;

	/// Get device count.
	U16BIT wRequiredDeviceCount; // The number of devices required for this test.
	wRequiredDeviceCount = (1 + max(max(s_swDevice1, s_swDevice2), s_swDevice3));
	U16BIT wAvailableDeviceCount; // The total number of devices available in the system.
	swResult = sitalDevice_GetCount (&wAvailableDeviceCount);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %i\n", swResult);
		return swResult;
	}
	sitalProcess_Log_PrintLine ("#StldTester.Device.Count: available - %i, required - %i\n", wAvailableDeviceCount, wRequiredDeviceCount);
	if (wAvailableDeviceCount < wRequiredDeviceCount)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %s\n", "not_enough_devices_installed");
		return sitalReturnCode_SUCCESS;
	}
	if ( (s_swDevice1 == s_swDevice2) || (s_swDevice1 == s_swDevice3) || (s_swDevice2 == s_swDevice3))
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Number.Error: error - %s\n", "same_device_used_twice");
		return sitalReturnCode_INVALID_DEVICE_NUMBER;
	}

	try
	{
		swResult = stldSyncStart (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncStart.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-3 (MT) as a real MT.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access - card, mode - mt, alloc_mem - none\n", s_swDevice3);
		swResult = sitalDevice_Initialize (s_swDevice3, sitalAccess_CARD, ((s_bIsRtMtModeTest ? sitalMode_RT_AND_MT : sitalMode_MT) | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Start device-3 (MT).
		sitalProcess_Log_PrintLine ("#StldTester.Mt.Start: unit - %i\n", s_swDevice3);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Start (s_swDevice3) : sitalMt_Start (s_swDevice3) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-1 (RT) as a real RT in advanced mode.
		// Note:
		// - The default configuration of initialized RTs is "separate broadcast data", so no special configuration shall be made in favor of this test.
		// - Advanced mode will enable r/w to the memory of device-1 (RT) while the device is running.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-rt, alloc_mem-none\n", s_swDevice1);
		swResult = sitalDevice_Initialize (s_swDevice1, sitalAccess_CARD, ((s_bIsRtMtModeTest ? sitalMode_RT_AND_MT : sitalMode_RT) | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Set device-1 (RT) address to RtAddress1.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set: unit - %i, address - %i\n", s_swDevice1, s_wRtAddress1);
		swResult = sitalRt_Address_Set (s_swDevice1, s_wRtAddress1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-1 (RT) data-block-0, a data block of 32 data words.
		S16BIT swRtDataBlockId_0; // A unique ID designating a data block (0-(sitalRtCounter_DATA_BLOCKS-1)).
		swRtDataBlockId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.Create: unit - %i, block - %i, type - %i, buf_size - %i\n", s_swDevice1, swRtDataBlockId_0, sitalRtDataBlockType_SINGLE_32, wWordCount);
		swResult = sitalRt_DataBlock_Create (s_swDevice1, swRtDataBlockId_0, sitalRtDataBlockType_SINGLE_32, NULL, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Map for device-1 (RT) data-block-0 with subaddress-RtSubAddress for Tx, and legalize this subaddress.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.MapToSa: unit - %i, block - %i, sa - %i, msg_types - x%X, irq_opts-x%X, legalize - %c\n", s_swDevice1, swRtDataBlockId_0, wRtSubaddress, sitalRtMessageType_TX, sitalRtDataBlockIrq_NONE, 'Y');
		swResult = sitalRt_DataBlock_MapToSubaddress (s_swDevice1, swRtDataBlockId_0, wRtSubaddress, sitalRtMessageType_TX, sitalRtDataBlockIrq_NONE, TRUE);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.MapToSa.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Start device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Start: unit - %i\n", s_swDevice1);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Start (s_swDevice1) : sitalRt_Start (s_swDevice1) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-2 (BC) as a real BC.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-bc, alloc_mem-none\n", s_swDevice2);
		swResult = sitalDevice_Initialize (s_swDevice2, sitalAccess_CARD, (sitalMode_BC | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) data-block-0, a data block of 32 data words, and fill it with unique data.
		S16BIT swDataBlockId_0; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId_0 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i\n", s_swDevice2, swDataBlockId_0, 32, 0);
		swResult = sitalBc_DataBlock_Create (s_swDevice2, swDataBlockId_0, sitalBcDataBlockSizeOption_SINGLE, NULL, wWordCount);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) message-0, a RT(RtAddress1/RtSubAddress)-to-BC message that uses data-block-0 to receive WordCount data words. Message bus: bus-1.
		S16BIT swMessageId_0; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBc.Create: unit - %i, message - %i, block - %i, tx_rt - %i, tx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddress, wWordCount, 0, s_wBus1);
		swResult = sitalBc_Message_CreateRtToBc (s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddress, wWordCount, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBc.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-0, a XEQ command that executes message-0.
		S16BIT swCommandId_0; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-1, a subroutine call command that executes minor-frame-0.
		S16BIT swCommandId_1; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_1 = 1;
		S16BIT swFrameId_0; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_1, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, swFrameId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_1, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, (U16BIT)swFrameId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) minor-frame-0, a frame that contains command-0 and command-1.
		S16BIT swaCommandIds[sitalBcCounter_COMMANDS]; // An array of command-IDs.
		swaCommandIds[0] = swCommandId_0;
		U16BIT wCommandIdCount; // The number of command IDs in the frame (1-sitalBcCounter_FRAME_COMMANDS).
		wCommandIdCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_0, "minor", wCommandIdCount, swCommandId_0, 0, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_0, sitalBcFrameType_MINOR, swaCommandIds, wCommandIdCount, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) major-frame-1, a frame that contains command-2 and whose time is FrameTime.
		S16BIT swFrameId_1; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_1 = 1;
		swaCommandIds[0] = swCommandId_1;
		wCommandIdCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_1, "major", wCommandIdCount, swCommandId_1, wFrameTime, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_1, sitalBcFrameType_MAJOR, swaCommandIds, wCommandIdCount, wFrameTime, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Set busy for the response status bits of device-1 (RT).
		// Note: In regular (i.e., non-alternate) RT mode, this operation turns off the BUSY bit in configuration register #1.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.ResponseStatusBits.Set: unit - %i, status_bits - %04X\n", s_swDevice1, sitalRtStatusWordBit_BUSY);
		swResult = sitalRt_ResponseStatusBits_Set (s_swDevice1, sitalRtStatusWordBit_BUSY);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.ResponseStatusBits.Set.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Verify for device-1 (RT) a proper configuration of response status bits.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.ResponseStatusBits.Get: unit - %i\n", s_swDevice1);
		U16BIT wRtResponseStatusBits; // The status enabler bit mask in configuration register #1 of an RT (an or-ed combination of sitalRtStatusWordBit_*).
		swResult = sitalRt_ResponseStatusBits_Get (s_swDevice1, &wRtResponseStatusBits);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.ResponseStatusBits.Get.Error: error - %i\n", swResult);
			throw 0;
		}
		if (sitalRtStatusWordBit_BUSY == (sitalRtStatusWordBit_BUSY & (~wRtResponseStatusBits)))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.ResponseStatusBits.Error: error - %04X\n", wRtResponseStatusBits);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		///	Start device-2 (BC) running major-frame-1 for 1-shot.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", s_swDevice2, swFrameId_1, wFrameCount);
		swResult = sitalBc_Start (s_swDevice2, swFrameId_1, wFrameCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		///		Wait for the frame run to end.
		U32BIT dwRemainingFrameRunTime; // The expected remaining frame run time (in ms).
		dwRemainingFrameRunTime = (wFrameCount * (wFrameTime / 10));
		swResult = stldTester_Bc_WaitForEndOfFrameRun (s_swDevice2, dwRemainingFrameRunTime, 0, NULL);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.WaitForEndOfFrameRun.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Verify no RT data detected by device-3 (MT).
		// Note:
		// - In normal messages, the word count is extracted from the message command word.
		// - When an RT responses BUSY to its BC, the monitoring MT detects a message whose BSW shows an error.
		// - In this case, the difference between the message data pointers (that are pointing to the MT data stack) of the current message and the next one
		//   is used to calculate the actual number of data words (what includes the status words) in the current message, instead of using the normal word count.
		// - Moreover, in this case the MT cannot say which of the detected data words is actually data, and which is a status word (recall that in erroneous cases
		//   some data may be lost, or, alternatively, some redundant data may appear). Therefore, the MT only reports the overall number of detected data words.
		// - Yet, when the last message in the MT command stack is analyzed, there's no next message, and therefore no next data pointer. The number of actual data
		//   words isn't therefore reported in case of message error in the last available message.
		// - In conclusion, in this part of the test, the MT should report the detection of one actual data word for all message besides the last one, and no actual
		//   data words for the last message.
		sitalDecodedMessageStructure dmsDecodedMessage; // A decoded message.
		for (S16BIT i=0; i<wFrameCount; i++)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded: unit - %i, purge - %i\n", s_swDevice3, TRUE);
			swResult = sitalMt_Message_GetFromStackDecoded (s_swDevice3, &dmsDecodedMessage, sitalMessageLocationAndRemoval_NEXT_PURGE, sitalMtStack_ACTIVE);
			if ((0 > swResult) || (1 < swResult))
			{
				sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Error: error - %i\n", swResult);
				throw 0;
			}
			if (0U == swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Message.Error: error - %s\n", "no_message");
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}
			sitalProcess_Log_PrintLine ("#StldTester.Mt.Message: unit - %i, command - %04X, data_count - %i, status_1_relevant - %i, status_1 - %04X, time_tag - %04X\n", s_swDevice3, dmsDecodedMessage.wCommandWord1, dmsDecodedMessage.wDataWordCount, dmsDecodedMessage.bIsStatus1Relevant, dmsDecodedMessage.wStatus1, dmsDecodedMessage.wTimeTag);
			U16BIT wExpectedTotalCountOfDataWords; // The expected total number of data words.
			wExpectedTotalCountOfDataWords = ( (i < (wFrameCount - 1)) ? 1 : 0 );
			if (wExpectedTotalCountOfDataWords != dmsDecodedMessage.wDataWordCount)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Message.Error: error - %s\n", "bad_data_count");
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}
			if ( ! dmsDecodedMessage.bIsStatus1Relevant )
			{
				sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Message.Error: error - %s\n", "no_status_1");
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}
			if (sitalIeee1553_Status_BUSY != (sitalIeee1553_Status_ERROR_SECTION & dmsDecodedMessage.wStatus1))
			{
				sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Message.Error: error - %s\n", "bad_status_1");
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}
		}

		///	Verify that no additional messages have been transmitted.
		sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded: unit - %i, purge - %i\n", s_swDevice3, TRUE);
		swResult = sitalMt_Message_GetFromStackDecoded (s_swDevice3, &dmsDecodedMessage, sitalMessageLocationAndRemoval_NEXT_PURGE, sitalMtStack_ACTIVE);
		if ((0 > swResult) || (1 < swResult))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Error: error - %i\n", swResult);
			throw 0;
		}
		if (1 == swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Error: error - %s\n", "extra_message");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// Stop device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop: unit - %i\n", s_swDevice2);
		swResult = sitalBc_Stop (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Unset busy for the response status bits for device-1 (RT).
		// Note: In regular (i.e., non-alternate) RT mode, this operation turns on the BUSY bit in configuration register #1.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.ResponseStatusBits.Unset: unit - %i, status_bits - %04X\n", s_swDevice1, sitalRtStatusWordBit_BUSY);
		swResult = sitalRt_ResponseStatusBits_Unset (s_swDevice1, sitalRtStatusWordBit_BUSY);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.ResponseStatusBits.Unset.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Verify for device-1 (RT) a proper configuration of response status bits.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.ResponseStatusBits.Get: unit - %i\n", s_swDevice1);
		swResult = sitalRt_ResponseStatusBits_Get (s_swDevice1, &wRtResponseStatusBits);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.ResponseStatusBits.Get.Error: error - %i\n", swResult);
			throw 0;
		}
		if (sitalRtStatusWordBit_BUSY != (sitalRtStatusWordBit_BUSY & (~wRtResponseStatusBits)))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.ResponseStatusBits.Error: error - %04X\n", wRtResponseStatusBits);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		///	Start device-2 (BC) running major-frame-1 for 1-shot.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", s_swDevice2, swFrameId_1, wFrameCount);
		swResult = sitalBc_Start (s_swDevice2, swFrameId_1, wFrameCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		///		Wait for the frame run to end.
		dwRemainingFrameRunTime = (wFrameCount * (wFrameTime / 10));
		swResult = stldTester_Bc_WaitForEndOfFrameRun (s_swDevice2, dwRemainingFrameRunTime, 0, NULL);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.WaitForEndOfFrameRun.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Verify RT data detected by device-3 (MT).
		for (S16BIT i=0; i<wFrameCount; i++)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded: unit - %i, purge - %i\n", s_swDevice3, TRUE);
			swResult = sitalMt_Message_GetFromStackDecoded (s_swDevice3, &dmsDecodedMessage, sitalMessageLocationAndRemoval_NEXT_PURGE, sitalMtStack_ACTIVE);
			if ((0 > swResult) || (1 < swResult))
			{
				sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Error: error - %i\n", swResult);
				throw 0;
			}
			if (0U == swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Message.Error: error - %s\n", "no_message");
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}
			sitalProcess_Log_PrintLine ("#StldTester.Mt.Message: unit - %i, command - %04X, data_count - %i, status_1_relevant - %i, status_1 - %04X, time_tag - %04X\n", s_swDevice3, dmsDecodedMessage.wCommandWord1, dmsDecodedMessage.wDataWordCount, dmsDecodedMessage.bIsStatus1Relevant, dmsDecodedMessage.wStatus1, dmsDecodedMessage.wTimeTag);
			if (wWordCount != dmsDecodedMessage.wDataWordCount)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Message.Error: error - %s\n", "bad_data_count");
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}
			if ( ! dmsDecodedMessage.bIsStatus1Relevant )
			{
				sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Message.Error: error - %s\n", "no_status_1");
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}
			if (0U != (sitalIeee1553_Status_ERROR_SECTION & dmsDecodedMessage.wStatus1))
			{
				sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Message.Error: error - %s\n", "bad_status_1");
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}
		}

		///	Verify that no additional messages have been transmitted.
		sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded: unit - %i, purge - %i\n", s_swDevice3, TRUE);
		swResult = sitalMt_Message_GetFromStackDecoded (s_swDevice3, &dmsDecodedMessage, sitalMessageLocationAndRemoval_NEXT_PURGE, sitalMtStack_ACTIVE);
		if ((0 > swResult) || (1 < swResult))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Error: error - %i\n", swResult);
			throw 0;
		}
		if (1 == swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Error: error - %s\n", "extra_message");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// Stop device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop: unit - %i\n", s_swDevice2);
		swResult = sitalBc_Stop (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		///		Stop device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Stop: unit - %i\n", s_swDevice1);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Stop (s_swDevice1) : sitalRt_Stop (s_swDevice1) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Stop device-3 (MT).
		sitalProcess_Log_PrintLine ("#StldTester.Mt.Stop: unit - %i\n", s_swDevice3);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Stop (s_swDevice3) : sitalMt_Stop (s_swDevice3) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice1);
		swResult = sitalDevice_Free (s_swDevice1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice2);
		swResult = sitalDevice_Free (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-3 (MT).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice3);
		swResult = sitalDevice_Free (s_swDevice3);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		swResult = stldSyncEnd (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncEnd.Error: error - %i\n", swResult);
			throw 0;
		}
	}
	catch (int)
	{
		sitalDevice_Free (s_swDevice1);
		sitalDevice_Free (s_swDevice2);
		sitalDevice_Free (s_swDevice3);
	}

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_13j.End: test_succeeded - %c\n", ((sitalReturnCode_SUCCESS == swResult) ? 'Y' : 'N'));
	return swResult;
}



/// <summary>
/// Test the stld1553 library in the henceforth described scenario:
/// Participant[s]: RT1, BC2.
/// Configuration: All participating devices must be able to communicate with each other.
/// Short description: Acceptance Test Plan (ATP) test 461_13k_1 - It shall be possible to set/clear the RT service request bit.
/// Along test print suitable info and indications into the log.
///
/// Note:
/// - This function may use any of above defined Acceptance Test Plan (ATP) tests variables.
///   The caller must therefore call the suitable stldTester_Atp_Set* functions prior to calling this function.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Atp_DoTest461_13k_1 ()
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_13k_1.Start\n");

	/// Set test arguments.
	U16BIT wWordCount; // Word count (1-32).
	wWordCount = 32;
	U16BIT wRtSubaddress; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31).
	wRtSubaddress = sitalRtSubaddress_1;
	U16BIT wFrameTime; // The time (in units of 100 us) that the frame will take to complete.
	wFrameTime = 1000; // 100ms.

	S16BIT swResult; // Result of operation or function call.
	swResult = sitalReturnCode_SUCCESS;

	/// Get device count.
	U16BIT wRequiredDeviceCount; // The number of devices required for this test.
	wRequiredDeviceCount = (1 + max(s_swDevice1, s_swDevice2));
	U16BIT wAvailableDeviceCount; // The total number of devices available in the system.
	swResult = sitalDevice_GetCount (&wAvailableDeviceCount);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %i\n", swResult);
		return swResult;
	}
	sitalProcess_Log_PrintLine ("#StldTester.Device.Count: available - %i, required - %i\n", wAvailableDeviceCount, wRequiredDeviceCount);
	if (wAvailableDeviceCount < wRequiredDeviceCount)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %s\n", "not_enough_devices_installed");
		return sitalReturnCode_SUCCESS;
	}
	if ( (s_swDevice1 == s_swDevice2) || (s_swDevice1 == s_swDevice3) || (s_swDevice2 == s_swDevice3))
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Number.Error: error - %s\n", "same_device_used_twice");
		return sitalReturnCode_INVALID_DEVICE_NUMBER;
	}

	try
	{
		swResult = stldSyncStart (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncStart.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-1 (RT) as a real RT in advanced mode.
		// Note:
		// - The default configuration of initialized RTs is "separate broadcast data", so no special configuration shall be made in favor of this test.
		// - Advanced mode will enable r/w to the memory of device-1 (RT) while the device is running.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-rt, alloc_mem-none\n", s_swDevice1);
		swResult = sitalDevice_Initialize (s_swDevice1, sitalAccess_CARD, ((s_bIsRtMtModeTest ? sitalMode_RT_AND_MT : sitalMode_RT) | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Set device-1 (RT) address to RtAddress1.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set: unit - %i, address - %i\n", s_swDevice1, s_wRtAddress1);
		swResult = sitalRt_Address_Set (s_swDevice1, s_wRtAddress1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-1 (RT) data-block-0, a data block of 32 data words.
		S16BIT swRtDataBlockId_0; // A unique ID designating a data block (0-(sitalRtCounter_DATA_BLOCKS-1)).
		swRtDataBlockId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.Create: unit - %i, block - %i, type - %i, buf_size - %i\n", s_swDevice1, swRtDataBlockId_0, sitalRtDataBlockType_SINGLE_32, wWordCount);
		swResult = sitalRt_DataBlock_Create (s_swDevice1, swRtDataBlockId_0, sitalRtDataBlockType_SINGLE_32, NULL, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Map for device-1 (RT) data-block-0 with subaddress-RtSubAddress for Tx, and legalize this subaddress.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.MapToSa: unit - %i, block - %i, sa - %i, msg_types - x%X, irq_opts-x%X, legalize - %c\n", s_swDevice1, swRtDataBlockId_0, wRtSubaddress, sitalRtMessageType_TX, sitalRtDataBlockIrq_NONE, 'Y');
		swResult = sitalRt_DataBlock_MapToSubaddress (s_swDevice1, swRtDataBlockId_0, wRtSubaddress, sitalRtMessageType_TX, sitalRtDataBlockIrq_NONE, TRUE);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.MapToSa.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Start device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Start: unit - %i\n", s_swDevice1);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Start (s_swDevice1) : sitalRt_Start (s_swDevice1) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-2 (BC) as a real BC.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-bc, alloc_mem-none\n", s_swDevice2);
		swResult = sitalDevice_Initialize (s_swDevice2, sitalAccess_CARD, (sitalMode_BC | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) data-block-0, a data block of 32 data words, and fill it with unique data.
		S16BIT swDataBlockId_0; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId_0 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i\n", s_swDevice2, swDataBlockId_0, 32, 0);
		swResult = sitalBc_DataBlock_Create (s_swDevice2, swDataBlockId_0, sitalBcDataBlockSizeOption_SINGLE, NULL, wWordCount);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) message-0, a RT(RtAddress1/RtSubAddress)-to-BC message that uses data-block-0 to receive WordCount data words. Message bus: bus-1.
		S16BIT swMessageId_0; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBc.Create: unit - %i, message - %i, block - %i, tx_rt - %i, tx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddress, wWordCount, 0, s_wBus1);
		swResult = sitalBc_Message_CreateRtToBc (s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddress, wWordCount, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBc.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-0, a XEQ command that executes message-0.
		S16BIT swCommandId_0; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-1, a subroutine call command that executes minor-frame-0.
		S16BIT swCommandId_1; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_1 = 1;
		S16BIT swFrameId_0; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_1, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, swFrameId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_1, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, (U16BIT)swFrameId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) minor-frame-0, a frame that contains command-0 and command-1.
		S16BIT swaCommandIds[sitalBcCounter_COMMANDS]; // An array of command-IDs.
		swaCommandIds[0] = swCommandId_0;
		U16BIT wCommandIdCount; // The number of command IDs in the frame (1-sitalBcCounter_FRAME_COMMANDS).
		wCommandIdCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_0, "minor", wCommandIdCount, swCommandId_0, 0, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_0, sitalBcFrameType_MINOR, swaCommandIds, wCommandIdCount, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) major-frame-1, a frame that contains command-2 and whose time is FrameTime.
		S16BIT swFrameId_1; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_1 = 1;
		swaCommandIds[0] = swCommandId_1;
		wCommandIdCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_1, "major", wCommandIdCount, swCommandId_1, wFrameTime, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_1, sitalBcFrameType_MAJOR, swaCommandIds, wCommandIdCount, wFrameTime, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		///	Start device-2 (BC) running major-frame-1 for 1-shot.
		U16BIT wFrameCount; // The number of frames to run.
		wFrameCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", s_swDevice2, swFrameId_1, wFrameCount);
		swResult = sitalBc_Start (s_swDevice2, swFrameId_1, wFrameCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Wait for the frame run to end.
		U32BIT dwRemainingFrameRunTime; // The expected remaining frame run time (in ms).
		dwRemainingFrameRunTime = (wFrameCount *  (wFrameTime / 10));
		swResult = stldTester_Bc_WaitForEndOfFrameRun (s_swDevice2, dwRemainingFrameRunTime, 0, NULL);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.WaitForEndOfFrameRun.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Stop device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop: unit - %i\n", s_swDevice2);
		swResult = sitalBc_Stop (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Verify that a clear status has been received by device-2 (BC) from device-1 (RT) for message-0.
		// Note: RT devices are initialized to reply with clear status.
		sitalDecodedMessageStructure dmsDecodedMessage; // A decoded message.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded: unit - %i, message - %i, purge-y\n", s_swDevice2, swMessageId_0);
		swResult = sitalBc_Message_GetByIdDecoded (s_swDevice2, swMessageId_0, &dmsDecodedMessage, TRUE);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded.Error: error - %i\n", swResult);
			throw 0;
		}
		if (1 != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded.Error: error - %i\n", swResult);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message: message - %i, bsw - %04X, status_1_relevant - %i, status_1 - %04X\n", swMessageId_0, dmsDecodedMessage.wBlockStatus, dmsDecodedMessage.bIsStatus1Relevant, dmsDecodedMessage.wStatus1);
		if (sitalBcBlockStatusWord_NO_RESPONSE == (sitalBcBlockStatusWord_NO_RESPONSE & dmsDecodedMessage.wBlockStatus))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BlockStatus.Error: error - %s\n", "bad_bsw");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		if (wWordCount != dmsDecodedMessage.wDataWordCount)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.Error: error - %s\n", "bad_data_count");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		if ( ! dmsDecodedMessage.bIsStatus1Relevant )
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.Error: error - %s\n", "no_status_1");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		if (sitalIeee1553_Status_SERVICE_REQUEST == (sitalIeee1553_Status_SERVICE_REQUEST & dmsDecodedMessage.wStatus1))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.Error: error - %s\n", "bad_status_1");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		///	Set service request for the response status bits of device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Rt.ResponseStatusBits.Set: unit - %i, status_bits - %04X\n", s_swDevice1, sitalRtStatusWordBit_SERVICE_REQUSET);
		swResult = sitalRt_ResponseStatusBits_Set (s_swDevice1, sitalRtStatusWordBit_SERVICE_REQUSET);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.ResponseStatusBits.Error: error - %i\n", swResult);
			throw 0;
		}

		///	Start device-2 (BC) running major-frame-1 for 1-shot.
		wFrameCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", s_swDevice2, swFrameId_1, wFrameCount);
		swResult = sitalBc_Start (s_swDevice2, swFrameId_1, wFrameCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Wait for the frame run to end.
		dwRemainingFrameRunTime = (wFrameCount *  (wFrameTime / 10));
		swResult = stldTester_Bc_WaitForEndOfFrameRun (s_swDevice2, dwRemainingFrameRunTime, 0, NULL);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.WaitForEndOfFrameRun.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Stop device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop: unit - %i\n", s_swDevice2);
		swResult = sitalBc_Stop (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Verify that a clear status has been received by device-2 (BC) from device-1 (RT) for message-0.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded: unit - %i, message - %i, purge-y\n", s_swDevice2, swMessageId_0);
		swResult = sitalBc_Message_GetByIdDecoded (s_swDevice2, swMessageId_0, &dmsDecodedMessage, TRUE);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded.Error: error - %i\n", swResult);
			throw 0;
		}
		if (1 != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded.Error: error - %i\n", swResult);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message: message - %i, bsw - %04X, status_1_relevant - %i, status_1 - %04X\n", swMessageId_0, dmsDecodedMessage.wBlockStatus, dmsDecodedMessage.bIsStatus1Relevant, dmsDecodedMessage.wStatus1);
		if (sitalBcBlockStatusWord_NO_RESPONSE == (sitalBcBlockStatusWord_NO_RESPONSE & dmsDecodedMessage.wBlockStatus))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BlockStatus.Error: error - %s\n", "bad_bsw");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		if (wWordCount != dmsDecodedMessage.wDataWordCount)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.Error: error - %s\n", "bad_data_count");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		if ( ! dmsDecodedMessage.bIsStatus1Relevant )
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.Error: error - %s\n", "no_status_1");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		if (sitalIeee1553_Status_SERVICE_REQUEST != (sitalIeee1553_Status_SERVICE_REQUEST & dmsDecodedMessage.wStatus1))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.Error: error - %s\n", "bad_status_1");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		///	Unset service request for the response status bits of device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Rt.ResponseStatusBits.Unset: unit - %i, status_bits - %04X\n", s_swDevice1, sitalRtStatusWordBit_SERVICE_REQUSET);
		swResult = sitalRt_ResponseStatusBits_Unset (s_swDevice1, sitalRtStatusWordBit_SERVICE_REQUSET);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.ResponseStatusBits.Error: error - %i\n", swResult);
			throw 0;
		}

		///	Start device-2 (BC) running major-frame-1 for 1-shot.
		wFrameCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", s_swDevice2, swFrameId_1, wFrameCount);
		swResult = sitalBc_Start (s_swDevice2, swFrameId_1, wFrameCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Wait for the frame run to end.
		dwRemainingFrameRunTime = (wFrameCount *  (wFrameTime / 10));
		swResult = stldTester_Bc_WaitForEndOfFrameRun (s_swDevice2, dwRemainingFrameRunTime, 0, NULL);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.WaitForEndOfFrameRun.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Stop device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop: unit - %i\n", s_swDevice2);
		swResult = sitalBc_Stop (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Verify that a clear status has been received by device-2 (BC) from device-1 (RT) for message-0.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded: unit - %i, message - %i, purge-y\n", s_swDevice2, swMessageId_0);
		swResult = sitalBc_Message_GetByIdDecoded (s_swDevice2, swMessageId_0, &dmsDecodedMessage, TRUE);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded.Error: error - %i\n", swResult);
			throw 0;
		}
		if (1 != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded.Error: error - %i\n", swResult);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message: message - %i, bsw - %04X, status_1_relevant - %i, status_1 - %04X\n", swMessageId_0, dmsDecodedMessage.wBlockStatus, dmsDecodedMessage.bIsStatus1Relevant, dmsDecodedMessage.wStatus1);
		if (sitalBcBlockStatusWord_NO_RESPONSE == (sitalBcBlockStatusWord_NO_RESPONSE & dmsDecodedMessage.wBlockStatus))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BlockStatus.Error: error - %s\n", "bad_bsw");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		if (wWordCount != dmsDecodedMessage.wDataWordCount)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.Error: error - %s\n", "bad_data_count");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		if ( ! dmsDecodedMessage.bIsStatus1Relevant )
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.Error: error - %s\n", "no_status_1");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		if (sitalIeee1553_Status_SERVICE_REQUEST == (sitalIeee1553_Status_SERVICE_REQUEST & dmsDecodedMessage.wStatus1))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.Error: error - %s\n", "bad_status_1");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		///		Stop device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Stop: unit - %i\n", s_swDevice1);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Stop (s_swDevice1) : sitalRt_Stop (s_swDevice1) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice1);
		swResult = sitalDevice_Free (s_swDevice1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice2);
		swResult = sitalDevice_Free (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		swResult = stldSyncEnd (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncEnd.Error: error - %i\n", swResult);
			throw 0;
		}
	}
	catch (int)
	{
		sitalDevice_Free (s_swDevice1);
		sitalDevice_Free (s_swDevice2);
	}

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_13k_1.End: test_succeeded - %c\n", ((sitalReturnCode_SUCCESS == swResult) ? 'Y' : 'N'));
	return swResult;
}



/// <summary>
/// Test the stld1553 library in the henceforth described scenario:
/// Participant[s]: RT1, BC2.
/// Configuration: All participating devices must be able to communicate with each other.
/// Short description: Acceptance Test Plan (ATP) test 461_13k_2 - It shall be possible to automatically clear the service request bit after RT responds to a "Transmit Vector Word" mode code command.
/// Along test print suitable info and indications into the log.
///
/// Note:
/// - This function may use any of above defined Acceptance Test Plan (ATP) tests variables.
///   The caller must therefore call the suitable stldTester_Atp_Set* functions prior to calling this function.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Atp_DoTest461_13k_2 ()
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_13k_2.Start\n");

	/// Set test arguments.
	U16BIT wWordCount; // Word count (1-32).
	wWordCount = 32;
	U16BIT wRtSubaddress; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31).
	wRtSubaddress = sitalRtSubaddress_1;
	U16BIT wFrameTime; // The time (in units of 100 us) that the frame will take to complete.
	wFrameTime = 1000; // 100ms.

	S16BIT swResult; // Result of operation or function call.
	swResult = sitalReturnCode_SUCCESS;

	/// Get device count.
	U16BIT wRequiredDeviceCount; // The number of devices required for this test.
	wRequiredDeviceCount = (1 + max(s_swDevice1, s_swDevice2));
	U16BIT wAvailableDeviceCount; // The total number of devices available in the system.
	swResult = sitalDevice_GetCount (&wAvailableDeviceCount);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %i\n", swResult);
		return swResult;
	}
	sitalProcess_Log_PrintLine ("#StldTester.Device.Count: available - %i, required - %i\n", wAvailableDeviceCount, wRequiredDeviceCount);
	if (wAvailableDeviceCount < wRequiredDeviceCount)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %s\n", "not_enough_devices_installed");
		return sitalReturnCode_SUCCESS;
	}
	if ( (s_swDevice1 == s_swDevice2) )
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Number.Error: error - %s\n", "same_device_used_twice");
		return sitalReturnCode_INVALID_DEVICE_NUMBER;
	}

	try
	{
		swResult = stldSyncStart (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncStart.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-1 (RT) as a real RT in advanced mode.
		// Note:
		// - The default configuration of initialized RTs is "separate broadcast data", so no special configuration shall be made in favor of this test.
		// - Advanced mode will enable r/w to the memory of device-1 (RT) while the device is running.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-rt, alloc_mem-none\n", s_swDevice1);
		swResult = sitalDevice_Initialize (s_swDevice1, sitalAccess_CARD, ((s_bIsRtMtModeTest ? sitalMode_RT_AND_MT : sitalMode_RT) | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Set device-1 (RT) address to RtAddress1.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set: unit - %i, address - %i\n", s_swDevice1, s_wRtAddress1);
		swResult = sitalRt_Address_Set (s_swDevice1, s_wRtAddress1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-1 (RT) data-block-0, a data block of 32 data words.
		S16BIT swRtDataBlockId_0; // A unique ID designating a data block (0-(sitalRtCounter_DATA_BLOCKS-1)).
		swRtDataBlockId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.Create: unit - %i, block - %i, type - %i, buf_size - %i\n", s_swDevice1, swRtDataBlockId_0, sitalRtDataBlockType_SINGLE_32, wWordCount);
		swResult = sitalRt_DataBlock_Create (s_swDevice1, swRtDataBlockId_0, sitalRtDataBlockType_SINGLE_32, NULL, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Verify that cannot create for device-1 (RT) another data-block-0.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.Create: unit - %i, block - %i, type - %i, buf_size - %i\n", s_swDevice1, swRtDataBlockId_0, sitalRtDataBlockType_SINGLE_32, wWordCount);
		swResult = sitalRt_DataBlock_Create (s_swDevice1, swRtDataBlockId_0, sitalRtDataBlockType_SINGLE_32, NULL, 0);
		if (sitalReturnCode_RT_DATA_BLOCK_EXISTS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.Create.Error: error - %i\n", swResult);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// Delete device-1 (RT) data-block-0.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.Delete: unit - %i, block - %i\n", s_swDevice2, swRtDataBlockId_0);
		swResult = sitalRt_DataBlock_Delete (s_swDevice1, swRtDataBlockId_0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.Delete.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-1 (RT) data-block-0, a data block of 32 data words.
		swRtDataBlockId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.Create: unit - %i, block - %i, type - %i, buf_size - %i\n", s_swDevice1, swRtDataBlockId_0, sitalRtDataBlockType_SINGLE_32, wWordCount);
		swResult = sitalRt_DataBlock_Create (s_swDevice1, swRtDataBlockId_0, sitalRtDataBlockType_SINGLE_32, NULL, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Map for device-1 (RT) data-block-0 with subaddress-RtSubAddress for Tx, and legalize this subaddress.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.MapToSa: unit - %i, block - %i, sa - %i, msg_types - x%X, irq_opts-x%X, legalize - %c\n", s_swDevice1, swRtDataBlockId_0, wRtSubaddress, sitalRtMessageType_TX, sitalRtDataBlockIrq_NONE, 'Y');
		swResult = sitalRt_DataBlock_MapToSubaddress (s_swDevice1, swRtDataBlockId_0, wRtSubaddress, sitalRtMessageType_TX, sitalRtDataBlockIrq_NONE, TRUE);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.MapToSa.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Start device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Start: unit - %i\n", s_swDevice1);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Start (s_swDevice1) : sitalRt_Start (s_swDevice1) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-2 (BC) as a real BC.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-bc, alloc_mem-none\n", s_swDevice2);
		swResult = sitalDevice_Initialize (s_swDevice2, sitalAccess_CARD, (sitalMode_BC | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) data-block-0, a data block of 32 data words.
		S16BIT swDataBlockId_0; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId_0 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i\n", s_swDevice2, swDataBlockId_0, 32, 0);
		swResult = sitalBc_DataBlock_Create (s_swDevice2, swDataBlockId_0, sitalBcDataBlockSizeOption_SINGLE, NULL, wWordCount);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) message-0, a RT(RtAddress1/RtSubAddress)-to-BC message that uses data-block-0 to receive WordCount data words. Message bus: bus-1.
		S16BIT swMessageId_0; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBc.Create: unit - %i, message - %i, block - %i, tx_rt - %i, tx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddress, wWordCount, 0, s_wBus1);
		swResult = sitalBc_Message_CreateRtToBc (s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddress, wWordCount, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBc.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-0, a XEQ command that executes message-0.
		S16BIT swCommandId_0; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-1, a subroutine call command that executes minor-frame-0.
		S16BIT swCommandId_1; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_1 = 1;
		S16BIT swFrameId_0; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_1, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, swFrameId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_1, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, (U16BIT)swFrameId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) minor-frame-0, a frame that contains command-0.
		S16BIT swaCommandIds[sitalBcCounter_COMMANDS]; // An array of command-IDs.
		swaCommandIds[0] = swCommandId_0;
		U16BIT wCommandIdCount; // The number of command IDs in the frame (1-sitalBcCounter_FRAME_COMMANDS).
		wCommandIdCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_0, "minor", wCommandIdCount, swCommandId_0, 0, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_0, sitalBcFrameType_MINOR, swaCommandIds, wCommandIdCount, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) major-frame-1, a frame that contains command-1 and whose time is FrameTime.
		S16BIT swFrameId_1; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_1 = 1;
		swaCommandIds[0] = swCommandId_1;
		wCommandIdCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_1, "major", wCommandIdCount, swCommandId_1, wFrameTime, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_1, sitalBcFrameType_MAJOR, swaCommandIds, wCommandIdCount, wFrameTime, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) data-block-1, a data block of 32 data words.
		S16BIT swDataBlockId_1; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId_1 = 2;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i\n", s_swDevice2, swDataBlockId_1, 32, 0);
		swResult = sitalBc_DataBlock_Create (s_swDevice2, swDataBlockId_1, sitalBcDataBlockSizeOption_SINGLE, NULL, wWordCount);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		///	Create for device-2 (BC) message-1, a RT(RtAddress)-to-BC-Mode(*) message. Message bus: bus-1.
		S16BIT swMessageId_1; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_1 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBcMode.Create: unit - %i, message - %i, block - %i, rt - %i, direction - %i, mode - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_1, swDataBlockId_1, s_wRtAddress1, sitalMessageDirection_TX, 16, 0, s_wBus1);
		swResult = sitalBc_Message_CreateBcToOrFromRtMode (s_swDevice2, swMessageId_1, swDataBlockId_1, s_wRtAddress1, sitalMessageDirection_TX, 16, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBcMode.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-2, a XEQ command that executes message-1.
		S16BIT swCommandId_2; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_2 = 2;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_2, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_1, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_2, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_1, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-3, a subroutine call command that executes minor-frame-2.
		S16BIT swCommandId_3; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_3 = 3;
		S16BIT swFrameId_2; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_2 = 2;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_3, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, swFrameId_2, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_3, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, (U16BIT)swFrameId_2, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) minor-frame-2, a frame that contains command-2.
		swaCommandIds[0] = swCommandId_2;
		wCommandIdCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_2, "minor", wCommandIdCount, swCommandId_2, 0, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_2, sitalBcFrameType_MINOR, swaCommandIds, wCommandIdCount, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) major-frame-3, a frame that contains command-3 and whose time is FrameTime.
		S16BIT swFrameId_3; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_3 = 3;
		swaCommandIds[0] = swCommandId_3;
		wCommandIdCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_3, "major", wCommandIdCount, swCommandId_3, wFrameTime, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_3, sitalBcFrameType_MAJOR, swaCommandIds, wCommandIdCount, wFrameTime, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		///	Start device-2 (BC) running major-frame-1 for 1-shot.
		U16BIT wFrameCount; // The number of frames to run.
		wFrameCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", s_swDevice2, swFrameId_1, wFrameCount);
		swResult = sitalBc_Start (s_swDevice2, swFrameId_1, wFrameCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Wait for the frame run to end.
		U32BIT dwRemainingFrameRunTime; // The expected remaining frame run time (in ms).
		dwRemainingFrameRunTime = (wFrameCount * (wFrameTime / 10));
		swResult = stldTester_Bc_WaitForEndOfFrameRun (s_swDevice2, dwRemainingFrameRunTime, 0, NULL);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.WaitForEndOfFrameRun.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Stop device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop: unit - %i\n", s_swDevice2);
		swResult = sitalBc_Stop (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Verify that a clear status has been received by device-2 (BC) from device-1 (RT) for message-0.
		// Note: RT devices are initialized to reply with clear status.
		sitalDecodedMessageStructure dmsDecodedMessage; // A decoded message.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded: unit - %i, message - %i, purge-y\n", s_swDevice2, swMessageId_0);
		swResult = sitalBc_Message_GetByIdDecoded (s_swDevice2, swMessageId_0, &dmsDecodedMessage, TRUE);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded.Error: error - %i\n", swResult);
			throw 0;
		}
		if (1 != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded.Error: error - %i\n", swResult);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message: message - %i, bsw - %04X, status_1_relevant - %i, status_1 - %04X\n", swMessageId_0, dmsDecodedMessage.wBlockStatus, dmsDecodedMessage.bIsStatus1Relevant, dmsDecodedMessage.wStatus1);
		if (sitalBcBlockStatusWord_NO_RESPONSE == (sitalBcBlockStatusWord_NO_RESPONSE & dmsDecodedMessage.wBlockStatus))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BlockStatus.Error: error - %s\n", "bad_bsw");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		if (wWordCount != dmsDecodedMessage.wDataWordCount)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.Error: error - %s\n", "bad_data_count");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		if ( ! dmsDecodedMessage.bIsStatus1Relevant )
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.Error: error - %s\n", "no_status_1");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		if (sitalIeee1553_Status_SERVICE_REQUEST == (sitalIeee1553_Status_SERVICE_REQUEST & dmsDecodedMessage.wStatus1))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.Error: error - %s\n", "bad_status_1");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		///	Set service request for the response status bits of device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Rt.ResponseStatusBits.Set: unit - %i, status_bits - %04X\n", s_swDevice1, sitalRtStatusWordBit_SERVICE_REQUSET);
		swResult = sitalRt_ResponseStatusBits_Set (s_swDevice1, sitalRtStatusWordBit_SERVICE_REQUSET);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.ResponseStatusBits.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Configure device-1 (RT) to clear service request.
		U16BIT wRegisterValue; // The current value of a target register, or the value to set for a target register.
		wRegisterValue = sitalDevice_Register_Read (s_swDevice1, sitalRegisterAddress_CONFIGURATION_2);
		sitalProcess_Log_PrintLine ("#StldTester.Device.ClearServiceRequest: unit - %i, configuration_reg2(pre) - %04X, configuration_reg2(post) - %04X\n", s_swDevice1, wRegisterValue, (sitalConfigurationRegister2_RT_CLEAR_SERVICE_REQUSET | wRegisterValue));
		swResult = sitalDevice_Register_Write (s_swDevice1, sitalRegisterAddress_CONFIGURATION_2, (sitalConfigurationRegister2_RT_CLEAR_SERVICE_REQUSET | wRegisterValue));
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.ClearServiceRequest.Error: error - %i\n", swResult);
			throw 0;
		}

		for (U16BIT i=0; i<3; i++)
		{
			///	Start device-2 (BC) running major-frame-1 for 1-shot.
			U16BIT wFrameCount; // The number of frames to run.
			wFrameCount = 1;
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", s_swDevice2, swFrameId_1, wFrameCount);
			swResult = sitalBc_Start (s_swDevice2, swFrameId_1, wFrameCount);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
				throw 0;
			}

			///	Wait for the frame run to end.
			U32BIT dwRemainingFrameRunTime; // The expected remaining frame run time (in ms).
			dwRemainingFrameRunTime = (wFrameCount * (wFrameTime / 10));
			swResult = stldTester_Bc_WaitForEndOfFrameRun (s_swDevice2, dwRemainingFrameRunTime, 0, NULL);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.WaitForEndOfFrameRun.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Stop device-2 (BC).
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop: unit - %i\n", s_swDevice2);
			swResult = sitalBc_Stop (s_swDevice2);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Verify that a clear status has been received by device-2 (BC) from device-1 (RT) for message-0.
			// Note: RT devices are initialized to reply with clear status.
			sitalDecodedMessageStructure dmsDecodedMessage; // A decoded message.
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded: unit - %i, message - %i, purge-y\n", s_swDevice2, swMessageId_0);
			swResult = sitalBc_Message_GetByIdDecoded (s_swDevice2, swMessageId_0, &dmsDecodedMessage, TRUE);
			if (0 > swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded.Error: error - %i\n", swResult);
				throw 0;
			}
			if (1 != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded.Error: error - %i\n", swResult);
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message: message - %i, bsw - %04X, status_1_relevant - %i, status_1 - %04X\n", swMessageId_0, dmsDecodedMessage.wBlockStatus, dmsDecodedMessage.bIsStatus1Relevant, dmsDecodedMessage.wStatus1);
			if (sitalBcBlockStatusWord_NO_RESPONSE == (sitalBcBlockStatusWord_NO_RESPONSE & dmsDecodedMessage.wBlockStatus))
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BlockStatus.Error: error - %s\n", "bad_bsw");
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}
			if (wWordCount != dmsDecodedMessage.wDataWordCount)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.Error: error - %s\n", "bad_data_count");
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}
			if ( ! dmsDecodedMessage.bIsStatus1Relevant )
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.Error: error - %s\n", "no_status_1");
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}
			if (sitalIeee1553_Status_SERVICE_REQUEST != (sitalIeee1553_Status_SERVICE_REQUEST & dmsDecodedMessage.wStatus1))
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.Error: error - %s\n", "bad_status_1");
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}
		}

		///	Start device-2 (BC) running major-frame-3 for 1-shot.
		wFrameCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", s_swDevice2, swFrameId_3, wFrameCount);
		swResult = sitalBc_Start (s_swDevice2, swFrameId_3, wFrameCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Wait for the frame run to end.
		dwRemainingFrameRunTime = (wFrameCount * (wFrameTime / 10));
		swResult = stldTester_Bc_WaitForEndOfFrameRun (s_swDevice2, dwRemainingFrameRunTime, 0, NULL);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.WaitForEndOfFrameRun.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Stop device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop: unit - %i\n", s_swDevice2);
		swResult = sitalBc_Stop (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Verify that a clear status has been received by device-2 (BC) from device-1 (RT) for message-1.
		// Note: RT devices are initialized to reply with clear status.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded: unit - %i, message - %i, purge-y\n", s_swDevice2, swMessageId_1);
		swResult = sitalBc_Message_GetByIdDecoded (s_swDevice2, swMessageId_1, &dmsDecodedMessage, TRUE);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded.Error: error - %i\n", swResult);
			throw 0;
		}
		if (1 != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded.Error: error - %i\n", swResult);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message: message - %i, bsw - %04X, status_1_relevant - %i, status_1 - %04X\n", swMessageId_1, dmsDecodedMessage.wBlockStatus, dmsDecodedMessage.bIsStatus1Relevant, dmsDecodedMessage.wStatus1);
		if (sitalBcBlockStatusWord_NO_RESPONSE == (sitalBcBlockStatusWord_NO_RESPONSE & dmsDecodedMessage.wBlockStatus))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BlockStatus.Error: error - %s\n", "bad_bsw");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		if (1 != dmsDecodedMessage.wDataWordCount)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.Error: error - %s\n", "bad_data_count");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		if ( ! dmsDecodedMessage.bIsStatus1Relevant )
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.Error: error - %s\n", "no_status_1");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		if (sitalIeee1553_Status_SERVICE_REQUEST != (sitalIeee1553_Status_SERVICE_REQUEST & dmsDecodedMessage.wStatus1))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.Error: error - %s\n", "bad_status_1");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		///	Unset service request for the response status bits of device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Rt.ResponseStatusBits.Unset: unit - %i, status_bits - %04X\n", s_swDevice1, sitalRtStatusWordBit_SERVICE_REQUSET);
		swResult = sitalRt_ResponseStatusBits_Unset (s_swDevice1, sitalRtStatusWordBit_SERVICE_REQUSET);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.ResponseStatusBits.Error: error - %i\n", swResult);
			throw 0;
		}

		///	Start device-2 (BC) running major-frame-3 for 1-shot.
		wFrameCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", s_swDevice2, swFrameId_3, wFrameCount);
		swResult = sitalBc_Start (s_swDevice2, swFrameId_3, wFrameCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Wait for the frame run to end.
		dwRemainingFrameRunTime = (wFrameCount * (wFrameTime / 10));
		swResult = stldTester_Bc_WaitForEndOfFrameRun (s_swDevice2, dwRemainingFrameRunTime, 0, NULL);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.WaitForEndOfFrameRun.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Stop device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop: unit - %i\n", s_swDevice2);
		swResult = sitalBc_Stop (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Verify that a clear status has been received by device-2 (BC) from device-1 (RT) for message-1.
		// Note: RT devices are initialized to reply with clear status.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded: unit - %i, message - %i, purge-y\n", s_swDevice2, swMessageId_1);
		swResult = sitalBc_Message_GetByIdDecoded (s_swDevice2, swMessageId_1, &dmsDecodedMessage, TRUE);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded.Error: error - %i\n", swResult);
			throw 0;
		}
		if (1 != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded.Error: error - %i\n", swResult);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message: message - %i, bsw - %04X, status_1_relevant - %i, status_1 - %04X\n", swMessageId_1, dmsDecodedMessage.wBlockStatus, dmsDecodedMessage.bIsStatus1Relevant, dmsDecodedMessage.wStatus1);
		if (sitalBcBlockStatusWord_NO_RESPONSE == (sitalBcBlockStatusWord_NO_RESPONSE & dmsDecodedMessage.wBlockStatus))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BlockStatus.Error: error - %s\n", "bad_bsw");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		if (1 != dmsDecodedMessage.wDataWordCount)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.Error: error - %s\n", "bad_data_count");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		if ( ! dmsDecodedMessage.bIsStatus1Relevant )
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.Error: error - %s\n", "no_status_1");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		if (sitalIeee1553_Status_SERVICE_REQUEST == (sitalIeee1553_Status_SERVICE_REQUEST & dmsDecodedMessage.wStatus1))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.Error: error - %s\n", "bad_status_1");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		///		Stop device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Stop: unit - %i\n", s_swDevice1);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Stop (s_swDevice1) : sitalRt_Stop (s_swDevice1) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice1);
		swResult = sitalDevice_Free (s_swDevice1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice2);
		swResult = sitalDevice_Free (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		swResult = stldSyncEnd (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncEnd.Error: error - %i\n", swResult);
			throw 0;
		}
	}
	catch (int)
	{
		sitalDevice_Free (s_swDevice1);
		sitalDevice_Free (s_swDevice2);
	}

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_13k_2.End: test_succeeded - %c\n", ((sitalReturnCode_SUCCESS == swResult) ? 'Y' : 'N'));
	return swResult;
}



/// <summary>
/// Test the stld1553 library in the henceforth described scenario:
/// Participant[s]: RT1, BC2.
/// Configuration: All participating devices must be able to communicate with each other.
/// Short description: Acceptance Test Plan (ATP) test 461_13l - All the Transmit & Receive & Broadcast sub-addresses shall support circular buffer for large data transfers.
/// Along test print suitable info and indications into the log.
///
/// Note:
/// - This function may use any of above defined Acceptance Test Plan (ATP) tests variables.
///   The caller must therefore call the suitable stldTester_Atp_Set* functions prior to calling this function.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Atp_DoTest461_13l ()
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_13l.Start\n");

	/// Set test arguments.
	U16BIT wFrameTime; // The time (in units of 100 us) that the frame will take to complete.
	wFrameTime = 100; // 10ms.

	S16BIT swResult; // Result of operation or function call.
	swResult = sitalReturnCode_SUCCESS;

	/// Get device count.
	U16BIT wRequiredDeviceCount; // The number of devices required for this test.
	wRequiredDeviceCount = (1 + max(s_swDevice1, s_swDevice2));
	U16BIT wAvailableDeviceCount; // The total number of devices available in the system.
	swResult = sitalDevice_GetCount (&wAvailableDeviceCount);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %i\n", swResult);
		return swResult;
	}
	sitalProcess_Log_PrintLine ("#StldTester.Device.Count: available - %i, required - %i\n", wAvailableDeviceCount, wRequiredDeviceCount);
	if (wAvailableDeviceCount < wRequiredDeviceCount)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %s\n", "not_enough_devices_installed");
		return sitalReturnCode_SUCCESS;
	}
	if ( (s_swDevice1 == s_swDevice2) )
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Number.Error: error - %s\n", "same_device_used_twice");
		return sitalReturnCode_INVALID_DEVICE_NUMBER;
	}

	try
	{
		swResult = stldSyncStart (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncStart.Error: error - %i\n", swResult);
			throw 0;
		}

		U16BIT wRtSubaddress; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31).
		for (wRtSubaddress=sitalRtSubaddress_1; wRtSubaddress<=sitalRtSubaddress_30; wRtSubaddress++)
		{
			sitalProcess_Log_PrintLine ("#StldTester.SubTest: subaddress - %i\n", wRtSubaddress);

			/// Initialize device-1 (RT) as a real RT.
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-rt, alloc_mem-none\n", s_swDevice1);
			swResult = sitalDevice_Initialize (s_swDevice1, sitalAccess_CARD, ((s_bIsRtMtModeTest ? sitalMode_RT_AND_MT : sitalMode_RT) | sitalModeVariant_ADVANCED), 0, 0, 0);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Set device-1 (RT) address to RT-address-1.
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set: unit - %i, address - %i\n", s_swDevice1, s_wRtAddress1);
			swResult = sitalRt_Address_Set (s_swDevice1, s_wRtAddress1);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Create for device-1 (RT) data-block-Broadcast, a circular data block of 256 data words.
			S16BIT swRtBroadcastDataBlockId; // A unique ID designating a data block (0-(sitalRtCounter_DATA_BLOCKS-1)).
			swRtBroadcastDataBlockId = 0;
			sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.Create: unit - %i, block - %i, size - %i\n", s_swDevice1, swRtBroadcastDataBlockId, 256);
			swResult = sitalRt_DataBlock_Create (s_swDevice1, swRtBroadcastDataBlockId, sitalRtDataBlockType_CIRCULAR_256, NULL, 0);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.Create.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Create for device-1 (RT) data-block-Rx, a circular data block of 256 data words.
			S16BIT swRtRxDataBlockId; // A unique ID designating a data block (0-(sitalRtCounter_DATA_BLOCKS-1)).
			swRtRxDataBlockId = 1;
			sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.Create: unit - %i, block - %i, size - %i\n", s_swDevice1, swRtRxDataBlockId, 256);
			swResult = sitalRt_DataBlock_Create (s_swDevice1, swRtRxDataBlockId, sitalRtDataBlockType_CIRCULAR_256, NULL, 0);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.Create.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Create for device-1 (RT) data-block-Tx, a circular data block of 256 data words.
			S16BIT swRtTxDataBlockId; // A unique ID designating a data block (0-(sitalRtCounter_DATA_BLOCKS-1)).
			swRtTxDataBlockId = 2;
			U16BIT waBuffer[256]; // A buffer to read-from/write-into along data-block write/read operations, respectively.
			for (int i=0; i<256; i++) waBuffer[i] = i;
			sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i, buf_contents-%i...%i\n", s_swDevice1, swRtTxDataBlockId, 256, (sizeof(waBuffer)/sizeof(U16BIT)), waBuffer[0], waBuffer[(sizeof(waBuffer)/sizeof(U16BIT)) - 1]);
			swResult = sitalRt_DataBlock_Create (s_swDevice1, swRtTxDataBlockId, sitalRtDataBlockType_CIRCULAR_256, waBuffer, (sizeof(waBuffer)/sizeof(U16BIT)));
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.Create.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Map for device-1 (RT) data-block-Broadcast with message-type-Broadcast for current-RtSubAddress, and legalize this subaddress.
			sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.MapToSa: unit - %i, block - %i, sa - %i, msg_types - x%X, irq_opts-x%X, legalize - %c\n", s_swDevice1, swRtBroadcastDataBlockId, wRtSubaddress, sitalRtMessageType_BROADCAST, sitalRtDataBlockIrq_NONE, 'Y');
			swResult = sitalRt_DataBlock_MapToSubaddress (s_swDevice1, swRtBroadcastDataBlockId, wRtSubaddress, sitalRtMessageType_BROADCAST, sitalRtDataBlockIrq_NONE, TRUE);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.MapToSa.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Map for device-1 (RT) data-block-Rx with message-type-Rx for current-RtSubAddress, and legalize this subaddress.
			sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.MapToSa: unit - %i, block - %i, sa - %i, msg_types - x%X, irq_opts-x%X, legalize - %c\n", s_swDevice1, swRtRxDataBlockId, wRtSubaddress, sitalRtMessageType_RX, sitalRtDataBlockIrq_NONE, 'Y');
			swResult = sitalRt_DataBlock_MapToSubaddress (s_swDevice1, swRtRxDataBlockId, wRtSubaddress, sitalRtMessageType_RX, sitalRtDataBlockIrq_NONE, TRUE);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.MapToSa.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Map for device-1 (RT) data-block-Tx with message-type-Tx for current-RtSubAddress, and legalize this subaddress.
			sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.MapToSa: unit - %i, block - %i, sa - %i, msg_types - x%X, irq_opts-x%X, legalize - %c\n", s_swDevice1, swRtTxDataBlockId, wRtSubaddress, sitalRtMessageType_TX, sitalRtDataBlockIrq_NONE, 'Y');
			swResult = sitalRt_DataBlock_MapToSubaddress (s_swDevice1, swRtTxDataBlockId, wRtSubaddress, sitalRtMessageType_TX, sitalRtDataBlockIrq_NONE, TRUE);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.MapToSa.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Start device-1 (RT).
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Start: unit - %i\n", s_swDevice1);
			swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Start (s_swDevice1) : sitalRt_Start (s_swDevice1) );
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Rt.Start.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Initialize device-2 (BC) as a real BC.
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-bc, alloc_mem-none\n", s_swDevice2);
			swResult = sitalDevice_Initialize (s_swDevice2, sitalAccess_CARD, (sitalMode_BC | sitalModeVariant_ADVANCED), 0, 0, 0);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Create for device-2 (BC) data-block-Broadcast, a data block of 32 data words.
			S16BIT swBroadcastDataBlockId; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
			swBroadcastDataBlockId = 1;
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i\n", s_swDevice2, swBroadcastDataBlockId, 32, 0);
			swResult = sitalBc_DataBlock_Create (s_swDevice2, swBroadcastDataBlockId, sitalBcDataBlockSizeOption_SINGLE, NULL, 0);
			if (0 > swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Create for device-2 (BC) data-block-Rx, a data block of 32 data words.
			S16BIT swRxDataBlockId; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
			swRxDataBlockId = 2;
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i\n", s_swDevice2, swRxDataBlockId, 32, 0);
			swResult = sitalBc_DataBlock_Create (s_swDevice2, swRxDataBlockId, sitalBcDataBlockSizeOption_SINGLE, NULL, 0);
			if (0 > swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Create for device-2 (BC) data-block-Tx, a data block of 32 data words.
			S16BIT swTxDataBlockId; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
			swTxDataBlockId = 3;
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i\n", s_swDevice2, swTxDataBlockId, 32, 0);
			swResult = sitalBc_DataBlock_Create (s_swDevice2, swTxDataBlockId, sitalBcDataBlockSizeOption_SINGLE, NULL, 0);
			if (0 > swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Create for device-2 (BC) message-Broadcast, a BC-to-Broadcast(current-RtSubAddress) message that uses data-block-1 to transmit 2 data words. Message bus: bus-1.
			// Note: This message is intentionally wrongly defined, what will fail the test unless the message will be fixed later on (see below).
			S16BIT swBroadcastMessageId; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
			swBroadcastMessageId = 0;
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToBroadcast.Create: unit - %i, message - %i, block - %i, rx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swBroadcastMessageId, swBroadcastDataBlockId, wRtSubaddress, 2, 0, s_wBus1);
			swResult = sitalBc_Message_CreateBcToBroadcast (s_swDevice2, swBroadcastMessageId, swBroadcastDataBlockId, wRtSubaddress, 2, 0, s_wBus1);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToBroadcast.Create.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Modify for device-2 (BC) message-Broadcast, into a BC-to-Broadcast(current-RtSubAddress) message that uses data-block-1 to transmit 32 data words. Message bus: bus-1.
			// Note: This message has been intentionally wrongly defined (see above), so it it therefore fixed here, and only if the fix actually succeeds will the test pass.
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToBroadcast.Modify: unit - %i, message - %i, block - %i, rx_rt_sa - %i, wc - %i, gap - %i, opts-x%X, modified - x%X\n", s_swDevice2, swBroadcastMessageId, swBroadcastDataBlockId, wRtSubaddress, 32, 0, s_wBus1, sitalBcMessageModifier_MESSAGE1_COMMAND1);
			swResult = sitalBc_Message_ModifyBcToBroadcast (s_swDevice2, swBroadcastMessageId, swBroadcastDataBlockId, wRtSubaddress, 32, 0, s_wBus1, sitalBcMessageModifier_MESSAGE1_COMMAND1);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToBroadcast.Modify.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Create for device-2 (BC) message-Tx, a BC-to-RT(RT-address-1/current-RtSubAddress) message that uses data-block-1 to transmit WordCount data words. Message bus: bus-1.
			S16BIT swTxMessageId; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
			swTxMessageId = 1;
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create: unit - %i, message - %i, block - %i, rx_rt - %i, rx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swTxMessageId, swTxDataBlockId, s_wRtAddress1, wRtSubaddress, 32, 0, s_wBus1);
			swResult = sitalBc_Message_CreateBcToRt (s_swDevice2, swTxMessageId, swTxDataBlockId, s_wRtAddress1, wRtSubaddress, 32, 0, s_wBus1);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Create for device-2 (BC) message-Rx, a RT(RT-address-1/current-RtSubAddress)-to-BC message that uses data-block-1 to transmit WordCount data words. Message bus: bus-1.
			S16BIT swRxMessageId; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
			swRxMessageId = 2;
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBc.Create: unit - %i, message - %i, block - %i, tx_rt - %i, tx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swRxMessageId, swRxDataBlockId, s_wRtAddress1, wRtSubaddress, 32, 0, s_wBus1);
			swResult = sitalBc_Message_CreateRtToBc (s_swDevice2, swRxMessageId, swRxDataBlockId, s_wRtAddress1, wRtSubaddress, 32, 0, s_wBus1);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBc.Create.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Create for device-2 (BC) command-0, a XEQ command that executes message-Broadcast.
			S16BIT swCommandId_0; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
			swCommandId_0 = 0;
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swBroadcastMessageId, 0);
			swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swBroadcastMessageId, 0, 0);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Create for device-2 (BC) command-1, a XEQ command that executes message-Tx.
			S16BIT swCommandId_1; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
			swCommandId_1 = 1;
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_1, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swTxMessageId, 0);
			swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_1, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swTxMessageId, 0, 0);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Create for device-2 (BC) command-2, a XEQ command that executes message-Rx.
			S16BIT swCommandId_2; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
			swCommandId_2 = 2;
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_2, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swRxMessageId, 0);
			swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_2, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swRxMessageId, 0, 0);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Create for device-2 (BC) command-3, a subroutine call command that executes minor-frame-0. 
			S16BIT swCommandId_3; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
			swCommandId_3 = 3;
			S16BIT swFrameId_0; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
			swFrameId_0 = 0;
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_3, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, swFrameId_0, 0);
			swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_3, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, (U16BIT)swFrameId_0, 0, 0);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Create for device-2 (BC) minor-frame-0, a frame that contains commands 0-2.
			S16BIT swaCommandIds[sitalBcCounter_COMMANDS]; // An array of command-IDs.
			swaCommandIds[0] = swCommandId_0;
			swaCommandIds[1] = swCommandId_1;
			swaCommandIds[2] = swCommandId_2;
			U16BIT wCommandIdCount; // The number of command IDs in the frame (1-sitalBcCounter_FRAME_COMMANDS).
			wCommandIdCount = 3;
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i, %i, %i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_0, "minor", wCommandIdCount, swCommandId_0, swCommandId_1, swCommandId_2, (wFrameTime / 10), 0);
			swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_0, sitalBcFrameType_MINOR, swaCommandIds, wCommandIdCount, wFrameTime, 0);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Create for device-2 (BC) major-frame-1, a frame that contains command-3 and whose frame time is FrameTime us.
			S16BIT swFrameId_1; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
			swFrameId_1 = 1;
			swaCommandIds[0] = swCommandId_3;
			wCommandIdCount = 1;
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_1, "major", wCommandIdCount, swCommandId_3, (wFrameTime / 10), 0);
			swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_1, sitalBcFrameType_MAJOR, swaCommandIds, wCommandIdCount, wFrameTime, 0);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
				throw 0;
			}

			U16BIT uiRequiredIterationCount; // The number of iterations of Rx/Tx of 32 data words required in order to ultimately verify the correct functioning of a circular data block of 256 data words.
			uiRequiredIterationCount = ((256 / 32) + 1);
			U16BIT wAscendingDataValue; // An ascending data value.
			wAscendingDataValue = 0;
			for (U16BIT iIteration=0; iIteration<uiRequiredIterationCount; iIteration++)
			{
				sitalProcess_Log_PrintLine ("#StldTester.SubTest.Iteration: subaddress - %i, iteration - %i\n", wRtSubaddress, iIteration);

				///	Update for device-2 (BC) the data in the buffer of data-block-Broadcast.
				for (int i=0; i<32; i++) waBuffer[i] = (iIteration + 0);
				sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Write: unit - %i, block - %i, offset - %i, buf_size - %i, buf_contents - %04X...\n", s_swDevice2, swBroadcastDataBlockId, 0, 32, waBuffer[0]);
				swResult = sitalBc_DataBlock_Write (s_swDevice2, swBroadcastDataBlockId, waBuffer, 32, 0);
				if (sitalReturnCode_SUCCESS != swResult)
				{
					sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Write.Error: error - %i\n", swResult);
					throw 0;
				}

				///	Update for device-2 (BC) the data in the buffer of data-block-Tx.
				for (int i=0; i<32; i++) waBuffer[i] = (iIteration + 1);
				sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Write: unit - %i, block - %i, offset - %i, buf_size - %i, buf_contents - %04X...\n", s_swDevice2, swTxDataBlockId, 0, 32, waBuffer[0]);
				swResult = sitalBc_DataBlock_Write (s_swDevice2, swTxDataBlockId, waBuffer, 32, 0);
				if (sitalReturnCode_SUCCESS != swResult)
				{
					sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Write.Error: error - %i\n", swResult);
					throw 0;
				}

				///	Start device-2 (BC) running major-frame-1 for 1-shot.
				U16BIT wFrameCount; // The number of frames to run.
				wFrameCount = 1;
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", s_swDevice2, swFrameId_1, wFrameCount);
				swResult = sitalBc_Start (s_swDevice2, swFrameId_1, wFrameCount);
				if (sitalReturnCode_SUCCESS != swResult)
				{
					sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
					throw 0;
				}

				/// Wait for the frame run to end.
				U32BIT dwRemainingFrameRunTime; // The expected remaining frame run time (in ms).
				dwRemainingFrameRunTime = (wFrameCount * (wFrameTime / 10));
				swResult = stldTester_Bc_WaitForEndOfFrameRun (s_swDevice2, dwRemainingFrameRunTime, 0, NULL);
				if (sitalReturnCode_SUCCESS != swResult)
				{
					sitalProcess_Log_PrintLine ("#StldTester.Bc.WaitForEndOfFrameRun.Error: error - %i\n", swResult);
					throw 0;
				}

				/// Stop device-2 (BC).
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop: unit - %i\n", s_swDevice2);
				swResult = sitalBc_Stop (s_swDevice2);
				if (sitalReturnCode_SUCCESS != swResult)
				{
					sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
					throw 0;
				}

				/// With device-2 (BC) read from data-block-Rx 32 data words.
				sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Read: unit - %i, block - %i, buf_size - %i, offset - %i\n", s_swDevice2, swRxDataBlockId, 32, 0);
				swResult = sitalBc_DataBlock_Read (s_swDevice2, swRxDataBlockId, waBuffer, 32, 0);
				if (0 > swResult)
				{
					sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Read.Error: error - %i\n", swResult);
					throw 0;
				}

				///	With device-2 (BC) verify that data-block-Rx contains the expected data.
				for (int i=0; i<32; i++)
				{
					if (wAscendingDataValue != waBuffer[i])
					{
						sitalProcess_Log_PrintLine ("#StldTester.Bc.ReceivedData.Error: error - %i\n", waBuffer[i]);
						swResult = sitalReturnCode_TEST_FAIL;
						throw 0;
					}

					wAscendingDataValue++;
					wAscendingDataValue = ( (256 == wAscendingDataValue) ? 0 : wAscendingDataValue );
				}

				/// With device-1 (RT) read from data-block-Rx 32 data words.
				sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.Read: unit - %i, block - %i, buf_size - %i, offset - %i\n", s_swDevice1, swRtRxDataBlockId, 32, (iIteration * 32));
				swResult = sitalRt_DataBlock_Read (s_swDevice1, swRtRxDataBlockId, waBuffer, 32, ((iIteration * 32) % 256));
				if (0 > swResult)
				{
					sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.Read.Error: error - %i\n", swResult);
					throw 0;
				}

				///	With device-1 (RT) verify that data-block-Rx contains the expected data.
				for (int i=0; i<32; i++)
				{
					if ((iIteration + 1) != waBuffer[i])
					{
						sitalProcess_Log_PrintLine ("#StldTester.Rt.ReceivedData.Error: error - %i\n", waBuffer[i]);
						swResult = sitalReturnCode_TEST_FAIL;
						throw 0;
					}
				}

				/// With device-1 (RT) read from data-block-Broadcast 32 data words.
				sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.Read: unit - %i, block - %i, buf_size - %i, offset - %i\n", s_swDevice1, swRtBroadcastDataBlockId, 32, (iIteration * 32));
				swResult = sitalRt_DataBlock_Read (s_swDevice1, swRtBroadcastDataBlockId, waBuffer, 32, ((iIteration * 32) % 256));
				if (0 > swResult)
				{
					sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.Read.Error: error - %i\n", swResult);
					throw 0;
				}

				///	With device-1 (RT) verify that data-block-Broadcast contains the expected data.
				for (int i=0; i<32; i++)
				{
					if ((iIteration + 0) != waBuffer[i])
					{
						sitalProcess_Log_PrintLine ("#StldTester.Bc.ReceivedData.Error: error - %i\n", waBuffer[i]);
						swResult = sitalReturnCode_TEST_FAIL;
						throw 0;
					}
				}
			}

			/// Stop device-1 (RT).
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Stop: unit - %i\n", s_swDevice1);
			swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Stop (s_swDevice1) : sitalRt_Stop (s_swDevice1) );
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Rt.Stop.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Free device-2 (BC).
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice2);
			swResult = sitalDevice_Free (s_swDevice2);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
				throw 0;
			}

			/// Free device-1 (RT).
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice1);
			swResult = sitalDevice_Free (s_swDevice1);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
				throw 0;
			}
		}

		swResult = stldSyncEnd (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncEnd.Error: error - %i\n", swResult);
			throw 0;
		}
	}
	catch (int)
	{
		sitalDevice_Free (s_swDevice2);
		sitalDevice_Free (s_swDevice1);
	}

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_13l.End: test_succeeded - %c\n", ((sitalReturnCode_SUCCESS == swResult) ? 'Y' : 'N'));
	return swResult;
}



/// <summary>
/// Test the stld1553 library in the henceforth described scenario:
/// Participant[s]: RT1, BC2.
/// Configuration: All participating devices must be able to communicate with each other.
/// Short description: Acceptance Test Plan (ATP) test 461_13a - RT features: a) Received broadcast messages to illegalized subaddresses are stored in the RT stack though considered illegal. b) RT(legalized-subaddress)-to-RT(illegalized-subaddress)-Broadcast message isn't considered illegal by the transmitter RT.
/// Along test print suitable info and indications into the log.
///
/// Note:
/// - This function may use any of above defined Acceptance Test Plan (ATP) tests variables.
///   The caller must therefore call the suitable stldTester_Atp_Set* functions prior to calling this function.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Atp_DoTest461_13m ()
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_13m.Start\n");

	/// Set test arguments.
	U16BIT wWordCount; // Word count (1-32).
	wWordCount = 32;
	U16BIT wLegalRtSubaddress; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31).
	wLegalRtSubaddress = sitalRtSubaddress_1;
	U16BIT wIllegalRtSubaddress; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31).
	wIllegalRtSubaddress = sitalRtSubaddress_2;
	U16BIT wFrameTime; // The time (in units of 100 us) that the frame will take to complete.
	wFrameTime = 20; // 2ms.

	S16BIT swResult; // Result of operation or function call.
	swResult = sitalReturnCode_SUCCESS;

	/// Get device count.
	U16BIT wRequiredDeviceCount; // The number of devices required for this test.
	wRequiredDeviceCount = (1 + max(s_swDevice1, s_swDevice2));
	U16BIT wAvailableDeviceCount; // The total number of devices available in the system.
	swResult = sitalDevice_GetCount (&wAvailableDeviceCount);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %i\n", swResult);
		return swResult;
	}
	sitalProcess_Log_PrintLine ("#StldTester.Device.Count: available - %i, required - %i\n", wAvailableDeviceCount, wRequiredDeviceCount);
	if (wAvailableDeviceCount < wRequiredDeviceCount)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %s\n", "not_enough_devices_installed");
		return sitalReturnCode_SUCCESS;
	}
	if ( (s_swDevice1 == s_swDevice2) )
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Number.Error: error - %s\n", "same_device_used_twice");
		return sitalReturnCode_INVALID_DEVICE_NUMBER;
	}

	try
	{
		swResult = stldSyncStart (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncStart.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-1 (RT) as a real RT in advanced mode.
		// Note: Advanced mode will enable r/w to the memory of device-1 (RT) while the device is running.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-rt, alloc_mem-none\n", s_swDevice1);
		swResult = sitalDevice_Initialize (s_swDevice1, sitalAccess_CARD, ((s_bIsRtMtModeTest ? sitalMode_RT_AND_MT : sitalMode_RT) | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Set device-1 (RT) address to RtAddress1.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set: unit - %i, address - %i\n", s_swDevice1, s_wRtAddress1);
		swResult = sitalRt_Address_Set (s_swDevice1, s_wRtAddress1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Legalize on device-1 (RT) all address-types in all directions for all subaddresses and all word counts.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.MessageLegality.Enable: unit - %i, address_type - %04X, directions - %04X, subaddress - %04X, word_count - %08X\n", s_swDevice1, sitalRtAddressType_BOTH, sitalMessageDirection_BOTH, sitalRtSubaddress_ALL, 0xFFFFFFFF);
		swResult = sitalRt_MessageLegality_Enable (s_swDevice1, sitalRtAddressType_BOTH, sitalMessageDirection_BOTH, sitalRtSubaddress_ALL, 0xFFFFFFFF);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.MessageLegality.Enable.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Illegalize on device-1 (RT) all address-types in all directions for subaddress-IllegalRtSubaddress and all word counts.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.MessageLegality.Disable: unit - %i, address_type - %04X, directions - %04X, subaddress - %04X, word_count - %08X\n", s_swDevice1, sitalRtAddressType_BOTH, sitalMessageDirection_BOTH, wIllegalRtSubaddress, 0xFFFFFFFF);
		swResult = sitalRt_MessageLegality_Disable (s_swDevice1, sitalRtAddressType_BOTH, sitalMessageDirection_BOTH, wIllegalRtSubaddress, 0xFFFFFFFF);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.MessageLegality.Disable.Error: error - %i\n", swResult);
			throw 0;
		}

		U32BIT dwMessageLegality; // Message legality.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.MessageLegality.Get: unit - %i, address_type - %04X, directions - %04X, subaddress - %04X\n", s_swDevice1, sitalRtAddressType_OWN, sitalMessageDirection_RX, wIllegalRtSubaddress);
		swResult = sitalRt_MessageLegality_GetStatus (s_swDevice1, sitalRtAddressType_OWN, sitalMessageDirection_RX, wIllegalRtSubaddress, &dwMessageLegality);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.MessageLegality.Get.Error: error - %i\n", swResult);
			throw 0;
		}
		sitalProcess_Log_PrintLine ("#StldTester.Rt.MessageLegality.Get: unit - %i, address_type - %04X, directions - %04X, subaddress - %04X, legality - %08X\n", s_swDevice1, sitalRtAddressType_OWN, sitalMessageDirection_RX, wIllegalRtSubaddress, dwMessageLegality);
		if (0xFFFFFFFF != dwMessageLegality)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.MessageLegality.Get.Error: error - %s\n", "bad_legality");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// Start device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Start: unit - %i\n", s_swDevice1);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Start (s_swDevice1) : sitalRt_Start (s_swDevice1) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-2 (BC) as a real BC.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-bc, alloc_mem-none\n", s_swDevice2);
		swResult = sitalDevice_Initialize (s_swDevice2, sitalAccess_CARD, (sitalMode_BC | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) data-block-0, a data block of 32 data words.
		S16BIT swDataBlockId_0; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId_0 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i\n", s_swDevice2, swDataBlockId_0, 32, 0);
		swResult = sitalBc_DataBlock_Create (s_swDevice2, swDataBlockId_0, sitalBcDataBlockSizeOption_SINGLE, NULL, wWordCount);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) data-block-1, a data block of 32 data words.
		S16BIT swDataBlockId_1; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId_1 = 2;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i\n", s_swDevice2, swDataBlockId_1, 32, 0);
		swResult = sitalBc_DataBlock_Create (s_swDevice2, swDataBlockId_1, sitalBcDataBlockSizeOption_SINGLE, NULL, wWordCount);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) message-0, a BC-to-Broadcast(IllegalRtSubaddress) message that uses data-block-0 to transmit WordCount data words. Message bus: bus-1.
		S16BIT swMessageId_0; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToBroadcast.Create: unit - %i, message - %i, block - %i, rx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_0, swDataBlockId_0, wIllegalRtSubaddress, wWordCount, 0, s_wBus1);
		swResult = sitalBc_Message_CreateBcToBroadcast (s_swDevice2, swMessageId_0, swDataBlockId_0, wIllegalRtSubaddress, wWordCount, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToBroadcast.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Use given message descriptors to code the command word of above defined message.
		U16BIT wCommandWord0; // Command word for above defined BC-to-Broadcast message-0.
		swResult = sitalBcStld1553_Command_GetCode (&wCommandWord0, sital1553_BROADCAST, sitalCommand_RX, wIllegalRtSubaddress, wWordCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Command.GetCode.Error: error - %i\n", swResult);
			throw 0;
		}
		sitalProcess_Log_PrintLine ("#StldTester.Command.GetCode.BcToBroadcast: command - %04X\n", wCommandWord0);

		/// Create for device-2 (BC) message-1, a RT(RtAddress/LegalRtSubaddress)-to-Broadcast(IllegalRtSubaddress) message that transmits WordCount data words. BC side data-block: data-block-2. Message bus: Wrong-bus-1.
		// Note: This message is intentionally wrongly defined, what will fail the test unless the message will be fixed later on (see below).
		S16BIT swMessageId_1; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_1 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBroadcast.Create: unit - %i, message - %i, block - %i, tx_rt - %i, tx_rt_sa - %i, rx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_1, swDataBlockId_1, s_wRtAddress1, wLegalRtSubaddress, wIllegalRtSubaddress, wWordCount, 0, s_wBus1);
		swResult = sitalBc_Message_CreateRtToBroadcast (s_swDevice2, swMessageId_1, swDataBlockId_1, wIllegalRtSubaddress, wWordCount, s_wRtAddress1, wLegalRtSubaddress, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBroadcast.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Use given message descriptors to code the command word of both above defined messages.
		// Note: This message will be read from the stack of the transmitter RT, where only the transmission command is recorded, as if it was the single command.
		U16BIT wCommandWord1; // Command word for above defined RT-to-Broadcast message-1.
		swResult = sitalBcStld1553_Command_GetCode (&wCommandWord1, s_wRtAddress1, sitalCommand_TX, wLegalRtSubaddress, wWordCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Command.GetCode.Error: error - %i\n", swResult);
			throw 0;
		}
		sitalProcess_Log_PrintLine ("#StldTester.Command.GetCode.RtToBroadcast: command - %04X\n", wCommandWord1);

		/// Create for device-2 (BC) command-0, a XEQ command that executes message-0.
		S16BIT swCommandId_0; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-1, a XEQ command that executes message-1.
		S16BIT swCommandId_1; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_1 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_1, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_1, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_1, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_1, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-2, a subroutine call command that executes minor-frame-0.
		S16BIT swCommandId_2; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_2 = 2;
		S16BIT swFrameId_0; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_2, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, swFrameId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_2, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, (U16BIT)swFrameId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) minor-frame-0, a frame that contains command-0 and command-1.
		S16BIT swaCommandIds[sitalBcCounter_COMMANDS]; // An array of command-IDs.
		swaCommandIds[0] = swCommandId_0;
		swaCommandIds[1] = swCommandId_1;
		U16BIT wCommandIdCount; // The number of command IDs in the frame (1-sitalBcCounter_FRAME_COMMANDS).
		wCommandIdCount = 2;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i,%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_0, "minor", wCommandIdCount, swCommandId_0, swCommandId_1, 0, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_0, sitalBcFrameType_MINOR, swaCommandIds, wCommandIdCount, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) major-frame-1, a frame that contains command-2 and whose time is FrameTime.
		S16BIT swFrameId_1; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_1 = 1;
		swaCommandIds[0] = swCommandId_2;
		wCommandIdCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_1, "major", wCommandIdCount, swCommandId_1, wFrameTime, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_1, sitalBcFrameType_MAJOR, swaCommandIds, wCommandIdCount, wFrameTime, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		///	Start device-2 (BC) running major-frame-1 for 1-shot.
		U16BIT wFrameCount; // The number of frames to run.
		wFrameCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", s_swDevice2, swFrameId_1, wFrameCount);
		swResult = sitalBc_Start (s_swDevice2, swFrameId_1, wFrameCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Wait for the frame run to end.
		U32BIT dwRemainingFrameRunTime; // The expected remaining frame run time (in ms).
		dwRemainingFrameRunTime = (wFrameCount * (wFrameTime / 10));
		swResult = stldTester_Bc_WaitForEndOfFrameRun (s_swDevice2, dwRemainingFrameRunTime, 0, NULL);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.WaitForEndOfFrameRun.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Stop device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop: unit - %i\n", s_swDevice2);
		swResult = sitalBc_Stop (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Stop device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Stop: unit - %i\n", s_swDevice1);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Stop (s_swDevice1) : sitalRt_Stop (s_swDevice1) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// With device-1 (RT) verify detection of message-1 with expected status.
		/// Note: Message-0 is sent to an illegal RT subaddress, so it should not put it in its stack.
		sitalDecodedMessageStructure dmsDecodedMessage; // A decoded message.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Message.GetFromStackDecoded: unit - %i, message - %i, purge - %i\n", s_swDevice1, swMessageId_1, TRUE);
		swResult = sitalRt_Message_GetFromStackDecoded (s_swDevice1, &dmsDecodedMessage, sitalMessageLocationAndRemoval_NEXT_PURGE);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Message.GetDecoded.Error: error - %i\n", swResult);
			throw 0;
		}
		if (1 != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Message.GetDecoded.Error: error - %i\n", swResult);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Message.GetDecoded: unit - %i, message - %i, command1 - %04X, command2 - %04X, is_command2 - %i, expected_command1 - %04X, message_block_status - %04X, status1 - %04X, status2 - %04X\n", s_swDevice1, swMessageId_1, dmsDecodedMessage.wCommandWord1, dmsDecodedMessage.wCommandWord2, dmsDecodedMessage.bIsCommandWord2Relevant, wCommandWord1, dmsDecodedMessage.wBlockStatus, dmsDecodedMessage.wStatus1, dmsDecodedMessage.wStatus2);
		// Note: This message will be read from the stack of the transmitter RT, where only the transmission command is recorded, as if it was the single command.
		if (dmsDecodedMessage.bIsCommandWord2Relevant)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Message.GetDecoded.Error: error - %s\n", "one_command");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		if (wCommandWord1 != dmsDecodedMessage.wCommandWord1)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Message.GetDecoded.Error: error - %s\n", "bad_command");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// With device-2 (BC) verify detection of message-1 with expected block status and status.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded: unit - %i, message - %i, purge-y\n", s_swDevice2, swMessageId_1);
		swResult = sitalBc_Message_GetByIdDecoded (s_swDevice2, swMessageId_1, &dmsDecodedMessage, TRUE);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded.Error: error - %i\n", swResult);
			throw 0;
		}
		if (1 != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded.Error: error - %i\n", swResult);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message: message - %i, bsw - %04X, status_1_relevant - %i, status_1 - %04X\n", swMessageId_1, dmsDecodedMessage.wBlockStatus, dmsDecodedMessage.bIsStatus1Relevant, dmsDecodedMessage.wStatus1);
		if ( ! dmsDecodedMessage.bIsStatus1Relevant )
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.Error: error - %s\n", "no_status_1");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		// NOTE: The following check will fail the test, as the Sital RT device raises the ME status bit in case it is the transmitter RT in a RT(transmitter-RT/legal-subaddress)-to-RT(illegal-subaddress)-Broadcast message.
		if (sitalIeee1553_Status_MESSAGE_ERROR == (sitalIeee1553_Status_MESSAGE_ERROR & dmsDecodedMessage.wStatus1))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.GetDecoded.Error: error - %s\n", "bad_status1");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// Free device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice1);
		swResult = sitalDevice_Free (s_swDevice1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice2);
		swResult = sitalDevice_Free (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		swResult = stldSyncEnd (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncEnd.Error: error - %i\n", swResult);
			throw 0;
		}
	}
	catch (int)
	{
		sitalDevice_Free (s_swDevice1);
		sitalDevice_Free (s_swDevice2);
	}

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_13m.End: test_succeeded - %c\n", ((sitalReturnCode_SUCCESS == swResult) ? 'Y' : 'N'));
	return swResult;
}



/// <summary>
/// Test the stld1553 library in the henceforth described scenario:
/// Participant[s]: BC2, MT3.
/// Configuration: All participating devices must be able to communicate with each other.
/// Short description: Acceptance Test Plan (ATP) test 461_15b - Selective Message Monitor.
/// Along test print suitable info and indications into the log.
///
/// Note:
/// - This function may use any of above defined Acceptance Test Plan (ATP) tests variables.
///   The caller must therefore call the suitable stldTester_Atp_Set* functions prior to calling this function.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Atp_DoTest461_15b ()
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_15b.Start\n");

	/// Set test arguments.
	U16BIT wWordCount; // Word count (1-32).
	wWordCount = 32;
	U16BIT wFrameTime; // The time (in units of 100 us) that the frame will take to complete.
	wFrameTime = 1000; // 100ms.

	S16BIT swResult; // Result of operation or function call.
	swResult = sitalReturnCode_SUCCESS;

	/// Get device count.
	U16BIT wRequiredDeviceCount; // The number of devices required for this test.
	wRequiredDeviceCount = (1 + max(s_swDevice2, s_swDevice3));
	U16BIT wAvailableDeviceCount; // The total number of devices available in the system.
	swResult = sitalDevice_GetCount (&wAvailableDeviceCount);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %i\n", swResult);
		return swResult;
	}
	sitalProcess_Log_PrintLine ("#StldTester.Device.Count: available - %i, required - %i\n", wAvailableDeviceCount, wRequiredDeviceCount);
	if (wAvailableDeviceCount < wRequiredDeviceCount)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %s\n", "not_enough_devices_installed");
		return sitalReturnCode_SUCCESS;
	}
	if ( (s_swDevice2 == s_swDevice3) )
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Number.Error: error - %s\n", "same_device_used_twice");
		return sitalReturnCode_INVALID_DEVICE_NUMBER;
	}

	try
	{
		swResult = stldSyncStart (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncStart.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-3 (MT) as a real MT.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access - card, mode - mt, alloc_mem - none\n", s_swDevice3);
		swResult = sitalDevice_Initialize (s_swDevice3, sitalAccess_CARD, ((s_bIsRtMtModeTest ? sitalMode_RT_AND_MT : sitalMode_MT) | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		// Disable device-2 (MT) from recording any messages
		sitalProcess_Log_PrintLine ("#StldTester.Mt.MessageMonitoring.Disable: unit - %i, rt_address - %i, direction - %i, subaddresses - x%08X\n", s_swDevice3, sitalRtAddress_ALL, sitalMessageDirection_BOTH, sitalRtSubaddressMask_ALL);
		swResult = sitalMt_MessageMonitoring_Disable (s_swDevice3, sitalRtAddress_ALL, sitalMessageDirection_BOTH, sitalRtSubaddressMask_ALL);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.MessageMonitoring.Disable.Error: error - %i\n", swResult);
			throw 0;
		}

		// Enable device-2 (MT) to record any Rx messages.
		sitalProcess_Log_PrintLine ("#StldTester.Mt.MessageMonitoring.Enable: unit - %i, rt_address - %04X, direction - %i, subaddresses - x%08X\n", s_swDevice3, sitalRtAddress_ALL, sitalMessageDirection_RX, sitalRtSubaddressMask_ALL);
		swResult = sitalMt_MessageMonitoring_Enable (s_swDevice3, sitalRtAddress_ALL, sitalMessageDirection_RX, sitalRtSubaddressMask_ALL);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.MessageMonitoring.Enable.Error: error - %i\n", swResult);
			throw 0;
		}

		// Verify that device-2 (MT) is enabled to record any Rx messages.
		sitalProcess_Log_PrintLine ("#StldTester.Mt.MessageMonitoring.Status.Get: unit - %i, rt_address - %i, direction - %i\n", s_swDevice3, sitalRtAddress_0, sitalMessageDirection_RX);
		U32BIT dwSubaddressMask; // A mask of monitored subaddresses, 0-31, where bit #i corresponds subaddress #i, what forms an or-ed combination of sitalRtSubaddressMask_*.
		swResult = sitalMt_MessageMonitoring_GetStatus (s_swDevice3, sitalRtAddress_0, sitalMessageDirection_RX, &dwSubaddressMask);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.MessageMonitoring.GetStatus.Error: error - %i\n", swResult);
			throw 0;
		}
		if (sitalRtSubaddressMask_ALL != dwSubaddressMask)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.MessageMonitoring.GetStatus.Error: error - %08X\n", dwSubaddressMask);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// Start device-3 (MT).
		sitalProcess_Log_PrintLine ("#StldTester.Mt.Start: unit - %i\n", s_swDevice3);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Start (s_swDevice3) : sitalMt_Start (s_swDevice3) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-2 (BC) as a real BC.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-bc, alloc_mem-none\n", s_swDevice2);
		swResult = sitalDevice_Initialize (s_swDevice2, sitalAccess_CARD, (sitalMode_BC | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) data-block-0, a data block of 64 data words.
		S16BIT swDataBlockId_0; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId_0 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i\n", s_swDevice2, swDataBlockId_0, 64, 0);
		swResult = sitalBc_DataBlock_Create (s_swDevice2, swDataBlockId_0, sitalBcDataBlockSizeOption_DOUBLE, NULL, 0);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) messsitalProcess_Log_Start (char const *szProfileName);age-0, a BC-to-RT(RT-address-1/RtSubAddress message that uses data-block-0 to transmit WordCount data words. Message bus: bus-1.
		S16BIT swMessageId_0; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_0 = 0;
		U16BIT wRtSubaddress; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31).
		wRtSubaddress = sitalRtSubaddress_1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create: unit - %i, message - %i, block - %i, rx_rt - %i, rx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddress, wWordCount, 0, s_wBus1);
		swResult = sitalBc_Message_CreateBcToRt (s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddress, wWordCount, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		///	Use given message descriptors to code the command word of above defined message.
		// Note: Defined messages have been intentionally devised in a way that their [first] command word will differ.
		U16BIT wExpectedCommandWord; // A 1553 command word.
		swResult = sitalBcStld1553_Command_GetCode (&wExpectedCommandWord, s_wRtAddress1, sitalCommand_RX, wRtSubaddress, wWordCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Command.GetCode.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) message-1, a RT(RT-address-1/RtSubAddress)-to-BC message that uses data-block-0 to transmit WordCount data words. Message bus: bus-1.
		S16BIT swMessageId_1; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_1 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBc.Create: unit - %i, message - %i, block - %i, tx_rt - %i, tx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_1, swDataBlockId_0, s_wRtAddress1, wRtSubaddress, wWordCount, 0, s_wBus1);
		swResult = sitalBc_Message_CreateRtToBc (s_swDevice2, swMessageId_1, swDataBlockId_0, s_wRtAddress1, wRtSubaddress, wWordCount, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBc.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-0, a XEQ command that executes message-0.
		S16BIT swCommandId_0; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-1, a XEQ command that executes message-1.
		S16BIT swCommandId_1; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_1 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_1, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_1, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_1, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_1, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-2, a subroutine call command that executes minor-frame-0.
		S16BIT swCommandId_2; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_2 = 2;
		S16BIT swFrameId_0; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_2, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, swFrameId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_2, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, (U16BIT)swFrameId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) minor-frame-0, a frame that contains command-0 and command-1.
		S16BIT swaCommandIds[sitalBcCounter_COMMANDS]; // An array of command-IDs.
		swaCommandIds[0] = swCommandId_0;
		swaCommandIds[1] = swCommandId_1;
		U16BIT wCommandIdCount; // The number of command IDs in the frame (1-sitalBcCounter_FRAME_COMMANDS).
		wCommandIdCount = 2;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i %i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_0, "minor", wCommandIdCount, swCommandId_0, swCommandId_1, 0, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_0, sitalBcFrameType_MINOR, swaCommandIds, wCommandIdCount, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) major-frame-1, a frame that contains command-2.
		S16BIT swFrameId_1; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_1 = 1;
		swaCommandIds[0] = swCommandId_2;
		wCommandIdCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_1, "major", wCommandIdCount, swCommandId_2, wFrameTime, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_1, sitalBcFrameType_MAJOR, swaCommandIds, wCommandIdCount, wFrameTime, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		///	Start device-2 (BC) running major-frame-2 for 1-shot.
		U16BIT wFrameCount; // The number of frames to run.
		wFrameCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", s_swDevice2, swFrameId_1, wFrameCount);
		swResult = sitalBc_Start (s_swDevice2, swFrameId_1, wFrameCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		///	Wait for the frame run to end.
		U32BIT dwRemainingFrameRunTime; // The expected remaining frame run time (in ms).
		dwRemainingFrameRunTime = (wFrameCount * (wFrameTime / 10));
		swResult = stldTester_Bc_WaitForEndOfFrameRun (s_swDevice2, dwRemainingFrameRunTime, 0, NULL);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.WaitForEndOfFrameRun.Error: error - %i\n", swResult);
			throw 0;
		}

		///	Stop device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop: unit - %i\n", s_swDevice2);
		swResult = sitalBc_Stop (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		///	Verify that device-3 (MT) has only detected/avoided the expected messages.
		sitalDecodedMessageStructure dmsDecodedMessage; // A decoded message.
		sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded: unit - %i, purge - %i\n", s_swDevice3, TRUE);
		swResult = sitalMt_Message_GetFromStackDecoded (s_swDevice3, &dmsDecodedMessage, sitalMessageLocationAndRemoval_NEXT_PURGE, sitalMtStack_ACTIVE);
		if ((0 > swResult) || (1 < swResult))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Error: error - %i\n", swResult);
			throw 0;
		}
		if (0U == swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Message.Error: error - %s\n", "no_message");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		sitalProcess_Log_PrintLine ("#StldTester.Mt.Message: unit - %i, command - %04X, expected_command - %04X, data_count - %i, data - %i...%i, time_tag - %04X\n", s_swDevice3, dmsDecodedMessage.wCommandWord1, wExpectedCommandWord, dmsDecodedMessage.wDataWordCount, dmsDecodedMessage.waData[0], dmsDecodedMessage.waData[dmsDecodedMessage.wDataWordCount - 1], dmsDecodedMessage.wTimeTag);
		if (wExpectedCommandWord != dmsDecodedMessage.wCommandWord1)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Message.Error: error - %s\n", "bad_command");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		///	Verify that device-2 (MT) hasn't recorded any additional messages.
		sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded: unit - %i, purge - %i\n", s_swDevice3, TRUE);
		swResult = sitalMt_Message_GetFromStackDecoded (s_swDevice3, &dmsDecodedMessage, sitalMessageLocationAndRemoval_NEXT_PURGE, sitalMtStack_ACTIVE);
		if ((0 > swResult) || (1 < swResult))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Error: error - %i\n", swResult);
			throw 0;
		}
		if (1 == swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Error: error - %s\n", "extra_message");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// Stop device-3 (MT).
		sitalProcess_Log_PrintLine ("#StldTester.Mt.Stop: unit - %i\n", s_swDevice3);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Stop (s_swDevice3) : sitalMt_Stop (s_swDevice3) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice2);
		swResult = sitalDevice_Free (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-3 (MT).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice3);
		swResult = sitalDevice_Free (s_swDevice3);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		swResult = stldSyncEnd (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncEnd.Error: error - %i\n", swResult);
			throw 0;
		}
	}
	catch (int)
	{
		sitalDevice_Free (s_swDevice2);
		sitalDevice_Free (s_swDevice3);
	}

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_15b.End: test_succeeded - %c\n", ((sitalReturnCode_SUCCESS == swResult) ? 'Y' : 'N'));
	return swResult;
}



/// <summary>
/// Test the stld1553 library in the henceforth described scenario:
/// Participant[s]: RT1, BC2.
/// Configuration: All participating devices must be able to communicate with each other.
/// Short description: Acceptance Test Plan (ATP) test 461_15e - RT response time.
/// Along test print suitable info and indications into the log.
///
/// Note:
/// - This function may use any of above defined Acceptance Test Plan (ATP) tests variables.
///   The caller must therefore call the suitable stldTester_Atp_Set* functions prior to calling this function.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Atp_DoTest461_15e ()
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	S16BIT swResult; // Result of operation or function call.
	swResult = sitalReturnCode_SUCCESS;

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_15e.Start\n");

	swResult = stldTester_Atp_DoTest461_11k_3 ();

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_15e.End: test_succeeded - %c\n", ((sitalReturnCode_SUCCESS == swResult) ? 'Y' : 'N'));
	return swResult;
}



/// <summary>
/// Test the stld1553 library in the henceforth described scenario:
/// Participant[s]: BC2, MT3.
/// Configuration: All participating devices must be able to communicate with each other.
/// Short description: Acceptance Test Plan (ATP) test 461_15f - Interrupts can be selected by RT.
/// Along test print suitable info and indications into the log.
///
/// Note:
/// - This function may use any of above defined Acceptance Test Plan (ATP) tests variables.
///   The caller must therefore call the suitable stldTester_Atp_Set* functions prior to calling this function.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Atp_DoTest461_15f ()
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_15f.Start\n");

	/// Set test arguments.
	U16BIT wWordCount; // Word count (1-32).
	wWordCount = 32;
	U16BIT wFrameTime; // The time (in units of 100 us) that the frame will take to complete.
	wFrameTime = 1000; // 100ms.

	S16BIT swResult; // Result of operation or function call.
	swResult = sitalReturnCode_SUCCESS;

	/// Get device count.
	U16BIT wRequiredDeviceCount; // The number of devices required for this test.
	wRequiredDeviceCount = (1 + max(s_swDevice2, s_swDevice3));
	U16BIT wAvailableDeviceCount; // The total number of devices available in the system.
	swResult = sitalDevice_GetCount (&wAvailableDeviceCount);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %i\n", swResult);
		return swResult;
	}
	sitalProcess_Log_PrintLine ("#StldTester.Device.Count: available - %i, required - %i\n", wAvailableDeviceCount, wRequiredDeviceCount);
	if (wAvailableDeviceCount < wRequiredDeviceCount)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %s\n", "not_enough_devices_installed");
		return sitalReturnCode_SUCCESS;
	}
	if ( (s_swDevice2 == s_swDevice3) )
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Number.Error: error - %s\n", "same_device_used_twice");
		return sitalReturnCode_INVALID_DEVICE_NUMBER;
	}

	try
	{
		swResult = stldSyncStart (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncStart.Error: error - %i\n", swResult);
			throw 0;
		}

		if (s_wRtAddress1 == s_wRtAddress2)
		{
			sitalProcess_Log_PrintLine ("#StldTester.TestConfiguration.Error: error - %s\n", "same_rt_address");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// Initialize device-3 (MT) as a real MT.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access - card, mode - mt, alloc_mem - none\n", s_swDevice3);
		swResult = sitalDevice_Initialize (s_swDevice3, sitalAccess_CARD, ((s_bIsRtMtModeTest ? sitalMode_RT_AND_MT : sitalMode_MT) | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		// Disable device-2 (MT) from recording any messages
		sitalProcess_Log_PrintLine ("#StldTester.Mt.MessageMonitoring.Disable: unit - %i, rt_address - %i, direction - %i, subaddresses - x%08X\n", s_swDevice3, sitalRtAddress_ALL, sitalMessageDirection_BOTH, sitalRtSubaddressMask_ALL);
		swResult = sitalMt_MessageMonitoring_Disable (s_swDevice3, sitalRtAddress_ALL, sitalMessageDirection_BOTH, sitalRtSubaddressMask_ALL);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.MessageMonitoring.Disable.Error: error - %i\n", swResult);
			throw 0;
		}

		// Enable device-2 (MT) to record all messages to/from RT-address-1.
		sitalProcess_Log_PrintLine ("#StldTester.Mt.MessageMonitoring.Enable: unit - %i, rt_address - %04X, direction - %i, subaddresses - x%08X\n", s_swDevice3, s_wRtAddress1, sitalMessageDirection_RX, sitalRtSubaddressMask_ALL);
		swResult = sitalMt_MessageMonitoring_Enable (s_swDevice3, s_wRtAddress1, sitalMessageDirection_BOTH, sitalRtSubaddressMask_ALL);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.MessageMonitoring.Enable.Error: error - %i\n", swResult);
			throw 0;
		}

		// Verify that device-2 (MT) is enabled to record all messages to/from RT-address-1.
		sitalProcess_Log_PrintLine ("#StldTester.Mt.MessageMonitoring.Status.Get: unit - %i, rt_address - %i, direction - %i\n", s_swDevice3, s_wRtAddress1, sitalMessageDirection_RX);
		U32BIT dwSubaddressMask; // A mask of monitored subaddresses, 0-31, where bit #i corresponds subaddress #i, what forms an or-ed combination of sitalRtSubaddressMask_*.
		swResult = sitalMt_MessageMonitoring_GetStatus (s_swDevice3, s_wRtAddress1, sitalMessageDirection_RX, &dwSubaddressMask);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.MessageMonitoring.GetStatus.Error: error - %i\n", swResult);
			throw 0;
		}
		if (sitalRtSubaddressMask_ALL != dwSubaddressMask)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.MessageMonitoring.GetStatus.Error: error - %08X\n", dwSubaddressMask);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		sitalProcess_Log_PrintLine ("#StldTester.Mt.MessageMonitoring.Status.Get: unit - %i, rt_address - %i, direction - %i\n", s_swDevice3, s_wRtAddress1, sitalMessageDirection_TX);
		swResult = sitalMt_MessageMonitoring_GetStatus (s_swDevice3, s_wRtAddress1, sitalMessageDirection_TX, &dwSubaddressMask);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.MessageMonitoring.GetStatus.Error: error - %i\n", swResult);
			throw 0;
		}
		if (sitalRtSubaddressMask_ALL != dwSubaddressMask)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.MessageMonitoring.GetStatus.Error: error - %08X\n", dwSubaddressMask);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		// Verify that device-2 (MT) is disabled to record all messages to/from RT-address-2.
		sitalProcess_Log_PrintLine ("#StldTester.Mt.MessageMonitoring.Status.Get: unit - %i, rt_address - %i, direction - %i\n", s_swDevice3, s_wRtAddress2, sitalMessageDirection_RX);
		swResult = sitalMt_MessageMonitoring_GetStatus (s_swDevice3, s_wRtAddress2, sitalMessageDirection_RX, &dwSubaddressMask);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.MessageMonitoring.GetStatus.Error: error - %i\n", swResult);
			throw 0;
		}
		if (0U != dwSubaddressMask)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.MessageMonitoring.GetStatus.Error: error - %08X\n", dwSubaddressMask);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		sitalProcess_Log_PrintLine ("#StldTester.Mt.MessageMonitoring.Status.Get: unit - %i, rt_address - %i, direction - %i\n", s_swDevice3, s_wRtAddress2, sitalMessageDirection_TX);
		swResult = sitalMt_MessageMonitoring_GetStatus (s_swDevice3, s_wRtAddress2, sitalMessageDirection_TX, &dwSubaddressMask);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.MessageMonitoring.GetStatus.Error: error - %i\n", swResult);
			throw 0;
		}
		if (0U != dwSubaddressMask)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.MessageMonitoring.GetStatus.Error: error - %08X\n", dwSubaddressMask);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// Start device-3 (MT).
		sitalProcess_Log_PrintLine ("#StldTester.Mt.Start: unit - %i\n", s_swDevice3);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Start (s_swDevice3) : sitalMt_Start (s_swDevice3) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-2 (BC) as a real BC.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-bc, alloc_mem-none\n", s_swDevice2);
		swResult = sitalDevice_Initialize (s_swDevice2, sitalAccess_CARD, (sitalMode_BC | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) data-block-0, a data block of 64 data words.
		S16BIT swDataBlockId_0; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId_0 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i\n", s_swDevice2, swDataBlockId_0, 64, 0);
		swResult = sitalBc_DataBlock_Create (s_swDevice2, swDataBlockId_0, sitalBcDataBlockSizeOption_DOUBLE, NULL, 0);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) message-0, a BC-to-RT(RT-address-1/RtSubAddress) message that uses data-block-0 to transmit WordCount data words. Message bus: bus-1.
		S16BIT swMessageId_0; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_0 = 0;
		U16BIT wRtSubaddress; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31).
		wRtSubaddress = sitalRtSubaddress_1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create: unit - %i, message - %i, block - %i, rx_rt - %i, rx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddress, wWordCount, 0, s_wBus1);
		swResult = sitalBc_Message_CreateBcToRt (s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddress, wWordCount, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		///	Use given message descriptors to code the command word of above defined message.
		// Note: Defined messages have been intentionally devised in a way that their [first] command word will differ.
		U16BIT wExpectedCommandWord; // A 1553 command word.
		swResult = sitalBcStld1553_Command_GetCode (&wExpectedCommandWord, s_wRtAddress1, sitalCommand_RX, wRtSubaddress, wWordCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Command.GetCode.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) message-1, a BC-to-RT(RT-address-2/RtSubAddress2 message that uses data-block-0 to transmit WordCount data words. Message bus: bus-1.
		S16BIT swMessageId_1; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_1 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create: unit - %i, message - %i, block - %i, rx_rt - %i, rx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_1, swDataBlockId_0, s_wRtAddress2, wRtSubaddress, wWordCount, 0, s_wBus1);
		swResult = sitalBc_Message_CreateBcToRt (s_swDevice2, swMessageId_1, swDataBlockId_0, s_wRtAddress2, wRtSubaddress, wWordCount, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-0, a XEQ command that executes message-0.
		S16BIT swCommandId_0; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-1, a XEQ command that executes message-1.
		S16BIT swCommandId_1; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_1 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_1, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_1, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_1, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_1, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-2, a subroutine call command that executes minor-frame-0.
		S16BIT swCommandId_2; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_2 = 2;
		S16BIT swFrameId_0; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_2, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, swFrameId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_2, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, (U16BIT)swFrameId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) minor-frame-0, a frame that contains command-0 and command-1.
		S16BIT swaCommandIds[sitalBcCounter_COMMANDS]; // An array of command-IDs.
		swaCommandIds[0] = swCommandId_0;
		swaCommandIds[1] = swCommandId_1;
		U16BIT wCommandIdCount; // The number of command IDs in the frame (1-sitalBcCounter_FRAME_COMMANDS).
		wCommandIdCount = 2;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i %i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_0, "minor", wCommandIdCount, swCommandId_0, swCommandId_1, 0, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_0, sitalBcFrameType_MINOR, swaCommandIds, wCommandIdCount, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) major-frame-1, a frame that contains command-2.
		S16BIT swFrameId_1; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_1 = 1;
		swaCommandIds[0] = swCommandId_2;
		wCommandIdCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_1, "major", wCommandIdCount, swCommandId_2, wFrameTime, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_1, sitalBcFrameType_MAJOR, swaCommandIds, wCommandIdCount, wFrameTime, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		///	Start device-2 (BC) running major-frame-2 for 1-shot.
		U16BIT wFrameCount; // The number of frames to run.
		wFrameCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", s_swDevice2, swFrameId_1, wFrameCount);
		swResult = sitalBc_Start (s_swDevice2, swFrameId_1, wFrameCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		///	Wait for the frame run to end.
		U32BIT dwRemainingFrameRunTime; // The expected remaining frame run time (in ms).
		dwRemainingFrameRunTime = (wFrameCount * (wFrameTime / 10));
		swResult = stldTester_Bc_WaitForEndOfFrameRun (s_swDevice2, dwRemainingFrameRunTime, 0, NULL);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.WaitForEndOfFrameRun.Error: error - %i\n", swResult);
			throw 0;
		}

		///	Stop device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop: unit - %i\n", s_swDevice2);
		swResult = sitalBc_Stop (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		///	Verify that device-3 (MT) has only detected/avoided the expected messages.
		sitalDecodedMessageStructure dmsDecodedMessage; // A decoded message.
		sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded: unit - %i, purge - %i\n", s_swDevice3, TRUE);
		swResult = sitalMt_Message_GetFromStackDecoded (s_swDevice3, &dmsDecodedMessage, sitalMessageLocationAndRemoval_NEXT_PURGE, sitalMtStack_ACTIVE);
		if ((0 > swResult) || (1 < swResult))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Error: error - %i\n", swResult);
			throw 0;
		}
		if (0U == swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Message.Error: error - %s\n", "no_message");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		sitalProcess_Log_PrintLine ("#StldTester.Mt.Message: unit - %i, command - %04X, expected_command - %04X, data_count - %i, data - %i...%i, time_tag - %04X\n", s_swDevice3, dmsDecodedMessage.wCommandWord1, wExpectedCommandWord, dmsDecodedMessage.wDataWordCount, dmsDecodedMessage.waData[0], dmsDecodedMessage.waData[dmsDecodedMessage.wDataWordCount - 1], dmsDecodedMessage.wTimeTag);
		if (wExpectedCommandWord != dmsDecodedMessage.wCommandWord1)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Message.Error: error - %s\n", "bad_command");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		///	Verify that device-2 (MT) hasn't recorded any additional messages.
		sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded: unit - %i, purge - %i\n", s_swDevice3, TRUE);
		swResult = sitalMt_Message_GetFromStackDecoded (s_swDevice3, &dmsDecodedMessage, sitalMessageLocationAndRemoval_NEXT_PURGE, sitalMtStack_ACTIVE);
		if ((0 > swResult) || (1 < swResult))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Error: error - %i\n", swResult);
			throw 0;
		}
		if (1 == swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Error: error - %s\n", "extra_message");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// Stop device-3 (MT).
		sitalProcess_Log_PrintLine ("#StldTester.Mt.Stop: unit - %i\n", s_swDevice3);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Stop (s_swDevice3) : sitalMt_Stop (s_swDevice3) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice2);
		swResult = sitalDevice_Free (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-3 (MT).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice3);
		swResult = sitalDevice_Free (s_swDevice3);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		swResult = stldSyncEnd (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncEnd.Error: error - %i\n", swResult);
			throw 0;
		}
	}
	catch (int)
	{
		sitalDevice_Free (s_swDevice2);
		sitalDevice_Free (s_swDevice3);
	}

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_15f.End: test_succeeded - %c\n", ((sitalReturnCode_SUCCESS == swResult) ? 'Y' : 'N'));
	return swResult;
}



/// <summary>
/// Test the stld1553 library in the henceforth described scenario:
/// Participant[s]: BC2, MT3.
/// Configuration: All participating devices must be able to communicate with each other.
/// Short description: Acceptance Test Plan (ATP) test 461_15h - Extensive filtering options by individual RT sub address.
/// Along test print suitable info and indications into the log.
///
/// Note:
/// - This function may use any of above defined Acceptance Test Plan (ATP) tests variables.
///   The caller must therefore call the suitable stldTester_Atp_Set* functions prior to calling this function.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Atp_DoTest461_15h ()
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_15h.Start\n");

	/// Set test arguments.
	U16BIT wWordCount; // Word count (1-32).
	wWordCount = 32;
	U16BIT wFrameTime; // The time (in units of 100 us) that the frame will take to complete.
	wFrameTime = 1000; // 100ms.

	S16BIT swResult; // Result of operation or function call.
	swResult = sitalReturnCode_SUCCESS;

	/// Get device count.
	U16BIT wRequiredDeviceCount; // The number of devices required for this test.
	wRequiredDeviceCount = (1 + max(s_swDevice2, s_swDevice3));
	U16BIT wAvailableDeviceCount; // The total number of devices available in the system.
	swResult = sitalDevice_GetCount (&wAvailableDeviceCount);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %i\n", swResult);
		return swResult;
	}
	sitalProcess_Log_PrintLine ("#StldTester.Device.Count: available - %i, required - %i\n", wAvailableDeviceCount, wRequiredDeviceCount);
	if (wAvailableDeviceCount < wRequiredDeviceCount)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %s\n", "not_enough_devices_installed");
		return sitalReturnCode_SUCCESS;
	}
	if ( (s_swDevice2 == s_swDevice3) )
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Number.Error: error - %s\n", "same_device_used_twice");
		return sitalReturnCode_INVALID_DEVICE_NUMBER;
	}

	try
	{
		swResult = stldSyncStart (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncStart.Error: error - %i\n", swResult);
			throw 0;
		}

		U16BIT wRtSubaddress1; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31).
		wRtSubaddress1 = sitalRtSubaddress_1;
		U16BIT wRtSubaddress2; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31).
		wRtSubaddress2 = sitalRtSubaddress_2; // Must be different than wRtSubaddress1!
		if (wRtSubaddress1 == wRtSubaddress2)
		{
			sitalProcess_Log_PrintLine ("#StldTester.TestConfiguration.Error: error - %s\n", "same_rt_subaddress");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// Initialize device-3 (MT) as a real MT.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access - card, mode - mt, alloc_mem - none\n", s_swDevice3);
		swResult = sitalDevice_Initialize (s_swDevice3, sitalAccess_CARD, ((s_bIsRtMtModeTest ? sitalMode_RT_AND_MT : sitalMode_MT) | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		// Disable device-2 (MT) from recording any messages
		sitalProcess_Log_PrintLine ("#StldTester.Mt.MessageMonitoring.Disable: unit - %i, rt_address - %i, direction - %i, subaddresses - x%08X\n", s_swDevice3, sitalRtAddress_ALL, sitalMessageDirection_BOTH, sitalRtSubaddressMask_ALL);
		swResult = sitalMt_MessageMonitoring_Disable (s_swDevice3, sitalRtAddress_ALL, sitalMessageDirection_BOTH, sitalRtSubaddressMask_ALL);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.MessageMonitoring.Disable.Error: error - %i\n", swResult);
			throw 0;
		}

		// Enable device-2 (MT) to record all messages to/from RT-subaddress-RtSubAddress1.
		sitalProcess_Log_PrintLine ("#StldTester.Mt.MessageMonitoring.Enable: unit - %i, rt_address - %04X, direction - %i, subaddresses - x%08X\n", s_swDevice3, sitalRtAddress_ALL, sitalMessageDirection_RX, ((U32BIT)(1 << wRtSubaddress1)));
		swResult = sitalMt_MessageMonitoring_Enable (s_swDevice3, sitalRtAddress_ALL, sitalMessageDirection_BOTH, ((U32BIT)(1 << wRtSubaddress1)));
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.MessageMonitoring.Enable.Error: error - %i\n", swResult);
			throw 0;
		}

		// Verify that device-2 (MT) is enabled to record all messages to/from RT-subaddress-RtSubAddress1.
		sitalProcess_Log_PrintLine ("#StldTester.Mt.MessageMonitoring.Status.Get: unit - %i, rt_address - %i, direction - %i\n", s_swDevice3, sitalRtAddress_0, sitalMessageDirection_RX);
		U32BIT dwSubaddressMask; // A mask of monitored subaddresses, 0-31, where bit #i corresponds subaddress #i, what forms an or-ed combination of sitalRtSubaddressMask_*.
		swResult = sitalMt_MessageMonitoring_GetStatus (s_swDevice3, sitalRtAddress_0, sitalMessageDirection_RX, &dwSubaddressMask);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.MessageMonitoring.GetStatus.Error: error - %i\n", swResult);
			throw 0;
		}
		if (((U32BIT)(1 << wRtSubaddress1)) != dwSubaddressMask)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.MessageMonitoring.GetStatus.Error: error - %08X\n", dwSubaddressMask);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		sitalProcess_Log_PrintLine ("#StldTester.Mt.MessageMonitoring.Status.Get: unit - %i, rt_address - %i, direction - %i\n", s_swDevice3, sitalRtAddress_0, sitalMessageDirection_TX);
		swResult = sitalMt_MessageMonitoring_GetStatus (s_swDevice3, sitalRtAddress_0, sitalMessageDirection_TX, &dwSubaddressMask);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.MessageMonitoring.GetStatus.Error: error - %i\n", swResult);
			throw 0;
		}
		if (((U32BIT)(1 << wRtSubaddress1)) != dwSubaddressMask)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.MessageMonitoring.GetStatus.Error: error - %08X\n", dwSubaddressMask);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// Start device-3 (MT).
		sitalProcess_Log_PrintLine ("#StldTester.Mt.Start: unit - %i\n", s_swDevice3);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Start (s_swDevice3) : sitalMt_Start (s_swDevice3) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-2 (BC) as a real BC.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-bc, alloc_mem-none\n", s_swDevice2);
		swResult = sitalDevice_Initialize (s_swDevice2, sitalAccess_CARD, (sitalMode_BC | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) data-block-0, a data block of 64 data words.
		S16BIT swDataBlockId_0; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId_0 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i\n", s_swDevice2, swDataBlockId_0, 64, 0);
		swResult = sitalBc_DataBlock_Create (s_swDevice2, swDataBlockId_0, sitalBcDataBlockSizeOption_DOUBLE, NULL, 0);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) message-0, a BC-to-RT(RT-address-1/RtSubAddress1) message that uses data-block-0 to transmit WordCount data words. Message bus: bus-1.
		S16BIT swMessageId_0; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create: unit - %i, message - %i, block - %i, rx_rt - %i, rx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddress1, wWordCount, 0, s_wBus1);
		swResult = sitalBc_Message_CreateBcToRt (s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddress1, wWordCount, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		///	Use given message descriptors to code the command word of above defined message.
		// Note: Defined messages have been intentionally devised in a way that their [first] command word will differ.
		U16BIT wExpectedCommandWord; // A 1553 command word.
		swResult = sitalBcStld1553_Command_GetCode (&wExpectedCommandWord, s_wRtAddress1, sitalCommand_RX, wRtSubaddress1, wWordCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Command.GetCode.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) message-1, a BC-to-RT(RT-address-1/RtSubAddress2 message that uses data-block-0 to transmit WordCount data words. Message bus: bus-1.
		S16BIT swMessageId_1; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_1 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create: unit - %i, message - %i, block - %i, rx_rt - %i, rx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_1, swDataBlockId_0, s_wRtAddress1, wRtSubaddress2, wWordCount, 0, s_wBus1);
		swResult = sitalBc_Message_CreateBcToRt (s_swDevice2, swMessageId_1, swDataBlockId_0, s_wRtAddress1, wRtSubaddress2, wWordCount, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-0, a XEQ command that executes message-0.
		S16BIT swCommandId_0; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-1, a XEQ command that executes message-1.
		S16BIT swCommandId_1; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_1 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_1, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_1, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_1, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_1, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-2, a subroutine call command that executes minor-frame-0.
		S16BIT swCommandId_2; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_2 = 2;
		S16BIT swFrameId_0; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_2, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, swFrameId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_2, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, (U16BIT)swFrameId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) minor-frame-0, a frame that contains command-0 and command-1.
		S16BIT swaCommandIds[sitalBcCounter_COMMANDS]; // An array of command-IDs.
		swaCommandIds[0] = swCommandId_0;
		swaCommandIds[1] = swCommandId_1;
		U16BIT wCommandIdCount; // The number of command IDs in the frame (1-sitalBcCounter_FRAME_COMMANDS).
		wCommandIdCount = 2;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i %i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_0, "minor", wCommandIdCount, swCommandId_0, swCommandId_1, 0, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_0, sitalBcFrameType_MINOR, swaCommandIds, wCommandIdCount, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) major-frame-1, a frame that contains command-2.
		S16BIT swFrameId_1; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_1 = 1;
		swaCommandIds[0] = swCommandId_2;
		wCommandIdCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_1, "major", wCommandIdCount, swCommandId_2, wFrameTime, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_1, sitalBcFrameType_MAJOR, swaCommandIds, wCommandIdCount, wFrameTime, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		///	Start device-2 (BC) running major-frame-2 for 1-shot.
		U16BIT wFrameCount; // The number of frames to run.
		wFrameCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", s_swDevice2, swFrameId_1, wFrameCount);
		swResult = sitalBc_Start (s_swDevice2, swFrameId_1, wFrameCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		///	Wait for the frame run to end.
		U32BIT dwRemainingFrameRunTime; // The expected remaining frame run time (in ms).
		dwRemainingFrameRunTime = (wFrameCount * (wFrameTime / 10));
		swResult = stldTester_Bc_WaitForEndOfFrameRun (s_swDevice2, dwRemainingFrameRunTime, 0, NULL);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.WaitForEndOfFrameRun.Error: error - %i\n", swResult);
			throw 0;
		}

		///	Stop device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop: unit - %i\n", s_swDevice2);
		swResult = sitalBc_Stop (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		///	Verify that device-3 (MT) has only detected/avoided the expected messages.
		sitalDecodedMessageStructure dmsDecodedMessage; // A decoded message.
		sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded: unit - %i, purge - %i\n", s_swDevice3, TRUE);
		swResult = sitalMt_Message_GetFromStackDecoded (s_swDevice3, &dmsDecodedMessage, sitalMessageLocationAndRemoval_NEXT_PURGE, sitalMtStack_ACTIVE);
		if ((0 > swResult) || (1 < swResult))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Error: error - %i\n", swResult);
			throw 0;
		}
		if (0U == swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Message.Error: error - %s\n", "no_message");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		sitalProcess_Log_PrintLine ("#StldTester.Mt.Message: unit - %i, command - %04X, expected_command - %04X, data_count - %i, data - %i...%i, time_tag - %04X\n", s_swDevice3, dmsDecodedMessage.wCommandWord1, wExpectedCommandWord, dmsDecodedMessage.wDataWordCount, dmsDecodedMessage.waData[0], dmsDecodedMessage.waData[dmsDecodedMessage.wDataWordCount - 1], dmsDecodedMessage.wTimeTag);
		if (wExpectedCommandWord != dmsDecodedMessage.wCommandWord1)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Message.Error: error - %s\n", "bad_command");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		///	Verify that device-2 (MT) hasn't recorded any additional messages.
		sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded: unit - %i, purge - %i\n", s_swDevice3, TRUE);
		swResult = sitalMt_Message_GetFromStackDecoded (s_swDevice3, &dmsDecodedMessage, sitalMessageLocationAndRemoval_NEXT_PURGE, sitalMtStack_ACTIVE);
		if ((0 > swResult) || (1 < swResult))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Error: error - %i\n", swResult);
			throw 0;
		}
		if (1 == swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Error: error - %s\n", "extra_message");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// Stop device-3 (MT).
		sitalProcess_Log_PrintLine ("#StldTester.Mt.Stop: unit - %i\n", s_swDevice3);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Stop (s_swDevice3) : sitalMt_Stop (s_swDevice3) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice2);
		swResult = sitalDevice_Free (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-3 (MT).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice3);
		swResult = sitalDevice_Free (s_swDevice3);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		swResult = stldSyncEnd (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncEnd.Error: error - %i\n", swResult);
			throw 0;
		}
	}
	catch (int)
	{
		sitalDevice_Free (s_swDevice2);
		sitalDevice_Free (s_swDevice3);
	}

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_15h.End: test_succeeded - %c\n", ((sitalReturnCode_SUCCESS == swResult) ? 'Y' : 'N'));
	return swResult;
}



/// <summary>
/// Test the stld1553 library in the henceforth described scenario:
/// Participant[s]: BC2, MT3.
/// Configuration: All participating devices must be able to communicate with each other.
/// Short description: Acceptance Test Plan (ATP) test 461_15i - Transmit, receive or broadcast mode codes.
/// Along test print suitable info and indications into the log.
///
/// Note:
/// - This function may use any of above defined Acceptance Test Plan (ATP) tests variables.
///   The caller must therefore call the suitable stldTester_Atp_Set* functions prior to calling this function.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Atp_DoTest461_15i ()
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_15i.Start\n");

	/// Set test arguments.
	U16BIT wWordCount; // Word count (1-32).
	wWordCount = 32;
	U16BIT wFrameTime; // The time (in units of 100 us) that the frame will take to complete.
	wFrameTime = 1000; // 100ms.

	S16BIT swResult; // Result of operation or function call.
	swResult = sitalReturnCode_SUCCESS;

	/// Get device count.
	U16BIT wRequiredDeviceCount; // The number of devices required for this test.
	wRequiredDeviceCount = (1 + max(s_swDevice2, s_swDevice3));
	U16BIT wAvailableDeviceCount; // The total number of devices available in the system.
	swResult = sitalDevice_GetCount (&wAvailableDeviceCount);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %i\n", swResult);
		return swResult;
	}
	sitalProcess_Log_PrintLine ("#StldTester.Device.Count: available - %i, required - %i\n", wAvailableDeviceCount, wRequiredDeviceCount);
	if (wAvailableDeviceCount < wRequiredDeviceCount)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %s\n", "not_enough_devices_installed");
		return sitalReturnCode_SUCCESS;
	}
	if ( (s_swDevice2 == s_swDevice3) )
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Number.Error: error - %s\n", "same_device_used_twice");
		return sitalReturnCode_INVALID_DEVICE_NUMBER;
	}

	try
	{
		swResult = stldSyncStart (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncStart.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-3 (MT) as a real MT.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access - card, mode - mt, alloc_mem - none\n", s_swDevice3);
		swResult = sitalDevice_Initialize (s_swDevice3, sitalAccess_CARD, ((s_bIsRtMtModeTest ? sitalMode_RT_AND_MT : sitalMode_MT) | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		// Disable device-2 (MT) from recording any messages
		sitalProcess_Log_PrintLine ("#StldTester.Mt.MessageMonitoring.Disable: unit - %i, rt_address - %i, direction - %i, subaddresses - x%08X\n", s_swDevice3, sitalRtAddress_ALL, sitalMessageDirection_BOTH, sitalRtSubaddressMask_ALL);
		swResult = sitalMt_MessageMonitoring_Disable (s_swDevice3, sitalRtAddress_ALL, sitalMessageDirection_BOTH, sitalRtSubaddressMask_ALL);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.MessageMonitoring.Disable.Error: error - %i\n", swResult);
			throw 0;
		}

		// Enable device-2 (MT) to record all mode code messages.
		sitalProcess_Log_PrintLine ("#StldTester.Mt.MessageMonitoring.Enable: unit - %i, rt_address - %04X, direction - %i, subaddresses - x%08X\n", s_swDevice3, sitalRtAddress_ALL, sitalMessageDirection_RX, ((U32BIT)((1 << sitalRtSubaddress_0) | (1 << sitalRtSubaddress_31))));
		swResult = sitalMt_MessageMonitoring_Enable (s_swDevice3, sitalRtAddress_ALL, sitalMessageDirection_BOTH, ((U32BIT)((1 << sitalRtSubaddress_0) | (1 << sitalRtSubaddress_31))));
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.MessageMonitoring.Enable.Error: error - %i\n", swResult);
			throw 0;
		}

		// Verify that device-2 (MT) is enabled to record all messages to/from RT-subaddress-RtSubAddress0.
		sitalProcess_Log_PrintLine ("#StldTester.Mt.MessageMonitoring.Status.Get: unit - %i, rt_address - %i, direction - %i\n", s_swDevice3, sitalRtAddress_0, sitalMessageDirection_RX);
		U32BIT dwSubaddressMask; // A mask of monitored subaddresses, 0-31, where bit #i corresponds subaddress #i, what forms an or-ed combination of sitalRtSubaddressMask_*.
		swResult = sitalMt_MessageMonitoring_GetStatus (s_swDevice3, sitalRtAddress_0, sitalMessageDirection_RX, &dwSubaddressMask);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.MessageMonitoring.GetStatus.Error: error - %i\n", swResult);
			throw 0;
		}
		if (((U32BIT)((1 << sitalRtSubaddress_0) | (1 << sitalRtSubaddress_31))) != dwSubaddressMask)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.MessageMonitoring.GetStatus.Error: error - %08X\n", dwSubaddressMask);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		sitalProcess_Log_PrintLine ("#StldTester.Mt.MessageMonitoring.Status.Get: unit - %i, rt_address - %i, direction - %i\n", s_swDevice3, sitalRtAddress_0, sitalMessageDirection_TX);
		swResult = sitalMt_MessageMonitoring_GetStatus (s_swDevice3, sitalRtAddress_0, sitalMessageDirection_TX, &dwSubaddressMask);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.MessageMonitoring.GetStatus.Error: error - %i\n", swResult);
			throw 0;
		}
		if (((U32BIT)((1 << sitalRtSubaddress_0) | (1 << sitalRtSubaddress_31))) != dwSubaddressMask)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.MessageMonitoring.GetStatus.Error: error - %08X\n", dwSubaddressMask);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// Start device-3 (MT).
		sitalProcess_Log_PrintLine ("#StldTester.Mt.Start: unit - %i\n", s_swDevice3);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Start (s_swDevice3) : sitalMt_Start (s_swDevice3) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-2 (BC) as a real BC.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-bc, alloc_mem-none\n", s_swDevice2);
		swResult = sitalDevice_Initialize (s_swDevice2, sitalAccess_CARD, (sitalMode_BC | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) data-block-0, a data block of 64 data words.
		S16BIT swDataBlockId_0; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId_0 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i\n", s_swDevice2, swDataBlockId_0, 64, 0);
		swResult = sitalBc_DataBlock_Create (s_swDevice2, swDataBlockId_0, sitalBcDataBlockSizeOption_DOUBLE, NULL, 0);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		///	Create for device-2 (BC) message-0, a RT(RtAddress)-to-BC-Mode(code=4) message. Message bus: bus-1.
		// Note: This message is intentionally wrongly defined, what will fail the test unless the message will be fixed later on (see below).
		S16BIT swMessageId_0; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_0 = 0;
		U16BIT wModeCommand; // Mode command (1-32).
		wModeCommand = 0x0004; // A wrong code.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBcMode.Create: unit - %i, message - %i, block - %i, rt - %i, direction - %i, mode - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, sitalMessageDirection_TX, wModeCommand, 0, s_wBus1);
		swResult = sitalBc_Message_CreateBcToOrFromRtMode (s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, sitalMessageDirection_TX, wModeCommand, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBcMode.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Modify for device-2 (BC) message-2, into a RT(RtAddress)-to-BC-Mode(code=2) message. Message bus: bus-1.
		// Note: This message has been intentionally wrongly defined (see above), so it it therefore fixed here, and only if the fix actually succeeds will the test pass.
		wModeCommand = 0x0002; // The correct code.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBcMode.Modify: unit - %i, message - %i, block - %i, rt - %i, direction - %i, mode - %i, gap - %i, opts-x%X, modified - x%X\n", s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, sitalMessageDirection_TX, wModeCommand, 0, s_wBus1, sitalBcMessageModifier_MESSAGE1_COMMAND1);
		swResult = sitalBc_Message_ModifyBcToOrFromRtMode (s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, sitalMessageDirection_TX, wModeCommand, 0, s_wBus1, sitalBcMessageModifier_MESSAGE1_COMMAND1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToRt.Modify.Error: error - %i\n", swResult);
			throw 0;
		}

		///	Use given message descriptors to code the command word of above defined message.
		// Note: Defined messages have been intentionally devised in a way that their [first] command word will differ.
		U16BIT wExpectedCommandWord; // A 1553 command word.
		swResult = sitalBcStld1553_Command_GetCode (&wExpectedCommandWord, s_wRtAddress1, sitalCommand_TX, sitalRtSubaddress_0, wModeCommand);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Command.GetCode.Error: error - %i\n", swResult);
			throw 0;
		}

		///	Verify the correctness of the command word that was calculated for above defined message.
		sitalProcess_Log_PrintLine ("#StldTester.Command.Parse: command - %04X\n", wExpectedCommandWord);
		U16BIT wRtAddress; // RT address.
		U16BIT wMessageDirection; // RT-related message direction (sitalMessageDirection_RX or sitalMessageDirection_TX).
		U16BIT wSubaddressOrMode; // Given message's subaddress-or-mode field.
		U16BIT wWordCountOrModeCode; // Word-count/mode-word, depending whether regular/mode command, respectively (word-count: 1-32; mode-word: 1-32).
		swResult = sitalBcStld1553_Command_Parse (wExpectedCommandWord, &wRtAddress, &wMessageDirection, &wSubaddressOrMode, &wWordCountOrModeCode);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Command.Parse.Error: error - %i\n", swResult);
			throw 0;
		}
		sitalProcess_Log_PrintLine ("#StldTester.Command.Parse: rt - %i, direction - %i, mode - %i, mode_command - %i\n", s_wRtAddress1, sitalMessageDirection_TX, wSubaddressOrMode, wWordCountOrModeCode);
		if ( (s_wRtAddress1 != wRtAddress) || (sitalMessageDirection_TX != wMessageDirection) || (sitalRtSubaddress_0 != wSubaddressOrMode) || (wModeCommand != wWordCountOrModeCode) )
		{
			sitalProcess_Log_PrintLine ("#StldTester.Command.Parse.Error: error - %s\n", "bad_1553_command");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// Create for device-2 (BC) message-1, a RT(RT-address-1/RtSubAddress)-to-BC message that uses data-block-0 to receive WordCount data words. Message bus: bus-1.
		S16BIT swMessageId_1; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_1 = 1;
		U16BIT wRtSubaddress; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31).
		wRtSubaddress = sitalRtSubaddress_1; // Must be different than wRtSubaddress0!
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBc.Create: unit - %i, message - %i, block - %i, tx_rt - %i, tx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_1, swDataBlockId_0, s_wRtAddress1, wRtSubaddress, wWordCount, 0, s_wBus1);
		swResult = sitalBc_Message_CreateRtToBc (s_swDevice2, swMessageId_1, swDataBlockId_0, s_wRtAddress1, wRtSubaddress, wWordCount, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBc.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-0, a XEQ command that executes message-0.
		S16BIT swCommandId_0; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-1, a XEQ command that executes message-1.
		S16BIT swCommandId_1; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_1 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_1, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_1, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_1, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_1, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-2, a subroutine call command that executes minor-frame-0.
		S16BIT swCommandId_2; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_2 = 2;
		S16BIT swFrameId_0; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_2, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, swFrameId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_2, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, (U16BIT)swFrameId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) minor-frame-0, a frame that contains command-0 and command-1.
		S16BIT swaCommandIds[sitalBcCounter_COMMANDS]; // An array of command-IDs.
		swaCommandIds[0] = swCommandId_0;
		swaCommandIds[1] = swCommandId_1;
		U16BIT wCommandIdCount; // The number of command IDs in the frame (1-sitalBcCounter_FRAME_COMMANDS).
		wCommandIdCount = 2;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i %i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_0, "minor", wCommandIdCount, swCommandId_0, swCommandId_1, 0, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_0, sitalBcFrameType_MINOR, swaCommandIds, wCommandIdCount, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) major-frame-1, a frame that contains command-2.
		S16BIT swFrameId_1; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_1 = 1;
		swaCommandIds[0] = swCommandId_2;
		wCommandIdCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_1, "major", wCommandIdCount, swCommandId_2, wFrameTime, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_1, sitalBcFrameType_MAJOR, swaCommandIds, wCommandIdCount, wFrameTime, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		///	Start device-2 (BC) running major-frame-2 for 1-shot.
		U16BIT wFrameCount; // The number of frames to run.
		wFrameCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", s_swDevice2, swFrameId_1, wFrameCount);
		swResult = sitalBc_Start (s_swDevice2, swFrameId_1, wFrameCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Wait for the frame run to end.
		U32BIT dwRemainingFrameRunTime; // The expected remaining frame run time (in ms).
		dwRemainingFrameRunTime = (wFrameCount * (wFrameTime / 10));
		swResult = stldTester_Bc_WaitForEndOfFrameRun (s_swDevice2, dwRemainingFrameRunTime, 0, NULL);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.WaitForEndOfFrameRun.Error: error - %i\n", swResult);
			throw 0;
		}

		///	Stop device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop: unit - %i\n", s_swDevice2);
		swResult = sitalBc_Stop (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		///	Verify that device-3 (MT) has only detected/avoided the expected messages.
		sitalDecodedMessageStructure dmsDecodedMessage; // A decoded message.
		sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded: unit - %i, purge - %i\n", s_swDevice3, TRUE);
		swResult = sitalMt_Message_GetFromStackDecoded (s_swDevice3, &dmsDecodedMessage, sitalMessageLocationAndRemoval_NEXT_PURGE, sitalMtStack_ACTIVE);
		if ((0 > swResult) || (1 < swResult))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Error: error - %i\n", swResult);
			throw 0;
		}
		if (0U == swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Message.Error: error - %s\n", "no_message");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		sitalProcess_Log_PrintLine ("#StldTester.Mt.Message: unit - %i, command - %04X, expected_command - %04X, data_count - %i, data - %i...%i, time_tag - %04X\n", s_swDevice3, dmsDecodedMessage.wCommandWord1, wExpectedCommandWord, dmsDecodedMessage.wDataWordCount, dmsDecodedMessage.waData[0], dmsDecodedMessage.waData[dmsDecodedMessage.wDataWordCount - 1], dmsDecodedMessage.wTimeTag);
		if (wExpectedCommandWord != dmsDecodedMessage.wCommandWord1)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Message.Error: error - %s\n", "bad_command");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		///	Verify that device-2 (MT) hasn't recorded any additional messages.
		sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded: unit - %i, purge - %i\n", s_swDevice3, TRUE);
		swResult = sitalMt_Message_GetFromStackDecoded (s_swDevice3, &dmsDecodedMessage, sitalMessageLocationAndRemoval_NEXT_PURGE, sitalMtStack_ACTIVE);
		if ((0 > swResult) || (1 < swResult))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Error: error - %i\n", swResult);
			throw 0;
		}
		if (1 == swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Error: error - %s\n", "extra_message");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// Stop device-3 (MT).
		sitalProcess_Log_PrintLine ("#StldTester.Mt.Stop: unit - %i\n", s_swDevice3);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Stop (s_swDevice3) : sitalMt_Stop (s_swDevice3) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice2);
		swResult = sitalDevice_Free (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-3 (MT).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice3);
		swResult = sitalDevice_Free (s_swDevice3);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		swResult = stldSyncEnd (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncEnd.Error: error - %i\n", swResult);
			throw 0;
		}
	}
	catch (int)
	{
		sitalDevice_Free (s_swDevice2);
		sitalDevice_Free (s_swDevice3);
	}

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_15i.End: test_succeeded - %c\n", ((sitalReturnCode_SUCCESS == swResult) ? 'Y' : 'N'));
	return swResult;
}



/// <summary>
/// Test the stld1553 library in the henceforth described scenario:
/// Participant[s]: BC2, MT3.
/// Configuration: All participating devices must be able to communicate with each other.
/// Short description: Acceptance Test Plan (ATP) test 461_15g - Up to 60 store buffers with 16 bit time tag.
/// Along test print suitable info and indications into the log.
///
/// Note:
/// - This function may use any of above defined Acceptance Test Plan (ATP) tests variables.
///   The caller must therefore call the suitable stldTester_Atp_Set* functions prior to calling this function.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Atp_DoTest461_15g ()
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_15g.Start\n");

	/// Set test arguments.
	U16BIT wRtSubaddress; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31).
	wRtSubaddress = 1;
	U16BIT wWordCount; // Word count (1-32).
	wWordCount = 32;
	U16BIT wFrameTime; // The time (in units of 100 us) that the frame will take to complete.
	wFrameTime = 100; // 10ms.
	U16BIT wTimeTagCalculationInaccuracy; // The maximum tolerated difference between the expected time tag and the actual one.
	wTimeTagCalculationInaccuracy = 25;

	S16BIT swResult; // Result of operation or function call.
	swResult = sitalReturnCode_SUCCESS;

	/// Get device count.
	U16BIT wRequiredDeviceCount; // The number of devices required for this test.
	wRequiredDeviceCount = (1 + max(s_swDevice2, s_swDevice3));
	U16BIT wAvailableDeviceCount; // The total number of devices available in the system.
	swResult = sitalDevice_GetCount (&wAvailableDeviceCount);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %i\n", swResult);
		return swResult;
	}
	sitalProcess_Log_PrintLine ("#StldTester.Device.Count: available - %i, required - %i\n", wAvailableDeviceCount, wRequiredDeviceCount);
	if (wAvailableDeviceCount < wRequiredDeviceCount)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %s\n", "not_enough_devices_installed");
		return sitalReturnCode_SUCCESS;
	}
	if ( (s_swDevice2 == s_swDevice3) )
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Number.Error: error - %s\n", "same_device_used_twice");
		return sitalReturnCode_INVALID_DEVICE_NUMBER;
	}

	try
	{
		swResult = stldSyncStart (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncStart.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-3 (MT) as a real MT.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access - card, mode - mt, alloc_mem - none\n", s_swDevice3);
		swResult = sitalDevice_Initialize (s_swDevice3, sitalAccess_CARD, ((s_bIsRtMtModeTest ? sitalMode_RT_AND_MT : sitalMode_MT) | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Set for device-3 (MT) a time tag resolution of 2 us.
		U16BIT wTimeTagResolution; // Time tag resolution (in units of 1 us).
		wTimeTagResolution = 2;
		sitalProcess_Log_PrintLine ("#StldTester.Device.TimeTagResolution.Set: unit - %i, resolution - %ius\n", s_swDevice3);
		swResult = sitalDevice_TimeTag_SetResolution (s_swDevice3, sitalTimeTagResolution_2US);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.TimeTagResolution.Set.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Start device-3 (MT).
		sitalProcess_Log_PrintLine ("#StldTester.Mt.Start: unit - %i\n", s_swDevice3);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Start (s_swDevice3) : sitalMt_Start (s_swDevice3) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-2 (BC) as a real BC.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-bc, alloc_mem-none\n", s_swDevice2);
		swResult = sitalDevice_Initialize (s_swDevice2, sitalAccess_CARD, (sitalMode_BC | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) data-block-0, a data block of 64 data words.
		S16BIT swDataBlockId_0; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId_0 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i\n", s_swDevice2, swDataBlockId_0, 64, 0);
		swResult = sitalBc_DataBlock_Create (s_swDevice2, swDataBlockId_0, sitalBcDataBlockSizeOption_DOUBLE, NULL, 0);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) message-0, a BC-to-RT(RT-address-1/RtSubAddress1) message that uses data-block-0 to transmit WordCount data words. Message bus: bus-1.
		S16BIT swMessageId_0; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create: unit - %i, message - %i, block - %i, rx_rt - %i, rx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddress, wWordCount, 0, s_wBus1);
		swResult = sitalBc_Message_CreateBcToRt (s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddress, wWordCount, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-0, a XEQ command that executes message-0.
		S16BIT swCommandId_0; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-1, a subroutine call command that executes minor-frame-0.
		S16BIT swCommandId_1; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_1 = 1;
		S16BIT swFrameId_0; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_1, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, swFrameId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_1, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, (U16BIT)swFrameId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) minor-frame-0, a frame that contains command-0.
		S16BIT swaCommandIds[sitalBcCounter_COMMANDS]; // An array of command-IDs.
		swaCommandIds[0] = swCommandId_0;
		U16BIT wCommandIdCount; // The number of command IDs in the frame (1-sitalBcCounter_FRAME_COMMANDS).
		wCommandIdCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_0, "minor", wCommandIdCount, swCommandId_0, 0, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_0, sitalBcFrameType_MINOR, swaCommandIds, wCommandIdCount, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) major-frame-1, a frame that contains command-2.
		S16BIT swFrameId_1; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_1 = 1;
		swaCommandIds[0] = swCommandId_1;
		wCommandIdCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_1, "major", wCommandIdCount, swCommandId_1, wFrameTime, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_1, sitalBcFrameType_MAJOR, swaCommandIds, wCommandIdCount, wFrameTime, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		///	Start device-2 (BC) running major-frame-2 for 64-shots.
		U16BIT wFrameCount; // The number of frames to run.
		wFrameCount = 64;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", s_swDevice2, swFrameId_1, wFrameCount);
		swResult = sitalBc_Start (s_swDevice2, swFrameId_1, wFrameCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Wait for the frame run to end.
		U32BIT dwRemainingFrameRunTime; // The expected remaining frame run time (in ms).
		dwRemainingFrameRunTime = (wFrameCount * (wFrameTime / 10));
		swResult = stldTester_Bc_WaitForEndOfFrameRun (s_swDevice2, dwRemainingFrameRunTime, 0, NULL);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.WaitForEndOfFrameRun.Error: error - %i\n", swResult);
			throw 0;
		}

		///	Stop device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop: unit - %i\n", s_swDevice2);
		swResult = sitalBc_Stop (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		///	Verify that device-3 (MT) has detected the expected number of messages, and that the time tags of these messages compose an appropriate sequence of time tags.
		sitalDecodedMessageStructure dmsDecodedMessage; // A decoded message.
		U16BIT wExpectedTimeTag; // The next expected time tag.
		wExpectedTimeTag = 0;
		for (S16BIT i=0; i<wFrameCount; i++)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded: unit - %i, purge - %i\n", s_swDevice3, TRUE);
			swResult = sitalMt_Message_GetFromStackDecoded (s_swDevice3, &dmsDecodedMessage, sitalMessageLocationAndRemoval_NEXT_PURGE, sitalMtStack_ACTIVE);
			if ((0 > swResult) || (1 < swResult))
			{
				sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Error: error - %i\n", swResult);
				throw 0;
			}
			if (0U == swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Message.Error: error - %s\n", "no_message");
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}
			sitalProcess_Log_PrintLine ("#StldTester.Mt.Message: unit - %i, command - %04X, data_count - %i, data - %i...%i, time_tag - %04X\n", s_swDevice3, dmsDecodedMessage.wCommandWord1, dmsDecodedMessage.wDataWordCount, dmsDecodedMessage.waData[0], dmsDecodedMessage.waData[dmsDecodedMessage.wDataWordCount - 1], dmsDecodedMessage.wTimeTag);

			wExpectedTimeTag = ( (0U == i) ? dmsDecodedMessage.wTimeTag : wExpectedTimeTag );
			U16BIT wTimeTagDifference; // The difference between the expected time tag and the actual one.
			wTimeTagDifference = (wExpectedTimeTag - dmsDecodedMessage.wTimeTag);
			wTimeTagDifference = ( (0x8000 < wTimeTagDifference) ? (0x10000 - wTimeTagDifference) : wTimeTagDifference); // To handle a case of time tag rollover.
			sitalProcess_Log_PrintLine ("#StldTester.Mt.Message: unit - %i, time_tag - %04X, expected_time_tag - ~%04X\n", s_swDevice3, dmsDecodedMessage.wTimeTag, wExpectedTimeTag);
			if (wTimeTagCalculationInaccuracy < wTimeTagDifference)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Mt.Message.Error: error - %s\n", "bad_timing");
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}
			wExpectedTimeTag = ( dmsDecodedMessage.wTimeTag + (wFrameTime * 100 / wTimeTagResolution) ); // Frame time is expressed in units of 100us.
		}

		///	Verify that device-2 (MT) hasn't recorded any additional messages.
		sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded: unit - %i, purge - %i\n", s_swDevice3, TRUE);
		swResult = sitalMt_Message_GetFromStackDecoded (s_swDevice3, &dmsDecodedMessage, sitalMessageLocationAndRemoval_NEXT_PURGE, sitalMtStack_ACTIVE);
		if ((0 > swResult) || (1 < swResult))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Error: error - %i\n", swResult);
			throw 0;
		}
		if (1 == swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Error: error - %s\n", "extra_message");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// Stop device-3 (MT).
		sitalProcess_Log_PrintLine ("#StldTester.Mt.Stop: unit - %i\n", s_swDevice3);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Stop (s_swDevice3) : sitalMt_Stop (s_swDevice3) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice2);
		swResult = sitalDevice_Free (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-3 (MT).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice3);
		swResult = sitalDevice_Free (s_swDevice3);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		swResult = stldSyncEnd (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncEnd.Error: error - %i\n", swResult);
			throw 0;
		}
	}
	catch (int)
	{
		sitalDevice_Free (s_swDevice2);
		sitalDevice_Free (s_swDevice3);
	}

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_15g.End: test_succeeded - %c\n", ((sitalReturnCode_SUCCESS == swResult) ? 'Y' : 'N'));
	return swResult;
}



/// <summary>
/// Test the stld1553 library in the henceforth described scenario:
/// Participant[s]: BC2, MT3.
/// Configuration: All participating devices must be able to communicate with each other.
/// Short description: Acceptance Test Plan (ATP) test 461_16a - Causes of interrupt in BM - Completion of performance of any operation, e.g. completion of chain of cycle for information exchange.
/// Along test print suitable info and indications into the log.
///
/// Note:
/// - This function may use any of above defined Acceptance Test Plan (ATP) tests variables.
///   The caller must therefore call the suitable stldTester_Atp_Set* functions prior to calling this function.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Atp_DoTest461_16a ()
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_16a.Start\n");

	/// Set test arguments.
	U16BIT wRtSubaddress; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31).
	wRtSubaddress = 1;
	U16BIT wWordCount; // Word count (1-32).
	wWordCount = 32;
	U16BIT wFrameTime; // The time (in units of 100 us) that the frame will take to complete.
	wFrameTime = 1000; // 100ms.
	U16BIT wFrameCount; // The number of frames to run.
	wFrameCount = 64;

	S16BIT swResult; // Result of operation or function call.
	swResult = sitalReturnCode_SUCCESS;

	/// Get device count.
	U16BIT wRequiredDeviceCount; // The number of devices required for this test.
	wRequiredDeviceCount = (1 + max(s_swDevice2, s_swDevice3));
	U16BIT wAvailableDeviceCount; // The total number of devices available in the system.
	swResult = sitalDevice_GetCount (&wAvailableDeviceCount);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %i\n", swResult);
		return swResult;
	}
	sitalProcess_Log_PrintLine ("#StldTester.Device.Count: available - %i, required - %i\n", wAvailableDeviceCount, wRequiredDeviceCount);
	if (wAvailableDeviceCount < wRequiredDeviceCount)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %s\n", "not_enough_devices_installed");
		return sitalReturnCode_SUCCESS;
	}
	if ( (s_swDevice2 == s_swDevice3) )
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Number.Error: error - %s\n", "same_device_used_twice");
		return sitalReturnCode_INVALID_DEVICE_NUMBER;
	}

	try
	{
		swResult = stldSyncStart (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncStart.Error: error - %i\n", swResult);
			throw 0;
		}

		// When using the DDC (or a DDC-like) library, un-include code that is unique to Sital and isn't supported by DDC.
#ifndef USE_DDC_LIBRARY

		/// Get full version information for the stld1553 library.
		sitalProcess_Log_PrintLine ("#StldTester.LibraryVersion.Full.Get\n");
		U16BIT wMajorVersionDll, wMinorVersionDll, wBuildNumberDll, wRevisionNumberDll;
		swResult = sitalStld1553_GetLibraryVersion (&wMajorVersionDll, &wMinorVersionDll, &wBuildNumberDll, &wRevisionNumberDll);
		S8BIT szErrorString[256];
		sitalStld1553_GetReturnCodeDescriberString (swResult, szErrorString, 256);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.LibraryVersion.Full.Get.Error: error - %s\n", szErrorString);
			throw 0;
		}
		sitalProcess_Log_PrintLine ("#StldTester.LibraryVersion.Full.Get: version - %i.%i.%i.%i, result_string - %s\n", wMajorVersionDll, wMinorVersionDll, wBuildNumberDll, wRevisionNumberDll, szErrorString);

		/// Get short version information for the stld1553 library.
		sitalProcess_Log_PrintLine ("#StldTester.LibraryVersion.Short.Get\n");
		U16BIT wShortLibraryVersion; // Short library version number.
		wShortLibraryVersion = sitalStld1553_GetShortLibraryVersion ();
		if (0U == wShortLibraryVersion)
		{
			sitalProcess_Log_PrintLine ("#StldTester.LibraryVersion.Short.Get.Error\n");
			throw 0;
		}
		sitalProcess_Log_PrintLine ("#StldTester.LibraryVersion.Short.Get: version - %i.%i.%i\n", ((U16BIT)(wShortLibraryVersion >> 8)), (((U16BIT)(wShortLibraryVersion << 8)) >> 12), (((U16BIT)(wShortLibraryVersion << 12)) >> 12));

		/// Verify that the version information is consistent.
		if ( (((U16BIT)(wShortLibraryVersion >> 8)) != wMajorVersionDll) || ((((U16BIT)(wShortLibraryVersion << 8)) >> 12) != wMinorVersionDll) || ((((U16BIT)(wShortLibraryVersion << 12)) >> 12) != wBuildNumberDll) )
		{
			sitalProcess_Log_PrintLine ("#StldTester.LibraryVersion.Error: error - %s\n", "version_information_mismatch");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// Get short version information for the driver-interface library.
		sitalProcess_Log_PrintLine ("#StldTester.DriverInterface.LibraryVersion.Short.Get\n");
		swResult = sitalStld1553_GetShortCoreVersion ();
		if (0U == swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.DriverInterface.LibraryVersion.Short.Get.Error\n");
			throw 0;
		}
		sitalProcess_Log_PrintLine ("#StldTester.DriverInterface.LibraryVersion.Short.Get: version - %i.%i.%i\n", (swResult >> 8), ((swResult << 8) >> 12), ((swResult << 12) >> 12));

#endif // USE_DDC_LIBRARY

		// When using the DDC (or a DDC-like) library, un-include code that is unique to Sital and isn't supported by DDC.
#ifndef USE_DDC_LIBRARY

		/// Configure desired printings.
		sitalProcess_Log_PrintLine ("#StldTester.ConfigurePrintings: console - %s, level - %i\n", "no", sitalPrintingsLevel_Information);
		sitalStld1553_ConfigurePrintings (false, sitalPrintingsLevel_Information);

#endif // USE_DDC_LIBRARY

		/// Initialize device-3 (MT) as a real MT.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access - card, mode - mt, alloc_mem - none\n", s_swDevice3);
		swResult = sitalDevice_Initialize (s_swDevice3, sitalAccess_CARD, ((s_bIsRtMtModeTest ? sitalMode_RT_AND_MT : sitalMode_MT) | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Configure device-3 (MT) to enable its ISQ.
		sitalProcess_Log_PrintLine ("#StldTester.Mt.GetInformation: unit - %i\n", s_swDevice2);
		sitalMtInformationStructure misMtInformation;
		swResult = sitalMt_GetInformation (s_swDevice3, &misMtInformation);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetInformation.Error: error - %i\n", swResult);
			throw 0;
		}
		sitalProcess_Log_PrintLine ("#StldTester.Mt.GetInformation: unit - %i, stack_mode - %i, command_stack_size - %i, data_stack_size - %i, 1553A_supported - %i, host_buffer_size - %i\n", s_swDevice2, misMtInformation.wStackMode, misMtInformation.wCommandStackSize, misMtInformation.wDataStackSize, misMtInformation.bIsProtocol1553aSupported, misMtInformation.dwHostBufferSize);

		///// Configure device-3 (MT) to enable its ISQ.
		//sitalProcess_Log_PrintLine ("#StldTester.Device.Isq.Configure: unit - %i, enable - %i\n", s_swDevice2, TRUE);
		//swResult = sitalDevice_Isq_Configure (s_swDevice3, TRUE);
		//if (sitalReturnCode_SUCCESS != swResult)
		//{
		//	sitalProcess_Log_PrintLine ("#StldTester.Device.Isq.Configure.Error: error - %i\n", swResult);
		//	throw 0;
		//}

		/// Get for device-3 (MT) the current interrupt configuration.
		U16BIT wInterruptMode; // Current configuration for type of interrupt signal (sitalIrqMode_*).
		U16BIT wAutoClear; // Current configuration for post-read auto-clearing of interrupt status (sitalIrqClear_*).
		swResult = sitalDevice_Irq_GetMode (s_swDevice3, &wInterruptMode, &wAutoClear);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Mode.Get.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Configure device-3 (MT) to issue suitable interrupts and perform post-read auto-clearing of interrupt status.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Configure: unit - %i, mode - %i, auto_clear - %i\n", s_swDevice2, wInterruptMode, sitalIrqClear_NO_AUTO_CLEAR);
		swResult = sitalDevice_Irq_Configure (s_swDevice3, wInterruptMode, sitalIrqClear_NO_AUTO_CLEAR);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Configure.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Configure device-3 (MT) to get desired notifications using an ISR that collects interrupt data, counts ISR calls, records the interrupt mask, and signals interested threads.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Manipulate: unit - %i, enable - %i, irq - %08X, callback - %s\n", s_swDevice3, TRUE, sitalInterruptRegister1_END_OF_MESSAGE, "stldTester_Bc_UserIsr");
		swResult = sitalDevice_Irq_Manipulate (s_swDevice3, TRUE, sitalInterruptRegister1_END_OF_MESSAGE, stldTester_Mt_UserIsr);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Set.Error: error - %i\n", swResult);
			throw 0;
		}
		s_dwMtIsrCallCounter = 0;
		s_dwMtIrqStatus = 0;

		///// Clear the ISQ of device-3 (MT).
		//sitalProcess_Log_PrintLine ("#StldTester.Device.Isq.Clear: unit - %i\n", s_swDevice3);
		//swResult = sitalDevice_Isq_Clear (s_swDevice3);
		//if (sitalReturnCode_SUCCESS != swResult)
		//{
		//	sitalProcess_Log_PrintLine ("#StldTester.Device.Isq.Clear.Error: error - %i\n", swResult);
		//	throw 0;
		//}

		/// Configure device-3 (MT) to collect operational statistics.
		sitalProcess_Log_PrintLine ("#StldTester.Device.OperationalStatisticsCollection.Configure: unit - %i, collect - %i\n", s_swDevice3, TRUE);
		swResult = sitalDevice_OperationalStatisticsCollection_Configure (s_swDevice3, TRUE);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.OperationalStatisticsCollection.Configure.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Write something to the memory of device-3 (MT).
		U16BIT wMemoryAddress; // Memory address.
		wMemoryAddress = 0x0400;
		U16BIT wWrittenValue; // Written value.
		wWrittenValue = 0xABCD;
		sitalProcess_Log_PrintLine ("#StldTester.Device.MemoryWord.Write: unit - %i, address - %04X, value - %04X\n", s_swDevice3, wMemoryAddress, wWrittenValue);
		swResult = sitalDevice_Memory_Write (s_swDevice3, wMemoryAddress, wWrittenValue);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.MemoryWord.Write.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Read from the memory of device-3 (MT) what was just written.
		U16BIT wReadValue; // Read value.
		wReadValue = sitalDevice_Memory_Read (s_swDevice3, wMemoryAddress);
		sitalProcess_Log_PrintLine ("#StldTester.Device.MemoryWord.Read: unit - %i, address - %04X, value - %04X\n", s_swDevice3, wMemoryAddress, wReadValue);

		/// Verify that really read from the memory of device-3 (MT) what was just written.
		if (wWrittenValue != wReadValue)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.MemoryWord.Read.Error: error - %i\n", wReadValue);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// Start device-3 (MT).
		sitalProcess_Log_PrintLine ("#StldTester.Mt.Start: unit - %i\n", s_swDevice3);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Start (s_swDevice3) : sitalMt_Start (s_swDevice3) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-2 (BC) as a real BC.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-bc, alloc_mem-none\n", s_swDevice2);
		swResult = sitalDevice_Initialize (s_swDevice2, sitalAccess_CARD, (sitalMode_BC | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) data-block-0, a data block of 64 data words.
		S16BIT swDataBlockId_0; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId_0 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i\n", s_swDevice2, swDataBlockId_0, 64, 0);
		swResult = sitalBc_DataBlock_Create (s_swDevice2, swDataBlockId_0, sitalBcDataBlockSizeOption_DOUBLE, NULL, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Verify that cannot create for device-2 (BC) another data-block-0.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i\n", s_swDevice2, swDataBlockId_0, 64, 0);
		swResult = sitalBc_DataBlock_Create (s_swDevice2, swDataBlockId_0, sitalBcDataBlockSizeOption_DOUBLE, NULL, 0);
		if (sitalReturnCode_BC_OBJECT_ALREADY_EXISTS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// Delete device-2 (BC) data-block-0.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Delete: unit - %i, block - %i\n", s_swDevice2, swDataBlockId_0);
		swResult = sitalBc_DataBlock_Delete (s_swDevice2, swDataBlockId_0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Delete.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) data-block-0, a data block of 64 data words.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i\n", s_swDevice2, swDataBlockId_0, 64, 0);
		swResult = sitalBc_DataBlock_Create (s_swDevice2, swDataBlockId_0, sitalBcDataBlockSizeOption_DOUBLE, NULL, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) message-0, a BC-to-RT(RT-address-1/RtSubAddress1) message that uses data-block-0 to transmit WordCount data words. Message bus: bus-1.
		S16BIT swMessageId_0; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create: unit - %i, message - %i, message_type - %s, block - %i, rx_rt - %i, rx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_0, sitalStld1553_GetMessageTypeString (sitalMessageType_BC_TO_RT), swDataBlockId_0, s_wRtAddress1, wRtSubaddress, wWordCount, 0, s_wBus1);
		swResult = sitalBc_Message_CreateBcToRt (s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddress, wWordCount, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Verify that cannot create for device-2 (BC) another frame-0.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create: unit - %i, message - %i, message_type - %s, block - %i, rx_rt - %i, rx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_0, sitalStld1553_GetMessageTypeString (sitalMessageType_BC_TO_RT), swDataBlockId_0, s_wRtAddress1, wRtSubaddress, wWordCount, 0, s_wBus1);
		swResult = sitalBc_Message_CreateBcToRt (s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddress, wWordCount, 0, s_wBus1);
		if (sitalReturnCode_BC_OBJECT_ALREADY_EXISTS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.Create.Error: error - %i\n", swResult);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// Delete device-2 (BC) message-0.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.Delete: unit - %i, frame - %i\n", s_swDevice2, swMessageId_0);
		swResult = sitalBc_Message_Delete (s_swDevice2, swMessageId_0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.Delete.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) message-0, a BC-to-RT(RT-address-1/RtSubAddress1) message that uses data-block-0 to transmit WordCount data words. Message bus: bus-1.
		swMessageId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create: unit - %i, message - %i, message_type - %s, block - %i, rx_rt - %i, rx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_0, sitalStld1553_GetMessageTypeString (sitalMessageType_BC_TO_RT), swDataBlockId_0, s_wRtAddress1, wRtSubaddress, wWordCount, 0, s_wBus1);
		swResult = sitalBc_Message_CreateBcToRt (s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddress, wWordCount, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-0, a XEQ command that executes message-0.
		S16BIT swCommandId_0; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Verify that cannot create for device-2 (BC) another command-0.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0, 0);
		if (sitalReturnCode_BC_OBJECT_ALREADY_EXISTS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// Delete device-2 (BC) command-0.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Delete: unit - %i, command - %i\n", s_swDevice2, swCommandId_0);
		swResult = sitalBc_Command_Delete (s_swDevice2, swCommandId_0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Delete.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-0, a XEQ command that executes message-0.
		swCommandId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-1, a PUSH command to push a user-specific GPQ-header-flag (i.e., one different from any defined sitalGeneralPurposeQueueFlag_*).
		/// Create for device-2 (BC) command-2, a PUSH command to push a desired data word.
		// Note: These commands will together make the device fill a GPQ entry with a user-specific GPQ-header-flag and desired data.
		U16BIT wUserSpecificGpqHeaderFlag; // A user-specific GPQ-header-flag (i.e., one different from any defined sitalGeneralPurposeQueueFlag_*).
		wUserSpecificGpqHeaderFlag = 0x1234;
		U16BIT wDataWord; // A data word.
		wDataWord = 0x5678;
		S16BIT swCommandId_1; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_1 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_1, sitalOpcode_PUSH_VALUE, sitalOpcodeCondition_ALWAYS, wUserSpecificGpqHeaderFlag, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_1, sitalOpcode_PUSH_VALUE, sitalOpcodeCondition_ALWAYS, wUserSpecificGpqHeaderFlag, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}
		S16BIT swCommandId_2; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_2 = 2;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_2, sitalOpcode_PUSH_VALUE, sitalOpcodeCondition_ALWAYS, wDataWord, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_2, sitalOpcode_PUSH_VALUE, sitalOpcodeCondition_ALWAYS, wDataWord, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-3, a subroutine call command that executes minor-frame-0.
		S16BIT swCommandId_3; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_3 = 3;
		S16BIT swFrameId_0; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice3, swCommandId_2, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, swFrameId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_3, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, (U16BIT)swFrameId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) minor-frame-0, a frame that contains command-0.
		S16BIT swaCommandIds[sitalBcCounter_COMMANDS]; // An array of command-IDs.
		swaCommandIds[0] = swCommandId_0;
		U16BIT wCommandIdCount; // The number of command IDs in the frame (1-sitalBcCounter_FRAME_COMMANDS).
		wCommandIdCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_0, "minor", wCommandIdCount, swCommandId_0, 0, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_0, sitalBcFrameType_MINOR, swaCommandIds, wCommandIdCount, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Verify that cannot create for device-2 (BC) another frame-0.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_0, "minor", wCommandIdCount, swCommandId_0, 0, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_0, sitalBcFrameType_MINOR, swaCommandIds, wCommandIdCount, 0, 0);
		if (sitalReturnCode_BC_OBJECT_ALREADY_EXISTS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// Delete device-2 (BC) frame-0.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Delete: unit - %i, frame - %i\n", s_swDevice2, swFrameId_0);
		swResult = sitalBc_Frame_Delete (s_swDevice2, swFrameId_0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Delete.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) minor-frame-0, a frame that contains command-0.
		swaCommandIds[0] = swCommandId_0;
		wCommandIdCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_0, "minor", wCommandIdCount, swCommandId_0, 0, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_0, sitalBcFrameType_MINOR, swaCommandIds, wCommandIdCount, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) major-frame-1, a frame that contains command-2.
		S16BIT swFrameId_1; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_1 = 1;
		swaCommandIds[0] = swCommandId_1;
		swaCommandIds[1] = swCommandId_2;
		swaCommandIds[2] = swCommandId_3;
		wCommandIdCount = 3;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i %i %i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_1, "major", wCommandIdCount, swCommandId_1, swCommandId_2, swCommandId_3, wFrameTime, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_1, sitalBcFrameType_MAJOR, swaCommandIds, wCommandIdCount, wFrameTime, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		///	Start device-2 (BC) running major-frame-2 for FrameCount-shots.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", s_swDevice2, swFrameId_1, wFrameCount);
		swResult = sitalBc_Start (s_swDevice2, swFrameId_1, wFrameCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		///	Wait for the frame run to end.
		U32BIT dwRemainingFrameRunTime; // The expected remaining frame run time (in ms).
		dwRemainingFrameRunTime = (wFrameCount * (wFrameTime / 10));
		swResult = stldTester_Bc_WaitForEndOfFrameRun (s_swDevice2, dwRemainingFrameRunTime, 0, NULL);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.WaitForEndOfFrameRun.Error: error - %i\n", swResult);
			throw 0;
		}

		///	Stop device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop: unit - %i\n", s_swDevice2);
		swResult = sitalBc_Stop (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Get for device-2 (BC) the overall number of available GPQ entries.
		// Note: As no automatic reading of GPQ entries in favor of filling a host buffer takes place in this test, filled entries aren't recorded, and the expected count of entries is zero.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Gpq.Entries.Count: unit - %i\n", s_swDevice2);
		swResult = sitalBc_Gpq_GetEntriesCount (s_swDevice2);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Gpq.Entries.Count.Error: error - %i\n", swResult);
			throw 0;
		}
		U16BIT wGpqEntriesCount; // Count of available GPQ entries.
		wGpqEntriesCount = swResult;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Gpq.Entries.Count: unit - %i, count - %i\n", s_swDevice2, wGpqEntriesCount);
		if (0U != wGpqEntriesCount)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Gpq.Entries.Count.Error: error - %s\n", "bad_count");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// Read from device-2 (BC) all available GPQ entries.
		/// Verify that entries in a correct number and correct contents have been read for device-2 (BC).
		// Note: As major-frame-1 only pushes a single user-specific GPQ entry, only one GPQ entry should be available.
		sitalGpqEntryStructure gesGpqEntry; // GPQ entry.
		U16BIT wUserSpecificGpqEntriesCount; // The count of user-specific GPQ entries that have been read.
		wUserSpecificGpqEntriesCount = 0;
		do 
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Gpq.Entry.Read: unit - %i\n", s_swDevice2);
			swResult = sitalBc_Gpq_Read (s_swDevice2, &gesGpqEntry);
			if ( (sitalReturnCode_ReadGpqEntry_READ_NEW_ENTRY == swResult) || (sitalReturnCode_ReadGpqEntry_READ_NEW_ENTRY_AND_DETECTED_OVERRUN == swResult) )
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.Gpq.Entry.Read: unit - %i, gpq_entry_index - %i, gpq_header - %04X, gpq_data - %04X\n", s_swDevice2, wUserSpecificGpqEntriesCount, gesGpqEntry.wHeader, gesGpqEntry.wData);
				wUserSpecificGpqEntriesCount++;
				if (wUserSpecificGpqHeaderFlag != gesGpqEntry.wHeader)
				{
					sitalProcess_Log_PrintLine ("#StldTester.Bc.Gpq.Entry.Read.Header.Error: error - %s\n", "bad_header");
					swResult = sitalReturnCode_TEST_FAIL;
					throw 0;
				}
				if (wDataWord != gesGpqEntry.wData)
				{
					sitalProcess_Log_PrintLine ("#StldTester.Bc.Gpq.Entry.Read.Data.Error: error - %s\n", "bad_data");
					swResult = sitalReturnCode_TEST_FAIL;
					throw 0;
				}
			}
		}
		while ( (sitalReturnCode_ReadGpqEntry_READ_NEW_ENTRY == swResult) || (sitalReturnCode_ReadGpqEntry_READ_NEW_ENTRY_AND_DETECTED_OVERRUN == swResult) );
		if (sitalReturnCode_SUCCESS > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Gpq.Entry.Read.Error: error - %i\n", swResult);
			throw 0;
		}
		if (1 != wUserSpecificGpqEntriesCount)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Gpq.Entry.Read.Count.Error: error - %i\n", wUserSpecificGpqEntriesCount);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// As long a not all expected interrupts occurred AND the maximum possible interrupt latency still didn't pass:
		///		Wait.
		stldInterruptLatency_Wait (&s_dwMtIsrCallCounter, wFrameCount);

		/// Verify that a correct number of interrupts of a suitable type occurred.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Interrupts: unit - %i, count - %i, expected_count - %i, last_mask - %08X, expected_mask - %08X\n", s_swDevice3, s_dwMtIsrCallCounter, wFrameCount, s_dwMtIrqStatus, sitalInterruptRegister1_END_OF_MESSAGE);
		if (wFrameCount != s_dwMtIsrCallCounter)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Interrupt.CallCount.Error: error - %i\n", s_dwMtIsrCallCounter);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		if (0U == (sitalInterruptRegister1_END_OF_MESSAGE & s_dwMtIrqStatus))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Interrupt.Mask.Error: error - %08X\n", s_dwMtIrqStatus);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		///// Read from device-3 (MT) all available ISQ entries.
		//// Note: As ISQ support wasn't yet implemented in RT/MT devices, the following code won't really discover and get any actual ISQ entries.
		//sitalIsqEntryStructure iesIsqEntry; // ISQ entry.
		//do 
		//{
		//	sitalProcess_Log_PrintLine ("#StldTester.Device.Isq.Entry.Read: unit - %i\n", s_swDevice3);
		//	swResult = sitalDevice_Isq_ReadEntry (s_swDevice3, &iesIsqEntry);
		//	if ( (sitalReturnCode_ReadIsqEntry_READ_NEW_ENTRY == swResult) || (sitalReturnCode_ReadIsqEntry_READ_NEW_ENTRY_AND_DETECTED_OVERRUN == swResult) )
		//	{
		//		sitalProcess_Log_PrintLine ("#StldTester.Device.Isq.Entry.Read: unit - %i\n", s_swDevice3, iesIsqEntry.wHeader, iesIsqEntry.wData);
		//	}
		//}
		//while ( (sitalReturnCode_ReadIsqEntry_READ_NEW_ENTRY == swResult) || (sitalReturnCode_ReadIsqEntry_READ_NEW_ENTRY_AND_DETECTED_OVERRUN == swResult) );
		//if (sitalReturnCode_SUCCESS > swResult)
		//{
		//	sitalProcess_Log_PrintLine ("#StldTester.Device.Isq.Entry.Read.Error: error - %i\n", swResult);
		//	throw 0;
		//}

		///	Get from device-3 (MT) all the recorded messages.
		// See below the note on MT operational statistics.
		sitalDecodedMessageStructure dmsDecodedMessage; // A decoded message.
		do
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded: unit - %i, purge - %i\n", s_swDevice3, TRUE);
			swResult = sitalMt_Message_GetFromStackDecoded (s_swDevice3, &dmsDecodedMessage, sitalMessageLocationAndRemoval_NEXT_PURGE, sitalMtStack_ACTIVE);
			if ((0 > swResult) || (1 < swResult))
			{
				sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Error: error - %i\n", swResult);
				throw 0;
			}
			if (1 == swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Mt.Message: unit - %i, command - %04X, data_count - %i, data - %i...%i, time_tag - %04X\n", s_swDevice3, dmsDecodedMessage.wCommandWord1, dmsDecodedMessage.wDataWordCount, dmsDecodedMessage.waData[0], dmsDecodedMessage.waData[dmsDecodedMessage.wDataWordCount - 1], dmsDecodedMessage.wTimeTag);
			}
		}
		while (1 == swResult);

		/// Get from device-3 (MT) its current operational statistics.
		// Note:
		// - MT operational statistics actually include stack statistics (for stack A/B/combined), which is only updated when messages are read from the relevant
		//   stack in result of user request or in order to update the host buffer. In this test no host buffer is installed, so some message must be read in order
		//   for the statistics to be recorded. That's the reason why MT messages were read above, though seemingly not needed for the purpose of this test.
		// - Now, as all available messages have been read above from the MT stack, the currently expected fullness percentage must be zero, and the maximum fullness
		//   percentage must reflect the fact that 64 messages have been sent, where each message occupies a number of sitalRtMemoryObjectSize_COMMAND_STACK_ENTRY (=4)
		//   words in the single MT stack that's initialized by default (stack A) at a size that depends by default on the total memory size of the MT device.
		sitalStackOperationalStatisticsStructure sossStackOperationalStatistics; // Stack operational statistics.
		sitalProcess_Log_PrintLine ("#StldTester.Mt.Stack.OperationalStatistics.Get: unit - %i, stack - %i, reset - %i\n", s_swDevice3, sitalMtStackSelector_STACK_A, FALSE);
		swResult = sitalMt_Stack_GetOperationalStatistics (s_swDevice3, &sossStackOperationalStatistics, sitalMtStackSelector_STACK_A, FALSE);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.Stack.OperationalStatistics.Get.Error: error - %i\n", swResult);
			throw 0;
		}
		sitalProcess_Log_PrintLine ("#StldTester.Mt.Stack.OperationalStatistics: unit - %i, stack - %i, lost - %i, fullness(percent) - %i, maximum_fulness(percent) - %i\n", s_swDevice3, sitalMtStackSelector_STACK_A, sossStackOperationalStatistics.dwLostMessageCount, sossStackOperationalStatistics.dwFullnessPercentage, sossStackOperationalStatistics.dwMaximumFullnessPercentage);
		if (0U != sossStackOperationalStatistics.dwFullnessPercentage)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.Stack.OperationalStatistics.Get.Error: error - %s\n", "bad_fullness");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		if (0 >= sossStackOperationalStatistics.dwMaximumFullnessPercentage)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.Stack.OperationalStatistics.Get.Error: error - %s\n", "bad_maximum_fullness");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// Free device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice2);
		swResult = sitalDevice_Free (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Stop device-3 (MT).
		sitalProcess_Log_PrintLine ("#StldTester.Mt.Stop: unit - %i\n", s_swDevice3);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Stop (s_swDevice3) : sitalMt_Stop (s_swDevice3) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-3 (MT).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice3);
		swResult = sitalDevice_Free (s_swDevice3);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		swResult = stldSyncEnd (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncEnd.Error: error - %i\n", swResult);
			throw 0;
		}
	}
	catch (int)
	{
		sitalDevice_Free (s_swDevice2);
		sitalDevice_Free (s_swDevice3);
	}

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_16a.End: test_succeeded - %c\n", ((sitalReturnCode_SUCCESS == swResult) ? 'Y' : 'N'));
	return swResult;
}



/// <summary>
/// Test the stld1553 library in the henceforth described scenario:
/// Participant[s]: BC2, MT3.
/// Configuration: All participating devices must be able to communicate with each other.
/// Short description: Acceptance Test Plan (ATP) test 461_EmptyFrame - Support for 0 message frames.
/// Along test print suitable info and indications into the log.
///
/// Note:
/// - This function may use any of above defined Acceptance Test Plan (ATP) tests variables.
///   The caller must therefore call the suitable stldTester_Atp_Set* functions prior to calling this function.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Atp_DoTest461_EmptyFrame ()
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_EmptyFrame.Start\n");

	/// Set test arguments.
	U16BIT wFrameCount; // The number of frames to run.
	wFrameCount = 64;
	U16BIT wFrameTime; // The time (in units of 100 us) that the frame will take to complete.
	wFrameTime = 100; // 10ms.

	S16BIT swResult; // Result of operation or function call.
	swResult = sitalReturnCode_SUCCESS;

	/// Get device count.
	U16BIT wRequiredDeviceCount; // The number of devices required for this test.
	wRequiredDeviceCount = (1 + max(s_swDevice2, s_swDevice3));
	U16BIT wAvailableDeviceCount; // The total number of devices available in the system.
	swResult = sitalDevice_GetCount (&wAvailableDeviceCount);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %i\n", swResult);
		return swResult;
	}
	sitalProcess_Log_PrintLine ("#StldTester.Device.Count: available - %i, required - %i\n", wAvailableDeviceCount, wRequiredDeviceCount);
	if (wAvailableDeviceCount < wRequiredDeviceCount)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %s\n", "not_enough_devices_installed");
		return sitalReturnCode_SUCCESS;
	}
	if ( (s_swDevice2 == s_swDevice3) )
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Number.Error: error - %s\n", "same_device_used_twice");
		return sitalReturnCode_INVALID_DEVICE_NUMBER;
	}

	try
	{
		swResult = stldSyncStart (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncStart.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-3 (MT) as a real MT.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access - card, mode - mt, alloc_mem - none\n", s_swDevice3);
		swResult = sitalDevice_Initialize (s_swDevice3, sitalAccess_CARD, ((s_bIsRtMtModeTest ? sitalMode_RT_AND_MT : sitalMode_MT) | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Start device-3 (MT).
		sitalProcess_Log_PrintLine ("#StldTester.Mt.Start: unit - %i\n", s_swDevice3);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Start (s_swDevice3) : sitalMt_Start (s_swDevice3) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-2 (BC) as a real BC.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-bc, alloc_mem-none\n", s_swDevice2);
		swResult = sitalDevice_Initialize (s_swDevice2, sitalAccess_CARD, (sitalMode_BC | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-2, a FOCUS-ON-VAR command that focuses on variable-0.
		// Note: In this test a frame that contains no messages is created; as a frame must contain at least one command, a casual and functionless command that never executes is inserted.
		S16BIT swCommandId_0; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_0 = 0;
		U32BIT dwVariable_0; // A number of device variable.
		dwVariable_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_0, sitalOpcode_FOCUS_ON_VARIABLE, sitalOpcodeCondition_NEVER, dwVariable_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_0, sitalOpcode_FOCUS_ON_VARIABLE, sitalOpcodeCondition_NEVER, dwVariable_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-1, a subroutine call command that executes minor-frame-0.
		S16BIT swCommandId_1; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_1 = 1;
		S16BIT swFrameId_0; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_1, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, swFrameId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_1, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, (U16BIT)swFrameId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) minor-frame-0, a frame that contains command-0.
		S16BIT swaCommandIds[sitalBcCounter_COMMANDS]; // An array of command-IDs.
		swaCommandIds[0] = swCommandId_0;
		U16BIT wCommandIdCount; // The number of command IDs in the frame (1-sitalBcCounter_FRAME_COMMANDS).
		wCommandIdCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_0, "minor", wCommandIdCount, swCommandId_0, 0, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_0, sitalBcFrameType_MINOR, swaCommandIds, wCommandIdCount, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) major-frame-1, a frame that contains command-2.
		S16BIT swFrameId_1; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_1 = 1;
		swaCommandIds[0] = swCommandId_1;
		wCommandIdCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_1, "major", wCommandIdCount, swCommandId_1, wFrameTime, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_1, sitalBcFrameType_MAJOR, swaCommandIds, wCommandIdCount, wFrameTime, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		///	Start device-2 (BC) running major-frame-2 for FrameCount-shots.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", s_swDevice2, swFrameId_1, wFrameCount);
		swResult = sitalBc_Start (s_swDevice2, swFrameId_1, wFrameCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Wait for the frame run to end.
		U32BIT dwRemainingFrameRunTime; // The expected remaining frame run time (in ms).
		dwRemainingFrameRunTime = (wFrameCount * (wFrameTime / 10));
		swResult = stldTester_Bc_WaitForEndOfFrameRun (s_swDevice2, dwRemainingFrameRunTime, 0, NULL);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.WaitForEndOfFrameRun.Error: error - %i\n", swResult);
			throw 0;
		}

		///	Stop device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop: unit - %i\n", s_swDevice2);
		swResult = sitalBc_Stop (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		///	Verify that device-3 (MT) has detected no messages.
		sitalDecodedMessageStructure dmsDecodedMessage; // A decoded message.
		sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded: unit - %i, purge - %i\n", s_swDevice3, TRUE);
		swResult = sitalMt_Message_GetFromStackDecoded (s_swDevice3, &dmsDecodedMessage, sitalMessageLocationAndRemoval_NEXT_PURGE, sitalMtStack_ACTIVE);
		if ((0 > swResult) || (1 < swResult))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Error: error - %i\n", swResult);
			throw 0;
		}
		if (1 == swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Error: error - %s\n", "extra_message");
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// Stop device-3 (MT).
		sitalProcess_Log_PrintLine ("#StldTester.Mt.Stop: unit - %i\n", s_swDevice3);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Stop (s_swDevice3) : sitalMt_Stop (s_swDevice3) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Mt.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice2);
		swResult = sitalDevice_Free (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-3 (MT).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice3);
		swResult = sitalDevice_Free (s_swDevice3);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		swResult = stldSyncEnd (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncEnd.Error: error - %i\n", swResult);
			throw 0;
		}
	}
	catch (int)
	{
		sitalDevice_Free (s_swDevice2);
		sitalDevice_Free (s_swDevice3);
	}

	sitalProcess_Log_PrintLine ("#StldTester.AtpTest461_EmptyFrame.End: test_succeeded - %c\n", ((sitalReturnCode_SUCCESS == swResult) ? 'Y' : 'N'));
	return swResult;
}



/// <summary>
/// Test the stld1553 library in the henceforth described scenario:
/// Participant[s]: RT1, BC2.
/// Configuration: All participating devices must be able to communicate with each other.
/// Short description: Verify that after a RT receives a mode synchronize with data message, it may immediately receive messages (i.e., without having to recover).
/// Along test print suitable info and indications into the log.
///
/// Note:
/// - This function may use any of above defined Acceptance Test Plan (ATP) tests variables.
///   The caller must therefore call the suitable stldTester_Atp_Set* functions prior to calling this function.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_BcRt_RtPostSynchronizationBehavior ()
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	sitalProcess_Log_PrintLine ("#StldTester.BcRt_RtPostSynchronizationBehavior.Start\n");

	/// Set test arguments.
	U16BIT wRtSubaddressRx; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31) that's set for a RT for Rx.
	wRtSubaddressRx = sitalRtSubaddress_1;
	U16BIT wWordCount; // Word count (1-32).
	wWordCount = 32;
	U16BIT wFrameTime; // The time (in units of 100 us) that the frame will take to complete.
	wFrameTime = 10*10; // 1ms, used for both minor frames, the one containing the BC-to-RT message, and the one containing the BC-to-RT-mode message.
	U16BIT wCountOfBcToRtMessages; // Number of leading BC-to-RT messages to transmit before a trailing BC-to-RT-mode-synchronize with data message in each iteration.
	wCountOfBcToRtMessages = 100 / IP_FREQUENCY_FACTOR; //reducing the number of messages to match RT stack depth
	U16BIT wFrameShotCount; // Number of frame shots.
	wFrameShotCount = 100;
	U16BIT wInterRtMessageReadTime; // The time (in units of 100 us) to wait between each to consequent message reading sessions from the RT stack along frame run.
	wInterRtMessageReadTime = (100 /*/ IP_FREQUENCY_FACTOR*/); // 10ms.

	S16BIT swResult; // Result of operation or function call.
	swResult = sitalReturnCode_SUCCESS;

	/// Get device count.
	U16BIT wRequiredDeviceCount; // The number of devices required for this test.
	wRequiredDeviceCount = (1 + max(s_swDevice1, s_swDevice2));
	U16BIT wAvailableDeviceCount; // The total number of devices available in the system.
	swResult = sitalDevice_GetCount (&wAvailableDeviceCount);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %i\n", swResult);
		return swResult;
	}
	sitalProcess_Log_PrintLine ("#StldTester.Device.Count: available - %i, required - %i\n", wAvailableDeviceCount, wRequiredDeviceCount);
	if (wAvailableDeviceCount < wRequiredDeviceCount)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %s\n", "not_enough_devices_installed");
		return sitalReturnCode_SUCCESS;
	}
	if ( (s_swDevice1 == s_swDevice2) )
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Number.Error: error - %s\n", "same_device_used_twice");
		return sitalReturnCode_INVALID_DEVICE_NUMBER;
	}

	try
	{
		swResult = stldSyncStart (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncStart.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-1 (RT) as a real RT in advanced mode.
		// Note:
		// - The default configuration of initialized RTs is "separate broadcast data", so no special configuration shall be made in favor of this test.
		// - Advanced mode will enable r/w to the memory of device-1 (RT) while the device is running.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-rt, alloc_mem-none\n", s_swDevice1);
		swResult = sitalDevice_Initialize (s_swDevice1, sitalAccess_CARD, ((s_bIsRtMtModeTest ? sitalMode_RT_AND_MT : sitalMode_RT) | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Set device-1 (RT) address to RtAddress1.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set: unit - %i, address - %i\n", s_swDevice1, s_wRtAddress1);
		swResult = sitalRt_Address_Set (s_swDevice1, s_wRtAddress1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-1 (RT) data-block-0, a data block of 32 data words.
		S16BIT swRtDataBlockId_0; // A unique ID designating a data block (0-(sitalRtCounter_DATA_BLOCKS-1)).
		swRtDataBlockId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.Create: unit - %i, block - %i, type - %i, buf_size - %i\n", s_swDevice1, swRtDataBlockId_0, sitalRtDataBlockType_SINGLE_32, wWordCount);
		swResult = sitalRt_DataBlock_Create (s_swDevice1, swRtDataBlockId_0, sitalRtDataBlockType_SINGLE_32, NULL, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Map for device-1 (RT) data-block-0 with subaddress-RtSubAddressRx for Rx, and legalize this subaddress.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.MapToSa: unit - %i, block - %i, sa - %i, msg_types - x%X, irq_opts-x%X, legalize - %c\n", s_swDevice1, swRtDataBlockId_0, wRtSubaddressRx, sitalRtMessageType_RX, sitalRtDataBlockIrq_NONE, 'Y');
		swResult = sitalRt_DataBlock_MapToSubaddress (s_swDevice1, swRtDataBlockId_0, wRtSubaddressRx, sitalRtMessageType_RX, sitalRtDataBlockIrq_NONE, TRUE);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.DataBlock.MapToSa.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Start device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Start: unit - %i\n", s_swDevice1);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Start (s_swDevice1) : sitalRt_Start (s_swDevice1) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-2 (BC) as a real BC.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-bc, alloc_mem-none\n", s_swDevice2);
		swResult = sitalDevice_Initialize (s_swDevice2, sitalAccess_CARD, (sitalMode_BC | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) data-block-0, a data block of 32 data words.
		S16BIT swDataBlockId_0; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId_0 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i\n", s_swDevice2, swDataBlockId_0, 32, 0);
		swResult = sitalBc_DataBlock_Create (s_swDevice2, swDataBlockId_0, sitalBcDataBlockSizeOption_SINGLE, NULL, wWordCount);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) message-0, a BC-to-RT(RT-address-1/RtSubAddressRx message that uses data-block-0 to transmit WordCount data words. Message bus: bus-1.
		S16BIT swMessageId_0; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create: unit - %i, message - %i, block - %i, rx_rt - %i, rx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddressRx, wWordCount, 0, s_wBus1);
		swResult = sitalBc_Message_CreateBcToRt (s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddressRx, wWordCount, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		///	Use given message descriptors to code the command word of above defined message.
		// Note: Defined messages have a different command word.
		U16BIT wCommandWord0; // A 1553 command word.
		swResult = sitalBcStld1553_Command_GetCode (&wCommandWord0, s_wRtAddress1, sitalCommand_RX, wRtSubaddressRx, wWordCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Command.GetCode.Error: error - %i\n", swResult);
			throw 0;
		}
		sitalProcess_Log_PrintLine ("#StldTester.Command.GetCode.BcToRt: command - %04X\n", wCommandWord0);

		/// Create for device-2 (BC) command-0, a XEQ command that executes message-0.
		S16BIT swCommandId_0; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) minor-frame-0, a frame that contains command-0 and whose time is FrameTime.
		S16BIT swaCommandIds[sitalBcCounter_COMMANDS]; // An array of command-IDs.
		swaCommandIds[0] = swCommandId_0;
		U16BIT wCommandIdCount; // The number of command IDs in the frame (1-sitalBcCounter_FRAME_COMMANDS).
		wCommandIdCount = 1;
		S16BIT swFrameId_0; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_0, "minor", wCommandIdCount, swCommandId_0, wFrameTime, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_0, sitalBcFrameType_MINOR, swaCommandIds, wCommandIdCount, wFrameTime, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) message-1, a BC-to-RT(RT-address-1)-Mode(10001) message that uses data-block-0. Message bus: bus-1.
		S16BIT swMessageId_1; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_1 = 1;
		U16BIT wMessageDirection; // RT-related message direction.
		wMessageDirection = sitalMessageDirection_RX;
		U16BIT wModeCommand; // A mode command.
		wModeCommand = 0x11;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRtMode.Create: unit - %i, message - %i, block - %i, rx_rt - %i, direction - %i, mode - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_1, swDataBlockId_0, s_wRtAddress1, wMessageDirection, wModeCommand, 0, s_wBus1);
		swResult = sitalBc_Message_CreateBcToOrFromRtMode (s_swDevice2, swMessageId_1, swDataBlockId_0, s_wRtAddress1, wMessageDirection, wModeCommand, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRtMode.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		///	Use given message descriptors to code the command word of above defined message.
		// Note: Defined messages have a different command word.
		U16BIT wCommandWord1; // A 1553 command word.
		swResult = sitalBcStld1553_Command_GetCode (&wCommandWord1, s_wRtAddress1, sitalCommand_RX, sitalRtSubaddress_0, wModeCommand);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Command.GetCode.Error: error - %i\n", swResult);
			throw 0;
		}
		sitalProcess_Log_PrintLine ("#StldTester.Command.GetCode.BcToRtMode: command - %04X\n", wCommandWord1);

		/// Create for device-2 (BC) command-1, a XEQ command that executes message-1.
		S16BIT swCommandId_1; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_1 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_1, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_1, 0);
		//swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_1, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0, 0); // This way the test succeeds.
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_1, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_1, 0, 0); // That's the real test: This way the test fails.
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) minor-frame-1, a frame that contains command-1 and whose time is FrameTime.
		swaCommandIds[0] = swCommandId_1;
		wCommandIdCount = 1;
		S16BIT swFrameId_1; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_1 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_1, "minor", wCommandIdCount, swCommandId_1, wFrameTime, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_1, sitalBcFrameType_MINOR, swaCommandIds, wCommandIdCount, wFrameTime, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-2, a FOCUS-ON-VAR command that focuses on variable-0.
		S16BIT swCommandId_2; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_2 = 2;
		U32BIT dwVariable_0; // A number of device variable.
		dwVariable_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_2, sitalOpcode_FOCUS_ON_VARIABLE, sitalOpcodeCondition_ALWAYS, dwVariable_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_2, sitalOpcode_FOCUS_ON_VARIABLE, sitalOpcodeCondition_ALWAYS, dwVariable_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-3, a LOAD-VAR command that loads wCountOfBcToRtMessages to variable-0.
		S16BIT swCommandId_3; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_3 = 3;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_3, sitalOpcode_LOAD_TO_VARIABLE, sitalOpcodeCondition_ALWAYS, wCountOfBcToRtMessages, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_3, sitalOpcode_LOAD_TO_VARIABLE, sitalOpcodeCondition_ALWAYS, wCountOfBcToRtMessages, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-4, a CMP-VAR command that compares variable-0 to zero.
		S16BIT swCommandId_4; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_4 = 4;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_4, sitalOpcode_COMPARE_VARIABLE, sitalOpcodeCondition_ALWAYS, 0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_4, sitalOpcode_COMPARE_VARIABLE, sitalOpcodeCondition_ALWAYS, 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-5, a JMP command that jumps out of the BC-to-RT message transmission loop once the comparison made by the previous command proves true.
		S16BIT swCommandId_5; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_5 = 5;
		U32BIT dwJumpOffset; // An offset (in number of commands) to jump (in two's-complement representation).
		dwJumpOffset = +4; // The offset to the 9th command in below defined major-frame-2, command-9, which is the first command after the BC-to-RT message transmission loop.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_5, sitalOpcode_JUMP, sitalOpcodeCondition_EQUAL, dwJumpOffset, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_5, sitalOpcode_JUMP, sitalOpcodeCondition_EQUAL, dwJumpOffset, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-6, a ADD-VAR command that decrements the value of variable-0 by one.
		S16BIT swCommandId_6; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_6 = 6;
		U32BIT dwTwoComplementValue; // A value in two's-complement representation.
		dwTwoComplementValue = -1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %X, prm2 - %i\n", s_swDevice2, swCommandId_6, sitalOpcode_ADD_TO_VARIABLE, sitalOpcodeCondition_ALWAYS, dwTwoComplementValue, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_6, sitalOpcode_ADD_TO_VARIABLE, sitalOpcodeCondition_ALWAYS, dwTwoComplementValue, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-7, a CAL command that executes minor-frame-0.
		S16BIT swCommandId_7; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_7 = 7;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_7, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, swFrameId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_7, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, (U16BIT)swFrameId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-8, a JMP command that jumps to the beginning of the BC-to-RT message transmission loop.
		S16BIT swCommandId_8; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_8 = 8;
		dwJumpOffset = -5; // The offset to the 3rd command in below defined major-frame-2, command-2, which is the first command after the BC-to-RT message transmission loop.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_8, sitalOpcode_JUMP, sitalOpcodeCondition_ALWAYS, dwJumpOffset, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_8, sitalOpcode_JUMP, sitalOpcodeCondition_ALWAYS, dwJumpOffset, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-9, a CAL command that executes minor-frame-1.
		S16BIT swCommandId_9; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_9 = 9;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_9, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, swFrameId_1, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_9, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, (U16BIT)swFrameId_1, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) major-frame-2, a frame that contains commands 2, 3, 2, 4, 5, 6, 7, 8, 9.
		S16BIT swFrameId_2; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_2 = 2;
		swaCommandIds[0] = swCommandId_2;
		swaCommandIds[1] = swCommandId_3;
		swaCommandIds[2] = swCommandId_2;
		swaCommandIds[3] = swCommandId_4;
		swaCommandIds[4] = swCommandId_5;
		swaCommandIds[5] = swCommandId_6;
		swaCommandIds[6] = swCommandId_7;
		swaCommandIds[7] = swCommandId_8;
		swaCommandIds[8] = swCommandId_9;
		wCommandIdCount = 9;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i,%i,%i,%i-%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_2, "major", wCommandIdCount, swCommandId_2, swCommandId_3, swCommandId_2, swCommandId_4, swCommandId_9, 0, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_2, sitalBcFrameType_MAJOR, swaCommandIds, wCommandIdCount, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Start device-2 (BC) running major-frame-2 for FrameShotCount-shots.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", s_swDevice2, swFrameId_2, wFrameShotCount);
		swResult = sitalBc_Start (s_swDevice2, swFrameId_2, wFrameShotCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// As long as device-2 (BC) is busy:
		///		Wait for InterRtMessageReadTime ms.
		///		Get the activation state of device-2 (BC).
		///		As long as device-1 (RT) has newly recorded messages in its stack:
		///			With device-1 (RT) get the next message.
		///			Increment the count of received messages for device-1 (RT).
		///			Record the sequence of received messages.
		sitalDecodedMessageStructure dmsDecodedMessage; // A decoded message.
		U32BIT dwRtCountOfReceivedMessages; // The number of messages actually received by a RT device.
		dwRtCountOfReceivedMessages = 0;
		U16BIT wCurrentBcState; // The current state of a BC device (sitalBcActivationState_*).
		U16BIT wPreviousCommandWord; // A 1553 command word.
		wPreviousCommandWord = 0;
		U32BIT dwCountOfSuccessiveSameCommandWords; // The number of successive same command words.
		dwCountOfSuccessiveSameCommandWords = 0;
		do
		{
			sitalProcess_Log_PrintLine ("#StldTester.Wait: time - %ims\n", (wInterRtMessageReadTime / 10));
			sitalProcess_Log_PrintLine ("\tWAIT_FOR %i us\n", ((U32BIT)wInterRtMessageReadTime * 100));
			PrecisionTimer pt; // A precision timer.
			pt.AccuratelySleep(wInterRtMessageReadTime / 10);
						
			sitalProcess_Log_PrintLine ("#StldTester.Bc.GetActivationState: unit - %i\n", s_swDevice2);
			swResult = sitalBc_GetActivationState (s_swDevice2, &wCurrentBcState);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.GetActivationState.Error: error - %i\n", swResult);
				throw 0;
			}

			do
			{
				sitalProcess_Log_PrintLine ("#StldTester.Rt.Message.GetFromStackDecoded: unit - %i, purge - %i\n", s_swDevice1, TRUE);
				swResult = sitalRt_Message_GetFromStackDecoded (s_swDevice1, &dmsDecodedMessage, sitalMessageLocationAndRemoval_NEXT_PURGE);
				if (0 > swResult)
				{
					sitalProcess_Log_PrintLine ("#StldTester.Rt.Message.GetDecoded.Error: error - %i\n", swResult);
					swResult = sitalReturnCode_TEST_FAIL -1;
					throw 0;
				}
				if (0 == swResult) {
					pt.AccuratelySleep(0);
					continue;
				}
				sitalProcess_Log_PrintLine ("#StldTester.Rt.Message: unit - %i, command - %04X, data_count - %i, time_tag - %04X\n", s_swDevice1, dmsDecodedMessage.wCommandWord1, dmsDecodedMessage.wDataWordCount, dmsDecodedMessage.wTimeTag);
				dwRtCountOfReceivedMessages += swResult; // when isn't negative, the result must be either 0 (no message read) or 1 (single message read).

				if (wCommandWord0 == dmsDecodedMessage.wCommandWord1)
				{
					if ( (wCommandWord0 != wPreviousCommandWord) && (0 < dwCountOfSuccessiveSameCommandWords) )
					{
						sitalProcess_Log_PrintLine ("#StldTester.Rt.Message.Sequence: unit - %i, type - Bc-to-RT-Mode, count - %i\n", s_swDevice1, dwCountOfSuccessiveSameCommandWords);
						dwCountOfSuccessiveSameCommandWords = 0;
					}
				}
				else
				if (wCommandWord1 == dmsDecodedMessage.wCommandWord1)
				{
					if ( (wCommandWord1 != wPreviousCommandWord) && (0 < dwCountOfSuccessiveSameCommandWords) )
					{
						sitalProcess_Log_PrintLine ("#StldTester.Rt.Message.Sequence: unit - %i, type - Bc-to-RT, count - %i\n", s_swDevice1, dwCountOfSuccessiveSameCommandWords);
						dwCountOfSuccessiveSameCommandWords = 0;
					}
				}
				else
				{
					sitalProcess_Log_PrintLine ("#StldTester.Mt.GetFromStackDecoded.Message.Error: error - %s\n", "bad_command");
					swResult = sitalReturnCode_TEST_FAIL -2;
					throw 0;
				}

				wPreviousCommandWord = dmsDecodedMessage.wCommandWord1;
				dwCountOfSuccessiveSameCommandWords++;
			}
			while (1 == swResult);
		}
		while (sitalBcActivationState_BUSY == wCurrentBcState);
		if (0 < dwCountOfSuccessiveSameCommandWords)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Message.Sequence: unit - %i, type - %s, count - %i\n", s_swDevice1, ( (wCommandWord0 == dmsDecodedMessage.wCommandWord1) ? "Bc-to-RT" : "Bc-to-RT-Mode" ), dwCountOfSuccessiveSameCommandWords);
		}

		///	Verify that the count of received messages for device-1 (RT) is the expected one.
		U32BIT dwExpectedRtCountOfReceivedMessages; // The expected number of messages to receive by a RT device.
		dwExpectedRtCountOfReceivedMessages = ( (U32BIT)wFrameShotCount * ( (U32BIT)wCountOfBcToRtMessages + 1/*Mode-Message*/ ) );
		sitalProcess_Log_PrintLine ("#StldTester.Rt.MessageReception.Summary: unit - %i, expected_rx_message_count - %i, actual_rx_message_count - %i\n", s_swDevice1, dwExpectedRtCountOfReceivedMessages, dwRtCountOfReceivedMessages);
		if (dwExpectedRtCountOfReceivedMessages != dwRtCountOfReceivedMessages)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.MessageReception.Error: error - %s\n", "bad_total");
			swResult = sitalReturnCode_TEST_FAIL -3;
			throw 0;
		}

		/// Stop device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Stop: unit - %i\n", s_swDevice1);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Stop (s_swDevice1) : sitalRt_Stop (s_swDevice1) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Stop device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop: unit - %i\n", s_swDevice2);
		swResult = sitalBc_Stop (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice1);
		swResult = sitalDevice_Free (s_swDevice1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice2);
		swResult = sitalDevice_Free (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		swResult = stldSyncEnd (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncEnd.Error: error - %i\n", swResult);
			throw 0;
		}
	}
	catch (int)
	{
		sitalDevice_Free (s_swDevice2);
		sitalDevice_Free (s_swDevice1);
	}

	sitalProcess_Log_PrintLine ("#StldTester.BcRt_RtPostSynchronizationBehavior.End: test_succeeded - %c\n", ((sitalReturnCode_SUCCESS == swResult) ? 'Y' : 'N'));
	return swResult;
}



/// <summary>
/// Test the stld1553 library in the henceforth described scenario:
/// Participant[s]: TEST1.
/// Configuration: All participating devices must be able to communicate with each other.
/// Short description: Verify that the stld1553 internal tests pass.
/// Along test print suitable info and indications into the log.
///
/// Note:
/// - This function may use any of above defined Acceptance Test Plan (ATP) tests variables.
///   The caller must therefore call the suitable stldTester_Atp_Set* functions prior to calling this function.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Bc_InternalTests ()
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	sitalProcess_Log_PrintLine ("#StldTester.Bc_InternalTests.Start\n");

	/// Set test arguments.
	BOOLEAN bTestRegisters; // A flag that says whether a specific internal test should be performed.
	bTestRegisters = TRUE;
	BOOLEAN bTestMemory; // A flag that says whether a specific internal test should be performed.
	bTestMemory = TRUE;
	BOOLEAN bTestProtocol; // A flag that says whether a specific internal test should be performed.
	bTestProtocol = TRUE; // DEBUG FAILURE !!!
	BOOLEAN bTestInterrupts; // A flag that says whether a specific internal test should be performed.
	bTestInterrupts = TRUE;
	BOOLEAN bTestVector; // A flag that says whether a specific internal test should be performed.
	bTestVector = TRUE;
	U16BIT wMemoryTestWrittenValue; // A value to fill device memory with along the internal memory test.
	wMemoryTestWrittenValue = 0x1234;
	char szVectorsFilePath[] = "VectorsTestFile.txt"; // The path to the vectors file to be used with the internal vectors test.

	S16BIT swResult; // Result of operation or function call.
	swResult = sitalReturnCode_SUCCESS;

	/// Get device count.
	U16BIT wRequiredDeviceCount; // The number of devices required for this test.
	wRequiredDeviceCount = (1 + s_swDevice1);
	U16BIT wAvailableDeviceCount; // The total number of devices available in the system.
	swResult = sitalDevice_GetCount (&wAvailableDeviceCount);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %i\n", swResult);
		return swResult;
	}
	sitalProcess_Log_PrintLine ("#StldTester.Device.Count: available - %i, required - %i\n", wAvailableDeviceCount, wRequiredDeviceCount);
	if (wAvailableDeviceCount < wRequiredDeviceCount)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %s\n", "not_enough_devices_installed");
		return sitalReturnCode_SUCCESS;
	}

	try
	{
		swResult = stldSyncStart (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncStart.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-1 (TEST) as a TEST device.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-test, alloc_mem-none\n", s_swDevice1);
		swResult = sitalDevice_Initialize (s_swDevice1, sitalAccess_CARD, sitalMode_TEST, 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		sitalDeviceTestResultStructure dtrs; // A structure to receive within test results.
		if (TRUE == bTestRegisters)
		{
			/// Perform registers test with device-1 (TEST).
			sitalProcess_Log_PrintLine ("#StldTester.Device.Test.Registers: unit - %i\n", s_swDevice1);
			swResult = sitalDevice_Test_Registers (s_swDevice1, &dtrs);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Device.Test.Registers.Error: error - %i\n", swResult);
				throw 0;
			}
			if (sitalDeviceTestResult_PASS != dtrs.wResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Device.Test.Registers.Fail: fail - %i, check_id - %i, address - %04X, expected_value - %04X, actual_value - %04X\n", dtrs.wResult, dtrs.wCheckId, dtrs.wDeviceMemoryAddress, dtrs.wExpectedValue, dtrs.wActualValue);
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}
		}

		if (TRUE == bTestMemory)
		{
			/// Perform memory test with device-1 (TEST).
			sitalProcess_Log_PrintLine ("#StldTester.Device.Test.Memory: unit - %i, written_value - %04X\n", s_swDevice1, wMemoryTestWrittenValue);
			swResult = sitalDevice_Test_Memory (s_swDevice1, &dtrs, wMemoryTestWrittenValue);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Device.Test.Memory.Error: error - %i\n", swResult);
				throw 0;
			}
			if (sitalDeviceTestResult_PASS != dtrs.wResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Device.Test.Memory.Fail: fail - %i, check_id - %i, address - %04X, expected_value - %04X, actual_value - %04X\n", dtrs.wResult, dtrs.wCheckId, dtrs.wDeviceMemoryAddress, dtrs.wExpectedValue, dtrs.wActualValue);
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}
		}

		if (TRUE == bTestProtocol)
		{
			/// Perform protocol test with device-1 (TEST).
			sitalProcess_Log_PrintLine ("#StldTester.Device.Test.Protocol: unit - %i\n", s_swDevice1);
			swResult = sitalDevice_Test_Protocol (s_swDevice1, &dtrs);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Device.Test.Protocol.Error: error - %i\n", swResult);
				throw 0;
			}
			if (sitalDeviceTestResult_PASS != dtrs.wResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Device.Test.Protocol.Fail: fail - %i, check_id - %i, address - %04X, expected_value - %04X, actual_value - %04X\n", dtrs.wResult, dtrs.wCheckId, dtrs.wDeviceMemoryAddress, dtrs.wExpectedValue, dtrs.wActualValue);
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}
		}

		if (TRUE == bTestInterrupts)
		{
			/// Perform interrupt test with device-1 (TEST).
			sitalProcess_Log_PrintLine ("#StldTester.Device.Test.Interrupts: unit - %i\n", s_swDevice1);
			swResult = sitalDevice_Test_Interrupts (s_swDevice1, &dtrs);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Device.Test.Interrupts.Error: error - %i\n", swResult);
				throw 0;
			}
			if (sitalDeviceTestResult_PASS != dtrs.wResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Device.Test.Interrupts.Fail: fail - %i, check_id - %i, address - %04X, expected_value - %04X, actual_value - %04X\n", dtrs.wResult, dtrs.wCheckId, dtrs.wDeviceMemoryAddress, dtrs.wExpectedValue, dtrs.wActualValue);
				swResult = sitalReturnCode_TEST_FAIL;
				throw 0;
			}
		}

		if (TRUE == bTestVector)
		{
			/// Perform vectors test with device-1 (TEST).
			sitalProcess_Log_PrintLine ("#StldTester.Device.Test.Vectors: unit - %i, vectors_file_path - %s\n", s_swDevice1, szVectorsFilePath);
			swResult = sitalDevice_Test_Vectors (s_swDevice1, &dtrs, szVectorsFilePath);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Device.Test.Vectors.Error: error - %i\n", swResult);
				throw 0;
			}
			if (sitalDeviceTestResult_PASS != dtrs.wResult)
			{
				swResult = sitalReturnCode_TEST_FAIL;
				sitalProcess_Log_PrintLine ("#StldTester.Device.Test.Vectors.Fail: fail - %i, check_id - %i, address - %04X, expected_value - %04X, actual_value - %04X\n", dtrs.wResult, dtrs.wCheckId, dtrs.wDeviceMemoryAddress, dtrs.wExpectedValue, dtrs.wActualValue);
				throw 0;
			}
		}

		/// Free device-1 (TEST).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice1);
		swResult = sitalDevice_Free (s_swDevice1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		// Now perform vectors test on a real device (i.e., not on a device that has been initialized as a test device, unlike above).

		/// Initialize device-1 (RT) as a real RT in advanced mode.
		// Note: In this test the device is always initialized as a regular RT, and not as RT&MT.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-rt, alloc_mem-none\n", s_swDevice1);
		swResult = sitalDevice_Initialize (s_swDevice1, sitalAccess_CARD, (sitalMode_RT | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Set device-1 (RT) address to RtAddress1.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set: unit - %i, address - %i\n", s_swDevice1, s_wRtAddress1);
		swResult = sitalRt_Address_Set (s_swDevice1, s_wRtAddress1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Start device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Start: unit - %i\n", s_swDevice1);
		swResult = sitalRt_Start (s_swDevice1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		if (TRUE == bTestVector)
		{
			/// Perform vectors test with device-1 (RT).
			sitalProcess_Log_PrintLine ("#StldTester.Device.Test.Vectors: unit - %i, vectors_file_path - %s\n", s_swDevice1, szVectorsFilePath);
			swResult = sitalDevice_Test_Vectors (s_swDevice1, &dtrs, szVectorsFilePath);
			if (sitalReturnCode_SUCCESS != swResult)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Device.Rt.Vectors.Error: error - %i\n", swResult);
				throw 0;
			}
			if (sitalDeviceTestResult_PASS != dtrs.wResult)
			{
				swResult = sitalReturnCode_TEST_FAIL;
				sitalProcess_Log_PrintLine ("#StldTester.Device.Test.Vectors.Fail: fail - %i, check_id - %i, address - %04X, expected_value - %04X, actual_value - %04X\n", dtrs.wResult, dtrs.wCheckId, dtrs.wDeviceMemoryAddress, dtrs.wExpectedValue, dtrs.wActualValue);
				throw 0;
			}
		}

		/// Stop device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Stop: unit - %i\n", s_swDevice1);
		swResult = sitalRt_Stop (s_swDevice1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice1);
		swResult = sitalDevice_Free (s_swDevice1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		swResult = stldSyncEnd (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncEnd.Error: error - %i\n", swResult);
			throw 0;
		}
	}
	catch (int)
	{
		sitalDevice_Free (s_swDevice1);
	}

	sitalProcess_Log_PrintLine ("#StldTester.Bc_InternalTests.End: test_succeeded - %c\n", ((sitalReturnCode_SUCCESS == swResult) ? 'Y' : 'N'));
	return swResult;
}



/// <summary>
/// Test the stld1553 library in the henceforth described scenario:
/// Participant[s]: RT1, BC2.
/// Configuration: All participating devices must be able to communicate with each other.
/// Short description: Verify that the expected condition codes are received.
/// Along test print suitable info and indications into the log.
///
/// Note:
/// - This function may use any of above defined Acceptance Test Plan (ATP) tests variables.
///   The caller must therefore call the suitable stldTester_Atp_Set* functions prior to calling this function.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL stldTester_Bc_ConditionCode ()
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	sitalProcess_Log_PrintLine ("#StldTester.Bc_ConditionCode.Start\n");

	/// Set test arguments.
	U16BIT wRtSubaddress; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31) that's set for a RT for Rx.
	wRtSubaddress = 1;
	U16BIT wFrameCount; // The number of frames to run.
	wFrameCount = 100;
	U16BIT wFrameTime; // The time (in units of 100 us) that the frame will take to complete.
	wFrameTime = 1000; // 1000 * 100us = 100000us = 100ms.

	S16BIT swResult; // Result of operation or function call.
	swResult = sitalReturnCode_SUCCESS;

	/// Get device count.
	U16BIT wRequiredDeviceCount; // The number of devices required for this test.
	wRequiredDeviceCount = (1 + max(s_swDevice1, s_swDevice2));
	U16BIT wAvailableDeviceCount; // The total number of devices available in the system.
	swResult = sitalDevice_GetCount (&wAvailableDeviceCount);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %i\n", swResult);
		return swResult;
	}
	sitalProcess_Log_PrintLine ("#StldTester.Device.Count: available - %i, required - %i\n", wAvailableDeviceCount, wRequiredDeviceCount);
	if (wAvailableDeviceCount < wRequiredDeviceCount)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %s\n", "not_enough_devices_installed");
		return sitalReturnCode_SUCCESS;
	}
	if ( (s_swDevice1 == s_swDevice2) )
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Number.Error: error - %s\n", "same_device_used_twice");
		return sitalReturnCode_INVALID_DEVICE_NUMBER;
	}

	try
	{
		swResult = stldSyncStart (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncStart.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-1 (RT) as a real RT in advanced mode.
		// Note: Advanced mode will enable r/w to the memory of device-1 (RT) while the device is running.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-rt, alloc_mem-none\n", s_swDevice1);
		swResult = sitalDevice_Initialize (s_swDevice1, sitalAccess_CARD, ((s_bIsRtMtModeTest ? sitalMode_RT_AND_MT : sitalMode_RT) | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Set device-1 (RT) address to RtAddress0.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set: unit - %i, address - %i\n", s_swDevice1, s_wRtAddress1);
		swResult = sitalRt_Address_Set (s_swDevice1, s_wRtAddress1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Address.Set.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Legalize on device-1 (RT) all address-types in all directions for all subaddresses and all word counts.
		sitalProcess_Log_PrintLine ("#StldTester.Rt.MessageLegality.Enable: unit - %i, address_type - %i, directions - %i, subaddress - %i, word_count - %08X\n", s_swDevice1, sitalRtAddressType_BOTH, sitalMessageDirection_BOTH, sitalRtSubaddress_ALL, 0xFFFFFFFF);
		swResult = sitalRt_MessageLegality_Enable (s_swDevice1, sitalRtAddressType_BOTH, sitalMessageDirection_BOTH, sitalRtSubaddress_ALL, 0xFFFFFFFF);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.MessageLegality.Enable.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Start device-1 (RT).
		sitalProcess_Log_PrintLine ("#StldTester.Rt.Start: unit - %i\n", s_swDevice1);
		swResult = ( s_bIsRtMtModeTest ? sitalRtMt_Start (s_swDevice1) : sitalRt_Start (s_swDevice1) );
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Rt.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-2 (BC) as a real BC.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-bc, alloc_mem-none\n", s_swDevice2);
		swResult = sitalDevice_Initialize (s_swDevice2, sitalAccess_CARD, (sitalMode_BC | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-2 (BC) as one that supports high priority asynchronous messages.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Init: unit - %i, options - %i\n", s_swDevice2, sitalBcSetupOption_ASYNCHRONOUS_HIGH_PRIORITY_MODE);
		swResult = sitalBc_Initialize (s_swDevice2, sitalBcSetupOption_ASYNCHRONOUS_HIGH_PRIORITY_MODE);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) data-block-0, a data block of 32 data words.
		S16BIT swDataBlockId_0; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId_0 = 1;
		U16BIT wWordCount; // Word count (1-32).
		wWordCount = 32; // See the remark for wFrameTime (below).
		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i\n", s_swDevice2, swDataBlockId_0, 32, 0);
		swResult = sitalBc_DataBlock_Create (s_swDevice2, swDataBlockId_0, sitalBcDataBlockSizeOption_SINGLE, NULL, wWordCount);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) message-0, a BC-to-RT(RT-address-1/RtSubAddressRx message that uses data-block-0 to transmit WordCount data words. Message bus: bus-1.
		S16BIT swMessageId_0; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create: unit - %i, message - %i, block - %i, rx_rt - %i, rx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddress, wWordCount, 0, s_wBus1);
		swResult = sitalBc_Message_CreateBcToRt (s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddress, wWordCount, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-0, a XEQ command that executes message-0.
		S16BIT swCommandId_0; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-1, a subroutine call command that executes minor-frame-0.
		S16BIT swCommandId_1; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_1 = 1;
		S16BIT swFrameId_0; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_1, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, swFrameId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_1, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, (U16BIT)swFrameId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) minor-frame-0, a frame that contains command-0 twice.
		S16BIT swaCommandIds[sitalBcCounter_COMMANDS]; // An array of command-IDs.
		swaCommandIds[0] = swCommandId_0;
		swaCommandIds[1] = swCommandId_0;
		U16BIT wCommandIdCount; // The number of command IDs in the frame (1-sitalBcCounter_FRAME_COMMANDS).
		wCommandIdCount = 2;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_0, "minor", wCommandIdCount, swCommandId_0, swCommandId_0, 0, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_0, sitalBcFrameType_MINOR, swaCommandIds, wCommandIdCount, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) major-frame-1, a frame that contains command-1 and whose time is FrameTime.
		S16BIT swFrameId_1; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_1 = 1;
		swaCommandIds[0] = swCommandId_1;
		wCommandIdCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_1, "major", wCommandIdCount, swCommandId_1, wFrameTime, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_1, sitalBcFrameType_MAJOR, swaCommandIds, wCommandIdCount, wFrameTime, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Get for device-2 (BC) the current interrupt configuration.
		U16BIT wInterruptMode; // Current configuration for type of interrupt signal (sitalIrqMode_*).
		U16BIT wAutoClear; // Current configuration for post-read auto-clearing of interrupt status (sitalIrqClear_*).
		swResult = sitalDevice_Irq_GetMode (s_swDevice2, &wInterruptMode, &wAutoClear);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Mode.Get.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Configure device-2 (BC) to issue suitable interrupts and perform post-read auto-clearing of interrupt status.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Configure: unit - %i, mode - %i, auto_clear - %i\n", s_swDevice2, wInterruptMode, sitalIrqClear_NO_AUTO_CLEAR);
		swResult = sitalDevice_Irq_Configure (s_swDevice2, wInterruptMode, sitalIrqClear_NO_AUTO_CLEAR);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Configure.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Configure device-2 (BC) to get IRQ-3 notifications using an ISR that collects interrupt data, counts ISR calls, and signals interested threads.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Manipulate: unit - %i, enable - %i, irq - %08X, callback - %s\n", s_swDevice2, TRUE, sitalInterruptRegister2_BC_IRQ3, "stldTester_Bc_UserIsr");
		swResult = sitalDevice_Irq_Manipulate (s_swDevice2, TRUE, sitalInterruptRegister2_BC_IRQ3, stldTester_Bc_UserIsr);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Set.Error: error - %i\n", swResult);
			throw 0;
		}
		s_dwBcIsrCallCounter = 0;
		s_dwBcIrqStatus = sitalInterruptRegister2_BC_IRQ3; // So that below conditions won't fail the test right before the first frame is transmitted.

		///	Start device-2 (BC) running major-frame-1 for FrameCount shots.
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", s_swDevice2, swFrameId_1, wFrameCount);
		swResult = sitalBc_Start (s_swDevice2, swFrameId_1, wFrameCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Verify expected condition codes.
		U16BIT wIteration; // The current iteration.
		wIteration = 0;
		U16BIT wNoResponseConditionState; // No response condition state.
		wNoResponseConditionState = 0;
		U16BIT wGoodDataConditionState; // Good data condition state.
		wGoodDataConditionState = 0;
		U16BIT wFormatErrorConditionState; // Format error condition state.
		wFormatErrorConditionState = 0;
		U16BIT wBadMessageConditionState; // Bad message condition state.
		wBadMessageConditionState = 0;
		PrecisionTimer pt; // A precision timer.
		while ((wFrameCount > wIteration) && (wFrameCount > s_dwBcIsrCallCounter) && (0U != (sitalInterruptRegister2_BC_IRQ3 & s_dwBcIrqStatus)))
		{
			wIteration++;
			sitalProcess_Log_PrintLine ("#StldTester.Wait: time_passed - %ims, isr_calls - %i\n", wFrameTime, s_dwMinorFrameCounter);
			sitalProcess_Log_PrintLine ("\tWAIT_FOR %i us\n", (wFrameTime * 1000));
			pt.AccuratelySleep (wFrameTime / 10);

			if (wFrameCount > wIteration)
			{
				sitalProcess_Log_PrintLine ("#StldTester.Bc.ConditionState.Get: unit - %i, condition - %i\n", s_swDevice2, sitalBcConditionCode_NO_RESPONSE);
				swResult = sitalBc_GetConditionState (s_swDevice2, sitalBcConditionCode_NO_RESPONSE, &wNoResponseConditionState);
				if (sitalReturnCode_SUCCESS != swResult)
				{
					sitalProcess_Log_PrintLine ("#StldTester.Bc.ConditionState.Get.NoResponse.Error: error - %i\n", swResult);
					throw 0;
				}
				if (0U != wNoResponseConditionState)
				{
					swResult = sitalReturnCode_TEST_FAIL;
					sitalProcess_Log_PrintLine ("#StldTester.Bc.ConditionState.Error: error - %s\n", "no_response");
					throw 0;
				}

				swResult = sitalBc_GetConditionState (s_swDevice2, sitalBcConditionCode_GOOD_DATA_BLOCK_TRANSFER, &wGoodDataConditionState);
				if (sitalReturnCode_SUCCESS != swResult)
				{
					sitalProcess_Log_PrintLine ("#StldTester.Bc.ConditionState.Get.GoodData.Error: error - %i\n", swResult);
					throw 0;
				}
				if (0U != wGoodDataConditionState)
				{
					swResult = sitalReturnCode_TEST_FAIL;
					sitalProcess_Log_PrintLine ("#StldTester.Bc.ConditionState.Error: error - %s\n", "good_data");
					throw 0;
				}

				swResult = sitalBc_GetConditionState (s_swDevice2, sitalBcConditionCode_FORMAT_ERROR, &wFormatErrorConditionState);
				if (sitalReturnCode_SUCCESS != swResult)
				{
					sitalProcess_Log_PrintLine ("#StldTester.Bc.ConditionState.Get.FormatError.Error: error - %i\n", swResult);
					throw 0;
				}
				if (0U != wFormatErrorConditionState)
				{
					swResult = sitalReturnCode_TEST_FAIL;
					sitalProcess_Log_PrintLine ("#StldTester.Bc.ConditionState.Error: error - %s\n", "format_error");
					throw 0;
				}

				swResult = sitalBc_GetConditionState (s_swDevice2, sitalBcConditionCode_BAD_MESSAGE, &wBadMessageConditionState);
				if (sitalReturnCode_SUCCESS != swResult)
				{
					sitalProcess_Log_PrintLine ("#StldTester.Bc.ConditionState.Get.BadMessage.Error: error - %i\n", swResult);
					throw 0;
				}
				if (0U != wBadMessageConditionState)
				{
					swResult = sitalReturnCode_TEST_FAIL;
					sitalProcess_Log_PrintLine ("#StldTester.Bc.ConditionState.Error: error - %s\n", "bad_message");
					throw 0;
				}
			}
		}

		/// Verify that a correct number of interrupts of a suitable type occurred.
		if (0U == (sitalInterruptRegister2_BC_IRQ3 & s_dwBcIrqStatus))
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Interrupt.Mask.Error: error - %08X\n", s_dwBcIrqStatus);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}
		sitalProcess_Log_PrintLine ("#StldTester.Device.Interrupts: unit - %i, count - %i, expected_count - %i, last_mask - %08X, expected_mask - %08X\n", s_swDevice2, s_dwBcIsrCallCounter, wFrameCount, s_dwBcIrqStatus, sitalInterruptRegister1_BC_END_OF_FRAME);
		if (wFrameCount != s_dwBcIsrCallCounter)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Interrupt.CallCount.Error: error - %i\n", s_dwBcIsrCallCounter);
			swResult = sitalReturnCode_TEST_FAIL;
			throw 0;
		}

		/// Stop device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop: unit - %i\n", s_swDevice2);
		swResult = sitalBc_Stop (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Free device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice2);
		swResult = sitalDevice_Free (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		swResult = stldSyncEnd (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncEnd.Error: error - %i\n", swResult);
			throw 0;
		}
	}
	catch (int)
	{
		sitalDevice_Free (s_swDevice2);
	}

	sitalProcess_Log_PrintLine ("#StldTester.Bc_ConditionCode.End: test_succeeded - %c\n", ((sitalReturnCode_SUCCESS == swResult) ? 'Y' : 'N'));
	return swResult;
}



///// <summary>
///// Temporary.
/////
///// @return sitalReturnCode_SUCCESS if function successfully completed.
///// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
///// </summary>
//S16BIT _DECL temp ()
//{
//	/// @pseudocode
//
//	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
//	///		Return error.
//
//	sitalProcess_Log_PrintLine ("#StldTester.Temp.Start\n");
//
//	/// Set test arguments.
//	U16BIT wFrameTime; // The time (in units of 100 us) that the frame will take to complete.
//	wFrameTime = s_wFrameTime;
//	U32BIT dwTestIterationsCount; // The number of test iterations to perform.
//	dwTestIterationsCount = s_dwTestIterationsCount;
//	U16BIT wRtSubaddress1; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31).
//	wRtSubaddress1 = 30;
//
//	S16BIT swResult; // Result of operation or function call.
//	swResult = sitalReturnCode_SUCCESS;
//
//	/// Get device count.
//	U16BIT wRequiredDeviceCount; // The number of devices required for this test.
//	wRequiredDeviceCount = (1 + s_swDevice2);
//	U16BIT wAvailableDeviceCount; // The total number of devices available in the system.
//	swResult = sitalDevice_GetCount (&wAvailableDeviceCount);
//	if (sitalReturnCode_SUCCESS != swResult)
//	{
//		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %i\n", swResult);
//		return swResult;
//	}
//	sitalProcess_Log_PrintLine ("#StldTester.Device.Count: available - %i, required - %i\n", wAvailableDeviceCount, wRequiredDeviceCount);
//	if (wAvailableDeviceCount < wRequiredDeviceCount)
//	{
//		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %s\n", "not_enough_devices_installed");
//		return sitalReturnCode_SUCCESS;
//	}
//
//	try
//	{
//		swResult = stldSyncStart (__FUNCTION__);
//		if (sitalReturnCode_SUCCESS != swResult)
//		{
//			sitalProcess_Log_PrintLine ("#StldTester.stldSyncStart.Error: error - %i\n", swResult);
//			throw 0;
//		}
//
//		/// Initialize device-2 (BC) as a real BC.
//		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-bc, alloc_mem-none\n", s_swDevice2);
//		swResult = sitalDevice_Initialize (s_swDevice2, sitalAccess_CARD, (sitalMode_BC | sitalModeVariant_ADVANCED), 0, 0, 0);
//		if (sitalReturnCode_SUCCESS != swResult)
//		{
//			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
//			throw 0;
//		}
//
//		const U16BIT wWordCount = 32; // Word count (1-32).
//		U16BIT waBuffer[wWordCount]; // A buffer to read-from/write-into along data-block write/read operations, respectively.
//		for (int i=0; i<wWordCount; i++)
//		{
//			waBuffer[i] = 0xAA55;
//		}
//
//		/// Create for device-2 (BC) data-block-0, a data block of 32 data words.
//		S16BIT swDataBlockId_0; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
//		swDataBlockId_0 = 1;
//		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i\n", s_swDevice2, swDataBlockId_0, wWordCount, 0);
//		swResult = sitalBc_DataBlock_Create (s_swDevice2, swDataBlockId_0, sitalBcDataBlockSizeOption_SINGLE, NULL, wWordCount);
//		if (0 > swResult)
//		{
//			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
//			throw 0;
//		}
//
//		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Write: unit - %i, block - %i, size - %i, buf_size - %i (%04X...)\n", s_swDevice2, swDataBlockId_0, 32, wWordCount, waBuffer[0]);
//		swResult = sitalBc_DataBlock_Write (s_swDevice2, swDataBlockId_0, waBuffer, wWordCount, 0);
//		if (0 > swResult)
//		{
//			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Write.Error: error - %i\n", swResult);
//			throw 0;
//		}
//
//		/// Create for device-2 (BC) message-1, a BC-to-RT(RT-address-1/RtSubAddress1) message that uses data-block-0 to transmit WordCount data words. Message bus: bus-1.
//		S16BIT swMessageId_0; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
//		swMessageId_0 = 0;
//		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create: unit - %i, message - %i, block - %i, rx_rt - %i, rx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddress1, wWordCount, 0, s_wBus1);
//		swResult = sitalBc_Message_CreateBcToRt (s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddress1, wWordCount, 0, s_wBus1);
//		if (sitalReturnCode_SUCCESS != swResult)
//		{
//			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create.Error: error - %i\n", swResult);
//			throw 0;
//		}
//
//		/// Create for device-2 (BC) command-0, a XEQ command that executes message-0.
//		S16BIT swCommandId_0; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
//		swCommandId_0 = 0;
//		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0);
//		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0, 0);
//		if (sitalReturnCode_SUCCESS != swResult)
//		{
//			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
//			throw 0;
//		}
//
//		/// Create for device-2 (BC) data-block-1, a data block of 32 data words.
//		S16BIT swDataBlockId_1; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
//		swDataBlockId_1 = 2;
//		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i\n", s_swDevice2, swDataBlockId_1, wWordCount, 0);
//		swResult = sitalBc_DataBlock_Create (s_swDevice2, swDataBlockId_1, sitalBcDataBlockSizeOption_SINGLE, NULL, wWordCount);
//		if (0 > swResult)
//		{
//			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
//			throw 0;
//		}
//
//		/// Create for device-2 (BC) message-0, a RT(RtAddress1/RtSubAddress1)-to-BC message that uses data-block-1 to receive WordCount data words. Message bus: bus-1.
//		S16BIT swMessageId_1; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
//		swMessageId_1 = 1;
//		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBc.Create: unit - %i, message - %i, block - %i, tx_rt - %i, tx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_1, swDataBlockId_1, s_wRtAddress1, wRtSubaddress1, wWordCount, 0, s_wBus1);
//		swResult = sitalBc_Message_CreateRtToBc (s_swDevice2, swMessageId_1, swDataBlockId_1, s_wRtAddress1, wRtSubaddress1, wWordCount, 0, s_wBus1);
//		if (sitalReturnCode_SUCCESS != swResult)
//		{
//			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBc.Create.Error: error - %i\n", swResult);
//			throw 0;
//		}
//
//		/// Create for device-2 (BC) command-1, a XEQ command that executes message-0.
//		S16BIT swCommandId_1; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
//		swCommandId_1 = 1;
//		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_1, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_1, 0);
//		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_1, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_1, 0, 0);
//		if (sitalReturnCode_SUCCESS != swResult)
//		{
//			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
//			throw 0;
//		}
//
//		/// Create for device-2 (BC) command-2, a subroutine call command that executes minor-frame-0. 
//		S16BIT swCommandId_2; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
//		swCommandId_2 = 2;
//		S16BIT swFrameId_0; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
//		swFrameId_0 = 0;
//		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_2, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, swFrameId_0, 0);
//		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_2, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, (U16BIT)swFrameId_0, 0, 0);
//		if (sitalReturnCode_SUCCESS != swResult)
//		{
//			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
//			throw 0;
//		}
//
//		/// Create for device-2 (BC) minor-frame-0, a frame that contains command-0 and command-1.
//		S16BIT swaCommandIds[sitalBcCounter_COMMANDS]; // An array of command-IDs.
//		swaCommandIds[0] = swCommandId_0;
//		swaCommandIds[1] = swCommandId_1;
//		U16BIT wCommandIdCount; // The number of command IDs in the frame (1-sitalBcCounter_FRAME_COMMANDS).
//		wCommandIdCount = 2;
//		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i, %i), time - %i, flags - %i\n", s_swDevice2, swFrameId_0, "minor", wCommandIdCount, swCommandId_0, swCommandId_1, wFrameTime, 0);
//		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_0, sitalBcFrameType_MINOR, swaCommandIds, wCommandIdCount, wFrameTime, 0);
//		if (sitalReturnCode_SUCCESS != swResult)
//		{
//			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
//			throw 0;
//		}
//
//		/// Create for device-2 (BC) major-frame-1, a frame that contains command-2 and whose frame time is FrameTime us.
//		S16BIT swFrameId_1; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
//		swFrameId_1 = 1;
//		swaCommandIds[0] = swCommandId_2;
//		wCommandIdCount = 1;
//		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_1, "major", wCommandIdCount, swCommandId_2, (wFrameTime / 10), 0);
//		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_1, sitalBcFrameType_MAJOR, swaCommandIds, wCommandIdCount, wFrameTime, 0);
//		if (sitalReturnCode_SUCCESS != swResult)
//		{
//			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
//			throw 0;
//		}
//
//		/// Configure given device (BC) to get IRQ-3 notifications using an ISR that collects interrupt data, counts ISR calls, and signals interested threads.
//		sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Manipulate: unit - %i, enable - %i, irq - %08X, callback - %s\n", s_swDevice2, TRUE, sitalInterruptRegister2_BC_IRQ3, "stldTester_Bc_UserIsr");
//		swResult = sitalDevice_Irq_Manipulate (s_swDevice2, TRUE, sitalInterruptRegister2_BC_IRQ3, stldTester_Bc_UserIsr);
//		if (sitalReturnCode_SUCCESS != swResult)
//		{
//			sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Set.Error: error - %i\n", swResult);
//			sitalDevice_Free (s_swDevice2);
//			return swResult;
//		}
//
//		/// Zero the global ISR counter of ended minor frames.
//		s_dwMinorFrameCounter = 0;
//
//		///	Start device-2 (BC) running major-frame-1 for FrameCount shots.
//		S32BIT swFrameCount; // The number of frames to run.
//		swFrameCount = ( INFINITE_ITERATIONS ? sitalBcFrameIterations_RUN_FOREVER : (S32BIT)dwTestIterationsCount );
//		sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", s_swDevice2, swFrameId_1, swFrameCount);
//		swResult = sitalBc_Start (s_swDevice2, swFrameId_1, swFrameCount);
//		if (sitalReturnCode_SUCCESS != swResult)
//		{
//			sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
//			throw 0;
//		}
//
//		///	Wait forever.
//		// The user must abort in order to stop this function.
//		PrecisionTimer pt; // A precision timer.
//		U32BIT dwSleepTime; // Sleeping period (in ms).
//		dwSleepTime = 1000;
//		while (true)
//		{
//			sitalProcess_Log_PrintLine ("#StldTester.Wait: time_passed - %ims, isr_calls - %i\n", dwSleepTime, s_dwMinorFrameCounter);
//			sitalProcess_Log_PrintLine ("\tWAIT_FOR %i us\n", (dwSleepTime * 1000));
//			pt.AccuratelySleep (dwSleepTime); // 1 millisecond.
//		}
//
//		/// Stop device-2 (BC).
//		sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop: unit - %i\n", s_swDevice2);
//		swResult = sitalBc_Stop (s_swDevice2);
//		if (sitalReturnCode_SUCCESS != swResult)
//		{
//			sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
//			throw 0;
//		}
//
//		sitalProcess_Log_PrintLine ("#StldTester.Bc.MinorFrames.ActualCount: unit - %i, count - %i\n", s_swDevice2, s_dwMinorFrameCounter);
//
//		/// Free device-2 (BC).
//		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice2);
//		swResult = sitalDevice_Free (s_swDevice2);
//		if (sitalReturnCode_SUCCESS != swResult)
//		{
//			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
//			throw 0;
//		}
//
//		swResult = stldSyncEnd (__FUNCTION__);
//		if (sitalReturnCode_SUCCESS != swResult)
//		{
//			sitalProcess_Log_PrintLine ("#StldTester.stldSyncEnd.Error: error - %i\n", swResult);
//			throw 0;
//		}
//	}
//	catch (int)
//	{
//		sitalDevice_Free (s_swDevice2);
//	}
//
//	sitalProcess_Log_PrintLine ("#StldTester.Temp.End: test_succeeded - %c\n", ((sitalReturnCode_SUCCESS == swResult) ? 'Y' : 'N'));
//	return swResult;
//}



/// <summary>
/// BC user's ISR.
/// Count the overall number of BC interrupts and the overall number of transmitted minor frames into the dedicated global variables.
/// Record frame end time with each notification on end of minor frame, so that if the currently executing test recorded the frame run start time, it will be able to calculate in the end the overall duration of the frame run.
///
/// Note:
/// - Debug printouts shall be better disabled, as printing along callbacks may cause the loss of interrupts.
///   The caller is therefore encouraged to disable this function's printout, unless along debug session, by setting the global flag s_bIsIsrPrintoutEnabled to false.
///
/// @param swDeviceId (in)	Logical number of device (0-1)
/// @param dwIrqStatus (in)	IRQ status
/// </summary>
void _DECL stldTester_Bc_UserIsr2 (S16BIT swDeviceId, U32BIT dwIrqStatus)
{
	/// @pseudocode

	/// Increment the count of ISR calls.
	s_dwBcIsrCallCounter++;

	/// Record the informed IRQ status.
	s_dwBcIrqStatus = dwIrqStatus;

	/// If called for IRQ3 (end of minor frame):
	///		Increment the count of minor frames.
	///		Record the frame end time.
	BOOLEAN bSignaledInterruptOccurred; // A flag that says whether any interrupt about which waiting threads must be signaled occured.
	bSignaledInterruptOccurred = FALSE;
	if (sitalInterruptRegister2_BC_IRQ3 == (sitalInterruptRegister2_BC_IRQ3 & dwIrqStatus))
	{
		s_dwMinorFrameCounter++;
		s_ptMinorFrameRunTimeMeasurer.SetEndTime ();
		bSignaledInterruptOccurred = TRUE;


		/// Stop device-2 (BC).
		S16BIT swResult; // Result of operation or function call.
		U16BIT wRegisterValue; // The current value of a target register, or the value to set for a target register.
		swResult = sitalDevice_AccessMemory	(s_swDevice2, sitalDeviceAccessOperation_Read, sitalRegisterAddress_TIME_TAG, sitalDeviceMemorySection_Registers, 1U, &wRegisterValue);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
			throw 0;
		}
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop: unit - %i, time_tag - %04X\n", s_swDevice2, wRegisterValue);
		swResult = sitalBc_Stop (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		///	Start device-2 (BC) running major-frame-1 for FrameCount shots.
		S32BIT swFrameCount; // The number of frames to run.
		swFrameCount = 1;
		S16BIT swFrameId_1; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_1 = 1; // The major frame that was created by the test function (temp).
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", s_swDevice2, swFrameId_1, swFrameCount);
		swResult = sitalBc_Start (s_swDevice2, swFrameId_1, swFrameCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
			throw 0;
		}
	}
	if (sitalInterruptRegister2_BC_IRQ2 == (sitalInterruptRegister2_BC_IRQ2 & dwIrqStatus))
	{
		s_ptFullMinorFrameRunTimeMeasurer.SetEndTime ();
		bSignaledInterruptOccurred = TRUE;
	}

	if (TRUE == s_bIsIsrPrintoutEnabled)
	{
		sitalProcess_Log_PrintLine ("#StldTester.UserIsr.Bc.Recorded: device - %i, irq_status - %08X, bc_isr_calls - %i, minor_frame_count - %i\n", swDeviceId, dwIrqStatus, s_dwBcIsrCallCounter, s_dwMinorFrameCounter);
	}

	/// If any signaled interrupt occurred and any thread[s] may be waiting now for interrupts signals:
	///		Signal the possibly waiting threads about the interrupt.
	if ((TRUE == bSignaledInterruptOccurred) && (NULL != s_ipspBcSemaphore) && s_ipspBcSemaphore->IsActive ())
	{
		if (TRUE == s_bIsIsrPrintoutEnabled)
		{
			sitalProcess_Log_PrintLine ("#StldTester.UserIsr.Bc.Semaphore.Release: name - %s\n", s_szBcSemaphoreName);
		}
		if ( ! s_ipspBcSemaphore->Release () )
		{
			if (TRUE == s_bIsIsrPrintoutEnabled)
			{
				sitalProcess_Log_PrintLine ("#StldTester.UserIsr.Bc.Semaphore.Release.Error: error - %i\n", GetLastError_NonOsDependent());
			}
		}
	}
}



/// <summary>
/// Temporary.
///
/// @return sitalReturnCode_SUCCESS if function successfully completed.
/// @return Negative sitalReturnCode_* if an error condition encountered or function failed.
/// </summary>
S16BIT _DECL temp ()
{
	/// @pseudocode

	/// If any state irrelevancy or illegal input parameter or operation failure is identified:
	///		Return error.

	sitalProcess_Log_PrintLine ("#StldTester.Temp.Start\n");

	/// Set test arguments.
	U16BIT wFrameTime; // The time (in units of 100 us) that the frame will take to complete.
	wFrameTime = s_wFrameTime;
	U16BIT wRtSubaddress1; // A RT subaddress (sitalRtSubaddress_0-sitalRtSubaddress_31).
	wRtSubaddress1 = 30;

	S16BIT swResult; // Result of operation or function call.
	swResult = sitalReturnCode_SUCCESS;

	/// Get device count.
	U16BIT wRequiredDeviceCount; // The number of devices required for this test.
	wRequiredDeviceCount = (1 + s_swDevice2);
	U16BIT wAvailableDeviceCount; // The total number of devices available in the system.
	swResult = sitalDevice_GetCount (&wAvailableDeviceCount);
	if (sitalReturnCode_SUCCESS != swResult)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %i\n", swResult);
		return swResult;
	}
	sitalProcess_Log_PrintLine ("#StldTester.Device.Count: available - %i, required - %i\n", wAvailableDeviceCount, wRequiredDeviceCount);
	if (wAvailableDeviceCount < wRequiredDeviceCount)
	{
		sitalProcess_Log_PrintLine ("#StldTester.Device.Count.Get.Error: error - %s\n", "not_enough_devices_installed");
		return sitalReturnCode_SUCCESS;
	}

	try
	{
		swResult = stldSyncStart (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncStart.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Initialize device-2 (BC) as a real BC.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Init: unit - %i, access-card, mode-bc, alloc_mem-none\n", s_swDevice2);
		swResult = sitalDevice_Initialize (s_swDevice2, sitalAccess_CARD, (sitalMode_BC | sitalModeVariant_ADVANCED), 0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Init.Error: error - %i\n", swResult);
			throw 0;
		}

		const U16BIT wWordCount = 32; // Word count (1-32).
		U16BIT waBuffer[wWordCount]; // A buffer to read-from/write-into along data-block write/read operations, respectively.
		for (int i=0; i<wWordCount; i++)
		{
			waBuffer[i] = i;
		}

		/// Create for device-2 (BC) data-block-0, a data block of 32 data words.
		S16BIT swDataBlockId_0; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId_0 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i\n", s_swDevice2, swDataBlockId_0, 64, wWordCount);
		swResult = sitalBc_DataBlock_Create (s_swDevice2, swDataBlockId_0, sitalBcDataBlockSizeOption_DOUBLE, waBuffer, wWordCount);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Write: unit - %i, block - %i, size - %i, buf_size - %i (%04X...)\n", s_swDevice2, swDataBlockId_0, 64, wWordCount, waBuffer[0]);
		swResult = sitalBc_DataBlock_Write (s_swDevice2, swDataBlockId_0, waBuffer, wWordCount, 0);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Write.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) message-1, a BC-to-RT(RT-address-1/RtSubAddress1) message that uses data-block-0 to transmit WordCount data words. Message bus: bus-1.
		S16BIT swMessageId_0; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create: unit - %i, message - %i, block - %i, rx_rt - %i, rx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddress1, wWordCount, 0, s_wBus1);
		swResult = sitalBc_Message_CreateBcToRt (s_swDevice2, swMessageId_0, swDataBlockId_0, s_wRtAddress1, wRtSubaddress1, wWordCount, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.BcToRt.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-0, a XEQ command that executes message-0.
		S16BIT swCommandId_0; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_0, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		for (int i=0; i<wWordCount; i++)
		{
			waBuffer[i] = 0;
		}

		/// Create for device-2 (BC) data-block-1, a data block of 32 data words.
		S16BIT swDataBlockId_1; // A unique ID designating a data block (0-(sitalBcCounter_DATA_BLOCKS-1)).
		swDataBlockId_1 = 2;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create: unit - %i, block - %i, size - %i, buf_size - %i\n", s_swDevice2, swDataBlockId_1, 64, wWordCount);
		swResult = sitalBc_DataBlock_Create (s_swDevice2, swDataBlockId_1, sitalBcDataBlockSizeOption_DOUBLE, waBuffer, wWordCount);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Write: unit - %i, block - %i, size - %i, buf_size - %i (%04X...)\n", s_swDevice2, swDataBlockId_1, 64, wWordCount, waBuffer[0]);
		swResult = sitalBc_DataBlock_Write (s_swDevice2, swDataBlockId_1, waBuffer, wWordCount, 0);
		if (0 > swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.DataBlock.Write.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) message-0, a RT(RtAddress1/RtSubAddress1)-to-BC message that uses data-block-1 to receive WordCount data words. Message bus: bus-1.
		S16BIT swMessageId_1; // A unique ID designating a message (0-(sitalBcCounter_MESSAGES-1)).
		swMessageId_1 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBc.Create: unit - %i, message - %i, block - %i, tx_rt - %i, tx_rt_sa - %i, wc - %i, gap - %i, opts-x%X\n", s_swDevice2, swMessageId_1, swDataBlockId_1, s_wRtAddress1, wRtSubaddress1, wWordCount, 0, s_wBus1);
		swResult = sitalBc_Message_CreateRtToBc (s_swDevice2, swMessageId_1, swDataBlockId_1, s_wRtAddress1, wRtSubaddress1, wWordCount, 0, s_wBus1);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Message.RtToBc.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-1, a XEQ command that executes message-0.
		S16BIT swCommandId_1; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_1 = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_1, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_1, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_1, sitalOpcode_EXECUTE_MESSAGE, sitalOpcodeCondition_ALWAYS, swMessageId_1, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) command-2, a subroutine call command that executes minor-frame-0. 
		S16BIT swCommandId_2; // A unique ID designating a command (0-(sitalBcCounter_COMMANDS-1)).
		swCommandId_2 = 2;
		S16BIT swFrameId_0; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_0 = 0;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create: unit - %i, cmd - %i, opcode - %i, cond - %i, prm1 - %i, prm2 - %i\n", s_swDevice2, swCommandId_2, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, swFrameId_0, 0);
		swResult = sitalBc_Command_Create (s_swDevice2, swCommandId_2, sitalOpcode_CALL_SUBROUTINE, sitalOpcodeCondition_ALWAYS, (U16BIT)swFrameId_0, 0, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Command.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) minor-frame-0, a frame that contains command-0 and command-1.
		S16BIT swaCommandIds[sitalBcCounter_COMMANDS]; // An array of command-IDs.
		swaCommandIds[0] = swCommandId_0;
		swaCommandIds[1] = swCommandId_1;
		U16BIT wCommandIdCount; // The number of command IDs in the frame (1-sitalBcCounter_FRAME_COMMANDS).
		wCommandIdCount = 2;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i, %i), time - %i, flags - %i\n", s_swDevice2, swFrameId_0, "minor", wCommandIdCount, swCommandId_0, swCommandId_1, wFrameTime, 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_0, sitalBcFrameType_MINOR, swaCommandIds, wCommandIdCount, wFrameTime, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Create for device-2 (BC) major-frame-1, a frame that contains command-2 and whose frame time is FrameTime us.
		S16BIT swFrameId_1; // A unique ID designating a frame (0-(sitalBcCounter_FRAMES-1)).
		swFrameId_1 = 1;
		swaCommandIds[0] = swCommandId_2;
		wCommandIdCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create: unit - %i, frame - %i, type - %s, commands - %i(%i), time - %ims, flags - %i\n", s_swDevice2, swFrameId_1, "major", wCommandIdCount, swCommandId_2, (wFrameTime / 10), 0);
		swResult = sitalBc_Frame_Create (s_swDevice2, swFrameId_1, sitalBcFrameType_MAJOR, swaCommandIds, wCommandIdCount, wFrameTime, 0);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Frame.Create.Error: error - %i\n", swResult);
			throw 0;
		}

		/// Configure given device (BC) to get IRQ-3 notifications using an ISR that collects interrupt data, counts ISR calls, and signals interested threads.
		sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Manipulate: unit - %i, enable - %i, irq - %08X, callback - %s\n", s_swDevice2, TRUE, sitalInterruptRegister2_BC_IRQ3, "stldTester_Bc_UserIsr");
		swResult = sitalDevice_Irq_Manipulate (s_swDevice2, TRUE, sitalInterruptRegister2_BC_IRQ3, stldTester_Bc_UserIsr2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Irq.Set.Error: error - %i\n", swResult);
			sitalDevice_Free (s_swDevice2);
			return swResult;
		}

		/// Zero the global ISR counter of ended minor frames.
		s_dwMinorFrameCounter = 0;

		///	Start device-2 (BC) running major-frame-1 for FrameCount shots.
		S32BIT swFrameCount; // The number of frames to run.
		swFrameCount = 1;
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Start: unit - %i, frame - %i, count - %i\n", s_swDevice2, swFrameId_1, swFrameCount);
		swResult = sitalBc_Start (s_swDevice2, swFrameId_1, swFrameCount);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Start.Error: error - %i\n", swResult);
			throw 0;
		}

		///	Wait forever.
		// The user must abort in order to stop this function.
		PrecisionTimer pt; // A precision timer.
		U32BIT dwSleepTime; // Sleeping period (in ms).
		dwSleepTime = 1000;
		while (true)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Wait: time_passed - %ims, isr_calls - %i\n", dwSleepTime, s_dwMinorFrameCounter);
			sitalProcess_Log_PrintLine ("\tWAIT_FOR %i us\n", (dwSleepTime * 1000));
			pt.AccuratelySleep (dwSleepTime); // 1 millisecond.
		}

		/// Stop device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop: unit - %i\n", s_swDevice2);
		swResult = sitalBc_Stop (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Bc.Stop.Error: error - %i\n", swResult);
			throw 0;
		}

		sitalProcess_Log_PrintLine ("#StldTester.Bc.MinorFrames.ActualCount: unit - %i, count - %i\n", s_swDevice2, s_dwMinorFrameCounter);

		/// Free device-2 (BC).
		sitalProcess_Log_PrintLine ("#StldTester.Device.Free: unit - %i\n", s_swDevice2);
		swResult = sitalDevice_Free (s_swDevice2);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.Device.Free.Error: error - %i\n", swResult);
			throw 0;
		}

		swResult = stldSyncEnd (__FUNCTION__);
		if (sitalReturnCode_SUCCESS != swResult)
		{
			sitalProcess_Log_PrintLine ("#StldTester.stldSyncEnd.Error: error - %i\n", swResult);
			throw 0;
		}
	}
	catch (int)
	{
		sitalDevice_Free (s_swDevice2);
	}

	sitalProcess_Log_PrintLine ("#StldTester.Temp.End: test_succeeded - %c\n", ((sitalReturnCode_SUCCESS == swResult) ? 'Y' : 'N'));
	return swResult;
}
